/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("CLIPEXH.C")

#define INCL_DEV             // include all DEV definitions
#define INCL_WIN             // include Win definitions
#define INCL_GPI             // include all Gpi definitions
#define INCL_GRE             // include all Gre definitions
#define INCL_GRE_CLIP
#define INCL_GRE_DCS         // GreOpenDC and GreCloseDC
#define INCL_GRE_DEVSUPPORT  // for DC_BLOCK for GetDriverInfo
#define INCL_GRE_DEVMISC2    // for GreDeviceInvalidateVisRegion
#define INCL_ERRORS          // include all Gre error definitions

#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <pmddi.h>              // needed for GRE function definitions.
#include <stdio.h>
#include "DTTDLLDF.h"           // test case defines.
#include "common1.h"            // Graphics testing common file.
#include "common2.h"
#define EXTERN extern           // for globals in .h to be extern defs
#include "greclip.h"            // Test Definition File.

/****************************************************************************
 *
 * Subtask Name: GreClip.dll
 *
 * Module name:  clipexh.c
 *
 * Purpose:      module for GRECLIP exhautive entry points
 *
 * Revision Log: 12/17/91,      Graphics Suite Extension, Thom Bain
 *                               Initial Design.
 *
 * Documentation Reference:  Section 13.4
 *
 * Contents:     DrawPSBox
 *               SetMaxRegion
 *               GetRandomRect
 *               GreIntersectClipRectangleExh
 *               GreExcludeClipRectangleExh
 *               GreOffsetClipRegionExh
 *               GreSelectAndQueryClipRegionExh
 *               GreCopyClipRegionExh
 *               GreSetXformRectExh
 *               GreGetClipBoxAndRctsAndVChkExh
 *               GreSaveAndRestoreRegionExh
 *               GreSelectPathRegionExh
 *               GreRegionSelectBitmapExh
 *               GreClipPathCurvesExh
 *
****************************************************************************/

// static global variables for exhaustive entry points

static struct {
        LONG lPatternIndex;
        CHAR *szPatternIndex;
        }
        pat[] = {
                PATSYM_VERT,      "PATSYM_VERT       ",
                PATSYM_HORIZ,     "PATSYM_HORIZ      ",
                PATSYM_DIAG2,    "PATSYM_DIAG2      ",
                PATSYM_DIAG4,    "PATSYM_DIAG4      ",
                PATSYM_HALFTONE, "PATSYM_HALFTONE   ",
                PATSYM_DENSE6,   "PATSYM_DENSE6     ",
                PATSYM_DENSE7,   "PATSYM_DENSE7     "
                } ; // 8 mixes

static struct {
        LONG lColorIndex;
        CHAR *szColorIndex;
        }
        clr[] = {
                CLR_RED,       "CLR_RED           ",
                CLR_DARKPINK,   "CLR_DARKPINK      ",
                CLR_PINK,      "CLR_PINK          ",
                CLR_WHITE,     "CLR_WHITE         ",
                CLR_YELLOW,     "CLR_YELLOW        ",
                CLR_CYAN,      "CLR_CYAN          ",
                CLR_DARKCYAN , "CLR_DARKCYAN      ",
                CLR_GREEN,     "CLR_GREEN         ",
                CLR_DARKGREEN, "CLR_DARKGREEN     ",
                CLR_DARKBLUE,   "CLR_DARKBLUE      ",
                CLR_BLUE,      "CLR_BLUE          ",
                CLR_PALEGRAY,   "CLR_PALEGRAY      ",
                CLR_DARKGRAY,   "CLR_DARKGRAY      ",
                CLR_BROWN,     "CLR_BROWN         "
                } ;     // 14 colors

static struct {
        LONG lBWIndex;
        CHAR *szBWIndex;
        }
        BW[] = {
                CLR_TRUE,   "CLR_TRUE  ",
                CLR_FALSE,  "CLR_FALSE "
                } ;     // 2 colors

static RECTL rcRect;   // rectangle for setting clip regions
static RECTL rcLimits; // maximum region coordinates - set by SetMaxRegion

/****************************************************************************
 *\\ddd
 * Routine Name: DrawPsBox()
 *
 * Purpose: This routine draws a pattern-filled box that will cover
 *          the page in order to indicate the current clip region.
 *
 * System Requirements: NONE
 *
 * Revision Log: 12/07/90, PDVT, Hal Bray (AiC Austin)
 *                      - original implementation as fill_page()
 *               11/01/91, GRE Testing, David McCombs
 *                      - re-written as DrawPsBox()
 *               01/02/92, GRE Testing, Thom Bain
 *                      - re-written for optimization improvements
 *
 * Inputs:      hPS         - presentation space handle
 *              hWndClient  - Client Window handle (for SHOWERROR)
 *
 * Outputs: NONE
 *
 * Subroutines Required:
 *          GpiBox
 *          GpiMove
 *          SHOWERROR
 *          MapY
 *
 * Limitations: NONE
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *
 *      Move the current point to ( 0, 7/10ths inch).
 *      Draw a pattern filled box the size of the PS (less the .7" at bottom)
 *
 *\\end
 ***************************************************************************/

VOID DrawPsBox(HPS hPS,HWND hWndClient)
{
  static POINTL ptlLLC,     // upper and lower points of box
                          ptlURC;

  // Set the lower corner to point ( 0.0, 0.7 ).
  // The Y coordinate 0.7 Prevents the Box from overwriting TEST_SIGNATURE.
  ptlLLC.x = 0L;
  ptlLLC.y = MapY( 70L );

  // Set the upper corner to the size of the Presentation Page.
  ptlURC.x = pg_width - 1L;
  ptlURC.y = pg_height - 1L;

  // Draw the Box.
  if (GpiMove(hPS, &ptlLLC) == FALSE)
    SHOWERROR("GpiMove", "DrawPsBox");

  if (GpiBox(hPS, DRO_FILL, &ptlURC, 0L, 0L) == GPI_ERROR)
    SHOWERROR("GpiBox", "DrawPsBox");
  return;

}  /* end DrawPsBox */

/****************************************************************************
 *\\ddd
 * Routine Name: SetMaxRegion()
 *
 * Purpose: This routine sets the rcLimits rectangle structure to the
 *          maximum allowable region coordinates in world space for the
 *          given "Range" (0=> max/min region coordinates, 1=> twice the
 *          page size, and 2=>concentrate on the visible range (page size)).
 *
 * System Requirements: NONE
 *
 * Revision Log: 11/01/91, GRE Testing, David McCombs
 *               01/02/92, GRE extension suite, Thom Bain
 *
 * Inputs:  SHORT Range // Range Selection variable.
 *
 * Outputs: rcLimits rectangle set to limits.
 *
 * Subroutines Required: GpiConvert
 *
 * Limitations: NONE
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *
 *    set local rectangle to max/min values for region in device coords
 *    call GpiConvert to convert these to world coords
 *    test if still in valid region coord range and adjust if necessary
 *    Test for which range is being used
 *    If Range = 0 set global RECTL structure to max/min valid region coords
 *      just calculated.
 *    If Range = 1 set global RECTL structure to plus/minus twice the page
 *    If Range = 2 set global RECTL structure to the page size
 *    Compare these values to local max/min values and adjust if necessary
 *    Return the calculated values.
 *
 *\\end
 ***************************************************************************/

VOID SetMaxRegion(SHORT Range, HPS hPS, HWND hWndClient)
{
  RECTL rcWorldRgnCoordLimits;

  // Determine maximum region coordinate in world space
  rcWorldRgnCoordLimits.xLeft   = MINRGNCOORD;
  rcWorldRgnCoordLimits.xRight  = MAXRGNCOORD;
  rcWorldRgnCoordLimits.yBottom = MINRGNCOORD;
  rcWorldRgnCoordLimits.yTop    = MAXRGNCOORD;
  // convert to device coordinates.
  if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L,
                                            (PPOINTL)&rcWorldRgnCoordLimits))
    SHOWERROR("GpiConvert", NULL);

  // check to see if in valid range in device coordinates
  if (rcWorldRgnCoordLimits.xLeft < MINRGNCOORD)
    rcWorldRgnCoordLimits.xLeft = MINRGNCOORD;
  if (rcWorldRgnCoordLimits.yBottom < MINRGNCOORD)
    rcWorldRgnCoordLimits.yBottom = MINRGNCOORD;
  if (rcWorldRgnCoordLimits.xRight > MAXRGNCOORD)
    rcWorldRgnCoordLimits.xRight = MAXRGNCOORD;
  if (rcWorldRgnCoordLimits.yTop > MAXRGNCOORD)
    rcWorldRgnCoordLimits.yTop = MAXRGNCOORD;

  // convert back to world coordinates
  if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L,
                                            (PPOINTL)&rcWorldRgnCoordLimits))
    SHOWERROR("GpiConvert", NULL);

  // This is the last selection in the three stage loop
  // of clip ranges.  It concentrates on the visible region.
  switch (Range) {
    case 2:
      rcLimits.xLeft   = 0L;
      rcLimits.xRight  = pg_width - 1;
      rcLimits.yBottom = 0L;
      rcLimits.yTop    = pg_height - 1;
      break;

  // This is the 2nd selection in the three stage loop
  // of clip ranges.  it is intended to go +/-
  // twice the size of the presentation page.
    case 1:
      rcLimits.xLeft   = -2 * pg_width;
      rcLimits.xRight  =  2 * pg_width;
      rcLimits.yBottom = -2 * pg_height;
      rcLimits.yTop    =  2 * pg_height;
      break;

  // This is the first selection in the three stage loop
  // of clip ranges.  It is intended test to the clip region limits.
    case 0:
      rcLimits.xRight  = rcWorldRgnCoordLimits.xRight;
      rcLimits.yTop    = rcWorldRgnCoordLimits.yTop;
      rcLimits.xLeft   = rcWorldRgnCoordLimits.xLeft;
      rcLimits.yBottom = rcWorldRgnCoordLimits.yBottom;
      break;
  } /* end switch */

  // make sure still in valid region coordinate range
  if (rcLimits.xLeft < rcWorldRgnCoordLimits.xLeft)
    rcLimits.xLeft = rcWorldRgnCoordLimits.xLeft;
  if (rcLimits.yBottom < rcWorldRgnCoordLimits.yBottom)
    rcLimits.yBottom = rcWorldRgnCoordLimits.yBottom;
  if (rcLimits.xRight > rcWorldRgnCoordLimits.xRight)
    rcLimits.xRight = rcWorldRgnCoordLimits.xRight;
  if (rcLimits.yTop > rcWorldRgnCoordLimits.yTop)
    rcLimits.yTop = rcWorldRgnCoordLimits.yTop;
  return;

}  /* end SetMaxRegion */

/****************************************************************************
 *\\ddd
 * Routine Name: GetRandomRect()
 *
 * Purpose: This routine sets the global rectangle "rcRect" to random values
 *          in the current range limits set in the global rectangle, "rcLimits",
 *          set "SetMaxRange" function.
 *
 * System Requirements: NONE
 *
 * Revision Log: 11/01/91, GRE Testing, David McCombs
 *               01/02/92, GRE extension suite, Thom Bain
 *
 * Inputs:  HPS hPS     // Presentation space handle passed to entry point
 *          HWND hWndClient // window handle (for SHOWERROR)
 *
 * Outputs: rcRect rectangle set to random values in the current range.
 *
 * Subroutines Required: GpiConvert
 *
 * Limitations: NONE
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *
 *    generate random rectangle values in rcLimits range
 *    use GpiConvert to convert to help avoid round-off errors in the
 *      device transform by converting to device, then world, then device
 *      and back to world coordinates.
 *
 *\\end
 ***************************************************************************/

VOID GetRandomRect(HPS hPS, HWND hWndClient)
{
  LONG temp;

  // get a random rectangle
  rcRect.xLeft=randL(rcLimits.xLeft,rcLimits.xRight);
  rcRect.xRight=randL(rcLimits.xLeft,rcLimits.xRight);
  // make sure xRight is greater than xLeft
  if (rcRect.xLeft > rcRect.xRight)  // swap if necessary
  {
    temp = rcRect.xLeft;
    rcRect.xLeft = rcRect.xRight;
    rcRect.xRight = temp;
  }
  rcRect.yBottom=randL(rcLimits.yBottom,rcLimits.yTop);
  rcRect.yTop=randL(rcLimits.yBottom,rcLimits.yTop);
  // make sure yTop is greater than yBottom
  if (rcRect.yBottom > rcRect.yTop) // swap if necessary
  {
    temp = rcRect.yBottom;
    rcRect.yBottom = rcRect.yTop;
    rcRect.yTop = temp;
  }

  // adjust for possible round-off error from the device transform.
  // by converting to device coordinates and back to world.
  if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L, (PPOINTL)&rcRect))
    SHOWERROR("GpiConvert", NULL);
  if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L, (PPOINTL)&rcRect))
    SHOWERROR("GpiConvert", NULL);
  if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L, (PPOINTL)&rcRect))
    SHOWERROR("GpiConvert", NULL);
  if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L, (PPOINTL)&rcRect))
    SHOWERROR("GpiConvert", NULL);
  return;
} // end GetRandomRect

/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreIntersectClipRectangleExh() exhaustive algorithm
 *
 * Purpose: Exhaustively test GreIntersectClipRectangle function
 *
 * System Requirements: Section 13.4.1
 *
 *      Revision Log: 12/31/91 Initial creation, Thom Bain
 *
 * Inputs:      hPS   -Presentations Space handle.
 *              hWndClient    -Client Window handle.
 *              SelectionCall -Selection Indicator, is 0 if routine is selected.
 *
 * Outputs: random patterns (and colors on display) are used in random
 *          generated clip regions to draw a PS size pattern filled box
 *          after establishing each clip region.
 *
 * Subroutines Required:   WinSendMsg, ABORTENTRY, strcmp, ENTER_MAIN,
 *                         DeviceIsPrinter, WinQueryAnchorBlock,
 *                         WinGetLastError, CLEANUP, WinQueryDevice,
 *                         SETENTER, srand, GpiQueryRegionBox, SETEXIT,
 *                         sprintf, SUBENTER, SetMaxRegion, GetRandomRect,
 *                         randL, GpiConvert,
 *                         GpiSetClipRegion, GpiRectInRegion, SHOWERROR,
 *                         ERRSTATUS, GreIntersectClipRectangle, LOGINFO,
 *                         GpiSetColor, GpiSetPattern, GpiDestroyRegion,
 *                         randI, DrawPsBox, SUBEXIT, EXIT_MAIN
 *
 * Limitations:
 *                      Not Valid for OD_QUEUED, PM_Q_STD output.
 *
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *        if (SelectionCall)
 *        {
 *          WndSendMsg();
 *        }
 *        else
 *   {
 *      test for environmental restriction OD_QUEUED/PM_Q_STD
 *      Setup
 *      {
 *        Query for hDC from hPS;
 *        Set number of loops;
 *        Set random seed;
 *        fetch the handle to the anchor block for next step
 *        clear error buffer for ERRSTATUS by calling WinGetLastError
 *        initialize region handle to NULL for first time through loop
 *           }
 *
 *      For three ranges extending from region limits to just visible region
 *      {
 *        establish max values for this range
 *        generate a random rectangle in this range
 *        if any region established delete it
 *        create first region for this range by intersecting random rect.
 *          by calling GreIntersectClipRectangle
 *        item entry
 *        {
 *          generate a random rectangle in current range
 *          convert rectangle to device coordinates for bounds calculation
 *          fetch the current region bounds.
 *          call GpiRectInRegion to determine if this rectangle will inter-
 *            sect with current region.
 *          use this information to calculate new bounds when the rectangle
 *            is intersected with the current region.
 *          convert rectangle back to world coordinates
 *          create new region by intersecting the rectangle with the current
 *            region (GreIntersectClipRectangle)
 *          query the new region bounds via GpiQueryRegionBox
 *          if new region is NULL delete it and randomly create another
 *          compare calculated bounds to actual and log mismatch if any.
 *            note that calculated and actual bounds are logged along with
 *            current rectangle coordinates (all are in device coords.)
 *          if concentrating on visible device range and device is display
 *            or this attempt modulo 20 equal 0
 *          {
 *            randomly set color and pattern (use black for printers)
 *            DrawPSBox
 *          }
 *        } // end item entry
 *      } // end three ranges
 *      destroy the last region
 *    } // end else
 *\\end
 ***************************************************************************/

VOID APIENTRY GreIntersectClipRectangleExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  INT    iValueLoop,
         MaxLoop,
         RangeLoop;
  INT    sColor,
         sPattern;
  RECTL  rcBounds, rcCheck;
  HAB    hAB;
  HDC    hDC;
  HRGN   hrgn, hrgnNULL;
  LONG   lRgnType, lRgnInside;
  static CHAR *EntryName = "GreIntersectClipRectangleExh";

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    MaxLoop = 100;  // how many times to test function in each range
    srand(0); // initialize the random number generator seed.
    hrgn = NULLHANDLE; //16KQ initialize for first time through loop

    SETEXIT();  // End of Set Up to Test.

    TEST_PROGRESS ( 00, 3 * MaxLoop );

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreIntersectClipRectangle,szAddMessage);

      // establish rcLimits values for this RangeLoop
      SetMaxRegion(RangeLoop, hPS, hWndClient);

      // start with a random region in this range
      GetRandomRect(hPS, hWndClient);

      // delete existing region, if any
      if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
        SHOWERROR("GpiDestroyRegion", NULL);
      if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
        SHOWERROR("GpiSetClipRegion", NULL);

      if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
        SHOWERROR("GpiDestroyRegion", NULL);

      // create first region
      if ((GreIntersectClipRectangle(hDC, &rcRect)) == RGN_ERROR)
      {
        ErrorCount++;
        ERRSTATUS(TRUE, "GreIntersectClipRectangle", "failure");
      }

      // de-select the region
      if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
        SHOWERROR("GpiSetClipRegion", NULL);

      // test MaxLoop times in this range
      for (iValueLoop = 1; iValueLoop <= MaxLoop; iValueLoop++)
      {
        // get a random rectangle
        GetRandomRect(hPS, hWndClient);

        // switch to device coordinates for bounds calculation
        if ( !GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L,
                                                       (PPOINTL)&rcRect) )
          SHOWERROR("GpiConvert", NULL);

        // fetch the current bounds.
        if (GpiQueryRegionBox(hPS, hrgn, &rcBounds) == RGN_ERROR)
          SHOWERROR("GpiQueryRegionBox", NULL);

        rcRect.xRight++;   // reflect exclusive boundary on top & right
        rcRect.yTop++;
        // determine if the new region will intersect with the current
        lRgnInside = GpiRectInRegion(hPS, hrgn, &rcRect);
        rcRect.xRight--;   // go back for actual region settting.
        rcRect.yTop--;

        // calculate new rcBounds
        switch(lRgnInside)
        {
          case RRGN_ERROR:
            SHOWERROR("GpiRectInRegion", NULL);
            break;
          case RRGN_PARTIAL:
            if(rcRect.xLeft > rcBounds.xLeft)
              rcBounds.xLeft = rcRect.xLeft;
            if(rcRect.xRight + 1 < rcBounds.xRight)
              rcBounds.xRight = rcRect.xRight + 1;
            if(rcRect.yBottom > rcBounds.yBottom)
              rcBounds.yBottom = rcRect.yBottom;
            if(rcRect.yTop + 1 < rcBounds.yTop)
              rcBounds.yTop = rcRect.yTop + 1;
            break;
          case RRGN_INSIDE:
            // new bounds will be the new rectangle
            rcBounds = rcRect;
            rcBounds.xRight++;  // reflect exlusion of top and right sides
            rcBounds.yTop++;
            break;
          case RRGN_OUTSIDE:
            // if new rectangle completely surrounds current region
            // the bounds are uneffected otherwise
            // a NULL region should be created
            if (     rcRect.xLeft < rcBounds.xLeft &&
                rcRect.xRight + 1 > rcBounds.xRight &&
                   rcRect.yBottom < rcBounds.yBottom &&
                  rcRect.yTop + 1 > rcBounds.yTop )
            {
               // do nothing, current bounds should not change
            }
            else
            {
              rcBounds.xLeft = 0L;
              rcBounds.xRight = 0L;
              rcBounds.yBottom = 0L;
              rcBounds.yTop = 0L;
            }
            break;
        } // end switch

        // restore the region definiton
        // Because of de-selection, hrgnNULL will alway be NULL
        if (GpiSetClipRegion(hPS, hrgn, &hrgnNULL) == RGN_ERROR)
        {
          SHOWERROR("GpiSetClipRegion:", NULL);
        }

        // establish the new clip region, intersect with new rectangle

        // switch to world coordinates to intersect new rectangle
        if ( !GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L,
                                                       (PPOINTL)&rcRect) )
          SHOWERROR("GpiConvert", NULL);

        if ((lRgnType = GreIntersectClipRectangle(hDC,&rcRect)) == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreIntersectClipRectangle:", "failure" );
        }

        // get new region box for bounds comparison check
        if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
          SHOWERROR("GpiSetClipRegion", NULL);
        if (GpiQueryRegionBox(hPS, hrgn, &rcCheck) == RGN_ERROR)
          SHOWERROR("GpiQueryRegionBox", NULL);

        // see if a region exists, if rectangles do not intersect
        // a NULL region should exist
        if (lRgnType == RGN_NULL)
        {
          // create a region of some kind for next loop
          GetRandomRect(hPS, hWndClient);

          // delete existing region
          if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
            SHOWERROR("GpiDestroyRegion", NULL);

          if ((GreIntersectClipRectangle(hDC, &rcRect)) == RGN_ERROR)
          {
            ErrorCount++;
            ERRSTATUS(TRUE, "GreIntersectClipRectangle", "failure");
          }

          // de-select the region
          if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
            SHOWERROR("GpiSetClipRegion", NULL);
        }

        // compare returned by GpiQueryRegionBox to calculated
        if (   rcBounds.xLeft != rcCheck.xLeft ||
              rcBounds.xRight != rcCheck.xRight ||
             rcBounds.yBottom != rcCheck.yBottom ||
                rcBounds.yTop != rcCheck.yTop)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreIntersectClipRectangle:", "Field Mismatch");
          sprintf(szAddMessage,"Compare GpiQueryRegionBox:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcCheck.xLeft, rcCheck.xRight,
                               rcCheck.yBottom, rcCheck.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);
          sprintf(szAddMessage," to calculated bounds:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcBounds.xLeft, rcBounds.xRight,
                               rcBounds.yBottom, rcBounds.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);

          sprintf(szAddMessage,"xl=%ld, xr=%ld, yb=%ld, yt=%ld", rcRect.xLeft,
                                rcRect.xRight, rcRect.yBottom, rcRect.yTop);
          LOGINFO(L_TRACE,"*",szAddMessage);
        }

        // drawing here insures that at least the most recent rectangle
        // will be drawn.
        if ((RangeLoop == 2) && (DeviceIsDisplay() || (iValueLoop % 20 == 0)))
        {
          sColor=randI(0,13);
          sPattern=randI(0,7);

          if (DeviceIsPrinter())
          {
            sColor= 1; // randI(0,1);
            if ( GpiSetColor(hPS, BW[sColor].lBWIndex) == FALSE)
              SHOWERROR("GpiSetColor:", BW[sColor].szBWIndex);
          }
          else
          {
            if ( GpiSetColor(hPS, clr[sColor].lColorIndex) == FALSE)
              SHOWERROR("GpiSetColor:", clr[sColor].szColorIndex);
          }

          if (GpiSetPattern(hPS, pat[sPattern].lPatternIndex) == FALSE)
            SHOWERROR("GpiSetPattern:", pat[sPattern].szPatternIndex);

          // activate the region for drawing.
          if (GpiSetClipRegion(hPS, hrgn, &hrgnNULL) == RGN_ERROR)
            SHOWERROR("GpiSetClipRegion", NULL);

          DrawPsBox(hPS, hWndClient);

          // de-select and save in handle
          if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
            SHOWERROR("GpiSetClipRegion", NULL);
        }


        TEST_PROGRESS ( RangeLoop * MaxLoop + iValueLoop, 3 * MaxLoop );

      }

      SUBEXIT();

    }

    // cleanup last region
    if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    EXIT_MAIN(EntryName);
  } // end else

  return;

} /* end GreIntersectClipRectangleExh */

/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreExcludeClipRectangleExh() exhaustive algorithm
 *
 * Purpose: Exhaustively test GreExcludeClipRectangle function
 *
 * System Requirements: Section 13.4.2
 *
 *      Revision Log: 12/31/91 Initial creation, Thom Bain
 *
 * Inputs:      hPS   -Presentations Space handle.
 *              hWndClient    -Client Window handle.
 *              SelectionCall -Selection Indicator, is 0 if routine is selected.
 *
 * Outputs: random patterns (and colors on display) are used in random
 *          generated clip regions to draw a PS size pattern filled box
 *          after establishing each clip region.
 *
 * Subroutines Required:   WinSendMsg, ABORTENTRY, strcmp, ENTER_MAIN,
 *     DeviceIsPrinter, WinQueryAnchorBlock, WinGetLastError, CLEANUP,
 *     WinQueryDevice, SETENTER, srand, GpiQueryRegionBox, SETEXIT, sprintf,
 *     SUBENTER, SetMaxRegion, GetRandomRect, randL, GpiConvert,
 *     GpiSetClipRegion, GpiRectInRegion, GpiQueryRegionRects, SHOWERROR,
 *     ERRSTATUS, GreExcludeClipRectangle, LOGINFO, DeviceIsDisplay,
 *     GpiSetColor, GpiSetPattern, GpiDestroyRegion, randI, DrawPsBox,
 *     SUBEXIT, EXIT_MAIN
 *
 * Limitations:
 *                      Not Valid for OD_QUEUED, PM_Q_STD output.
 *
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *        if (SelectionCall)
 *        {
 *          WndSendMsg();
 *        }
 *        else
 *   {
 *      test for environmental restriction OD_QUEUED/PM_Q_STD
 *      Setup
 *      {
 *        Query for hDC from hPS;
 *        Set number of loops;
 *        Set random seed;
 *        initialize region handle to NULL for first time through loop
 *        fetch the handle to the anchor block for next step
 *        clear error buffer for ERRSTATUS by calling WinGetLastError
 *           }
 *
 *      For three ranges extending from region limits to just visible region
 *      {
 *        establish rcLimits region values for this range.
 *        if any existing regions destroy them
 *        create a clip region of rcLimits size.
 *        item entry
 *        {
 *          get a random rectangle in this range
 *          convert rectangle to device coordinates for bounds calculation
 *          query the current region bounds via GpiQueryRegionBox
 *          use GpiRectInRegion to determine if new rectangle will effect
 *            current region when excluded
 *          use this information and GpiQueryRegionRects to calculate new
 *            bounds
 *          convert rectangle back to world coords for exclusion attempt
 *          call GreExcludeClipRectangle to exclude this rectangle
 *          query the new bounds via GpiQueryRegionBox
 *          compare calculated to actual bounds and log mismatch if any
 *            note that calculated and actual bounds are logged on mismatches
 *            and that they and the new rectangle are logged in device coords
 *          compare results to calculated resutlant region values
 *          if concentrating on visible device range and device is display
 *            or this attempt modulo 20 equal 0
 *          {
 *            randomly set color and pattern (use black for printers)
 *            DrawPSBox
 *          }
 *        } // end item entry
 *      } // end three ranges
 *      destroy the last region
 *    } // end else
 *\\end
 ***************************************************************************/

VOID APIENTRY GreExcludeClipRectangleExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  INT    iValueLoop,
         MaxLoop,
         RangeLoop;
  INT    sColor,
         sPattern;
  INT    i;
  RECTL  rcBounds, rcCheck, Rects[5];
  RGNRECT Control;
  HAB    hAB;
  HDC    hDC;
  HRGN   hrgn, hrgnNULL;
  LONG   minxLeft, maxxRight, minyBottom, maxyTop;
  LONG   lRgnType, lRgnInside;
  static CHAR *EntryName = "GreExcludeClipRectangleExh";

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    MaxLoop = 100;  // how many times to test function in each range
    srand(0); // initialize the random number generator seed.
    hrgn = NULLHANDLE;   // initialize for first time through loop

    SETEXIT();  // End of Set Up to Test.

    TEST_PROGRESS ( 00, 3 * MaxLoop );

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreExcludeClipRectangle,szAddMessage);

      // establish rcLimits values for this RangeLoop
      SetMaxRegion(RangeLoop, hPS, hWndClient);

      // start with largest possible region
      rcRect.xLeft   = rcLimits.xLeft;
      rcRect.yBottom = rcLimits.yBottom;
      rcRect.xRight  = rcLimits.xRight;
      rcRect.yTop    = rcLimits.yTop;

      // delete existing region, if any
      if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
        SHOWERROR("GpiDestroyRegion", NULL);
      if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
        SHOWERROR("GpiSetClipRegion", NULL);
      if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
        SHOWERROR("GpiDestroyRegion", NULL);

      // create first region
      if (GpiIntersectClipRectangle(hPS, &rcRect) == RGN_ERROR)
      {
        SHOWERROR("GpiIntersectClipRectangle", "failure");
      }

      // de-select the region
      if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
        SHOWERROR("GpiSetClipRegion", NULL);

      // test MaxLoop times in this range
      for (iValueLoop = 1; iValueLoop <= MaxLoop; iValueLoop++)
      {

        // start with a random rectangle in this range
        GetRandomRect(hPS, hWndClient);

        // switch to device coordinates for bounds calculation
        if ( !GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L,
                                                       (PPOINTL)&rcRect) )
          SHOWERROR("GpiConvert", NULL);

        // fetch the current bounds.
        if (GpiQueryRegionBox(hPS, hrgn, &rcBounds) == RGN_ERROR)
          SHOWERROR("GpiQueryRegionBox", NULL);

        // determine if the rectangle is within the current
        rcRect.xRight++;  // reflect exclusion of top and right of rect.
        rcRect.yTop++;
        if ((lRgnInside=GpiRectInRegion(hPS, hrgn, &rcRect)) == RRGN_ERROR)
        {
          SHOWERROR("GpiRectInRegion", NULL);
        }
        rcRect.xRight--;
        rcRect.yTop--;

        // lRgnInside has been set for new rect in current region

        // calculate new bounds for comparison
        switch (lRgnInside)
        {
          case RRGN_OUTSIDE:                  // rectangle not in region
            // no effect on rcBounds, do nothing!
            break;
          case RRGN_PARTIAL:
          case RRGN_INSIDE:
            // check to see if new region will be NULL
            if ( rcRect.xLeft <= rcBounds.xLeft &&
            rcRect.xRight + 1 >= rcBounds.xRight &&
               rcRect.yBottom <= rcBounds.yBottom &&
              rcRect.yTop + 1 >= rcBounds.yTop )
            {
              rcBounds.xLeft = 0L;   // set to NULL region
              rcBounds.yBottom = 0L;
              rcBounds.xRight = 0L;
              rcBounds.yTop = 0L;
            }
            else // non-null resultant region
            {
              // fetch the rectangles in the old region and compare each
              // rectangle to the new one, adjusting for clip and
              // recalculating rcBounds.
              Control.ircStart = 1;
              Control.crc = 5;
              Control.ulDirection = RECTDIR_LFRT_TOPBOT;

              // initialize bounds to opposites
              minxLeft   =  MAXRGNCOORD;
              maxxRight  =  MINRGNCOORD;
              minyBottom =  MAXRGNCOORD;
              maxyTop    =  MINRGNCOORD;
              do {
                // get the rectangles for region before new exclude rect.
                // NOTE: top and right sides are not inclusive!
                if (GpiQueryRegionRects(hPS, hrgn, NULL, &Control,
                                                   (PRECTL)&Rects) == FALSE)
                {
                  SHOWERROR("GpiQueryRegionRects", NULL);
                }
                else
                {
                  // process each rectangle.
                  for (i = 0; i < (INT)Control.crcReturned; i++)
                  {
                    // testing not done for clip from middle as this would
                    // not effect the bounds.

                    // test for complete clip of Rects[i]
                    if ( rcRect.yTop + 1 >=  Rects[i].yTop &&
                          rcRect.yBottom <= Rects[i].yBottom &&
                       rcRect.xRight + 1 >=  Rects[i].xRight &&
                            rcRect.xLeft <= Rects[i].xLeft )
                    {
                      continue; // skip to next Rects[i]
                    }

                    // test for partial clip on left/right side of Rects[i]
                    if ( rcRect.yTop + 1 >= Rects[i].yTop &&
                          rcRect.yBottom <= Rects[i].yBottom )
                    {
                      // test if partial on left
                      if ( rcRect.xLeft <= Rects[i].xLeft &&
                          rcRect.xRight >= Rects[i].xLeft &&
                          rcRect.xRight <  Rects[i].xRight )
                      {
                        // reset Rects[i].xLeft
                        Rects[i].xLeft = rcRect.xRight + 1;
                      }
                      // test for partial clip on right side of Rects[i]
                      if ( rcRect.xRight + 1 >= Rects[i].xRight &&
                                rcRect.xLeft >  Rects[i].xLeft &&
                            rcRect.xLeft + 1 <= Rects[i].xRight )
                      {
                        // reset Rects[i].xRight
                        Rects[i].xRight = rcRect.xLeft;
                      }
                    } /* if left or right side clip */

                    // test for partial clip on bottom/top side of Rects[i]
                    if ( rcRect.xRight + 1 >=  Rects[i].xRight &&
                              rcRect.xLeft <= Rects[i].xLeft )
                    {
                      // test if partial on bottom
                      if ( rcRect.yBottom <= Rects[i].yBottom &&
                              rcRect.yTop >= Rects[i].yBottom &&
                              rcRect.yTop <  Rects[i].yTop )
                      {
                        // reset Rects[i].yBottom
                        Rects[i].yBottom = rcRect.yTop + 1;
                      }
                      // test for partial clip on top side of Rects[i]
                      if ( rcRect.yTop + 1 >= Rects[i].yTop &&
                            rcRect.yBottom >  Rects[i].yBottom &&
                        rcRect.yBottom + 1 <= Rects[i].yTop )
                      {
                        // reset Rects[i].yTop
                        Rects[i].yTop = rcRect.yBottom;
                      }
                    } /* if top or bottom side clip */

                    // compare and adjust bounds
                    if (Rects[i].xLeft < minxLeft)
                      minxLeft = Rects[i].xLeft;
                    if (Rects[i].xRight > maxxRight)
                      maxxRight = Rects[i].xRight;
                    if (Rects[i].yBottom < minyBottom)
                      minyBottom = Rects[i].yBottom;
                    if (Rects[i].yTop > maxyTop)
                      maxyTop = Rects[i].yTop;

                  } /* for */
                } /* else GpiQueryRegionRects == FALSE */
                Control.ircStart += Control.crcReturned;
              } while (Control.crcReturned == Control.crc);

              // set rcBounds to new values
              rcBounds.xLeft = minxLeft;
              rcBounds.xRight = maxxRight;
              rcBounds.yBottom = minyBottom;
              rcBounds.yTop = maxyTop;
            }
            break;
        } /* end switch */

        // restore to world coordinates
        GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L, (PPOINTL)&rcRect);

        if (GpiSetClipRegion(hPS, hrgn, &hrgnNULL) == RGN_ERROR)
          SHOWERROR("GpiSetClipRegion", EntryName);

        // establish the new clip region, exclude new rectangle
        if ((lRgnType = GreExcludeClipRectangle(hDC,&rcRect)) == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreExcludeClipRectangle:", "failure" );
        }

        // de-select for bounds fetching
        if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
          SHOWERROR("GpiSetClipRegion", EntryName);

        // fetch current bounds for comparison
        if (GpiQueryRegionBox(hPS, hrgn, &rcCheck) == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GpiQueryRegionBox:", "failure" );
        }

        // compare calc to actual.
        if (   rcBounds.xLeft != rcCheck.xLeft ||
              rcBounds.xRight != rcCheck.xRight ||
             rcBounds.yBottom != rcCheck.yBottom ||
                rcBounds.yTop != rcCheck.yTop)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreExcludeClipRectangle:", "Field Mismatch");
          sprintf(szAddMessage,"Compare GpiQueryRegionBox:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcCheck.xLeft, rcCheck.xRight,
                               rcCheck.yBottom, rcCheck.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);
          sprintf(szAddMessage," to calculated bounds:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcBounds.xLeft, rcBounds.xRight,
                               rcBounds.yBottom, rcBounds.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);

          sprintf(szAddMessage,"xl=%ld, xr=%ld, yb=%ld, yt=%ld", rcRect.xLeft,
                                rcRect.xRight, rcRect.yBottom, rcRect.yTop);
          LOGINFO(L_TRACE,"*",szAddMessage);
        }

        // draw when concentrating on visible output device once in a while!
        // the DrawPsBox is very slow with clip regions established!
        if ((RangeLoop == 2) && (DeviceIsDisplay() || (iValueLoop % 20 == 0)))
        {
          sColor=randI(0,13);
          sPattern=randI(0,7);

          if (DeviceIsPrinter())
          {
            sColor = 1; // randI(0,1);
            if ( GpiSetColor(hPS, BW[sColor].lBWIndex) == FALSE)
              SHOWERROR("GpiSetColor:", BW[sColor].szBWIndex);
          }
          else
          {
            if ( GpiSetColor(hPS, clr[sColor].lColorIndex) == FALSE)
              SHOWERROR("GpiSetColor:", clr[sColor].szColorIndex);
          }

          if (GpiSetPattern(hPS, pat[sPattern].lPatternIndex) == FALSE)
            SHOWERROR("GpiSetPattern:", pat[sPattern].szPatternIndex);

          // activate the region for drawing.
          if (GpiSetClipRegion(hPS, hrgn, &hrgnNULL) == RGN_ERROR)
            SHOWERROR("GpiSetClipRegion", NULL);

          DrawPsBox(hPS, hWndClient);

          // de-select and save in handle
          if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
            SHOWERROR("GpiSetClipRegion", NULL);
        }

        // see if a region exists
        if (lRgnType == RGN_NULL || (iValueLoop % 20 == 0))
        {
          // create a region of some kind for next loop
          GetRandomRect(hPS, hWndClient);

          // delete existing region
          if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
            SHOWERROR("GpiDestroyRegion", NULL);

          if ((GpiIntersectClipRectangle(hPS, &rcRect)) == RGN_ERROR)
          {
            SHOWERROR("GpiIntersectClipRectangle", "failure");
          }

          // de-select the region
          if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
            SHOWERROR("GpiSetClipRegion", NULL);
        }  // if lRgnType == RGN_NULL


        TEST_PROGRESS ( RangeLoop * MaxLoop + iValueLoop, 3 * MaxLoop );

      }

      SUBEXIT();

    }

    // cleanup last region
    if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    EXIT_MAIN(EntryName);
  } // end else

  return;

} /* end GreExcludeClipRectangleExh */

/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreOffsetClipRegionExh() exhaustive algorithm
 *
 * Purpose: Exhaustively test GreOffsetClipRegion function
 *
 * System Requirements: Section 13.4.3
 *
 *      Revision Log: 01/10/91 Initial creation, Thom Bain
 *
 * Inputs:      hPS   -Presentations Space handle.
 *              hWndClient    -Client Window handle.
 *              SelectionCall -Selection Indicator, is 0 if routine is selected.
 *
 * Outputs: random patterns (and colors on display) are used in a one inch
 *          square clip region that if randomly moved around by the function
 *          test.
 *
 * Subroutines Required:   WinSendMsg, ABORTENTRY, strcmp, ENTER_MAIN,
 *                         DeviceIsPrinter, WinQueryAnchorBlock,
 *                         WinGetLastError, CLEANUP, WinQueryDevice,
 *                         SETENTER, srand, GpiQueryRegionBox, SETEXIT,
 *                         sprintf, SUBENTER, SetMaxRegion, randL, GpiConvert,
 *                         GpiIntersectClipRectangle, GpiSetClipRegion,
 *                         GpiRectInRegion, SHOWERROR, ERRSTATUS,
 *                         GreOffsetClipRegion, LOGINFO, GpiSetColor,
 *                         GpiSetPattern, GpiDestroyRegion,
 *                         randI, DrawPsBox, SUBEXIT, EXIT_MAIN
 *
 * Limitations:
 *                      Not Valid for OD_QUEUED, PM_Q_STD output.
 *
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *        if (SelectionCall)
 *        {
 *          WndSendMsg();
 *        }
 *        else
 *   {
 *      test for environmental restriction OD_QUEUED/PM_Q_STD
 *      Setup
 *      {
 *        Query for hDC from hPS;
 *        Set number of loops;
 *        Set random seed;
 *        fetch the handle to the anchor block for next step
 *        clear error buffer for ERRSTATUS by calling WinGetLastError
 *           }
 *
 *
 *      For three ranges extending from region limits to just visible region
 *      {
 *        establish rcLimits region coordinates for this range
 *        create a one inch square region at [1", 1"]
 *        query (GpiQueryRegionBox) current bounds
 *        convert rcLimits to device coords for bounds calculations
 *        item entry
 *        {
 *          generate a random offset based on the current bounds
 *            (GpiQueryRegionBox) and rcLimits range
 *          calculate new bounds for comparison, check for device transform
 *           round-off error by adding in offset in device coordinates and
 *           in world coordinates then converting the world coordinates to
 *           device coordinates and adjusting by the difference.
 *          compare calculated to actual bounds and log mismatch if any
 *            note that calculated and actual bounds are logged on mismatches
 *            and that they and the new rectangle are logged in device coords
 *          compare results to calculated resutlant region values
 *          if concentrating on visible device range
 *          {
 *            randomly set color and pattern (use black for printers)
 *            DrawPSBox
 *          }
 *        } // end item entry
 *      } // end three ranges
 *      destroy the last region
 *    } else
 *\\end
 ***************************************************************************/

VOID APIENTRY GreOffsetClipRegionExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  INT    iValueLoop,
         MaxLoop,
         RangeLoop;
  INT    sColor,
         sPattern;
  POINTL ptlOffset;
  RECTL  rcBounds, rcCheck, rcTemp;
  HAB    hAB;
  HDC    hDC;
  HRGN   hrgn, hrgnNULL;
  LONG   lNegOffset, lPosOffset;
  LONG   lRgnType;
  static CHAR *EntryName = "GreOffsetClipRegionExh";

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    MaxLoop = 100;  // how many times to test function in each range
    srand(0); // initialize the random number generator seed.

    SETEXIT();  // End of Set Up to Test.

    TEST_PROGRESS ( 00, 3 * MaxLoop );

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreOffsetClipRegionExh,szAddMessage);

      // establish rcLimits values for this RangeLoop
      SetMaxRegion(RangeLoop, hPS, hWndClient);

      // initialize Bounds rectangle
      rcRect.xLeft = MapX(100L);
      rcRect.xRight = MapX(200L);
      rcRect.yBottom = MapY(100L);
      rcRect.yTop = MapY(200L);

      // check for active region and destroy
      if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
        SHOWERROR("GpiSetClipRegion", NULL);
      if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
        SHOWERROR("GpiDestroyRegion", NULL);

      // establish this as the current clip region
      if ((lRgnType = GpiIntersectClipRectangle(hPS, &rcRect)) == RGN_ERROR)
      {
        SHOWERROR("GpiIntersectClipRectangle:", EntryName);
      }

      //de-select and put in region handle for bounds fetching
      if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
        SHOWERROR("GpiSetClipRegion", NULL);

      // fetch current bounds for comparison and offset range calculation
      if (GpiQueryRegionBox(hPS, hrgn, &rcCheck) == RGN_ERROR)
      {
        SHOWERROR("GpiQueryRegionBox:", "failure" );
      }

      if (GpiSetClipRegion(hPS, hrgn, &hrgnNULL) == RGN_ERROR)
        SHOWERROR("GpiSetClipRegion", NULL);

      // convert rcLimits to device coordinates for offset bounds calculations
      if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L, (PPOINTL)&rcLimits))
        SHOWERROR("GpiConvert", NULL);

      // test MaxLoop times in this range
      for (iValueLoop = 1; iValueLoop <= MaxLoop; iValueLoop++)
      {

        // get a random offset based on current range limits
        // filter numbers for possible invalid region coordinates.
        lNegOffset = rcLimits.xLeft - rcCheck.xLeft;
        if (lNegOffset < MINRGNCOORD) lNegOffset = MINRGNCOORD;
        if (lNegOffset > MAXRGNCOORD) lNegOffset = MAXRGNCOORD;
        lPosOffset = rcLimits.xRight - rcCheck.xRight;
        if (lPosOffset < MINRGNCOORD) lPosOffset = MINRGNCOORD;
        if (lPosOffset > MAXRGNCOORD) lPosOffset = MAXRGNCOORD;
        if (lNegOffset > lPosOffset)
          ptlOffset.x = randL(lPosOffset, lNegOffset);
        else
          ptlOffset.x = randL(lNegOffset, lPosOffset);

        lNegOffset = rcLimits.yBottom - rcCheck.yBottom;
        if (lNegOffset < MINRGNCOORD) lNegOffset = MINRGNCOORD;
        if (lNegOffset > MAXRGNCOORD) lNegOffset = MAXRGNCOORD;
        lPosOffset = rcLimits.yTop - rcCheck.yTop;
        if (lPosOffset < MINRGNCOORD) lPosOffset = MINRGNCOORD;
        if (lPosOffset > MAXRGNCOORD) lPosOffset = MAXRGNCOORD;
        if (lNegOffset > lPosOffset)
          ptlOffset.y = randL(lPosOffset, lNegOffset);
        else
          ptlOffset.y = randL(lNegOffset, lPosOffset);

        // make sure no world coordinate range problems
        if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 1L, &ptlOffset))
          SHOWERROR("GpiConvert", NULL);

        if (ptlOffset.x > MAXRGNCOORD) ptlOffset.x = MAXRGNCOORD;
        if (ptlOffset.x < MINRGNCOORD) ptlOffset.x = MINRGNCOORD;
        if (ptlOffset.y > MAXRGNCOORD) ptlOffset.y = MAXRGNCOORD;
        if (ptlOffset.y < MINRGNCOORD) ptlOffset.y = MINRGNCOORD;

        if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 1L, &ptlOffset))
          SHOWERROR("GpiConvert", NULL);

        // fetch the region handle
        if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
          SHOWERROR("GpiSetClipRegion", NULL);

        // fetch current bounds for comparison
        if (GpiQueryRegionBox(hPS, hrgn, &rcBounds) == RGN_ERROR)
        {
          SHOWERROR("GpiQueryRegionBox:", "failure" );
        }

        // restore the region from the handle
        if (GpiSetClipRegion(hPS, hrgn, &hrgnNULL) == RGN_ERROR)
          SHOWERROR("GpiSetClipRegion", NULL);

        // calculate new bounds for comparison, check for device transform
        // round-off error by adding in offset in device coordinates and
        // in world coordinates then converting the world coordinates to
        // device coordinates and adjusting by the difference.
        rcTemp.xLeft = rcBounds.xLeft + ptlOffset.x;
        rcTemp.yBottom = rcBounds.yBottom + ptlOffset.y;
        rcTemp.xRight = rcBounds.xRight + ptlOffset.x;
        rcTemp.yTop = rcBounds.yTop + ptlOffset.y;
        if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 1L, &ptlOffset))
          SHOWERROR("GpiConvert", NULL);
        if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L, (PPOINTL)&rcBounds))
          SHOWERROR("GpiConvert", NULL);
        rcBounds.xLeft += ptlOffset.x;
        rcBounds.yBottom += ptlOffset.y;
        rcBounds.xRight += ptlOffset.x;
        rcBounds.yTop += ptlOffset.y;
        GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L, (PPOINTL)&rcBounds);

        if (rcBounds.xLeft != rcTemp.xLeft)
          rcBounds.xLeft -= rcBounds.xLeft - rcTemp.xLeft;
        if (rcBounds.yBottom != rcTemp.yBottom)
          rcBounds.yBottom -= rcBounds.yBottom - rcTemp.yBottom;
        if (rcBounds.xRight != rcTemp.xRight)
          rcBounds.xRight -= rcBounds.xRight - rcTemp.xRight;
        if (rcBounds.yTop != rcTemp.yTop)
          rcBounds.yTop -= rcBounds.yTop - rcTemp.yTop;

        // establish the new clip region, offset by random point
        if ((lRgnType = GreOffsetClipRegion(hDC, &ptlOffset)) == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreOffsetClipRegion:", "failure" );
        }

        // fetch the region handle
        if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
          SHOWERROR("GpiSetClipRegion", NULL);

        // fetch current bounds for comparison
        if (GpiQueryRegionBox(hPS, hrgn, &rcCheck) == RGN_ERROR)
        {
          SHOWERROR("GpiQueryRegionBox:", "failure" );
        }

        // restore the region from the handle
        if (GpiSetClipRegion(hPS, hrgn, &hrgnNULL) == RGN_ERROR)
          SHOWERROR("GpiSetClipRegion", NULL);

        // convert to device coordinates for debug output
        if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 1L, &ptlOffset))
          SHOWERROR("GpiConvert", NULL);

        // compare calc to actual. (only need to check lower left corner)
        if (   rcBounds.xLeft != rcCheck.xLeft ||
              rcBounds.xRight != rcCheck.xRight ||
             rcBounds.yBottom != rcCheck.yBottom ||
                rcBounds.yTop != rcCheck.yTop)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreOffsetClipRegion:", "Field Mismatch");
          sprintf(szAddMessage,"Compare GpiQueryRegionBox:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcCheck.xLeft, rcCheck.xRight,
                               rcCheck.yBottom, rcCheck.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);
          sprintf(szAddMessage," to calculated bounds:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcBounds.xLeft, rcBounds.xRight,
                               rcBounds.yBottom, rcBounds.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);

          sprintf(szAddMessage,"Offset.x=%ld, Offset.y=%ld", ptlOffset.x,
                                                                ptlOffset.y);
          LOGINFO(L_TRACE,"*",szAddMessage);
        }

        // put bounds calculation into world coordinates for range calc.
        if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L,(PPOINTL)&rcBounds))
          SHOWERROR("GpiConvert", NULL);

        // draw when concentrating on visible output device once in a while!
        // the DrawPsBox is very slow with clip regions established!
        if ( RangeLoop == 2 )
        {
          sColor=randI(0,13);
          sPattern=randI(0,7);

          if (DeviceIsPrinter())
          {
            sColor=randI(0,1);
            if ( GpiSetColor(hPS, BW[sColor].lBWIndex) == FALSE)
              SHOWERROR("GpiSetColor:", BW[sColor].szBWIndex);
          }
          else
          {
            if ( GpiSetColor(hPS, clr[sColor].lColorIndex) == FALSE)
              SHOWERROR("GpiSetColor:", clr[sColor].szColorIndex);
          }

          if (GpiSetPattern(hPS, pat[sPattern].lPatternIndex) == FALSE)
            SHOWERROR("GpiSetPattern:", pat[sPattern].szPatternIndex);

          DrawPsBox(hPS, hWndClient);
        } // end if RangeLoop == 2


        TEST_PROGRESS ( RangeLoop * MaxLoop + iValueLoop, 3 * MaxLoop );

      } // end for(LoopVar)

      SUBEXIT();

    } // end for RangeVar

    // clean up!
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if ((hrgn != NULLHANDLE) && (GpiDestroyRegion(hPS, hrgn) == RGN_ERROR))
      SHOWERROR("GpiDestroyRegion", NULL);

    EXIT_MAIN(EntryName);
  } // end else

  return;

} /* end GreOffsetClipRegionExh */

/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreSelectAndQueryClipRegionExh() exhaustive algorithm
 *
 * Purpose: Exhaustively test GreSelectClipRegion and GreQueryClipRegion
 *
 * System Requirements: Section 13.4.4
 *
 *      Revision Log: 01/14/92 Initial creation, Thom Bain
 *
 * Inputs:      hPS   -Presentations Space handle.
 *              hWndClient    -Client Window handle.
 *              SelectionCall -Selection Indicator, is 0 if routine is selected.
 *
 * Outputs: random patterns (and colors on display) are used in random
 *          generated clip regions to draw a PS size pattern filled box
 *          after establishing a random clip region.
 *
 * Subroutines Required:   WinSendMsg, ABORTENTRY, strcmp, ENTER_MAIN,
 *     DeviceIsPrinter, WinQueryAnchorBlock, WinGetLastError, CLEANUP,
 *     WinQueryDevice, SETENTER, srand, GpiQueryRegionBox, SETEXIT, sprintf,
 *     SUBENTER, SetMaxRegion, GetRandomRect, randL, GpiConvert,
 *     DeviceIsDisplay, GreQueryClipRegion, GreSelectClipRegion,
 *     GpiRectInRegion, SHOWERROR, ERRSTATUS, GpiIntersectClipRectangle,
 *     LOGINFO, GpiSetColor, GpiSetPattern, GpiDestroyRegion, randI,
 *     DrawPsBox, SUBEXIT, EXIT_MAIN
 *
 * Limitations:
 *                      Not Valid for OD_QUEUED, PM_Q_STD output.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *        if (SelectionCall)
 *        {
 *          WndSendMsg();
 *        }
 *        else
 *   {
 *      test for environmental restriction OD_QUEUED/PM_Q_STD
 *      Setup
 *      {
 *        Query for hDC from hPS;
 *        Set number of loops;
 *        Set random seed;
 *        fetch the handle to the anchor block for next step
 *        clear error buffer for ERRSTATUS by calling WinGetLastError
 *           }
 *
 *
 *      For three ranges extending from region limits to just visible region
 *      {
 *        establish rcLimits values for this range
 *        delete any existing clip region (if any)
 *        create first region for this range to be visible page.
 *        item entry
 *        {
 *          get a random rectangle in the current range (GetRandomRect)
 *          create new region by intersecting rectangle with current by
 *            calling GpiIntersectClipRectangle with this rectangle
 *          get the new region handle (GreQueryClipRegion)
 *          deselect the active region and put into a different handle
 *            using GreSelectClipRegion
 *          compare the two regions with GpiEqualRegion and log failure
 *            if they do not match
 *          if range is concentrating on visible then DrawPSBox (only draw
 *            every 10th time for printers and don't try to draw in a NULL
 *            region to speed things up) Use randomly set color and pattern
 *            (use black for printers)
 *          if NULL region resulted reset to visible device region and
 *            destroy old.
 *        } // end item entry
 *      } // end three ranges
 *      cleanup any region left active.
 *    } // end else
 *\\end
 ***************************************************************************/

VOID APIENTRY GreSelectAndQueryClipRegionExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  INT    iValueLoop,
         MaxLoop,
         RangeLoop;
  INT    sColor,
         sPattern;
  HAB    hAB;
  HDC    hDC;
  HRGN   hrgnS, hrgnQ, hrgnNULL;
  LONG   lRgnType, rgnEqual;
  static CHAR *EntryName = "GreSelectAndQueryClipRegionExh";

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    MaxLoop = 100;  // how many times to test function in each range
    srand(0); // initialize the random number generator seed.

    SETEXIT();  // End of Set Up to Test.

    TEST_PROGRESS ( 00, 3 * MaxLoop );

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreSelectAndQueryClipRegionExh,szAddMessage);

      // establish rcLimits values for this RangeLoop
      SetMaxRegion(RangeLoop, hPS, hWndClient);

      // start with visible device for first clip region
      rcRect.xLeft = 0L;
      rcRect.yBottom = 0L;
      rcRect.xRight = pg_width - 1;
      rcRect.yTop = pg_height - 1;

      // delete existing region, if any
      if (GreSelectClipRegion(hDC, NULLHANDLE, &hrgnS) == RGN_ERROR)
      {
        ErrorCount++;
        ERRSTATUS(TRUE, "GreSelectClipRegion", "De-activating1");
      }
      if (hrgnS != NULLHANDLE && GpiDestroyRegion(hPS, hrgnS) == RGN_ERROR)
        SHOWERROR("GpiDestroyRegion", NULL);

      // create first region
      if ((GpiIntersectClipRectangle(hPS, &rcRect)) == RGN_ERROR)
        SHOWERROR("GpiIntersectClipRectangle", "failure");

      // test MaxLoop times in this range
      for (iValueLoop = 1; iValueLoop <= MaxLoop; iValueLoop++)
      {
        // get a random rectangle
        GetRandomRect(hPS, hWndClient);

        if ((lRgnType = GpiIntersectClipRectangle(hPS,&rcRect)) == RGN_ERROR)
        {
          SHOWERROR("GpiIntersectClipRectangle:", "failure" );
        }

        // Query for the region handle
        hrgnQ = GreQueryClipRegion(hDC);
        if (hrgnQ == HRGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreQueryClipRegion", NULL);
        }

        // fetch the region handle with GreSelectClipRegion
        if (GreSelectClipRegion(hDC, NULL, &hrgnS) == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreSelectClipRegion", "De-activating2");
        }

        rgnEqual = GpiEqualRegion(hPS, hrgnS, hrgnQ);
        switch (rgnEqual)
        {
          case EQRGN_NOTEQUAL:
            ErrorCount++;
            ERRSTATUS(TRUE, EntryName, "Region Mismatch");
            sprintf(szAddMessage,"xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                   rcRect.xLeft, rcRect.xRight, rcRect.yBottom, rcRect.yTop);
            LOGINFO(L_TRACE,"*",szAddMessage);
            break;
          case EQRGN_EQUAL:
            break;
          case EQRGN_ERROR:
            SHOWERROR("GpiEqualRegion", NULL);
            break;
        } // end switch

        // restore the region with GreSelectClipRegion
        if (GreSelectClipRegion(hDC, hrgnS, &hrgnNULL) == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreSelectClipRegion", "Restoring");
        }

        // establish random color and pattern and draw the a PS sized box
        // to show the region - don't do this if region is NULL or range
        // is not concentrating on the visible device, also only draw to
        // printers every tenth time to speed up testing.
        if ((RangeLoop == 2 && lRgnType != RGN_NULL)
            && (DeviceIsDisplay() || (iValueLoop % 10 == 0)))
        {
          sColor=randI(0,13);
          sPattern=randI(0,7);

          if (DeviceIsPrinter())
          {
            sColor= 1; // randI(0,1);
            if ( GpiSetColor(hPS, BW[sColor].lBWIndex) == FALSE)
              SHOWERROR("GpiSetColor:", BW[sColor].szBWIndex);
          }
          else
          {
            if ( GpiSetColor(hPS, clr[sColor].lColorIndex) == FALSE)
              SHOWERROR("GpiSetColor:", clr[sColor].szColorIndex);
          }

          if (GpiSetPattern(hPS, pat[sPattern].lPatternIndex) == FALSE)
            SHOWERROR("GpiSetPattern:", pat[sPattern].szPatternIndex);

          DrawPsBox(hPS, hWndClient);
        }

        // see if a region exists, if rectangles do not intersect
        // a NULL region should exist
        if (lRgnType == RGN_NULL)
        {
          // start with visible device for clip region
          rcRect.xLeft = 0L;
          rcRect.yBottom = 0L;
          rcRect.xRight = pg_width - 1;
          rcRect.yTop = pg_height - 1;

          // delete existing region
          if (GreSelectClipRegion(hDC, NULLHANDLE, &hrgnS) == RGN_ERROR)
          {
            ErrorCount++;
            ERRSTATUS(TRUE, "GreSelectClipRegion", "De-activating3");
          }
          if (hrgnS != NULLHANDLE && GpiDestroyRegion(hPS, hrgnS) == RGN_ERROR)
            SHOWERROR("GpiDestroyRegion", NULL);

          if ((GpiIntersectClipRectangle(hPS, &rcRect)) == RGN_ERROR)
          {
            SHOWERROR("GpiIntersectClipRectangle", "failure");
          }
        } // end if lRgnType


        TEST_PROGRESS ( RangeLoop * MaxLoop + iValueLoop, 3 * MaxLoop );
      }

      SUBEXIT();

    }

    // cleanup last region

    // fetch the region handle with GreSelectClipRegion
    if (GreSelectClipRegion(hDC, NULL, &hrgnS) == RGN_ERROR)
    {
      ErrorCount++;
      ERRSTATUS(TRUE, "GreSelectClipRegion", "De-activating4");
    }
    if (hrgnS != NULLHANDLE && GpiDestroyRegion(hPS, hrgnS) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    EXIT_MAIN(EntryName);
  } // end else

  return;

} /* end GreSelectAndQueryClipRegionExh */

/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreCopyClipRegionExh()
 *
 * Purpose: Exhaustively test GreCopyClipRegion
 *
 * System Requirements: Section 13.4.5
 *
 *      Revision Log: 01/16/92 Initial creation, Thom Bain
 *
 * Inputs:      hPS   -Presentations Space handle.
 *              hWndClient    -Client Window handle.
 *              SelectionCall -Selection Indicator, is 0 if routine is selected.
 *
 * Outputs: random patterns (and colors on display) are used in random
 *          generated DC regions to draw a PS size pattern filled box
 *          after establishing a random clip regions. The test copies this
 *          clip region and compares it to the current region with
 *          GpiEqualRegion to insure that the copy worked. It also copies
 *          this region with ALLINTERSECT and VISRGN options and compares
 *          the bounds given via GreCopyClipRegion function to that queried
 *          via GpiQueryRegionBox.
 *
 * Subroutines Required:   WinSendMsg, ABORTENTRY, strcmp, ENTER_MAIN,
 *     DeviceIsPrinter, WinQueryAnchorBlock, WinGetLastError, CLEANUP,
 *     WinQueryDevice, SETENTER, srand, GpiQueryRegionBox, SETEXIT, sprintf,
 *     SUBENTER, SetMaxRegion, GetRandomRect, randL, GpiConvert,
 *     DeviceIsDisplay, GpiSetClipRegion, GpiIntersectClipRectangle,
 *     GreCopyClipRegion, SHOWERROR, ERRSTATUS, GpiEqualRegion, LOGINFO
 *     GpiSetColor, GpiSetPattern, randI, DrawPsBox, SUBEXIT, EXIT_MAIN,
 *     GpiDestroyRegion.
 *
 * Limitations:
 *                      Not Valid for OD_QUEUED, PM_Q_STD output.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *        if (SelectionCall)
 *        {
 *          WndSendMsg();
 *        }
 *        else
 *   {
 *      test for environmental restriction OD_QUEUED/PM_Q_STD
 *      Setup
 *      {
 *        Query for hDC from hPS;
 *        Set number of loops;
 *        Set random seed;
 *        fetch the handle to the anchor block for next step
 *        clear error buffer for ERRSTATUS by calling WinGetLastError
 *        Create a clip region in the middle of the page half the page
 *          size in dimension. Save this region in hrgnC handle. This
 *          will be the handle for copying new regions over.
 *           }
 *
 *
 *      For three ranges extending from region limits to just visible region
 *      {
 *        establish rcLimits values for this range
 *        create a page sized clip region (deselecting and deleting any
 *          other that may still be active from previous range.
 *        item entry
 *        {
 *          get a random rectangle in the current range (GetRandomRect)
 *          create new region by intersecting rectangle with current by
 *            calling GpiIntersectClipRectangle with this rectangle
 *          copy this region into hrgnC handle with the COPYCRGN_CLIPRGN
 *            option.
 *          put the new region into hrgnS via GpiSetClipRegion
 *          compare the two regions with GpiEqualRegion and log failure
 *            if they do not match
 *          copy the new region into hrgnC again but use the
 *            COPYCRGN_ALLINTERSECT option and putting its bounds in a
 *            rectangle, rcAll, for comparsion to query.
 *          fetch DC region bounds into rcCheck rect. via GpiQueryRegionBox.
 *          compare the two rectangles rcAll and rcCheck. Log mismatches
 *          copy the new region into hrgnC again but use the
 *            COPYCRGN_VISRGN option and putting its bounds in a
 *            rectangle, rcVis, for comparsion to query.
 *          fetch DC region bounds into rcCheck rect. via GpiQueryRegionBox.
 *          compare the two rectangles rcAll and rcCheck. Log mismatches
 *          if range is concentrating on visible then DrawPSBox (only draw
 *            every 10th time for printers and don't try to draw in a NULL
 *            region to speed things up) Use randomly set color and pattern
 *            (use black for printers)
 *          if NULL region resulted reset to visible device region and
 *            destroy old.
 *        } // end item entry
 *      } // end three ranges
 *      cleanup the active region and the region in the copy handle
 *    } // end else
 *\\end
 ***************************************************************************/

VOID APIENTRY GreCopyClipRegionExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  INT    iValueLoop,
         MaxLoop,
         RangeLoop;
  INT    sColor,
         sPattern;
  HAB    hAB;
  HDC    hDC;
  HRGN   hrgnS, hrgnC, hrgnNULL;
  RECTL  rcVis, rcAll, rcCheck;
  LONG   lRgnType, rgnEqual, ErrorInCount;
  static CHAR *EntryName = "GreCopyClipRegionExh";

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    MaxLoop = 100;  // how many times to test function in each range
    srand(0); // initialize the random number generator seed.

    // create a region and save in hrgnC as the GreCopyClipRegion function
    // requires a real region to copy into, destroying its current contents
    rcRect.xLeft = pg_width / 4;
    rcRect.yBottom = pg_height / 4;
    rcRect.xRight = 3 * pg_width / 4;
    rcRect.yTop = 3 * pg_height / 4;

    // create region to be copied over
    if ((GpiIntersectClipRectangle(hPS, &rcRect)) == RGN_ERROR)
      SHOWERROR("GpiIntersectClipRectangle", "failure");

    // save into region handle to be copied over
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgnC) == RGN_ERROR)
    {
      SHOWERROR("GpiSetClipRegion", "De-activating1");
    }

    SETEXIT();  // End of Set Up to Test.

    TEST_PROGRESS ( 00, 3 * MaxLoop );

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreCopyClipRegionExh,szAddMessage);

      // establish rcLimits values for this RangeLoop
      SetMaxRegion(RangeLoop, hPS, hWndClient);

      // start with visible device for a region
      rcRect.xLeft = 0L;
      rcRect.yBottom = 0L;
      rcRect.xRight = pg_width - 1;
      rcRect.yTop = pg_height - 1;

      // delete existing region, if any
      if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgnS) == RGN_ERROR)
      {
        SHOWERROR("GpiSetClipRegion", "De-activating1");
      }
      if (hrgnS != NULLHANDLE && GpiDestroyRegion(hPS, hrgnS) == RGN_ERROR)
        SHOWERROR("GpiDestroyRegion", NULL);

      // create first region
      if ((GpiIntersectClipRectangle(hPS, &rcRect)) == RGN_ERROR)
        SHOWERROR("GpiIntersectClipRectangle", "failure");

      // test MaxLoop times in this range
      for (iValueLoop = 1; iValueLoop <= MaxLoop; iValueLoop++)
      {
        ErrorInCount = ErrorCount;
        // get a random rectangle
        GetRandomRect(hPS, hWndClient);

        if ((lRgnType = GpiIntersectClipRectangle(hPS,&rcRect)) == RGN_ERROR)
        {
          SHOWERROR("GpiIntersectClipRectangle:", "failure" );
        }

        // Copy the current clip region to the copy region (handle)
        if (GreCopyClipRegion(hDC, hrgnC, NULL, COPYCRGN_CLIPRGN)
                                                              == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreCopyClipRegion", "Clip Region");
        }

        // fetch the region handle with GpiSetClipRegion
        if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgnS) == RGN_ERROR)
        {
          SHOWERROR("GpiSetClipRegion", "De-activating2");
        }

        rgnEqual = GpiEqualRegion(hPS, hrgnS, hrgnC);
        switch (rgnEqual)
        {
          case EQRGN_NOTEQUAL:
            ErrorCount++;
            ERRSTATUS(TRUE, EntryName, "Clip Region Mismatch");
            break;
          case EQRGN_EQUAL:
            break;
          case EQRGN_ERROR:
            SHOWERROR("GpiEqualRegion", NULL);
            break;
        } // end switch

        // restore the region with GpiSetClipRegion
        if (GpiSetClipRegion(hPS, hrgnS, &hrgnNULL) == RGN_ERROR)
        {
          SHOWERROR("GpiSetClipRegion", "Restoring");
        }

        // Copy the current DC region to the All region (handle)
        if (GreCopyClipRegion(hDC, hrgnC, &rcAll, COPYCRGN_ALLINTERSECT)
                                                              == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreCopyClipRegion", "DC Region");
        }

        // fetch bounds from GpiQueryRegionBox and compare
        if (GpiQueryRegionBox(hPS, hrgnC, &rcCheck) == RGN_ERROR)
          SHOWERROR("GpiQueryRegionBox", NULL);
        if (rcAll.xLeft != rcCheck.xLeft &&
           rcAll.xRight != rcCheck.xRight &&
          rcAll.yBottom != rcCheck.yBottom &&
             rcAll.yTop != rcCheck.yTop )
        {
          ErrorCount++;
          ERRSTATUS(TRUE, EntryName, "DC Region Mismatch");
        }

        // Copy the current visible region to the region (handle)
        if (GreCopyClipRegion(hDC, hrgnC, &rcVis, COPYCRGN_VISRGN)
                                                              == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreCopyClipRegion", "Visible Region");
        }

        if (GpiQueryRegionBox(hPS, hrgnC, &rcCheck) == RGN_ERROR)
          SHOWERROR("GpiQueryRegionBox", NULL);

        if (rcVis.xLeft != rcCheck.xLeft &&
           rcVis.xRight != rcCheck.xRight &&
          rcVis.yBottom != rcCheck.yBottom &&
             rcVis.yTop != rcCheck.yTop )
        {
          ErrorCount++;
          ERRSTATUS(TRUE, EntryName, "Visible Region Mismatch");
        }

        if (ErrorCount > ErrorInCount)
        {
          sprintf(szAddMessage,"xl=%ld, xr=%ld, yb=%ld, yt=%ld", rcRect.xLeft,
                                rcRect.xRight, rcRect.yBottom, rcRect.yTop);
          LOGINFO(L_TRACE,"*",szAddMessage);
        }

        // draw only when concentrating on visible device and there is
        // a region to draw in. Don't draw but every tenth region when
        // the output device is a printer as this is very time consuming.
        if ((RangeLoop == 2 && lRgnType != RGN_NULL)
            && (DeviceIsDisplay() || (iValueLoop % 10 == 0)))
        {
          sColor = randI(0,13);
          sPattern = randI(0,7);

          if (DeviceIsPrinter())
          {
            sColor = 1; // randI(0,1);
            if ( GpiSetColor(hPS, BW[sColor].lBWIndex) == FALSE)
              SHOWERROR("GpiSetColor:", BW[sColor].szBWIndex);
          }
          else
          {
            if ( GpiSetColor(hPS, clr[sColor].lColorIndex) == FALSE)
              SHOWERROR("GpiSetColor:", clr[sColor].szColorIndex);
          }

          if (GpiSetPattern(hPS, pat[sPattern].lPatternIndex) == FALSE)
            SHOWERROR("GpiSetPattern:", pat[sPattern].szPatternIndex);

          DrawPsBox(hPS, hWndClient);
        }

        // see if a NULL region exists
        if (lRgnType == RGN_NULL)
        {
          // start with visible device for a clip region
          rcRect.xLeft = 0L;
          rcRect.yBottom = 0L;
          rcRect.xRight = pg_width - 1;
          rcRect.yTop = pg_height - 1;

          // delete existing region
          if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgnS) == RGN_ERROR)
          {
            SHOWERROR("GpiSetClipRegion", "De-activating3");
          }
          if (hrgnS != NULLHANDLE && GpiDestroyRegion(hPS, hrgnS) == RGN_ERROR)
            SHOWERROR("GpiDestroyRegion", NULL);

          if ((GpiIntersectClipRectangle(hPS, &rcRect)) == RGN_ERROR)
          {
            SHOWERROR("GpiIntersectClipRectangle", "failure");
          }
        } // end if lRgnType

        TEST_PROGRESS ( RangeLoop * MaxLoop + iValueLoop, 3 * MaxLoop );
      }

      SUBEXIT();

    } // end for RangeVar

    // cleanup - free the region space

    // fetch the region handle with GpiSetClipRegion
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgnS) == RGN_ERROR)
    {
      SHOWERROR("GpiSetClipRegion", "De-activating4");
    }
    if (hrgnS != NULLHANDLE && GpiDestroyRegion(hPS, hrgnS) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);
    // delete the copy (free up memory)
    if (hrgnC != NULLHANDLE && GpiDestroyRegion(hPS, hrgnC) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    EXIT_MAIN(EntryName);
  } // end else

  return;

} /* end GreCopyClipRegionExh */

/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreSetXformRectExh()
 *
 * Purpose: Exhaustively test GreSetXformRect function
 *
 * System Requirements: 13.4.6
 *
 *      Revision Log: 01/16/92 Initial creation, Thom Bain
 *
 * Inputs:      hPS   -Presentations Space handle.
 *              hWndClient    -Client Window handle.
 *              SelectionCall -Selection Indicator, is 0 if routine is selected.
 *
 * Outputs: random patterns (and colors on display) are used in random
 *          generated DC regions to draw a PS size pattern filled box
 *          after establishing a random DC region via GreSetXformRect
 *
 * Subroutines Required:   WinSendMsg, ABORTENTRY, strcmp, ENTER_MAIN,
 *     DeviceIsPrinter, WinQueryAnchorBlock, WinGetLastError, CLEANUP,
 *     WinQueryDevice, SETENTER, srand, GpiQueryClipBox, SETEXIT, sprintf,
 *     SUBENTER, SetMaxRegion, GetRandomRect, randL, GpiConvert,
 *     GpiRectVisible, DeviceIsDisplay, GreSetXformRect, SHOWERROR,
 *     ERRSTATUS, LOGINFO, GpiSetColor, GpiSetPattern, randI, DrawPsBox,
 *     SUBEXIT, EXIT_MAIN
 *
 * Limitations:
 *                      Not Valid for OD_QUEUED, PM_Q_STD output.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *        if (SelectionCall)
 *        {
 *          WndSendMsg();
 *        }
 *        else
 *   {
 *      test for environmental restriction OD_QUEUED/PM_Q_STD
 *      Setup
 *      {
 *        Query for hDC from hPS;
 *        Set number of loops;
 *        Set random seed;
 *        fetch the handle to the anchor block for next step
 *        clear error buffer for ERRSTATUS by calling WinGetLastError
 *        get the default visible region and save in rcVisible rectangle
 *           }
 *
 *
 *      For three ranges extending from region limits to just visible region
 *      {
 *        establish rcLimits values for this range
 *        item entry
 *        {
 *          get a random rectangle in the current range (GetRandomRect)
 *          query the current DC region bounds and save in rcBounds rect.
 *          determine if new rectangle would be visible in the current
 *            DC region via GpiRectVisible
 *          adjust rcBounds for intersection of new rectangle with current
 *            DC region. Use rcVisible and rectangle visibility from
 *            recent query to calculate new bounds (see comments in code)
 *          create new region by intersecting rectangle with current by
 *            calling GreSetXformRect with this rectangle (NOTE: the bounds
 *            calculation is done in world coordinates and as if the rect.
 *            was not exclusive at the top and right sides. The rectangle
 *            is converted to device coordinates and bumped up one for the
 *            right and top sides to adjust for this. This should prevent
 *            any device round-off errors)
 *          get the new bounds via GpiQueryClipBox
 *          compare calculated to actual and log mismatch if any.
 *          if range is concentrating on visible then DrawPSBox (only draw
 *            every 20th time for printers and don't try to draw in a NULL
 *            region to speed things up) Use randomly set color and pattern
 *            (use black for printers)
 *        } // end item entry
 *      } // end three ranges
 *    } // end else
 *\\end
 ***************************************************************************/

VOID APIENTRY GreSetXformRectExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  INT    iValueLoop,
         MaxLoop,
         RangeLoop;
  INT    sColor,
         sPattern;
  RECTL  rcBounds, rcCheck, rcVisible;
  HAB    hAB;
  HDC    hDC;
  LONG   lRgnType, lVisibility;
  static CHAR *EntryName = "GreSetXformRectExh";

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    MaxLoop = 100;  // how many times to test function in each range
    srand(0); // initialize the random number generator seed.

    // fetch the default visible bounds.
    if (GpiQueryClipBox(hPS, &rcVisible) == RGN_ERROR)
    {
      SHOWERROR("GpiQueryClipBox", NULL);
    }

    SETEXIT();  // End of Set Up to Test.

    TEST_PROGRESS ( 00, 3 * MaxLoop );

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreSetXformRectExh,szAddMessage);

      // establish rcLimits values for this RangeLoop
      SetMaxRegion(RangeLoop, hPS, hWndClient);

      // test MaxLoop times in this range
      for (iValueLoop = 1; iValueLoop <= MaxLoop; iValueLoop++)
      {
        // get a random rectangle
        GetRandomRect(hPS, hWndClient);

        // fetch the current bounds.
        if (GpiQueryClipBox(hPS, &rcBounds) == RGN_ERROR)
        {
          SHOWERROR("GpiQueryClipBox", NULL);
        }

        // see if rectangle is in current region
        lVisibility = GpiRectVisible(hPS, &rcRect);

        // calculate new rcBounds
        switch(lVisibility)
        {
          case RVIS_ERROR:
            SHOWERROR("GpiRectVisible", NULL);
            break;
          case RVIS_PARTIAL:
            // merge default visible with current bounds
            if (rcVisible.xLeft < rcBounds.xLeft)
              rcBounds.xLeft = rcVisible.xLeft;
            if (rcVisible.xRight > rcBounds.xRight)
              rcBounds.xRight = rcVisible.xRight;
            if (rcVisible.yTop > rcBounds.yTop)
              rcBounds.yTop = rcVisible.yTop;
            if (rcVisible.yBottom < rcBounds.yBottom)
              rcBounds.yBottom = rcVisible.yBottom;

            // now adjust for new rectangle intersection with current bounds
            if(rcRect.xLeft > rcBounds.xLeft)
              rcBounds.xLeft = rcRect.xLeft;
            if(rcRect.xRight < rcBounds.xRight)
              rcBounds.xRight = rcRect.xRight;
            if(rcRect.yBottom > rcBounds.yBottom)
              rcBounds.yBottom = rcRect.yBottom;
            if(rcRect.yTop < rcBounds.yTop)
              rcBounds.yTop = rcRect.yTop;

            // filter bounds with default visible bounds
            if (rcBounds.xLeft < rcVisible.xLeft)
              rcBounds.xLeft = rcVisible.xLeft;
            if (rcBounds.xRight > rcVisible.xRight)
              rcBounds.xRight = rcVisible.xRight;
            if (rcBounds.yTop > rcVisible.yTop)
              rcBounds.yTop = rcVisible.yTop;
            if (rcBounds.yBottom < rcVisible.yBottom)
              rcBounds.yBottom = rcVisible.yBottom;

            break;
          case RVIS_VISIBLE:
            // new bounds will be the new rectangle
            rcBounds = rcRect;
            break;
          case RVIS_INVISIBLE:
            // merge default visible with current bounds
            if (rcVisible.xLeft < rcBounds.xLeft)
              rcBounds.xLeft = rcVisible.xLeft;
            if (rcVisible.xRight > rcBounds.xRight)
              rcBounds.xRight = rcVisible.xRight;
            if (rcVisible.yTop > rcBounds.yTop)
              rcBounds.yTop = rcVisible.yTop;
            if (rcVisible.yBottom < rcBounds.yBottom)
              rcBounds.yBottom = rcVisible.yBottom;

            // check to see if outside the current bounds, if so a
            // NULL region will be created. Set bounds appropriately
            if (    rcRect.xLeft > rcBounds.xRight ||
                   rcRect.xRight < rcBounds.xLeft  ||
                  rcRect.yBottom > rcBounds.yTop   ||
                     rcRect.yTop < rcBounds.yBottom )
            {
              rcBounds.xRight  = 0L;
              rcBounds.xLeft   = 0L;
              rcBounds.yTop    = 0L;
              rcBounds.yBottom = 0L;
            }
            else
            {
              // adjust bounds based on new rectangle
              if (rcRect.xLeft > rcBounds.xLeft &&
                  rcRect.xLeft <= rcBounds.xRight)
                rcBounds.xLeft = rcRect.xLeft;
              if (rcRect.xRight < rcBounds.xRight &&
                  rcRect.xRight >= rcBounds.xLeft)
                rcBounds.xRight = rcRect.xRight;
              if (rcRect.yBottom > rcBounds.yBottom &&
                  rcRect.yBottom <= rcBounds.yTop)
                rcBounds.yBottom = rcRect.yBottom;
              if (rcRect.yTop < rcBounds.yTop &&
                  rcRect.yTop >= rcBounds.yBottom)
                rcBounds.yTop = rcRect.yTop;

              // filter bounds with default visible bounds
              if (rcBounds.xLeft < rcVisible.xLeft)
                rcBounds.xLeft = rcVisible.xLeft;
              if (rcBounds.xRight > rcVisible.xRight)
                rcBounds.xRight = rcVisible.xRight;
              if (rcBounds.yTop > rcVisible.yTop)
                rcBounds.yTop = rcVisible.yTop;
              if (rcBounds.yBottom < rcVisible.yBottom)
                rcBounds.yBottom = rcVisible.yBottom;
            }
            break;
        } // end switch

        // switch to device coordinates for GreSetXformRect
        if ( !GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L,
                                                       (PPOINTL)&rcRect) )
          SHOWERROR("GpiConvert", NULL);

        // adjust for exclusivity of top and right sides of rectangle
        // as all calculations were as if they weren't
        rcRect.xRight++;
        rcRect.yTop++;

        // establish the new DC region, intersect with new rectangle
        if ((lRgnType = GreSetXformRect(hDC,&rcRect)) == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreSetXformRect:", "failure" );
        }
        // adjust back for conversion back to world coordinates
        rcRect.xRight--;
        rcRect.yTop--;

        // convert back to world coordinates for possible logging
        if ( !GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L,
                                                       (PPOINTL)&rcRect) )
          SHOWERROR("GpiConvert", NULL);

        // get new bounds for comparison check
        if (GpiQueryClipBox(hPS, &rcCheck) == RGN_ERROR)
        {
          SHOWERROR("GpiQueryClipBox", NULL);
        }

        // compare returned by GreGetClipBox to calculated
        if (   rcBounds.xLeft != rcCheck.xLeft ||
              rcBounds.xRight != rcCheck.xRight ||
             rcBounds.yBottom != rcCheck.yBottom ||
                rcBounds.yTop != rcCheck.yTop)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, EntryName, "Field Mismatch");
          sprintf(szAddMessage,"Compare GreGetClipBox:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcCheck.xLeft, rcCheck.xRight,
                               rcCheck.yBottom, rcCheck.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);
          sprintf(szAddMessage," to calculated bounds:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcBounds.xLeft, rcBounds.xRight,
                               rcBounds.yBottom, rcBounds.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);

          sprintf(szAddMessage,"xl=%ld, xr=%ld, yb=%ld, yt=%ld", rcRect.xLeft,
                                rcRect.xRight, rcRect.yBottom, rcRect.yTop);
          LOGINFO(L_TRACE,"*",szAddMessage);
        }

        // drawing here insures that at least the most recent rectangle
        // will be drawn.
        if ((RangeLoop == 2) && (DeviceIsDisplay() || (iValueLoop % 20 == 0)))
        {
          sColor=randI(0,13);
          sPattern=randI(0,7);

          if (DeviceIsPrinter())
          {
            sColor= 1; // randI(0,1);
            if ( GpiSetColor(hPS, BW[sColor].lBWIndex) == FALSE)
              SHOWERROR("GpiSetColor:", BW[sColor].szBWIndex);
          }
          else
          {
            if ( GpiSetColor(hPS, clr[sColor].lColorIndex) == FALSE)
              SHOWERROR("GpiSetColor:", clr[sColor].szColorIndex);
          }

          if (GpiSetPattern(hPS, pat[sPattern].lPatternIndex) == FALSE)
            SHOWERROR("GpiSetPattern:", pat[sPattern].szPatternIndex);

          DrawPsBox(hPS, hWndClient);
        }

        TEST_PROGRESS ( RangeLoop * MaxLoop + iValueLoop, 3 * MaxLoop );
      }

      SUBEXIT();

    }

   // switch to device coordinates for GreSetXformRect
   if ( !GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L,
                                                  (PPOINTL)&rcVisible) )
     SHOWERROR("GpiConvert", NULL);

   // adjust for exclusivity of top and right sides of rectangle
   // as all calculations were as if they weren't
   rcVisible.xRight++;
   rcVisible.yTop++;

   // establish the new DC region, intersect with new rectangle
   if ((lRgnType = GreSetXformRect(hDC,&rcVisible)) == RGN_ERROR)
   {
     ErrorCount++;
     ERRSTATUS(TRUE, "GreSetXformRect:", "failure" );
   }

    EXIT_MAIN(EntryName);
  } // end else

  return;

} /* end GreSetXformRectExh */

/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreGetClipBoxAndRctsAndVChkExh
 *
 * Purpose: Exhaustively test GreGetClipBox, GreGetClipRects,
 *          GrePtVisible, and GreRectVisible functions
 *
 * System Requirements: Section 13.4.7
 *
 *      Revision Log: 01/20/92 Initial creation, Thom Bain
 *
 * Inputs:      hPS   -Presentations Space handle.
 *              hWndClient    -Client Window handle.
 *              SelectionCall -Selection Indicator, is 0 if routine is selected.
 *
 * Outputs: random patterns (and colors on display) are used in random
 *          generated DC regions to draw a PS size pattern filled box
 *          after establishing a random DC region via GpiExcludeClipRectangle
 *
 * Subroutines Required:   WinSendMsg, ABORTENTRY, strcmp, ENTER_MAIN,
 *     DeviceIsPrinter, WinQueryAnchorBlock, WinGetLastError, CLEANUP,
 *     WinQueryDevice, SETENTER, srand, GreGetClipBox, SETEXIT, sprintf,
 *     SUBENTER, SetMaxRegion, GetRandomRect, randL, GpiConvert,
 *     GreRectVisible, DeviceIsDisplay, GreGetClipRects, SHOWERROR,
 *     ERRSTATUS, LOGINFO, GpiSetColor, GpiSetPattern, randI, DrawPsBox,
 *     SUBEXIT, EXIT_MAIN, GpiExcludeClipRectangle, GrePtVisible
 *
 * Limitations:
 *                      Not Valid for OD_QUEUED, PM_Q_STD output.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *        if (SelectionCall)
 *        {
 *          WndSendMsg();
 *        }
 *        else
 *   {
 *      test for environmental restriction OD_QUEUED/PM_Q_STD
 *      Setup
 *      {
 *        Query for hDC from hPS;
 *        Set number of loops;
 *        Set random seed;
 *        fetch the handle to the anchor block for next step
 *        clear error buffer for ERRSTATUS by calling WinGetLastError
 *           }
 *
 *
 *      For three ranges extending from region limits to just visible region
 *      {
 *        establish rcLimits values for this range
 *        de-select and delete any existing clip region
 *        item entry
 *        {
 *          get a random rectangle in the current range (GetRandomRect)
 *          query the current DC region bounds and save in rcBounds rect.
 *          determine if new rectangle would be visible in the current
 *            DC region via GreRectVisible
 *          adjust rcBounds for exclusion of new rectangle with current
 *            DC region. Use rcVisible and rectangle visibility from
 *            recent query to calculate new bounds (see comments in code)
 *          create new region by excluding rectangle with current by
 *            calling GpiExcludeClipRectangle with this rectangle (NOTE: the
 *            bounds calculation is done in device coordinates and converted
 *            back to world for comparison.
 *          get the new bounds via GreGetClipBox
 *          compare calculated to actual and log mismatch if any.
 *          generate a random point in the current visible range (rcCheck)
 *          initialize flag to FALSE
 *          use GreGetClipRects to get the rectangles that make up this
 *            region and check each to see if the random point is inside
 *            one of them, if so set flag to TRUE and stop search.
 *          call GrePtVisible and compare its result to the flag and log
 *            any mismatch.
 *          if range is concentrating on visible then DrawPSBox (only draw
 *            every 20th time for printers and don't try to draw in a NULL
 *            region to speed things up) Use randomly set color and pattern
 *            (use black for printers)
 *        } // end item entry
 *      } // end three ranges
 *    } // end else
 *\\end
 ***************************************************************************/

VOID APIENTRY GreGetClipBoxAndRctsAndVChkExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  INT    iValueLoop,
         MaxLoop,
         RangeLoop;
  INT    sColor,
         sPattern;
  INT    i;
  RECTL  rcBounds, rcCheck, Rects[5];
  RGNRECT Control;
  HAB    hAB;
  HDC    hDC;
  HRGN   hrgn;
  LONG   minxLeft, maxxRight, minyBottom, maxyTop;
  LONG   lRgnType, lRgnInside, lPtVisible;
  POINTL ptlPtVisible;
  BOOL   fNull, fPtVisible;
  static CHAR *EntryName = "GreGetClipBoxAndRctsAndVChkExh";

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    MaxLoop = 100;  // how many times to test function in each range
    srand(0); // initialize the random number generator seed.

    SETEXIT();  // End of Set Up to Test.

    TEST_PROGRESS ( 00, 3 * MaxLoop );

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreGetClipBoxAndRctsAndVChkExh, szAddMessage);

      // establish rcLimits values for this RangeLoop
      SetMaxRegion(RangeLoop, hPS, hWndClient);

      // delete existing region, if any
      if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
        SHOWERROR("GpiSetClipRegion", NULL);
      if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
        SHOWERROR("GpiDestroyRegion", NULL);

      // test MaxLoop times in this range
      for (iValueLoop = 1; iValueLoop <= MaxLoop; iValueLoop++)
      {

        // start with a random rectangle in this range
        GetRandomRect(hPS, hWndClient);

        // fetch the current visible bounds.(in world coords)
        if (GreGetClipBox(hDC, &rcBounds) == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreGetClipBox", NULL);
        }

        // determine if the rectangle (in world coords) is visible;
        // i.e. will it effect visible bounds when excluded
        if ((lRgnInside=GreRectVisible(hDC, &rcRect)) == RVIS_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreRectVisible", NULL);
        }

        // lRgnInside has been set for new rect in current region

        // convert to device coordinates for bounds calculation
        if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L, (PPOINTL)&rcBounds))
          SHOWERROR("GpiConvert", NULL);
        if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 2L, (PPOINTL)&rcRect))
          SHOWERROR("GpiConvert", NULL);

        fNull = FALSE;
        // calculate new bounds for comparison
        switch (lRgnInside)
        {
          case RVIS_INVISIBLE:   // rectangle not in visible region
            // no effect on rcBounds, do nothing!
            break;
          case RVIS_PARTIAL:
          case RVIS_VISIBLE:
            // check to see if new region will be NULL
            if ( rcRect.xLeft <= rcBounds.xLeft &&
                rcRect.xRight >= rcBounds.xRight &&
               rcRect.yBottom <= rcBounds.yBottom &&
                  rcRect.yTop >= rcBounds.yTop )
            {
              fNull = TRUE;
              rcBounds.xLeft = 0L;   // set to NULL region
              rcBounds.yBottom = 0L;
              rcBounds.xRight = 0L;
              rcBounds.yTop = 0L;
            }
            else // non-null resultant region
            {
              // assume GetClipRects returns device coordinate and the top
              // and right boundaries are exclusive

              // fetch the rectangles in the old region and compare each
              // rectangle to the new one, adjusting for clip and
              // recalculating rcBounds.
              Control.ircStart = 1;
              Control.crc = 5;
              Control.ulDirection =
                 (USHORT)randL(RECTDIR_LFRT_TOPBOT, RECTDIR_RTLF_BOTTOP);

              // initialize bounds to opposites
              minxLeft   =  MAXRGNCOORD;
              maxxRight  =  MINRGNCOORD;
              minyBottom =  MAXRGNCOORD;
              maxyTop    =  MINRGNCOORD;
              do {
                // get the rectangles for region before new exclude rect.
                if (GreGetClipRects(hDC, NULL, &Control,
                                                   (PRECTL)&Rects) == FALSE)
                {
                  ErrorCount++;
                  ERRSTATUS(TRUE, "GreGetClipRects", NULL);
                }
                else
                {
                  // process each rectangle.
                  for (i = 0; i < (INT)Control.crcReturned; i++)
                  {
                    // testing not done for clip from middle as this would
                    // not effect the bounds.
                    Rects[i].yTop--;   // remove exlusive edges
                    Rects[i].xRight--;

                    // test for complete exclusion of Rects[i]
                    if (  rcRect.yTop >=  Rects[i].yTop &&
                       rcRect.yBottom <= Rects[i].yBottom &&
                        rcRect.xRight >=  Rects[i].xRight &&
                         rcRect.xLeft <= Rects[i].xLeft )
                      continue; // skip to next Rects[i]

                    // test for partial clip on left/right side of Rects[i]
                    if ( rcRect.yTop >= Rects[i].yTop &&
                      rcRect.yBottom <= Rects[i].yBottom )
                    {
                      // test if partial on left
                      if ( rcRect.xLeft <= Rects[i].xLeft &&
                          rcRect.xRight >= Rects[i].xLeft &&
                          rcRect.xRight <  Rects[i].xRight )
                      {
                        // reset Rects[i].xLeft
                        Rects[i].xLeft = rcRect.xRight + 1;
                      }
                      // test for partial clip on right side of Rects[i]
                      if ( rcRect.xRight >= Rects[i].xRight &&
                            rcRect.xLeft >  Rects[i].xLeft &&
                            rcRect.xLeft <= Rects[i].xRight )
                      {
                        // reset Rects[i].xRight
                        Rects[i].xRight = rcRect.xLeft - 1;
                      }
                    } /* if left or right side clip */

                    // test for partial clip on bottom/top side of Rects[i]
                    if ( rcRect.xRight >=  Rects[i].xRight &&
                          rcRect.xLeft <= Rects[i].xLeft )
                    {
                      // test if partial on bottom
                      if ( rcRect.yBottom <= Rects[i].yBottom &&
                              rcRect.yTop >= Rects[i].yBottom &&
                              rcRect.yTop <  Rects[i].yTop )
                      {
                        // reset Rects[i].yBottom
                        Rects[i].yBottom = rcRect.yTop + 1;
                      }
                      // test for partial clip on top side of Rects[i]
                      if ( rcRect.yTop >= Rects[i].yTop &&
                        rcRect.yBottom >  Rects[i].yBottom &&
                        rcRect.yBottom <=  Rects[i].yTop )
                      {
                        // reset Rects[i].yTop
                        Rects[i].yTop = rcRect.yBottom - 1;
                      }
                    } /* if top or bottom side clip */

                    // compare and adjust bounds
                    if (Rects[i].xLeft < minxLeft)
                      minxLeft = Rects[i].xLeft;
                    if (Rects[i].xRight > maxxRight)
                      maxxRight = Rects[i].xRight;
                    if (Rects[i].yBottom < minyBottom)
                      minyBottom = Rects[i].yBottom;
                    if (Rects[i].yTop > maxyTop)
                      maxyTop = Rects[i].yTop;

                  } /* for */
                } /* else GreGetClipRects == FALSE */
                Control.ircStart += Control.crcReturned;
              } while (Control.crcReturned == Control.crc);
              // set rcBounds to new values
              rcBounds.xLeft = minxLeft;
              rcBounds.xRight = maxxRight;
              rcBounds.yBottom = minyBottom;
              rcBounds.yTop = maxyTop;
            }
            break;
        } /* end switch */

        // convert back to world coordinates for Gpi to establish new region
        if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L, (PPOINTL)&rcRect))
          SHOWERROR("GpiConvert", NULL);

        // establish the new clip region, exclude new rectangle
        if ((lRgnType = GpiExcludeClipRectangle(hPS,&rcRect)) == RGN_ERROR)
        {
          SHOWERROR("GpiExcludeClipRectangle:", "failure" );
        }

        // fetch current bounds for comparison
        if (GreGetClipBox(hDC, &rcCheck) == RGN_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreGetClipBox:", "failure" );
        }

        if (!fNull && !GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L,
                                                         (PPOINTL)&rcBounds))
          SHOWERROR("GpiConvert", NULL);

        // compare calc to actual.
        if (   rcBounds.xLeft != rcCheck.xLeft ||
              rcBounds.xRight != rcCheck.xRight ||
             rcBounds.yBottom != rcCheck.yBottom ||
                rcBounds.yTop != rcCheck.yTop)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreGetClipRects:", "Field Mismatch");
          sprintf(szAddMessage,"Compare GreGetClipBox:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcCheck.xLeft, rcCheck.xRight,
                               rcCheck.yBottom, rcCheck.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);
          sprintf(szAddMessage," to calculated bounds:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcBounds.xLeft, rcBounds.xRight,
                               rcBounds.yBottom, rcBounds.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);

          sprintf(szAddMessage,"xl=%ld, xr=%ld, yb=%ld, yt=%ld", rcRect.xLeft,
                                rcRect.xRight, rcRect.yBottom, rcRect.yTop);
          LOGINFO(L_TRACE,"*",szAddMessage);
        }

        // generate a random point somewhere in current visible bounds
        ptlPtVisible.x = randL(rcCheck.xLeft, rcCheck.xRight);
        ptlPtVisible.y = randL(rcCheck.yBottom, rcCheck.yTop);

        // use GrePtVisible to see random point is visible
        lPtVisible = GrePtVisible(hDC, &ptlPtVisible);
        if (lPtVisible == PVIS_ERROR)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GrePtVisible", NULL);
        }

        // convert to device coordinates for rectangle comparisons
        if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 1L, &ptlPtVisible))
          SHOWERROR("GpiConvert", NULL);

        fPtVisible = FALSE;

        // fetch the rectangles in the new region and check if point
        // is inside any rectangle, if so set flag to TRUE
        Control.ircStart = 1;
        Control.crc = 5;
        Control.ulDirection =
           (USHORT)randL(RECTDIR_LFRT_TOPBOT, RECTDIR_RTLF_BOTTOP);

        do {
          // get the rectangles for region
          if (GreGetClipRects(hDC, NULL, &Control,
                                             (PRECTL)&Rects) == FALSE)
          {
            ErrorCount++;
            ERRSTATUS(TRUE, "GreGetClipRects", NULL);
          }
          else
          {
            // process each rectangle.
            for (i = 0; i < (INT)Control.crcReturned; i++)
            {
              Rects[i].yTop--;   // remove exlusive edges
              Rects[i].xRight--;

              // see if random point will be visible
              if (!fPtVisible &&
                  ptlPtVisible.x >= Rects[i].xLeft &&
                  ptlPtVisible.x <= Rects[i].xRight &&
                  ptlPtVisible.y >= Rects[i].yBottom &&
                  ptlPtVisible.y <= Rects[i].yTop)
                fPtVisible = TRUE;

            } /* for */
          } /* else GreGetClipRects == FALSE */
          Control.ircStart += Control.crcReturned;
        } while ((Control.crcReturned == Control.crc) && !fPtVisible);

        // convert random point back to world coordinates
        if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 1L, &ptlPtVisible))
          SHOWERROR("GpiConvert", NULL);

        // see if point visibility chech matches expected result
        if (fPtVisible != (BOOL)(lPtVisible == PVIS_VISIBLE))
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GrePtVisible", "Unexpected Value");

          sprintf(szAddMessage,"Point: x=%ld, y=%ld is ",
                                             ptlPtVisible.x, ptlPtVisible.y);
          if (lPtVisible == PVIS_VISIBLE)
            strcat(szAddMessage, "PVIS_VISIBLE");
          else
            strcat(szAddMessage, "PVIS_INVISIBLE");
          LOGINFO(L_TRACE,"@",szAddMessage);
        }

        // draw when concentrating on visible output device once in a while!
        // the DrawPsBox is very slow with clip regions established!
        if ((RangeLoop == 2) && (DeviceIsDisplay() || (iValueLoop % 20 == 0)))
        {
          sColor=randI(0,13);
          sPattern=randI(0,7);

          if (DeviceIsPrinter())
          {
            sColor = 1; // randI(0,1);
            if ( GpiSetColor(hPS, BW[sColor].lBWIndex) == FALSE)
              SHOWERROR("GpiSetColor:", BW[sColor].szBWIndex);
          }
          else
          {
            if ( GpiSetColor(hPS, clr[sColor].lColorIndex) == FALSE)
              SHOWERROR("GpiSetColor:", clr[sColor].szColorIndex);
          }

          if (GpiSetPattern(hPS, pat[sPattern].lPatternIndex) == FALSE)
            SHOWERROR("GpiSetPattern:", pat[sPattern].szPatternIndex);

          DrawPsBox(hPS, hWndClient);

        }

        // see if a region exists
        if (lRgnType == RGN_NULL || (iValueLoop % 20 == 0))
        {
          // de-select the region
          if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
            SHOWERROR("GpiSetClipRegion", NULL);
          // delete existing region
          if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
            SHOWERROR("GpiDestroyRegion", NULL);
        }  // if lRgnType == RGN_NULL

        TEST_PROGRESS ( RangeLoop * MaxLoop + iValueLoop, 3 * MaxLoop );
      }

      SUBEXIT();
    }

    // cleanup last region
    // de-select the region
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    EXIT_MAIN(EntryName);
  } // end else

  return;

} /* end GreGetClipBoxAndRctsAndVChkExh */

/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreSelectPathRegionExh() exhaustive algorithm
 *
 * Purpose: exhaustively test GreSelectPathRegion
 *
 * System Requirements: Section 13.4.8
 *
 *      Revision Log: 01/23/92 Initial creation , Thom Bain
 *
 * Inputs: hPS           -Presentations Space handle.
 *                        hWndClient    -Client Window handle.
 *         SelectionCall -Selection Indicator, is 0 if routine is selected.
 *
 * Outputs: randomly generated rectangles with part of center excluded.
 *
 * Subroutines Required: WndSendMsg, DeviceIsPrinter, strcmp, ABORTENTRY,
 *    SHOWERROR, WinQueryAnchorBlock, WinGetLastError, ENTER_MAIN, SETENTER,
 *    GpiQueryDevice, GpiQueryClipBox, srand, GpiBeginPath, GpiMove, GpiBox,
 *    GpiEndPath, GpiSetClipPath, SETEXIT, sprintf, SUBENTER, SetMaxRegion,
 *    randL, GpiConvert, GpiExcludeClipRectangle, GreSelectPathRegion,
 *    ERRSTATUS, LOGINFO, DeviceIsDisplay, randI, GpiSetColor, GpiSetPattern,
 *    DrawPsBox, SUBEXIT, EXIT_MAIN
 *
 * Limitations: OD_QUEUED/PM_Q_STD restricted.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *        if (SelectionCall)
 *        {
 *          WndSendMsg();
 *        }
 *        else
 *   {
 *      test for environmental restriction OD_QUEUED/PM_Q_STD
 *      Setup
 *      {
 *        Query for hDC from hPS;
 *        Set number of loops;
 *        Set random seed;
 *        fetch the handle to the anchor block for next step
 *        clear error buffer for ERRSTATUS by calling WinGetLastError
 *        get the default visible bounds and save for bounds calculations
 *        determine limits of path size (up to device size) that won't
 *          cause PMERR_PATH_LIMIT_EXCEEDED (do binary search)
 *           }
 *
 *      For three ranges extending from region limits to just visible region
 *      {
 *        establish rcLimits values for this range
 *        item entry
 *        {
 *          get a random rectangle size (from range calculated above)
 *          get a random point for lower left corner such that the whole
 *            rectangle will remain in the current limits.
 *          create a path with this box and select it as the active path
 *          create a clip region that will exclude about 50% of the middle
 *            of the path rectangle.
 *          deselect the clip region and put it in a handle
 *          merge this region with the path (GreSelectPathRegion)
 *          Query the current visible bounds with this path established
 *          Use the rectangle and saved visible bounds and calculate the
 *            expected visible bounds.
 *          compare calculated to actual and log mismatch if any.
 *          if range is concentrating on visible then DrawPSBox (only draw
 *            every 20th time for printers and every other time for displays
 *            Use randomly set color and pattern (use black for printers)
 *          destroy the path and reset to none selected (GreSelectPathRegion)
 *        } // end item entry
 *      } // end three ranges
 *    } // end else
 *      }
 *\\end
 ***************************************************************************/

VOID APIENTRY GreSelectPathRegionExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  INT    iValueLoop,
         MaxLoop,
         RangeLoop;
  INT    sColor,
         sPattern;
  LONG   lMaxXsize, lMaxYsize, lXAmount, lYAmount;
  LONG   lCurXsize, lCurYsize;
  POINTL ptlPathLLC, ptlPathURC;
  RECTL  rcExclude, rcBounds, rcCheck, rcVisible;
  BOOL   fUp;
  HAB    hAB;
  HDC    hDC;
  HRGN   hrgnP;
  static CHAR *EntryName = "GreSelectPathRegionExh";

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    // get the default visible bounds
    if (GpiQueryClipBox(hPS, &rcVisible) == RGN_ERROR)
      SHOWERROR("GpiQueryClipBox", NULL);

    MaxLoop = 100;  // how many times to test function in each range
    srand(0); // initialize the random number generator seed.

    // establish limits for box size
    // Do binary search of up to device size path
    ptlPathLLC.x = ptlPathLLC.y = 0L;
    lMaxXsize = pg_width / 2;   // start in middle
    lMaxYsize = pg_height / 2;
    lXAmount = lMaxXsize / 2;   // step size starts at half that
    lYAmount = lMaxYsize / 2;
    fUp = FALSE;   // direction flag for binary search
    do {
      ptlPathURC.x = lMaxXsize;   // set box size to current attempt size
      ptlPathURC.y = lMaxYsize;

      // create the path
      if (GpiBeginPath(hPS, 1L) == FALSE)
      {
        SHOWERROR("GpiBeginPath", EntryName);
      }

      GpiMove(hPS, &ptlPathLLC);
      GpiBox(hPS, DRO_OUTLINE, &ptlPathURC, 0L, 0L);

      if (GpiEndPath(hPS) == FALSE)
      {
        SHOWERROR("GpiEndPath", EntryName);
      }

      if (GpiSetClipPath(hPS, 1L, SCP_ALTERNATE | SCP_AND) == FALSE)
      {
        // this attempt failed, go down to next step
        fUp = FALSE;    // indicate this step is downward
        lMaxXsize -= lXAmount;
        lMaxYsize -= lYAmount;
      }
      else  // this attempt succeeded, try next step up
      {
        fUp = TRUE;    // indicate this step is upward
        lMaxXsize += lXAmount;
        lMaxYsize += lYAmount;
      }

      // discard the clip path - so can create another one!
      if (GpiSetClipPath(hPS, 0L, SCP_RESET) == FALSE)
      {
        SHOWERROR("GpiSetClipPath", EntryName);
      }

      // do next value in binary search
      lXAmount /= 2;
      lYAmount /= 2;
    } while(lXAmount > 0L && lYAmount > 0L);

    SETEXIT();

    TEST_PROGRESS ( 00, 3 * MaxLoop );

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreSelectPathRegion,szAddMessage);

      // establish rcLimits values for this RangeLoop
      SetMaxRegion(RangeLoop, hPS, hWndClient);

      // test MaxLoop times in this range
      for (iValueLoop = 1; iValueLoop <= MaxLoop; iValueLoop++)
      {
        // get a point in this range to create path at
        lCurXsize = randL(50L, lMaxXsize);  // get random size in max range
        lCurYsize = randL(50L, lMaxYsize);
        // get random lower left corner but be sure that path won't
        // overflow with PMERR_COORDINATE_OVERFLOW
        ptlPathLLC.x = randL(rcLimits.xLeft, rcLimits.xRight - lCurXsize);
        ptlPathLLC.y = randL(rcLimits.yBottom, rcLimits.yTop - lCurYsize);
        // add size to lower left corner for upper right coordinates
        ptlPathURC.x = ptlPathLLC.x + lCurXsize;
        ptlPathURC.y = ptlPathLLC.y + lCurYsize;
        // stabalize this value to avoid round-off errors from device xform
        if (!GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 1L, &ptlPathLLC) ||
            !GpiConvert(hPS, CVTC_WORLD, CVTC_DEVICE, 1L, &ptlPathURC) ||
            !GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 1L, &ptlPathLLC) ||
            !GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 1L, &ptlPathURC) )
          SHOWERROR("GpiConvert", NULL);

        // create the path
        if (GpiBeginPath(hPS, 1L) == FALSE)
        {
          SHOWERROR("GpiBeginPath", EntryName);
        }

        GpiMove(hPS, &ptlPathLLC);
        GpiBox(hPS, DRO_OUTLINE, &ptlPathURC, 0L, 0L);

        if (GpiEndPath(hPS) == FALSE)
        {
          SHOWERROR("GpiEndPath", EntryName);
        }

        // establish the circle path as the clip path
        if (GpiSetClipPath(hPS, 1L, SCP_ALTERNATE | SCP_AND) == FALSE)
        {
          SHOWERROR("GpiSetClipPath", EntryName);
        }

        // create a region to exclude part of path region
        rcExclude.xLeft   = ptlPathLLC.x + (lCurXsize / 5L);
        rcExclude.xRight  = ptlPathURC.x - (lCurXsize / 5L);
        rcExclude.yBottom = ptlPathLLC.y + (lCurYsize / 4L);
        rcExclude.yTop    = ptlPathURC.y - (lCurYsize / 4L);

        if (GpiExcludeClipRectangle(hPS, &rcExclude) == RGN_ERROR)
          SHOWERROR("GpiExcludeClipRectangle", NULL);

        // save region handle and set to no region selected
        if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgnP) == RGN_ERROR)
          SHOWERROR("GpiSetClipRegion", NULL);

        // re-establish clip path via region handle and gre under test
        if (GreSelectPathRegion(hDC, hrgnP) == FALSE)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreSelectPathRegion:", "select");
        }

        // fetch the bounds and compare to expected
        if (GpiQueryClipBox(hPS, &rcBounds) == RGN_ERROR)
        {
          SHOWERROR("GpiQueryClipBox", NULL);
        }

        // calculate expected bounds data
        rcCheck.xLeft   = ptlPathLLC.x;
        rcCheck.xRight  = ptlPathURC.x;
        rcCheck.yBottom = ptlPathLLC.y;
        rcCheck.yTop    = ptlPathURC.y;
        // test for completely outside visible region
        if ((ptlPathLLC.x > rcVisible.xRight) ||
            (ptlPathLLC.y > rcVisible.yTop) ||
            (ptlPathURC.x < rcVisible.xLeft) ||
            (ptlPathURC.y < rcVisible.yBottom) )
        {
          // indicate NULL region
          rcCheck.xLeft=rcCheck.xRight=rcCheck.yBottom=rcCheck.yTop = 0L;
        }
        else  // adjust to default visible bounds
        {
          if (ptlPathLLC.x < rcVisible.xLeft)
            rcCheck.xLeft = rcVisible.xLeft;
          if (ptlPathLLC.y < rcVisible.yBottom)
            rcCheck.yBottom = rcVisible.yBottom;
          if (ptlPathURC.x > rcVisible.xRight)
            rcCheck.xRight = rcVisible.xRight;
          if (ptlPathURC.y > rcVisible.yTop)
            rcCheck.yTop = rcVisible.yTop;
        }

        // compare calc to actual.
        if (   rcBounds.xLeft != rcCheck.xLeft ||
              rcBounds.xRight != rcCheck.xRight ||
             rcBounds.yBottom != rcCheck.yBottom ||
                rcBounds.yTop != rcCheck.yTop)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, EntryName, "Region Mismatch");
          sprintf(szAddMessage,"Compare GpiQueryClipBox:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcBounds.xLeft, rcBounds.xRight,
                               rcBounds.yBottom, rcBounds.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);
          sprintf(szAddMessage," to calculated bounds:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcCheck.xLeft, rcCheck.xRight,
                               rcCheck.yBottom, rcCheck.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);

          sprintf(szAddMessage,"xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                    ptlPathLLC.x, ptlPathURC.x, ptlPathLLC.y, ptlPathURC.y);
          LOGINFO(L_TRACE,"*",szAddMessage);
        }

        // draw when concentrating on visible output device once in a while!
        // the DrawPsBox is very slow with clip regions established!
        if ((RangeLoop == 2) && ((DeviceIsDisplay() && (iValueLoop % 2 == 1))
                                                  || (iValueLoop % 20 == 0)))
        {
          sColor=randI(0,13);
          sPattern=randI(0,7);

          if (DeviceIsPrinter())
          {
            sColor= 1; // randI(0,1);
            if ( GpiSetColor(hPS, BW[sColor].lBWIndex) == FALSE)
              SHOWERROR("GpiSetColor:", BW[sColor].szBWIndex);
          }
          else
          {
            if ( GpiSetColor(hPS, clr[sColor].lColorIndex) == FALSE)
              SHOWERROR("GpiSetColor:", clr[sColor].szColorIndex);
          }

          if (GpiSetPattern(hPS, pat[sPattern].lPatternIndex) == FALSE)
            SHOWERROR("GpiSetPattern:", pat[sPattern].szPatternIndex);

          DrawPsBox(hPS, hWndClient);
        } // end if RangeLoop == 2

        // discard the clip path - so can create another one!
        if (GpiSetClipPath(hPS, 0L, SCP_RESET) == FALSE)
        {
          SHOWERROR("GpiSetClipPath", EntryName);
        }

        // reset to no clip path
        if (GreSelectPathRegion(hDC, NULL) == FALSE)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreSelectPathRegion:", "reset");
        }

        TEST_PROGRESS ( RangeLoop * MaxLoop + iValueLoop, 3 * MaxLoop );
      }

      SUBEXIT();
    }

    EXIT_MAIN(EntryName);
  } // end else

  return;

} // end GreSelectPathRegion exhaustive test

// The following two functions are still under development. I will probably
// have to go to WM_TEST_DOES_DEV_CALLS and create my own DC. The functions
// under test seems to function OK in that they don't error but it also seems
// to have no effect. GreSaveRegion doesn't change anything (perhaps ok) but
// GreRestoreRegion hangs (infinite loop) on any save level other than three.

#define GETBOUNDS(a)                                \
   if (GpiQueryClipBox(hpsTest, &a) == RGN_ERROR)   \
     SHOWERROR("GpiQueryClipBox", NULL)

#define SHOWBOUNDS(a,b,c) GpiQueryClipBox(hpsTest, &c);                    \
     sprintf(szAddMessage," %s %d:\n xl=%ld, xr=%ld, yb=%ld, yt=%ld",      \
                              a, b, c.xLeft, c.xRight, c.yBottom, c.yTop); \
     LOGINFO(L_TRACE,"@",szAddMessage)

// Section 13.4.10
VOID APIENTRY GreSaveAndRestoreRegionExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static CHAR *EntryName = "GreSaveAndRestoreRegionExh";
  INT    iValueLoop,
         MaxLoop,
         RangeLoop;
//  INT    sColor,
//         sPattern;
  RECTL  rcBounds, rcCheck;
  HAB    hAB;
  HDC    hDC, hdcTest;
  HPS    hpsTest;
//  HRGN   hrgn, hrgnNULL;
  LONG   lRgnType, cSave;
  SIZEL   sizel;          //Size PS to create.
  ULONG   flOptions;
  LONG    idDC;

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    if (DeviceIsDisplay())
    {
      ABORTENTRY(EntryName, "Cannot create DCs for displays");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // get device open structure for creating new DC
    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    // fetch DTT selections for environment to use for new PS
    flOptions = GpiQueryPS(hPS, &sizel);
    // turn off PS_NORESET if on
    flOptions ^= (flOptions & PS_NORESET);
    flOptions |= PS_ASSOCIATE; // make sure associate flag is on

    MaxLoop = 20;  // how many times to test function in each range
    srand(0); // initialize the random number generator seed.

    SETEXIT();  // End of Set Up to Test.

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreSaveAndRestoreRegionExh, szAddMessage);

      // establish rcLimits values for this RangeLoop
      SetMaxRegion(RangeLoop, hPS, hWndClient);

      // test MaxLoop times in this range
      for (iValueLoop = 1; iValueLoop <= MaxLoop; iValueLoop++)
      {

        // Open a device context that will hopefully work with Save & Restore
        if ((hdcTest = GreOpenDC( (HDC)hDC, lDCType, "*", 9L,
                                       (PDEVOPENDATA)&dop)) == DEV_ERROR)
        {
          CLEANUP("GreOpenDC",EntryName);
          return;
        }

        // start the document
        if (DevEscape(hdcTest, DEVESC_STARTDOC, 0L ,NULL, NULL, NULL)
                                                            == DEVESC_ERROR)
          SHOWERROR("DevEscape",EntryName);

        // Create a presentation space to associate with the Device Context
        if ((hpsTest=GpiCreatePS(hAB,hdcTest,&sizel,flOptions)) == GPI_ERROR)
        {
          CLEANUP("GpiCreatePS",EntryName);
          return;
        }

        // save current DC state
        if ((idDC = GreSaveDC(hdcTest)) == GPI_ERROR)
          SHOWERROR("GreSaveDC", NULL);

        cSave = 3; // iValueLoop; // + 3;

        // create first region
        GetRandomRect(hPS, hWndClient);

        // establish this as the current clip region
        if ((lRgnType = GpiIntersectClipRectangle(hpsTest, &rcRect))
                                                                == RGN_ERROR)
        {
          SHOWERROR("GpiIntersectClipRectangle:", EntryName);
        }

//        SHOWBOUNDS("after intersect", cSave, rcBounds);
        GETBOUNDS(rcBounds);

        // save current DC state
        if ((idDC = GreSaveDC(hdcTest)) == GPI_ERROR)
          SHOWERROR("GreSaveDC", NULL);

        // save the current region
        if (GreSaveRegion(hdcTest, cSave) == FALSE)
        {
          ErrorCount++;
          SHOWERROR("GreSaveRegion", NULL);
        }

//        SHOWBOUNDS("after save", cSave, rcBounds);

        // save current DC state
//        if ((idDC = GreSaveDC(hdcTest)) == GPI_ERROR)
//          SHOWERROR("GreSaveDC", NULL);

        if (GreResetDC(hdcTest, 0L) == DEV_ERROR)
          SHOWERROR("GreResetDC", NULL);

        // restore old region, destroying the current region
        if (GreRestoreRegion(hdcTest, cSave) == FALSE)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, "GreRestoreRegion", NULL);
        }

//        SHOWBOUNDS("after restore", cSave, rcCheck);
        GETBOUNDS(rcCheck);

        // restore the DC to original state
        if (!GreRestoreDC(hdcTest, -1L))
          SHOWERROR("GreRestoreDC", NULL);

//        // reset to clean DC
//        if (GreRestoreRegion(hdcTest, -1L) == FALSE)
//          SHOWERROR("GreRestoreRegion", EntryName);
//
//        // initialize DC regions
//        if (GreSaveRegion(hdcTest, 1L) == FALSE)
//          SHOWERROR("GreSaveRegion", EntryName);

        if (GreResetDC(hdcTest, 0L) == DEV_ERROR)
          SHOWERROR("GreResetDC", NULL);

//        SHOWBOUNDS("after resetDC", cSave, rcBounds);

        // compare calc to actual. (only need to check lower left corner)
        if (   rcBounds.xLeft != rcCheck.xLeft ||
              rcBounds.xRight != rcCheck.xRight ||
             rcBounds.yBottom != rcCheck.yBottom ||
                rcBounds.yTop != rcCheck.yTop)
        {
          ErrorCount++;
          ERRSTATUS(TRUE, EntryName, "Region Mismatch");
          sprintf(szAddMessage,"After Region Create:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcCheck.xLeft, rcCheck.xRight,
                               rcCheck.yBottom, rcCheck.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);
          sprintf(szAddMessage," After Restore Region:\n"
                               " xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                               rcBounds.xLeft, rcBounds.xRight,
                               rcBounds.yBottom, rcBounds.yTop);
          LOGINFO(L_TRACE,"@",szAddMessage);

          sprintf(szAddMessage,"xl=%ld, xr=%ld, yb=%ld, yt=%ld", rcRect.xLeft,
                                rcRect.xRight, rcRect.yBottom, rcRect.yTop);
          LOGINFO(L_TRACE,"*",szAddMessage);

        }

        // restore the DC to original state
        if (!GreRestoreDC(hdcTest, -1L))
          SHOWERROR("GreRestoreDC", NULL);

        if ( (flOptions & PS_TYPE) == GPIT_NORMAL )
        {
           if (!GpiAssociate(hpsTest, NULLHANDLE))
           {
              SHOWERROR("GpiAssociate", EntryName);
           }
        }

        // dispose of presentation space. No longer needed.
        if (GpiDestroyPS(hpsTest)==FALSE)
             SHOWERROR("GpiDestroyPS",EntryName);

        // end the document
        if (DevEscape(hdcTest, DEVESC_ENDDOC, 0L ,NULL, NULL, NULL)
                                                             == DEVESC_ERROR)
          SHOWERROR("DevEscape",EntryName);
        if (DevEscape(hdcTest, DEVESC_ABORTDOC, 0L ,NULL, NULL, NULL)
                                                             == DEVESC_ERROR)
          SHOWERROR("DevEscape",EntryName);

        //Close the device context
        if (GreCloseDC(hdcTest)==DEV_ERROR)
        SHOWERROR("GreCloseDC()",EntryName);

      } // end for(LoopVar)

      SUBEXIT();

    } // end for RangeVar

    EXIT_MAIN(EntryName);
    DeletePrinterOutput(EntryName);
  } // end else

  return;

} /* end GreSaveAndRestoreRegionExh */

ULONG GetDDC(void);

// Section 13.4.9
VOID APIENTRY GreRegionSelectBitmapExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  HAB    hAB;
  extern HDC    hDC;
  static CHAR *EntryName = "GreRegionSelectBitmapExh";
  SIZEL sizel;          //Size PS to create.
  HBITMAP hBM1, hBM2;                  //Bitmap handle
  extern HDC   hdcMemory;
  HPS   hpsMemory;
  BITMAPINFOHEADER bmp1, bmp2;      //bitmap info header
  BYTE   Temp_buffer[200];
  RECTL  rcRect;
  POINTL ptlLLC, ptlURC;
//  POINTL aptl[4];
  DC_BLOCK DCBlock;
  ULONG flOptions;

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    DCBlock.hdc = (ULONG)hdcMemory;
    DCBlock.hddc = GetDDC();

    SETEXIT();

    sizel.cx = 0L; // dev_width;
    sizel.cy = 0L; // dev_height;

    // initialize corner points for drawing PS sized box (in memory device
    // context set to PU_PELS.
    ptlLLC.x = 0L;
    ptlLLC.y = 0L;
    ptlURC.x = dev_width - 1;
    ptlURC.y = dev_height - 1;

    // initialize bitmap header info for bitmap creation
    bmp1.cbFix = sizeof(BITMAPINFOHEADER);
    bmp1.cx = 128;                 //Width
    bmp1.cy = 128;                //height
    bmp1.cPlanes = 4;
    bmp1.cBitCount = 16;

    /***********************************************************
    * This code segment is intended to create new Presentation *
    * Space  and associate it with memory Device Context. The  *
    * The need for a new PS and a new DC stem from the fact    *
    * that bitmaps belongs to DCs of OD_MEMORY type.           *
    ***********************************************************/
    // Open a Memory device context  to use for bitmap operations.
      if ((hdcMemory = GreOpenDC( (HDC)hDC, OD_MEMORY, "*", 0L,
                                     (PDEVOPENDATA)NULL)) == DEV_ERROR) {
//    if ((hdcMemory=DevOpenDC(hAB,OD_MEMORY,"*",0L,(PDEVOPENDATA)NULL,
//                        (HDC)hDC))==DEV_ERROR){
       SHOWERROR("GreOpenDC",EntryName);
       return;
    }

    // Create a presentation space to associate with the
    // memory Device Context
    if ((hpsMemory=GpiCreatePS(hAB,hdcMemory,&sizel,PU_PELS|
                          GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR){
       SHOWERROR("GpiCreatePS",EntryName);
       return;
    }

    flOptions = GpiQueryPS(hpsMemory, &sizel);
    bmp2.cbFix = sizeof(BITMAPINFOHEADER);
    bmp2.cx = (USHORT)sizel.cx;                 //Width
    bmp2.cy = (USHORT)sizel.cy;                //height
    bmp2.cPlanes = 4;
    bmp2.cBitCount = 16;

    // Create an empty bitmap
    if ((hBM1=GpiCreateBitmap(hpsMemory,(PBITMAPINFOHEADER2) &bmp1,
                              0L, NULL,NULL))==GPI_ERROR){
      SHOWERROR("GpiCreateBitmap",EntryName);
      return;
    }
    if ((hBM2=GpiCreateBitmap(hpsMemory,(PBITMAPINFOHEADER2) &bmp2,
                              0L, NULL,NULL))==GPI_ERROR){
      SHOWERROR("GpiCreateBitmap",EntryName);
      return;
    }

    GpiResetPS(hpsMemory, GRES_ALL);
    GpiSetBackMix(hpsMemory, BM_OVERPAINT);
    GpiSetMix(hpsMemory, FM_OVERPAINT);
    GpiSetPattern(hpsMemory, PATSYM_DENSE8);
    GpiSetColor(hpsMemory, CLR_BLUE);
    GpiErase(hpsMemory);

    // Draw the PS sized box
//    GpiMove(hpsMemory, &ptlLLC);
//    GpiBox(hpsMemory, DRO_FILL, &ptlURC, 0L, 0L);

    // select the bitmap
    if (GpiSetBitmap(hpsMemory, hBM2)==HBM_ERROR){
      SHOWERROR("GpiSetBitmap",EntryName);
      return;
    }

    GpiQueryClipBox(hpsMemory, &rcRect);
    sprintf(Temp_buffer, "After Set:xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                 rcRect.xLeft, rcRect.xRight, rcRect.yBottom, rcRect.yTop);
    LOGINFO(L_TRACE, "*", Temp_buffer);

    GpiSetPattern(hpsMemory, PATSYM_HORIZ);
    GpiSetColor(hpsMemory, CLR_RED);

    // Draw the PS sized box
    GpiMove(hpsMemory, &ptlLLC);
    GpiBox(hpsMemory, DRO_FILL, &ptlURC, 0L, 0L);

    GpiQueryClipBox(hpsMemory, &rcRect);
    sprintf(Temp_buffer, "After draw:xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                 rcRect.xLeft, rcRect.xRight, rcRect.yBottom, rcRect.yTop);
    LOGINFO(L_TRACE, "*", Temp_buffer);

    // de-select the bitmap
    if (GpiSetBitmap(hpsMemory, NULLHANDLE)==HBM_ERROR){
      SHOWERROR("GpiSetBitmap",EntryName);
      return;
    }

    // select the bitmap region (function undertest);
    if (GreRegionSelectBitmap(hdcMemory, hBM2) == FALSE)
    {
      ErrorCount++;
      ERRSTATUS(TRUE, "GreRegionSelectBitmap", EntryName);
    }

    if (!GreDeviceInvalidateVisRegion(hdcMemory, 1L, &DCBlock))
    {
      ErrorCount++;
      ERRSTATUS(TRUE, "GreDeviceInvalidateVisRegion", EntryName);
    }

    GpiSetPattern(hpsMemory, PATSYM_VERT);
    GpiSetColor(hpsMemory, CLR_WHITE);

    // Draw the PS sized box
    GpiMove(hpsMemory, &ptlLLC);
    GpiBox(hpsMemory, DRO_FILL, &ptlURC, 0L, 0L);

    GpiQueryClipBox(hpsMemory, &rcRect);
    sprintf(Temp_buffer, "After invalidate:xl=%ld, xr=%ld, yb=%ld, yt=%ld",
                 rcRect.xLeft, rcRect.xRight, rcRect.yBottom, rcRect.yTop);
    LOGINFO(L_TRACE, "*", Temp_buffer);

//    // select the bitmap
//    if (GpiSetBitmap(hpsMemory, hBM1)==HBM_ERROR){
//      SHOWERROR("GpiSetBitmap",EntryName);
//      return;
//    }
//
//    GpiSetPattern(hpsMemory, PATSYM_VERT);
//    GpiSetColor(hpsMemory, CLR_WHITE);
//
//    // Draw the PS sized box
//    GpiMove(hpsMemory, &ptlLLC);
//    GpiBox(hpsMemory, DRO_FILL, &ptlURC, 0L, 0L);
//
//    GpiQueryClipBox(hpsMemory, &rcRect);
//    sprintf(Temp_buffer, "xl=%ld, xr=%ld, yb=%ld, yt=%ld", rcRect.xLeft,
//       rcRect.xRight, rcRect.yBottom, rcRect.yTop);
//    LOGINFO(L_TRACE, "*", Temp_buffer);
//
//    // target coordinates
//    aptl[0].x = 0L; // dev_width / 2 - 64L;   // center in middle of screen horz.
////    aptl[0].y = 0L; // dev_height / 2 - 64L;  // center in middle of screen vert.
//    aptl[0].y = (LONG)(((0.0254 * 0.7) * (double)vert_pels_per_m) + 0.5);
//    aptl[1].x = dev_width - 1; // aptl[0].x + 127;
//    aptl[1].y = dev_height - 1; // aptl[0].y + 127;
//    // source coordinates
//    aptl[2].x = 0L;  // dev_width;
//    aptl[2].y = 0L;  // dev_height;
//    aptl[3].x = sizel.cx - 1; // 127;
//    aptl[3].y = sizel.cy - 1; // 127;
//    // copy from memory device to test device
//    if (GpiBitBlt(hPS, hpsMemory, 4L, aptl, ROP_SRCCOPY, BBO_AND) == GPI_ERROR)
//      SHOWERROR("GpiBitBlt", NULL);

    // De-select the bitmap handle. otherwise we get error when closing
    // device context
     if (GpiSetBitmap(hpsMemory,NULLHANDLE)==HBM_ERROR){
         SHOWERROR("GpiSetBitmap",EntryName);
     }

    // Delete the bitmap handle.
    if (GpiDeleteBitmap(hBM1)==FALSE){
         SHOWERROR("GpiDeleteBitmap",EntryName);
     }
    if (GpiDeleteBitmap(hBM2)==FALSE){
         SHOWERROR("GpiDeleteBitmap",EntryName);
     }

    // dispose of presentation space. No longer needed.
    if (GpiDestroyPS(hpsMemory)==FALSE){
         SHOWERROR("GpiDestroyPS",EntryName);
    }

    //Close the device context
    if (GreCloseDC(hdcMemory)==DEV_ERROR){
      SHOWERROR("GreCloseDC()",EntryName);
    }

    EXIT_MAIN(EntryName);
  } // end else

  return;

} // end GreRegionSelectBitmap exhaustive test

// Section 13.4.11
VOID APIENTRY GreClipPathCurvesExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  HAB    hAB;
  HDC    hDC;
  static CHAR *EntryName = "GreClipPathCurvesExh";

  if (SelectionCall)
  {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  } // end if
  else
  {
    // We need to check for a valid test environment here.

    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop),
                                                          MPFROMP(&lDCType));
    if (DeviceIsPrinter() && lDCType==OD_QUEUED &&
                                    strcmp("PM_Q_STD", dop.pszDataType)==0)
    {
      ABORTENTRY(EntryName, "OD_QUEUED/PM_Q_STD");
      return;
    }

    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    // If the environment is o.k. then begin testing.

    ENTER_MAIN(EntryName);

    SETENTER(EntryName, "Initializations" );

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    SETEXIT();

    EXIT_MAIN(EntryName);
  } // end else

  return;

} // end GreClipPathCurves exhaustive test

