/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/**************************************************************************
 *
 * SOURCE FILE NAME = DTTDLLVR.C
 *
 * DESCRIPTIVE NAME = Test case dll header file
 *
 *
 *      DISCLAIMER OF WARRANTIES.  The following enclosed code is
 *      sample code created by IBM Corporation. This sample code is not
 *      part of any standard or IBM product and is provided to you solely
 *      for  the purpose of assisting you in the development of your
 *      applications.  The code is provided "AS IS", without
 *      warranty of any kind.  IBM shall not be liable for any damages
 *      arising out of your use of the sample code, even if they have been
 *      advised of the possibility of such damages.
 *
 * VERSION = V1.1
 *
 * DATE      10/31/92
 *
 * DESCRIPTION  Variables, procedures and macros that are used by
 *              test case DLLs.
 *
 * FUNCTIONS    InitTestSegVars()          Used by InitTest to make pDimX, pDimY.
 *              VerifyReturnedErrorCode()  Verify proper error code is returned.
 *              VerifyReturnCode()         Verify proper error code is returned.
 *              ReportSetupError()         Return unexpected API errors.
 *              MakePSBusy()               Create a PM_PS_BUSY condition.
 *              FreePS()                   Reverse the PS_BUSY condition.
 *              BusyThread()               Thread to make presentation space busy.
 *              BusyErrorExit()            Error exit function for the BusyThread.
 *              ErrorMessage()             Return name of the error identified.
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 * EXTERNAL REFERENCES
 *
 * EXTERNAL FUNCTIONS
 *
*/
#define  INCL_GPI
#define  INCL_32
#define  INCL_DEV
#define  INCL_DOS
#define  INCL_ERRORS
#include <os2.h>
#include <stdio.h>
#include <string.h>
#include "DTTdlldf.h"
#include "errors.h"
#define  ARRAYLEN(ArrayName) (sizeof(ArrayName)/sizeof(ArrayName[0]))

 /*
 **  These variables are used by the MAP macros to resolve Presentation
 **  Space unit locations.  The pDimX and pDimY variables are set by DoUnit
 **  everytime a presentation space is created.  alDevCaps contains the
 **  data structure returned by a DevQueryCaps call.
 */

LONG *pDimX,*pDimY,*pBasisX,*pBasisY,*alDevCaps;

 /*
 **  This var points to the HAB of the Display thread.  This is needed to
 **  retrieve error information (ie, WinGetLastError).
 */

PHAB phabDThread;
CHAR *ErrorMessage(USHORT);

/***************************************************************************
 *
 * FUNCTION NAME = InitTestSegVars()
 *
 * DESCRIPTION   = Used by InitTest to make pDimX, pDimY, and alDevCaps
 *                 variables point into the shared segment.
 *
 * INPUT         = Selector of shared segment
 *
 * OUTPUT        = None
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 ****************************************************************************/

VOID InitTestSegVars(PVOID selSeg)

{


   pDimX       = (LONG *) selSeg;
   pDimY       = (LONG *) selSeg + (sizeof (LONG));
   pBasisX     = (LONG *) selSeg + (2 * sizeof (LONG));
   pBasisY     = (LONG *) selSeg + (3 * sizeof (LONG));
   phabDThread = (PHAB)   selSeg + (4 * sizeof (LONG));
   alDevCaps   = (LONG *) selSeg + (5 * sizeof (LONG));
}

#define  MANYSPACES    \
"                                                                                "

/***************************************************************************
 *
 * FUNCTION NAME = VerifyReturnedErrorCode()
 *
 * DESCRIPTION   = Verify that the proper error code is returned for a given
 *                 error condition.  This function verifies that the error id
 *                 and severity match those expected by the test case.
 *
 * INPUT         = Handle to Client window
 *                 Return Code
 *                 Test Case Name
 *                 Expected Severity
 *                 Expected Error ID
 *                 File Name
 *                 Line Number
 *
 * OUTPUT        = False - Sucess
 *                 True  - Failure
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 ****************************************************************************/

SHORT VerifyReturnedErrorCode(HWND hWndClient,LONG ReturnCode,CHAR *szTestName
                               ,USHORT usExpSev,USHORT usExpErrorid,PSZ
                               szFileName,USHORT usLineNo)
{
  PERRINFO perrorinfo;
  ERRORID ActErrorid;
  CHAR achLogLine[100];
  BOOL bSuccess;
  USHORT usActErrId;
  CHAR ErrorString[100];

  bSuccess = TRUE;

  if (ReturnCode <= 0L)
  {                                /* This means the API call failed        */
    perrorinfo = WinGetErrorInfo(*phabDThread);

    if (perrorinfo)
      ActErrorid = perrorinfo->idError;

    else
    {
      CWRITELOG(L_FAIL, 0, "WinGetErrorInfo returned NULL");
      ActErrorid = 0;
    }
    usActErrId = ERRORIDERROR(ActErrorid);

    if (usActErrId != usExpErrorid)
    {
      LOG_SIGNATURE;
      strcpy(ErrorString, ErrorMessage(usExpErrorid));
      sprintf(achLogLine, "*MISMATCH - Expected %04x: %s;\n", usExpErrorid,
         ErrorString);
      CWRITELOG(L_FAIL, 0, achLogLine);
      strcpy(ErrorString, ErrorMessage(usActErrId));
      sprintf(achLogLine, "            Actual value %04x: %s.\n", usActErrId,
         ErrorString);
      CWRITELOG(L_FAIL, 0, achLogLine);
      sprintf(achLogLine, "*LOCATION - %s, line %d.\n", szFileName, usLineNo);
      CWRITELOG(L_FAIL, 0, achLogLine);
      bSuccess = FALSE;
    }

    if (perrorinfo)
      WinFreeErrorInfo(perrorinfo);
  }

  else
  {
    LOG_SIGNATURE;
    strcpy(ErrorString, ErrorMessage(usExpErrorid));
    sprintf(achLogLine, "*MISMATCH - Expected %04x: %s;\n", usExpErrorid,
       ErrorString);
    CWRITELOG(L_FAIL, 0, achLogLine);
    sprintf(achLogLine, "             worked properly instead.\n");
    CWRITELOG(L_FAIL, 0, achLogLine);
    sprintf(achLogLine, "*LOCATION - %s, line %d.\n", szFileName, usLineNo);
    CWRITELOG(L_FAIL, 0, achLogLine);
    bSuccess = FALSE;
  }

  /*
  **  Display a line stating the item being tested and whether it passed
  ** or failed.
  */

  strcpy(achLogLine, "@");
  strcat(achLogLine, szTestName);
  strncat(achLogLine, MANYSPACES, 53-strlen(szTestName));

  if (bSuccess)
    strcat(achLogLine, "-    PASS.\n");

  else
    strcat(achLogLine, "-    FAIL.\n");
  CWRITELOG(L_TRACE, 0, achLogLine);
  return ((bSuccess?0:1));
}

/***************************************************************************
 *
 * FUNCTION NAME = VerifyReturnCode()
 *
 * DESCRIPTION   = Verify that the proper error code is returned for a given
 *                 error condition.  This function verifies that the error
 *                 id and severity match those expected by the test case.
 *
 *                 NOTE:  This function is virtually a copy of the above fun
 *                 funtion VerifyReturnedErrorCode(), with the exception
 *                 that this function is modified to consider a return code
 *                 an error code only if rc < 0 (and not rc <= 0 as in the
 *                 above function).  This is necessary to test those API
 *                 calls that can return a 0 as a valid code (and not an
 *                 error), e.g., GpiQueryPel().
 *
 * INPUT         = Handle to Client window
 *                 Return Code
 *                 Test Case Name
 *                 Expected Severity
 *                 Expected Error ID
 *                 File Name
 *                 Line Number
 *
 * OUTPUT        = False - Sucess
 *                 True  - Failure
 *
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 ****************************************************************************/

SHORT VerifyReturnCode(HWND hWndClient,LONG ReturnCode,CHAR *szTestName,USHORT
                        usExpSev,USHORT usExpErrorid,PSZ szFileName,USHORT
                        usLineNo)
{
  PERRINFO perrorinfo;
  ERRORID ActErrorid;
  CHAR achLogLine[100];
  BOOL bSuccess;
  USHORT junk = usExpSev;
  USHORT usActErrId;
  CHAR ErrorString[100];

  bSuccess = TRUE;

  if (ReturnCode < 0L  )
  {                                /* This means the API call failed        */
    perrorinfo = WinGetErrorInfo(*phabDThread);

    if (perrorinfo)
      ActErrorid = perrorinfo->idError;

    else
    {
      CWRITELOG(L_FAIL, 0, "WinGetErrorInfo returned NULL");
      ActErrorid = 0;
    }
    usActErrId = ERRORIDERROR(ActErrorid);

    if (usActErrId != usExpErrorid)
    {
      LOG_SIGNATURE;

      /*
      **  Log a message stating that the expected and actual return codes
      **  don't match.
      */

      strcpy(ErrorString, ErrorMessage(usExpErrorid));
      sprintf(achLogLine, "*MISMATCH - Expected %04x: %s;\n", usExpErrorid,
         ErrorString);
      CWRITELOG(L_FAIL, 0, achLogLine);
      strcpy(ErrorString, ErrorMessage(usActErrId));
      sprintf(achLogLine, "            Actual value %04x: %s.\n", usActErrId,
         ErrorString);
      CWRITELOG(L_FAIL, 0, achLogLine);

      /*
      **  Tell the location in the file of the error.
      */

      sprintf(achLogLine, "*LOCATION - %s, line %d.\n", szFileName, usLineNo);
      CWRITELOG(L_FAIL, 0, achLogLine);
      bSuccess = FALSE;
    }

    if (perrorinfo)
      WinFreeErrorInfo(perrorinfo);
  }

  else
  {
    /*
    **  If we are here it means we were expecting an error, but the call worked
    **  and reported no error.
    */

    LOG_SIGNATURE;

    /*
    **  Log a message stating that the expected and actual return codes
    **  don't match.
    */

    strcpy(ErrorString, ErrorMessage(usExpErrorid));
    sprintf(achLogLine, "*MISMATCH - Expected %04x: %s;\n", usExpErrorid,
       ErrorString);
    CWRITELOG(L_FAIL, 0, achLogLine);
    sprintf(achLogLine, "             worked properly instead.\n");
    CWRITELOG(L_FAIL, 0, achLogLine);

    /*
    **  Tell the location in the file of the error.
    */

    sprintf(achLogLine, "*LOCATION - %s, line %d.\n", szFileName, usLineNo);
    CWRITELOG(L_FAIL, 0, achLogLine);
    bSuccess = FALSE;
  }

  /*
  **  Display a line stating the item being tested and whether it passed or
  **  failed.
  */

  strcpy(achLogLine, "@");
  strcat(achLogLine, szTestName);
  strncat(achLogLine, MANYSPACES, 53-strlen(szTestName));

  if (bSuccess)
    strcat(achLogLine, "-    PASS.\n");

  else
    strcat(achLogLine, "-    FAIL.\n");
  CWRITELOG(L_TRACE, 0, achLogLine);
  return ((SHORT)(bSuccess?0:1));
}

/***************************************************************************
 *
 * FUNCTION NAME = ReportSetupError()
 *
 * DESCRIPTION   = Return unexpected API errors.  It assumes that an error
 *                 has occurred so it does a WinGetErrorInfo to get the
 *                 specifics about the error and displays it.
 *
 * INPUT         = Handle to Client window
 *                 Function call name
 *                 File name
 *                 Line number
 *
 * OUTPUT        = False - Sucess
 *                 True  - Failure
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 ****************************************************************************/

VOID ReportSetupError(HWND hWndClient,CHAR *pCallName,PSZ szFileName,USHORT
                       usLineNo)
{
  PERRINFO perrorinfo;
  CHAR achLogLine[100];
  USHORT usActErrId;
  CHAR ErrorString[100];
  LOG_SIGNATURE;

  perrorinfo = WinGetErrorInfo(*phabDThread);

  if (perrorinfo)
    usActErrId = ERRORIDERROR(perrorinfo->idError);

  else
    usActErrId = 0;
  sprintf(achLogLine,
     "*Failure on API %s with error id = %04x.\n", pCallName, usActErrId);
  CWRITELOG(L_FAIL, 0, achLogLine);
  strcpy(ErrorString, ErrorMessage(usActErrId));
  sprintf(achLogLine, "            Actual value %04x: %s.\n", usActErrId,
     ErrorString);
  CWRITELOG(L_FAIL, 0, achLogLine);
  sprintf(achLogLine, "*LOCATION - File %s, line %d.\n", szFileName, usLineNo)
     ;
  CWRITELOG(L_FAIL, 0, achLogLine);
}

 /*
 **  These routines can be used by test cases that verify return codes to
 **  create PMERR_PM_BUSY situation.  To create a PM_PS_BUSY situation, call
 **  MakePSBusy immediately followed by the API to be tested.  After calling
 **  the API, call FreePS to return the PS to normal.
 */

 /*
 ** Define variables to create the PMERR_HPS_BUSY condition.
 */

#define  STACKSIZE     4096
CHAR FAR *TStack;
ULONG BusyVal = 0L;                    /* BusyThread semaphore value        */
HSEM BusySem = & BusyVal;              /* BusyThread sem address            */
ULONG BusyCnt;
TID ThreadID;
CHAR BusyThreadErrorMsg[125];

 /*
 ** Function prototype to create busy condition
 */

VOID BusyThread(HPS hPS);
VOID BusyErrorExit(CHAR *message);

/***************************************************************************
 *
 * FUNCTION NAME = MakePSBusy()
 *
 * DESCRIPTION   = Create a PM_PS_BUSY condition.
 *
 * INPUT         = Handle to Client window
 *                 Handle to presentation space
 *
 * OUTPUT        = True  - Success
 *                 False - Failure
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 ****************************************************************************/

BOOL MakePSBusy(HWND hWndClient,HPS hPS)
{
#ifdef INCL_32
  HPS     junk=hPS;
#else
  USHORT  Sel;
#endif

  /*
  **  Preset the error message to "".
  */

  strcpy(BusyThreadErrorMsg, "");

  /*
  **  Set the semaphore to wait on.
  */

#ifdef INCL_32
  if (!BusyVal && DosCreateEventSem(NULL, &BusyVal, 0, TRUE) != NO_ERROR) {
    REPORTSETUPERROR("DosCreateEventSem");
    return(FALSE);
  }

  if (DosResetEventSem(BusyVal, &BusyCnt) != NO_ERROR){
    REPORTSETUPERROR("DosResetEventSem");
    return(FALSE);
  }
#else
  if (DosSemSet(BusySem) != NO_ERROR){
    REPORTSETUPERROR("DosSemSet");
    return(FALSE);
  }
#endif

#ifdef INCL_32
  if (DosCreateThread ((PTID)&ThreadID, (PFNTHREAD) BusyThread, 0L, 0L,
                      (ULONG)STACKSIZE)) {
    REPORTSETUPERROR("DosCreateThread");
    return(FALSE);
  }
#else
  /*
  **  Allocate a stack for the new thread and create it.
  */

  if (DosAllocSeg(STACKSIZE, (PSEL)&Sel, 0))
  {
    REPORTSETUPERROR("DosAllocSeg");
    return (FALSE);
  }

  OFFSETOF(TStack) = STACKSIZE-6;  /* Access the threads stack              */
  SELECTOROF(TStack) = Sel;
  *((PHPS)TStack) = hPS;      /* Pass the hPS as a parameter to BusyThread  */

  if (DosCreateThread((PFNTHREAD)BusyThread, (PTID)&ThreadID, (PBYTE)TStack))
  {
    REPORTSETUPERROR("DosCreateThread");
    return (FALSE);
  }
#endif

  /*
  **  Wait until the BusyThread is just about to start its long PS hogging
  **  instruction before letting this thread continue.
  */

#ifdef INCL_32
  if (DosWaitEventSem(BusyVal, SEM_INDEFINITE_WAIT) != NO_ERROR){
    REPORTSETUPERROR("DosSemWait");
    return(FALSE);
  }
#else
  if (DosSemWait(BusySem, -1l) != NO_ERROR){
    REPORTSETUPERROR("DosSemWait");
    return(FALSE);
  }
#endif


  /*
  **  In case this thread gets control after the BusyThread issues
  **  the DosSemClear but before the PS hogging instruction, we'll
  **  sleep for a second.
  */

  if (DosSleep(500L) != NO_ERROR)
  {
    REPORTSETUPERROR("DosSleep");
    return (FALSE);
  }
  return (TRUE);
}

/***************************************************************************
 *
 * FUNCTION NAME = FreePS()
 *
 * DESCRIPTION   = Reverse the PS_BUSY condition.
 *
 * INPUT         = Handle to Client window
 *
 * OUTPUT        = True  - Success
 *                 False - Failure
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 ****************************************************************************/

BOOL FreePS(HWND hWndClient)
{
  /*
  **  Wait for the BusyThread to let us know it is complete.
  */

#ifdef INCL_32
  if (DosWaitEventSem(BusyVal, SEM_INDEFINITE_WAIT) != NO_ERROR){
    REPORTSETUPERROR("DosSemWait");
    return(FALSE);
  }
#else
  if (DosSemWait(BusySem, -1l) != NO_ERROR){
    REPORTSETUPERROR("DosSemWait");
    return(FALSE);
  }
#endif

  /*
  ** Clear the DosEnterCritSec()
  */

  DosExitCritSec();

  /*
  **  If BusyThreadErrorMsg == "" is means BusyThread executed with no
  ** errors.  Otherwise it contains the name of the API that failed.
  */

  if (!strcmp(BusyThreadErrorMsg, ""))
  {
#ifndef INCL_32
    if  (DosFreeSeg(SELECTOROF(TStack)) != NO_ERROR){
      REPORTSETUPERROR("DosFreeSeg");
      return(FALSE);
    }
#endif

    return (TRUE);
  }

  else
  {
    REPORTSETUPERROR(BusyThreadErrorMsg);
    return (FALSE);
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = BusyThread()
 *
 * DESCRIPTION   = This thread is used to make the presentation space busy.
 *                 When the test case issues the MakePSBusy command followed
 *                 by the API to be tested, the API will compete with the
 *                 thread over the use of the presentation space.
 *
 * INPUT         = Handle to presentation space
 *
 * OUTPUT        = None
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 ****************************************************************************/

VOID BusyThread(HPS hPS)
{
  HDC hDC;
  HAB habBusy;
  HMQ hmqBusy;
  static POINTL pPoints[8000];
  USHORT i;


  for (i = 0; i < 8000; i++)
  {
    pPoints[i].x = i%2L*1000;
    pPoints[i].y = i/90L*5;
  }

  /*
  **  Make sure the presentation space passed to us is valid
  **  and associated with a device context.
  */

  hDC = GpiQueryDevice(hPS);

  if ((hDC == HDC_ERROR) || (hDC == NULLHANDLE))
    BusyErrorExit("GpiQueryDevice (in BusyThread)");

  /*
  ** Create an anchor block and message queue for this thread.
  */

  if ((habBusy = WinInitialize((SHORT)NULL)) == NULLHANDLE)
    BusyErrorExit("WinInitialize (in BusyThread)");

  /*
  **  Create Message Queue ... but ignore any errors ...
  **  any errors returned will indicate either Message Queue
  **  already created or resource problem ... in either case
  **  the next set of calls will capture the resource problem
  **  while the already created error isn't a problem.
  */

  hmqBusy = WinCreateMsgQueue(habBusy, 0);

  /*
  **  Clear the semaphone to let MakePSBusy continue.  Reset the semaphone
  ** for use later to communicate with FreePS.
  */


  /*
  **  This is the command that hogs the hPS long enough to force the
  **  PMERR_HPS_BUSY condition to occur.
  */

  if (GpiPolyLine(hPS, 8000L, pPoints) != GPI_OK)
    BusyErrorExit("GpiPolyLine (in BusyThread)");

  /*
  **  Clean up.
  */

  if (hmqBusy)
    WinDestroyMsgQueue(hmqBusy);
  WinTerminate(habBusy);
  DosEnterCritSec();
  // DosSemClear(BusySem);
  DosExit(EXIT_THREAD,
          0);
}

/***************************************************************************
 *
 * FUNCTION NAME = BusyErrorExit()
 *
 * DESCRIPTION   = This is an error exit function fo the BusyThread.  It
 *                 copies the error message that is passed in as a parameter
 *                 into the BusyThreadErrorMsg.  It clears the semaphore
 *                 BusySem to allow MakePSBusy to complete, and exits the
 *                 thread.
 *
 * INPUT         = Error message
 *
 * OUTPUT        = None
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 ****************************************************************************/

VOID BusyErrorExit(CHAR *message)
{
  strcpy(BusyThreadErrorMsg, message);
#ifdef INCL_32
  DosPostEventSem(BusyVal);
#else
  DosSemClear(BusySem);
#endif
  DosExit(0, 0);
}

/***************************************************************************
 *
 * FUNCTION NAME = ErrorMessage()
 *
 * DESCRIPTION   = Returns in a string the name of the error identified via
 *                 usErrorId.
 *
 * INPUT         = Error ID
 *
 * OUTPUT        = Error message character string
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 ****************************************************************************/

CHAR *ErrorMessage(USHORT usErrorId)
{
  USHORT index,current_error;
  CHAR message[100];

  index = 0;
  current_error = errorcodes[index].ErrorId;

  while ((current_error != usErrorId) && (index < NUMERRORS))
    current_error = errorcodes[++index].ErrorId;

  if (index == NUMERRORS)
    sprintf(message, "THE ERROR CODE %hu IS UNKNOWN", usErrorId);

  else
    strcpy(message, errorcodes[index].ErrorName);
  return (message);
}

