/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = EXTRAS.H
 *
 * DESCRIPTIVE NAME = Miscellaneous OS/2 Definitions
 *
 *
 *      DISCLAIMER OF WARRANTIES.  The following enclosed code is
 *      sample code created by IBM Corporation. This sample code is not
 *      part of any standard or IBM product and is provided to you solely
 *      for  the purpose of assisting you in the development of your
 *      applications.  The code is provided "AS IS", without
 *      warranty of any kind.  IBM shall not be liable for any damages
 *      arising out of your use of the sample code, even if they have been
 *      advised of the possibility of such damages.
 *
 * VERSION = V1.1
 *
 * DATE      10/31/92
 *
 * DESCRIPTION  Miscellaneous OS/2 Definitions
 *
 * FUNCTIONS    None
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 * EXTERNAL REFERENCES
 *
 * EXTERNAL FUNCTIONS
 *
*/

 /*
 **  Fast, Safe RAM Semaphore declarations
 */

typedef struct _FSRSEM
{
    USHORT Length;
    USHORT Padding;
    USHORT ProcID;
    USHORT ThrdID;
    USHORT Usage;
    USHORT Client;
    ULONG  Timeout;
    ULONG  RAMsem;
} FSRSEM;

typedef FSRSEM *PFSRSEM;

int   APIENTRY FSRSemEnter( PFSRSEM );
VOID  APIENTRY FSRSemLeave( PFSRSEM );
VOID  APIENTRY FSRSemExit ( PFSRSEM );
ULONG APIENTRY FSRSemCheck( PFSRSEM );

#define EXLST_PRTY_UNLOCK       0x1080   /* Unlock DLL: free semaphores, etc */
#define EXLST_PRTY_PMDRIVERS    0xad00   /* Drivers: display                 */


 /*
 **  File manager
 */

 /*
 **  DosChgFilePtr() file position codes
 */

#define FILE_BEGIN      0x0000
#define FILE_CURRENT    0x0001
#define FILE_END        0x0002

 /*
 **  Directory handle types
 */

#ifndef HDIR_SYSTEM
   #define HDIR_SYSTEM  0x0001
   #define HDIR_CREATE  0xFFFF
#endif

 /*
 **  Dosopen/DosQFHandState file attributes
 */

#define FILE_NORMAL     0x0000
#define FILE_READONLY   0x0001
#define FILE_HIDDEN     0x0002
#define FILE_SYSTEM     0x0004
#define FILE_DIRECTORY  0x0010
#define FILE_ARCHIVED   0x0020

 /*
 **  DosOpen() actions
 */

#define FILE_EXISTED    0x0001
#define FILE_CREATED    0x0002
#define FILE_TRUNCATED  0x0003

 /*
 **  DosOpen() open flags
 */

#define FILE_OPEN      0x0001
#define FILE_TRUNCATE  0x0002
#define FILE_CREATE    0x0010

 /*
 **  DosOpen/DosSetFHandState flags
 */

#define OPEN_ACCESS_READONLY      0x0000
#define OPEN_ACCESS_WRITEONLY     0x0001
#define OPEN_ACCESS_READWRITE     0x0002
#define OPEN_SHARE_DENYREADWRITE  0x0010
#define OPEN_SHARE_DENYWRITE      0x0020
#define OPEN_SHARE_DENYREAD       0x0030
#define OPEN_SHARE_DENYNONE       0x0040
#define OPEN_FLAGS_NOINHERIT      0x0080
#define OPEN_FLAGS_FAIL_ON_ERROR  0x2000
#define OPEN_FLAGS_WRITE_THROUGH  0x4000
#define OPEN_FLAGS_DASD           0x8000

 /*
 **  DosSearchPath() constants
 */

#define SEARCH_PATH            0x0000
#define SEARCH_CUR_DIRECTORY   0x0001
#define SEARCH_ENVIRONMENT     0x0002


 /*
 ** SetErrorInfo API
 */

ERRORID cdecl FAR WinSetErrorInfo(ERRORID, USHORT, ...);

#define SEI_BREAKPOINT      0x8000  /* Always enter an INT 3 breakpt */
#define SEI_NOBEEP          0x4000  /* Do not call DosBeep    */
#define SEI_NOPROMPT        0x2000  /* Do not prompt the user */
#define SEI_DBGRSRVD        0x1000  /* Reserved for debug use */
#define SEI_DEBUGONLY       (SEI_BREAKPOINT | SEI_NOBEEP | SEI_NOPROMPT | SEI_RESERVED)

#define SEI_STACKTRACE      0x0001  /* save the stack trace */
#define SEI_REGISTERS       0x0002  /* save the registers   */
#define SEI_ARGCOUNT        0x0004  /* first USHORT in args is arg count */
#define SEI_DOSERROR        0x0008  /* first USHORT in args is OS2 error code */
#define SEI_MSGSTR          0x0010  /* first PSZ in arg -> msg string */
#define SEI_RESERVED        0x0FE0  /* Reserved for future use   */

 /*
 **  Note that when SEI_ARGCOUNT, SEI_DOSERROR and DOS_MSGSTR are specified
 **  together, then the implied order of the parameters is:
 **
 **
 **   WinSetErrorInfo( MAKEERRORID( .... ),
 **                    SEI_ARGCOUNT | SEI_DOSERROR | SEI_MSGSTR,
 **                    argCount,
 **                    dosErrorCode,
 **                    "This is the error msg string: %s\n",
 **                    "This is an insert for the %s format specifier" );
 **
 **
 ** Using SEI_MSGSTR prevents having to put the message text in the .RC file
 ** which is useful for debugging only message that are not in the retail
 ** product.
 */


 /*
 **  Error information structure built by WinSetErrorInfo function.
 **  This structure is kept in a dynamically created segment.
 */

typedef struct _IERRINFO {
    USHORT sig;            /* signature word                             */
    USHORT pArgs;          /* SS:pArgs -> first arg                      */
    USHORT errCode;        /* LOW(errCode) parameter to WinSetErrorInfo  */
    USHORT severity;       /* HIGH(errCode) parameter to WinSetErrorInfo */
    PSZ    pszErrorMsg;    /* -> error message (NULL => DosGetResource)  */
    USHORT dosErrCode;     /* OS/2 error code                            */
    USHORT fOptions;       /* fOptions parameter to WinSetErrorInfo      */
    USHORT cParmWords;     /* count of parameter words                   */
    USHORT offParmWords;   /* offset to saved copy of parameter words    */
    USHORT cStackFrames;   /* count of active stack frames when called   */
    USHORT offStackFrames; /* offset to saved copy of stack frames (CS:IP)*/
    USHORT AX;             /* Saved register contents                    */
    USHORT BX;
    USHORT CX;
    USHORT DX;
    USHORT SI;
    USHORT DI;
    USHORT BP;
    USHORT SP;
    USHORT SS;
    USHORT IP;
    USHORT CS;
    USHORT DS;
    USHORT ES;
    USHORT flags;
} IERRINFO;
typedef IERRINFO *PIERRINFO;
