/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma pagesize(59)

/*const char header=$header$*/

#define INCL_DEV      // include all DEV definitions
#define INCL_WIN
#define INCL_GPI      // include all Gpi definitions
#define INCL_ERRORS   // include all Gre error definitions

#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <errno.h>
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "common2.h"
#include "grearc.h"


/****************************************************************************
 *\\ddd
 * Subtask Name: GreArc.dll
 *
 * Module name: GreArc.c
 *
 * Purpose: dll test functions
 *
 * Revision Log:
 *        10/14/91 ,DTT, Ken McFadden
 *              -Initial Creation.
 *
 *
 *
 * SAD References:
 *
 * SRS References:
 *
 * Contents:  InitTest()
 *            GreArcPre1()
 *            GreArcPre2()
 *            GreSetandGetArcParametersExh()
 *            GreSetandGetDefArcParametersExh()
 *            GreArcExh()
 *            GrePartialArcExh()
 *            GreFullArcInteriorExh()
 *            GreFullArcBoundaryExh()
 *            GreFullArcBothExh()
 *            GreBoxInteriorExh()
 *            GreBoxBoundaryExh()
 *            GreBoxBothExh()
 *            GrePolyFilletExh()
 *            GrePolyFilletSharpExh()
 *            GrePolySplineExh()
 *            GreDrawConicsInPathExh()
 *            GreCookWholePathExh()
 *            GreCookPathCurvesExh()
 *            GreRenderPathExh()
 *            GreSetArcParametersRc()
 *            GreGetArcParametersRc()
 *            GreSetDefArcParametersRc()
 *            GreGetDefArcParametersRc()
 *            GreArcRc()
 *            GrePartialArcRc()
 *            GreFullArcInteriorRc()
 *            GreFullArcBoundaryRc()
 *            GreFullArcBothRc()
 *            GreBoxInteriorRc()
 *            GreBoxBoundaryRc()
 *            GreBoxBothRc()
 *            GrePolyFilletRc()
 *            GrePolyFilletSharpRc()
 *            GrePolySplineRc()
 *            GreDrawConicsInPathRc()
 *            GreCookWholePathRc()
 *            GreCookPathCurvesRc()
 *            GreRenderPathRc()
 *
 * Design Overview:
 *
 * Limitations:
 *
 *
 *
 *\\end
****************************************************************************/
/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose:  Initialize function names and address array for DTT.
 *
 * System Requirements:
 *
 * Revision Log:
 *
 *                                              10/24/90, pdvt, Mike Abuzant
 *               - initial file
 *
 * Inputs:
 *   sel          // passed through to the InitTestSegVars function
 *   init_info_p  // name and address array used by DTT interface
 *
 * Outputs:
 *   init_info_p  // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()    // used by the DTT interface
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *        Get the number of tests.
 *   for the number of tests {
 *                      Get the test names;
 *                      Get the test addresses;
 *                      }
 *   call InitTestSegVars function
 *   }
 *\\end
 ***************************************************************************/
VOID APIENTRY InitTest(PVOID Selector, PTESTINITINFO init_info_p)
{
  register USHORT usTestCounter;

  init_info_p->ntests = NTESTS;
  strcpy(init_info_p->group_name,TestCaseName); //Testcase Name

  for (usTestCounter=0 ;usTestCounter< NTESTS; ++usTestCounter){

     init_info_p->test_addrs[usTestCounter]=
                               EntryPointList[usTestCounter].TestAddress;
     strcpy(init_info_p->test_names[usTestCounter],
                               EntryPointList[usTestCounter].TestName);
  }

  InitTestSegVars(Selector);
  return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: absl()
 *
 * Purpose: This function will return the absolute value of a LONG INT.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 09/09/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    n              // long integer
 *
 * Outputs:
 *              LONG:   returns the abs of n.
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              NONE
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *     if n < 0 then return -n;
 *     else return n;
 * }
 *\\end
 *****************************************************************************/

LONG absl(LONG n)
{
   if (n < 0)
      return (-n);
   else
      return (n);
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: theta()
 *
 * Purpose: This function will return angle theta between two points.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 09/09/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    p1             // first point
 *    p2             // second point
 *
 * Outputs:
 *              float:  returns theta.
 *
 * Subroutines Required:
 *              absl();
 *
 * Limitations:
 *              NONE
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *     x = p2.x - p1.x;
 *     y = p2.y - p1.y
 *     if x = 0 and y = 0 then theta = 0;
 *     else theta = y / absl(x) + absl(y);
 *     if x < 0 then theta = 2 - theta;
 *     else if y < 0 then theta = 4 + theta;
 *     return theta * 90;
 * }
 *\\end
 *****************************************************************************/
#pragma page()
float theta(POINTL *p1, POINTL *p2)
{
LONG dx, dy, ax, ay;
float t;

   dx = p2->x - p1->x; ax = absl(dx);
   dy = p2->y - p1->y; ay = absl(dy);
   if ((dx == 0) && (dy == 0))
      t = 0;
   else
      t = ((float) dy) / (float)(ax + ay);
   if (dx < 0)
      t = 2 - t;
   else if (dy < 0)
      t = 4 + t;
   return (t * 90.0);
}

LONG getDiff(LONG v1, LONG v2)
{
   if (v1 < v2)
      return v2 - v1;
   return v1 - v2;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: RandArc()
 *
 * Purpose: This function will get a random arc structure.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 09/09/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *
 * Outputs:
 *              BOOL:   TRUE  - Function Passed
 *                      FALSE - Function Failed
 *
 * Subroutines Required:
 *              theta();
 *
 * Limitations:
 *              NONE
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *     loop from 0 to sizeBox1;
 *          loop from 1 to sizeBox2;
 *              get theta for each point of box1 to each point of box2;
 *              if feetheta = 180 between 2 points then
 *                  return TRUE point lies on a line of box2;
 *          qsort theta array;
 *          if angle theta[0] == angle theta[1] then
 *              apptheta to use is (theta[2] - theta[0])/2 + theta[0];
 *          else
 *              apptheta to use will be (theta[1] - theta[0])/2 + theta[0];
 *          if inside returns TRUE then return TRUE;
 *          else return FALSE;
 * }
 *\\end
 *****************************************************************************/
#pragma page()
LONG randArc(PPSRANGE p_ranges, POINTL *ptl, POINTL *aptl)
{
float   angTheta;
POINTL  pTemp;
LONG    radius;

   aptl[0].x = randL(p_ranges->Min.x, p_ranges->Max.x);
   aptl[0].y = randL(p_ranges->Min.y, p_ranges->Max.y);
   angTheta = theta(ptl, &aptl[0]);

   if (aptl[0].x < ptl->x)
     {
        aptl[0].x = (ptl->x - aptl[0].x)/2 + aptl[0].x;
        pTemp.x = aptl[0].x - p_ranges->Min.x;
     }
   else
     {
        aptl[0].x = (aptl[0].x - ptl->x)/2 + ptl->x;
        pTemp.x = p_ranges->Max.x - aptl[0].x;
     }

   if (aptl[0].y < ptl->y)
     {
        aptl[0].y = (ptl->y - aptl[0].y)/2 + aptl[0].y;
        pTemp.y = aptl[0].y - p_ranges->Min.y;
     }
   else
     {
        aptl[0].y = (aptl[0].y - ptl->y)/2 + ptl->y;
        pTemp.y = p_ranges->Max.y - aptl[0].y;
     }

   radius = (pTemp.x < pTemp.y) ? pTemp.x : pTemp.y;

   pTemp.x = randL(-radius, radius);
   pTemp.y = randL(0L, (LONG) sqrt(pow((double) radius, 2.0) -
                                   pow((double) pTemp.x, 2.0)));
   aptl[1].x = (LONG) ((cos((double) (-90.0 + angTheta)/57.29578) * pTemp.x) -
                  (sin((double) (-90.0 + angTheta)/57.29578) * pTemp.y));
   aptl[1].y = (LONG) ((sin((double) (-90.0 + angTheta)/57.29578) * pTemp.x) +
                  (cos((double) (-90.0 + angTheta)/57.29578) * pTemp.y));
   aptl[1].x += aptl[0].x;
   aptl[1].y += aptl[0].y;

   return (LONG) angTheta;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: RandArcParms()
 *
 * Purpose: This function will get a set of random arc params.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 09/09/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *
 * Outputs:
 *              BOOL:   TRUE  - Function Passed
 *                      FALSE - Function Failed
 *
 * Subroutines Required:
 *              theta();
 *
 * Limitations:
 *              NONE
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *     loop from 0 to sizeBox1;
 *          loop from 1 to sizeBox2;
 *              get theta for each point of box1 to each point of box2;
 *              if feetheta = 180 between 2 points then
 *                  return TRUE point lies on a line of box2;
 *          qsort theta array;
 *          if angle theta[0] == angle theta[1] then
 *              apptheta to use is (theta[2] - theta[0])/2 + theta[0];
 *          else
 *              apptheta to use will be (theta[1] - theta[0])/2 + theta[0];
 *          if inside returns TRUE then return TRUE;
 *          else return FALSE;
 * }
 *\\end
 *****************************************************************************/
#pragma page()
LONG randArcParms(PPSRANGE p_ranges, POINTL *ptl, ARCPARAMS *arcParm)
{
float   angTheta;
POINTL  p1;
LONG    tempL;
double  tempD;

   arcParm->lR = p1.x = randL(p_ranges->Min.x, p_ranges->Max.x);
   arcParm->lQ = p1.y = randL(p_ranges->Min.y, p_ranges->Max.y);

   p1.x = p1.x + ptl->x;                                     // get major axis.
   p1.y = p1.y + ptl->y;
   angTheta = theta(ptl, &p1);

   tempL = randL(p_ranges->Min.y, p_ranges->Max.y);          // get minor axis
                                                             // of x=0, y=tempL.
   tempD = (double) angTheta/57.29578;
   arcParm->lP = (LONG) -(sin(tempD) * tempL);               // rotate minor
   arcParm->lS = (LONG) (cos(tempD) * tempL);                // axis to 90 deg.
                                                             // off of major axis.
   return (LONG) angTheta;
}

VOID SetParams(HPS hPS, HWND hWndClient, PARCPARAMS arcParm1)
{
static CHAR     szParm[50];                                          // Current Tag varable in displayable form.
static ARCPARAMS arcParm2;

   sprintf(szParm,"r = %ld, q = %ld, p = %ld, s = %ld",
                                           arcParm1->lR,
                                           arcParm1->lQ,
                                           arcParm1->lP,
                                           arcParm1->lS);
   LOGINFO(L_DEBUG,"*",szParm);
   if (!GpiSetArcParams(hPS, arcParm1))            // set parameters.
     {
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiSetArcParams:",NULL);
     }
   if (!GpiQueryArcParams(hPS, &arcParm2))            // get current parameters.
     {
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiQueryArcParams:",NULL);
     }
   if (memcmp(arcParm1, &arcParm2, sizeof(ARCPARAMS)))  // compare current parameters
     {                                                     // with set parameters.
        sprintf(szParm,"r = %ld, q = %ld, p = %ld, s = %ld",
                                                arcParm2.lR,
                                                arcParm2.lQ,
                                                arcParm2.lP,
                                                arcParm2.lS);
        ErrorCount++;
        ERRSTATUS(TRUE,szDontMatch,szParm);
     }
   return;
}
