/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma pagesize(59)

/* const char header=$header$ */

#define INCL_DOSPROCESS
#define INCL_DEV      // include all DEV definitions
#define INCL_WIN
#define INCL_GPI      // include all Gpi definitions
#define INCL_GRE_ARCS
#define INCL_GRE_DEVSUPPORT
#define INCL_DDIMISC
#define INCL_DDIBUNDLES
#define INCL_ERRORS   // include all Gpi error definitions

#include <os2.h>
#include <pmddi.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "common2.h"
#include "grearcx.h"

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreSetandGetArcParametersExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreSetArcParameters and GreGetArcParameters.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/15/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              set randomizer to pre-selected seed;
 *              loop from MINLOOP to MAXLOOP;
 *                      arcParm1 = random numbers;
 *                      run through set and get arc parameters;
 *                      check set parameters with queried parameters;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreSetandGetArcParametersExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreSetandGetArcParametersExh";        //      entry point name
static ARCPARAMS arcParm1, arcParm2;
static CHAR     szParm[50];
static HDC      hDC;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                arcParm1.lR = (LONG) rand();                                // set arc parameters for 1/2" circle.
                arcParm1.lQ = (LONG) rand();
                arcParm1.lP = (LONG) rand();
                arcParm1.lS = (LONG) rand();
                MapArc(arcParm1);
                sprintf(szParm,"r = %ld, q = %ld, p = %ld, s = %ld",
                                                        arcParm1.lR,
                                                        arcParm1.lQ,
                                                        arcParm1.lP,
                                                        arcParm1.lS);
                LOGINFO(L_DEBUG,"*",szParm);
                if (!GreSetArcParameters(hDC, &arcParm1))            // set parameters.
                  {
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreSetArcParameters:",NULL);
                  }
                if (!GreGetArcParameters(hDC, &arcParm2))            // get current parameters.
                  {
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreGetArcParameters:",NULL);
                  }
                if(memcmp(&arcParm1, &arcParm2, sizeof(ARCPARAMS)))  // compare current parameters
                  {                                                     // with set parameters.
                     sprintf(szParm,"r = %ld, q = %ld, p = %ld, s = %ld",
                                                             arcParm2.lR,
                                                             arcParm2.lQ,
                                                             arcParm2.lP,
                                                             arcParm2.lS);
                     ErrorCount++;
                     ERRSTATUS(TRUE,szDontMatch,szParm);
                  }
             }
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreSetandGetDefArcParametersExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreSetDefaultArcParameters and GreGetDefaultArcParameters.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/15/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              set randomizer to pre-selected seed;
 *              loop from MINLOOP to MAXLOOP;
 *                      arcParm1 = random numbers;
 *                      run through set and get default arc parameters;
 *                      check set parameters with queried parameters;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreSetandGetDefArcParametersExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreSetandGetDefArcParametersExh";        //      entry point name
static ARCPARAMS arcParm1, arcParm2;
static CHAR     szParm[50];
static HDC      hDC;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                arcParm1.lR = (LONG) rand();                                // set arc parameters for 1/2" circle.
                arcParm1.lQ = (LONG) rand();
                arcParm1.lP = (LONG) rand();
                arcParm1.lS = (LONG) rand();
                MapArc(arcParm1);
                sprintf(szParm,"r = %ld, q = %ld, p = %ld, s = %ld",
                                                        arcParm1.lR,
                                                        arcParm1.lQ,
                                                        arcParm1.lP,
                                                        arcParm1.lS);
                LOGINFO(L_DEBUG,"*",szParm);
                if (!GreSetDefaultArcParameters(hDC, &arcParm1))            // set parameters.
                  {
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreSetDefaultArcParameters:",NULL);
                  }
                if (!GreGetDefaultArcParameters(hDC, &arcParm2))            // get current parameters.
                  {
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreGetDefaultArcParameters:",NULL);
                  }
                if(memcmp(&arcParm1, &arcParm2, sizeof(ARCPARAMS)))  // compare current parameters
                  {                                                     // with set parameters.
                     sprintf(szParm,"r = %ld, q = %ld, p = %ld, s = %ld",
                                                             arcParm2.lR,
                                                             arcParm2.lQ,
                                                             arcParm2.lP,
                                                             arcParm2.lS);
                     ErrorCount++;
                     ERRSTATUS(TRUE,szDontMatch,szParm);
                  }
             }
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreArcExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreArc.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/15/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *              set arc parameters for a circle;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -32k/32k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              set max ranges for arc;
 *              loop from MINLOOP to MAXLOOP;
 *                      get random arc points;
 *                      change color;
 *                      call GreArc();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreArcExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreArcExh";        //      entry point name
static SIZEL    size;
static POINTL   ptl, aptl[2];
static ARCPARAMS arcParm;
static PSRANGE  p_ranges;
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);

           arcParm.lQ = 1L;
           arcParm.lP = 1L;
           arcParm.lR = 0L;                                // set arc parameters for circle.
           arcParm.lS = 0L;
           MapArc(arcParm);

           if (!GpiSetArcParams(hPS, &arcParm))            // set parameters.
             {
                ErrorCount++;
                ERRSTATUS(TRUE,"GpiSetArcParams:",NULL);
             }
           GpiQueryPS(hPS, &size);


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.
           MapPt(ptl);
           p_ranges.Max.x = MAX_32K;
           p_ranges.Max.y = MAX_32K;
           p_ranges.Min.x = -MAX_32K;
           p_ranges.Min.y = -MAX_32K;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArc(&p_ranges, &ptl, &aptl[0]);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreArc(hDC, aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[0].x,
                                                               aptl[0].y,
                                                               aptl[1].x,
                                                               aptl[1].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreArc:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           p_ranges.Max.x = size.cx * 2;
           p_ranges.Max.y = size.cy * 2;
           p_ranges.Min.x = 0L;
           p_ranges.Min.y = 0L;
           ptl.x = delta_x * pg_size_x;                     // set position for the location
           ptl.y = delta_y * pg_size_y;                     // of drawing to the PS.
           MapPt(ptl);
           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArc(&p_ranges, &ptl, &aptl[0]);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreArc(hDC, aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[0].x,
                                                               aptl[0].y,
                                                               aptl[1].x,
                                                               aptl[1].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreArc:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           p_ranges.Max.x = size.cx;
           p_ranges.Max.y = size.cy;
           p_ranges.Min.x = 0L;
           p_ranges.Min.y = 0L;
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);
           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArc(&p_ranges, &ptl, &aptl[0]);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreArc(hDC, aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[0].x,
                                                               aptl[0].y,
                                                               aptl[1].x,
                                                               aptl[1].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreArc:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GrePartialArcExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GrePartialArc.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/15/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *              set max ranges for arc;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -32k/32k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              loop from MINLOOP to MAXLOOP;
 *                      get random arc parameters;
 *                      set arc parameters;
 *                      change color;
 *                      call GrePartialArc();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GrePartialArcExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GrePartialArcExh";        //      entry point name
static ARCPARAMS arcParm;
static INT      tempint;
static POINTL   ptl;
static PSRANGE  p_ranges;
static LONG     angTheta, Maxfixed;
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);

           p_ranges.Max.x = delta_x;
           p_ranges.Max.y = delta_y;
           p_ranges.Min.x = -delta_x;
           p_ranges.Min.y = -delta_y;


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (delta_x < delta_y) ? (MAX_16K/delta_x) : (MAX_16K/delta_y);
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1L,Maxfixed);
                if (!GrePartialArc(hDC, &ptl, MAKEFIXED(tempint,0),
                                              MAKEFIXED((int) angTheta,0),
                                              MAKEFIXED((int) randL(0L,360L),0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePartialArc:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (pg_size_x < pg_size_y) ? pg_size_x : pg_size_y;
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GrePartialArc(hDC, &ptl, MAKEFIXED(tempint,0),
                                              MAKEFIXED((int) angTheta,0),
                                              MAKEFIXED((int) randL(0L,360L),0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePartialArc:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (pg_size_x < pg_size_y) ? (pg_size_x/2) : (pg_size_y/2);
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GrePartialArc(hDC, &ptl, MAKEFIXED(tempint,0),
                                              MAKEFIXED((int) angTheta,0),
                                              MAKEFIXED((int) randL(0L,360L),0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePartialArc:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreFullArcInteriorExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreFullArcInterior.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/15/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *              set max ranges for arc;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -16k/16k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              get max magnifier variable;
 *              loop from MINLOOP to MAXLOOP;
 *                      get random arc parameters;
 *                      set arc parameters;
 *                      get random magnifier parameter;
 *                      change color;
 *                      call GreFullArcInterior();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreFullArcInteriorExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreFullArcInteriorExh";        //      entry point name
static ARCPARAMS arcParm;
static INT      tempint;
static POINTL   ptl;
static PSRANGE  p_ranges;
static LONG     Maxfixed;
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.


   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);
           if (DeviceIsPrinter())
              GpiSetPattern(hPS, PATSYM_DENSE7);

           p_ranges.Max.x = delta_x;
           p_ranges.Max.y = delta_y;
           p_ranges.Min.x = -delta_x;
           p_ranges.Min.y = -delta_y;


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (delta_x < delta_y) ? (MAX_8K/delta_x) : (MAX_8K/delta_y);
           Maxfixed = (Maxfixed < 256) ? (Maxfixed - 1) : 255;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GreFullArcInterior(hDC, MAKEFIXED(tempint,
                                                       0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreFullArcInterior:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (pg_size_x < pg_size_y) ? pg_size_x : pg_size_y;
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GreFullArcInterior(hDC, MAKEFIXED(tempint,
                                                       0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreFullArcInterior:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (pg_size_x < pg_size_y) ? (pg_size_x/2) : (pg_size_y/2);
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GreFullArcInterior(hDC, MAKEFIXED(tempint,
                                                       0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreFullArcInterior:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreFullArcBoundaryExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreFullArcBoundary.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/15/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *              set max ranges for arc;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -32k/32k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              get max magnifier variable;
 *              loop from MINLOOP to MAXLOOP;
 *                      get random arc parameters;
 *                      set arc parameters;
 *                      get random magnifier parameter;
 *                      change color;
 *                      call GreFullArcBoundary();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreFullArcBoundaryExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreFullArcBoundaryExh";        //      entry point name
static ARCPARAMS arcParm;
static INT      tempint;
static POINTL   ptl;
static PSRANGE  p_ranges;
static LONG     Maxfixed;
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.


   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);

           p_ranges.Max.x = delta_x;
           p_ranges.Max.y = delta_y;
           p_ranges.Min.x = -delta_x;
           p_ranges.Min.y = -delta_y;


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (delta_x < delta_y) ? (MAX_16K/delta_x) : (MAX_16K/delta_y);
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GreFullArcBoundary(hDC, MAKEFIXED(tempint,
                                                       0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreFullArcBoundary:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (pg_size_x < pg_size_y) ? pg_size_x : pg_size_y;
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GreFullArcBoundary(hDC, MAKEFIXED(tempint,
                                                       0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreFullArcBoundary:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (pg_size_x < pg_size_y) ? (pg_size_x/2) : (pg_size_y/2);
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GreFullArcBoundary(hDC, MAKEFIXED(tempint,
                                                       0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreFullArcBoundary:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreFullArcBothExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreFullArcBoth.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/15/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *              set max ranges for arc;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -16k/16k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              get max magnifier variable;
 *              loop from MINLOOP to MAXLOOP;
 *                      get random arc parameters;
 *                      set arc parameters;
 *                      get random magnifier parameter;
 *                      change color;
 *                      call GreFullArcBoth();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreFullArcBothExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreFullArcBothExh";        //      entry point name
static ARCPARAMS arcParm;
static INT      tempint;
static POINTL   ptl;
static PSRANGE  p_ranges;
static LONG     Maxfixed;
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);
           GpiSetPattern(hPS, PATSYM_DENSE7);

           p_ranges.Max.x = delta_x;
           p_ranges.Max.y = delta_y;
           p_ranges.Min.x = -delta_x;
           p_ranges.Min.y = -delta_y;


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (delta_x < delta_y) ? (MAX_8K/delta_x) : (MAX_8K/delta_y);
           Maxfixed = (Maxfixed < 256) ? (Maxfixed - 1) : 255;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GreFullArcBoth(hDC, MAKEFIXED(tempint,
                                                       0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreFullArcBoth:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (pg_size_x < pg_size_y) ? pg_size_x : pg_size_y;
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GreFullArcBoth(hDC, MAKEFIXED(tempint,
                                                       0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreFullArcBoth:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           Maxfixed = (pg_size_x < pg_size_y) ? (pg_size_x/2) : (pg_size_y/2);
           Maxfixed = (Maxfixed < 256) ? Maxfixed : 256;

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                randArcParms(&p_ranges, &ptl, &arcParm);
                MapArc(arcParm);
                SetParams(hPS, hWndClient, &arcParm);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                tempint = (int) randL(1,Maxfixed);
                if (!GreFullArcBoth(hDC, MAKEFIXED(tempint,
                                                       0)))
                  {
                     sprintf(szParm,"%d",tempint);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreFullArcBoth:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreBoxInteriorExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreBoxInterior.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/28/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -16k/16k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              loop from MINLOOP to MAXLOOP;
 *                      get random point;
 *                      get random second point based on first point;
 *                      change color;
 *                      call GreBoxInterior();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreBoxInteriorExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreBoxInteriorExh";        //      entry point name
static SIZEL    size;
static POINTL   ptl;
static BOXPARAMS aptl;
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);
           GpiQueryPS(hPS, &size);
           if (DeviceIsPrinter())
              GpiSetPattern(hPS, PATSYM_DENSE7);


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl.ptl.x = randL(-MAX_16K, MAX_16K);
                aptl.ptl.y = randL(-MAX_16K, MAX_16K);
                aptl.sizl.cx = (aptl.ptl.x < ptl.x) ? randL(ptl.x, -aptl.ptl.x)
                                                    : randL(ptl.x, aptl.ptl.x);
                aptl.sizl.cy = (aptl.ptl.y < ptl.y) ? randL(ptl.y, -aptl.ptl.y)
                                                    : randL(ptl.y, aptl.ptl.y);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreBoxInterior(hDC, &aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl.ptl.x,
                                                               aptl.ptl.y,
                                                               aptl.sizl.cx,
                                                               aptl.sizl.cy);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreBoxInterior:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl.ptl.x = randL(0L, size.cx * 2);
                aptl.ptl.y = randL(0L, size.cy * 2);
                aptl.sizl.cx = (aptl.ptl.x < ptl.x) ? randL(ptl.x, -aptl.ptl.x)
                                                    : randL(ptl.x, aptl.ptl.x);
                aptl.sizl.cy = (aptl.ptl.y < ptl.y) ? randL(ptl.y, -aptl.ptl.y)
                                                    : randL(ptl.y, aptl.ptl.y);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreBoxInterior(hDC, &aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl.ptl.x,
                                                               aptl.ptl.y,
                                                               aptl.sizl.cx,
                                                               aptl.sizl.cy);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreBoxInterior:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl.ptl.x = randL(0L, size.cx);
                aptl.ptl.y = randL(0L, size.cy);
                aptl.sizl.cx = (aptl.ptl.x < ptl.x) ? randL(ptl.x, -aptl.ptl.x)
                                                    : randL(ptl.x, aptl.ptl.x);
                aptl.sizl.cy = (aptl.ptl.y < ptl.y) ? randL(ptl.y, -aptl.ptl.y)
                                                    : randL(ptl.y, aptl.ptl.y);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreBoxInterior(hDC, &aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl.ptl.x,
                                                               aptl.ptl.y,
                                                               aptl.sizl.cx,
                                                               aptl.sizl.cy);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreBoxInterior:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreBoxBoundaryExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreBoxBoundary.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/28/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -16k/16k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              loop from MINLOOP to MAXLOOP;
 *                      get random point;
 *                      get random second point based on first point;
 *                      change color;
 *                      call GreBoxBoundary();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreBoxBoundaryExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreBoxBoundaryExh";        //      entry point name
static SIZEL    size;
static POINTL   ptl;
static BOXPARAMS aptl;
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);
           GpiQueryPS(hPS, &size);


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl.ptl.x = randL(-MAX_16K, MAX_16K);
                aptl.ptl.y = randL(-MAX_16K, MAX_16K);
                aptl.sizl.cx = (aptl.ptl.x < ptl.x) ? randL(ptl.x, -aptl.ptl.x)
                                                    : randL(ptl.x, aptl.ptl.x);
                aptl.sizl.cy = (aptl.ptl.y < ptl.y) ? randL(ptl.y, -aptl.ptl.y)
                                                    : randL(ptl.y, aptl.ptl.y);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreBoxBoundary(hDC, &aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl.ptl.x,
                                                               aptl.ptl.y,
                                                               aptl.sizl.cx,
                                                               aptl.sizl.cy);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreBoxBoundary:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl.ptl.x = randL(0L, size.cx * 2);
                aptl.ptl.y = randL(0L, size.cy * 2);
                aptl.sizl.cx = (aptl.ptl.x < ptl.x) ? randL(ptl.x, -aptl.ptl.x)
                                                    : randL(ptl.x, aptl.ptl.x);
                aptl.sizl.cy = (aptl.ptl.y < ptl.y) ? randL(ptl.y, -aptl.ptl.y)
                                                    : randL(ptl.y, aptl.ptl.y);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreBoxBoundary(hDC, &aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl.ptl.x,
                                                               aptl.ptl.y,
                                                               aptl.sizl.cx,
                                                               aptl.sizl.cy);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreBoxBoundary:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl.ptl.x = randL(0L, size.cx);
                aptl.ptl.y = randL(0L, size.cy);
                aptl.sizl.cx = (aptl.ptl.x < ptl.x) ? randL(ptl.x, -aptl.ptl.x)
                                                    : randL(ptl.x, aptl.ptl.x);
                aptl.sizl.cy = (aptl.ptl.y < ptl.y) ? randL(ptl.y, -aptl.ptl.y)
                                                    : randL(ptl.y, aptl.ptl.y);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreBoxBoundary(hDC, &aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl.ptl.x,
                                                               aptl.ptl.y,
                                                               aptl.sizl.cx,
                                                               aptl.sizl.cy);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreBoxBoundary:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreBoxBothExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreBoxBoth.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/28/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -32k/32k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              loop from MINLOOP to MAXLOOP;
 *                      get random point;
 *                      get random second point based on first point;
 *                      change color;
 *                      call GreBoxBoth();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreBoxBothExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreBoxBothExh";        //      entry point name
static SIZEL    size;
static POINTL   ptl;
static BOXPARAMS aptl;
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);
           GpiSetPattern(hPS, PATSYM_DENSE7);
           GpiQueryPS(hPS, &size);


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl.ptl.x = randL(-MAX_16K, MAX_16K);
                aptl.ptl.y = randL(-MAX_16K, MAX_16K);
                aptl.sizl.cx = (aptl.ptl.x < ptl.x) ? randL(ptl.x, -aptl.ptl.x)
                                                    : randL(ptl.x, aptl.ptl.x);
                aptl.sizl.cy = (aptl.ptl.y < ptl.y) ? randL(ptl.y, -aptl.ptl.y)
                                                    : randL(ptl.y, aptl.ptl.y);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreBoxBoth(hDC, &aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl.ptl.x,
                                                               aptl.ptl.y,
                                                               aptl.sizl.cx,
                                                               aptl.sizl.cy);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreBoxBoth:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl.ptl.x = randL(0L, size.cx * 2);
                aptl.ptl.y = randL(0L, size.cy * 2);
                aptl.sizl.cx = (aptl.ptl.x < ptl.x) ? randL(ptl.x, -aptl.ptl.x)
                                                    : randL(ptl.x, aptl.ptl.x);
                aptl.sizl.cy = (aptl.ptl.y < ptl.y) ? randL(ptl.y, -aptl.ptl.y)
                                                    : randL(ptl.y, aptl.ptl.y);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreBoxBoth(hDC, &aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl.ptl.x,
                                                               aptl.ptl.y,
                                                               aptl.sizl.cx,
                                                               aptl.sizl.cy);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreBoxBoth:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl.ptl.x = randL(0L, size.cx);
                aptl.ptl.y = randL(0L, size.cy);
                aptl.sizl.cx = (aptl.ptl.x < ptl.x) ? randL(ptl.x, -aptl.ptl.x)
                                                    : randL(ptl.x, aptl.ptl.x);
                aptl.sizl.cy = (aptl.ptl.y < ptl.y) ? randL(ptl.y, -aptl.ptl.y)
                                                    : randL(ptl.y, aptl.ptl.y);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GreBoxBoth(hDC, &aptl))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl.ptl.x,
                                                               aptl.ptl.y,
                                                               aptl.sizl.cx,
                                                               aptl.sizl.cy);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GreBoxBoth:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}
