/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma pagesize(59)

/* const char header=$header$ */

#define INCL_DOSPROCESS
#define INCL_DEV      // include all DEV definitions
#define INCL_WIN
#define INCL_GPI      // include all Gpi definitions
#define INCL_GRE_ARCS
#define INCL_GRE_DEVSUPPORT
#define INCL_DDIMISC
#define INCL_DDIBUNDLES
#define INCL_ERRORS   // include all Gpi error definitions

#include <os2.h>
#include <pmddi.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "common2.h"
#include "grearcx.h"

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GrePolyFilletExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GrePolyFillet.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/28/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -32k/32k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              loop from MINLOOP to MAXLOOP;
 *                      get 2 random points;
 *                      get 3 more random points based on other points;
 *                      change color;
 *                      call GrePolyFillet();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GrePolyFilletExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GrePolyFilletExh";        //      entry point name
static POINTL   ptl, aptl[5];
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl[1].x = randL(ptl.x, MAX_32K);
                aptl[1].y = randL(ptl.y, MAX_32K);
                aptl[3].x = randL(-MAX_32K, ptl.x);
                aptl[3].y = randL(-MAX_32K, ptl.y);
                aptl[0].x = aptl[1].x;
                aptl[0].y = aptl[3].y;
                aptl[2].x = aptl[3].x;
                aptl[2].y = aptl[1].y;
                aptl[4].x = aptl[3].x + (aptl[1].x - aptl[3].x)/2;
                aptl[4].y = aptl[3].y;

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &aptl[4]);
                if (!GrePolyFillet(hDC, &aptl[0], 5L))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[1].x,
                                                               aptl[1].y,
                                                               aptl[3].x,
                                                               aptl[3].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePolyFillet:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl[1].x = randL(ptl.x, MapX(delta_x * pg_size_x * 2));
                aptl[1].y = randL(ptl.y, MapY(delta_y * pg_size_y * 2));
                aptl[3].x = randL(0L, ptl.x);
                aptl[3].y = randL(0L, ptl.y);
                aptl[0].x = aptl[1].x;
                aptl[0].y = aptl[3].y;
                aptl[2].x = aptl[3].x;
                aptl[2].y = aptl[1].y;
                aptl[4].x = aptl[3].x + (aptl[1].x - aptl[3].x)/2;
                aptl[4].y = aptl[3].y;

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &aptl[4]);
                if (!GrePolyFillet(hDC, &aptl[0], 5L))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[1].x,
                                                               aptl[1].y,
                                                               aptl[3].x,
                                                               aptl[3].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePolyFillet:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl[1].x = randL(ptl.x, MapX(delta_x * pg_size_x));
                aptl[1].y = randL(ptl.y, MapY(delta_y * pg_size_y));
                aptl[3].x = randL(0L, ptl.x);
                aptl[3].y = randL(0L, ptl.y);
                aptl[0].x = aptl[1].x;
                aptl[0].y = aptl[3].y;
                aptl[2].x = aptl[3].x;
                aptl[2].y = aptl[1].y;
                aptl[4].x = aptl[3].x + (aptl[1].x - aptl[3].x)/2;
                aptl[4].y = aptl[3].y;

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &aptl[4]);
                if (!GrePolyFillet(hDC, &aptl[0], 5L))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[1].x,
                                                               aptl[1].y,
                                                               aptl[3].x,
                                                               aptl[3].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePolyFillet:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GrePolyFilletSharpExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GrePolyFilletSharp.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/28/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *
 *              // Will enter here 2 times with max/min ranges of:
 *              //    0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              loop from MINLOOP to MAXLOOP;
 *                      get 2 random points;
 *                      get 3 more random points based on other points;
 *                      get 5 random fixed sharpness values;
 *                      change color;
 *                      call GrePolyFilletSharp();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GrePolyFilletSharpExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GrePolyFilletSharpExh";        //      entry point name
static POINTL   ptl, aptl[5];
static FIXED    afx[5];
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;

register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);


           TEST_PROGRESS ( 00, 2 * MAXLOOP );

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl[1].x = randL(ptl.x, MapX(delta_x * pg_size_x * 2));
                aptl[1].y = randL(ptl.y, MapY(delta_y * pg_size_y * 2));
                aptl[3].x = randL(0L, ptl.x);
                aptl[3].y = randL(0L, ptl.y);
                aptl[0].x = aptl[1].x;
                aptl[0].y = aptl[3].y;
                aptl[2].x = aptl[3].x;
                aptl[2].y = aptl[1].y;
                aptl[4].x = aptl[3].x + (aptl[1].x - aptl[3].x)/2;
                aptl[4].y = aptl[3].y;

                afx[0] = MAKEFIXED((int) randL(-9L,9L),0);
                afx[1] = MAKEFIXED((int) randL(-9L,9L),0);
                afx[2] = MAKEFIXED((int) randL(-9L,9L),0);
                afx[3] = MAKEFIXED((int) randL(-9L,9L),0);
                afx[4] = MAKEFIXED((int) randL(-9L,9L),0);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &aptl[4]);
                if (!GrePolyFilletSharp(hDC, &aptl[0], 10L, afx))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[1].x,
                                                               aptl[1].y,
                                                               aptl[3].x,
                                                               aptl[3].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePolyFilletSharp:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 2 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl[1].x = randL(ptl.x, MapX(delta_x * pg_size_x));
                aptl[1].y = randL(ptl.y, MapY(delta_y * pg_size_y));
                aptl[3].x = randL(0L, ptl.x);
                aptl[3].y = randL(0L, ptl.y);
                aptl[0].x = aptl[1].x;
                aptl[0].y = aptl[3].y;
                aptl[2].x = aptl[3].x;
                aptl[2].y = aptl[1].y;
                aptl[4].x = aptl[3].x + (aptl[1].x - aptl[3].x)/2;
                aptl[4].y = aptl[3].y;

                afx[0] = MAKEFIXED((int) randL(-9L,9L),0);
                afx[1] = MAKEFIXED((int) randL(-9L,9L),0);
                afx[2] = MAKEFIXED((int) randL(-9L,9L),0);
                afx[3] = MAKEFIXED((int) randL(-9L,9L),0);
                afx[4] = MAKEFIXED((int) randL(-9L,9L),0);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &aptl[4]);
                if (!GrePolyFilletSharp(hDC, &aptl[0], 10L, afx))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[1].x,
                                                               aptl[1].y,
                                                               aptl[3].x,
                                                               aptl[3].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePolyFilletSharp:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 2 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 2 * MAXLOOP, 2 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GrePolySplineExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GrePolySpline.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 10/28/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Can not Print.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ensure PS not to Printer;
 *              ENTER_MAIN();
 *              setup clip region;
 *              set randomizer to pre-selected seed;
 *
 *              // Will enter here 3 times with max/min ranges of:
 *              //    -32k/32k, 0/(2 * size of PS) and 0/(size of PS).
 *
 *              SUBENTER();
 *              set center point of arc;
 *              loop from MINLOOP to MAXLOOP;
 *                      get six random POINTL's;
 *                      change color;
 *                      call GrePolySpline();
 *              SUBEXIT();
 *
 *              distroy clip region;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GrePolySplineExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GrePolySplineExh";        //      entry point name
static POINTL   ptl, aptl[6];
static CHAR     szParm[50];
static HDC      hDC;
static RECTL    rect;
static HRGN     hrgn;
register INT    lvar;                                               // Loop varable.

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           ErrorCount = 0;
           hDC = GpiQueryDevice(hPS);
           srand(seed);                                     // Preset seed for randomization.

           rect.xLeft = 0;
           rect.yBottom = (alDevCaps[CAPS_HEIGHT] / pg_size_y) * 3/4; // Device height in pixels.
           rect.xRight = alDevCaps[CAPS_WIDTH ];              // Device width in pixels.
           rect.yTop = alDevCaps[CAPS_HEIGHT];
           hrgn=GpiCreateRegion(hPS,1L,&rect);
           GpiSetClipRegion(hPS,hrgn,NULL);


           TEST_PROGRESS ( 00, 3 * MAXLOOP );

           SUBENTER("MAX Size",NULL);
           ptl.x = 0;                         // set position for the location
           ptl.y = 0;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl[0].x = randL(-MAX_32K, MAX_32K);
                aptl[0].y = randL(-MAX_32K, MAX_32K);
                aptl[1].x = randL(-MAX_32K, MAX_32K);
                aptl[1].y = randL(-MAX_32K, MAX_32K);
                aptl[2].x = randL(-MAX_32K, MAX_32K);
                aptl[2].y = randL(-MAX_32K, MAX_32K);

                aptl[3].x = randL(-MAX_32K, MAX_32K);
                aptl[3].y = randL(-MAX_32K, MAX_32K);
                aptl[4].x = randL(-MAX_32K, MAX_32K);
                aptl[4].y = randL(-MAX_32K, MAX_32K);
                aptl[5].x = randL(-MAX_32K, MAX_32K);
                aptl[5].y = randL(-MAX_32K, MAX_32K);

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GrePolySpline(hDC, &aptl[0], 6L))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[0].x,
                                                               aptl[0].y,
                                                               aptl[1].x,
                                                               aptl[1].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePolySpline:",szParm);
                  }

                TEST_PROGRESS ( lvar + 1, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("2 times PS Size",NULL);
           ptl.x = delta_x * pg_size_x;                         // set position for the location
           ptl.y = delta_y * pg_size_y;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl[0].x = randL(0L, MapX(delta_x * pg_size_x * 2));
                aptl[0].y = randL(0L, MapY(delta_y * pg_size_y * 2));
                aptl[1].x = randL(0L, MapX(delta_x * pg_size_x * 2));
                aptl[1].y = randL(0L, MapY(delta_y * pg_size_y * 2));
                aptl[2].x = randL(0L, MapX(delta_x * pg_size_x * 2));
                aptl[2].y = randL(0L, MapY(delta_y * pg_size_y * 2));

                aptl[3].x = randL(0L, MapX(delta_x * pg_size_x * 2));
                aptl[3].y = randL(0L, MapY(delta_y * pg_size_y * 2));
                aptl[4].x = randL(0L, MapX(delta_x * pg_size_x * 2));
                aptl[4].y = randL(0L, MapY(delta_y * pg_size_y * 2));
                aptl[5].x = randL(0L, MapX(delta_x * pg_size_x * 2));
                aptl[5].y = randL(0L, MapY(delta_y * pg_size_y * 2));

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GrePolySpline(hDC, &aptl[0], 6L))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[0].x,
                                                               aptl[0].y,
                                                               aptl[1].x,
                                                               aptl[1].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePolySpline:",szParm);
                  }

                TEST_PROGRESS ( lvar + MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();

           SUBENTER("PS Size",NULL);
           ptl.x = (delta_x * pg_size_x)/2;                         // set position for the location
           ptl.y = (delta_y * pg_size_y)/2;                            // of drawing to the PS.
           MapPt(ptl);

           for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter test loop.
             {
                aptl[0].x = randL(0L, MapX(delta_x * pg_size_x));
                aptl[0].y = randL(0L, MapY(delta_y * pg_size_y));
                aptl[1].x = randL(0L, MapX(delta_x * pg_size_x));
                aptl[1].y = randL(0L, MapY(delta_y * pg_size_y));
                aptl[2].x = randL(0L, MapX(delta_x * pg_size_x));
                aptl[2].y = randL(0L, MapY(delta_y * pg_size_y));

                aptl[3].x = randL(0L, MapX(delta_x * pg_size_x));
                aptl[3].y = randL(0L, MapY(delta_y * pg_size_y));
                aptl[4].x = randL(0L, MapX(delta_x * pg_size_x));
                aptl[4].y = randL(0L, MapY(delta_y * pg_size_y));
                aptl[5].x = randL(0L, MapX(delta_x * pg_size_x));
                aptl[5].y = randL(0L, MapY(delta_y * pg_size_y));

                GpiSetColor(hPS, lvar % 16);
                GpiMove(hPS, &ptl);
                if (!GrePolySpline(hDC, &aptl[0], 6L))
                  {
                     sprintf(szParm,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                               aptl[0].x,
                                                               aptl[0].y,
                                                               aptl[1].x,
                                                               aptl[1].y);
                     ErrorCount++;
                     ERRSTATUS(TRUE,"GrePolySpline:",szParm);
                  }

                TEST_PROGRESS ( lvar + 2 * MAXLOOP, 3 * MAXLOOP );
             }
           SUBEXIT();


           TEST_PROGRESS ( 3 * MAXLOOP, 3 * MAXLOOP );

           GpiSetClipRegion(hPS,NULLHANDLE,&hrgn);
           GpiDestroyRegion(hPS,hrgn);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreDrawConicsInPathExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreDrawConicsInPath.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 11/25/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Not enough documentation to program.
 *
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreDrawConicsInPathExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreDrawConicsInPathExh";        //      entry point name

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreCookWholePathExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreCookWholePath.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 11/25/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Not enough documentation to program.
 *
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreCookWholePathExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreCookWholePathExh";        //      entry point name

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreCookPathCurvesExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreCookPathCurves.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 11/25/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Not enough documentation to program.
 *
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreCookPathCurvesExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreCookPathCurvesExh";        //      entry point name

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreRenderPathExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreRenderPath.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 11/25/91 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Not enough documentation to program.
 *
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreRenderPathExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR    szApiEntryName = "GreRenderPathExh";        //      entry point name

   if (SelectionCall)
     {
        WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
     }
   else
        {
           ENTER_MAIN(szApiEntryName);
           EXIT_MAIN(szApiEntryName);
        }
   return;
}
