/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/*const char header=$header$*/

#define INCL_DEV                    //Include all DEV definitions
#define INCL_WIN                    //Include all WIN definitions
#define INCL_GREALL                 // include all Gre definitions
#define INCL_GPI                    //Include all Gpi definitions
#define INCL_ERRORS                 //Include all error definitions
#define INCL_DOSERRORS
#define INCL_DOSMEMMGR               //Memory Management
#define INCL_DDIMISC

#define MAINSOURCEFILE              //This main source code.

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <os2.h>

#include <pmddi.h>                  // gre - device driver includes
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "greatt2.h"                // bit map dll header file

/****************************************************************************
 *\\ddd
 * Subtask Name: GreAtt2.DLL
 *
 * Module name: GreAtt2.c
 *
 * Purpose: Gre Attributes DLL test functions
 *
 * Revision Log:  09/15/91 ,PDVT ,Mike Abuzant
 *               -Taken from GpiBtmp.DLL Test Case
 *               - Test Case Design
 *               10/02/91 ,PDVT ,Mike Abuzant
 *               -Taken from GpiBtmp.DLL Test Case
 *               - Test Case Design
 *
 * SAD References:
 *
 * SRS References: Display Driver Verification Tool
 *
 * Contents:
 *
 * Design Overview:
 *   This TestCase is designed to test Gre Attributes functions. These functions
 *   change and query the attribute setting for most of the drawing primitives.
 *   primitives such as Lines, Images, Areas, Characters, and markers. Some of these
 *   functions affects the default attribute setting and others affect the current
 *   attribut setting.
 *
 *   Functions tested in this testcase
 *   are listed below :
 *
 *       GreInvalidateVisRegion()
 *       GreSetCursor()
 *       GreGetAttributes()
 *       GreSetAttributes()
 *       GreSetDefaultAttributes()
 *       GreGetDefaultAttributes()
 *       GreSetGlobalAttributes()
 *       GreInitializeAttributes()
 *
 *
 * Limitations:
 *
 * 1. The function GreSetCursor() is tested only for return codes due to
 *    the fact that it is not documented.
 *
 * 2. The function GreInvalidateVisRegion() is not test due to the fact that
 *    it is not documented.
 *
 * 3. Setting characters' default attributes to values other than the standard
 *    default values appears to work incorrectly. This suggests that the Gpi
 *    string functions do not work properly. This problem occurs only when
 *    the character mode is set to CM_MODE3.
 *
 * 4. No bitmaps are printed when using IBM4019. It seems to me that the
 *   IBM4019 device driver does not map colors and mixes correctly.
 *
 *\\end
****************************************************************************/
/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose:  Initialize function name and address array for DTT.
 *
 * System Requirements:
 *
 * Revision Log:
 *               02/03/91, pdvt, Mike Abuzant
 *               - Entry point names and address are now
 *               - put in an array.
 * Inputs:
 *   sel          // passed through to the InitTestSegVars function
 *   init_info_p  // name and address array used by DTT interface
 *
 * Outputs:
 *   init_info_p  // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()    // used by the DTT interface
 *
 * Limitations:
 *    None
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *   for the number of test functions {
 *       set function name
 *       set function address
 *   }
 *   call init set function
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY InitTest(PVOID Selector, PTESTINITINFO init_info_p)
{
  register USHORT usTestCounter;

  init_info_p->ntests = NTESTS;
  strcpy(init_info_p->group_name,TestCaseName); //Testcase Name

  for (usTestCounter=0 ;usTestCounter< NTESTS; ++usTestCounter){

     init_info_p->test_addrs[usTestCounter]=
                               EntryPointList[usTestCounter].TestAddress;
     strcpy(init_info_p->test_names[usTestCounter],
                               EntryPointList[usTestCounter].TestName);
  }

  InitTestSegVars(Selector);
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetCursorExh()
 *
 * Purpose:  This function exhaustively tests the functionality of
 *           Gre function GreSetCursor.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/11/91, pdvt ,Mike Abuzant
 *                  -Initial file.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *   None.
 * Subroutines Required:
 *    None.
 *
 * Limitations:
 *
 *    Currently this entry point is not implemented due to insufficeint
 *  documentation on this particular function.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetCursorExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PCHAR  EntryName="GreSetCursorExh";

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    EXIT_MAIN(EntryName);
  }
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreInvalidateVisRegionExh()
 *
 * Purpose:  This function exhaustively tests the functionality of
 *           Gre function GreInvalidateVisRegion.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/11/91, pdvt ,Mike Abuzant
 *                  -Initial file.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *   None.
 * Subroutines Required:
 *    None.
 *
 * Limitations:
 *
 *    Currently this entry point is not implemented due to fact that this
 *  Gre function is not documented.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreInvalidateVisRegionExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PCHAR  EntryName="GreDeviceSetCursorExh";

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    EXIT_MAIN(EntryName);
  }
}

/****************************************************************************
 *\\ddd
 * Routine Name: SetNewAttibutes()
 *
 * Purpose:  This function is used to set randomly selected attributes
 *           for all drawing primitives.
 *
 *
 * System Requirements:    None.
 *
 * Revision Log:
 *               10/15/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *
 * Inputs:
 *
 *        hPS                  -presentation space.
 *        hWndClient           -client window handle
 *        EntryName,           -Entry name where this function is called from.
 *        lGlobalAtttributFlag -A flag supplied to The Gre function.
 *
 * Outputs:
 *
 *        pusColorIndex        - The random foreground color index
 *        pusMixIndex          - The random foregound mix index
 *        pusBackMixIndex      - The random background mix index
 *        pusBackColorIndex    - The random background color index.
 *
 * Subroutines Required:  None.
 *
 * Limitations:           None.
 *
 *\\end
 ***************************************************************************/

/***************************************************************************
 *\\algorithm
 *
 *    Randomly select colors,mixes from a finite set of attributes.
 *
 *    Change the foreground color attribute for all drawing primitives.
 *    Change the background color attribute for all drawing primitives.
 *    Change the background Mix attribute for all drawing primitives.
 *    Change the foreground Mix attribute for all drawing primitives.
 *
 *    All the attributes above are set (changed) by calling the Gre
 *    function GreSetGlobalAttributes().
 *
 *    Return to calling entry point.
 *   returnto calling function.
 *\\end
 ***************************************************************************/

VOID SetNewAttibutes(HPS hPS, HWND hWndClient,
                     PCHAR    EntryName,
                     LONG     lGlobalAtttributFlag,
                     PUSHORT  pusColorIndex,
                     PUSHORT  pusMixIndex,
                     PUSHORT  pusBackMixIndex,
                     PUSHORT  pusBackColorIndex
                     )
{
  CHAR         Message[250];
  HDC          hDC;

   //Get  the default device context
   if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
       CLEANUP("GpiQueryDevice()",EntryName);
       return;   //Can't proceed without it
   }

   //Randomize new attribute settings.
    *pusColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
    *pusBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
    *pusMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
    *pusBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);

    sprintf(Message,"Setting Global Color= %s  BackColor=%s\n"
                    " Mix=%s   BackMix= %s",
                     asColor[*pusColorIndex].AttrSymbolicName,
                     asColor[*pusBackColorIndex].AttrSymbolicName,
                     asMix[*pusMixIndex].AttrSymbolicName,
                     asBackMix[*pusBackMixIndex].AttrSymbolicName);

    LOGINFO(L_DEBUG,"*",Message);

    //Set the Foreground color globally
    if (GreSetGlobalAttribute(hDC,ATYPE_COLOR,
                              asColor[*pusColorIndex].lAttrValue,
                              lGlobalAtttributFlag)==FALSE){
      ErrorCount++;
      ERRSTATUS(TRUE,"Setting Background color Failed",EntryName);
    }

    //Set the Background color globally.
    if (GreSetGlobalAttribute(hDC,ATYPE_BACK_COLOR,
                              asColor[*pusBackColorIndex].lAttrValue,
                              lGlobalAtttributFlag)==FALSE){
      ErrorCount++;
      ERRSTATUS(TRUE,"Setting Background color Failed",EntryName);
    }

    //Set the Background Mix globally.
    if (GreSetGlobalAttribute(hDC,ATYPE_BACK_MIX_MODE,
                              asBackMix[*pusBackMixIndex].lAttrValue,
                              lGlobalAtttributFlag)==FALSE){
      ErrorCount++;
      ERRSTATUS(TRUE,"Setting Background Mix Failed",EntryName);
    }

    //Set the Foreground Mix globally.
    if (GreSetGlobalAttribute(hDC,ATYPE_MIX_MODE,
                              asMix[*pusMixIndex].lAttrValue,
                              lGlobalAtttributFlag)==FALSE){
      ErrorCount++;
      ERRSTATUS(TRUE,"Setting Foreground Mix Failed",EntryName);
    }

  return;
}
/****************************************************************************
 *\\ddd
 * Routine Name: CompareDefaultAttibutesSetting()
 *
 * Purpose:  This function is used test if all primitive attributes are
 *           set to their standard defaults.
 *
 *
 * System Requirements:    None.
 *
 * Revision Log:
 *               10/15/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *
 * Inputs:
 *
 *        hPS                  -presentation space.
 *        hWndClient           -client window handle
 *        EntryName,           -Entry name where this function is called from.
 *
 * Outputs:        None.
 *
 * Subroutines Required:  None.
 *
 * Limitations:           None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *    Query current attributes for the Line primitive by calling the function
 *     GreGetAttributes(hDC,PRIM_LINE,..) and verify that attributes are set
 *     to their defaults.
 *
 *    Query current attributes for the Area primitive by calling the function
 *     GreGetAttributes(hDC.PRIM_AREA,..) and verify that attributes are
 *     set to their defaults.
 *
 *    Query current attributes for the Image primitive by calling the function
 *     GreGetAttributes(hDC.PRIM_IMAGE,..) and verify that attributes are
 *     set to their defaults.
 *
 *    Query current attributes for the Marker primitive by calling the function
 *     GreGetAttributes(hDC.PRIM_MARKER,..) and verify that attributes are
 *     set to their defaults.
 *
 *    Query current attributes for the Char primitive by calling the function
 *     GreGetAttributes(hDC.PRIM_CHAR,..) and verify that attributes are
 *     set to their defaults.
 *
 *    Return to calling entry point.
 *
 *\\end
 ***************************************************************************/
VOID CompareDefaultAttibutesSetting(HPS hPS, HWND hWndClient,PCHAR EntryName)
{
  CHAR         Message[250];
  HDC          hDC;

  BOOL         fOutCome;
  LONG         flDefAttrs;

  LINEBUNDLE   CurrentLineBundle={0};
  CHARBUNDLE   CurrentCharBundle={0};
  MARKERBUNDLE CurrentMarkerBundle={0};
  IMAGEBUNDLE  CurrentImageBundle={0};
  AREABUNDLE   CurrentAreaBundle={0};


   //Get  the default device context
   if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
       CLEANUP("GpiQueryDevice()",EntryName);
       return;   //Can't proceed without it
   }

   if ((flDefAttrs=GreGetAttributes(hDC,PRIM_AREA,AREA_ATTRIBUTES,
                        &CurrentAreaBundle))==GPI_ALTERROR){
      SHOWERROR("GreGetAttributes",EntryName);
   }

   ErrorCount+=!(fOutCome=VALIDATE_ATTRS(AREA_ATTRIBUTES,flDefAttrs));
   sprintf(Message,COMMON_SUFFICE1(Area),GOOD_FUNCTION(fOutCome));
   LOGINFO(L_DEBUG,"*",Message);


   //Get the current Line attributes.
   if ((flDefAttrs=GreGetAttributes(hDC,PRIM_LINE,LINE_ATTRIBUTES,
                        &CurrentLineBundle))==GPI_ALTERROR){
     SHOWERROR("GreGetAttributes",EntryName);
   }

   ErrorCount+=!(fOutCome=VALIDATE_ATTRS(LINE_ATTRIBUTES,flDefAttrs));
   sprintf(Message,COMMON_SUFFICE1(Line),GOOD_FUNCTION(fOutCome));
   LOGINFO(L_DEBUG,"*",Message);


   //Get the current Character attributes.They sould be equal to the defaults.
   if ((flDefAttrs=GreGetAttributes(hDC,PRIM_CHAR,CHAR_ATTRIBUTES,
                        &CurrentCharBundle))==GPI_ALTERROR){
     SHOWERROR("GreGetAttributes",EntryName);
   }

   ErrorCount+=!(fOutCome=VALIDATE_ATTRS(CHAR_ATTRIBUTES,flDefAttrs));
   sprintf(Message,COMMON_SUFFICE1(Character),GOOD_FUNCTION(fOutCome));
   LOGINFO(L_DEBUG,"*",Message);


   //Get the current Image attributes. They sould be equal to the defaults.
  if ((flDefAttrs=GreGetAttributes(hDC,PRIM_IMAGE,IMAGE_ATTRIBUTES,
                        &CurrentImageBundle))==GPI_ALTERROR){
     SHOWERROR("GreGetAttributes",EntryName);
  }

  ErrorCount+=!(fOutCome=VALIDATE_ATTRS(IMAGE_ATTRIBUTES,flDefAttrs));
  sprintf(Message,COMMON_SUFFICE1(Image),GOOD_FUNCTION(fOutCome));
  LOGINFO(L_DEBUG,"*",Message);


  //Get the current Marker attributes. They sould be equal to the defaults.
  if ((flDefAttrs=GreGetAttributes(hDC,PRIM_MARKER,MARKER_ATTRIBUTES,
                        &CurrentMarkerBundle))==GPI_ALTERROR){
     SHOWERROR("GreGetAttributes",EntryName);
  }

  ErrorCount+=!(fOutCome=VALIDATE_ATTRS(MARKER_ATTRIBUTES,flDefAttrs));
  sprintf(Message,COMMON_SUFFICE1(Marker),GOOD_FUNCTION(fOutCome));
  LOGINFO(L_DEBUG,"*",Message);

  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetGlobalAttribuesExh()
 *
 * Purpose:  This function will test the functionality of the Gre
 *           function GreSetGlobalAttribues()
 *
 * System Requirements:    None.
 *
 * Revision Log:
 *               10/11/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:     None.
 *
 * Subroutines Required:
 *
 *  SetNewAttibutes().
 *  CompareDefaultAttibutesSetting().
 *
 * Limitations:           None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *  DO the following for 10 time
 *    Randomly select colors,mixes from a finite set of attributes.
 *
 * 1. Change the foreground color attribute for all drawing primitives.
 *    Change the background color attribute for all drawing primitives.
 *    Change the background Mix attribute for all drawing primitives.
 *    Change the foreground Mix attribute for all drawing primitives.
 *
 *    Query current attributes for the Line primitive by calling the function
 *     GreGetAttributes(hDC,PRIM_LINE,..) and verify that the returned value matches
 *     our cuurent chosen attributes.
 *
 *    Query current attributes for the Area primitive by calling the function
 *     GreGetAttributes(hDC.PRIM_AREA,..) and verify that the returned value
 *     matches our cuurent chosen attributes.
 *
 *    Query current attributes for the Image primitive by calling the function
 *     GreGetAttributes(hDC.PRIM_IMAGE,..) and verify that the returned value
 *     matches our cuurent chosen attributes.
 *
 *    Query current attributes for the Marker primitive by calling the function
 *     GreGetAttributes(hDC.PRIM_MARKER,..) and verify that the returned value
 *     matches our cuurent chosen attributes.
 *
 *    Query current attributes for the Char primitive by calling the function
 *     GreGetAttributes(hDC.PRIM_CHAR,..) and verify that the returned value
 *     matches our cuurent chosen attributes.
 *
 * 2. Set all current attributes of all drawing primitives to their defaults
 *    by calling GreSetGlobalAttributes() for every primitive type. Next
 *    verify that all attributes are correctly set to their default by
 *    call  GreGetAttributes().
 *
 *   END //DO loop
 *
 *   return to DTT.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetGlobalAttributeExh(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetGlobalAttributeExh()";
  CHAR         Message[250];

  HDC          hDC;

  USHORT       usIndex,
               usColorIndex,
               usMixIndex,
               usBackMixIndex,
               usBackColorIndex;

  BOOL         fOutCome;

  LINEBUNDLE   CurrentLineBundle={0};
  CHARBUNDLE   CurrentCharBundle={0};
  MARKERBUNDLE CurrentMarkerBundle={0};
  IMAGEBUNDLE  CurrentImageBundle={0};
  AREABUNDLE   CurrentAreaBundle={0};


  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //The entry point does not have any output.
    DeletePrinterOutput(EntryName);

    srand(1);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }


    TEST_PROGRESS ( 00, 10 );

    SUBENTER(Setting Global Attributes for ALL Drawing Primitives,NULL);
    for (usIndex=0; usIndex < 10; usIndex++){

       (VOID)SetNewAttibutes(hPS,hWndClient,
                             EntryName,!GATTR_DEFAULT,
                             &usColorIndex,&usMixIndex,
                             &usBackMixIndex,&usBackColorIndex
                            );

       //Get the current Area attributes.
       if (GreGetAttributes(hDC,PRIM_AREA,AREA_ATTRIBUTES,
                            &CurrentAreaBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetAttributes",EntryName);
       }

       //Get the current Line attributes.
       if (GreGetAttributes(hDC,PRIM_LINE,LINE_ATTRIBUTES,
                            &CurrentLineBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetAttributes",EntryName);
       }

       //Get the current Character attributes.
       if (GreGetAttributes(hDC,PRIM_CHAR,CHAR_ATTRIBUTES,
                            &CurrentCharBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetAttributes",EntryName);
       }


       //Get the current Image attributes.
      if (GreGetAttributes(hDC,PRIM_IMAGE,IMAGE_ATTRIBUTES,
                            &CurrentImageBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetAttributes",EntryName);
      }

      //Get the current Marker attributes.
      if (GreGetAttributes(hDC,PRIM_MARKER,MARKER_ATTRIBUTES,
                            &CurrentMarkerBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetAttributes",EntryName);
      }


      fOutCome=(BOOL)(CurrentLineBundle.lColor==asColor[usColorIndex].lAttrValue &&
                CurrentLineBundle.usMixMode==(USHORT)asMix[usMixIndex].lAttrValue)
                ? TRUE : FALSE;

      sprintf(Message,COMMON_SUFFICE2(Line),GOOD_FUNCTION(fOutCome));
      LOGINFO(L_DEBUG,"*",Message);
      ErrorCount+=(!fOutCome);

      fOutCome=(BOOL)(CurrentCharBundle.lColor==asColor[usColorIndex].lAttrValue &&
                CurrentCharBundle.usMixMode==(USHORT)asMix[usMixIndex].lAttrValue &&
                CurrentCharBundle.usBackMixMode==(USHORT)asBackMix[usBackMixIndex].lAttrValue &&
                CurrentCharBundle.lBackColor==asColor[usBackColorIndex].lAttrValue)
                ? TRUE : FALSE;

      sprintf(Message,COMMON_SUFFICE2(Character),GOOD_FUNCTION(fOutCome));
      LOGINFO(L_DEBUG,"*",Message);
      ErrorCount+=(!fOutCome);

      fOutCome=(BOOL)(CurrentMarkerBundle.lColor==asColor[usColorIndex].lAttrValue &&
                CurrentMarkerBundle.usMixMode==(USHORT)asMix[usMixIndex].lAttrValue &&
                CurrentMarkerBundle.usBackMixMode==(USHORT)asBackMix[usBackMixIndex].lAttrValue &&
                CurrentMarkerBundle.lBackColor==asColor[usBackColorIndex].lAttrValue)
                ? TRUE : FALSE;

      sprintf(Message,COMMON_SUFFICE2(Marker),GOOD_FUNCTION(fOutCome));
      LOGINFO(L_DEBUG,"*",Message);
      ErrorCount+=(!fOutCome);


      fOutCome=(BOOL)(CurrentImageBundle.lColor==asColor[usColorIndex].lAttrValue &&
                CurrentImageBundle.usMixMode==(USHORT)asMix[usMixIndex].lAttrValue &&
                CurrentImageBundle.usBackMixMode==(USHORT)asBackMix[usBackMixIndex].lAttrValue &&
                CurrentImageBundle.lBackColor==asColor[usBackColorIndex].lAttrValue)
                ? TRUE : FALSE;

      sprintf(Message,COMMON_SUFFICE2(Image),GOOD_FUNCTION(fOutCome));
      LOGINFO(L_DEBUG,"*",Message);
      ErrorCount+=(!fOutCome);

      fOutCome=(BOOL)(CurrentAreaBundle.lColor==asColor[usColorIndex].lAttrValue &&
                CurrentAreaBundle.usMixMode==(USHORT)asMix[usMixIndex].lAttrValue &&
                CurrentAreaBundle.usBackMixMode==(USHORT)asBackMix[usBackMixIndex].lAttrValue &&
                CurrentAreaBundle.lBackColor==asColor[usBackColorIndex].lAttrValue)
                ? TRUE : FALSE;

      sprintf(Message,COMMON_SUFFICE2(Area),GOOD_FUNCTION(fOutCome));
      LOGINFO(L_DEBUG,"*",Message);
      ErrorCount+=(!fOutCome);


      LOGINFO(L_DEBUG,"*","> Setting Global Attributes of ALL Drawing"
                          " Primitives to their defaults");

      (VOID)SetNewAttibutes(hPS,hWndClient,
                            EntryName,GATTR_DEFAULT,
                            &usColorIndex,&usMixIndex,
                            &usBackMixIndex,&usBackColorIndex
                           );

      (VOID)CompareDefaultAttibutesSetting(hPS,hWndClient,EntryName);


      TEST_PROGRESS ( usIndex + 1, 10 );
    }

    SUBEXIT();

    EXIT_MAIN(EntryName);
  }

  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreInitializeAttributesExh1()
 *
 * Purpose:  This function will test the functionality of of the Gre
 *           function GreInitializeAttributes()
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/15/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *  DO the following for 10 times
 *    Randomly select colors,mixes from a finite set of attributes.
 *
 *    Change the foreground color attribute of all drawing primitives.
 *    Change the background color attribute of all drawing primitives.
 *    Change the background Mix attribute   of all drawing primitives.
 *    Change the foreground Mix attribute   of all drawing primitives.
 *
 *    Call the function GreIniatializeAttributes() to set the current
 *    attributes to their current defaults. Then call the Gre function
 *    GreGetAttributes(),for every drawing primitive, to verify that all
 *    attributes are set to their current defaults.
 *
 *
 *  Query current  attributes for the Line primitive by calling
 *    the function  GreGetAttributes(hDC,PRIM_LINE,..) and verify that
 *    the returned attributes matches the standard defaults.
 *
 *
 *  Query current attributes for the Area primitive by
 *    calling the function GreGetAttributes(hDC.PRIM_AREA,..) and verify that
 *    the returned attributes matches our cuurent chosen attributes.
 *
 *
 *  Query current attributes for the Image primitive by calling the function
 *    GreGetAttributes(hDC.PRIM_IMAGE,..) and verify that the returned value
 *    matches the standard default attributes.
 *
 *  Query current attributes for the Marker primitive by
 *    calling the function GreGetAttributes(hDC.PRIM_MARKER,..)
 *    and verify that the returned attributes matches their standard
 *    default attributes.
 *
 *  Query current attributes for the Char primitive by calling
 *    the function GreGetAttributes(hDC.PRIM_CHAR,..) and verify that the
 *    returned attributes matches their standard default attributes.
 *
 *   END //DO loop
 *
 *   return to DTT.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreInitializeAttributesExh1(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreInitializeAttributesExh1()";

  HDC          hDC;

  USHORT       usIndex         =0,
               usCount         =0,
               usColorIndex    =0,
               usMixIndex      =0,
               usBackMixIndex  =0,
               usBackColorIndex=0;

  LINEBUNDLE   CurrentLineBundle={0};
  CHARBUNDLE   CurrentCharBundle={0};
  MARKERBUNDLE CurrentMarkerBundle={0};
  IMAGEBUNDLE  CurrentImageBundle={0};
  AREABUNDLE   CurrentAreaBundle={0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //The entry point does not have any output.
    DeletePrinterOutput(EntryName);

    srand(1);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    SUBENTER(Setting All current attributes to their defaults.,NULL);
    for (usIndex=0; usIndex < 10; usIndex++){

       (VOID)SetNewAttibutes(hPS,hWndClient,
                             EntryName,!GATTR_DEFAULT,
                             &usColorIndex,&usMixIndex,
                             &usBackMixIndex,&usBackColorIndex
                            );


       if (GreInitializeAttributes(hDC,INAT_CURRENTATTRIBUTES)==FALSE){

         ErrorCount++;
         ERRSTATUS(TRUE,"Initializing attributes to their defaults failed.",EntryName);
       }

      (VOID)CompareDefaultAttibutesSetting(hPS,hWndClient,EntryName);

    } //End FOR loop.

    SUBEXIT();

    EXIT_MAIN(EntryName);
  }

  return;
}



/****************************************************************************
 *\\ddd
 * Routine Name: GreInitializeAttributesExh2()
 *
 * Purpose:  This function will test the functionality of the Gre
 *           function GreInitializeAttributes()
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/15/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *     None.
 * Subroutines Required:
 *    None
 *
 * Limitations:
 *    None at this time.
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *     Get the standard default attributes of all drawing primitives. These
 *   attribute values will be used to verify that the function
 *   GreInitializeAttributes() does indeed work properly when it is instructed
 *   to set default attributes to their standard defaults.
 *
 *
 *  DO the following for 10 times
 *    Randomly select colors,mixes from a finite set of attributes.
 *
 *    Change the default foreground color attribute of all drawing primitives.
 *    Change the default background color attribute of all drawing primitives.
 *    Change the default background Mix attribute   of all drawing primitives.
 *    Change the default foreground Mix attribute   of all drawing primitives.
 *
 *  Call the function GreIniatializeAttributes() to set the default
 *    attributes to their standard defaults. Then call the Gre function
 *    GreGetDefaultAttributes(),for every drawing primitive, to verify
 *    that all default attributes are set to their standard defaults.
 *
 *
 *  Query default attributes for the Line primitive by calling
 *    the function  GreGetdefaultAttributes(hDC,PRIM_LINE,..) and verify that
 *    the returned default attributes matches the standard defaults.
 *
 *
 *  Query default attributes for the Area primitive by
 *    calling the function GreGetdefaultAttributes(hDC.PRIM_AREA,..) and
 *    verify that default attributes matches the standard default attributes.
 *
 *
 *
 *  Query default attributes for the Image primitive by calling the function
 *    GreGetdefaultAttributes(hDC.PRIM_IMAGE,..) and verify that the returned value
 *    matches the standard default attributes.
 *
 *  Query default attributes for the Marker primitive by
 *    calling the function GreGetdefaultAttributes(hDC.PRIM_MARKER,..)
 *    and verify that the returned default attributes matches their standard
 *    default attributes.
 *
 *  Query default attributes for the Char primitive by calling
 *    the function GreGetdefaultAttributes(hDC.PRIM_CHAR,..) and verify
 *    that the returned default attributes matches their standard
 *    default attributes.
 *
 *   END //DO loop
 *
 *   return to DTT.
 *
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreInitializeAttributesExh2(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreInitializeAttributesExh2()";
  CHAR         Message[250];

  HDC          hDC;

  USHORT       usIndex         =0,
               usColorIndex    =0,
               usMixIndex      =0,
               usBackMixIndex  =0,
               usBackColorIndex=0;

  BOOL         fOutCome;

  LINEBUNDLE   CurrentLineBundle={0},
               DefaultLineBundle={0};

  CHARBUNDLE   CurrentCharBundle={0},
               DefaultCharBundle={0};

  MARKERBUNDLE CurrentMarkerBundle={0},
               DefaultMarkerBundle={0};

  IMAGEBUNDLE  CurrentImageBundle={0},
               DefaultImageBundle={0};

  AREABUNDLE   CurrentAreaBundle={0},
               DefaultAreaBundle={0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //The entry point does not have any output.
    DeletePrinterOutput(EntryName);

    srand(1);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    /*********************************************************************
    * This next code segment is used to query standard default attributes
    * of all drawing primitives. These values will be used to verify that
    * the Gre function under testing does indeed sets default attributes
    *  to their standard defaults.
    *********************************************************************/

    SETENTER("Getting all standard default attributes.",NULL);

    //Get the standard default Area attributes.
    if (GreGetDefaultAttributes(hDC,PRIM_AREA,AREA_ATTRIBUTES,
                            &DefaultAreaBundle)==FALSE){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
    }

    //Get the standard default Line attributes.
    if (GreGetDefaultAttributes(hDC,PRIM_LINE,LINE_ATTRIBUTES,
                            &DefaultLineBundle)==FALSE){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
    }

    //Get the standard default Marker attributes.
    if (GreGetDefaultAttributes(hDC,PRIM_MARKER,MARKER_ATTRIBUTES,
                            &DefaultMarkerBundle)==FALSE){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
    }

    //Get the standard default Image attributes.
    if (GreGetDefaultAttributes(hDC,PRIM_IMAGE,IMAGE_ATTRIBUTES,
                            &DefaultImageBundle)==FALSE){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
    }

    //Get the standard default Image attributes.
    if (GreGetDefaultAttributes(hDC,PRIM_CHAR,CHAR_ATTRIBUTES,
                            &DefaultCharBundle)==FALSE){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
    }

    SETEXIT();

    SUBENTER(Setting Attributes,NULL);

    for (usIndex=0; usIndex < 10; usIndex++){

       //Randomize next attribute settings.
       usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       CurrentLineBundle.lColor=
       CurrentMarkerBundle.lColor=
       CurrentCharBundle.lColor=
       CurrentAreaBundle.lColor=
       CurrentImageBundle.lColor=asColor[usColorIndex].lAttrValue;


       usBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       CurrentMarkerBundle.lBackColor=
       CurrentCharBundle.lBackColor=
       CurrentAreaBundle.lBackColor=
       CurrentImageBundle.lBackColor=asColor[usBackColorIndex].lAttrValue;


       usMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
       CurrentLineBundle.usMixMode=
       CurrentMarkerBundle.usMixMode=
       CurrentCharBundle.usMixMode=
       CurrentAreaBundle.usMixMode=
       CurrentImageBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;


       usBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);
       CurrentMarkerBundle.usBackMixMode=
       CurrentCharBundle.usBackMixMode=
       CurrentAreaBundle.usBackMixMode=
       CurrentImageBundle.usBackMixMode=(USHORT)asBackMix[usBackMixIndex].lAttrValue;


       sprintf(Message,"Setting Primitive Attributes to Color= %s   BackColor=%s\n"
                       " Mix=%s   BackMix= %s",
                        asColor[usColorIndex].AttrSymbolicName,
                        asColor[usBackColorIndex].AttrSymbolicName,
                        asMix[usMixIndex].AttrSymbolicName,
                        asBackMix[usBackMixIndex].AttrSymbolicName);

       LOGINFO(L_DEBUG,"*",Message);

       //Set the standard default Area attributes.
       if (GreSetDefaultAttributes(hDC,PRIM_AREA,AREA_ATTRIBUTES,
                               &CurrentAreaBundle)==FALSE){
            SHOWERROR("GreSetDefaultAttributes",EntryName);
       }

       //Set the standard default Line attributes.
       if (GreSetDefaultAttributes(hDC,PRIM_LINE,LINE_ATTRIBUTES,
                               &CurrentLineBundle)==FALSE){
            SHOWERROR("GreSetDefaultAttributes",EntryName);
       }

       //Set the standard default Marker attributes.
       if (GreSetDefaultAttributes(hDC,PRIM_MARKER,MARKER_ATTRIBUTES,
                               &CurrentMarkerBundle)==FALSE){
            SHOWERROR("GreSetDefaultAttributes",EntryName);
       }

       //Set the standard default Image attributes.
       if (GreSetDefaultAttributes(hDC,PRIM_IMAGE,IMAGE_ATTRIBUTES,
                               &CurrentImageBundle)==FALSE){
            SHOWERROR("GreSetDefaultAttributes",EntryName);
       }

       //Set the standard default Image attributes.
       if (GreSetDefaultAttributes(hDC,PRIM_CHAR,CHAR_ATTRIBUTES,
                               &CurrentCharBundle)==FALSE){
            SHOWERROR("GreSetDefaultAttributes",EntryName);
       }


       if (GreInitializeAttributes(hDC,INAT_DEFAULTATTRIBUTES)==FALSE){

         ErrorCount++;
         ERRSTATUS(TRUE,"Initializing attributes to their Standard defaults failed.",EntryName);
       }

       //Get the default Area attributes.They sould be equal
       // their standard defaults.
       if (GreGetDefaultAttributes(hDC,PRIM_AREA,AREA_ATTRIBUTES,
                            &CurrentAreaBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
       }
       ErrorCount+=!(
             fOutCome=memicmp(&CurrentAreaBundle,&DefaultAreaBundle,
                               sizeof (CurrentAreaBundle))==0 ? TRUE :FALSE);
       sprintf(Message,COMMON_SUFFICE3(Area),GOOD_FUNCTION(fOutCome));
       LOGINFO(L_DEBUG,"*",Message);


       //Get the default Line attributes.
       if (GreGetDefaultAttributes(hDC,PRIM_LINE,LINE_ATTRIBUTES,
                            &CurrentLineBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
       }
       ErrorCount+=!(
             fOutCome=memicmp(&CurrentLineBundle,&DefaultLineBundle,
                               sizeof (CurrentLineBundle))==0 ? TRUE :FALSE);
       sprintf(Message,COMMON_SUFFICE3(Line),GOOD_FUNCTION(fOutCome));
       LOGINFO(L_DEBUG,"*",Message);


       //Get the default Character attributes.They sould be equal
       // their standard defaults.
       if (GreGetDefaultAttributes(hDC,PRIM_CHAR,CHAR_ATTRIBUTES,
                            &CurrentCharBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
       }
       ErrorCount+=!(
             fOutCome=memicmp(&CurrentCharBundle,&DefaultCharBundle,
                               sizeof (CurrentCharBundle))==0 ? TRUE :FALSE);
       sprintf(Message,COMMON_SUFFICE3(Char),GOOD_FUNCTION(fOutCome));
       LOGINFO(L_DEBUG,"*",Message);



       //Get the default Image attributes.They sould be equal
       // their standard defaults.
      if (GreGetDefaultAttributes(hDC,PRIM_IMAGE,IMAGE_ATTRIBUTES,
                            &CurrentImageBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
      }
      ErrorCount+=!(
             fOutCome=memicmp(&CurrentImageBundle,&DefaultImageBundle,
                               sizeof (CurrentImageBundle))==0 ? TRUE :FALSE);
       sprintf(Message,COMMON_SUFFICE3(Image),GOOD_FUNCTION(fOutCome));
       LOGINFO(L_DEBUG,"*",Message);


       //Get the default Marker attributes.They sould be equal
       // their standard defaults.
      if (GreGetDefaultAttributes(hDC,PRIM_MARKER,MARKER_ATTRIBUTES,
                            &CurrentMarkerBundle)==GPI_ALTERROR){
         SHOWERROR("GreGetDefaultAttributes",EntryName);
      }
      ErrorCount+=!(
             fOutCome=memicmp(&CurrentMarkerBundle,&DefaultMarkerBundle,
                               sizeof (CurrentMarkerBundle))==0 ? TRUE :FALSE);
       sprintf(Message,COMMON_SUFFICE3(Marker),GOOD_FUNCTION(fOutCome));
       LOGINFO(L_DEBUG,"*",Message);



    } //End FOR loop.

    SUBEXIT();

    EXIT_MAIN(EntryName);
  }

  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetAttributesExh1()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetAttributes().
 *            GreGetAttributes().
 *
 *  Please note that this function only test setting/getting attributes
 *  for Markers.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/16/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to draw a finite number of marker utilizing the entire
 *   current presentation space. every 20 markers will have different set
 *   of attributes that have been randomly selected.
 *
 *    FOR (Do the following at least 50 time) DO
 *
 *     Randomly select colors, mixes, and marker symbols from a finite
 *     set of attributes. Then set the Marker's attributes to the
 *     newly selected attributes.
 *       GreSetAttributes(hDC,PRIM_MARKER,...);
 *
 *     Next Query the marker's current attributes by calling the
 *     Gre function GreGetAttributes() to verify that the
 *     current attributes are set correctly.
 *
 *       GreGetAttributes(hDC,PRIM_MARKER,....)
 *
 *      DO (draw 20 markers At Randomly generated coordinates )
 *
 *         Generate the X coordinate of Marker in question
 *         Generate the Y cordinate of  Marker  in question
 *
 *         Draw the Marker .
 *           GreMarker(hPS,...);
 *
 *      ENDDO
 *
 *      Use the function GreSetAttributes() to set the marker's current
 *      attributes to their current default attributes (this is one of
 *      the options supplied to function in question).
 *
 *       GreSetAttributes(hDC,PRIM_MARKER,...);
 *
 *     Next Query the marker's current attributes by calling the
 *     Gre function GreGetAttributes() to verify that the
 *     current attributes are set correctly to their current defaults.
 *
 *       GreGetAttributes(hDC,PRIM_MARKER,....)
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetAttributesExh1(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetAttributesExh1()";
  CHAR         Message[250];

  HDC          hDC;
  POINTL       ptl={0};

  LONG         flDefAttrs;

  USHORT       usIndex,
               usIndex1,
               usColorIndex,
               usMixIndex,
               usBackMixIndex,
               usSymbolIndex,
               usBackColorIndex;

  BOOL         fOutCome;

  MARKERBUNDLE QueriedMarkerBundle={0},
               CurrentMarkerBundle={0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //Select a random seed.
       srand(0);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }


    TEST_PROGRESS ( 00, 50 );

    SUBENTER(Setting New Marker attributes,NULL);
    for (usIndex=0; usIndex < 50; usIndex++){

       usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       usBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       usMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
       usBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);
       usSymbolIndex=GETRANDOM(0,NUMBEROFELEMENTS(asSymbol)-1);

       CurrentMarkerBundle.lColor=asColor[usColorIndex].lAttrValue;
       CurrentMarkerBundle.lBackColor=asColor[usBackColorIndex].lAttrValue;
       CurrentMarkerBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
       CurrentMarkerBundle.usBackMixMode=(USHORT)asBackMix[usBackMixIndex].lAttrValue;
       CurrentMarkerBundle.usSymbol=(USHORT)asSymbol[usSymbolIndex].lAttrValue;


       sprintf(Message,"Setting Marker's Attributes to Color= %s   BackColor=%s\n"
                    " Mix=%s  BackMix= %s  Symbol=%s.",
                     asColor[usColorIndex].AttrSymbolicName,
                     asColor[usBackColorIndex].AttrSymbolicName,
                     asMix[usMixIndex].AttrSymbolicName,
                     asBackMix[usBackMixIndex].AttrSymbolicName,
                     asSymbol[usSymbolIndex].AttrSymbolicName
                     );

       LOGINFO(L_DEBUG,"*",Message);

       //Sett marker's current attributes to new values.
       if (GreSetAttributes(hDC,PRIM_MARKER,(LONG)NULL,CHANGE_MARKER,
                             &CurrentMarkerBundle)==FALSE){
           ErrorCount++;
           ERRSTATUS(TRUE,"Setting Marker Attributes.",EntryName);

       }

         //Get marker's current attributes.
       if ((flDefAttrs=GreGetAttributes(hDC,PRIM_MARKER,CHANGE_MARKER,
                             &QueriedMarkerBundle))==GPI_ALTERROR){
          ErrorCount++;
          ERRSTATUS(TRUE,"Getting Marker Attributes.",EntryName);

       }

       ErrorCount+=!(
             fOutCome=memicmp(&CurrentMarkerBundle,&QueriedMarkerBundle,
                                sizeof (CurrentMarkerBundle))==0 ? TRUE :FALSE);
       sprintf(Message,COMMON_SUFFICE2(Marker),GOOD_FUNCTION(fOutCome));
       LOGINFO(L_DEBUG,"*",Message);



       // Now display the marker with the selected attributes for
       // 20 times. Each time the marker will be placed at a randomly
       // selected coordinates.

        for (usIndex1=0 ; usIndex1 < 25 ; usIndex1++) {
          //Select a random coordinates.
          ptl.x=GETRANDOM(0,pg_size_x*delta_x);
          ptl.y=GETRANDOM((USHORT)(.67*delta_y),pg_size_y*delta_y);
          MapPt(ptl);

          if (GpiMarker(hPS,&ptl)==FALSE){
            SHOWERROR(EntryName,"GpiMarker()");
          }

        } //End FOR Loop

        //Now try to set attributes to their defaults by having
        // flAttrs and flDedAttrs options to be equal. This will
        // instruct GreSetAttributes() to set attributes to their default
        // values.

        LOGINFO(L_DEBUG,"*","> Setting Marker's Current Attributes "
                          " to their default.");

        if (GreSetAttributes(hDC,PRIM_MARKER,CHANGE_MARKER,CHANGE_MARKER,
                             &CurrentMarkerBundle)==FALSE){
           ErrorCount++;
           ERRSTATUS(TRUE,"Setting Marker to default Attributes.",EntryName);

        }
        if ((flDefAttrs=GreGetAttributes(hDC,PRIM_MARKER,CHANGE_MARKER,
                             &QueriedMarkerBundle))==GPI_ALTERROR){
          ErrorCount++;
          ERRSTATUS(TRUE,"Getting Marker Attributes.",EntryName);

        }

        ErrorCount+=!(fOutCome=VALIDATE_ATTRS(CHANGE_MARKER,flDefAttrs));
        sprintf(Message,COMMON_SUFFICE1(Marker),GOOD_FUNCTION(fOutCome));
        LOGINFO(L_DEBUG,"*",Message);


        TEST_PROGRESS ( usIndex + 1, 50 );
    }

    SUBEXIT();

    EXIT_MAIN(EntryName);
  }

  return;
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetDefaultAttrsExh1()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetDefaultAttributes().
 *            GreGetDefaultAttributes().
 *
 *  Please note that this function only test setting and getting default
 *  marker's attributes
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/17/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to draw a spiral utilizing the entire
 *   current presentation space. The Spiral will be drawn with markers
 *   that have randomly selected attributes.
 *
 *    FOR (pointno=0, pointno <100...) DO
 *
 *     Randomly select colors, mixes, and marker symbols from a finite
 *     set of attributes. Then set the Marker's default attributes to the
 *     newly selected attributes.
 *       GreSetDefaultAttributes(hDC,PRIM_MARKER,...);
 *
 *     Next Query the marker's default attributes by calling the
 *     Gre function GreGetDefaultAttributes() to verify that the
 *     default attributes are set correctly.
 *
 *       GreGetDefaultAttributes(hDC,PRIM_MARKER,....)
 *
 *      DO (draw 20 markers along the spiral path )
 *
 *         Generate the X coordinate of Marker in question
 *         Generate the Y cordinate of  Marker  in question
 *
 *         Draw the Marker .
 *           GreMarker(hPS,...);
 *
 *      ENDDO
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetDefaultAttrsExh1(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetDefaultAttrsExh1()";
  CHAR         Message[250];

  HDC          hDC;
  POINTL       ptl={0};

  USHORT       usPointNumber,
               usSigntureBoundary,
               usIndex,
               usIndex1,
               usColorIndex,
               usMixIndex,
               usBackMixIndex,
               usSymbolIndex,
               usBackColorIndex;

  BOOL         fOutCome;

  double       dangle,
               dscale,
               WidthSpan,
               HeightSpan;

  MARKERBUNDLE QueriedMarkerBundle={0},
               CurrentMarkerBundle={0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //Select the random Generator's initial seed.
    srand(0);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

   //The next few calculations will make drawing the
   //Spiral path much quicker.

   WidthSpan=(double)((double)pg_size_x*delta_x)/2.0;
   HeightSpan=(double)((double)pg_size_y*delta_y)/2.0;
   usSigntureBoundary=(USHORT)MapY(.65*delta_y);


   TEST_PROGRESS ( 00, 100 );

   SUBENTER(Setting New Marker Default attributes,NULL);
   for (usIndex=0, usPointNumber=0 ; usIndex < 100 ; usIndex++){

       usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       usBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       usMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
       usBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);
       usSymbolIndex=GETRANDOM(0,NUMBEROFELEMENTS(asSymbol)-1);

       CurrentMarkerBundle.lColor=asColor[usColorIndex].lAttrValue;
       CurrentMarkerBundle.lBackColor=asColor[usBackColorIndex].lAttrValue;
       CurrentMarkerBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
       CurrentMarkerBundle.usBackMixMode=(USHORT)asBackMix[usBackMixIndex].lAttrValue;
       CurrentMarkerBundle.usSymbol=(USHORT)asSymbol[usSymbolIndex].lAttrValue;


       sprintf(Message,"Setting Marker's default Attributes to Color= %s   BackColor=%s\n"
                    " Mix=%s  BackMix= %s  Symbol=%s.",
                     asColor[usColorIndex].AttrSymbolicName,
                     asColor[usBackColorIndex].AttrSymbolicName,
                     asMix[usMixIndex].AttrSymbolicName,
                     asBackMix[usBackMixIndex].AttrSymbolicName,
                     asSymbol[usSymbolIndex].AttrSymbolicName
                     );

       LOGINFO(L_DEBUG,"*",Message);

       //Set marker's default  attributes to new chosen attributes.
       if (GreSetDefaultAttributes(hDC,PRIM_MARKER,CHANGE_MARKER,
                             &CurrentMarkerBundle)==FALSE){
           ErrorCount++;
           ERRSTATUS(TRUE,"Setting Marker's Default Attributes.",EntryName);

       }

        //Get marker's current attributes and verify these attributes are
       // set to the current chosen defaults.
       if (GreGetDefaultAttributes(hDC,PRIM_MARKER,CHANGE_MARKER,
                             &QueriedMarkerBundle)==FALSE){
          ErrorCount++;
          ERRSTATUS(TRUE,"Getting Marker's Default Attributes.",EntryName);

       }

       ErrorCount+=!(
             fOutCome=memicmp(&CurrentMarkerBundle,&QueriedMarkerBundle,
                                sizeof (CurrentMarkerBundle))==0 ? TRUE :FALSE);
       sprintf(Message,COMMON_SUFFICE3(Marker),GOOD_FUNCTION(fOutCome));
       LOGINFO(L_DEBUG,"*",Message);


       // Now Plot 20 markers with the selected default attributes
       // Along the spiral path.

        for (usIndex1=0 ; usIndex1 < 20 ; usIndex1++ , usPointNumber++) {
          //Get the coordinates of the next point to plot.

          dangle=(double)usPointNumber*SIMPLIES_FACTOR;
          dscale=1.0-(double)usPointNumber/(double)2000.0;
          ptl.x=(LONG)(WidthSpan*(1.0+dscale*cos(dangle)));
          ptl.y=(LONG)(HeightSpan*(1.0+dscale*sin(dangle)));

          MapPt(ptl);

          //Make sure we do not overwrite test signiture.
          if (ptl.y > (LONG) usSigntureBoundary){
            if (GpiMarker(hPS,&ptl)==FALSE){
              SHOWERROR(EntryName,"GpiMarker()");
            }
          }
        } //End FOR Loop

        TEST_PROGRESS ( usIndex + 1, 100 );
    }

    SUBEXIT();

    EXIT_MAIN(EntryName);
  }

  return;
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetAttributesExh2()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSettAttributes().
 *            GreGetAttributes().
 *
 *  Please note that this function only test setting/getting attributes
 *  for Areas.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/17/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to draw a finite set  of 1" circles and
 *   1" X 1" boxes within an area brackets. Each Area will have a
 *   set of attributes that have been randomly selected.
 *
 *
 *    DO (Fill the Device display height with 1" X 1" boxes)
 *
 *        DO (Fill the Device display width with 1" X 1" boxes)
 *
 *           Randomly select colors, mixes, and Area patterns from a finite
 *           set of attributes. Then set the Area's attributes to the
 *           newly selected attributes.
 *             GreSetAttributes(hDC,PRIM_AREA,...);
 *
 *           Next Query the marker's current attributes by calling the
 *           Gre function GreGetAttributes() to verify that the
 *           current attributes are set correctly.
 *
 *             GreGetAttributes(hDC,PRIM_AREA,....)
 *
 *           Open an Area bracket.
 *              GpiBeginArea(hPS,...)
 *
 *           Draw a 1" X 1" box .
 *           Draw a 1" circle. This circle will be containes with the
 *           prevouisly drawn box.
 *
 *
 *            Close the Area .
 *              GpiEndArea(hPS,...);
 *
 *            Use the function GreSetAttributes() to set the Area's current
 *            attributes to their current default attributes (this is one of
 *            the options supplied to function in question).
 *
 *             GreSetAttributes(hDC,PRIM_AREA,...);
 *
 *           Next Query the Area's current attributes by calling the
 *           Gre function GreGetAttributes() to verify that the
 *           current attributes are set correctly to their current defaults.
 *
 *             GreGetAttributes(hDC,PRIM_AREA,....)
 *
 *
 *        ENDDO.
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetAttributesExh2(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetAttributesExh2()";
  CHAR         Message[250];

  HDC          hDC;
  POINTL       ptl1={0},
               ptl2={0};

  LONG         flDefAttrs;

  USHORT       usIndex,
               usIndex1,
               usColorIndex,
               usMixIndex,
               usBackMixIndex,
               usPatternIndex,
               usBackColorIndex;

  BOOL         fOutCome;

  AREABUNDLE QueriedAreaBundle={0},
               CurrentAreaBundle={0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //Select a random seed.
       srand(0);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }


    usIndex=(USHORT)(.67*delta_y); //Avoid the test sigenture printed
    SUBENTER(Setting New Area attributes,NULL);

    for (; usIndex < (USHORT)(pg_size_y*delta_y); usIndex+=delta_y){

       for (usIndex1=0; usIndex1< (USHORT)(pg_size_x*delta_x);
                                                 usIndex1+=delta_x){

           usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
           usBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
           usMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
           usBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);
           usPatternIndex=GETRANDOM(0,NUMBEROFELEMENTS(asPattern)-1);

           CurrentAreaBundle.lColor=asColor[usColorIndex].lAttrValue;
           CurrentAreaBundle.lBackColor=asColor[usBackColorIndex].lAttrValue;
           CurrentAreaBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
           CurrentAreaBundle.usBackMixMode=(USHORT)asBackMix[usBackMixIndex].lAttrValue;
           CurrentAreaBundle.usSymbol=(USHORT)asPattern[usPatternIndex].lAttrValue;


           sprintf(Message,"Setting Area's Attributes to Color= %s   BackColor=%s\n"
                           " Mix=%s   BackMix= %s  Symbol=%s.",
                         asColor[usColorIndex].AttrSymbolicName,
                         asColor[usBackColorIndex].AttrSymbolicName,
                         asMix[usMixIndex].AttrSymbolicName,
                         asBackMix[usBackMixIndex].AttrSymbolicName,
                         asPattern[usPatternIndex].AttrSymbolicName
                         );

           LOGINFO(L_DEBUG,"*",Message);

           //Set area's current attributes to new values.
           if (GreSetAttributes(hDC,PRIM_AREA,(LONG)NULL,CHANGE_AREA,
                             &CurrentAreaBundle)==FALSE){
              ErrorCount++;
              ERRSTATUS(TRUE,"Setting Area Attributes.",EntryName);
           }

         //Get Area's current attributes.
         if ((flDefAttrs=GreGetAttributes(hDC,PRIM_AREA,CHANGE_AREA,
                               &QueriedAreaBundle))==GPI_ALTERROR){
            ErrorCount++;
            ERRSTATUS(TRUE,"Getting Area Attributes.",EntryName);

         }

         ErrorCount+=!(
               fOutCome=memicmp(&CurrentAreaBundle,&QueriedAreaBundle,
                               sizeof (CurrentAreaBundle))==0 ? TRUE :FALSE);
         sprintf(Message,COMMON_SUFFICE2(Area),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);


         //This next code segment will open an area and draw a box
         //circle inside of it. Then area will be closed and will
         // take on the current attribute setting.
         if (GpiBeginArea(hPS,BA_ALTERNATE)==FALSE){
            SHOWERROR("GpiBeginArea()",EntryName);
         }

         //Determine the box's first corner and advance the
         //cursor.
         ptl1.x=MapX(usIndex1);
         ptl1.y=MapY(usIndex);

         if (GpiMove(hPS,&ptl1)==GPI_ERROR){
           SHOWERROR("GpiMove()",EntryName);
         }

         //Determine the box's second corner and draw a  1" X 1" box.
         ptl2.x=MapX(usIndex1+delta_x);
         ptl2.y=MapY(usIndex+delta_y);

         if (GpiBox(hPS,DRO_OUTLINE,&ptl2,(LONG)NULL,(LONG)NULL)==GPI_ERROR){
           SHOWERROR("GpiBox()",EntryName);
         }


         //Determine the second box's first corner and advance the
         //cursor.
         if (GpiMove(hPS,&ptl1)==GPI_ERROR){
           SHOWERROR("GpiMove()",EntryName);
         }

         //Determine the box's second corner and draw a  1"  circle.
         if (GpiBox(hPS,DRO_OUTLINE,&ptl2,ptl2.x,ptl2.y)==GPI_ERROR){
           SHOWERROR("GpiBox()",EntryName);
         }

         if (GpiEndArea(hPS)==GPI_ERROR){
             SHOWERROR("GpiEndArea()",EntryName);
         }

         //Now try to set attributes to their defaults by having
         // flAttrs and flDedAttrs options to be equal. This will
         // instruct GreSetAttributes() to set attributes to their default
         // values.


          LOGINFO(L_DEBUG,"*","> Setting Areas's Current Attributes "
                          " to their default.");

         if (GreSetAttributes(hDC,PRIM_AREA,CHANGE_AREA,CHANGE_AREA,
                              &CurrentAreaBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Area to default Attributes.",EntryName);

         }
         if ((flDefAttrs=GreGetAttributes(hDC,PRIM_AREA,CHANGE_AREA,
                              &QueriedAreaBundle))==GPI_ALTERROR){
           ErrorCount++;
           ERRSTATUS(TRUE,"Getting Area Attributes.",EntryName);

         }

         ErrorCount+=!(fOutCome=VALIDATE_ATTRS(CHANGE_AREA,flDefAttrs));
         sprintf(Message,COMMON_SUFFICE1(Area),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);

       } //End FOR Loop
    } //End FOR loop.

    SUBEXIT();
    EXIT_MAIN(EntryName);
  }

  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetDefaultAttrsExh2()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetDefaultAttributes().
 *            GreGetDefaultAttributes().
 *
 *  Please note that this function only test setting/getting default
 *  attributes for Areas.
 *
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/17/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup conditions to draw a finite number  of 1" circles
 *   that are placed randomly across the entire Presenation Space.
 *   Every circle is drawn with an Area brackets and will have (Circle)
 *   a randomly selected attributes.
 *
 *    DO (Fill the display device  with 1" circles)
 *
 *       Randomly select colors, mixes, and fill patterns from a finite
 *       set of attributes. Then set the default Area attributes to the
 *       newly selected attributes.
 *         GreSetDefaultAttributes(hDC,PRIM_AREA,...);
 *
 *       Next Query the Area current Default attributes by calling the
 *       Gre function GreGetDefaultAttributes() to verify that the
 *       current default attributes are set correctly.
 *
 *         GreGetDefaultAttributes(hDC,PRIM_AREA,....)
 *
 *       Open an Area bracket.
 *          GpiBeginArea(hPS,...)
 *
 *       Generate the X coordinate of Circle in question
 *       Generate the Y cordinate of   Circle in question
 *
 *        Draw  1" circle with the randomly generated coordinates.
 *         GpiBox(hPS,....);
 *
 *        Close the Area .
 *          GpiEndArea(hPS,...);
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetDefaultAttrsExh2(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetDefaultAttrsExh2()";
  CHAR         Message[250];

  HDC          hDC;
  POINTL       ptl={0};

  USHORT       usIndex,
               usColorIndex,
               usMixIndex,
               usBackMixIndex,
               usPatternIndex,
               usBackColorIndex;

  BOOL         fOutCome;

  AREABUNDLE QueriedAreaBundle={0},
             CurrentAreaBundle={0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //Select a random seed.
     srand(0x1234);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }


    TEST_PROGRESS ( 00, 150 );

    SUBENTER(Setting New Area attributes,NULL);

    for (usIndex=0; usIndex < 150 ; usIndex++){

      usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
      usBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
      usMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
      usBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);
      usPatternIndex=GETRANDOM(0,NUMBEROFELEMENTS(asPattern)-1);

      CurrentAreaBundle.lColor=asColor[usColorIndex].lAttrValue;
      CurrentAreaBundle.lBackColor=asColor[usBackColorIndex].lAttrValue;
      CurrentAreaBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
      CurrentAreaBundle.usBackMixMode=(USHORT)asBackMix[usBackMixIndex].lAttrValue;
      CurrentAreaBundle.usSymbol=(USHORT)asPattern[usPatternIndex].lAttrValue;


      sprintf(Message,"Setting Area's Attributes to Color= %s   BackColor=%s\n"
                      " Mix=%s   BackMix= %s  Symbol=%s.",
                       asColor[usColorIndex].AttrSymbolicName,
                       asColor[usBackColorIndex].AttrSymbolicName,
                       asMix[usMixIndex].AttrSymbolicName,
                       asBackMix[usBackMixIndex].AttrSymbolicName,
                      asPattern[usPatternIndex].AttrSymbolicName
                    );

      LOGINFO(L_DEBUG,"*",Message);

      //Set area's current attributes to new values.
      if (GreSetDefaultAttributes(hDC,PRIM_AREA,CHANGE_AREA,
                           &CurrentAreaBundle)==FALSE){
         ErrorCount++;
         ERRSTATUS(TRUE,"Setting default Area Attributes.",EntryName);
      }

      //Get Area's current default attributes.
      if (GreGetDefaultAttributes(hDC,PRIM_AREA,CHANGE_AREA,
                            &QueriedAreaBundle)==GPI_ALTERROR){
         ErrorCount++;
         ERRSTATUS(TRUE,"Getting Default Area Attributes.",EntryName);

      }

      ErrorCount+=!(
            fOutCome=memicmp(&CurrentAreaBundle,&QueriedAreaBundle,
                            sizeof (CurrentAreaBundle))==0 ? TRUE :FALSE);
      sprintf(Message,COMMON_SUFFICE3(Area),GOOD_FUNCTION(fOutCome));
      LOGINFO(L_DEBUG,"*",Message);


      //This next code segment will open an area and draw a
      //circle inside of it. Then area will be closed and will
      // take on the current attribute setting.
      if (GpiBeginArea(hPS,BA_ALTERNATE)==FALSE){
         SHOWERROR("GpiBeginArea()",EntryName);
      }

      //Determine the box's first corner and advance the
      //cursor.

       ptl.x=MapX(GETRANDOM(0,pg_size_x*delta_x));
       ptl.y=MapY(GETRANDOM((USHORT)(.67*delta_y),pg_size_y*delta_y));

      //Determine the box's first corner and advance the
      //cursor.
      if (GpiMove(hPS,&ptl)==GPI_ERROR){
        SHOWERROR("GpiMove()",EntryName);
      }

      //Determine the box's second corner and draw a 1" circle.
       ptl.x+=MapX(delta_x);
       ptl.y+=MapY(delta_y);

      if (GpiBox(hPS,DRO_OUTLINE,&ptl,ptl.x,ptl.y)==GPI_ERROR){
        SHOWERROR("GpiBox()",EntryName);
      }

      if (GpiEndArea(hPS)==GPI_ERROR){
          SHOWERROR("GpiEndArea()",EntryName);
      }

      TEST_PROGRESS ( usIndex + 1, 150 );
    }

    SUBEXIT();
    EXIT_MAIN(EntryName);
  }

  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetAttributesExh3()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetAttributes().
 *            GreGetAttributes().
 *
 *  Please note that this function only test setting/getting attributes
 *  for Images.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/18/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:
 *   No bitmaps are printed when using IBM4019. This is because the printer
 *   does not map colors and mixes correctly.
 *
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to draw a finite set of bitmaps with randomly
 *   selected set of attributes. The bitmaps are displayed and streched
 *   to fill 1" X 1" square box using GpiWCBitBlt().
 *
 *
 *    DO (Fill the Device display height with 1" X 1" bitmaps)
 *
 *        DO (Fill the Device display width with 1" X 1" bitmaps)
 *
 *           Randomly select colors, and mixes  from a finite
 *           set of attributes. Then set the Image's attributes to the
 *           newly selected attributes.
 *             GreSetAttributes(hDC,PRIM_IMAGE,...);
 *
 *           Next Query the Image's current attributes by calling the
 *           Gre function GreGetAttributes() to verify that the
 *           current attributes are set correctly.
 *
 *             GreGetAttributes(hDC,PRIM_IMAGE,....)
 *
 *           Create and initialize arandomly selected bitmap.Then display
 *           the bitmap filling 1" X 1" square box.
 *             GreCreateBitMap(hDC,....)
 *             GpiWCBitblt(hDS,....)
 *
 *            Use the function GreSetAttributes() to set the Image's current
 *            attributes to their current default attributes (this is one of
 *            the options supplied to function in question).
 *
 *             GreSetAttributes(hDC,PRIM_IMAGE,...);
 *
 *           Next Query the Image's current attributes by calling the
 *           Gre function GreGetAttributes() to verify that the
 *           current attributes are set correctly to their current defaults.
 *
 *             GreGetAttributes(hDC,PRIM_IMAGE,....)
 *
 *        ENDDO.
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetAttributesExh3(HPS hPS, HWND hWndClient, BOOL SelectionCall){

   static MyBitMap  aBitMap[]={
                               {CROSS_HEIGHT,CROSS_WIDTH,abCross},
                               {SHAPE_HEIGHT,SHAPE_WIDTH,abShape},
                               {CAT_HEIGHT,CAT_WIDTH,abCat},
                               {PAT1_HEIGHT,PAT1_WIDTH,abPat1},
                               {PAT2_HEIGHT,PAT2_WIDTH,abPat2},
                               {BRICK_HEIGHT,BRICK_WIDTH,abBrick},
                               {K_SHAPE_HEIGHT,K_SHAPE_WIDTH,abKShape},
                               {DIAMOND_HEIGHT,DIAMOND_WIDTH,abDiamond},
                               };


  static       PCHAR EntryName="GreSetAndGetAttributesExh3()";
  CHAR         Message[250];
  BYTE         Temp_buffer[200];

  HBITMAP      hBM;
  HDC          hDC;
  POINTL       aPoints[4]={0};

  LONG         flDefAttrs;

  USHORT       usIndex,
               usIndex1,
               usColorIndex,
               usMixIndex,
               usBackMixIndex,
               usBackColorIndex,
               usBitmapIndex;

  BOOL         fOutCome;

  BITMAPINFOHEADER BitMapHeader=
                   {sizeof(BITMAPINFOHEADER),0,0,1,1};  //bitmap info header

  PBITMAPINFO      pBitMapInfo=(PBITMAPINFO)Temp_buffer;
                                      //bitmap info structure


  IMAGEBUNDLE       QueriedImageBundle={0},
                    CurrentImageBundle={0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //Select a random seed.
       srand(0x4321);

    pBitMapInfo->cbFix=sizeof(BITMAPINFOHEADER); // This always =12
    pBitMapInfo->cPlanes=                        // one color plane
    pBitMapInfo->cBitCount=1;                    // 1-bit /pixel
    pBitMapInfo->argbColor[0].bBlue=
    pBitMapInfo->argbColor[0].bGreen=
    pBitMapInfo->argbColor[0].bRed=0;
    pBitMapInfo->argbColor[1].bBlue=
    pBitMapInfo->argbColor[1].bGreen=
    pBitMapInfo->argbColor[1].bRed=0xFF;


    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    SUBENTER(Setting New Image attributes,NULL);

    usIndex=(USHORT)(.67*delta_y); //Avoid the test sigenture printed
                                  //at bottom of display.
    for (; usIndex < (USHORT)(pg_size_y*delta_y); usIndex+=delta_y){

       for (usIndex1=0; usIndex1< (USHORT)(pg_size_x*delta_x);
                                                 usIndex1+=delta_x){

           usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
           usBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
           usMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
           usBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);
           usBitmapIndex=GETRANDOM(0,NUMBEROFELEMENTS(aBitMap)-1);

           CurrentImageBundle.lColor=asColor[usColorIndex].lAttrValue;
           CurrentImageBundle.lBackColor=asColor[usBackColorIndex].lAttrValue;
           CurrentImageBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
           CurrentImageBundle.usBackMixMode=(USHORT)asBackMix[usBackMixIndex].lAttrValue;


           sprintf(Message,"Setting Image's Attributes to Color= %s   BackColor=%s\n"
                           " Mix=%s   BackMix= %s .",
                            asColor[usColorIndex].AttrSymbolicName,
                            asColor[usBackColorIndex].AttrSymbolicName,
                            asMix[usMixIndex].AttrSymbolicName,
                            asBackMix[usBackMixIndex].AttrSymbolicName
                           );

           LOGINFO(L_DEBUG,"*",Message);

           //Set Image's current attributes to new values.
           if (GreSetAttributes(hDC,PRIM_IMAGE,(LONG)NULL,CHANGE_IMAGE,
                             &CurrentImageBundle)==FALSE){
              ErrorCount++;
              ERRSTATUS(TRUE,"Setting Image Attributes.",EntryName);
           }

         //Get Image's current attributes.
         if ((flDefAttrs=GreGetAttributes(hDC,PRIM_IMAGE,CHANGE_IMAGE,
                               &QueriedImageBundle))==GPI_ALTERROR){
            ErrorCount++;
            ERRSTATUS(TRUE,"Getting Image Attributes.",EntryName);

         }

         ErrorCount+=!(
               fOutCome=memicmp(&CurrentImageBundle,&QueriedImageBundle,
                               sizeof (CurrentImageBundle))==0 ? TRUE :FALSE);
         sprintf(Message,COMMON_SUFFICE2(Image),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);


         //This next code segment will create a randomly selected bitmap
         // and it will be displayed on the device using GpiWCBitBlt().
         // The function GpiWCBitBlt() will stretch the bitmap to fill
         // 1" X 1" square box.

         pBitMapInfo->cx=
         BitMapHeader.cx=aBitMap[usBitmapIndex].width;
         pBitMapInfo->cy=
         BitMapHeader.cy=aBitMap[usBitmapIndex].height;

         if ((hBM=GreCreateBitmap(hDC,&BitMapHeader,
                             CBM_INIT,aBitMap[usBitmapIndex].BitMapBuffer
                             ,pBitMapInfo))==FALSE){
          CLEANUP("GreCreateBitmap()",EntryName);
          return ;
         }

         //Setup the source and target rectangles.

         aPoints[3].x=aBitMap[usBitmapIndex].width;
         aPoints[3].y=aBitMap[usBitmapIndex].height;

         aPoints[0].x=usIndex1;
         aPoints[0].y=usIndex;
         MapPt(aPoints[0]);

         aPoints[1].x=aPoints[0].x+MapX(delta_x);
         aPoints[1].y=aPoints[0].y+MapY(delta_y);

         if (GpiWCBitBlt(hPS,hBM,4L,aPoints,ROP_SRCINVERT,BBO_AND)==GPI_ERROR){
           SHOWERROR("GpiWCBitBlt()",EntryName);
         }

         // Delete this bitmap handle.
         if (GreDeleteBitmap(hBM)==FALSE){
           SHOWERROR("GreDeleteBitmap()",EntryName);
         }


         //Now try to set attributes to their defaults by having
         // flAttrs and flDedAttrs options to be equal. This will
         // instruct GreSetAttributes() to set attributes to their default
         // values.


         LOGINFO(L_DEBUG,"*","> Setting Image's Current Attributes "
                          " to their default.");
         if (GreSetAttributes(hDC,PRIM_IMAGE,CHANGE_IMAGE,CHANGE_IMAGE,
                              &CurrentImageBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Image to default Attributes.",EntryName);

         }
         if ((flDefAttrs=GreGetAttributes(hDC,PRIM_IMAGE,CHANGE_IMAGE,
                              &QueriedImageBundle))==GPI_ALTERROR){
           ErrorCount++;
           ERRSTATUS(TRUE,"Getting Image Attributes.",EntryName);
         }

         ErrorCount+=!(fOutCome=VALIDATE_ATTRS(CHANGE_IMAGE,flDefAttrs));
         sprintf(Message,COMMON_SUFFICE1(Image),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);
       } //End FOR Loop
    } //End FOR loop.

    SUBEXIT();
    EXIT_MAIN(EntryName);
  }

  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetDefaultAttrsExh3()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetAttributes().
 *            GreGetAttributes().
 *
 *  Please note that this function only test setting/getting attributes
 *  for Images.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/18/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *               10/21/91, pdvt ,Mike Abuzant
 *                  -Test Case Implementation
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to draw an image of a 'CAT' for a finite number of times.
 *   Every three images will be displayed using the same randomly selected
 *   default attributes.
 *
 *    DO ( the following at least 80 time)
 *
 *       Randomly select colors, and mixes  from a finite
 *       set of attributes. Then set the Image's default attributes to the
 *       newly selected attributes.
 *         GreSetDefaultAttributes(hDC,PRIM_IMAGE,...);
 *
 *       Next Query the Image's current default attributes by calling the
 *       Gre function GreGetDefaultAttributes() to verify that the
 *       current default attributes are set correctly.
 *
 *         GreGetdefaultAttributes(hDC,PRIM_IMAGE,....)
 *
 *        DO (the following at least 3 time.)
 *
 *          Generate a random X coordinate.
 *          Generate a random Y coordinate. Then display the image.
 *           GpiImage(hPS,CAT....)
 *
 *        ENDDO.
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetDefaultAttrsExh3(HPS hPS, HWND hWndClient, BOOL SelectionCall){


  static       PCHAR EntryName="GreSetAndGetDefaultAttrsExh3()";
  CHAR         Message[250];

  HDC          hDC;
  POINTL       ptl={0};
  SIZEL        sizel={CAT_WIDTH,CAT_HEIGHT};
  USHORT       usIndex,
               usIndex1,
               usColorIndex,
               usMixIndex,
               usBackMixIndex,
               usBackColorIndex;

  BOOL         fOutCome;


  IMAGEBUNDLE  QueriedImageBundle={0},
               CurrentImageBundle={0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //Select a random seed.
       srand(0x1324);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    SUBENTER(Setting Image New Default attributes,NULL);


    TEST_PROGRESS ( 00, 80 );

    for (usIndex=0; usIndex < 80 ; usIndex++ ){

      usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
      usBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
      usMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
      usBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);

      CurrentImageBundle.lColor=asColor[usColorIndex].lAttrValue;
      CurrentImageBundle.lBackColor=asColor[usBackColorIndex].lAttrValue;
      CurrentImageBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
      CurrentImageBundle.usBackMixMode=(USHORT)asBackMix[usBackMixIndex].lAttrValue;


      sprintf(Message,"Setting Default Image Attributes to Color= %s   BackColor=%s\n"
                      " Mix=%s  BackMix= %s .",
                       asColor[usColorIndex].AttrSymbolicName,
                       asColor[usBackColorIndex].AttrSymbolicName,
                       asMix[usMixIndex].AttrSymbolicName,
                       asBackMix[usBackMixIndex].AttrSymbolicName
                      );

      LOGINFO(L_DEBUG,"*",Message);

      //Set Image's current Default attributes to new values.
      if (GreSetDefaultAttributes(hDC,PRIM_IMAGE,CHANGE_IMAGE,
                        &CurrentImageBundle)==FALSE){
         ErrorCount++;
         ERRSTATUS(TRUE,"Setting Default Image Attributes.",EntryName);
      }

      //Get Image Default current attributes.
      if (GreGetDefaultAttributes(hDC,PRIM_IMAGE,CHANGE_IMAGE,
                            &QueriedImageBundle)==FALSE){
         ErrorCount++;
         ERRSTATUS(TRUE,"Getting Default Image Attributes.",EntryName);

      }

      ErrorCount+=!(
            fOutCome=memicmp(&CurrentImageBundle,&QueriedImageBundle,
                            sizeof (CurrentImageBundle))==0 ? TRUE :FALSE);
      sprintf(Message,COMMON_SUFFICE3(Image),GOOD_FUNCTION(fOutCome));
      LOGINFO(L_DEBUG,"*",Message);

      for (usIndex1=0; usIndex1 < 3 ; usIndex1++ ){
         ptl.x=MapX(GETRANDOM(0,pg_size_x*delta_x));
         ptl.y=MapY(GETRANDOM(delta_y,pg_size_y*delta_y));

         if (GpiMove(hPS,&ptl)==GPI_ERROR){
            SHOWERROR("GpiMove()",EntryName);
         }

         if (GpiImage(hPS,(ULONG)NULL,&sizel,sizeof(abCat),abCat)==GPI_ERROR){
           SHOWERROR("GpiImage()",EntryName);
         }
      }


      TEST_PROGRESS ( usIndex + 1, 80 );
    }

    SUBEXIT();
    EXIT_MAIN(EntryName);
  }

  return;
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetAttributesExh4()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetAttributes().
 *            GreGetAttributes().
 *
 *  Please note that this function only test setting/getting attributes
 *  for lines' color, mixes, and type.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/21/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to draw a finite number of lines with randomly
 *   selected set of attributes. This entry point will only test
 *   lines' mix, color, and type.
 *
 *    DO (Fill the Device display height with lines )
 *
 *        DO (Fill the Device display width with lines)
 *
 *           Randomly select colors, mixes, and line types from a finite
 *           set of attributes. Then set the line's attributes to the
 *           newly selected attributes.
 *             GreSetAttributes(hDC,PRIM_LINE,...);
 *
 *           Next Query the Line's current attributes by calling the
 *           Gre function GreGetAttributes() to verify that the
 *           current attributes are set correctly.
 *
 *             GreGetAttributes(hDC,PRIM_LINE,....)
 *
 *           next draw lines, .1" a part, filling 1" X 1" box
 *             GpiLine(hPS,....)
 *
 *            Use the function GreSetAttributes() to set the Line's current
 *            attributes to their current default attributes (this is one of
 *            the options supplied to function in question).
 *
 *             GreSetAttributes(hDC,PRIM_LINE,...);
 *
 *           Next Query the Lines's current attributes by calling the
 *           Gre function GreGetAttributes() to verify that the
 *           current attributes are set correctly to their current defaults.
 *
 *             GreGetAttributes(hDC,PRIM_LINE,....)
 *
 *        ENDDO.
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetAttributesExh4(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetAttributesExh4()";
  CHAR         Message[250];

  HDC          hDC;
  POINTL       ptl;

  LONG         flDefAttrs;

  USHORT       usIndex,
               usIndex1,
               usIndex2,
               usColorIndex,
               usMixIndex,
               usTypeIndex;

  BOOL         fOutCome;

  LINEBUNDLE   QueriedLineBundle={0},
               CurrentLineBundle={CLR_BLUE,0L,FM_XOR,0U,0L,LINEWIDTH_NORMAL,
                                  LINETYPE_DOT,LINEEND_ROUND,LINEJOIN_MITRE};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    gpi_grid (hPS,ONE_INCH);

    //Select a random seed.
       srand(0x1f1f);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    SUBENTER(Setting New Line attributes,NULL);

    usIndex=(USHORT)(delta_y);    //Avoid the test sigenture printed
                                  //at bottom of display.
    for (; usIndex < (USHORT)(pg_size_y * delta_y); usIndex+=delta_y){

       for (usIndex1=0; usIndex1< (USHORT)(pg_size_x*delta_x);
                                                 usIndex1+=delta_x){

         usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
         usTypeIndex =GETRANDOM(0,NUMBEROFELEMENTS(asLineType)-1);
         usMixIndex  =GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);

         CurrentLineBundle.lColor   =asColor[usColorIndex].lAttrValue;
         CurrentLineBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
         CurrentLineBundle.usType   =(USHORT)asLineType[usTypeIndex].lAttrValue;

         sprintf(Message,"Setting Line's Attributes to Color= %s LineType=%s\n"
                          "Foreground Mix=%s",
                          asColor[usColorIndex].AttrSymbolicName,
                          asLineType[usTypeIndex].AttrSymbolicName,
                          asMix[usMixIndex].AttrSymbolicName
                         );

         LOGINFO(L_DEBUG,"*",Message);

         //Set Line current attributes to new values.
         if (GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,CHANGE_LINE,
                           &CurrentLineBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Line Attributes.",EntryName);
         }

         //Get Line current attributes.
         if (GreGetAttributes(hDC,PRIM_LINE,CHANGE_LINE,
                               &QueriedLineBundle)==GPI_ALTERROR){
            ErrorCount++;
            ERRSTATUS(TRUE,"Getting Line Attributes.",EntryName);

         }

         ErrorCount+=!(
               fOutCome=memicmp(&CurrentLineBundle,&QueriedLineBundle,
                               sizeof (CurrentLineBundle))==0 ? TRUE :FALSE);
         sprintf(Message,COMMON_SUFFICE2(Line),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);


         for (usIndex2=10 ;usIndex2 <(USHORT)delta_y ;usIndex2+=10){
            ptl.x=MapX(usIndex1);
            ptl.y=MapY(usIndex+usIndex2);

            if (GpiMove(hPS,&ptl)==GPI_ERROR){
               SHOWERROR("GpiMove()",EntryName);
            }

            ptl.x+=MapX(delta_x);

            if (GpiLine(hPS,&ptl)==GPI_ERROR){
               SHOWERROR("GpiLine()",EntryName);
            }

         } //End FOR Loop

         //Now try to set attributes to their defaults by having
         // flAttrs and flDedAttrs options to be equal. This will
         // instruct GreSetAttributes() to set attributes to their default
         // values.


         LOGINFO(L_DEBUG,"*","> Setting Line's Current Attributes "
                          " to their default.");

         if (GreSetAttributes(hDC,PRIM_LINE,CHANGE_LINE,CHANGE_LINE,
                              &CurrentLineBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Line to default Attributes.",EntryName);

         }
         if ((flDefAttrs=GreGetAttributes(hDC,PRIM_LINE,CHANGE_LINE,
                              &QueriedLineBundle))==GPI_ALTERROR){
           ErrorCount++;
           ERRSTATUS(TRUE,"Getting Line Attributes.",EntryName);

         }

         ErrorCount+=!(fOutCome=VALIDATE_ATTRS(CHANGE_LINE,flDefAttrs));
         sprintf(Message,COMMON_SUFFICE1(Line),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);
       } //End FOR Loop
    } //End FOR loop.

    SUBEXIT();

    GpiSetBackMix(hPS,BM_OVERPAINT);
    MapAndWriteString(.5*delta_x,1.5*delta_y,"All Lines Must Be Visible");

    EXIT_MAIN(EntryName);
  }

  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetDefaultAttrsExh4()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetDefaultAttributes().
 *            GreGetDefaultAttributes().
 *
 *  Please note that this function only test setting/getting attributes
 *  for lines' color, mixes, and type.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/21/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to draw a finite number of lines with randomly
 *   selected set of attributes. This entry point will only test
 *   lines' mix, color, and type.
 *
 *    DO (Fill the Device display height with lines )
 *
 *        DO (Fill the Device display width with lines)
 *
 *           Randomly select colors, mixes, and line types from a finite
 *           set of attributes. Then set the line's default attributes to the
 *           newly selected attributes.
 *             GreSetDefaultAttributes(hDC,PRIM_LINE,...);
 *
 *           Next Query the Line's current default attributes by calling the
 *           Gre function GreGetDefaultAttributes() to verify that the
 *           current default attributes are set correctly.
 *
 *             GreGetDefaultAttributes(hDC,PRIM_LINE,....)
 *
 *           next draw lines, .1" a part, filling 1" X 1" box
 *             GpiLine(hPS,....)
 *
 *        ENDDO.
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetDefaultAttrsExh4(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetDefaultAttrsExh4()";
  CHAR         Message[250];

  HDC          hDC;
  POINTL       ptl;

  USHORT       usIndex,
               usIndex1,
               usIndex2,
               usColorIndex,
               usMixIndex,
               usTypeIndex;

  BOOL         fOutCome;

  LINEBUNDLE   QueriedLineBundle={0},
               CurrentLineBundle={CLR_BLUE,0L,FM_XOR,0U,0L,LINEWIDTH_NORMAL,
                                  LINETYPE_DOT,LINEEND_ROUND,LINEJOIN_MITRE};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);


    //Select a random seed.
       srand(0x1111);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    gpi_grid(hPS,ONE_INCH);

    //Make sure that current line attributes are set to their defaults
    //since gpi_grid function changes them. This way we will have
    //a clean enviromnet to test in.
    if (GreSetAttributes(hDC,PRIM_LINE,CHANGE_LINE,CHANGE_LINE,
                                  (PBUNDLE)NULL)==FALSE){
      SHOWERROR("GreSetAttributes()",EntryName);
    }

    SUBENTER(Setting New default Line attributes,NULL);

    usIndex=(USHORT)(delta_y);    //Avoid the test sigenture printed
                                  //at bottom of display.
    for (; usIndex < (USHORT)(pg_size_y * delta_y); usIndex+=delta_y){

       for (usIndex1=0; usIndex1< (USHORT)(pg_size_x*delta_x);
                                                 usIndex1+=delta_x){

         usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
         usTypeIndex =GETRANDOM(0,NUMBEROFELEMENTS(asLineType)-1);
         usMixIndex  =GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);

         CurrentLineBundle.lColor   =asColor[usColorIndex].lAttrValue;
         CurrentLineBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
         CurrentLineBundle.usType   =(USHORT)asLineType[usTypeIndex].lAttrValue;

         sprintf(Message,"Setting Line default Attributes to Color= %s LineType=%s\n"
                          "Foreground Mix=%s",
                          asColor[usColorIndex].AttrSymbolicName,
                          asLineType[usTypeIndex].AttrSymbolicName,
                          asMix[usMixIndex].AttrSymbolicName
                         );

         LOGINFO(L_DEBUG,"*",Message);

         //Set Line default current attributes to new values.
         if (GreSetDefaultAttributes(hDC,PRIM_LINE,CHANGE_LINE,
                           &CurrentLineBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Line default Attributes.",EntryName);
         }

         //Get Line's current  default attributes.
         if (GreGetDefaultAttributes(hDC,PRIM_LINE,CHANGE_LINE,
                               &QueriedLineBundle)==GPI_ALTERROR){
            ErrorCount++;
            ERRSTATUS(TRUE,"Getting Line default Attributes.",EntryName);
         }

         ErrorCount+=!(
               fOutCome=memicmp(&CurrentLineBundle,&QueriedLineBundle,
                               sizeof (CurrentLineBundle))==0 ? TRUE :FALSE);
         sprintf(Message,COMMON_SUFFICE3(Line),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);


         for (usIndex2=10 ;usIndex2 <(USHORT)delta_y ;usIndex2+=10){
            ptl.x=MapX(usIndex1);
            ptl.y=MapY(usIndex+usIndex2);

            if (GpiMove(hPS,&ptl)==GPI_ERROR){
               SHOWERROR("GpiMove()",EntryName);
            }

            ptl.x+=MapX(delta_x);

            if (GpiLine(hPS,&ptl)==GPI_ERROR){
               SHOWERROR("GpiLine()",EntryName);
            }
         } //End FOR Loop
       } //End FOR Loop
    } //End FOR loop.

    SUBEXIT();

    GpiSetBackMix(hPS,BM_OVERPAINT);
    MapAndWriteString(.5*delta_x,1.5*delta_y,"All Lines Must Be Visible");

    EXIT_MAIN(EntryName);
  }

  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetAttributesExh5()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetAttributes().
 *            GreGetAttributes().
 *
 *  Please note that this function only test setting/getting attributes
 *  for lines' geometric width, joins, and ends.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/22/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to draw a finite number of Polylines,an 'L' shaped,
 *   with randomly selected set of attributes. This entry point will
 *   only test lines' joins, ends, and geometric width .
 *
 *    DO (Fill the Device display height  with 'L' Shaped Polylines )
 *
 *        DO (Fill the Device display width with 'L' Shaped Polylines )
 *
 *           Randomly select Ends, Joins, and line Geometric width from a finite
 *           set of attributes. Then set the line's attributes to the
 *           newly selected attributes.
 *             GreSetAttributes(hDC,PRIM_LINE,...);
 *
 *           Next Query the Line's current attributes by calling the
 *           Gre function GreGetAttributes() to verify that the
 *           current attributes are set correctly.
 *
 *             GreGetAttributes(hDC,PRIM_LINE,....)
 *
 *          Next open a path brackets and draw 'L' shaped Polylines that
 *          are 1" a part.
 *             GpiPolyLine(hPS,....)
 *
 *          finally, Close the path.
 *           GpiEndPath(hPS,..)
 *
 *            Use the function GreSetAttributes() to set the Line's current
 *            attributes to their current default attributes (this is one of
 *            the options supplied to function in question).
 *
 *             GreSetAttributes(hDC,PRIM_LINE,...);
 *
 *           Next Query the Lines's current attributes by calling the
 *           Gre function GreGetAttributes() to verify that the
 *           current attributes are set correctly to their current defaults.
 *
 *             GreGetAttributes(hDC,PRIM_LINE,....)
 *
 *        ENDDO.
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetAttributesExh5(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetAttributesExh5()";
  CHAR         Message[250];

  HDC          hDC;
  POINTL       aPoints[3];

  LONG         flDefAttrs;

  USHORT       usIndex,
               usIndex1,
               usEndIndex,
               usJoinIndex;

  BOOL         fOutCome;

  LINEBUNDLE   QueriedLineBundle={0},
               CurrentLineBundle={CLR_BLUE,0L,FM_XOR,0U,0L,LINEWIDTH_NORMAL,
                                  LINETYPE_DOT,LINEEND_ROUND,LINEJOIN_MITRE};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //Select a random seed.
    srand(0xDEAD);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    SUBENTER(Setting New Line attributes,NULL);

    usIndex=(USHORT)(delta_y);    //Avoid the test sigenture printed
                                  //at bottom of display.
    for (; usIndex < (USHORT)(pg_size_y * delta_y); usIndex+=delta_y){

       for (usIndex1=(USHORT)(.5*delta_x);usIndex1<(USHORT)(pg_size_x*delta_x);
                                                 usIndex1+=delta_x){

        if (GpiSetPattern(hPS,PATSYM_HALFTONE)==FALSE){
           SHOWERROR("GpiSetPattern()",EntryName);
        }

        usEndIndex=GETRANDOM(0,NUMBEROFELEMENTS(asLineEnd)-1);
        usJoinIndex=GETRANDOM(0,NUMBEROFELEMENTS(asLineJoin)-1);

        CurrentLineBundle.usJoin=(USHORT)asLineJoin[usJoinIndex].lAttrValue;
        CurrentLineBundle.usEnd=(USHORT)asLineEnd[usEndIndex].lAttrValue;

        CurrentLineBundle.lGeomWidth=MapX(GETRANDOM(1,.3*delta_x));

        sprintf(Message,"Setting Line's Attributes to LineEnds= %s LineJoin=%s\n"
                        "Geom Width=0.%ld Inches.",
                         asLineEnd[usEndIndex].AttrSymbolicName,
                         asLineJoin[usJoinIndex].AttrSymbolicName,
                         CurrentLineBundle.lGeomWidth
                        );

         LOGINFO(L_DEBUG,"*",Message);

         //Set Line's current attributes to new values.
         if (GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,CHANGE_LINE,
                           &CurrentLineBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Line Attributes.",EntryName);
         }

         //Get Line's current attributes.
         if (GreGetAttributes(hDC,PRIM_LINE,CHANGE_LINE,
                               &QueriedLineBundle)==GPI_ALTERROR){
            ErrorCount++;
            ERRSTATUS(TRUE,"Getting Line Attributes.",EntryName);
         }

         ErrorCount+=!(
               fOutCome=memicmp(&CurrentLineBundle,&QueriedLineBundle,
                               sizeof (CurrentLineBundle))==0 ? TRUE :FALSE);
         sprintf(Message,COMMON_SUFFICE2(Line),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);


         //The next code segment is used to draw a L shape figure
         //using GpiPolyLine() in order to show line ends and joins.

         aPoints[1].x=
         aPoints[0].x=MapX(usIndex1);
         aPoints[0].y=MapY(usIndex+.5*delta_y);
         aPoints[2].x=MapX(usIndex1+.5*delta_x);
         aPoints[2].y=
         aPoints[1].y=MapY(usIndex);

         if (GpiBeginPath(hPS,1L)==FALSE){
           SHOWERROR("GpiBeginPath()",EntryName);
         }
         if (GpiMove(hPS,aPoints)==GPI_ERROR){
            SHOWERROR("GpiMove()",EntryName);
         }
         if (GpiPolyLine(hPS,2,aPoints+1)==GPI_ERROR){
            SHOWERROR("GpiPolyLine()",EntryName);
         }

         if (GpiEndPath(hPS)==GPI_ERROR){
            SHOWERROR("GpiEndPath()",EntryName);
         }
         if (GpiStrokePath(hPS, 1L, (LONG)NULL) == GPI_ERROR){
            SHOWERROR("GpiStrokePath",NULL);
         }

         //Now try to set attributes to their defaults by having
         // flAttrs and flDedAttrs options to be equal. This will
         // instruct GreSetAttributes() to set attributes to their default
         // values.


          LOGINFO(L_DEBUG,"*","> Setting Line's Current Attributes "
                          " to their default.");

         if (GreSetAttributes(hDC,PRIM_LINE,CHANGE_LINE,CHANGE_LINE,
                              &CurrentLineBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Line to default Attributes.",EntryName);

         }
         if ((flDefAttrs=GreGetAttributes(hDC,PRIM_LINE,CHANGE_LINE,
                              &QueriedLineBundle))==GPI_ALTERROR){
           ErrorCount++;
           ERRSTATUS(TRUE,"Getting Line Attributes.",EntryName);
         }

         ErrorCount+=!(fOutCome=VALIDATE_ATTRS(CHANGE_LINE,flDefAttrs));
         sprintf(Message,COMMON_SUFFICE1(Line),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);
       } //End FOR Loop
    } //End FOR loop.

    SUBEXIT();
    EXIT_MAIN(EntryName);
  }

  return;
}


/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetDefaultAttrsExh5()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetDefaultAttributes().
 *            GreGetDefaultAttributes().
 *
 *  Please note that this function only test setting/getting default
 *  attributes for lines' geometric width, joins, and ends.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/22/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to draw a finite number of Polylines,an 'L' shaped,
 *   with randomly selected set of attributes. This entry point will
 *   only test lines' joins, ends, and geometric width .
 *
 *    DO (Fill the Device display height  with 'L' Shaped Polylines )
 *
 *        DO (Fill the Device display width with 'L' Shaped Polylines )
 *
 *           Randomly select Ends, Joins, and line Geometric width from a finite
 *           set of attributes. Then set the line's default attributes to the
 *           newly selected attributes.
 *             GreSetDefaultAttributes(hDC,PRIM_LINE,...);
 *
 *           Next Query the Line's current default attributes by calling the
 *           Gre function GreGetDefaultAttributes() to verify that the
 *           current default attributes are set correctly.
 *
 *             GreGetDefaultAttributes(hDC,PRIM_LINE,....)
 *
 *          Next open a path brackets and draw 'L' shaped Polylines that
 *          are 1" a part.
 *             GpiPolyLine(hPS,....)
 *
 *          finally, Close the path.
 *           GpiEndPath(hPS,..)
 *
 *        ENDDO.
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetDefaultAttrsExh5(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetDefaultAttrsExh5()";
  CHAR         Message[250];

  HDC          hDC;
  POINTL       aPoints[3];

  USHORT       usIndex,
               usIndex1,
               usEndIndex,
               usJoinIndex;

  BOOL         fOutCome;

  LINEBUNDLE   QueriedLineBundle={0},
               CurrentLineBundle={CLR_BLUE,0L,FM_XOR,0U,0L,LINEWIDTH_NORMAL,
                                  LINETYPE_DOT,LINEEND_ROUND,LINEJOIN_MITRE};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    //Select a random seed.
    srand(0xDEAF);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    SUBENTER(Setting New Line default attributes,NULL);

    usIndex=(USHORT)(delta_y);    //Avoid the test sigenture printed
                                  //at bottom of display.
    for (; usIndex < (USHORT)(pg_size_y * delta_y); usIndex+=delta_y){

       for (usIndex1=(USHORT)(.5*delta_x);usIndex1<(USHORT)(pg_size_x*delta_x);
                                                 usIndex1+=delta_x){

        if (GpiSetPattern(hPS,PATSYM_HALFTONE)==FALSE){
           SHOWERROR("GpiSetPattern()",EntryName);
        }

        usEndIndex=GETRANDOM(0,NUMBEROFELEMENTS(asLineEnd)-1);
        usJoinIndex=GETRANDOM(0,NUMBEROFELEMENTS(asLineJoin)-1);

        CurrentLineBundle.usJoin=(USHORT)asLineJoin[usJoinIndex].lAttrValue;
        CurrentLineBundle.usEnd=(USHORT)asLineEnd[usEndIndex].lAttrValue;

        CurrentLineBundle.lGeomWidth=MapX(GETRANDOM(1,.3*delta_x));

        sprintf(Message,"Setting Line's default Attributes to LineEnds= %s LineJoin=%s\n"
                        "Geom Width=0.%ld Inches.",
                         asLineEnd[usEndIndex].AttrSymbolicName,
                         asLineJoin[usJoinIndex].AttrSymbolicName,
                         CurrentLineBundle.lGeomWidth
                        );

         LOGINFO(L_DEBUG,"*",Message);

         //Set Line's current default attributes to new values.
         if (GreSetDefaultAttributes(hDC,PRIM_LINE,CHANGE_LINE,
                           &CurrentLineBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Line default Attributes.",EntryName);
         }

         //Get Line's current default attributes.
         if (GreGetDefaultAttributes(hDC,PRIM_LINE,CHANGE_LINE,
                               &QueriedLineBundle)==GPI_ALTERROR){
            ErrorCount++;
            ERRSTATUS(TRUE,"Getting Line default Attributes.",EntryName);
         }

         ErrorCount+=!(
               fOutCome=memicmp(&CurrentLineBundle,&QueriedLineBundle,
                               sizeof (CurrentLineBundle))==0 ? TRUE :FALSE);
         sprintf(Message,COMMON_SUFFICE3(Line),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);


         //The next code segment is used to draw a L shape figure
         //using GpiPolyLine() in order to show line ends and joins.
         aPoints[1].x=
         aPoints[0].x=MapX(usIndex1);
         aPoints[0].y=MapY(usIndex+.5*delta_y);
         aPoints[2].x=MapX(usIndex1+.5*delta_x);
         aPoints[2].y=
         aPoints[1].y=MapY(usIndex);


         if (GpiBeginPath(hPS,1L)==FALSE){
           SHOWERROR("GpiBeginPath()",EntryName);
         }
         if (GpiMove(hPS,aPoints)==GPI_ERROR){
            SHOWERROR("GpiMove()",EntryName);
         }
         if (GpiPolyLine(hPS,2,aPoints+1)==GPI_ERROR){
            SHOWERROR("GpiPolyLine()",EntryName);
         }
         if (GpiEndPath(hPS)==GPI_ERROR){
            SHOWERROR("GpiEndPath()",EntryName);
         }
         if (GpiStrokePath(hPS, 1L, (LONG)NULL) == GPI_ERROR){
            SHOWERROR("GpiStrokePath",NULL);
         }
       } //End FOR Loop
    } //End FOR loop.
    SUBEXIT();
    EXIT_MAIN(EntryName);
  }

  return;
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetAttributesExh6()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions:
 *            GreSetAttributes().
 *            GreGetAttributes().
 *
 *  Please note that this function only test setting/getting attributes
 *  for character attributes.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/22/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to display a finite number of character strings.
 *   Every displayed string will have a randomly selected set of attributes.
 *
 *    DO (Fill the Device display height  with  charcter strings )
 *
 *        Randomly select charcher attributes from a finite
 *        set of attributes. Then set the  character's attributes to the
 *        newly selected attributes.
 *          GreSetAttributes(hDC,PRIM_CHAR,...);
 *
 *        Next Query the character's current attributes by calling the
 *        Gre function GreGetAttributes() to verify that the
 *        current attributes are set correctly.
 *
 *          GreGetAttributes(hDC,PRIM_CHAR,....)
 *
 *        Next display a string and advance to next line.
 *          GpiCharStringAt(hPS,...);
 *          Y=Y+height of current font.
 *
 *         Use the function GreSetAttributes() to set the character's current
 *         attributes to their current default attributes (this is one of
 *         the options supplied to function in question).
 *
 *          GreSetAttributes(hDC,PRIM_CHAR,...);
 *
 *        Next Query the Character's current attributes by calling the
 *        Gre function GreGetAttributes() to verify that the
 *        current attributes are set correctly to their current defaults.
 *
 *          GreGetAttributes(hDC,PRIM_CHAR,....)
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetAttributesExh6(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetAttributesExh6()";

  CHAR         Message[500];

  HDC          hDC;
  POINTL       ptl={0};

  LONG         flDefAttrs;

  USHORT       usDirectionIndex,
               usMixIndex,
               usBackMixIndex,
               usBackColorIndex,
               usColorIndex,
               usCharHite;

  USHORT       test_progress = 00;
  USHORT       test_limit;

  BOOL         fOutCome;

  FONTMETRICS  fm;

  CHARBUNDLE   QueriedCharBundle={0},
               CurrentCharBundle={0,0,0,0,0,CM_MODE3,{0},{1,0},{0}};

  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    //
    //Select a random seed.
    srand(0xDEAF);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    SUBENTER(Setting New Char attributes,NULL);

    ptl.y=MapY((.7*delta_y));   //Avoid the test sigenture printed
                                      //at bottom of display.

    test_limit = ( USHORT ) ( MapY ( pg_size_y * delta_y - ptl.y ) );

    while( UnMapY(ptl.y) < pg_size_y * delta_y) {

       TEST_PROGRESS ( test_progress, test_limit );

       usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       usBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       usMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
       usBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);
       usDirectionIndex=GETRANDOM(0,1);

       CurrentCharBundle.usDirection=(USHORT)asDirection[usDirectionIndex].lAttrValue;
       CurrentCharBundle.lColor=asColor[usColorIndex].lAttrValue;
       CurrentCharBundle.lBackColor=asColor[usBackColorIndex].lAttrValue;
       CurrentCharBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
       CurrentCharBundle.usBackMixMode=(USHORT)asBackMix[usBackMixIndex].lAttrValue;
       CurrentCharBundle.sizfxCell.cx=
                    MAKEFIXED(MapX(GETRANDOM(.2*delta_x,.6*delta_x)),0);
       CurrentCharBundle.sizfxCell.cy=
                    MAKEFIXED(MapY(GETRANDOM(.1*delta_y,.4*delta_y)),0);

       CurrentCharBundle.ptlShear.x=GETRANDOM(-2,2);
       CurrentCharBundle.ptlShear.y=GETRANDOM(-3,3);
       //
       //Shear in Y direction can't be zero.
       //
       if (CurrentCharBundle.ptlShear.y==0)
           CurrentCharBundle.ptlShear.y++;

       if (GpiQueryFontMetrics(hPS,(LONG)sizeof(fm),&fm )==FALSE){
         SHOWERROR("GpiQueryFontMetrics()",EntryName);
       }
       usCharHite=(USHORT)(1.5*fm.lMaxBaselineExt);

       sprintf(Message,"Setting Character's Attributes to Color=%s BackColor=%s\n"
                        "Mix=%s   BackMix=%s   Direction=%s  Char Mode=CM_MODE3"
                        "  Char Box=(%d,%d)  Char Shear= (%ld,%ld) ",

                        asColor[usColorIndex].AttrSymbolicName,
                        asColor[usBackColorIndex].AttrSymbolicName,
                        asMix[usMixIndex].AttrSymbolicName,
                        asBackMix[usBackMixIndex].AttrSymbolicName,
                        asDirection[usDirectionIndex].AttrSymbolicName,
                        FIXEDINT(CurrentCharBundle.sizfxCell.cx),
                        FIXEDINT(CurrentCharBundle.sizfxCell.cy),
                        CurrentCharBundle.ptlShear.x,
                        CurrentCharBundle.ptlShear.y
                        );

         LOGINFO(L_DEBUG,"*",Message);

         //Set character's current attributes to new values.
         if (GreSetAttributes(hDC,PRIM_CHAR,(LONG)NULL,CHANGE_CHAR,
                           &CurrentCharBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Character Attributes.",EntryName);
         }

         //Get Character's current attributes.
         if (GreGetAttributes(hDC,PRIM_CHAR,CHANGE_CHAR,
                               &QueriedCharBundle)==GPI_ALTERROR){
            ErrorCount++;
            ERRSTATUS(TRUE,"Getting Character Attributes.",EntryName);
         }

         ErrorCount+=!(
               fOutCome=memicmp(&CurrentCharBundle,&QueriedCharBundle,
                               sizeof (CurrentCharBundle))==0 ? TRUE :FALSE);
         sprintf(Message,COMMON_SUFFICE2(Character),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);

         //
        //Ouptut string to the display device
        //
         ptl.x=asDirection[usDirectionIndex].lAttrValue==CHDIRN_LEFTRIGHT ?
                                ((LONG)NULL) :  (MapX(pg_size_x*delta_x));

         strcpy(Message, "Testing GreGetAttributes()"
                         " and GreSetAttributes()."
                         " Text is displayed using GpiCharStringAt()."
                         );

         if (GpiCharStringAt(hPS,&ptl,strlen(Message),Message)==GPI_ERROR){
           SHOWERROR("GpiCharStringAt()",EntryName);
         }

         //Now try to set attributes to their defaults by having
         // flAttrs and flDedAttrs options to be equal. This will
         // instruct GreSetAttributes() to set attributes to their default
         // values.

         LOGINFO(L_DEBUG,"*","> Setting Character's Current Attributes "
                          " to their default.");

         if (GreSetAttributes(hDC,PRIM_CHAR,CHANGE_CHAR,CHANGE_CHAR,
                              &CurrentCharBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Character to default Attributes.",EntryName);

         }
         if ((flDefAttrs=GreGetAttributes(hDC,PRIM_CHAR,CHANGE_CHAR,
                              &QueriedCharBundle))==GPI_ALTERROR){
           ErrorCount++;
           ERRSTATUS(TRUE,"Getting Character Attributes.",EntryName);

         }

         ErrorCount+=!(fOutCome=VALIDATE_ATTRS(CHANGE_CHAR,flDefAttrs));
         sprintf(Message,COMMON_SUFFICE1(Character),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);

         //Advance to next line.
         ptl.y+=usCharHite;

         test_progress += usCharHite;
    }


    TEST_PROGRESS ( test_limit, test_limit );

    SUBEXIT();
    EXIT_MAIN(EntryName);
  }

  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAndGetDefaultAttrsExh6()
 *
 * Purpose:  This function will test the functionality of  thefollowing
 *           Gre functions:
 *            GreSetDefaultAttributes().
 *            GreGetDefaultAttributes().
 *
 *  Please note that this function only test setting/getting default
 *  attributes for character attributes.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/22/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:
 *  Setting character default attributes to different values other than the
 *  the standard default values has no effects when displaying strings.
 *  Colors, character sizes, and mixes always have the same standard
 *  default values. Note that this happens only when the character mode
 *  is set to CM_MODE3.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Setup condition to display a finite number of character strings.
 *   Every displayed string will have a randomly selected set of
 *   default attributes.
 *
 *    DO (Fill the Device display height  with  charcter strings )
 *
 *        Randomly select default character attributes from a finite
 *        set of attributes. Then set the character's default attributes
 *        to the newly selected attributes.
 *          GreSetDefaultAttributes(hDC,PRIM_CHAR,...);
 *
 *        Next Query the character's current default attributes by calling the
 *        Gre function GreGetAttributes() to verify that the
 *        current attributes are set correctly.
 *
 *          GreGetdefaultAttributes(hDC,PRIM_CHAR,....)
 *
 *        Next display a string and advance to next line.
 *          GpiCharStringAt(hPS,...);
 *          Y=Y+height of current font.
 *
 *    ENDDO.
 *
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAndGetDefaultAttrsExh6(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static       PCHAR EntryName="GreSetAndGetDefaultAttrsExh6()";

  CHAR         Message[500];

  HDC          hDC;
  POINTL       ptl={0};

  USHORT       usDirectionIndex,
               usMixIndex,
               usBackMixIndex,
               usBackColorIndex,
               usColorIndex,
               usCharHite;

  USHORT       test_progress = 00;
  USHORT       test_limit;

  BOOL         fOutCome;

  FONTMETRICS  fm;

  CHARBUNDLE   QueriedCharBundle={0},
               CurrentCharBundle={0,0,0,0,0,CM_MODE3,{0},{1,0},{0,1}};

  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    //
    //Select a random seed.
    srand(0xFEED);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    SUBENTER(Setting New character default attributes,NULL);

    ptl.y=MapY((.7*delta_y));   //Avoid the test sigenture printed
                                      //at bottom of display.
    test_limit = ( USHORT ) ( MapY ( pg_size_y * delta_y - ptl.y ) );

    while( UnMapY(ptl.y) < pg_size_y * delta_y) {

       TEST_PROGRESS ( test_progress, test_limit );

       usColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       usBackColorIndex=GETRANDOM(0,NUMBEROFELEMENTS(asColor)-1);
       usMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asMix)-1);
       usBackMixIndex=GETRANDOM(0,NUMBEROFELEMENTS(asBackMix)-1);
       usDirectionIndex=GETRANDOM(0,1);

       CurrentCharBundle.usDirection=(USHORT)asDirection[usDirectionIndex].lAttrValue;
       CurrentCharBundle.lColor=asColor[usColorIndex].lAttrValue;
       CurrentCharBundle.lBackColor=asColor[usBackColorIndex].lAttrValue;
       CurrentCharBundle.usMixMode=(USHORT)asMix[usMixIndex].lAttrValue;
       CurrentCharBundle.usBackMixMode=(USHORT)asBackMix[usBackMixIndex].lAttrValue;
       CurrentCharBundle.sizfxCell.cx=
                    MAKEFIXED(MapX(GETRANDOM(.2*delta_x,.6*delta_x)),0);
       CurrentCharBundle.sizfxCell.cy=
                    MAKEFIXED(MapY(GETRANDOM(.1*delta_y,.4*delta_y)),0);

       CurrentCharBundle.ptlShear.x=GETRANDOM(-2,2);
       CurrentCharBundle.ptlShear.y=GETRANDOM(-3,3);
       //
       //Shear in Y direction can't be zero.
       //
       if (CurrentCharBundle.ptlShear.y==0)
           CurrentCharBundle.ptlShear.y++;

       if (GpiQueryFontMetrics(hPS,(LONG)sizeof(fm),&fm )==FALSE){
         SHOWERROR("GpiQueryFontMetrics()",EntryName);
       }
       usCharHite=(USHORT)(1.5*fm.lMaxBaselineExt);

       sprintf(Message,"Setting Character's Attributes to Color=%s BackColor=%s\n"
                        "Mix=%s   BackMix=%s   Direction=%s  Char Mode=CM_MODE3"
                        "  Char Box=(%d,%d)  Char Shear= (%ld,%ld) ",

                        asColor[usColorIndex].AttrSymbolicName,
                        asColor[usBackColorIndex].AttrSymbolicName,
                        asMix[usMixIndex].AttrSymbolicName,
                        asBackMix[usBackMixIndex].AttrSymbolicName,
                        asDirection[usDirectionIndex].AttrSymbolicName,
                        FIXEDINT(CurrentCharBundle.sizfxCell.cx),
                        FIXEDINT(CurrentCharBundle.sizfxCell.cy),
                        CurrentCharBundle.ptlShear.x,
                        CurrentCharBundle.ptlShear.y
                       );

         LOGINFO(L_DEBUG,"*",Message);

         //Set character's current attributes to new values.
         if (GreSetDefaultAttributes(hDC,PRIM_CHAR,CHANGE_CHAR,
                           &CurrentCharBundle)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"Setting Character default Attributes.",EntryName);
         }

         //Get Character's current default attributes.
         if (GreGetDefaultAttributes(hDC,PRIM_CHAR,CHANGE_CHAR,
                               &QueriedCharBundle)==GPI_ALTERROR){
            ErrorCount++;
            ERRSTATUS(TRUE,"Getting Character default Attributes.",EntryName);
         }

         ErrorCount+=!(
               fOutCome=memicmp(&CurrentCharBundle,&QueriedCharBundle,
                               sizeof (CurrentCharBundle))==0 ? TRUE :FALSE);
         sprintf(Message,COMMON_SUFFICE3(Character),GOOD_FUNCTION(fOutCome));
         LOGINFO(L_DEBUG,"*",Message);

         //
        //Ouptut string to the display device
        //
         strcpy(Message, "Testing GreGetDefaultAttributes()"
                         " and GreSetDefaultAttributes()."
                         " Text is displayed using GpiCharStringAt()."
                         );
         ptl.x=asDirection[usDirectionIndex].lAttrValue==CHDIRN_LEFTRIGHT ?
                                ((LONG)NULL) :  (MapX(pg_size_x*delta_x));

         if (GpiCharStringAt(hPS,&ptl,strlen(Message),Message)==GPI_ERROR){
           SHOWERROR("GpiCharStringAt()",EntryName);
         }

         //Advance to next line.
         ptl.y+=usCharHite;

         test_progress += usCharHite;
    }


    TEST_PROGRESS ( test_limit, test_limit );

    SUBEXIT();
    EXIT_MAIN(EntryName);
  }

  return;
}
