/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/*const char header=$header$*/

#define INCL_DEV                    //Include all DEV definitions
#define INCL_WIN                    //Include all WIN definitions
#define INCL_GREALL                 // include all Gre definitions
#define INCL_GPI                    //Include all Gpi definitions
#define INCL_ERRORS                 //Include all error definitions
#define INCL_DOSMEMMGR
#define INCL_DOSERRORS
#define INCL_DDIMISC
#define INCL_GRE_DEVSUPPORT
#define INCL_GPIPRIMITIVES
#define INCL_DDIBUNDLES
#include <string.h>
#include <stdlib.h>
#include <os2.h>
#include <pmddi.h>                  // gre - device driver includes
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "greatt2.h"                // bit map dll header file

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetGlobalAttributesRc()
 *
 * Purpose:  This function exhaustively tests for most of the errors codes
 *           that may be returned by GreSetGlobalAttributes().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL) GreSetGlobalAttributes(HDC       hDC,
 *                                    ULONG    lAttrType,
 *                                    ULONG    lAttribute,
 *                                    ULONG    flOptions
 *                                  )
 *    This Gre sets the specified attribute in line,pattern,character,
 *    and maker bundles. The attribute can be set to its default value
 *    or to  aspecified.
 *
 *     The return value is:
 *        TRUE              Success
 *        FALSE              Error
 *
 *     Possible returns from WinGetLastError:
 *      PMERR_INV_IN_AREA
 *      PMERR_HDC_BUSY
 *      PMERR_DEV_FUNCTION_NOT_INSTALLED
 *      PMERR_INV_RESET_OPTION
 *      PMERR_INV_MIX_ATTR
 *      PMERR_INV_BACKGROUND_MIX_ATTR
 *      PMERR_INV_HDC
 *      PMERR_INV_COLOR_ATTR
 *      PMERR_INV_BACGROUND_COL_ATTR
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/02/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               10/02/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *      PMERR_INV_IN_AREA
 *      PMERR_HDC_BUSY
 *      PMERR_INV_RESET_OPTION
 *      PMERR_INV_MIX_ATTR
 *      PMERR_INV_BACKGROUND_MIX_ATTR
 *      PMERR_INV_HDC
 *      PMERR_INV_COLOR_ATTR
 *      PMERR_INV_BACGROUND_COL_ATTR
 *
 * Return Codes Not Tested Successfully Yet:
 *      PMERR_DEV_FUNCTION_NOT_INSTALLED
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *  None
 *
 *  NOTE:
 *   A return codes was tested even though it is not listed
 *   under this function. It seemed logical for this function to return
 *   the following return codes:
 *     PMERR_INV_IN_PATH
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSetGlobalAttributes(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetGlobalAttributeRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreSetGlobalAttributeRc()";
  BOOL           fSuccess;          //Holds return code.
  HDC            hDC;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER("Query hDC associated with PS",NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    SETEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreSetGlobalAttribute((HDC)NULL,ATYPE_COLOR,CLR_RED,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreSetGlobalAttribute((HDC)0x0fffffff,ATYPE_COLOR,CLR_RED,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreSetGlobalAttribute((HDC)0xfffffff,ATYPE_COLOR,CLR_RED,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/

    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);               //Make this PS busy.
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_COLOR,CLR_RED,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_HDC_BUSY,NULL);
    (VOID)FREEPS;                       //Kill BUSY thread.
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_AREA        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_AREA,NULL);

    SETENTER(EntryName," Opening AREA bracket");

    if (GpiBeginArea(hPS,BA_WINDING)==FALSE){
       SHOWERROR("GpiBeginArea()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open area
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_COLOR,CLR_RED,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_IN_AREA,NULL);

    //Close The Area

    SETENTER(EntryName," Closing AREA bracket");

    if (GpiEndArea(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndArea()",EntryName);
    }
    SETEXIT();

    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_PATH        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_PATH,NULL);

    SETENTER(EntryName," Opening Path bracket");
    if (GpiBeginPath(hPS,1L)==FALSE){
       SHOWERROR("GpiBeginPath()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open path
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_COLOR,CLR_RED,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_IN_PATH,NULL);

    //Close The Area
    SETENTER(EntryName," Closing Path bracket");

    if (GpiEndPath(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndPath()",EntryName);
    }
    SETEXIT();

    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_RESET_OPTIONS       *
    * error Code                                                          *
    ***********************************************************************/

    SUBENTER(PMERR_INV_RESET_OPTIONS,"Test1");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_COLOR,CLR_RED,0x0ffffffL);
    CHECKRC(fSuccess,PMERR_INV_RESET_OPTIONS,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_RESET_OPTIONS,"Test2");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_COLOR,CLR_RED,-1);
    CHECKRC(fSuccess,PMERR_INV_RESET_OPTIONS,"Test2");
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_MIX_ATTR            *
    * error Code                                                          *
    ***********************************************************************/

    SUBENTER(PMERR_INV_MIX_ATTR,"Test1");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_MIX_MODE,(LONG)NULL,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_MIX_ATTR,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_MIX_ATTR,"Test2");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_MIX_MODE,(LONG)0x0fffffffL,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_MIX_ATTR,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_MIX_ATTR,"Test3");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_MIX_MODE,(LONG)0xffffffffL,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_MIX_ATTR,"Test3");
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_BACKGROUND_MIX_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    SUBENTER(PMERR_INV_BACKGROUND_MIX_ATTR,"Test1");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_BACK_MIX_MODE,(LONG)NULL,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_MIX_ATTR,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_BACKGROUND_MIX_ATTR,"Test2");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_BACK_MIX_MODE,(LONG)0x0fffffffL,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_MIX_ATTR,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_BACKGROUND_MIX_ATTR,"Test3");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_BACK_MIX_MODE,(LONG)0xffffffffL,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_MIX_ATTR,"Test3");
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_COLOR_ATTR          *
    * error Code                                                          *
    ***********************************************************************/

    SUBENTER(PMERR_INV_COLOR_ATTR,"Test1");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_COLOR,(LONG)0x0fffffffL,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_COLOR_ATTR,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_COLOR_ATTR,"Test2");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_COLOR,(LONG)0x12345678L,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_COLOR_ATTR,"Test2");
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_BACKGROUND_COL_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    SUBENTER(PMERR_INV_BACKGROUND_COL_ATTR,"Test1");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_BACK_COLOR,(LONG)0x0fffffffL,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_COL_ATTR,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_BACKGROUND_COL_ATTR,"Test2");
    fSuccess=GreSetGlobalAttribute(hDC,ATYPE_BACK_COLOR,(LONG)0x12345678L,!GATTR_DEFAULT);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_COL_ATTR,"Test2");
    SUBEXIT();

    EXIT_MAIN(EntryName);
  }
  return;
}


/****************************************************************************
 *\\ddd
 * Routine Name: GreGetAttributesRc()
 *
 * Purpose:  This function exhaustively tests for most of the errors codes
 *           that may be returned by GreGetAttributes().
 *     The syntax used to call the Gre is :
 *
 *     (LONG) GreGetAttributes(HDC     hDC,
 *                             LONG    lPrimType,
 *                             ULONG   flAttrs,
 *                             PBUNDLE paAttrs
 *                              )
 *    This Gre returns default values of the attributes indicated in flAttrs.
 *
 *     The return value is:
 *        TRUE              Success
 *        FALSE              Error
 *
 *     Possible returns from WinGetLastError:
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/02/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               10/02/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *
 * Return Codes Not Tested Successfully Yet:
 *      None
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *  None
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreGetAttributes(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/


VOID  APIENTRY GreGetAttributesRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreGetAttributesRc()";
  LONG           flSuccess;          //Holds return code.
  HDC            hDC;
  DLINEBUNDLE    LineBundle={sizeof(LineBundle),0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER("Query hDC associated with PS",NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    SETEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    flSuccess=GreGetAttributes((HDC)NULL,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(flSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    flSuccess=GreGetAttributes((HDC)0x0fffffff,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(flSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    flSuccess=GreGetAttributes((HDC)0xffffffff,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(flSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/

    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);               //Make this PS busy.
    flSuccess=GreGetAttributes(hDC,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(flSuccess,PMERR_HDC_BUSY,NULL);
    (VOID)FREEPS;                       //Kill BUSY thread.
    SUBEXIT();

    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreGetDefaultAttributesRc()
 *
 * Purpose:  This function exhaustively tests for most of the errors codes
 *           that may be returned by GreGetDefaultAttributes().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL) GreGetDefaultAttributes(HDC     hDC,
 *                                    LONG    lPrimType,
 *                                    ULONG   flAttrs,
 *                                    PBUNDLE paAttrs
 *                              )
 *    This Gre returns default values of the attributes indicated in flAttrs.
 *
 *     The return value is:
 *        TRUE              Success
 *        FALSE              Error
 *
 *     Possible returns from WinGetLastError:
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/02/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               10/02/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *        PMERR_INV_PRIMITIVE_TYPE
 *
 * Return Codes Not Tested Successfully Yet:
 *      None
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *
 * Note:
 *   a return codes was tested even though it is not listed
 *   under this function. It seemed logical for this function to return
 *   the following return codes:
 *     PMERR_INV_PRIMITIVE_TYPE
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreGetDefaultAttributes(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreGetDefaultAttributesRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreGetDefaultAttributesRc()";
  BOOL           fSuccess;          //Holds return code.
  HDC            hDC;
  DLINEBUNDLE    LineBundle={sizeof(LineBundle),0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER("Query hDC associated with PS",NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    SETEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreGetDefaultAttributes((HDC)NULL,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreGetDefaultAttributes((HDC)0x0fffffff,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreGetDefaultAttributes((HDC)0xffffffff,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/

    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);               //Make this PS busy.
    fSuccess=GreGetDefaultAttributes(hDC,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_HDC_BUSY,NULL);
    (VOID)FREEPS;                       //Kill BUSY thread.
    SUBEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_PRIMITIVE_TYPE *
    * error Code                                                     *
    *****************************************************************/
    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test1");

    // Pass an invalid Primitive Type ,say 0xffffffff, to
    // the function expecting an error

    fSuccess=GreGetDefaultAttributes(hDC,0xffffffffL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test2");

    // Pass an invalid Primitive Type ,say 0x00, to
    // the function expecting an error

    fSuccess=GreGetDefaultAttributes(hDC,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test3");

    // Pass an invalid Primitive Type ,say 0x0ffffffL, to
    // the function expecting an error

    fSuccess=GreGetDefaultAttributes(hDC,0x0ffffffL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test3");
    SUBEXIT();

    EXIT_MAIN(EntryName);
  }
  return;
}


 /****************************************************************************
 *\\ddd
 * Routine Name: GreSetCursorRc()
 *
 * Purpose:  This function exhaustively tests for most of the errors codes
 *           that may be returned by GreSetCursor().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL) GreSetCursor(HDC  hDC,
 *                               PPOINTL  pHotspot,
 *                               HBITMAP hbm
 *                              )
 *    This Gre sets the cursor bit map that defines the cursor shape.
 *
 *     The return value is:
 *        TRUE              Success
 *        FALSE              Error
 *
 *     Possible returns from WinGetLastError:
 *        PMERR_INV_HDC
 *        PMERR_INV_HBITMAP
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INV_COORRDINATE
 *        PMERR_INV_CURSOR_HBITMAP
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/02/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               10/02/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HDC
 *        PMERR_INV_HBITMAP
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INV_COORRDINATE
 *        PMERR_INV_CURSOR_HBITMAP
 *
 * Return Codes Not Tested Successfully Yet:
 *      None
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSetCursor(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreSetCursorRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreSetCursorRc()";
  BOOL           fSuccess;          //Holds return code.
  HBITMAP        hBM;                 //Bitmap handle.
  HDC            hDC;
  POINTL         HotSpot={0,0};
  BITMAPINFOHEADER BitMapHeader;      //bitmap info header
  PBITMAPINFO      pBitMapInfo;       //bitmap info structure
  BYTE             Temp_buffer[200];

  char abImage[700];
  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    // Initalize the Bitmap information structure
    // first we need to  allocate memory to hold RGB array
    // since  BITMAPINFO structure is large enough to hold
    // one RGB structure.
    pBitMapInfo=(PBITMAPINFO) Temp_buffer;
    pBitMapInfo->cbFix=sizeof(BITMAPINFOHEADER); // This always =12
    pBitMapInfo->cx=32;                 //Width
    pBitMapInfo->cy=32;                 //height
    pBitMapInfo->cPlanes=1;                      // one color plane
    pBitMapInfo->cBitCount=1;                    // 1-bit /pixel
    pBitMapInfo->argbColor[0].bBlue=
    pBitMapInfo->argbColor[0].bGreen=
    pBitMapInfo->argbColor[0].bRed=0x00;
    pBitMapInfo->argbColor[1].bBlue=
    pBitMapInfo->argbColor[1].bGreen=
    pBitMapInfo->argbColor[1].bRed=0xff;

    BitMapHeader.cbFix=sizeof(BITMAPINFOHEADER); // This always =12
    BitMapHeader.cx=32;                 //Width
    BitMapHeader.cy=32;                //height
    BitMapHeader.cPlanes=1;                      // one color plane
    BitMapHeader.cBitCount=1;                    // 1-bit /pixel

    SETENTER(EntryName,NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }
    if ((hBM=GreCreateBitmap(hDC,&BitMapHeader,
                             CBM_INIT,abImage
                            ,pBitMapInfo))==FALSE){
          CLEANUP("GreCreateBitmap()",EntryName);
          return ;
     }

    SETEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreSetCursor((HDC)NULL,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreSetCursor((HDC)0x0ffffffL,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreSetCursor((HDC)-1L,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_CURSOR_HBITMAP *
    * error code. Note that NULL bitmap is valid.                    *
    ******************************************************************/
    SUBENTER(PMERR_INV_CURSOR_BITMAP,NULL);
    fSuccess=GreSetCursor(hDC,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_CURSOR_BITMAP,NULL);
    SUBEXIT();

    /**********************************************************
    * The following Code will check for the PMERR_INV_HBITMAP *
    * error code. Note that NULL bitmap is valid.             *
    ***********************************************************/
    SUBENTER(PMERR_INV_HBITMAP,"Test1");
    // Pass an invalid HBITMAP ,say hBM=1 ,to the function expecting
    // an error.
    fSuccess=GreSetCursor(hDC,&HotSpot,(HBITMAP)1);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HBITMAP,"Test2");
    // Pass an invalid HBITMAP ,say hBM=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreSetCursor(hDC,&HotSpot,(HBITMAP)0x0fffffL);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HBITMAP,"Test3");
    // Pass an invalid HBITMAP ,say hBM=-1L,to the function expecting
    // an error
    fSuccess=GreSetCursor(hDC,&HotSpot,(HBITMAP)0xfffffL);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test3");
    SUBEXIT();

    /********************************************************************
    * The following Code will check for the PMERR_DEV_FUNC_NOT_INSTALLED *
    * error  code.                                                       *
    *********************************************************************/
    SUBENTER(PMERR_DEV_FUNC_NOT_INSTALLED,NULL);

    fSuccess=GreSetCursor(hDC,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_DEV_FUNC_NOT_INSTALLED,NULL);

    SUBEXIT();

    /*************************************************************
    * The following Code will check for the PMERR_INV_COORDINATE *
    * error code.                                                *
    **************************************************************/
    SUBENTER(PMERR_INV_COORDINATE,"Test1");
    HotSpot.x=0x0fffffff;

    fSuccess=GreSetCursor(hDC,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_COORDINATE,"Test1");

    SUBEXIT();

    SUBENTER(PMERR_INV_COORDINATE,"Test2");
    HotSpot.x=0;
    HotSpot.y=0x0fffffff;

    fSuccess=GreSetCursor(hDC,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_COORDINATE,"Test2");

    SUBEXIT();


    /*=================================================================
      The purpose of next code segment is do some house cleaning
      before exiting this routine. the following is done:
      1. De-select any bitmaps that are selected into memory Device
         Contexts or/and Un-tag any bitmaps that are used as pattern.
      2. Delete any bitmaps created within this function.
      3. Delete any Presentation Spaces created within this function.
      4. Delate any Device Contexts created within this function.

         Note that these steps must be followed as presented above,
         otherwise; most GPIs used here will fail.
    ===================================================================*/

    SETENTER(EntryName, "Cleaning up...");

    // Delete the bitmap handle.
    // device context
    if (GreDeleteBitmap(hBM)==FALSE){
         CLEANUP("GreDeleteBitmap()",EntryName);
         return;
     }


    SETEXIT();
    EXIT_MAIN(EntryName);
  }
  return;
}


/****************************************************************************
 *\\ddd
 * Routine Name: GreInitializeAttributesRc()
 *
 * Purpose:  This function exhaustively tests for most of the errors codes
 *           that may be returned by GreInitializeAttributes().
 *     The syntax used to call the Gre is :
 *
 *      (BOOL) GreInitializeAttributes(HDC hDC,
 *                                LONG    flOptions
 *                               )
 *     This Gre sets the current default attributes to the initial standard
 *     default values. It can be used to reset the current attribues to the
 *     current default values.
 *
 *     Success indicator:
 *
 *        TRUE             Success.
 *        FALSE            Error.
 *
 *     Possible returns from WinGetLastError:
 *       None Listed.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/02/91, PDVT ,Mike Abuzant
 *               -TestCase Design and conding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *        PMERR_INV_IN_AREA
 *        PMERR_INV_IN_PATH
 *
 * Return Codes Not Tested Successfully Yet:
 *        None
 * Subroutines Required:
 *      None.
 *
 * Limitations:
 *
 * NOTE: This function has no documented return errors. If feel that this
 *       function should at least return the above listed error codes.
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreInitializeAttributes(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/

VOID APIENTRY GreInitializeAttributesRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static  PCHAR    EntryName="GreInitializeAttributesRc()";
  BOOL             fSuccess;
  HDC              hDC;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER("Getting hDC associated with current PS",NULL);

    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
         CLEANUP("GpiQueryDevice()",EntryName);
         return;
    }

    SETEXIT();

   /* ***************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say hDC=NULL ,to the function expecting
    // an error

    fSuccess=GreInitializeAttributes((HDC)NULL,NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreInitializeAttributes((HDC)0x0fffffff,NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreInitializeAttributes((HDC)0xffffffff,NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/

    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);               //Make this PS busy.
    fSuccess=GreInitializeAttributes(hDC,INAT_DEFAULTATTRIBUTES);
    CHECKRC(fSuccess,PMERR_HDC_BUSY,NULL);
    (VOID)FREEPS;                       //Kill BUSY thread.
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_AREA        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_AREA,NULL);

    SETENTER(EntryName," Opening AREA bracket");

    if (GpiBeginArea(hPS,BA_WINDING)==FALSE){
       SHOWERROR("GpiBeginArea()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open area
    fSuccess=GreInitializeAttributes(hDC,INAT_DEFAULTATTRIBUTES);
    CHECKRC(fSuccess,PMERR_INV_IN_AREA,NULL);

    //Close The Area

    SETENTER(EntryName," Closing AREA bracket");

    if (GpiEndArea(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndArea()",EntryName);
    }
    SETEXIT();

    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_PATH        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_PATH,NULL);

    SETENTER(EntryName," Opening Path bracket");
    if (GpiBeginPath(hPS,1L)==FALSE){
       SHOWERROR("GpiBeginPath()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open path
    fSuccess=GreInitializeAttributes(hDC,INAT_DEFAULTATTRIBUTES);
    CHECKRC(fSuccess,PMERR_INV_IN_PATH,NULL);

    //Close The Area
    SETENTER(EntryName," Closing Path bracket");

    if (GpiEndPath(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndPath()",EntryName);
    }

    SETEXIT();

    SUBEXIT();

    EXIT_MAIN(EntryName);

  }
  return;
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetDefaultAttributesRc()
 *
 * Purpose:  This function exhaustively tests for most of the errors codes
 *           that may be returned by GreSetDefaultAttributes().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL) GreSetDefaultAttributes(HDC    hDC,
 *                                    ULONG    lPrimTypeType,
 *                                    ULONG    flAttribute,
 *                                    PBUNDLE   pAttrs
 *                                  )
 *    This Gre sets the specified default attributes in line,pattern,character,
 *    and maker bundles.
 *
 *     The return value is:
 *        TRUE              Success
 *        FALSE              Error
 *
 *     Possible returns from WinGetLastError:
 *
 *     PMERR_HDC_BUSY
 *     PMERR_DEV_FUNC_NOT_INSTALLED
 *     PMERR_COORDINATE_OVERFLOW
 *     PMERR_INV_PRIMITIVE_TYPE
 *     PMERR_INV_SETID
 *     PMERR_INV_SET_CHAR_ATTR
 *     PMERR_INV_MARKER_CHAR_ATTR
 *     PMERR_INV_PATTERN_CHAR_ATTR
 *     PMERR_INV_MARKER_SYMBOL_ATTR
 *     PMERR_UNSUPPORTED_ATTR
 *     PMERR_INV_MIX_ATTR
 *     PMERR_INV_BACKGROUND_MIX_ATTR
 *     PMERR_INV_PATTREN_REF_PT_ATTR
 *     PMERR_INV_LINE_WIDTH_ATTR
 *     PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *     PMERR_INV_LINE_TYPE_ATTR
 *     PMERR_INV_LINE_JOIN_ATTR
 *     PMERR_INV_LINE_END_ATTR
 *     PMERR_INV_CHAR_MODE_ATTR
 *     PMERR_INV_CHAR_SHEAR_ATTR
 *     PMERR_INV_CHAR_DIRECTION_ATTR
 *     PMERR_UNSUPPORTED_ATTR_VALUE
 *     PMERR_INSUFFICIENT_MEMORY
 *     PMERR_INV_LENGTH_OR_COUNT
 *     PMERR_INV_CODEPAGE
 *     PMERR_EXCEEDS_MAX_SEG_LENGTH
 *     PMERR_INV_HDC
 *     PMERR_INV_COORD_SPACE
 *     PMERR_INV_PATTERN_SET_FONT
 *     PMERR_INV_COLOR_ATTR
 *     PMERR_INV_BACKGROUND_COL_ATTR
 *
 *
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/05/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               10/05/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *               10/13/91, PDVT ,Mike Abuzant
 *               -Reproduced more error codes.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *
 *     PMERR_INV_IN_AREA
 *     PMERR_INV_IN_PATH
 *     PMERR_HDC_BUSY
 *     PMERR_INV_PRIMITIVE_TYPE
 *     PMERR_INV_SET_CHAR_ATTR
 *     PMERR_INV_MARKER_CHAR_ATTR
 *     PMERR_INV_PATTERN_CHAR_ATTR
 *     PMERR_INV_MARKER_SYMBOL_ATTR
 *     PMERR_UNSUPPORTED_ATTR
 *     PMERR_INV_MIX_ATTR
 *     PMERR_INV_BACKGROUND_MIX_ATTR
 *     PMERR_INV_PATTREN_REF_PT_ATTR
 *     PMERR_INV_LINE_WIDTH_ATTR
 *     PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *     PMERR_INV_LINE_TYPE_ATTR
 *     PMERR_INV_LINE_JOIN_ATTR
 *     PMERR_INV_LINE_END_ATTR
 *     PMERR_INV_CHAR_MODE_ATTR
 *     PMERR_INV_CHAR_SHEAR_ATTR
 *     PMERR_INV_CHAR_DIRECTION_ATTR
 *     PMERR_INV_HDC
 *     PMERR_INV_PATTERN_SET_FONT
 *     PMERR_INV_COLOR_ATTR
 *     PMERR_INV_BACKGROUND_COL_ATTR
 *
 * Return Codes Not Tested Successfully Yet:
 *
 *     PMERR_DEV_FUNC_NOT_INSTALLED
 *     PMERR_COORDINATE_OVERFLOW
 *     PMERR_UNSUPPORTED_ATTR_VALUE
 *     PMERR_INSUFFICIENT_MEMORY
 *     PMERR_INV_LENGTH_OR_COUNT
 *     PMERR_INV_CODEPAGE
 *     PMERR_EXCEEDS_MAX_SEG_LENGTH
 *     PMERR_INV_COORD_SPACE
 *     PMERR_INV_PATTERN_SET_FONT
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   Almost half of the error codes listed above will not be returned by this
 *   function. The documenatation states that the GreSetDefaultAttribute()
 *   does not check validity of most of the given attributes instead,
 *   attributes checking is done when calling functions that make use of
 *   the attributes.
 *
 *  NOTE:
 *   Two return codes was tested even though it is not listed
 *   under this function. It seemed logical for this function to return
 *   the following return codes:
 *     PMERR_INV_IN_PATH
 *     PMERR_INV_IN_AREA
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSetDefaultAttributes(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetDefaultAttributesRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreSetDefaultAttributesRc()";
  BOOL           fSuccess;          //Holds return code.
  HDC            hDC;

  IMAGEBUNDLE   ImageBundle={sizeof(ImageBundle),0};
  LINEBUNDLE    LineBundle={sizeof(LineBundle),0};
  CHARBUNDLE    CharBundle={sizeof(CharBundle),0};
  MARKERBUNDLE  MarkerBundle={sizeof(MarkerBundle),0};
  AREABUNDLE    AreaBundle={sizeof(AreaBundle),0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER("Query hDC associated with PS",NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    SETEXIT();

    LineBundle.lColor=CLR_RED;
    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreSetDefaultAttributes((HDC)NULL,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreSetDefaultAttributes((HDC)0x0fffffff,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreSetDefaultAttributes((HDC)0xfffffff,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/

    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);               //Make this PS busy.
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_HDC_BUSY,NULL);
    (VOID)FREEPS;                       //Kill BUSY thread.
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_PATTERN_SET_ATTR    *
    * error Code                                                          *
    ***********************************************************************/

    AreaBundle.usSet=0x1fff;
    SUBENTER(PMERR_INV_PATTERN_SET_ATTR  ,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_AREA,ABB_SET,&AreaBundle);
    CHECKRC(fSuccess,PMERR_INV_PATTERN_SET_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_MARKER_SET_ATTR     *
    * error Code                                                          *
    ***********************************************************************/

    MarkerBundle.usSet=0x1fff;
    SUBENTER(PMERR_MARKER_SET_ATTR  ,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_MARKER,MBB_SET,&MarkerBundle);
    CHECKRC(fSuccess,PMERR_INV_MARKER_SET_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_PATTERN_REF_PT_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    AreaBundle.ptlRefPoint.x=0x0fffffffL;
    AreaBundle.ptlRefPoint.y=0x0fffffffL;

    SUBENTER(PMERR_INV_PATTERN_REF_PT_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_AREA,ABB_REF_POINT,&AreaBundle);
    CHECKRC(fSuccess,PMERR_INV_PATTERN_REF_PT_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_MODE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.usPrecision=0x1234;
    SUBENTER(PMERR_INV_CHAR_MODE_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_MODE,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_MODE_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_SHEAR_ATTR     *
    * error Code                                                          *
    ***********************************************************************/
    CharBundle.usPrecision=CM_MODE3;
    CharBundle.ptlShear.x=0xffffffffL;
    CharBundle.ptlShear.y=0x0L;
    SUBENTER(PMERR_INV_CHAR_SHEAR_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_MODE|CBB_SHEAR,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_SHEAR_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_MARKER_SYMBOL_ATTR  *
    * error Code                                                          *
    ***********************************************************************/

    MarkerBundle.usSymbol=0x7fff;
    SUBENTER(PMERR_INV_MARKER_SYMBOL_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_MARKER,MBB_SYMBOL,&MarkerBundle);
    CHECKRC(fSuccess,PMERR_INV_MARKER_SYMBOL_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_DIRECTION_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.usDirection=0x1234;
    SUBENTER(PMERR_INV_CHAR_DIRECTION_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_DIRECTION,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_DIRECTION_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_COLOR_ATTR          *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.lColor=0x0fffffffL;
    SUBENTER(PMERR_INV_COLOR_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_COLOR,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_COLOR_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_BACKGROUND_COL_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.lBackColor=0x0fffffffL;
    SUBENTER(PMERR_INV_BACKGROUND_COL_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_BACK_COLOR,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_COL_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_SET_ATTR       *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.usSet=0xffffu;
    SUBENTER(PMERR_INV_CHAR_SET_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_SET,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_SET_ATTR,NULL);
    SUBEXIT();


    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_AREA        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_AREA,NULL);
    CharBundle.lColor=CLR_RED;

    SETENTER(EntryName," Opening AREA bracket");

    if (GpiBeginArea(hPS,BA_WINDING)==FALSE){
       SHOWERROR("GpiBeginArea()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open area
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_IN_AREA,NULL);

    //Close The Area

    SETENTER(EntryName," Closing AREA bracket");

    if (GpiEndArea(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndArea()",EntryName);
    }
    SETEXIT();

    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_PATH        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_PATH,NULL);

    SETENTER(EntryName," Opening Path bracket");
    if (GpiBeginPath(hPS,1L)==FALSE){
       SHOWERROR("GpiBeginPath()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open path
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_IN_PATH,NULL);

    //Close The Area
    SETENTER(EntryName," Closing Path bracket");

    if (GpiEndPath(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndPath()",EntryName);
    }
    SETEXIT();

    SUBEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_UNSUPPORTED_ATTR  *
    * error Code                                                     *
    *****************************************************************/
    SUBENTER(PMERR_UNSUPPORTED_ATTR,"Test1");

    // Pass an invalid Primitive Type ,say 0xffffffff, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,0xffffffffL,&LineBundle);
    CHECKRC(fSuccess,PMERR_UNSUPPORTED_ATTR,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_UNSUPPORTED_ATTR,"Test2");

    // Pass an invalid Primitive Type ,say 0x0ffffffL, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,0x0fffffffL,&LineBundle);
    CHECKRC(fSuccess,PMERR_UNSUPPORTED_ATTR,"Test2");
    SUBEXIT();


    /****************************************************************
    * The following Code will check for the PMERR_INV_PRIMITIVE_TYPE *
    * error Code                                                     *
    *****************************************************************/
    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test1");

    // Pass an invalid Primitive Type ,say 0xffffffff, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,0xffffffff,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test2");

    // Pass an invalid Primitive Type ,say 0x00, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test3");

    // Pass an invalid Primitive Type ,say 0x0ffffffL, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,0x0fffffffL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test3");
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_MIX_ATTR            *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.usMixMode=0x1234;
    SUBENTER(PMERR_INV_MIX_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_MIX_MODE,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_MIX_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_BACKGROUND_MIX_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.usBackMixMode=0x1234;
    SUBENTER(PMERR_INV_BACKGROUND_MIX_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_BACK_MIX_MODE,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_MIX_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_END_TYPE_ATTR       *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.usEnd=0x1234;
    SUBENTER(PMERR_INV_LINE_END_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_END,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_END_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_JOIN_TYPE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.usJoin=0x1234;
    SUBENTER(PMERR_INV_LINE_JOIN_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_JOIN,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_JOIN_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_LINE_TYPE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.usType=0x1234;
    SUBENTER(PMERR_INV_LINE_TYPE_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_TYPE,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_TYPE_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_GEOM_LINE_WIDTH_ATTR*
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.lGeomWidth=MAKEFIXED(-1234,-7474);
    SUBENTER(PMERR_INV_GEOM_LINE_WIDTH_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_GEOM_WIDTH,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_GEOM_LINE_WIDTH_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_LINE_WIDTH_ATTR     *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.fxWidth=-34l;
    SUBENTER(PMERR_INV_LINE_WIDTH_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_WIDTH,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_WIDTH_ATTR,NULL);
    SUBEXIT();

    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetAttributesRc()
 *
 * Purpose:  This function exhaustively tests for most of the errors codes
 *           that may be returned by GreSetDefualtAttributes().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL) GreSetAttributes(HDC    hDC,
 *                                    ULONG    lPrimTypeType,
 *                                    ULONG    flDefAttribute,
 *                                    ULONG    flAttribute,
 *                                    PBUNDLE   pAttrs
 *                                  )
 *    This Gre sets the specified attributes in line,pattern,character,
 *    and maker bundles.
 *
 *     The return value is:
 *        TRUE              Success
 *        FALSE              Error
 *
 *     Possible returns from WinGetLastError:
 *
 *     PMERR_HDC_BUSY
 *     PMERR_DEV_FUNC_NOT_INSTALLED
 *     PMERR_COORDINATE_OVERFLOW
 *     PMERR_INV_PRIMITIVE_TYPE
 *     PMERR_INV_SETID
 *     PMERR_INV_SET_CHAR_ATTR
 *     PMERR_INV_MARKER_CHAR_ATTR
 *     PMERR_INV_PATTERN_CHAR_ATTR
 *     PMERR_INV_MARKER_SYMBOL_ATTR
 *     PMERR_UNSUPPORTED_ATTR
 *     PMERR_INV_MIX_ATTR
 *     PMERR_INV_BACKGROUND_MIX_ATTR
 *     PMERR_INV_PATTREN_REF_PT_ATTR
 *     PMERR_INV_LINE_WIDTH_ATTR
 *     PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *     PMERR_INV_LINE_TYPE_ATTR
 *     PMERR_INV_LINE_JOIN_ATTR
 *     PMERR_INV_LINE_END_ATTR
 *     PMERR_INV_CHAR_MODE_ATTR
 *     PMERR_INV_CHAR_SHEAR_ATTR
 *     PMERR_INV_CHAR_DIRECTION_ATTR
 *     PMERR_UNSUPPORTED_ATTR_VALUE
 *     PMERR_INSUFFICIENT_MEMORY
 *     PMERR_INV_LENGTH_OR_COUNT
 *     PMERR_INV_CODEPAGE
 *     PMERR_EXCEEDS_MAX_SEG_LENGTH
 *     PMERR_INV_HDC
 *     PMERR_INV_COORD_SPACE
 *     PMERR_INV_PATTERN_SET_FONT
 *     PMERR_INV_COLOR_ATTR
 *     PMERR_INV_BACKGROUND_COL_ATTR
 *
 *
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/07/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               10/07/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *               10/13/91, PDVT ,Mike Abuzant
 *               -Reproduced more error code.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *        None
 * Return Codes Successfully Tested:
 *
 *     PMERR_INV_IN_AREA
 *     PMERR_INV_IN_PATH
 *     PMERR_HDC_BUSY
 *     PMERR_INV_PRIMITIVE_TYPE
 *     PMERR_INV_SET_CHAR_ATTR
 *     PMERR_INV_MARKER_CHAR_ATTR
 *     PMERR_INV_PATTERN_CHAR_ATTR
 *     PMERR_INV_MARKER_SYMBOL_ATTR
 *     PMERR_UNSUPPORTED_ATTR
 *     PMERR_INV_MIX_ATTR
 *     PMERR_INV_BACKGROUND_MIX_ATTR
 *     PMERR_INV_PATTREN_REF_PT_ATTR
 *     PMERR_INV_LINE_WIDTH_ATTR
 *     PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *     PMERR_INV_LINE_TYPE_ATTR
 *     PMERR_INV_LINE_JOIN_ATTR
 *     PMERR_INV_LINE_END_ATTR
 *     PMERR_INV_CHAR_MODE_ATTR
 *     PMERR_INV_CHAR_SHEAR_ATTR
 *     PMERR_INV_CHAR_DIRECTION_ATTR
 *     PMERR_INV_HDC
 *     PMERR_INV_PATTERN_SET_FONT
 *     PMERR_INV_COLOR_ATTR
 *     PMERR_INV_BACKGROUND_COL_ATTR
 *
 * Return Codes Not Tested Successfully Yet:
 *
 *     PMERR_DEV_FUNC_NOT_INSTALLED
 *     PMERR_COORDINATE_OVERFLOW
 *     PMERR_UNSUPPORTED_ATTR_VALUE
 *     PMERR_INSUFFICIENT_MEMORY
 *     PMERR_INV_LENGTH_OR_COUNT
 *     PMERR_INV_CODEPAGE
 *     PMERR_EXCEEDS_MAX_SEG_LENGTH
 *     PMERR_INV_COORD_SPACE
 *     PMERR_INV_PATTERN_SET_FONT
 *
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   Almost half of the error codes listed above will not be returned by this
 *   function. The documenatation states that the GreSetAttribute()
 *   does not check validity of most of the given attributes instead,
 *   attributes checking is done when calling functions that make use of
 *   the attributes.
 *
 *  NOTE:
 *   Two return codes was tested even though it is not listed
 *   under this function. It seemed logical for this function to return
 *   the following return codes:
 *     PMERR_INV_IN_PATH
 *     PMERR_INV_IN_AREA
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSetAttributes(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetAttributesRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){
  static  PCHAR  EntryName="GreSetAttributesRc()";
  BOOL           fSuccess;          //Holds return code.
  HDC            hDC;

  IMAGEBUNDLE   ImageBundle={sizeof(ImageBundle),0};
  LINEBUNDLE    LineBundle={sizeof(LineBundle),0};
  CHARBUNDLE    CharBundle={sizeof(CharBundle),0};
  MARKERBUNDLE  MarkerBundle={sizeof(MarkerBundle),0};
  AREABUNDLE    AreaBundle={sizeof(AreaBundle),0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER("Query hDC associated with PS",NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    SETEXIT();

    LineBundle.lColor=CLR_RED;
    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreSetAttributes((HDC)NULL,PRIM_LINE,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreSetAttributes((HDC)0x0fffffff,PRIM_LINE,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreSetAttributes((HDC)0xfffffff,PRIM_LINE,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/

    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);               //Make this PS busy.
    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_HDC_BUSY,NULL);
    (VOID)FREEPS;                       //Kill BUSY thread.
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_PATTERN_SET_ATTR    *
    * error Code                                                          *
    ***********************************************************************/

    AreaBundle.usSet=0x1fff;
    SUBENTER(PMERR_INV_PATTERN_SET_ATTR  ,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_AREA,(LONG)NULL,ABB_SET,&AreaBundle);
    CHECKRC(fSuccess,PMERR_INV_PATTERN_SET_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_MARKER_SET_ATTR     *
    * error Code                                                          *
    ***********************************************************************/

    MarkerBundle.usSet=0x1fff;
    SUBENTER(PMERR_MARKER_SET_ATTR  ,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_MARKER,(LONG)NULL,MBB_SET,&MarkerBundle);
    CHECKRC(fSuccess,PMERR_INV_MARKER_SET_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_PATTERN_REF_PT_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    AreaBundle.ptlRefPoint.x=0x0fffffffL;
    AreaBundle.ptlRefPoint.y=0x0fffffffL;

    SUBENTER(PMERR_INV_PATTERN_REF_PT_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_AREA,(LONG)NULL,ABB_REF_POINT,&AreaBundle);
    CHECKRC(fSuccess,PMERR_INV_PATTERN_REF_PT_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_MODE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.usPrecision=0x1234;
    SUBENTER(PMERR_INV_CHAR_MODE_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_CHAR,(LONG)NULL,CBB_MODE,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_MODE_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_SHEAR_ATTR     *
    * error Code                                                          *
    ***********************************************************************/
    CharBundle.usPrecision=CM_MODE3;
    CharBundle.ptlShear.x=0xffffffffL;
    CharBundle.ptlShear.y=0x0L;
    SUBENTER(PMERR_INV_CHAR_SHEAR_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_CHAR,(LONG)NULL,CBB_MODE|CBB_SHEAR,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_SHEAR_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_MARKER_SYMBOL_ATTR  *
    * error Code                                                          *
    ***********************************************************************/

    MarkerBundle.usSymbol=0x7fff;
    SUBENTER(PMERR_INV_MARKER_SYMBOL_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_MARKER,(LONG)NULL,MBB_SYMBOL,&MarkerBundle);
    CHECKRC(fSuccess,PMERR_INV_MARKER_SYMBOL_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_DIRECTION_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.usDirection=0x1234;
    SUBENTER(PMERR_INV_CHAR_DIRECTION_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_CHAR,(LONG)NULL,CBB_DIRECTION,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_DIRECTION_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_COLOR_ATTR          *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.lColor=0x0fffffffL;
    SUBENTER(PMERR_INV_COLOR_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_CHAR,(LONG)NULL,CBB_COLOR,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_COLOR_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_BACKGROUND_COL_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.lBackColor=0x0fffffffL;
    SUBENTER(PMERR_INV_BACKGROUND_COL_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_CHAR,(LONG)NULL,CBB_BACK_COLOR,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_COL_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_SET_ATTR       *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.usSet=0xffffu;
    SUBENTER(PMERR_INV_CHAR_SET_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_CHAR,(LONG)NULL,CBB_SET,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_SET_ATTR,NULL);
    SUBEXIT();


    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_AREA        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_AREA,NULL);
    CharBundle.lColor=CLR_RED;

    SETENTER(EntryName," Opening AREA bracket");

    if (GpiBeginArea(hPS,BA_WINDING)==FALSE){
       SHOWERROR("GpiBeginArea()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open area
    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,LBB_GEOM_WIDTH,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_IN_AREA,NULL);

    //Close The Area

    SETENTER(EntryName," Closing AREA bracket");

    if (GpiEndArea(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndArea()",EntryName);
    }
    SETEXIT();

    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_PATH        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_PATH,NULL);

    SETENTER(EntryName," Opening Path bracket");
    if (GpiBeginPath(hPS,1L)==FALSE){
       SHOWERROR("GpiBeginPath()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open path
    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,LBB_GEOM_WIDTH,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_IN_PATH,NULL);

    //Close The Area
    SETENTER(EntryName," Closing Path bracket");

    if (GpiEndPath(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndPath()",EntryName);
    }
    SETEXIT();

    SUBEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_UNSUPPORTED_ATTR  *
    * error Code                                                     *
    *****************************************************************/
    SUBENTER(PMERR_UNSUPPORTED_ATTR,"Test1");

    // Pass an invalid Primitive Type ,say 0xffffffff, to
    // the function expecting an error

    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,0xffffffffL,&LineBundle);
    CHECKRC(fSuccess,PMERR_UNSUPPORTED_ATTR,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_UNSUPPORTED_ATTR,"Test2");

    // Pass an invalid Primitive Type ,say 0x0ffffffL, to
    // the function expecting an error

    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,0x0fffffffL,&LineBundle);
    CHECKRC(fSuccess,PMERR_UNSUPPORTED_ATTR,"Test2");
    SUBEXIT();


    /****************************************************************
    * The following Code will check for the PMERR_INV_PRIMITIVE_TYPE *
    * error Code                                                     *
    *****************************************************************/
    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test1");

    // Pass an invalid Primitive Type ,say 0xffffffff, to
    // the function expecting an error

    fSuccess=GreSetAttributes(hDC,0xffffffff,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test2");

    // Pass an invalid Primitive Type ,say 0x00, to
    // the function expecting an error

    fSuccess=GreSetAttributes(hDC,(LONG)NULL,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test3");

    // Pass an invalid Primitive Type ,say 0x0ffffffL, to
    // the function expecting an error

    fSuccess=GreSetAttributes(hDC,0x0fffffffL,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test3");
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_MIX_ATTR            *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.usMixMode=0x1234;
    SUBENTER(PMERR_INV_MIX_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,LBB_MIX_MODE,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_MIX_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_BACKGROUND_MIX_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.usBackMixMode=0x1234;
    SUBENTER(PMERR_INV_BACKGROUND_MIX_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_CHAR,(LONG)NULL,CBB_BACK_MIX_MODE,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_MIX_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_END_TYPE_ATTR       *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.usEnd=0x1234;
    SUBENTER(PMERR_INV_LINE_END_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,LBB_END,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_END_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_JOIN_TYPE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.usJoin=0x1234;
    SUBENTER(PMERR_INV_LINE_JOIN_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,LBB_JOIN,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_JOIN_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_LINE_TYPE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.usType=0x1234;
    SUBENTER(PMERR_INV_LINE_TYPE_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,LBB_TYPE,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_TYPE_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_GEOM_LINE_WIDTH_ATTR*
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.lGeomWidth=MAKEFIXED(-1234,-7474);
    SUBENTER(PMERR_INV_GEOM_LINE_WIDTH_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,LBB_GEOM_WIDTH,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_GEOM_LINE_WIDTH_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_LINE_WIDTH_ATTR     *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.fxWidth=-34l;
    SUBENTER(PMERR_INV_LINE_WIDTH_ATTR,NULL);
    fSuccess=GreSetAttributes(hDC,PRIM_LINE,(LONG)NULL,LBB_WIDTH,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_WIDTH_ATTR,NULL);
    SUBEXIT();

    EXIT_MAIN(EntryName);
  }
  return;

}

 /****************************************************************************
 *\\ddd
 * Routine Name: GreInvalidateVisRegionRc()
 *
 * Purpose:  This function exhaustively tests for most of the errors codes
 *           that may be returned by GreInvalidateVisRegion().
 *
 *
 *     Possible returns from WinGetLastError:
 *
 *      None.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/09/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               10/02/91, PDVT ,Mike Abuzant
 *               -TestCase Design .
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *      None.
 *
 * Return Codes Not Tested Successfully Yet:
 *      None
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   This entry point is not implemented due to the fact that this function
 *   is not documented.
 *
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess=GreInvalidateVisRegion(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreInvalidateVisRegionRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreInvalidateVisRegionRc()";

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);

    DeletePrinterOutput(EntryName);

    EXIT_MAIN(EntryName);
  }
  return;
}
