/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GREATTR.C")
#pragma subtitle("IBM Confidential")

#define INCL_DOS
#define INCL_WIN
#define INCL_GPI
#define INCL_GREALL
#define INCL_ERRORS
#define INCL_DDIMISC
#define INCL_DDIBUNDLES
#define INCL_DEV

#include <string.h>
#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <pmddi.h>
#include <pmfont.h>

#include "DTTDLLDF.h"   // test case defines
#include "std_util.h"    // map and write functions
#include "greattr.h"    // function prototypes

VOID display_text(HPS hPS, LONG column, LONG row, CHAR str_msg[]);
/****************************************************************************
 *\\ddd
 * Subtask Name: PDVT
 *
 * Module name:  GREATTR
 *
 * Purpose:  This module is called by DTT to test the Gre Attribute APIs
 *
 * Revision Log: 10/24/90, PDVT, Donna Williams
 *                     - initial creation
 *               01/22/91, pdvt, markrichardson
 *                     - removed the setupdc/getdcoriginExh entrypoint
 *                       and placed these tests in their respective RC
 *                       entrypoints
 *
 * SRS References: Section 6.5
 *
 * Contents: InitTest,
 *           GreDeviceSetGlobalAttributesExh,
 *           GreDeviceSetGetAttrExh,
 *           GreNotifyClipChangeExh,
 *           GreNotifyTransformChangeExh,
 *           GreDeviceSetGlobalAttributesRC,
 *           GreDeviceGetAttributesRC,
 *           GreDeviceSetAttributesRC,
 *           GreNotifyClipChangeRC,
 *           GreNotifyTransformChangeRC,
 *           GreSetupDCRC,
 *           GreGetDCOriginRC
 *
 * Design Overview:
 *        The GRE Attr testcase involves testing of the following GREs:
 *           GreDeviceSetGlobalAttributes,
 *           GreDeviceGetAttributes,
 *           GreDeviceSetAttributes,
 *           GreNotifyClipChange,
 *           GreNotifyTransformChange,
 *           GreSetupDC,
 *           GreGetDCOrigin.
 *        The individual testing of each GRE will include a functional
 *        test and an exhaustive test.
 *
 * Limitations: As Noted For Each Function.
 *
 *\\end
****************************************************************************/
/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose: Initialize function name and address array for DTT
 *
 * System Requirements: None
 *
 * Revision Log: 10/24/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs:  sel            - pass thru parameter for other function calls
 *          pInitInfo      - pointer to info array
 *
 * Outputs: pInitInfo      - updated with all entry point names and addresses
 *
 * Subroutines Required:  InitTestSegVars()
 *
 * Limitations:  None.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *   for every entry point{
 *      strcpy(pInitInfo->test_names[0],"name_of_entry_point");
 *      pInitInfo->test_addrs[0]=addr_of_entry_point;
 *\\end
 ***************************************************************************/
VOID APIENTRY InitTest(PVOID sel, PTESTINITINFO pInitInfo)
  {
    pInitInfo->ntests = NTESTS;

    strcpy(pInitInfo->group_name,"GRE Attr");

    strcpy(pInitInfo->test_names[0],"GreDeviceSetGlobalAttributesExh");
    pInitInfo->test_addrs[0]=GreDeviceSetGlobalAttributesExh;
    strcpy(pInitInfo->test_names[1],"GreDeviceSetGetAttrExh");
    pInitInfo->test_addrs[1]=GreDeviceSetGetAttrExh;
    strcpy(pInitInfo->test_names[2],"GreSetLineAttrExh");
    pInitInfo->test_addrs[2]=GreSetLineAttrExh;
    strcpy(pInitInfo->test_names[3],"GreNotifyClipChangeExh");
    pInitInfo->test_addrs[3]=GreNotifyClipChangeExh;
    strcpy(pInitInfo->test_names[4],"GreNotifyTransformChangeExh");
    pInitInfo->test_addrs[4]=GreNotifyTransformChangeExh;

    strcpy(pInitInfo->test_names[5],"GreDeviceSetGlobalAttributesRC");
    pInitInfo->test_addrs[5]=GreDeviceSetGlobalAttributesRC;
    strcpy(pInitInfo->test_names[6],"GreDeviceGetAttributesRC");
    pInitInfo->test_addrs[6]=GreDeviceGetAttributesRC;
    strcpy(pInitInfo->test_names[7],"GreDeviceSetAttributesRC");
    pInitInfo->test_addrs[7]=GreDeviceSetAttributesRC;
    strcpy(pInitInfo->test_names[8],"GreNotifyClipChangeRC");
    pInitInfo->test_addrs[8]=GreNotifyClipChangeRC;
    strcpy(pInitInfo->test_names[9],"GreNotifyTransformChangeRC");
    pInitInfo->test_addrs[9]=GreNotifyTransformChangeRC;
    strcpy(pInitInfo->test_names[10],"GreSetupDCRC");
    pInitInfo->test_addrs[10]=GreSetupDCRC;
    strcpy(pInitInfo->test_names[11],"GreGetDCOriginRC");
    pInitInfo->test_addrs[11]=GreGetDCOriginRC;

    InitTestSegVars(sel);
 }
/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetGlobalAttributesExh()
 *
 * Purpose: This entry point executes the functionality test for GreDeviceSetGlobalAttributes
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 10/24/90, PDVT, Donna Williams
 *                     - initial creation
 *               12/14/90, PDVT, Donna Williams
 *                     - design implementation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: None.
 *
 * Subroutines Required:
 *          CWRITELOG(),
 *          LOG_SIGNATURE,
 *          TEST_SIGNATURE(),
 *          BADERROR
 *
 * Limitations: NONE.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *   // for Foreground color attribute
 *   GreDeviceSetGlobalAttribute(hDC,ATYPE_COLOR,lAttribute,0);
 *   // for Background color attribute
 *   GreDeviceSetGlobalAttribute(hDC,ATYPE_BACK_COLOR,lAttribute,0);
 *   // for foreground mix attribute
 *   GreDeviceSetGlobalAttribute(hDC,ATYPE_MIX_MODE,lAttribute,0);
 *   // for background mix attribute
 *   GreDeviceSetGlobalAttribute(hDC,ATYPE_BACK_MIX_MODE,lAttribute,0);
 *   // display images on screen showing that attributes have been set
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceSetGlobalAttributesExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC    hDC;
  POINTL ptl,
         ptlBox[7];
  CHAR   out_string[257];
  INT    i;
  SIZEL  szImage;
  LONG   lIndex,
         lErrorCount;
  BYTE   abImage[2000];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    lErrorCount = 0;
    TEST_SIGNATURE(0,0);
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE,0,"+ee+ GreDeviceSetGlobalAttributeExh - Entry.\n");
    hDC = set_delta(hPS);

    /*
     * Test functionality of GreDeviceSetGlobalAttribute
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetGlobalAttribute Functionality Test  - Entry.\n");
    if(!GreDeviceSetGlobalAttribute(hDC,ATYPE_COLOR, CLR_BLUE, 0L) ){
        lErrorCount += BADERROR("GreDeviceSetGlobalAttribute Failed",FALSE,NULL);
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetGlobalAttribute Functionality Test  - Exit.\n");

    /*
     * Test functionality of GreDeviceSetGlobalAttribute
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetGlobalAttribute Functionality Test  - Entry.\n");
    if(!GreDeviceSetGlobalAttribute(hDC,ATYPE_BACK_COLOR, CLR_GREEN, 0L) ){
        lErrorCount += BADERROR("GreDeviceSetGlobalAttribute Failed",FALSE,NULL);
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetGlobalAttribute Functionality Test  - Exit.\n");

    /*
     * Test functionality of GreDeviceSetGlobalAttribute
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetGlobalAttribute Functionality Test  - Entry.\n");
    if(!GreDeviceSetGlobalAttribute(hDC,ATYPE_MIX_MODE, FM_OVERPAINT, 0L) ){
        lErrorCount += BADERROR("GreDeviceSetGlobalAttribute Failed",FALSE,NULL);
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetGlobalAttribute Functionality Test  - Exit.\n");

    /*
     * Test functionality of GreDeviceSetGlobalAttribute
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetGlobalAttribute Functionality Test  - Entry.\n");
    if(!GreDeviceSetGlobalAttribute(hDC,ATYPE_BACK_MIX_MODE, BM_OVERPAINT, 0L) ){
        lErrorCount += BADERROR("GreDeviceSetGlobalAttribute Failed",FALSE,NULL);
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetGlobalAttribute Functionality Test  - Exit.\n");

    /*
     * display attributes on screen by drawing a line
     */
    ptl.x = MapX(13 * delta_x / 2);
    ptl.y = MapY(2 * delta_y);
    GpiMove(hPS, &ptl);
    ptl.y = MapY(5 * delta_y);
    GpiLine(hPS, &ptl);
    ptl.x = MapX(15 * delta_x / 2);
    GpiLine(hPS, &ptl);
    ptl.y = MapY(2 * delta_y);
    GpiLine(hPS, &ptl);
    ptl.x = MapX(13 * delta_x / 2);
    GpiLine(hPS, &ptl);
    /*
     * initalize array for L-shape
     */
    ptlBox[0].x = delta_x;
    ptlBox[0].y = 5 * delta_y;
    ptlBox[1].x = delta_x;
    ptlBox[1].y = 2 * delta_y;
    ptlBox[2].x = 3 * delta_x;
    ptlBox[2].y = 2 * delta_y;
    ptlBox[3].x = 3 * delta_x;
    ptlBox[3].y = 3 * delta_y;
    ptlBox[4].x = 2 * delta_x;
    ptlBox[4].y = 3 * delta_y;
    ptlBox[5].x = 2 * delta_x;
    ptlBox[5].y = 5 * delta_y;
    ptlBox[6].x = delta_x;
    ptlBox[6].y = 5 * delta_y;
    /*
     * define a path and draw a L-shape within 5 grid squares
     */
    GpiBeginPath(hPS,1L);
    for(i=0;i<=6;++i)
       MapPt(ptlBox[i]);
    GpiMove(hPS, &ptlBox[0]);
    GpiPolyLine(hPS, 7l, ptlBox);
    GpiCloseFigure(hPS);
    GpiEndPath(hPS);
    GpiFillPath(hPS,1L, FPATH_WINDING);
    /*
     * display a marker on the screen
     */
    ptl.x = 4 * delta_x;
    ptl.y = 3 * delta_y;
    MapPt(ptl);
    GpiMarker(hPS, &ptl);
    /*
     * The following code segment will draw a bitmap image
     */
    szImage.cx = szImage.cy = 104;
    ptl.x = 5 * delta_x;
    ptl.y = 4 * delta_y;
    MapPt(ptl);
    memset(abImage,0x00,sizeof(abImage));  // Set all bits to OFF .

    for (lIndex=0; lIndex <104; lIndex+=2)
         memset(abImage+lIndex*13,0xFF ,13);
    GpiMove(hPS,&ptl);
    GpiImage(hPS, 0L, &szImage, (LONG)szImage.cx*szImage.cy/8,abImage);

    /*
     * Test functionality of GreDeviceSetGlobalAttribute
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetGlobalAttribute Functionality Test  - Entry.\n");
    if(!GreDeviceSetGlobalAttribute(hDC,ATYPE_BACK_MIX_MODE, BM_LEAVEALONE, 0L) ){
        lErrorCount += BADERROR("GreDeviceSetGlobalAttribute Failed",FALSE,NULL);
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetGlobalAttribute Functionality Test  - Exit.\n");

    MapAndWriteString(hPS,delta_x,(7*delta_y/4),"Foreground Color Attr = Blue");
    MapAndWriteString(hPS,delta_x,(3*delta_y/2),"Background Color Attr = Green");
    MapAndWriteString(hPS,delta_x,(5*delta_y/4),"Foreground Mix Mode = Overpaint");
    MapAndWriteString(hPS,delta_x,delta_y,"Background Mix Mode = Overpaint");
    MapAndWriteString(hPS,(5*delta_x/2),(6*delta_y),"Tests: GreSetGlobalAttribute");

    if(!lErrorCount){
      CWRITELOG(L_HDR,0,
"$GreDeviceSetGlobalAttributeExh                                 -    PASS\n");
    }
    else{
       sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
       CWRITELOG(L_HDR,0,out_string);
       CWRITELOG(L_HDR,0,
"$GreDeviceSetGlobalAttributeExh                                 -    FAIL\n");
    }
    CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceSetGlobalAttributeExh - Exit.\n");
 }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetGetAttrExh()
 *
 * Purpose: This entry point executes the functionality tests for
 *          GreDeviceGetAttributes and GreDeviceSetAttributes
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 10/24/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: None.
 *
 * Subroutines Required:
 *          CWRITELOG(),
 *          TEST_SIGNATURE(),
 *          LOG_SIGNATURE
 *
 * Limitations: Only 4 out of 5 basic primitive types are tested.
 *
 *              The GreDeviceGetAttributes function does not seem to
 *              work correctly.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *   // for line primitive type
 *   GreDeviceSetAttributes(hDC,PRIM_LINE,0L,line attributes,pLine);
 *   // display lines on screen showing that attributes have been set
 *   GreDeviceGetAttributes(hDC,PRIM_LINE,line attributes,pLineComp);
 *   // verify that the attributes received with the Get function
 *   // match the attributes that were set.
 *
 *   // for area primitive type
 *   GreDeviceSetAttributes(hDC,PRIM_AREA,0L,area attributes,pArea);
 *   // display lines on screen showing that attributes have been set
 *   GreDeviceGetAttributes(hDC,PRIM_AREA,area attributes,pAreaComp);
 *   // verify that the attributes received with the Get function
 *   // match the attributes that were set.
 *
 *   // for marker primitive type
 *   GreDeviceSetAttributes(hDC,PRIM_MARKER,0L,marker attributes,pMarker);
 *   // display lines on screen showing that attributes have been set
 *   GreDeviceGetAttributes(hDC,PRIM_MARKER,marker attributes,pMarkerComp);
 *   // verify that the attributes received with the Get function
 *   // match the attributes that were set.
 *
 *   // for image primitive type
 *   GreDeviceSetAttributes(hDC,PRIM_IMAGE,0L,image attributes,pImage);
 *   // display lines on screen showing that attributes have been set
 *   GreDeviceGetAttributes(hDC,PRIM_IMAGE,image attributes,pImageComp);
 *   // verify that the attributes received with the Get function
 *   // match the attributes that were set.
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceSetGetAttrExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC            hDC;
  POINTL         ptl,
                 ptlBox[7];
  CHAR           out_string[257];
  INT            i;
  PDLINEBUNDLE   pLine,
                 pLineComp;
  PDAREABUNDLE   pArea,
                 pAreaComp;
  PDMARKERBUNDLE pMarker,
                 pMarkerComp;
  PDIMAGEBUNDLE  pImage,
                 pImageComp;
  SIZEL          szImage;
  LONG           lIndex,
                 lErrorCount;
  BYTE           abImage[2000];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    lErrorCount = 0;
    TEST_SIGNATURE(0,0);
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE,0,"+ee+ GreDeviceSetGetAttrExh - Entry.\n");
    hDC = set_delta(hPS);
    /*
     * initialize attributes for line primitive bundle
     */
    pLine = malloc(sizeof(DLINEBUNDLE));
    pLineComp = malloc(sizeof(DLINEBUNDLE));
    pLine->lbnd.lColor = CLR_CYAN;
    pLine->lbnd.usMixMode = FM_OVERPAINT;
    pLine->lbnd.fxWidth = LINEWIDTH_NORMAL;
    pLine->lbnd.lGeomWidth = 1;
    pLine->lbnd.usType = LINETYPE_SHORTDASH;
    pLine->lbnd.usEnd = LINEEND_ROUND;
    pLine->lbnd.usJoin = LINEJOIN_BEVEL;

    /*
     * Test functionality of GreDeviceSetAttributes by changing
     * every attribute possible for the Line bundle.
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetAttributes (Line) Functionality Test  - Entry.\n");
    if(!GreDeviceSetAttributes(hDC,PRIM_LINE, 0,
                                   LBB_MIX_MODE | LBB_WIDTH |
                                   LBB_GEOM_WIDTH | LBB_END | LBB_JOIN |
                                   LBB_COLOR | LBB_TYPE,
                                   (PBUNDLE)pLine)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetAttributes Functionality Test  - Exit.\n");

    /*
     * display attributes on screen by drawing a line
     */
    ptl.x = MapX(13 * delta_x / 2);
    ptl.y = MapY(2 * delta_y);
    GpiMove(hPS, &ptl);
    ptl.y = MapY(5 * delta_y);
    GpiLine(hPS, &ptl);
    ptl.x = MapX(15 * delta_x / 2);
    GpiLine(hPS, &ptl);
    ptl.y = MapY(2 * delta_y);
    GpiLine(hPS, &ptl);
    ptl.x = MapX(13 * delta_x / 2);
    GpiLine(hPS, &ptl);
    MapAndWriteString(hPS,(6*delta_x),(7*delta_y/4),"Line-Type: ShortDash");
    MapAndWriteString(hPS,(6*delta_x),(3*delta_y/2),"Line-Color: Cyan");

    /*
     * Test functionality of GreDeviceGetAttributes.
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceGetAttributes (Line) Functionality Test  - Entry.\n");
    /*
     * only foreground color can be requested.
     */
    if(!GreDeviceGetAttributes(hDC,PRIM_LINE,
                                   LBB_COLOR,
                                   (PBUNDLE)pLineComp)) {
        lErrorCount += BADERROR("GreDeviceGetAttributes Failed",FALSE,NULL);
    }
    /*
     * if the attribute received is incorrect write to DTT_log
     */
    if( (pLineComp->lbnd.lColor != pLine->lbnd.lColor) ){
       CWRITELOG(L_TRACE,0,"GreDeviceGetAttributes received incorrect values         - FAIL.\n");
       ++lErrorCount;
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceGetAttributes Functionality Test  - Exit.\n");
    /*
     * initialize values for Area Bundle
     */
    pArea = malloc(sizeof(DAREABUNDLE));
    pAreaComp = malloc(sizeof(DAREABUNDLE));

    pArea->cAttr=sizeof(AREABUNDLE);
    pArea->cDefs=sizeof(AREADEFS);

    pArea->abnd.lColor = CLR_YELLOW;
    pArea->abnd.lBackColor = CLR_YELLOW;
    pArea->abnd.usMixMode = FM_OVERPAINT;
    pArea->abnd.usBackMixMode = BM_LEAVEALONE;
    pArea->abnd.usSet = 0;
    pArea->abnd.usSymbol = PATSYM_DIAG1;
    pArea->abnd.ptlRefPoint.x = 0;
    pArea->abnd.ptlRefPoint.y = 0;

    pArea->adef.defSet=0;
    pArea->adef.fFlags=CDEF_GENERIC;
    pArea->adef.CodePage=850;


    /*
     * Test functionality of GreDeviceSetAttributes by changing
     * every attribute possible for the Area bundle.
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetAttributes (Area) Functionality Test  - Entry.\n");
    if(!GreDeviceSetAttributes(hDC,PRIM_AREA, 0,
                                   ABB_COLOR | ABB_BACK_COLOR  |
                                   ABB_MIX_MODE | ABB_BACK_MIX_MODE |
                                   ABB_SYMBOL |  ABB_SET |
                                   ABB_REF_POINT,
                                   (PBUNDLE)pArea)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetAttributes Functionality Test  - Exit.\n");

    /*
     * initialize array for L-shape
     */
    ptlBox[0].x = delta_x;
    ptlBox[0].y = 5 * delta_y;
    ptlBox[1].x = delta_x;
    ptlBox[1].y = 2 * delta_y;
    ptlBox[2].x = 3 * delta_x;
    ptlBox[2].y = 2 * delta_y;
    ptlBox[3].x = 3 * delta_x;
    ptlBox[3].y = 3 * delta_y;
    ptlBox[4].x = 2 * delta_x;
    ptlBox[4].y = 3 * delta_y;
    ptlBox[5].x = 2 * delta_x;
    ptlBox[5].y = 5 * delta_y;
    ptlBox[6].x = delta_x;
    ptlBox[6].y = 5 * delta_y;
    /*
     * define a path and draw a L-Shape within 5 grid squares
     */
    GpiBeginPath(hPS,1L);
    for(i=0;i<=6;++i)
       MapPt(ptlBox[i]);
    GpiMove(hPS, &ptlBox[0]);
    GpiPolyLine(hPS, 7l, ptlBox);
    GpiCloseFigure(hPS);
    GpiEndPath(hPS);
    GpiFillPath(hPS,1L, FPATH_WINDING);
    MapAndWriteString(hPS,delta_x,(7*delta_y/4),"Area-Color: Yellow");
    MapAndWriteString(hPS,delta_x,(3*delta_y/2),"Area-Pattern: Diag1");

    /*
     * Test functionality of GreDeviceGetAttributes.
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceGetAttributes (Area) Functionality Test  - Entry.\n");
    /*
     * can only request foreground and background color attributes
     */
    if(!GreDeviceGetAttributes(hDC,PRIM_AREA,
                                   ABB_COLOR | ABB_BACK_COLOR,
                                   (PBUNDLE)pAreaComp)) {
        lErrorCount += BADERROR("GreDeviceGetAttributes Failed",FALSE,NULL);
    }
    if( (pAreaComp->abnd.lColor != pArea->abnd.lColor)  ||
        (pAreaComp->abnd.lBackColor != pArea->abnd.lBackColor) )  {
       CWRITELOG(L_TRACE,0,"GreDeviceGetAttributes received incorrect attribute values.     - FAIL\n");
       ++lErrorCount;
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceGetAttributes Functionality Test  - Exit.\n");
    /*
     * initialize values for Marker Bundle
     */
    pMarker = malloc(sizeof(DMARKERBUNDLE));
    pMarkerComp = malloc(sizeof(DMARKERBUNDLE));

    pMarker->cAttr=sizeof(MARKERBUNDLE);
    pMarker->cDefs=sizeof(MARKERDEFS);


    pMarker->mbnd.lColor = CLR_GREEN;
    pMarker->mbnd.lBackColor = CLR_YELLOW;
    pMarker->mbnd.usMixMode = FM_OVERPAINT;
    pMarker->mbnd.usBackMixMode = BM_LEAVEALONE;
    pMarker->mbnd.usSet = 0;
    pMarker->mbnd.usSymbol = MARKSYM_SOLIDDIAMOND;
    pMarker->mbnd.sizfxCell.cx = 2L;
    pMarker->mbnd.sizfxCell.cy = 2L;

    pMarker->mdef.defSet=0;
    pMarker->mdef.fFlags=0;
    pMarker->mdef.CodePage=850;


    /*
     * Test functionality of GreDeviceSetAttributes by changing
     * every attribute possible for the Marker bundle.
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetAttributes (Marker) Functionality Test  - Entry.\n");
//tss...old
//    if(!GreDeviceSetAttributes(hDC,PRIM_MARKER, MBB_BOX,
//                                   MBB_COLOR | MBB_BACK_COLOR  |
//                                   MBB_MIX_MODE | MBB_BACK_MIX_MODE |
//                                   MBB_SET | MBB_SYMBOL |
//                                   MBB_BOX,
//                                   (PBUNDLE)pMarker)) {
//        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
//     }


    if(!GreDeviceSetAttributes(hDC,PRIM_MARKER, MBB_SET,
                                   MBB_COLOR | MBB_BACK_COLOR |
                                   MBB_MIX_MODE | MBB_BACK_MIX_MODE |
                                   MBB_SYMBOL | MBB_BOX,
                                   (PBUNDLE)pMarker)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }



    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetAttributes Functionality Test  - Exit.\n");

    /*
     * display a marker on the screen
     */
    ptl.x = MapX(4 * delta_x);
    ptl.y = MapY(3 * delta_y);
    GpiMarker(hPS, &ptl);
    MapAndWriteString(hPS,(7*delta_x/2),(7*delta_y/4),"Marker-Color: Green");
    MapAndWriteString(hPS,(7*delta_x/2),(3*delta_y/2),"Marker-Shape: Diamond");

    /*
     * Test functionality of GreDeviceGetAttributes.
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceGetAttributes (Marker) Functionality Test  - Entry.\n");
    /*
     * can only request foreground and background color attributes
     */
    if(!GreDeviceGetAttributes(hDC,PRIM_MARKER,
                                   MBB_COLOR | MBB_BACK_COLOR,
                                   (PBUNDLE)pMarkerComp)) {
        lErrorCount += BADERROR("GreDeviceGetAttributes Failed",FALSE,NULL);
    }
    if( (pMarkerComp->mbnd.lColor != pMarker->mbnd.lColor)  ||
        (pMarkerComp->mbnd.lBackColor != pMarker->mbnd.lBackColor) )  {
       CWRITELOG(L_TRACE,0,"GreDeviceGetAttributes received incorrect attribute values.     - FAIL\n");
       ++lErrorCount;
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceGetAttributes Functionality Test  - Exit.\n");
    /*
     * initialize values for Image Bundle
     */
    pImage = malloc(sizeof(DIMAGEBUNDLE));
    pImageComp = malloc(sizeof(DIMAGEBUNDLE));

    pImage->cAttr=sizeof(IMAGEBUNDLE);

    pImage->ibnd.lColor = CLR_RED;
    pImage->ibnd.lBackColor = CLR_CYAN;
    pImage->ibnd.usMixMode = FM_OVERPAINT;
    pImage->ibnd.usBackMixMode = BM_LEAVEALONE;

    pImage->cDefs=0;

    /*
     * Test functionality of GreDeviceSetAttributes by changing
     * every attribute possible for the Image bundle.
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetAttributes (Image) Functionality Test  - Entry.\n");
    if(!GreDeviceSetAttributes(hDC,PRIM_IMAGE, 0,
                                   IBB_COLOR | IBB_BACK_COLOR  |
                                   IBB_MIX_MODE | IBB_BACK_MIX_MODE,
                                   (PBUNDLE)pImage)){
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetAttributes Functionality Test  - Exit.\n");
    /*
     * The following code segment will draw a bitmap image
     */
    szImage.cx = szImage.cy = 104;
    ptl.x = 5 * delta_x;
    ptl.y = 4 * delta_y;
    MapPt(ptl);
    memset(abImage,0x00,sizeof(abImage));  // Set all bits to OFF .

    for (lIndex=0; lIndex <104; lIndex+=2)
         memset(abImage+lIndex*13,0xFF ,13);
    GpiMove(hPS,&ptl);
    GpiImage(hPS, 0L, &szImage, (LONG)szImage.cx*szImage.cy/8,abImage);
    MapAndWriteString(hPS,(9*delta_x/2),(9*delta_y/2),"Image-Color: Red");

    /*
     * Test functionality of GreDeviceGetAttributes.
     */
    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceGetAttributes (Image) Functionality Test  - Entry.\n");
    /*
     * can only request foreground and background color attributes
     */
    if(!GreDeviceGetAttributes(hDC,PRIM_IMAGE,
                                   IBB_COLOR | IBB_BACK_COLOR,
                                   (PBUNDLE)pImageComp)){
        lErrorCount += BADERROR("GreDeviceGetAttributes Failed",FALSE,NULL);
    }
    if( (pImageComp->ibnd.lColor != pImage->ibnd.lColor)  ||
        (pImageComp->ibnd.lBackColor != pImage->ibnd.lBackColor) )  {
       CWRITELOG(L_TRACE,0,"GreDeviceGetAttributes received incorrect attribute values.     - FAIL\n");
       ++lErrorCount;
    }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceGetAttributes Functionality Test  - Exit.\n");
    MapAndWriteString(hPS,(5*delta_x/2),(6*delta_y),"Tests: GreDeviceGetAttributes");
    MapAndWriteString(hPS,(5*delta_x/2),(23*delta_y/4),"       GreDeviceSetAttributes");

    if(!lErrorCount){
       CWRITELOG(L_HDR,0,
"$GreDeviceSetGetAttrExh                                         -    PASS\n");
      }
    else{
       sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
       CWRITELOG(L_HDR,0,out_string);
       CWRITELOG(L_HDR,0,
"$GreDeviceSetGetAttrExh                                         -    FAIL\n");
      }
    CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceSetGetAttrExh - Exit.\n");
  }
}








VOID APIENTRY GreSetLineAttrExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC            hDC;
  POINTL         apt[3];
  POINTL         apt2;
  CHAR           out_string[257];
  LONG           rc;
  PDLINEBUNDLE   pLine,
                 pLineComp;
  LONG           lErrorCount;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    lErrorCount = 0;
    TEST_SIGNATURE(0,0);
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE,0,"+ee+ GreDeviceSetGetAttrExh - Entry.\n");
    hDC = set_delta(hPS);
    /*
     * initialize attributes for line primitive bundle
     */
    pLine = malloc(sizeof(DLINEBUNDLE));
    pLineComp = malloc(sizeof(DLINEBUNDLE));
    pLine->lbnd.lColor = CLR_CYAN;
    pLine->lbnd.usMixMode = FM_OVERPAINT;
    pLine->lbnd.fxWidth = LINEWIDTH_NORMAL;
    pLine->lbnd.lGeomWidth = 1;
    pLine->lbnd.usType = LINETYPE_DOT;
    pLine->lbnd.usEnd = LINEEND_ROUND;
    pLine->lbnd.usJoin = LINEJOIN_BEVEL;

    CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetAttributes (Line) Functionality Test  - Entry.\n");
    if(!GreDeviceSetAttributes(hDC,PRIM_LINE, 0,
                                   LBB_MIX_MODE | LBB_WIDTH |
                                   LBB_GEOM_WIDTH | LBB_END | LBB_JOIN |
                                   LBB_COLOR | LBB_TYPE,
                                   (PBUNDLE)pLine)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }
    CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetAttributes Functionality Test  - Exit.\n");



      apt[0].x = MapX( delta_x*1 );
      apt[0].y = MapY( delta_y*1 );
      apt[1].x = MapX( (LONG)(delta_x*1+delta_x/2) );
      apt[1].y = MapY( delta_y*2 );
      apt[2].x = MapX( delta_x*2 );
      apt[2].y = MapY( delta_y*1 );

      apt2.x = apt[2].x + MapX( (LONG)(delta_x/2) );
      apt2.y = apt[2].y;
      GpiMove( hPS, &apt2 );
      GpiCharString(hPS, 3L, "DOT");

      GpiMove(hPS, &apt[2]);

     rc = GrePolyLine(hDC,apt,3);
     if (rc == GPI_ERROR) {
        BADERROR("GrePoly",TRUE,"GreLinePre");
        return;
     }


    pLine->lbnd.usType = LINETYPE_SHORTDASH;
    if(!GreDeviceSetAttributes(hDC,PRIM_LINE, 0,
                                   LBB_MIX_MODE | LBB_WIDTH |
                                   LBB_GEOM_WIDTH | LBB_END | LBB_JOIN |
                                   LBB_COLOR | LBB_TYPE,
                                   (PBUNDLE)pLine)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }


      apt[0].x = MapX( delta_x*5 );
      apt[0].y = MapY( delta_y*1 );
      apt[1].x = MapX( (LONG)(delta_x*5+delta_x/2) );
      apt[1].y = MapY( delta_y*2 );
      apt[2].x = MapX( delta_x*6 );
      apt[2].y = MapY( delta_y*1 );

      apt2.x = apt[2].x + MapX( (LONG)(delta_x/2) );
      apt2.y = apt[2].y;
      GpiMove( hPS, &apt2 );
      GpiCharString(hPS, 9L, "SHORTDASH");

      GpiMove(hPS, &apt[2]);

     rc = GrePolyLine(hDC,apt,3);
     if (rc == GPI_ERROR) {
        BADERROR("GrePoly",TRUE,"GreLinePre");
        return;
     }



    pLine->lbnd.usType = LINETYPE_DASHDOT;
    if(!GreDeviceSetAttributes(hDC,PRIM_LINE, 0,
                                   LBB_MIX_MODE | LBB_WIDTH |
                                   LBB_GEOM_WIDTH | LBB_END | LBB_JOIN |
                                   LBB_COLOR | LBB_TYPE,
                                   (PBUNDLE)pLine)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }


      apt[0].x = MapX( delta_x*1 );
      apt[0].y = MapY( delta_y*3 );
      apt[1].x = MapX( (LONG)(delta_x*1+delta_x/2) );
      apt[1].y = MapY( delta_y*4 );
      apt[2].x = MapX( delta_x*2 );
      apt[2].y = MapY( delta_y*3 );

      apt2.x = apt[2].x + MapX( (LONG)(delta_x/2) );
      apt2.y = apt[2].y;
      GpiMove( hPS, &apt2 );
      GpiCharString(hPS, 7L, "DASHDOT");

      GpiMove(hPS, &apt[2]);

     rc = GrePolyLine(hDC,apt,3);
     if (rc == GPI_ERROR) {
        BADERROR("GrePoly",TRUE,"GreLinePre");
        return;
     }


    pLine->lbnd.usType = LINETYPE_DOUBLEDOT;
    if(!GreDeviceSetAttributes(hDC,PRIM_LINE, 0,
                                   LBB_MIX_MODE | LBB_WIDTH |
                                   LBB_GEOM_WIDTH | LBB_END | LBB_JOIN |
                                   LBB_COLOR | LBB_TYPE,
                                   (PBUNDLE)pLine)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }


      apt[0].x = MapX( delta_x*5 );
      apt[0].y = MapY( delta_y*3 );
      apt[1].x = MapX( (LONG)(delta_x*5+delta_x/2) );
      apt[1].y = MapY( delta_y*4 );
      apt[2].x = MapX( delta_x*6 );
      apt[2].y = MapY( delta_y*3 );

      apt2.x = apt[2].x + MapX( (LONG)(delta_x/2) );
      apt2.y = apt[2].y;
      GpiMove( hPS, &apt2 );
      GpiCharString(hPS, 9L, "DOUBLEDOT");

      GpiMove(hPS, &apt[2]);

     rc = GrePolyLine(hDC,apt,3);
     if (rc == GPI_ERROR) {
        BADERROR("GrePoly",TRUE,"GreLinePre");
        return;
     }


    pLine->lbnd.usType = LINETYPE_LONGDASH;
    if(!GreDeviceSetAttributes(hDC,PRIM_LINE, 0,
                                   LBB_MIX_MODE | LBB_WIDTH |
                                   LBB_GEOM_WIDTH | LBB_END | LBB_JOIN |
                                   LBB_COLOR | LBB_TYPE,
                                   (PBUNDLE)pLine)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }


      apt[0].x = MapX( delta_x*1 );
      apt[0].y = MapY( delta_y*5 );
      apt[1].x = MapX( (LONG)(delta_x*1+delta_x/2) );
      apt[1].y = MapY( delta_y*6 );
      apt[2].x = MapX( delta_x*2 );
      apt[2].y = MapY( delta_y*5 );

      apt2.x = apt[2].x + MapX( (LONG)(delta_x/2) );
      apt2.y = apt[2].y;
      GpiMove( hPS, &apt2 );
      GpiCharString(hPS, 8L, "LONGDASH");

      GpiMove(hPS, &apt[2]);

     rc = GrePolyLine(hDC,apt,3);
     if (rc == GPI_ERROR) {
        BADERROR("GrePoly",TRUE,"GreLinePre");
        return;
     }




    pLine->lbnd.usType = LINETYPE_ALTERNATE;
    if(!GreDeviceSetAttributes(hDC,PRIM_LINE, 0,
                                   LBB_MIX_MODE | LBB_WIDTH |
                                   LBB_GEOM_WIDTH | LBB_END | LBB_JOIN |
                                   LBB_COLOR | LBB_TYPE,
                                   (PBUNDLE)pLine)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }


      apt[0].x = MapX( delta_x*5 );
      apt[0].y = MapY( delta_y*5 );
      apt[1].x = MapX( (LONG)(delta_x*5+delta_x/2) );
      apt[1].y = MapY( delta_y*6 );
      apt[2].x = MapX( delta_x*6 );
      apt[2].y = MapY( delta_y*5 );

      apt2.x = apt[2].x + MapX( (LONG)(delta_x/2) );
      apt2.y = apt[2].y;
      GpiMove( hPS, &apt2 );
      GpiCharString(hPS, 9L, "ALTERNATE");

      GpiMove(hPS, &apt[2]);

     rc = GrePolyLine(hDC,apt,3);
     if (rc == GPI_ERROR) {
        BADERROR("GrePoly",TRUE,"GreLinePre");
        return;
     }


    pLine->lbnd.usType = LINETYPE_INVISIBLE;
    if(!GreDeviceSetAttributes(hDC,PRIM_LINE, 0,
                                   LBB_MIX_MODE | LBB_WIDTH |
                                   LBB_GEOM_WIDTH | LBB_END | LBB_JOIN |
                                   LBB_COLOR | LBB_TYPE,
                                   (PBUNDLE)pLine)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }


      apt[0].x = MapX( delta_x*1 );
      apt[0].y = MapY( delta_y*7 );
      apt[1].x = MapX( (LONG)(delta_x*1+delta_x/2) );
      apt[1].y = MapY( delta_y*8 );
      apt[2].x = MapX( delta_x*2 );
      apt[2].y = MapY( delta_y*7 );

      apt2.x = apt[2].x + MapX( (LONG)(delta_x/2) );
      apt2.y = apt[2].y;
      GpiMove( hPS, &apt2 );
      GpiCharString(hPS, 9L, "INVISIBLE");

      GpiMove(hPS, &apt[2]);

     rc = GrePolyLine(hDC,apt,3);
     if (rc == GPI_ERROR) {
        BADERROR("GrePoly",TRUE,"GreLinePre");
        return;
     }

    pLine->lbnd.usType = LINETYPE_SOLID;
    if(!GreDeviceSetAttributes(hDC,PRIM_LINE, 0,
                                   LBB_MIX_MODE | LBB_WIDTH |
                                   LBB_GEOM_WIDTH | LBB_END | LBB_JOIN |
                                   LBB_COLOR | LBB_TYPE,
                                   (PBUNDLE)pLine)) {
        lErrorCount += BADERROR("GreDeviceSetAttributes Failed",FALSE,NULL);
     }


      apt[0].x = MapX( delta_x*5 );
      apt[0].y = MapY( delta_y*7 );
      apt[1].x = MapX( (LONG)(delta_x*5+delta_x/2) );
      apt[1].y = MapY( delta_y*8 );
      apt[2].x = MapX( delta_x*6 );
      apt[2].y = MapY( delta_y*7 );

      apt2.x = apt[2].x + MapX( (LONG)(delta_x/2) );
      apt2.y = apt[2].y;
      GpiMove( hPS, &apt2 );
      GpiCharString(hPS, 5L, "SOLID");

      GpiMove(hPS, &apt[2]);

     rc = GrePolyLine(hDC,apt,3);
     if (rc == GPI_ERROR) {
        BADERROR("GrePoly",TRUE,"GreLinePre");
        return;
     }



    if(!lErrorCount){
       CWRITELOG(L_HDR,0,
"$GreSetLineAttrExh                                        -    PASS\n");
      }
    else{
       sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
       CWRITELOG(L_HDR,0,out_string);
       CWRITELOG(L_HDR,0,
"$GreSetLineAttrExh                                        -    FAIL\n");
      }
    CWRITELOG(L_LOTRACE,0,"+ex+ GreSetLineAttrExh - Exit.\n");
  }
}



/****************************************************************************
 *\\ddd
 * Routine Name: GreNotifyClipChangeExh()
 *
 * Purpose: This entry point executes the functionality test for GreNotifyClipChange
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 10/24/90, PDVT, Donna Williams
 *                     - initial creation
 *               12/17/90, PDVT, Donna Williams
 *                     - design implementation
 *               01/02/91, PDVT, Donna Williams
 *                     - design review
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: NONE.
 *
 * Subroutines Required:
 *          CWRITELOG(),
 *          LOG_SIGNATURE,
 *          TEST_SIGNATURE().
 *
 * Limitations: NONE.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *  // create clipping region
 *  GpiCreateRegion(hPS, 0L, &region);
 *  // check for clipping change
 *  GreNotifyClipChange(hDC,&region,cRect,idClipPath);
 *  // change clipping region
 *  GpiSetClipRegion(hPS, hrgn, NULL);
 *  // check for clipping change
 *  GreNotifyClipChange(hDC,&region,cRect,idClipPath);
 *\\end
 ***************************************************************************/
VOID APIENTRY GreNotifyClipChangeExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL   offset;
  RECTL    region;
  HRGN     hrgn;
  LONG     complx;
  CHAR     str1_msg[200],
           str2_msg[200];
  ULONG    cRect,
           idClipPath;
  PRGNRECT pControl;
  HDC      hDC;
  POINTL   ptl;
  CHAR     out_string[100];
  LONG     lErrorCount;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        lErrorCount = 0L;
        hDC = set_delta(hPS);
        TEST_SIGNATURE(0, 0);
        LOG_SIGNATURE;
        /* * * * * * * * * * */
        /*  INITIALIZATIONS  */
        /* * * * * * * * * * */
        memset(str1_msg, '/', sizeof(str1_msg));
        memset(str2_msg, 'X', sizeof(str2_msg));

        CWRITELOG(L_LOTRACE,0,"+ee+ GreNotifyClipChangeExh - Entry\n");
        /*
         * print out some screen messages before
         * any regions are defined
         */
        sprintf(out_string,"Clip two regions.");
        MapAndWriteString(hPS,(delta_x/4),delta_y,out_string);
        sprintf(out_string,"Notify each clip.");
        MapAndWriteString(hPS,(delta_x/4),(delta_y-(delta_y/4)),out_string);

        // define region
        region.xLeft = MapX(delta_x);
        region.yBottom = MapY(2 * delta_y);
        region.xRight = MapX(2 * delta_x);
        region.yTop = MapY(4 * delta_y);

        pControl = malloc(sizeof(RGNRECT));
        pControl->ircStart = 1;
        pControl->crc = 100;
        pControl->crcReturned = 0;
        pControl->ulDirection = RECTDIR_LFRT_TOPBOT;

        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");

        hrgn = GpiCreateRegion(hPS,    // Create region in presentation space
                           0L,       // of one rectangle using coordinates
                           &region); // from region1.
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");

        GpiSetClipRegion(hPS,  // set region created to a clipping region
                         hrgn,
                         NULL);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");

        offset.x = delta_x;
        offset.y = -1 *  delta_y;
        MapPt(offset);
        GpiOffsetClipRegion(hPS, &offset);

        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");
        GpiSetClipRegion(hPS, NULL, NULL);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");

        GpiDestroyRegion(hPS, hrgn);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");

        hrgn = GpiCreateRegion(hPS,  // Create region in presentation space
                               1L,   // of one rectangle using coordinates
                               &region); // from region3.
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");
        complx = GpiSetClipRegion(hPS, // set region created to a clipping region
                                  hrgn,
                                  NULL);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");

        offset.x = delta_x; //60;
        offset.y = delta_y - 240; //45;
        MapPt(offset);
        complx = GpiOffsetClipRegion(hPS, &offset);
        if (complx == RGN_RECT) {   // check if indicator equals RGN_RECT
          ptl.x = delta_x - 250; //30;
          ptl.y = 5 * delta_y - 200; //400;
          MapPt(ptl);
          display_text(hPS, ptl.x, ptl.y, str2_msg);
          CWRITELOG(L_LOTRACE,0,"GpiOffsetClipRegion RGN_RECT Test 1              - PASS\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");
        memset(str1_msg, '+', sizeof(str1_msg));
        offset.x = delta_x - 160; //25;
        offset.y = -2 * delta_y; //60;
        MapPt(offset);
        complx = GpiOffsetClipRegion(hPS, &offset);
        if (complx == RGN_RECT) {   // check if indicator equals RGN_RECT
           ptl.x = delta_x; //40;
           ptl.y = 6 * delta_y; //500;
           MapPt(ptl);
           display_text(hPS, ptl.x, ptl.y, str1_msg);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");
        GpiSetClipRegion(hPS, NULL, NULL);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");
        GpiDestroyRegion(hPS, hrgn);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange - Entry.\n");
        if(!GreNotifyClipChange(hDC,&region,cRect,idClipPath) ){
           lErrorCount += BADERROR("GreNotifyClipChange",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange - Exit.\n");
        if(!lErrorCount){
          CWRITELOG(L_HDR,0,
"$GreNotifyClipChangeExh                                         -    PASS\n");
          }
        else{
           sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreNotifyClipChangeExh                                         -    FAIL\n");
          }
        CWRITELOG(L_LOTRACE,0,"+ex+ Test GreNotifyClipChangeExh - Exit\n");
      }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreNotifyTransformChangeExh()
 *
 * Purpose: This entry point executes the functionality test for GreNotifyTransformChange
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 10/24/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: NONE.
 *
 * Subroutines Required:
 *          CWRITELOG(),
 *          LOG_SIGNATURE,
 *          TEST_SIGNATURE.
 *
 * Limitations: NONE.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *   // create a region in world coordinates
 *   GpiCreateRegion(hPS, 1L, &region);
 *
 *   // convert the region into device coordinates
 *   GreConvert(hDC,lSrc,lDst,paptPoints,cPoints);
 *
 *   // determine if coordinates have been transformed
 *   GreNotifyTransformChange(hDC,flFlags,pXformdata);
 *\\end
 ***************************************************************************/
VOID APIENTRY GreNotifyTransformChangeExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  RECTL                 region;
  HRGN                  hrgn;
  HDC                   hDC;
  POINTL                ptl;
  CHAR                  out_string[100];
  LONG                  lErrorCount;
  PNOTIFYTRANSFORMDATA  pXformdata;
  ULONG                 flFlags;
  BOOL                  b_rc;
  LONG                  lSrc,
                        lDst,
                        cPoints;
  PPOINTL               paptPoints;
  PRGNRECT              pControl;
  LONG                  complx;
  POINTL                offset;
  RECTL                 parcRect;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        lErrorCount = 0L;
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        hDC = set_delta(hPS);

        CWRITELOG(L_LOTRACE,0,"+ee+ GreNotifyTransformChangeExh - Entry.\n");

        /*
         * print out some screen messages before
         * any regions are defined
         */
        sprintf(out_string,"Region first drawn in world coordinates.");
        MapAndWriteString(hPS,delta_x,(3*delta_y/2),out_string);
        sprintf(out_string,"Region is overdrawn in device coordinates.");
        MapAndWriteString(hPS,delta_x,delta_y,out_string);

        region.xLeft = MapX(delta_x / 4);
        region.yBottom = MapY(2 * delta_y);
        region.xRight = MapX(8 * delta_x);
        region.yTop = MapY(6 * delta_y);
        pControl = malloc(sizeof(RGNRECT));
        pControl->ircStart = 1;
        pControl->crc = 100;
        pControl->crcReturned = 0;
        pControl->ulDirection = RECTDIR_LFRT_TOPBOT;

        hrgn = GpiCreateRegion(hPS,    // Create region in presentation space
                           1L,       // of one rectangle using coordinates
                           &region); // from region1.
        if (!hrgn) {
           BADERROR("GpiCreateRegion",TRUE,"GreNotifyTransformChangeExh");
           return;
        }
        GreGetClipRects(hDC,&region,pControl,&parcRect);
        hrgn = GpiCreateRegion(hPS,  // Create region in presentation space
                               1L,   // of one rectangle using coordinates
                               &region); // from region3.
        if (!hrgn) {
           BADERROR("GpiCreateRegion",TRUE,"GreNotifyTransformChangeExh");
           return;
        }
        complx = GpiSetClipRegion(hPS,  // set region created to a clipping region
                            hrgn,
                            NULL);
        offset.x = delta_x;
        offset.y = -1 *  delta_y;
        MapPt(offset);
        complx = GpiOffsetClipRegion(hPS, &offset);

        offset.x = delta_x; //60;
        offset.y = delta_y - 240; //45;
        MapPt(offset);
        complx = GpiOffsetClipRegion(hPS, &offset);
        if (complx == RGN_RECT) {   // check if indicator equals RGN_RECT
          ptl.x = delta_x - 250; //30;
          ptl.y = 5 * delta_y - 200; //400;
          MapPt(ptl);
          display_text(hPS, ptl.x, ptl.y,"                         World");
          CWRITELOG(L_LOTRACE,0,"GpiNotifyTransformChangeExh - PASS\n");
        }
        /*
         * convert from world coordinates to device coordinates
         */
        lSrc = CVTC_WORLD;
        lDst = CVTC_DEVICE;
        cPoints = 0;
        b_rc = GreConvert(hDC,lSrc,lDst,paptPoints,cPoints);
        if (b_rc == FALSE) {
           BADERROR("GreConvert",TRUE,"GreNotifyTransformChangeExh");
           return;
        }
        offset.x = 0;
        offset.y = 0;
        MapPt(offset);
        complx = GpiOffsetClipRegion(hPS, &offset);
        if (complx == RGN_RECT) {   // check if indicator equals RGN_RECT
          ptl.x = delta_x - 250; //30;
          ptl.y = 5 * delta_y - 200; //400;
          MapPt(ptl);
          display_text(hPS, ptl.x, ptl.y,"                                      Device");
          CWRITELOG(L_LOTRACE,0,"GpiNotifyTransformChangeExh - PASS\n");
        }
        flFlags = MATRIX_SIMPLE|MATRIX_UNITS;
        pXformdata = malloc(sizeof(NOTIFYTRANSFORMDATA));
        b_rc == GreNotifyTransformChange(hDC,flFlags,pXformdata);
        if (b_rc == FALSE) {
           BADERROR("GreNotifyTransformChange",TRUE,"GreNotifyTransformChangeExh");
           return;
        }

        flFlags = MATRIX_SIMPLE|MATRIX_UNITS|MATRIX_TRANSLATION;
        b_rc == GreNotifyTransformChange(hDC,flFlags,pXformdata);
        if (b_rc == FALSE) {
           BADERROR("GreNotifyTransformChange",TRUE,"GreNotifyTransformChangeExh");
           return;
        }

        flFlags = MATRIX_SIMPLE|MATRIX_UNITS|MATRIX_XY_EXCHANGE|
                  MATRIX_Y_NEGATE|MATRIX_TRANSLATION;
        b_rc == GreNotifyTransformChange(hDC,flFlags,pXformdata);
        if (b_rc == FALSE) {
           lErrorCount += BADERROR("GreNotifyTransformChange",FALSE,NULL);
           ++lErrorCount;
        }

        if(!lErrorCount){
          CWRITELOG(L_HDR,0,
"$GreNotifyTransformChangeExh                                    -    PASS\n");
          }
        else{
           sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreNotifyTransformChangeExh                                    -    FAIL\n");
          }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreNotifyTransformChangeExh - Exit.\n");
 }
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetGlobalAttributesRC()
 *
 * Purpose: This entry point executes the return codes test for GreDeviceSetGlobalAttributesExh
 *          Return Codes Tested:
 *             PMERR_INV_HDC
 *          Return Codes Not Tested:
 *             PMERR_DEV_FUNC_NOT_INSTALLED
 *             PMERR_INV_MIX_ATTR
 *             PMERR_INV_BACKGROUND_MIX_ATTR
 *             PMERR_INV_COLOR_ATTR
 *             PMERR_INV_BACKGROUND_COL_ATTR
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 12/05/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: None.
 *
 * Subroutines Required: CWRITELOG(), LOG_SIGNATURE, CHECKRETURNERROR()
 *
 * Limitations:  Does not test all possible return codes.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // test for Invalid Device Context Handle  PMERR_INV_HDC
 *    bad_hDC = meaningless value;
 *
 *    // call function
 *    return_code = GreDeviceSetGlobalAttribute(bad_hDC,lAttrType,lAttribute,flOptions);
 *
 *    // report error value to DTT_LOG
 *    CHECKRETURNERROR(return_code,"GreDeviceSetGlobalAttribute PMERR_INV_HDC",SEVERITY_ERROR,
 *                     PMERR_INV_HDC);
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceSetGlobalAttributesRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC    bad_hDC,
         hDC;
  LONG   return_code,
         lAttrType,
         lAttribute;
  ULONG  flOptions;
  LONG   lErrorCount;
  CHAR   out_string[100];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else{
       hDC = set_delta(hPS);
       lErrorCount = 0;
       LOG_SIGNATURE;
       CWRITELOG(L_LOTRACE,0,"+ee+ GreDeviceSetGlobalAttributeRC - Entry.\n");
       /*
        * Test the following Gre call for an invalid device context
        * handle error.  The hDC passed is invalid if a 0 is passed.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetGlobalAttribute PMERR_INV_HDC Test - Entry.\n");
       bad_hDC = 0;
       return_code = GreDeviceSetGlobalAttribute(bad_hDC,lAttrType,lAttribute,flOptions);
       lErrorCount += CHECKRETURNERROR(return_code,"GreDeviceSetGlobalAttribute PMERR_INV_HDC",SEVERITY_ERROR,
                                    PMERR_INV_HDC);
       CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetGlobalAttribute PMERR_INV_HDC Test - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULL
                                      ,NULL, NULL,NULL);

       if(!lErrorCount){
          CWRITELOG(L_HDR,0,
"$GreDeviceSetGlobalAttributeRC                                  -    PASS\n");
       }
       else{
          sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
          CWRITELOG(L_HDR,0,out_string);
          CWRITELOG(L_HDR,0,
"$GreDeviceSetGlobalAttributeRC                                  -    FAIL\n");
       }
       CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceSetGlobalAttributeRC - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceGetAttributesRC()
 *
 * Purpose: This entry point executes the return codes test for GreDeviceGetAttributesExh
 *          Return Codes Tested:
 *             PMERR_INV_HDC
 *          Return Codes Not Tested:
 *             PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 12/05/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: None.
 *
 * Subroutines Required: CWRITELOG(), LOG_SIGNATURE, CHECKRETURNERROR()
 *
 * Limitations: Does not test all possible return codes
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // test for Invalid Device Context Handle  PMERR_INV_HDC
 *    bad_hDC = meaningless value;
 *
 *    // call function
 *    return_code = GreDeviceGetAttributes(bad_hDC,lPrimType,flAttrsMask,pAttrs);
 *
 *    // report error value to DTT_LOG
 *    CHECKRETURNERROR(return_code,"GreDeviceGetAttributes PMERR_INV_HDC",SEVERITY_ERROR,
 *                     PMERR_INV_HDC);
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceGetAttributesRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC     bad_hDC,
          hDC;
  LONG    lPrimType,
          return_code;
  ULONG   flAttrsMask;
  PBUNDLE pAttrs;
  LONG    lErrorCount;
  CHAR    out_string[100];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else{
       hDC = set_delta(hPS);
       lErrorCount = 0;
       LOG_SIGNATURE;
       CWRITELOG(L_LOTRACE,0,"+ee+ GreDeviceGetAttributesRC - Entry.\n");
       /*
        * Test the following Gre call for an invalid device context
        * handle error.  The hDC passed is invalid if a 0 is passed.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceGetAttributes PMERR_INV_HDC Test - Entry.\n");
       bad_hDC = 0;
       return_code = GreDeviceGetAttributes(bad_hDC,lPrimType,flAttrsMask,pAttrs);
       lErrorCount += CHECKRETURNERROR(return_code,"GreDeviceGetAttributes PMERR_INV_HDC",SEVERITY_ERROR,
                                    PMERR_INV_HDC);
       CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceGetAttributes PMERR_INV_HDC Test - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULL
                                      ,NULL, NULL,NULL);

       if(!lErrorCount){
          CWRITELOG(L_HDR,0,
"$GreDeviceGetAttributesRC                                       -    PASS\n");
       }
       else{
          sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
          CWRITELOG(L_HDR,0,out_string);
          CWRITELOG(L_HDR,0,
"$GreDeviceGetAttributesRC                                       -    FAIL\n");
       }
       CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceGetAttributesRC - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetAttributesRC()
 *
 * Purpose: This entry point executes the return codes test for GreDeviceSetAttributesExh
 *          Return Codes Tested:
 *              PMERR_INV_HDC
 *              PMERR_HDC_BUSY
 *          Return Codes Not Tested:
 *              PMERR_DEV_FUNC_NOT_INSTALLED
 *              PMERR_COORDINATE_OVERFLOW
 *              PMERR_INV_PATTERN_SET_ATTR
 *              PMERR_INV_MIX_ATTR
 *              PMERR_INV_BACKGROUND_MIX_ATTR
 *              PMERR_INV_PATTERN_REF_PT_ATTR
 *              PMERR_INV_LINE_TYPE_ATTR
 *              PMERR_INV_CHAR_MODE_ATTR
 *              PMERR_INV_CHAR_DIRECTION_ATTR
 *              PMERR_INSUFFICIENT_MEMORY
 *              PMERR_INV_LENGTH_OR_COUNT
 *              PMERR_INV_CODEPAGE
 *              PMERR_EXCEEDS_MAX_SEG_LENGTH
 *              PMERR_INV_COORD_SPACE
 *              PMERR_INV_PATTERN_SET_FONT
 *              PMERR_INV_COLOR_ATTR
 *              PMERR_INV_BACKGROUND_COL_ATTR
 *          Return Codes Not Tested Due To Implementation Problems:
 *              PMERR_HUGE_FONTS_NOT_SUPPORTED
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 12/05/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: None.
 *
 * Subroutines Required: CWRITELOG(), LOG_SIGNATURE, CHECKRETURNERROR
 *
 * Limitations: Does not test all possible return codes.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // test for Invalid Device Context Handle  PMERR_INV_HDC
 *    bad_hDC = meaningless value;
 *
 *    // call function
 *    return_code = GreDeviceSetAttributes(bad_hDC,lBType,flDefsMask.flAttrsMask,pAttrs);
 *
 *    // report error value to DTT_LOG
 *    CHECKRETURNERROR(return_code,"GreDeviceSetAttributes PMERR_INV_HDC",SEVERITY_ERROR,
 *                     PMERR_INV_HDC);
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceSetAttributesRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC            bad_hDC,
                 hDC;
  LONG           return_code;
  LONG           lBType;
  ULONG          flDefsMask,
                 flAttrsMask;
  PBUNDLE        pAttrs;
  PDIMAGEBUNDLE  pImage;
  LONG           lErrorCount;
  CHAR           out_string[100];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else{
       hDC = set_delta(hPS);
       lErrorCount = 0;
       LOG_SIGNATURE;
       CWRITELOG(L_LOTRACE,0,"+ee+ GreDeviceSetAttributesRC - Entry.\n");
       /*
        * Test the following Gre call for an invalid device context
        * handle error.  The hDC passed is invalid if a 0 is passed.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetAttributes PMERR_INV_HDC Test - Entry.\n");
       bad_hDC = 0;
       return_code = GreDeviceSetAttributes(bad_hDC,&lBType,&flDefsMask,&flAttrsMask,pAttrs);
       lErrorCount += CHECKRETURNERROR(return_code,"GreDeviceSetAttributes PMERR_INV_HDC",SEVERITY_ERROR,
                                    PMERR_INV_HDC);
       CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetAttributes PMERR_INV_HDC Test - Exit.\n");

       /*
        * Test for hdc_busy return code.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreDeviceSetAttributes PMERR_HDC_BUSY Test - Exit.\n");
       pImage = malloc(sizeof(DIMAGEBUNDLE));
       pImage->ibnd.lColor = CLR_RED;
       pImage->ibnd.lBackColor = CLR_CYAN;
       pImage->ibnd.usMixMode = FM_OVERPAINT;
       pImage->ibnd.usBackMixMode = BM_LEAVEALONE;

        //    BUSY CODE STARTS
       if(!MAKEPSBUSY(hPS)){
         CWRITELOG(L_FAIL,0,"GetBusy failed\n");
       }
       return_code = GreDeviceSetAttributes(hDC,PRIM_IMAGE, 0,
                                            IBB_COLOR | IBB_BACK_COLOR  |
                                            IBB_MIX_MODE | IBB_BACK_MIX_MODE,
                                            (PBUNDLE)&pImage);
       lErrorCount += CHECKRETURNERROR(return_code,"GreDeviceSetAttributes PMERR_HDC_BUSY",SEVERITY_ERROR,
                                       PMERR_HDC_BUSY);
       FREEPS;
       //    BUSY CODE ENDS
       CWRITELOG(L_LOTRACE,0,"+ix+ GreDeviceSetAttributes PMERR_HDC_BUSY Test - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULL
                                      ,NULL, NULL,NULL);

       if(!lErrorCount){
           CWRITELOG(L_HDR,0,
"$GreDeviceSetAttributesRC                                       -    PASS\n");
       }
       else{
           sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreDeviceSetAttributesRC                                       -    FAIL\n");
       }
       CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceSetAttributesRC - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreNotifyClipChangeRC()
 *
 * Purpose: This entry point executes the return codes test for GreNotifyClipChangeExh
 *          Return Codes Tested:
 *              PMERR_INV_HDC
 *              PMERR_HDC_BUSY
 *          Return Codes Not Tested:
 *              PMERR_DEV_FUNC_NOT_INSTALLED
 *              PMERR_INV_RECT
 *              PMERR_INV_COORDINATE
 *              PMERR_INV_REGION_CONTROL
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 12/05/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: None.
 *
 * Subroutines Required:  CWRITELOG(), LOG_SIGNATURE, CHECKRETURNERROR().
 *
 * Limitations: Does not test all possible return codes.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // test for Invalid Device Context Handle  PMERR_INV_HDC
 *    bad_hDC = meaningless value;
 *
 *    // call function
 *    return_code = GreNotifyClipChange(bad_hDC,prcBound,cRect,idClipPath);
 *
 *    // report error value to DTT_LOG
 *    CHECKRETURNERROR(return_code,"GreNotifyClipChangeRC PMERR_INV_HDC",SEVERITY_ERROR,
 *                     PMERR_INV_HDC);
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreNotifyClipChangeRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC     bad_hDC,
          hDC;
  PRECTL  prcBound;
  RECTL   region;
  ULONG   cRect,
          idClipPath;
  LONG    return_code,
          lErrorCount;
  CHAR    out_string[100];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else{
       hDC = set_delta(hPS);
       lErrorCount = 0;
       LOG_SIGNATURE;
       CWRITELOG(L_LOTRACE,0,"+ee+ GreNotifyClipChangeRC - Entry.\n");
       /*
        * Test the following Gre call for an invalid device context
        * handle error.  The hDC passed is invalid if a 0 is passed.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange PMERR_INV_HDC Test - Entry.\n");
       bad_hDC = 0;
       return_code = GreNotifyClipChange(bad_hDC,prcBound,cRect,idClipPath);
       lErrorCount += CHECKRETURNERROR(return_code,"GreNotifyClipChange PMERR_INV_HDC",SEVERITY_ERROR,
                                    PMERR_INV_HDC);
       CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange PMERR_INV_HDC Test - Exit.\n");

       /*
        * Test for hdc_busy return code.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyClipChange PMERR_HDC_BUSY Test - Exit.\n");
       // define region
       region.xLeft = MapX(delta_x);
       region.yBottom = MapY(2 * delta_y);
       region.xRight = MapX(2 * delta_x);
       region.yTop = MapY(4 * delta_y);

        //    BUSY CODE STARTS
       if(!MAKEPSBUSY(hPS)){
         CWRITELOG(L_FAIL,0,"GetBusy failed\n");
       }
       return_code=GreNotifyClipChange(hDC,&region,cRect,NCC_CLEANDC);
       lErrorCount += CHECKRETURNERROR(return_code,"GreNotifyClipChange PMERR_HDC_BUSY",SEVERITY_ERROR,
                                       PMERR_HDC_BUSY);
       FREEPS;
       //    BUSY CODE ENDS
       CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyClipChange PMERR_HDC_BUSY Test - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULL
                                      ,NULL, NULL,NULL);

       if(!lErrorCount){
            CWRITELOG(L_HDR,0,
"$GreNotifyClipChangeRC                                          -    PASS\n");
       }
       else{
            sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
            CWRITELOG(L_HDR,0,out_string);
            CWRITELOG(L_HDR,0,
"$GreNotifyClipChangeRC                                          -    FAIL\n");
       }
       CWRITELOG(L_LOTRACE,0,"+ex+ GreNotifyClipChangeRC - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreNotifyTransformChangeRC()
 *
 * Purpose: This entry point executes the return codes test for GreNotifyTransformChangeExh
 *          Return Codes Tested:
 *              PMERR_INV_HDC
 *              PMERR_HDC_BUSY
 *          Return Codes Not Tested:
 *              PMERR_INV_IN_AREA
 *              PMERR_PATH_LIMIT_EXCEEDED
 *              PMERR_INV_IN_PATH
 *              PMERR_DEV_FUNC_NOT_INSTALLED
 *              PMERR_COORDINATE_OVERFLOW
 *              PMERR_INV_PATTERN_REF_PT_ATTR
 *              PMERR_INV_LENGTH_OR_COUNT
 *              PMERR_INV_COORD_SPACE
 *              PMERR_INV_PICK_APERTURE_POSN
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 12/05/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: None.
 *
 * Subroutines Required: CWRITELOG(), LOG_SIGNATURE, CHECKRETURNERROR()
 *
 * Limitations:  Does not test all possible return codes.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // test for Invalid Device Context Handle  PMERR_INV_HDC
 *    bad_hDC = meaningless value;
 *
 *    // call function
 *    return_code = GreNotifyTransformChange(bad_hDC,flFlags,pXformdata);
 *
 *    // report error value to DTT_LOG
 *    CHECKRETURNERROR(return_code,"GreNotifyTransformChange PMERR_INV_HDC",SEVERITY_ERROR,
 *                     PMERR_INV_HDC);
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreNotifyTransformChangeRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC                  bad_hDC,
                       hDC;
  LONG                 return_code;
  ULONG                flFlags;
  PNOTIFYTRANSFORMDATA pXformdata;
  LONG                 lErrorCount;
  CHAR                 out_string[100];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else{
       hDC = set_delta(hPS);
       lErrorCount = 0;
       LOG_SIGNATURE;
       CWRITELOG(L_LOTRACE,0,"+ee+ GreNotifyTransformChangeRC - Entry.\n");
       /*
        * Test the following Gre call for an invalid device context
        * handle error.  The hDC passed is invalid if a 0 is passed.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyTransformChangePMERR_INV_HDC Test - Entry.\n");
       bad_hDC = 0;
       return_code = GreNotifyTransformChange(bad_hDC,flFlags,pXformdata);
       lErrorCount += CHECKRETURNERROR(return_code,"GreNotifyTransformChangePMERR_INV_HDC",SEVERITY_ERROR,
                                    PMERR_INV_HDC);
       CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyTransformChange PMERR_INV_HDC Test - Exit.\n");

       /*
        * Test for hdc_busy return code.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreNotifyTransformChange PMERR_HDC_BUSY Test - Exit.\n");
       flFlags = MATRIX_SIMPLE|MATRIX_UNITS;
       pXformdata = malloc(sizeof(NOTIFYTRANSFORMDATA));

        //    BUSY CODE STARTS
       if(!MAKEPSBUSY(hPS)){
         CWRITELOG(L_FAIL,0,"GetBusy failed\n");
       }
       return_code = GreNotifyTransformChange(hDC,flFlags,pXformdata);
       lErrorCount += CHECKRETURNERROR(return_code,"GreNotifyTransformChange PMERR_HDC_BUSY",SEVERITY_ERROR,
                                       PMERR_HDC_BUSY);
       FREEPS;
       //    BUSY CODE ENDS
       CWRITELOG(L_LOTRACE,0,"+ix+ GreNotifyTransformChange PMERR_HDC_BUSY Test - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULL
                                      ,NULL, NULL,NULL);

       if(!lErrorCount){
            CWRITELOG(L_HDR,0,
"$GreNotifyTransformChangeRC                                     -    PASS\n");
       }
       else{
            sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
            CWRITELOG(L_HDR,0,out_string);
            CWRITELOG(L_HDR,0,
"$GreNotifyTransformChangeRC                                     -    FAIL\n");
       }
       CWRITELOG(L_LOTRACE,0,"+ex+ GreNotifyTransformChangeRC - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetupDCRC()
 *
 * Purpose: This entry point executes the return codes test for GreSetupDCExh
 *          Return Codes Tested:
 *              PMERR_INV_HDC
 *              PMERR_HDC_BUSY
 *          Return Codes Not Tested:
 *              PMERR_DEV_FUNC_NOT_INSTALLED
 *              PMERR_COORDINATE_OVERFLOW
 *              PMERR_INSUFFICIENT_MEMORY
 *              PMERR_INV_LENGTH_OR_COUNT
 *              PMERR_INV_RECT
 *              PMERR_INV_COORDINATE
 *              PMERR_INV_HRGN
 *              PMERR_HRGN_BUSY
 *              PMERR_INV_REGION_CONTROL
 *              PMERR_INV_COORD_SPACE
 *          Return Codes Not Able To Reproduce:
 *              PMERR_REGION_IS_CLIP_REGION
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 12/05/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: None.
 *
 * Subroutines Required: CWRITELOG(), LOG_SIGNATURE, CHECKRETURNERROR()
 *
 * Limitations:  Does not test all possible return codes.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // test for Invalid Device Context Handle  PMERR_INV_HDC
 *    bad_hDC = meaningless value;
 *
 *    // call function
 *    return_code = GreSetupDC(bad_hDC,hrgnVis,xOrg,yOrg,prcBounds,flOptions);
 *
 *    // report error value to DTT_LOG
 *    CHECKRETURNERROR(return_code,"GreSetupDC PMERR_INV_HDC",SEVERITY_ERROR,
 *                     PMERR_INV_HDC);
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetupDCRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC     bad_hDC,
          hDC;
  LONG    return_code;
  HRGN    hrgnVis;
  LONG    xOrg,
          yOrg;
  RECTL   prcBounds;
  ULONG   flOptions;
  LONG    lErrorCount;
  CHAR    out_string[100];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else{
       hDC = set_delta(hPS);
       lErrorCount = 0;
       LOG_SIGNATURE;
       CWRITELOG(L_LOTRACE,0,"+ee+ GreSetupDCRC - Entry.\n");
       /*
        * Test the following Gre call for an invalid device context
        * handle error.  The hDC passed is invalid if a 0 is passed.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreSetupDC PMERR_INV_HDC Test - Entry.\n");
       bad_hDC = 0;
       return_code = GreSetupDC(bad_hDC,hrgnVis,xOrg,yOrg,&prcBounds,flOptions);
       lErrorCount += CHECKRETURNERROR(return_code,"GreSetupDC PMERR_INV_HDC",SEVERITY_ERROR,
                                       PMERR_INV_HDC);
       CWRITELOG(L_LOTRACE,0,"+ix+ GreSetupDC PMERR_INV_HDC Test - Exit.\n");

       /*
        * Test for hdc_busy return code.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreSetupDC PMERR_HDC_BUSY Test - Exit.\n");
       prcBounds.xLeft = 625;
       prcBounds.yBottom = 1250;
       prcBounds.xRight = 5000;
       prcBounds.yTop = 1925;
       hrgnVis = GpiCreateRegion( hPS, 1L, &prcBounds);
       xOrg = 0;
       yOrg = 0;

        //    BUSY CODE STARTS
       if(!MAKEPSBUSY(hPS)){
         CWRITELOG(L_FAIL,0,"GetBusy failed\n");
       }
       return_code=GreSetupDC(hDC, hrgnVis, xOrg, yOrg, &prcBounds,
                              SETUPDC_VISRGN | SETUPDC_ACCUMBOUNDSOFF |
                              SETUPDC_CLEANDC);
       lErrorCount += CHECKRETURNERROR(return_code,"GreSetupDC PMERR_HDC_BUSY",SEVERITY_ERROR,
                                       PMERR_HDC_BUSY);
       FREEPS;
       //    BUSY CODE ENDS
       GpiDestroyRegion(hPS, hrgnVis);
       CWRITELOG(L_LOTRACE,0,"+ix+ GreSetupDC PMERR_HDC_BUSY Test - Exit.\n");

       /*
        * Test for region_is_clip_region return code.
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreSetupDC PMERR_REGION_IS_CLIP_REGION Test - Exit.\n");
       if(GreSelectClipRegion(hDC,NULL,NULL) == RGN_ERROR)
           lErrorCount += BADERROR("SelectClipRegion",FALSE,NULL);
       prcBounds.xLeft = 625;
       prcBounds.yBottom = 1250;
       prcBounds.xRight = 5000;
       prcBounds.yTop = 1925;
       hrgnVis = GpiCreateRegion( hPS, 1L, &prcBounds);
       if(GreSelectClipRegion(hDC,hrgnVis,NULL) == RGN_ERROR)
           lErrorCount += BADERROR("SelectClipRegion",FALSE,NULL);
       xOrg = 0;
       yOrg = 0;

       return_code=GreSetupDC(hDC, hrgnVis, xOrg, yOrg, &prcBounds,
                              SETUPDC_VISRGN | SETUPDC_ACCUMBOUNDSOFF |
                              SETUPDC_CLEANDC);
       lErrorCount += CHECKRETURNERROR(return_code,"GreSetupDC PMERR_REGION_IS_CLIP_REGION",SEVERITY_ERROR,
                                       PMERR_REGION_IS_CLIP_REGION);
       CWRITELOG(L_LOTRACE,0,"+ix+ GreSetupDC PMERR_REGION_IS_CLIP_REGION Test - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULL
                                      ,NULL, NULL,NULL);

       if(!lErrorCount){
           CWRITELOG(L_HDR,0,
"$GreSetupDCRC                                                   -    PASS\n");
       }
       else{
           sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreSetupDCRC                                                   -    FAIL\n");
       }
       CWRITELOG(L_LOTRACE,0,"+ex+ GreSetupDCRC - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreGetDCOriginRC()
 *
 * Purpose: This entry point executes the return codes test for GreGetDCOriginExh
 *          Return Codes Tested:
 *              PMERR_INV_HDC
 *
 * System Requirements: Section 6.5.3
 *
 * Revision Log: 12/05/90, PDVT, Donna Williams
 *                     - initial creation
 *
 * Inputs: hPS              - presentation space
 *         hWndClient       - client window handle
 *         SelectionCall    - selection indicator
 *
 * Outputs: None.
 *
 * Subroutines Required:  CWRITELOG(), LOG_SIGNATURE, CHECKRETURNERROR().
 *
 * Limitations:  Does not test all possible return codes.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // test for Invalid Device Context Handle  PMERR_INV_HDC
 *    bad_hDC = meaningless value;
 *
 *    // call function
 *    return_code = GreGetDCOrigin(bad_hDC,pptOrigin);
 *
 *    // report error value to DTT_LOG
 *    CHECKRETURNERROR(return_code,"GreGetDCOrigin PMERR_INV_HDC",SEVERITY_ERROR,
 *                     PMERR_INV_HDC);
 * // test GreSetupDC
 *  GreSetupDC(hDC,hrgnVis,ptl.x,ptl.y,&prcBounds,
 *             SETUPDC_ORIGIN | SETUPDC_ACCUMBOUNDSON |
 *             SETUPDC_SETOWNER);
 * // report any errors
 *
 * // test GreGetDCOrigin
 *    GreGetDCOrigin(hDC,&ptlComp);
 * // report any errors
 * // verify that value received was the origin that was set
 *
 * // test rest of attributes
 *    GreSetupDC(hDC,hrgnVis,ptl.x,ptl.y,&prcBounds,
 *                  SETUPDC_VISRGN | SETUPDC_ACCUMBOUNDSOFF |
 *                  SETUPDC_CLEANDC);
 *\\end
 ***************************************************************************/
VOID APIENTRY GreGetDCOriginRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC       bad_hDC,
            hDC;
  PPOINTL   pptOrigin;
  LONG      return_code;
  LONG      lErrorCount;
  CHAR      out_string[100];
  POINTL    ptl,
            ptlComp;
  HRGN      hrgnVis;
  RECTL     prcBounds;


  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else{
       hDC = set_delta(hPS);
       lErrorCount = 0;
       LOG_SIGNATURE;
       CWRITELOG(L_LOTRACE,0,"+ee+ GreGetDCOriginRC - Entry.\n");
       /*
        * Test the following Gre call for an invalid device context
        * handle error.  The hDC passed is invalid if a 0 is passed.
        */
       /*
        * Test GreSetupDC
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreSetupDC - Entry\n");
       prcBounds.xLeft = 625;
       prcBounds.yBottom = 1250;
       prcBounds.xRight = 5000;
       prcBounds.yTop = 1925;
       hrgnVis = GpiCreateRegion( hPS, 1L, &prcBounds);
       ptl.x = 0;
       ptl.y = 0;
       if(!GreSetupDC(hDC,hrgnVis,ptl.x,ptl.y,&prcBounds,
                       SETUPDC_ORIGIN | SETUPDC_ACCUMBOUNDSON |
                       SETUPDC_SETOWNER) ){
          lErrorCount += BADERROR("GreSetupDC",FALSE,NULL);
       }
       CWRITELOG(L_LOTRACE,0,"+ix+ GreSetupDC - Exit\n");
       /*
        * Test GreGetDCOrigin
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreGetDCOrigin - Entry\n");
       if(!GreGetDCOrigin(hDC,&ptlComp) ){
          lErrorCount += BADERROR("GreGetDCOrigin",FALSE,NULL);
        }
       if( (ptl.x != ptlComp.x) || (ptl.y != ptlComp.y) ){
          lErrorCount += BADERROR("GreGetDCOrigin Values",FALSE,NULL);
        }
       CWRITELOG(L_LOTRACE,0,"+ix+ GreGetDCOrigin - Exit\n");
       /*
        *  Test GreSetupDC
        */
       CWRITELOG(L_LOTRACE,0,"+ie+ GreSetupDC - Entry\n");
       if(!GreSetupDC(hDC,hrgnVis,ptl.x,ptl.y,&prcBounds,
                       SETUPDC_VISRGN | SETUPDC_ACCUMBOUNDSOFF |
                       SETUPDC_CLEANDC) ){
          lErrorCount += BADERROR("GreSetupDC",FALSE,NULL);
       }
       CWRITELOG(L_LOTRACE,0,"+ix+ GreSetupDC - Exit\n");

       CWRITELOG(L_LOTRACE,0,"+ie+ GreGetDCOriginPMERR_INV_HDC Test - Entry.\n");
       bad_hDC = 0;
       return_code = GreGetDCOrigin(bad_hDC,pptOrigin);
       lErrorCount += CHECKRETURNERROR(return_code,"GreGetDCOrigin PMERR_INV_HDC",SEVERITY_ERROR,
                                    PMERR_INV_HDC);
       CWRITELOG(L_LOTRACE,0,"+ix+ GreGetDCOrigin PMERR_INV_HDC Test - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULL
                                      ,NULL, NULL,NULL);

       if(!lErrorCount){
           CWRITELOG(L_HDR,0,
"$GreGetDCOriginRC                                               -    PASS\n");
       }
       else{
            sprintf(out_string,"$ERRORCOUNT = %d\n",lErrorCount);
            CWRITELOG(L_HDR,0,out_string);
            CWRITELOG(L_HDR,0,
"$GreGetDCOriginRC                                               -    FAIL\n");
       }
       CWRITELOG(L_LOTRACE,0,"+ex+ GreGetDCOriginRC - Exit.\n");
  }
}
/***************************************************************************
 *\\ddd
 * Routine Name: display_text
 *
 * Purpose: This function displays string in clipping area.
 *
 * Revision Log: 11/16/90, PDVT, Gisselle Labault
 *                 - testcase design
 *               11/16/90, PDVT, Gisselle Labault
 *                 - design implementation
 *
 * Inputs:
 *   hPS        // presentation space
 *   column     // column position
 *   row        // row position
 *   str_msg    // string to be display
 *
 * Outputs: none
 *
 * Subroutines Required: GpiMove(), GpiCharString(), strlen()
 *
 * Limitations: none
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 * {
 *   set i to zero
 *   while i is less than or equal row
 *     display string
 *     get to next row
 *   enddo
 * }
 *\\end
 ***************************************************************************/

VOID display_text(HPS hPS, LONG column, LONG row, CHAR str_msg[])
{
  POINTL   ptl;
  INT      i=0;

  ptl.y = MapY(0);
  /*
   * display text/characters in clipping area
   */
  while(ptl.y <= row)
  {
    ptl.x = column;
    GpiMove(hPS, &ptl);
    GpiCharString(hPS, (LONG)strlen(str_msg), str_msg);
    i = i + 250;
    ptl.y = MapY(i);
  }
}

