/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/*const char header=$header$*/

#define INCL_DEV                    //Include all DEV definitions
#define INCL_WIN                    //Include all WIN definitions
#define INCL_GREALL                 // include all Gre definitions
#define INCL_GPI                    //Include all Gpi definitions
#define INCL_ERRORS                 //Include all error definitions
#define INCL_DOSMEMMGR
#define INCL_DOSERRORS
#define INCL_DOSFILEMGR              // File Management
#define INCL_DOSMEMMGR               //Memory Management
#define INCL_DDIMISC
#define INCL_DDIBUNDLES


#define MAINSOURCEFILE              //This main source code.

#include <string.h>
#include <stdlib.h>
#include <os2.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#include <pmddi.h>                  // gre - device driver includes
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "grebit2.h"                // bit map dll header file

/****************************************************************************
 *\\ddd
 * Subtask Name: GreBit2.DLL
 *
 * Module name: GreBit2.c
 *
 * Purpose: Gre Bitmap DLL test functions
 *
 * Revision Log:  09/15/91 ,PDVT ,Mike Abuzant
 *               -Taken from GpiBtmp.DLL Test Case
 *               - Test Case Design
 *
 *
 * SAD References:
 *
 * SRS References: Display Driver Verification Tool
 *
 * Contents:
 *
 * Design Overview:
 *   This TestCase is designed to test Gre Bitmap manipulation functions.These
 *   range form Creating,Deleting,Querying, and Selecting bitmaps to functions
 *   that actually manipulate the bitmaps. Functions tested in this testcase
 *   are listed below :
 *
 *     GreBitApp1()
 *     GreBitApp2()
 *     GreBitApp3()
 *     GreBitApp4()
 *     GreSelectBitmapExh()
 *     GreDeviceSetCursorExh()
 *     GreSetBitmapIDExh()
 *     GreGetBitmapParametersExh()
 *     GreSetBitmapDimensionExh()
 *     GreCopyDCLoadDataExh()
 *     GreRestoreScreenBitsExh()
 *     GreCopyDCLoadDataRc()
 *     GreRestoreScreenBitsRc()
 *     GreDeviceSetCursorRc()
 *     GreSetBitmapOwnerRc()
 *     GreSaveScreenBitsRc()
 *     GreQueryBitmapHandleRc()
 *     GreSetBitmapIdRc()
 *     GreCreateBitmapRc()
 *     GreDeleteBitmapRc()
 *     GreSetBitmapDimensionRc()
 *     GreGetBitmapDimensionRc()
 *     GreSelectBitmapRc()
 *     GreGetBitmapParametersRc()
 *
 *
 * Limitations:
 *
 *     1. GreCopyDCLoadData() can not copy bitmaps that are attached to pattern
 *     IDs. It returns the error code PMERR_INV_FONT_ATTR that causes the
 *     function GreDeleteBitmap() to return with the error code
 *     PMERR_BITMAP_IS_SELECTED. GreDeleteBitmap()returns the error code in
 *     question only when deleting the first bitmap.
 *
 *     2. The function GreSaveScreenBits()returns with a zero to indicate
 *        that the calling routine must save screen bits on its own,thus
 *        the function GreRestoreScreenBits()can't be tested since it
 *        depends on a handle returned by GreSaveScrrenBits().
 *
 *     3. When using GreSetBitmapID(), the bitmap format must be one of
 *        the bitmap formats that are supported by the device;otherwise,
 *        a PMERR_INV_HBITMAP is generated by the device driver.
 *        An automatic format conversion occurs only when the bitmap is
 *        created and initialized by GreCreateBitmap().
 *
 *     4. The functions GreSaveScreenBits()and GreRestoreScreenBits() will
 *        return with PMERR_DEV_FUNC_NOT_INSTALLED error code when the
 *        Device Context is of a printer.
 *
 *     5. Bitmap used as  patterns should be 8 X 8 pels; otherwise,
 *        the first 8 pels will be used as the pattern.
 *
 *     6. Bitmaps do not work properly  with PM_Q_STD type print devices.
 *        The bitmap functions do not return any error codes when the print
 *        device is configured as PM_Q_STD instead, a blank page is printed.
 *
 *     7. The function GreDeviceSetCursor()is not implemented due to
 *        insufficeint documentation on this particular function.
 *
 *     8. The function GreSetBitmapOwner() is not implemented since due to
 *        fact that this function is not documented.
 *
 *
 *\\end
****************************************************************************/
/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose:  Initialize function name and address array for DTT.
 *
 * System Requirements:
 *
 * Revision Log:
 *               02/03/91, pdvt, Mike Abuzant
 *               - Entry point names and address are now
 *               - put in an array.
 * Inputs:
 *   sel          // passed through to the InitTestSegVars function
 *   init_info_p  // name and address array used by DTT interface
 *
 * Outputs:
 *   init_info_p  // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()    // used by the DTT interface
 *
 * Limitations:
 *    None
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *   for the number of test functions {
 *       set function name
 *       set function address
 *   }
 *   call init set function
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY InitTest(PVOID Selector, PTESTINITINFO init_info_p)
{
  register USHORT usTestCounter;

  init_info_p->ntests = NTESTS;
  strcpy(init_info_p->group_name,TestCaseName); //Testcase Name

  for (usTestCounter=0 ;usTestCounter< NTESTS; ++usTestCounter){

     init_info_p->test_addrs[usTestCounter]=
                               EntryPointList[usTestCounter].TestAddress;
     strcpy(init_info_p->test_names[usTestCounter],
                               EntryPointList[usTestCounter].TestName);
  }

  InitTestSegVars(Selector);
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreBitApp1()
 *
 *  Purpose :This is an entry point for the DTT.EXE that causes Gre
 *            Application tests  to be run.
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/18/1991 ,PDVT , Mike Abuzant
 *               - TestCase coding and design
 *
 * Inputs:
 *    hps              // presentation space
 *    hWndClient     // client window
 *    SelectionCall   // selection parameter
 *
 * Outputs:
 *
 * Subroutiynes Required:
 *     GreApp1().
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   // call the functionality  subfunction
 *     GpiApp1( hPS, hWndClient );
 *
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreBitApp1(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    (VOID)GreApp1( hPS, hWndClient );

  }
    return ;
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreApp1()
 *
 * Purpose: This function tests the basic functionality of the following
 *                    Gre functions:
 *
 *          GreSetBitmapID()
 *          GreQueryBitmapHandle()
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/18/1991 ,PDVT , Mike Abuzant
 *               -initial file.
 *               - Test case design and conding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *
 * Outputs:
 *
 * Subroutines Required:
 *   None.
 *
 * Limitations:
 *   When using GreSetBitmapID(), the bitmap format must be one of the
 *   bitmap format that is supported by the device. If the bitmap format
 *   is not one of the bitmap formats supported by the device,
 *   a PMERR_INV_HBITMAP is generated by the device. An automatic format
 *   conversion occures only when the bitmap is created and initialized by
 *   GreCreateBitmap().
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *    allocate memory for the bitmap information structure and
 *    the bitmap information header structure. Then,
 *    initialize bitmap information header structure and bitmap information
 *    structure to the size and characterisics with the bitmap to be created.
 *
 *    Setup conditions to draw a finite set of filled circles. Each circle will be
 *    filled with a different pattern. The patterns used here will be bitmaps
 *    that are created using GreCreateBitmap() and tagged with a pattern ID.
 *    The first circle's size will be as large as presentation space and then
 *    its dimensions are shortened by 1/2" from both X-direction and
 *    Y-direction until circles diameter is reduced to 1".
 *
 *    //Set back mix to OVERPAINT .
 *     GpiSetBackMix(hPS,BM_OVERPAINT)
 *
 *    FOR (index1=0; index2 < min(height,width) ;index1++) DO
 *          Create the next bitmap to use as a pattern.
 *            HBM=GreCreateBitmap(hDC...)
 *
 *          Associate an ID Tag newly created bitmap
 *            GreSetBitmapID(hDC,hBM,MY_BITMAP_ID).
 *
 *          Make this bitmap the default pattern to be used to fill circles.
 *            GpiSetPatternSet(hPS,MY_BITMAP_ID).
 *
 *          Setup conditions to draw the next circle which is 1/2" smaller
 *          than the previous circle .
 *               GpiBox(hPS, &TheOtherCorner);
 *
 *          Set pattern to the default Pattern.
 *            GpiSetPatternSet(hPS,LCID_DEFAULT).
 *
 *          Query the bitmap that is tagged to MY_BITMAP_ID. This
 *          will serve as a test for GreQueryBitmapHandle().
 *            hBM1=GreQueryBitmapHandle(hDC,MY_BITMAP_ID).
 *
 *          Un-tag the ID associated with current bitmap
 *            GreDeleteSetId(hBM,MY_BITMAP_ID).
 *
 *          Delete the current bitmap handle.
 *           GreDeletetBitmap(hBM);
 *        ENDDO
 *
 *\\end
 ***************************************************************************/
INT GreApp1(HPS hPS,HWND hWndClient)
{
  static MyBitMap  aBitMap[]={
                               {PAT1_HEIGHT,PAT1_WIDTH,abPat1},
                               {SHAPE_HEIGHT,SHAPE_WIDTH,abShape},
                               {PAT2_HEIGHT,PAT2_WIDTH,abPat2}
                             };

  static PCHAR EntryName="GreBitApp1()";
  BYTE  Temp_buffer[100];
  BITMAPINFOHEADER BitMapHeader;      //bitmap info header
  PBITMAPINFO      pBitMapInfo;       //bitmap info structure
  POINTL           ptl;
  HBITMAP          hBM=(HBITMAP)NULL ;
  HBITMAP          hBM2=(HBITMAP)NULL ;
  HDC              hDC;
  static SIZEL sizel={0L,0L};
  LONG  lIndex1,
        lIndex2;
  INT   ColorIndex=0,
        BitMapCount=0;

  ENTER_MAIN(EntryName);

  // Initalize the Bitmap information structure
  // first we need to  allocate memory to hold RGB array
  // since  BITMAPINFO structure is large enough to hold
  // one RGB structure.

   pBitMapInfo=(PBITMAPINFO) Temp_buffer;
   pBitMapInfo->cbFix=sizeof(BITMAPINFOHEADER); // This always =12
   pBitMapInfo->cPlanes=1;                      // one color plane
   pBitMapInfo->cBitCount=1;                    // 1-bit /pixel
   pBitMapInfo->argbColor[0].bBlue=
   pBitMapInfo->argbColor[0].bGreen=
   pBitMapInfo->argbColor[0].bRed=0;
   pBitMapInfo->argbColor[1].bBlue=
   pBitMapInfo->argbColor[1].bGreen=
   pBitMapInfo->argbColor[1].bRed=0xFF;

   BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
   BitMapHeader.cPlanes=                    // one color plane
   BitMapHeader.cBitCount=1;                  // 1-bit /pixel

   SETENTER("Setting background mix mode to OVERPAINT",NULL);

   if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
       CLEANUP("GpiQueryDevice()",EntryName);
       return FALSE;
   }

   //Set the background mix to OVERPAINT. This is important
   //since each filled circle is drawn inside the previously
   //drawn circles.
   if (GpiSetBackMix(hPS,BM_OVERPAINT)==FALSE){
          CLEANUP("GpiSetBackMix()",EntryName);
          return FALSE ;
   }
   SETEXIT();

   lIndex2=(LONG)min(pg_size_x,pg_size_y);

 /*************************************************************************
   Setup conditions to draw a finite set of filled circles. Each circle will
   be filled with a different pattern. The patterns used here will be bitmaps
   that are created using GreCreateBitmap() and tagged with a pattern ID.
   The first circle's size will be as large as presentation space and then
   its diameter is shortened by 1" until it(diameter) reaches 1" mark. Note
   that circles will be drawn using GpiBox() calls.
 *************************************************************************/

  for (lIndex1=0 ;lIndex1<lIndex2;lIndex1++){
     //Setup the dimensions for the next bitmap to be used
     //as filling pattern.
     pBitMapInfo->cx=
     BitMapHeader.cx=aBitMap[BitMapCount].width;
     pBitMapInfo->cy=
     BitMapHeader.cy=aBitMap[BitMapCount].height;

     //Create bitmap and initialize it with the data stored in the
     //buffer pointed to by 'BitMapBuffer'.
      SETENTER("Creating and setting bitmap to use as pattern filling ",NULL);

     if ((hBM=GreCreateBitmap(hDC,&BitMapHeader,
                             CBM_INIT,aBitMap[BitMapCount].BitMapBuffer
                            ,pBitMapInfo))==FALSE){
          CLEANUP("GreCreateBitmap()",EntryName);
          return FALSE ;
     }

     //Tag this bitmap with a pattern ID.
     if (GreSetBitmapID(hDC,hBM,MY_BITMAP_ID)==FALSE){
         CLEANUP("GreSetBitmapID()",EntryName);
         return FALSE;
     }

     //Next circle will be filled with a different bitmap pattern.
     INCROMENTWITHWRAP(BitMapCount,aBitMap);

     //Make This Tagged Pattern the defautl one.
     if (GpiSetPatternSet(hPS,MY_BITMAP_ID)==FALSE){
         CLEANUP("GpiSetPatternSet()",EntryName);
         return FALSE;
     }

     // Change default foreground color
     if (GpiSetColor(hPS,colors[ColorIndex])==GPI_ERROR){
        CLEANUP("GpiSetColor()",EntryName);
        return FALSE ;
     }
     SETEXIT();
     //Incroment the color counter
     INCROMENTWITHWRAP(ColorIndex,colors);

     //Setup coordinates for box first corner.
     ptl.x=MapX(lIndex1*(delta_x/2L));
     ptl.y=MapY(lIndex1*(delta_y/2L)+(delta_y*2L/3L));
     if (GpiMove(hPS,&ptl)==GPI_ERROR){
        CLEANUP("GpiMove()",EntryName);
        return FALSE ;
     }

     //Setup coordinates for the other box corner and draw a filled circle.
     ptl.x=MapX((LONG)(lIndex2*2 -lIndex1)*delta_x/2);
     ptl.y=MapY((LONG)((lIndex2*2-lIndex1)*delta_y/2)+(delta_y*2L/3L));

     if (GpiBox(hPS,DRO_OUTLINEFILL,&ptl,ptl.x,ptl.y)==GPI_ERROR){
         CLEANUP("GpiBox()",EntryName);
         return FALSE;
     }

     if (GpiSetPatternSet(hPS,LCID_DEFAULT)==FALSE){
         CLEANUP("GpiSetPatternSet()",EntryName);
         return FALSE;
     }

     //Query the bitmap handle tagged with this ID(MY_BITMAP_ID).
     if ((hBM2=GreQueryBitmapHandle(hDC,MY_BITMAP_ID))==GPI_ERROR){
         CLEANUP("GpiQueryBitmapHandle()",EntryName);
         return FALSE;
     }

     SETENTER("Deleting current bitmap",NULL);
     //Un-tag this bitmap ID.
     if (GreDeleteSetId(hDC,MY_BITMAP_ID)==FALSE){
         CLEANUP("GreDeleteSetId()",EntryName);
         return FALSE;
     }

    // Delete this bitmap handle.
     if (GreDeleteBitmap(hBM2)==FALSE){
         CLEANUP("GreDeleteBitmap()",EntryName);
         return FALSE;
     }
     SETEXIT();
  }

  EXIT_MAIN(EntryName);
  return TRUE;
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetBitmapIDExh()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions
 *           GreSetBitmapID().
 *           GreQueryBitmapHandle().
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/19/91, pdvt ,Mike Abuzant
 *                  -Initial file creation
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *     None.
 * Subroutines Required:
 *    None
 *
 * Limitations:
 *   1. The bitmap that is used as a pattern should be 8 X 8 pels, otherwise;
 *   if it is larger than 8 X 8 pels , the first 8 pels will be used as
 *   the pattern.
 *
 *   2. If a bitmap format is not one of the formats supported by the device,
 *      the call GreSetBitmapID() fails since this bitmap will not be
 *      converted to a bitmap format supported by the device. The only way
 *      to convert a bitmap format to a format supported by the device is
 *      to create the bitmap using GreCreateBitmap() with the option
 *      CBM_INIT.
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *    allocate memory for the bitmap information structure and
 *    the bitmap information header structure. Then,
 *    initialize bitmap information header structure and bitmap information
 *    structure to the size and characterisics with the bitmap to be created.
 *
 *    FOR (index1=0; index2 < (display height ;index1+=delta_y) DO
 *      FOR (index2=0; index2 < (display width; index2+=delta_x) DO
 *          Create the next bitmap to use as a pattern .
 *            HBM=GreCreateBitmap(hDC...)
 *
 *          Associate an ID Tag newly created bitmap
 *            GreSetBitmapID(hDC,hBM,MY_BITMAP_ID).
 *
 *          Make this bitmap the default pattern used
 *          to fill circles
 *            GpiSetPatternSet(hPS,MY_BITMAP_ID).
 *
 *          Setup conditions to draw circles 1" in diameter.these circles
 *          will be filled with current bitmap.
 *
 *          Set pattern to the default Pattern.
 *            GpiSetPatternSet(hPS,LCID_DEFAULT).
 *
 *          Query the bitmap that is tagged to MY_BITMAP_ID. This
 *          will serve as a test for GreQueryBitmapHandle().
 *            hBM1=GreQueryBitmapHandle(MY_BITMAP_ID).
 *
 *          Un-tag the ID associated with current bitmap
 *            GreDeleteSetId(hBM,MY_BITMAP_ID).
 *
 *          De-Select the current bitmap form the device context passing
 *          NULL as a bitmap handle.This will serve as De-selecting.
 *           GreSelectBitmap(hDC,NULL);
 *
 *          Delete the current bitmap handle.
 *           GreDeletetBitmap(hBM);
 *        ENDDO
 *       ENDDO
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetBitmapIDExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static MyBitMap  aBitMap[]={
                               {PAT1_HEIGHT,PAT1_WIDTH,abPat1},
                               {SHAPE_HEIGHT,SHAPE_WIDTH,abShape},
                               {PAT2_HEIGHT,PAT2_WIDTH,abPat2}
                              };
  static PCHAR EntryName="GreSetBitmapIDExh()";
  BYTE  Temp_buffer[100];
  BITMAPINFOHEADER BitMapHeader;      //bitmap info header
  PBITMAPINFO      pBitMapInfo;       //bitmap info structure

  CHAR             Message[200];
  POINTL   ptl;
  HBITMAP hBM=(HBITMAP)NULL ;
  HBITMAP hBM1=(HBITMAP)NULL ;
  HDC     hDC;
  LONG  lIndex1,
        lIndex2,
        Counter=1;
  INT   ColorIndex=0,
        LastBitMap=0,
        BitMapCount=0;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    SETENTER("Query Current hDC assoicated with this PS",NULL);

    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
         CLEANUP("GpiQueryDevice()",EntryName);
         return ;
    }
    SETEXIT();
    /*=========================================================
     Initalize the Bitmap information structure
     first we need to  allocate memory to hold RGB array
     since  BITMAPINFO structure is large enough to hold
     one RGB structure.
    ==========================================================*/
      pBitMapInfo=(PBITMAPINFO)Temp_buffer;
      pBitMapInfo->cbFix=sizeof(BitMapHeader);   // This always =12
      pBitMapInfo->cPlanes=                      // one color plane
      pBitMapInfo->cBitCount=1;                  // 1-bit /pixel
      pBitMapInfo->argbColor[0].bBlue=
      pBitMapInfo->argbColor[0].bGreen=
      pBitMapInfo->argbColor[0].bRed=0;
      pBitMapInfo->argbColor[1].bBlue=
      pBitMapInfo->argbColor[1].bGreen=
      pBitMapInfo->argbColor[1].bRed=0xFF;

      BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
      BitMapHeader.cPlanes=                      // one color plane
      BitMapHeader.cBitCount=1;                  // 1-bit /pixel


      lIndex1=(LONG)(delta_y*2L/3L); //Avoid the test sigenture printed
                                    //at bottom of display.

     /*=========================================================
      This next code segment is used to fill the entire display
      surface with rows of 1" filled circles. the patterns
      used to fill these circles are actually bitmaps that are
      tagged to pattern IDs. The following step are taken in
      order to use bitmaps as patterns:

      1.Create the bitmap and initialize it with the desired
        data. Use GreCreatBitmap().
      2.Tag the bitmap with a pattern ID. Use GreSetBitmapID().
      3.Activate the tagged pattern ID,i.e bitmap becomes the
        default filling pattern. Use GpiSetPatternSet().
     ==========================================================*/

     for (;lIndex1<=(pg_size_y*delta_y); lIndex1+=delta_y){
        BitMapCount=LastBitMap;
        sprintf(Message," Using bitmap as fill pattern. ROW #%d",Counter++);
        SUBENTER(GreSetBitmapID, Message);

        for (lIndex2=0 ;lIndex2<(pg_size_x*delta_x); lIndex2+=delta_x){

            pBitMapInfo->cx=
            BitMapHeader.cx=aBitMap[BitMapCount].width;
            pBitMapInfo->cy=
            BitMapHeader.cy=aBitMap[BitMapCount].height;

            //Create the next bitmap to be used as pattern.
            if ((hBM=GreCreateBitmap(hDC,&BitMapHeader,
                                   CBM_INIT,aBitMap[BitMapCount].BitMapBuffer
                                  ,pBitMapInfo))==GPI_ERROR){
                 CLEANUP("GreCreateBitmap()",EntryName);
                 return ;
            }

            //Tag this bitmap with a pattern ID.

            if (GreSetBitmapID(hDC,hBM,MY_BITMAP_ID)==FALSE){
                ErrorCount++;
                ERRSTATUS(TRUE,"GreSetBitmapID()",NULL);
                return;
            }

            //Next circle is filled with different pattern.
            INCROMENTWITHWRAP(BitMapCount,aBitMap);

            //Activate the bitmap as the default filling pattern.
            if (GpiSetPatternSet(hPS,MY_BITMAP_ID)==FALSE){
                CLEANUP("GpiSetPatternSet()",EntryName);
                return;
            }

            // Set foreground color to a different color than default_color
            if (GpiSetColor(hPS,colors[ColorIndex])==GPI_ERROR){
               CLEANUP("GpiSetColor()",EntryName);
               return ;
            }
            INCROMENTWITHWRAP(ColorIndex,colors);

            //
            // Draw a filled circle.
            //

            ptl.x=MapX(lIndex2);
            ptl.y=MapY(lIndex1);
            if (GpiMove(hPS,&ptl)==GPI_ERROR){
               CLEANUP("GpiMove()",EntryName);
               return ;
            }

            ptl.x=MapX(lIndex2+delta_x);
            ptl.y=MapY(lIndex1+delta_y);
            if (GpiBox(hPS,DRO_OUTLINEFILL,&ptl,ptl.x,ptl.y)==GPI_ERROR){
                 CLEANUP("GpiBox()",EntryName);
                 return ;
            }

            //Prepare to delete the tagged bitmap.
            if (GpiSetPatternSet(hPS,LCID_DEFAULT)==FALSE){
                CLEANUP("GpiSetPatternSet()",EntryName);
                return;
            }

            //Make sure that the pattern ID is tagged with the correct
            //bitmap. This will serve as a test for GreQueryBitmapHandle().
            if ((hBM1=GreQueryBitmapHandle(hDC,MY_BITMAP_ID))==GPI_ERROR){
                ErrorCount++;
                ERRSTATUS(TRUE,"GpiQueryBitmapHandle()",NULL);
                CLEANUP("GreQueryBitmapHandle()",EntryName);
            }

            if (hBM1 != hBM ){
              LOGINFO(L_HDR,"*","GpiQueryBitmapHandle() Failed\n");
            }

            //Un-tag the bitmap
            if (GreDeleteSetId(hDC,MY_BITMAP_ID)==FALSE){
                CLEANUP("GreDeleteSetId()",EntryName);
                return;
            }

            //Releas the memory space occupied by this bitmap
            if (GreDeleteBitmap(hBM1)==FALSE){
                CLEANUP("GreDeleteBitmap()",EntryName);
                return;
            }
        }
        INCROMENTWITHWRAP(ColorIndex,colors);
        INCROMENTWITHWRAP(LastBitMap,aBitMap);
        SUBEXIT();

     }
     EXIT_MAIN(EntryName);
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetBitmapDimensionExh()
 *
 * Purpose:  This function exhaustively tests the functionality of
 *            The Gpi functions,
 *            GreSetBitmapDimension()
 *            GreGetBitmapDimension()
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/19/91, PDVT ,Mike Abuzant
 *               - initial file
 *               09/19/91, PDVT ,Mike Abuzant
 *                  -Test Case Implementation and design.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *       None.
 * Subroutines Required:
 *     set_delta();
 *
 * Limitations:
 *    None at this time.
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   Obtain a bitmap handle to one of the system bitmap.
 *      hBM=WinGetSysBitmap(HWND_DESKTOP,SBMP_SYSMENU)
 *
 *   Initialize  Bitmap dimensions.
 *      sizel.cx=10;
 *      sizel.cy=10;
 *
 *   WHILE ( Top of display is not reached ) DO
 *
 *      Set the bitmap dimensions .
 *        GreSetBitmapDimension(hBM,&sizel...)
 *
 *      Print a message informing the user that the bitmap's dimensions
 *      have been set to some values .
 *       GpiCharStringAt(hPS,Message,...);
 *
 *      Advance to next line position .
 *        ptl.y+=syPosition
 *
 *      Query the bitmap dimensions .
 *        GreQueryBitmapDimension(hBM,&sizel1...)
 *
 *      Print a message informing the user that the bitmap's dimensions
 *      have been queried using the function GpiQueryBitmapDimension().
 *        GpiCharStringAt(hPS,Message,...);
 *
 *      Advance to next line position.
 *        ptl.y+=(3*syPosition/2);
 *
 *      Incroment the bitmap dimensions.
 *        sizel.cx+=20;
 *        sizel.cx+=10;
 *
 *   ENDWHILE
 *
 *   Delete the bitmap.
 *      GreDeletetBitmap(hBM);
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetBitmapDimensionExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static PCHAR EntryName="GreSetBitmapDimensionExh()";
   POINTL  ptl;
   CHAR   Message[120];
   HBITMAP hBM=(HBITMAP)NULL ;
   SIZEL sizel;
   SIZEL sizel1;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
      ENTER_MAIN(EntryName);
      srand(1);

      SETENTER("Getting a system Bitmap for testing",NULL);
      //Obtain  a handle of on of the system bitmaps.
      if ((hBM=WinGetSysBitmap(HWND_DESKTOP,SBMP_SYSMENU))==NULL){
          CLEANUP("WinGetSysBitmap()",EntryName);
          return;
      }
      SETEXIT();

      ptl.x=MapX(delta_x/5);
      ptl.y=MapY(delta_y);

      while ( (LONG)UnMapY(ptl.y) < (LONG)(delta_y*pg_size_y)) {

        //Set the bitmap dimensions.
         sizel.cx=rand();
         sizel.cy=rand();

        if (GreSetBitmapDimension(hBM,&sizel)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,EntryName,"GreSetBitmapDimension");
        }

        sprintf(Message,"Setting current Bitmap Dimensions to (%05ld mm, %05ld mm).",
                            sizel.cx,sizel.cy);
        WorldWriteString(ptl.x,ptl.y,Message);

        sprintf(Message,"Set Dimensions (%05ld mm, %05ld mm).",
                            sizel.cx,sizel.cy);
        LOGINFO(L_DEBUG,"*",Message);
        ptl.y += syPosition; //Advance to next line position.

        //Query the bitmap dimensions.
        if (GreGetBitmapDimension(hBM,&sizel1)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,EntryName,"GreGetBitmapDimension");

        }
        sprintf(Message,"Queried Bitmap Dimensions for current bitmap are (%05ld mm, %05ld mm)",
                            sizel1.cx,sizel1.cy);

        WorldWriteString(ptl.x,ptl.y,Message);

        sprintf(Message,"Queried Dimensions (%05ld mm, %05ld mm).",
                            sizel.cx,sizel.cy);
        LOGINFO(L_DEBUG,"*",Message);

        if ((sizel1.cx!=sizel.cx) ||(sizel1.cy!=sizel.cy)){
           ErrorCount++;
           ERRSTATUS(TRUE,"Queried dimensions are not equal to "
                              "set dimensions.",NULL);
        }
        ptl.y +=(LONG)(3*syPosition/2); //Advance to next line position.

      }

      SETENTER("Releasing Memeory allocated for system bitmap",NULL);
      //Release memory space occupied by this bitmap.
      if (GreDeleteBitmap(hBM)==FALSE){
         CLEANUP("GreDeleteBitmap()",EntryName);
         return ;
      }

      SETEXIT();

      EXIT_MAIN (EntryName);
  }
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreGetBitmapParametersExh()
 *
 * Purpose:  This function exhaustively tests the functionality of
 *            The Gre function GreGetBitmapParameters().
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/19/91, PDVT ,Mike Abuzant
 *               - initial file
 *               09/19/91, PDVT ,Mike Abuzant
 *                  -Test Case Design and Implementation.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *       None.
 * Subroutines Required:
 *    None.
 *
 * Limitations:
 *    None at this time.
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *
 *   WHILE ( Top of display is not reached ) DO
 *
 *      Randomly select bitmap width and height.

 *      Incroment the bitmap parameters(BitCount,ColorPlanes ,etc...)
 *      and Create a bitmap .
 *         GreCreateBitmap(hDC,&NewParameters,...);
 *      Print a message informing the user that a bitmap has
 *      been created with these parameters.
 *         GpiCharStringAt(hPS,Message,...);
 *
 *      Advance to next line.
 *         ptl.y+=syPosition
 *
 *      Query the bitmap Parameters.
 *        GreGetBitmapParameters(hBM,&Parameter.)
 *
 *      Print a message informing the user that the following
 *      parameters have been retrieved using  GreGetBitmapParameters().
 *         GpiCharStringAt(hPS,Message,...);
 *
 *      Advance to next line .
 *          ptl.y+=(3*syPosition/2);
 *
 *   ENDWHILE
 *
 *   Delete the bitmap.
 *      GreDeletetBitmap(hBM);
 *
 *\\end
 ***************************************************************************/


VOID  APIENTRY GreGetBitmapParametersExh(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
   static PCHAR EntryName="GreGetBitmapParametersExh()";
   POINTL      ptl;
   CHAR        Message[120];
   HBITMAP hBM=(HBITMAP)NULL ;
   BITMAPINFOHEADER BitMapHeader;      //bitmap info header

   USHORT      Planes=0,
               BitCount=0,
               Width,
               Height;
   HDC         hDC;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
      ENTER_MAIN(EntryName);

      SETENTER("Query Current hDC assoicated with this PS",NULL);

      if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
          CLEANUP("GpiQueryDevice()",EntryName);
          return ;
      }
      SETEXIT();

      ptl.x=MapX(delta_x/5);
      ptl.y=MapY(delta_y);

      srand(1);

      while ( (LONG)UnMapY(ptl.y) < (LONG)(delta_y*pg_size_y)) {

       Width=GETRANDOM(1,MAX_RANDOM);          //Get A random bitmap width.
       Height=GETRANDOM(1,MAX_RANDOM);         //Get a random bitmap height.
       Planes=GETRANDOM(1,10);
       BitCount=GETRANDOM(1,24);

        // Initalize the Bitmap Header structure
        BitMapHeader.cbFix=sizeof(BITMAPINFOHEADER);   // This always =12
        BitMapHeader.cPlanes=Planes;
        BitMapHeader.cBitCount=BitCount;
        BitMapHeader.cx=(USHORT)Width;
        BitMapHeader.cy=(USHORT)Height;

        // Create and obtain a bitmap handle to use in testing
        // GreGetBitmapParameters().
        if ((hBM=GreCreateBitmap(hDC,&BitMapHeader,
                                 NULL,NULL,NULL))==GPI_ERROR){
            CLEANUP("GreCreateBitmap()",EntryName);
            return ;
        }

        //Print a message informing the user that a bitmap has
       // been created with these parameters.
        sprintf(Message,"Creating a Bitmap : Width=%04d  Height=%04d "
                        "BitCount=%02d ColorPlanes=%02d",
                         BitMapHeader.cx,BitMapHeader.cy,
                         BitMapHeader.cBitCount,BitMapHeader.cPlanes);

        WorldWriteString(ptl.x,ptl.y,Message);

        ptl.y+=syPosition; // Advance to the next line

        sprintf(Message,"Bitmap Parameters: Width=%04d  Height=%04d "
                        "BitCount=%02d ColorPlanes=%02d",
                         BitMapHeader.cx,BitMapHeader.cy,
                         BitMapHeader.cBitCount,BitMapHeader.cPlanes);

        LOGINFO(L_DEBUG,"*",Message);

        //Retrieve the bitmap parameters.

        SUBENTER(GreGetBitmapParameters, "Get New bitmap parameters");
        if (GreGetBitmapParameters(hBM,&BitMapHeader)==FALSE){
            ErrorCount++;
            ERRSTATUS(TRUE,"GreGetBitmapParameters()",NULL);
        }

        SUBEXIT();

        //Print a message informing the user that the following
        //parameters have been retrieved using GreGetBitmapParameters().
        sprintf(Message,"Queried Parameters: Width=%04d  Height=%04d "
                        "BitCount=%02d ColorPlanes=%02d",
                         BitMapHeader.cx,BitMapHeader.cy,
                         BitMapHeader.cBitCount,BitMapHeader.cPlanes);

        LOGINFO(L_DEBUG,"*",Message);

        WorldWriteString(ptl.x,ptl.y,Message);

        if ((BitMapHeader.cx!=Width) || (BitMapHeader.cy!=Height)||
            (BitMapHeader.cBitCount!=BitCount) ||
            (BitMapHeader.cPlanes!=Planes)){
          ErrorCount++;
          ERRSTATUS(TRUE,"Queried Parameters are not equal to "
                              "set craetion parameters.",NULL);
        }

        ptl.y+=(LONG)(3*syPosition/2); //Advance to next line position.

        SETENTER("Releasing Memeory allocated for system bitmap",NULL);

        //Release memory space occupied by this bitmap.
        if (GreDeleteBitmap(hBM)==FALSE){
            CLEANUP("GreDeleteBitmap()",EntryName);
            return ;
        }
        SETEXIT();
      }
      EXIT_MAIN (EntryName);
  }
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSelectBitmapExh()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions
 *           GreCreateBitmap().
 *           GreSelectBitmap().
 *           GreDeleteBitmap().
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/20/91, pdvt ,Mike Abuzant
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *   None.
 * Subroutines Required:
 *   GreSelectBitmap_test1()
 *
 * Limitations:
 *   None.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *  // Call the functionality test routine
 *     GreSelectBitmap_test1(hPS , hWndClient)
 * }
 *\\end
 ***************************************************************************/

VOID APIENTRY GreSelectBitmapExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    GreSelectBitmap_test1(hPS ,hWndClient);
  }
}

/***************************************************************************
 *\\ddd
 * Routine Name: GreSelectBitmap_test1()
 *
 * Purpose:  This function will test the functionality of the following
 *           Gre functions
 *           GreCreateBitmap().
 *           GreSelectBitmap().
 *           GreDeleteBitmap().
 *
 * System Requirements: None
 *
 * Revision Log:
 *               09/20/91, pdvt ,Mike Abuzant
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *
 * Outputs:
 *    None
 * Subroutines Required:
 *   None.
 * Limitations:
 *  None.
 *
 *\\end
 ***************************************************************************/

/***************************************************************************
 *\\algorithm
 *
 *    allocate memory for the bitmap information structure and
 *    the bitmap information header structure. Then,
 *    initialize bitmap information header structure and bitmap information
 *    structure to the size and characterisics with the bitmap to be created.
 *
 *    open a memory device context since bitmaps belongs to 'OD_MEMORY'
 *    device context  ONLY
 *        HDC1=DevOpenDC(hAB,OD_MEMORY,...);
 *
 *    Create a new presentation space and associate it with the newly
 *    created device context.
 *        hPS1=GpiCreatePS(hAB,....);
 *
 *    Obtain the current device context
 *        hDC=DevQueryDevice(hPS....);
 *
 *    Obtain the width and the height of the current device
 *        height=alDevCaps[CAPS_HEIGHT];  //alDevCaps[] is provided by DTT.
 *        width =alDevCaps[CAPS_WIDTH ];
 *
 *  REPEAT (steps below 10 times) DO
 *
 *    DO (util you fill the entire Display)
 *       DO (until you fill one row of the display sureface;..... )
 *         Setup conditions to create a bitmap. This bitmap will be selected
 *         randomly.
 *         hBM=GreCreateBitmap(hDC1...)
 *
 *         Select bitmap into OD_MEMORY device context.
 *         GreSelectBitmap(hDC1,hBM.....)
 *
 *          Display the bitmap.
 *          GpiBitBlt(hPS,hPS1,ROP_SRCCOPY...);
 *
 *          De-Select the current bitmap form the device context passing
 *          NULL as a bitmap handle.This will serve as De-selecting.
 *          GreSelectBitmap(hDC1,NULL);
 *
 *          Delete the current bitmap handle.
 *               GreDeletetBitmap(hBM);
 *
 *       ENDDO
 *    ENDDO
 *  ENDREPEAT
 *
 *   Destory the presentation space.
 *     GpiDestoryPS(hPS1);
 *
 *   Close the device context
 *    DevCloseDC(hDC1);
 *
 *   return to DTT.EXE
 *
 *\\end
 ***************************************************************************/
VOID GreSelectBitmap_test1(HPS hPS,HWND hWndClient)
{
   static MyBitMap  aBitMap[]={
                               {CROSS_HEIGHT,CROSS_WIDTH,abCross},
                               {SHAPE_HEIGHT,SHAPE_WIDTH,abShape},
                               {CAT_HEIGHT,CAT_WIDTH,abCat},
                               {PAT1_HEIGHT,PAT1_WIDTH,abPat1},
                               {PAT2_HEIGHT,PAT2_WIDTH,abPat2}
                              };

   static PCHAR EntryName="GreSelectBitmapExh()";
   SIZEL sizel={0};          // size of bitmap to draw
   HDC   hDC=(HDC)NULL,
         hDC1=(HDC)NULL;

   HAB   hAB=(HAB)NULL;
   HBITMAP hBM=(HBITMAP)NULL;
   HPS   hPS1=(HPS)NULL;

   BYTE  Temp_buffer[100];
   BYTE  Message[200];

   BITMAPINFOHEADER BitMapHeader;      //bitmap info header
   PBITMAPINFO      pBitMapInfo;       //bitmap info structure

   POINTL  aptl[4]={0};
   INT color_index=0;                      // color index
   LONG lIndex1,                           // long Counter
        lIndex2,                           // long Counter
        lIndex4,                           // long Counter
        width,
        height,
        RandBitMap;
   double   xPelsPerInch,
            yPelsPerInch;

   ENTER_MAIN(EntryName);

   //Initialize randomize's seed.
   srand( 1);

   SETENTER("Setting up for creating bitmaps.",NULL);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    // Obtain the width and the height of the current device
    height=alDevCaps[CAPS_HEIGHT];  // Device height in pixels.
    width =alDevCaps[CAPS_WIDTH ];  // Device width  in pixels.

    //Calculate pixles per inch in X and Y dirction.
    xPelsPerInch=(width/pg_size_x);
    yPelsPerInch=(height/pg_size_y);


    //Query anchor Block. This is used when creating new Presentation Space.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
        CLEANUP("WinQueryAnchorBlock()",EntryName);
        return ;
    }

    /*=========================================================
     Initalize the Bitmap information structure
     first we need to  allocate memory to hold RGB array
     since  BITMAPINFO structure is large enough to hold
     one RGB structure.
    ==========================================================*/

    pBitMapInfo=(PBITMAPINFO)Temp_buffer;
    pBitMapInfo->cbFix=sizeof(BitMapHeader);   // This always =12
    pBitMapInfo->cPlanes=                      // one color plane
    pBitMapInfo->cBitCount=1;                  // 1-bit /pixel
    pBitMapInfo->argbColor[0].bBlue=
    pBitMapInfo->argbColor[0].bGreen=
    pBitMapInfo->argbColor[0].bRed=0;
    pBitMapInfo->argbColor[1].bBlue=
    pBitMapInfo->argbColor[1].bGreen=
    pBitMapInfo->argbColor[1].bRed=0xFF;

    BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
    BitMapHeader.cPlanes=                      // one color plane
    BitMapHeader.cBitCount=1;                  // 1-bit /pixel

    /***********************************************************
    * This code segment is intended to create new Presentation *
    * Space  and associate it with memory Device Context. The  *
    * The need for a new PS and a new DC stems from the fact   *
    * that bitmaps belongs to DCs of OD_MEMORY type.           *
    ***********************************************************/
    // Open a Memory device context  to use for bitmap operations.
    if ((hDC1=DevOpenDC(hAB,OD_MEMORY,"*",0L,(PDEVOPENDATA)NULL,
                        (HDC)hDC))==DEV_ERROR){
       CLEANUP("DevOpenDC()",EntryName);
       return ;
    }
    // Create a presentation space to associate with the
    // memory device context
    if ((hPS1=GpiCreatePS(hAB,hDC1,&sizel,PU_PELS |
                          GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR){
       CLEANUP("GpiCreatePS()",EntryName);
       return ;
    }

    SETEXIT();

    TEST_PROGRESS ( 00, 10 );

    for (lIndex4=0; lIndex4< 10 ;lIndex4++){
      sprintf(Message,", Selecting, and Deleting bitmaps. PASS #%d",lIndex4+1);
      SUBENTER(Creating ,Message);
      srand( 1);

      //Get next color to use for displaying bitmaps.
      INCROMENTWITHWRAP(color_index,colors);

      for (lIndex1=(LONG)(.7*yPelsPerInch)+lIndex4; lIndex1< height ;
                                                   lIndex1+=CAT_HEIGHT){
        //Fill an entire row with randomly selected bitmaps.
        for (lIndex2=(LONG)(.1*xPelsPerInch)+lIndex4 ; lIndex2<(width);
                                                   lIndex2+=CAT_WIDTH){

           //Select randomly the next bitmap to create.
           RandBitMap=GETRANDOM(0,NUMBEROFELEMENTS(aBitMap)-1);

           pBitMapInfo->cx=
           BitMapHeader.cx=aBitMap[RandBitMap].width;
           pBitMapInfo->cy=
           BitMapHeader.cy=aBitMap[RandBitMap].height;

           //Setup conditions to fill a third of the display height
           //with the next bitmap. First Create an empty Bitmap.
           if ((hBM=GreCreateBitmap(hDC1,&BitMapHeader,
                                   CBM_INIT,aBitMap[RandBitMap].BitMapBuffer
                                  ,pBitMapInfo))==GPI_ERROR){
               CLEANUP("GreCreateBitmap",EntryName);
               return;
           }

           //Select this bitmap handle into the memory Device Context.
           if (GreSelectBitmap(hDC1,hBM)==HBM_ERROR){
               CLEANUP("GreSelectBitmap",EntryName);
               return;
           }

           // Change current foreground color
           if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR){
               SHOWERROR("GpiSetColor",EntryName);
           }

           //Incroment the color counter
           INCROMENTWITHWRAP(color_index,colors);

           //Setup the dimensions of the target rectangle. Note that
           //displayed bitmaps are not stretched or compressed.
           aptl[0].x=lIndex2;
           aptl[0].y=lIndex1;
           aptl[1].x=lIndex2+aBitMap[RandBitMap].width;
           aptl[1].y=lIndex1+aBitMap[RandBitMap].height;

           //Copy this bitmap to the display.
           if (GpiBitBlt(hPS,hPS1,3L,aptl,ROP_SRCCOPY,BBO_AND)==GPI_ERROR){
              SHOWERROR("GpiBitBlt",EntryName);
           }

           if (GreSelectBitmap(hDC1,NULL)==HBM_ERROR){
              CLEANUP("GreSelectBitmap()",EntryName);
              return ;
           }

            if (GreDeleteBitmap(hBM)==FALSE){
              ErrorCount++;
              ERRSTATUS(TRUE,EntryName,"GreDeleteBitmap failed");
           }
        }

        TEST_PROGRESS ( lIndex4 + 1, 10 );
      }
      SUBEXIT();
    }
    /*=================================================================
      The purpose of next code segment is do some house cleaning
      before exiting this routine. the following is done:
      1. De-select any bitmaps that are selected into memory Device
         Contexts or/and Un-tag any bitmaps that are used as pattern.
      2. Delete any bitmaps created within this function.
      3. Delete any Presentation Spaces created within this function.
      4. Delate any Device Contexts created within this function.

         Note that these steps must be followed as presented above,
         otherwise; most GPIs used here will fail.
    ===================================================================*/

    // dispose of presentation space. No longer needed.
    if (GpiDestroyPS(hPS1)==FALSE){
         CLEANUP("GpiDestroyPS()",EntryName);
         return ;
    }

    //Close the device context
    if (DevCloseDC(hDC1)==DEV_ERROR){
      CLEANUP("DevCloseDC()",EntryName);
      return ;
    }
    EXIT_MAIN (EntryName);

  return ;
}

 /****************************************************************************
 *\\ddd
 * Routine Name: GreBitApp2()
 *
 * Purpose:  This function tests the basic functionality of the following
 *           Gre functions:
 *           GreSelectBitmap().
 *           GreCreatBitmap().
 *           GreDeleteBitmap().
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/24/1991 ,PDVT , Mike Abuzant
 *               - TestCase coding  and design
 *
 * Inputs:
 *    hps              // presentation space
 *    hWndClient       // client window
 *    spiral_control   // Controls the apperance of the spiral.
 *
 * Outputs:
 *   None.
 * Subroutiynes Required:
 *
 *  GreApp2(hPS,hWndClient);
 *
 * Limitations:
 *  None at this time.
 *
 *\\end
 ***************************************************************************/

 /****************************************************************************
 *\\algorithm
 * {
 *   // Call this subfunction to perform the functionality test.
 *
 *   (VOID)GreApp2(hPS,hWndClient);
 *
 * }
 *\\end
 ***************************************************************************/
 VOID APIENTRY GreBitApp2(HPS hPS,HWND hWndClient,BOOL SelectionCall)
 {

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    GreApp2(hPS,hWndClient);
  }

  return ;
 }

 /****************************************************************************
 *\\ddd
 * Routine Name: GpiApp2()
 *
 * Purpose:  This function tests the basic functionality of the following
 *           Gre functions:
 *            GreSelectBitmap().
 *            GreCreateBitmap().
 *            GreDeleteBitmap().
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/24/1991 ,PDVT , Mike Abuzant
 *               - TestCase coding  and design
 *
 * Inputs:
 *    hps              // presentation space
 *    hWndClient       // client window
 *
 * Outputs:
 *
 * Subroutiynes Required:
 *   None at this time.
 *
 * Limitations:
 *   None at this time
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *
 *    allocate memory for the bitmap information structure and
 *    the bitmap information header structure. Then,
 *    initialize bitmap information header structure and bitmap information
 *    structure to the size and characterisics with the bitmap to be created.
 *
 *    open a memory device context since bitmaps belongs to 'OD_MEMORY'
 *    device context  ONLY
 *        HDC1=DevOpenDC(hAB,OD_MEMORY,...);
 *
 *    Create a new presentation space and associate it with the newly
 *    created device context.
 *        hPS1=GpiCreatePS(hAB,....);
 *
 *    Obtain the current device context
 *        hDC=DevQueryDevice(hPS....);
 *
 *    Obtain the width and the height of the current device
 *        height=alDevCaps[CAPS_HEIGHT];  //alDevCaps[] is provided by DTT.
 *        width =alDevCaps[CAPS_WIDTH ];
 *
 *    FOR (all the bitmaps to Display) DO
 *       Setup conditions to fill a third of the display width
 *       with the next bitmap. First Create an empty Bitmap.
 *          HBM=GreCreateBitmap(hDC1...)
 *
 *       Select bitmap into OD_MEMORY device context
 *       GreSelectBitmap(hDC1,hBM...)
 *
 *       FOR (index=0 ;index<1/3 Devicewidth ;..... ) DO
 *
 *         FOR (index2=0 ;index2<Deviceheight ;..... ) DO
 *
 *             Display the bitmap.
 *               GpiBitBlt(hPS,hPS1,ROP_SRCCOPY...);
 *
 *        ENDDO
 *       ENDDO
 *
 *      De-Select the current bitmap form the device context passing
 *      NULL as a bitmap handle.This will serve as De-selecting.
 *         GreSelectBitmap(hDC1,NULL);
 *
 *      Delete the current bitmap handle.
 *          GreDeletetBitmap(hBM);
 *    ENDDO
 *
 *   Destory the presentation space.
 *     GpiDestoryPS(hPS1);
 *
 *   Close the device context
 *    DevCloseDC(hDC1);
 *
 *   return to DTT
 *
 *\\end
 ***************************************************************************/
 INT GreApp2(HPS hPS,HWND hWndClient)
 {
   static PCHAR EntryName="GreApp2()";

   static MyBitMap  aBitMap[]={
                               {BRICK_HEIGHT,BRICK_WIDTH,abBrick},
                               {K_SHAPE_HEIGHT,K_SHAPE_WIDTH,abKShape},
                               {CROSS_HEIGHT,CROSS_WIDTH,abCross}
                              };

   SIZEL sizel={0};          // size of bitmap to draw
   HDC   hDC=(HDC)NULL,
         hDC1=(HDC)NULL;

   HAB   hAB=(HAB)NULL;
   HBITMAP hBM=(HBITMAP)NULL ;
   HPS   hPS1=(HPS)NULL;
   BYTE  Temp_buffer[100];
   BITMAPINFOHEADER BitMapHeader;      //bitmap info header
   PBITMAPINFO      pBitMapInfo;       //bitmap info structure

   double           yPelsPerInch;

   POINTL  aptl[4]={0};
   INT color_index=0;                      // color index
   LONG lIndex1,                           // long Counter
        lIndex2,                           // long Counter
        lIndex4,                           // long Counter
        width,
        height;

    ENTER_MAIN(EntryName);

    SETENTER("Creating OD_MEMORY Device Context to hold bitmaps.",NULL);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return FALSE;   //Can't proceed without it
    }

    // Obtain the width and the height of the current device
    height=alDevCaps[CAPS_HEIGHT];  // Device height in pixels.
    width =alDevCaps[CAPS_WIDTH ];  // Device width  in pixels.

    //Calculate pixles per inch ind Y direction.
    yPelsPerInch=(height/pg_size_y);

    //Query anchor Block. This is used when creating new Presentation Space.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
        CLEANUP("WinQueryAnchorBlock()",EntryName);
        return FALSE;
    }

    /*=========================================================
     Initalize the Bitmap information structure
     first we need to  allocate memory to hold RGB array
     since  BITMAPINFO structure is large enough to hold
     one RGB structure.
    ==========================================================*/

    pBitMapInfo=(PBITMAPINFO)Temp_buffer;
    pBitMapInfo->cbFix=sizeof(BitMapHeader);   // This always =12
    pBitMapInfo->cPlanes=                      // one color plane
    pBitMapInfo->cBitCount=1;                  // 1-bit /pixel
    pBitMapInfo->argbColor[0].bBlue=
    pBitMapInfo->argbColor[0].bGreen=
    pBitMapInfo->argbColor[0].bRed=0;
    pBitMapInfo->argbColor[1].bBlue=
    pBitMapInfo->argbColor[1].bGreen=
    pBitMapInfo->argbColor[1].bRed=0xFF;

    BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
    BitMapHeader.cPlanes=                      // one color plane
    BitMapHeader.cBitCount=1;                  // 1-bit /pixel

    /***********************************************************
    * This code segment is intended to create new Presentation *
    * Space  and associate it with memory Device Context. The  *
    * The need for a new PS and a new DC stems from the fact   *
    * that bitmaps belongs to DCs of OD_MEMORY type.           *
    ***********************************************************/
    // Open a Memory device context  to use for bitmap operations.
    if ((hDC1=DevOpenDC(hAB,OD_MEMORY,"*",0L,(PDEVOPENDATA)NULL,
                        (HDC)hDC))==DEV_ERROR){
       CLEANUP("DevOpenDC()",EntryName);
       return FALSE;
    }
    // Create a presentation space to associate with the
    // memory device context
    if ((hPS1=GpiCreatePS(hAB,hDC1,&sizel,PU_PELS |
                          GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR){
       CLEANUP("GpiCreatePS()",EntryName);
       return FALSE;
    }

    SETEXIT();


    for (lIndex4=0 ;lIndex4<NUMBEROFELEMENTS(aBitMap); lIndex4++){

       pBitMapInfo->cx=
       BitMapHeader.cx=aBitMap[lIndex4].width;
       pBitMapInfo->cy=
       BitMapHeader.cy=aBitMap[lIndex4].height;

       //Setup conditions to fill a third of the display with
       //with the next bitmap.


       if ((hBM=GreCreateBitmap(hDC1,&BitMapHeader,
                               CBM_INIT,aBitMap[lIndex4].BitMapBuffer,
                               pBitMapInfo))==GPI_ERROR){
           CLEANUP("GreCreateBitmap()",EntryName);
           return FALSE;
       }

       //Select this bitmap handle into the memory Device Context.
       if (GreSelectBitmap(hDC1,hBM)==HBM_ERROR){
           CLEANUP("GreSelectBitmap()",EntryName);
           return FALSE;
       }

       for (lIndex1=(LONG)(0.7*yPelsPerInch);lIndex1<((lIndex4+1)*height) ;
                       lIndex1+=aBitMap[lIndex4].height){

            // Change current foreground color
            if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR){
                CLEANUP("GpiSetColor()",EntryName);
                return FALSE;
            }
            INCROMENTWITHWRAP(color_index,colors);

            // This next loop will fill the entire height of the display
            // with the current bitmap.
            for (lIndex2=10+(lIndex4*width/3); lIndex2<((lIndex4+1)*width/3);
                                        lIndex2+=aBitMap[lIndex4].width){

              //Setup the dimensions of the target rectangle. Note that
              //displayed bitmaps are not stretched or compressed.
              aptl[0].x=lIndex2;
              aptl[0].y=lIndex1;
              aptl[1].x=lIndex2+aBitMap[lIndex4].width;
              aptl[1].y=lIndex1+aBitMap[lIndex4].height;

              //Copy this bitmap to the display.
              if (GpiBitBlt(hPS,hPS1,3L,aptl,ROP_SRCCOPY,BBO_AND)==GPI_ERROR){
                 CLEANUP("GpiBitBlt()",EntryName);
                 return FALSE;
              }

            }
       }

       // De-select the bitmap handle. otherwise we get error when closing
       // device context
       if (GreSelectBitmap(hDC1,NULL)==FALSE){
          CLEANUP("GreSelectBitmap()",EntryName);
          return FALSE;
       }
       if (GreDeleteBitmap(hBM)==FALSE){
          CLEANUP("GreDeleteBitmap()",EntryName);
          return FALSE;
       }
    }

    /*=================================================================
      The purpose of next code segment is do some house cleaning
      before exiting this routine. the following is done:
      1. De-select any bitmaps that are selected into memory Device
         Contexts or/and Un-tag any bitmaps that are used as pattern.
      2. Delete any bitmaps created within this function.
      3. Delete any Presentation Spaces created within this function.
      4. Delate any Device Contexts created within this function.

         Note that these steps must be followed as presented above,
         otherwise; most GPIs used here will fail.
    ===================================================================*/

    SETENTER(EntryName,"Clean up...");
   // dispose of presentation space. No longer needed.
    if (GpiDestroyPS(hPS1)==FALSE){
         CLEANUP("GpiDestroyPS()",EntryName);
         return FALSE;
    }

    //Close the device context
    if (DevCloseDC(hDC1)==DEV_ERROR){
      CLEANUP("DevCloseDC()",EntryName);
      return FALSE;
    }

    SETEXIT();

    EXIT_MAIN (EntryName);

 return TRUE;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreBitApp3()
 *
 * Purpose:  This function will test the basic functionality of the following
 *           Gre functions:
 *           GreGetBitmapParameter().
 *           GreGetBitmapDimension().
 *           GreSetBitmapDimension().
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/30/91, pdvt ,Mike Abuzant
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *   None.
 * Subroutines Required:
 *   GreApp3()
 *
 * Limitations:
 *   None.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *  // Call the functionality test routine
 *     GreApp3(hPS , hWndClient)
 * }
 *\\end
 ***************************************************************************/

VOID APIENTRY GreBitApp3(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    GreApp3(hPS ,hWndClient);
  }
}

/***************************************************************************
 *\\ddd
 * Routine Name: GreApp3()
 *
 * Purpose:  This function will test the basic functionality of the following
 *           Gre functions:
 *           GreGetBitmapParameter().
 *           GreGetBitmapDimension().
 *           GreSetBitmapDimension().
 *
 * System Requirements: None
 *
 * Revision Log:
 *               09/30/91, pdvt ,Mike Abuzant
 *                  -Test Case Implementation and Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *
 * Outputs:
 *     None
 * Subroutines Required:
 *     None.
 * Limitations:
 *     None.
 *
 *\\end
 ***************************************************************************/

/***************************************************************************
 *\\algorithm
 *
 *    allocate memory for the bitmap information structure and
 *    the bitmap information header structure. Then,
 *    initialize bitmap information header structure and bitmap information
 *    structure to the size and characterisics with the bitmap to be created.
 *
 *    open a memory device context since bitmaps belongs to 'OD_MEMORY'
 *    device context  ONLY
 *        HDC1=DevOpenDC(hAB,OD_MEMORY,...);
 *
 *    Create a new presentation space and associate it with the newly
 *    created device context.
 *        hPS1=GpiCreatePS(hAB,....);
 *
 *    Obtain the current device context
 *        hDC=DevQueryDevice(hPS....);
 *
 *    Obtain the width and the height of the current device
 *        height=alDevCaps[CAPS_HEIGHT];  //alDevCaps[] is provided by DTT.
 *        width =alDevCaps[CAPS_WIDTH ];
 *
 *    FOR (all the bitmaps to Display) DO
 *       Setup conditions to display next bitmap. The next bitmap will
 *       stretched to fit 1/2 device width and 1/3 device height.
 *
 *        HBM=GreCreateBitmap(hDC1...)
 *
 *       Select bitmap into OD_MEMORY device context
 *       GreSelectBitmap(hDC1,hBM...)
 *
 *       Call the function rand() to generate random dimensions for
 *       this bitmap and then call GreSetBitmapDimension() to attched
 *       then to the bitmap.
 *
 *       Call the function LOGINFO() to record the bitmap dimensions
 *       and the bitmap parameters.
 *
 *       Display the stretched bitmap.
 *         GpiBitBlt(hPS,hPS1,ROP_SRCCOPY...);
 *
 *       Query the current bitmap dimensions by calling the function
 *       GreGetBitmapDimension().
 *
 *       Call the function LOGINFO() to record the queried bitmap dimensions.
 *
 *       Query the current bitmap parameters by calling the function
 *       GreGetBitmapParameter().
 *
 *       Call the function LOGINFO() to record the queried bitmap parameters.
 *
 *       Finally, display the current bitmap parameters and dimensions on
 *       current device.
 *
 *     ENDDO //Create next bitmap.
 *
 *      De-Select the current bitmap form the device context passing
 *      NULL as a bitmap handle.This will serve as De-selecting.
 *         GreSelectBitmap(hDC1,NULL);
 *
 *      Delete the current bitmap handle.
 *          GreDeletetBitmap(hBM);
 *    ENDDO
 *
 *   Destory the presentation space.
 *     GpiDestoryPS(hPS1);
 *
 *   Close the device context
 *    DevCloseDC(hDC1);
 *
 *   return to DTT
 *
 *\\end
 ***************************************************************************/
VOID GreApp3(HPS hPS,HWND hWndClient)
{
   static  PCHAR EntryName="GreBitApp3()";

   static  MyBitMap  aBitMap[]={{K_SHAPE_HEIGHT,K_SHAPE_WIDTH,abKShape},
                               {DIAMOND_HEIGHT,DIAMOND_WIDTH,abDiamond},
                               {SHAPE_HEIGHT  ,SHAPE_WIDTH,abShape},
                               };
   SIZEL   sizel={0};          // size of PS to create.
   HDC     hDC=(HDC)NULL,
           hDC1=(HDC)NULL;

   HAB     hAB=(HAB)NULL;
   HBITMAP hBM=(HBITMAP)NULL;
   HPS     hPS1=(HPS)NULL;

   BYTE    Temp_buffer[100];
   BYTE    Message[200];

   BITMAPINFOHEADER BitMapHeader;      //bitmap info header
   PBITMAPINFO      pBitMapInfo;       //bitmap info structure

   POINTL  aptl[4]={0},
           ptl    ={0};

   LONG    width,
           height;

   double  xPelsPerInch,
           yPelsPerInch;

   USHORT  NumberOfBitmaps=NUMBEROFELEMENTS(aBitMap),
           usIndex;

   ENTER_MAIN(EntryName);

   //Initialize randomize's seed.
    srand(1);

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    // Obtain the width and the height of the current device
    height=alDevCaps[CAPS_HEIGHT];  // Device height in pixels.
    width =alDevCaps[CAPS_WIDTH ];  // Device width  in pixels.

    //Calculate pixles per inch in X and Y dirction.
    xPelsPerInch=(width/pg_size_x);
    yPelsPerInch=(height/pg_size_y);

    //Query anchor Block. This is used when creating new Presentation Space.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
        CLEANUP("WinQueryAnchorBlock()",EntryName);
        return ;
    }

    /*=========================================================
     Initalize the Bitmap information structure
     first we need to  allocate memory to hold RGB array
     since  BITMAPINFO structure is large enough to hold
     one RGB structure.
    ==========================================================*/

    pBitMapInfo=(PBITMAPINFO)Temp_buffer;
    pBitMapInfo->cbFix=sizeof(BitMapHeader);   // This always =12
    pBitMapInfo->cPlanes=                      // one color plane
    pBitMapInfo->cBitCount=1;                  // 1-bit /pixel
    pBitMapInfo->argbColor[0].bBlue=
    pBitMapInfo->argbColor[0].bGreen=
    pBitMapInfo->argbColor[0].bRed=0;
    pBitMapInfo->argbColor[1].bBlue=
    pBitMapInfo->argbColor[1].bGreen=
    pBitMapInfo->argbColor[1].bRed=0xFF;

    BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
    BitMapHeader.cPlanes=                      // one color plane
    BitMapHeader.cBitCount=1;                  // 1-bit /pixel

    /***********************************************************
    * This code segment is intended to create new Presentation *
    * Space  and associate it with memory Device Context. The  *
    * The need for a new PS and a new DC stems from the fact   *
    * that bitmaps belongs to DCs of OD_MEMORY type.           *
    ***********************************************************/
    // Open a Memory device context  to use for bitmap operations.
    if ((hDC1=DevOpenDC(hAB,OD_MEMORY,"*",0L,(PDEVOPENDATA)NULL,
                        (HDC)hDC))==DEV_ERROR){
       CLEANUP("DevOpenDC()",EntryName);
       return ;
    }
    // Create a presentation space to associate with the
    // memory device context
    if ((hPS1=GpiCreatePS(hAB,hDC1,&sizel,PU_PELS |
                          GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR){
       CLEANUP("GpiCreatePS()",EntryName);
       return ;
    }

    for (usIndex=0 ; usIndex<NumberOfBitmaps ; usIndex++){

      sprintf(Message,"Creating and Selecting Bitmap #%d",usIndex+1);
      SETENTER(Message,NULL);

      pBitMapInfo->cx=
      BitMapHeader.cx=aBitMap[usIndex].width;
      pBitMapInfo->cy=
      BitMapHeader.cy=aBitMap[usIndex].height;

      //Setup conditions to creat a bitmap and stretch it ti fill
      // 1/2 device width and 1/3 device height.
      if ((hBM=GreCreateBitmap(hDC1,&BitMapHeader,
                              CBM_INIT,aBitMap[usIndex].BitMapBuffer
                             ,pBitMapInfo))==GPI_ERROR){
          CLEANUP("GreCreateBitmap",EntryName);
          return;
      }

      //Select this bitmap handle into the memory Device Context.
      if (GreSelectBitmap(hDC1,hBM)==HBM_ERROR){
          CLEANUP("GreSelectBitmap",EntryName);
          return;
      }

      // Set this bitmaps width and height dimensions.
      sizel.cx=rand();        //Get A random bitmap width.
      sizel.cy=rand();        //Get A random bitmap height.

      if (GreSetBitmapDimension(hBM,&sizel)==FALSE){
          SHOWERROR("GreSetBitmapDimension",EntryName);
      }

      sprintf(Message,"Setting Bitmap Dimensions to(%05ld mm, %05ld mm)",
                            sizel.cx,sizel.cy);
      LOGINFO(L_DEBUG,"*",Message);

      sprintf(Message,"Creating a Bitmap : Width=%04d  Height=%04d\n"
                        "BitCount=%02d ColorPlanes=%02d",
                         BitMapHeader.cx,BitMapHeader.cy,
                         BitMapHeader.cBitCount,BitMapHeader.cPlanes);

      LOGINFO(L_DEBUG,"*",Message);

      SETEXIT();

      //Setup the dimensions of the target rectangle. Note that
      //displayed bitmap will be stretched to fill 1/2 the display
      //width and 1/3 the display height.

      aptl[0].x=(LONG)(xPelsPerInch/10);
      aptl[0].y=(LONG)(height*usIndex/3)+(LONG)
                   (usIndex==0 ?(.7*yPelsPerInch)  //spacing between bitmaps.
                               :(.2*yPelsPerInch));

      aptl[1].x=aptl[0].x+(LONG)(width/2);
      aptl[1].y=(LONG)(height*(usIndex+1)/3);

      //Setup the source rectangle.
      aptl[2].x=aBitMap[usIndex].width;
      aptl[2].y=aBitMap[usIndex].height;

      //Copy this bitmap to the display.
      if (GpiBitBlt(hPS,hPS1,4L,aptl,ROP_SRCCOPY,BBO_AND)==GPI_ERROR){
         SHOWERROR("GpiBitBlt",EntryName);
      }

      SUBENTER(Querying Bitmap Parameters and Dimensions,NULL);

      if (GreGetBitmapDimension(hBM,&sizel)==FALSE){
          SHOWERROR("GreGetBitmapDimension",EntryName);
      }

      sprintf(Message,"Queried Bitmap Dimensions o(%05ld mm, %05ld mm)",
                            sizel.cx,sizel.cy);
      LOGINFO(L_DEBUG,"*",Message);

      if (GreGetBitmapParameters(hBM,&BitMapHeader)==FALSE){
          SHOWERROR("GreGetBitmapParameters()",EntryName);
      }
      sprintf(Message,"Queried Bitmap parameters: Width=%04d  Height=%04d\n"
                        "BitCount=%02d ColorPlanes=%02d",
                         BitMapHeader.cx,BitMapHeader.cy,
                         BitMapHeader.cBitCount,BitMapHeader.cPlanes);

      LOGINFO(L_DEBUG,"*",Message);

      SUBEXIT();

      //Now Display the bitmap parameters and dimensions on
      //display device.
      ptl.x=(LONG)(delta_x*(.55 * pg_size_x));
      ptl.y=(LONG)(delta_y*usIndex*pg_size_y/3+(0.8* delta_y));

      sprintf(Message,"\tDimensions :(%05ld mm , %05ld mm)",sizel.cx,sizel.cy);
      MapAndWriteString(ptl.x,ptl.y,Message);


      ptl.y+=UnMapY(syPosition);
      sprintf(Message,"\tBitCount=%02d ColorPlanes=%02d",
                      BitMapHeader.cBitCount,BitMapHeader.cPlanes);
      MapAndWriteString(ptl.x,ptl.y,Message);


      ptl.y+=UnMapY(syPosition);
      sprintf(Message,"\tWidth=%04d Pixels Height=%04d Pixels ",
                       BitMapHeader.cx,BitMapHeader.cy);
      MapAndWriteString(ptl.x,ptl.y,Message);


      ptl.y+=UnMapY(1.5*syPosition);
      MapAndWriteString(ptl.x,ptl.y,"\t\t Bitmap's Parameters and Dimensions.\t\t");

      //Get current bitmap ready for deletion.
      if (GreSelectBitmap(hDC1,NULL)==HBM_ERROR){
         CLEANUP("GreSelectBitmap()",EntryName);
         return ;
      }

      if (GreDeleteBitmap(hBM)==FALSE){
         SHOWERROR("GreDeleteBitmap()",EntryName);
      }
    }
    /*=================================================================
      The purpose of next code segment is do some house cleaning
      before exiting this routine. the following is done:
      1. De-select any bitmaps that are selected into memory Device
         Contexts or/and Un-tag any bitmaps that are used as pattern.
      2. Delete any bitmaps created within this function.
      3. Delete any Presentation Spaces created within this function.
      4. Delate any Device Contexts created within this function.

         Note that these steps must be followed as presented above,
         otherwise; most GPIs used here will fail.
    ===================================================================*/

    // dispose of presentation space. No longer needed.
    if (GpiDestroyPS(hPS1)==FALSE){
         CLEANUP("GpiDestroyPS()",EntryName);
         return ;
    }

    //Close the device context
    if (DevCloseDC(hDC1)==DEV_ERROR){
      CLEANUP("DevCloseDC()",EntryName);
      return ;
    }
    EXIT_MAIN (EntryName);

  return ;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreBitApp4()
 *
 * Purpose:  This function will test the basic functionality of the following
 *           Gre functions:
 *           GreSaveScreenBits().
 *           GreRestoreScreenBits().
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/08/91, pdvt ,Mike Abuzant
 *                  -Initial file.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *   None.
 * Subroutines Required:
 *    None.
 *
 * Limitations:
 *
 *    Currently  this entry point is not implemented for  the
 *    following reasons:
 *
 *    a. The function GreSaveScreenBits() return a zero indicating
 *    that the calling routine should be saving the screen bits on
 *    it own; thus we can't fully test it.
 *
 *    b.  Two  defined constant,passed  to GreRestoreScreenBits(),
 *    are  not  defined anywhere in the include(.h) files. This
 *    function can't be test unless we obtain these two constants.
 *
 *    c. Since GreSaveScreenBits() does not  save the screen bits,
 *    the internal format of buffer returned by this function must
 *    be supplied in order to save screen bits and restore them by
 *     calling GreRestoreScreenBits().
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreBitApp4(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PCHAR  EntryName="GreBitApp4";

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    EXIT_MAIN(EntryName);
  }
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetCursorExh()
 *
 * Purpose:  This function exhaustively tests the functionality of
 *           Gre function GreDeviceSetCursor.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/08/91, pdvt ,Mike Abuzant
 *                  -Initial file.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *   None.
 * Subroutines Required:
 *    None.
 *
 * Limitations:
 *
 *    Currently this entry point is not implemented due to insufficeint
 *  documentation.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceSetCursorExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PCHAR  EntryName="GreDeviceSetCursorExh";

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    EXIT_MAIN(EntryName);
  }
}



/****************************************************************************
 *\\ddd
 * Routine Name: GreSetBitmapOwnerExh()
 *
 * Purpose:  This function exhaustively tests the functionality of
 *           Gre function GreSetBitmapOwner.
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/08/91, pdvt ,Mike Abuzant
 *                  -Initial file.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *   None.
 * Subroutines Required:
 *    None.
 *
 * Limitations:
 *
 *    Currently this entry point is not implemented  due to the fact
 *  that it is not documented.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetBitmapOwnerExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PCHAR  EntryName="GreSetBitmapOwnerExh";

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    EXIT_MAIN(EntryName);
  }
}



/****************************************************************************
 *\\ddd
 * Routine Name: GreRestoreScreenBitsExh()
 *
 * Purpose:  This function exhaustively tests the functionality of
 *           Gre functions:
 *           GreSaveScreenBits().
 *           GreRestoreScreenBits().
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/08/91, pdvt ,Mike Abuzant
 *                  -Initial file.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *   None.
 * Subroutines Required:
 *    None.
 *
 * Limitations:
 *
 *    Currently  this entry point is not implemented for  the
 *    following reasons:
 *
 *    a. The function GreSaveScreenBits() return a zero indicating
 *    that the calling routine should be saving the screen bits on
 *    it own; thus we can't fully test it.
 *
 *    b.  Two  defined constant,passed  to GreRestoreScreenBits(),
 *    are  not  defined anywhere in the include(.h) files. This
 *    function can't be test unless we obtain these two constants.
 *
 *    c. Since GreSaveScreenBits() does not  save the screen bits,
 *    the internal format of buffer returned by this function must
 *    be supplied in order to save screen bits and restore them by
 *     calling GreRestoreScreenBits().
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreRestoreScreenBitsExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PCHAR  EntryName="GreRestoreScreenBitsExh";

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

    EXIT_MAIN(EntryName);
  }
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreCopyDCLoadDataExh()
 *
 * Purpose:  This function exhaustively tests the functionality of
 *            The Gre function GreCopyDCLoadData().
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/24/91, PDVT ,Mike Abuzant
 *               - initial file
 *               09/25/91, PDVT ,Mike Abuzant
 *                  -Test Case Implementation and design.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *       None.
 * Subroutines Required:
 *    None.
 *
 * Limitations:
 *   This Gre can not copy bitmaps that are attached to pattern IDs. It
 *   returns the error code PMERR_INV_FONT_ATTR that causes the function
 *   GreDeleteBitmap() to return with the error code PMERR_BITMAP_IS_SELECTED.
 *   GreDeleteBitmap() retuns the error code in question only when deleting
 *   the first bitmap. I believe that  GreCopyDCLoadData() is trying to
 *   copy the bitmaps as if they were fonts without doing any checking.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *    allocate memory for the bitmap information structure and
 *    the bitmap information header structure. Then,
 *    initialize bitmap information header structure and bitmap information
 *    structure to the size and characterisics with the bitmap to be created.
 *
 *     Load a font as a PRIVATE font using the function GpiLoadFonts().
 *        GpiLoadFonts("CURIOR.FON").
 *
 *     Store all available PUBLIC and PRIVATE fonts in the structure
 *     'FontStorage'.
 *        EzQueryFonts(QF_PUBLIC |QF_PRIVATE...)
 *
 *      setup contitions to create 20 logical fonts .
 *        WHILE (have not load 20 fonts)
 *          call EzCreatelogicalfont.
 *        ENDDO.
 *
 *      Change the default ARC parameters and log any error
 *      Change the default Line attributes.
 *      Change the default Character attributes.
 *      Change the default Image attributes.
 *      Change the default Marker attributes.
 *      Change the default Viewing limits.
 *      Create a logical RGB color table.
 *
 *     call the function GreCopyDCLoadData() and verify that
 *     all the above default primitives' attributes have been
 *     copied(set) into the target Device Context. Also setup
 *     the enviroment to verify that the created logical RGB color
 *     table and all created logical fonts have all been loaded and created.
 *
 *     delete all logical fonts  and unload the PRIVATE fonts since they
 *     are no longer need.
 *
 *
 *     Setup conditions to verify that the function GreCopyDCLoadData
 *     can copy bitmaps and their associated ID's to the target Device
 *     Context.
 *
 *     //Create atmost 20 bitmap.
 *       WHILE (have not created 20 bitmaps)
 *        // Create next bitmap
 *           GreCreateBitmap(hDC...)
 *
 *        //  Attached this bitmap to a pattern ID.
 *            GreSetBitmapID(hDC,...)
 *       ENDDO.
 *
 *       Call the function under test.
 *        GreCopyDCLoadData(hDC...)
 *
 *       Verify that all bitmaps and all their IDs are copied into
 *       the target Device Context.
 *
 *       Delete all bitmaps and destroy any PSs and DCs created within
 *       this entry point.
 *
 *       return to DTT.EXE
 *\\end
 ***************************************************************************/

VOID APIENTRY GreCopyDCLoadDataExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   PTEMPTYPE  FontStorage=(PTEMPTYPE)NULL;
   static     PCHAR EntryName="GreCopyDCLoadDataExh()";
   CHAR       Message[200];
   BYTE       Temp_buffer[200];
   CHAR       szFullFontPathName[_MAX_PATH];
   ARCPARAMS  ArcParams={3,4,5,6},   //New ARC default parameters
              ArcParams2={0};

   SIZEL      sizel={0};

   HDC        hDC,
              hDC1;
   HPS        hPS1,
              TargethPS;
   HAB        hAB;
   HBITMAP    ahBM[20];
   LONG       alFonts[50];
   STR8       aszFontNames[50];
   LONG       alTypes[50];
   USHORT     usIndex,
              ActualNumberOfFonts=0,
              Count;

   LONG       NumberOfFonts,
              Return,
              ReturnValue;
   BOOL       fOutcome;
   RECTL      Rectl={20,50,100,200},
              Rectl2={0};

   //New default Line attributes
   LINEBUNDLE LineBundle={CLR_BLUE,0L,FM_XOR,0U,LINEWIDTH_NORMAL,10,
                         LINETYPE_DOT,LINEEND_ROUND,LINEJOIN_MITRE},
              LineBundle2={0};

   //New default Character attributes
   CHARBUNDLE CharBundle={CLR_YELLOW,CLR_BLUE,BM_OVERPAINT,FM_XOR,0L,
                         0L,{670000l,670000l},{-45,-90},{1,1},
                         CHDIRN_BOTTOMTOP},
              CharBundle2={0};

   //New default Marker attributes
   MARKERBUNDLE  MarkerBundle={CLR_CYAN,CLR_GREEN,FM_AND,BM_XOR,0L,
                              MARKSYM_SIXPOINTSTAR,{89999l,79000L}},
                 MarkerBundle2={0};

   //New default Image attributes
   IMAGEBUNDLE   ImageBundle={CLR_BROWN,CLR_DARKBLUE,FM_ONE,BM_OR},
                 ImageBundle2={0};

   //Default bitmap header information.
   BITMAPINFOHEADER BitMapHeader=
                   {sizeof(BITMAPINFOHEADER),8,8,1,1};  //bitmap info header

   PBITMAPINFO      pBitMapInfo;       //bitmap info structure


  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);
    //The entry point does not hav any output.
    DeletePrinterOutput(EntryName);

    pBitMapInfo=(PBITMAPINFO) Temp_buffer;
    pBitMapInfo->cbFix=sizeof(BITMAPINFOHEADER); // This always =12
    pBitMapInfo->cx=8;                 //Width
    pBitMapInfo->cy=8;                 //height
    pBitMapInfo->cPlanes=1;                      // one color plane
    pBitMapInfo->cBitCount=1;                    // 1-bit /pixel
    pBitMapInfo->argbColor[0].bBlue=
    pBitMapInfo->argbColor[0].bGreen=
    pBitMapInfo->argbColor[0].bRed=0;
    pBitMapInfo->argbColor[1].bBlue=
    pBitMapInfo->argbColor[1].bGreen=
    pBitMapInfo->argbColor[1].bRed=0xFF;

    memset(ahBM,0x00,sizeof(ahBM));

    //Get  the default device context
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return;   //Can't proceed without it
    }

    //Query anchor Block. This is used when creating new Presentation Space.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
        CLEANUP("WinQueryAnchorBlock()",EntryName);
        return ;
    }

    /***********************************************************
    * This code segment is intended to create new Presentation *
    * Space  and associate it with memory Device Context. The  *
    * The need for a new PS and a new DC stems from the fact   *
    * that bitmaps belongs to DCs of OD_MEMORY type.           *
    ***********************************************************/
    // Open a Memory device context  to use for bitmap operations.
    if ((hDC1=DevOpenDC(hAB,OD_MEMORY,"*",0L,(PDEVOPENDATA)NULL,
                        (HDC)hDC))==DEV_ERROR){
       CLEANUP("DevOpenDC()",EntryName);
       return ;
    }
    // Create a presentation space to associate with the
    // memory device context
    if ((hPS1=GpiCreatePS(hAB,hDC1,&sizel,PU_PELS |
                          GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR){
       CLEANUP("GpiCreatePS()",EntryName);
       return ;
    }

    SETENTER("Setting up testing ENV.",NULL);
    //Get the full path name for the OS/2 Courier font form the
    // INI system files. On an error set the default to
    // C:\OS2\DLL\COURIER.FON

    if ( PrfQueryProfileString(HINI_PROFILE, (PSZ) "PM_Fonts", (PSZ) "COURIER", (PSZ) "ERROR",
                                         szFullFontPathName,sizeof(szFullFontPathName)) <= 0 ) {

       SHOWERROR("WinQueryProfileString()",EntryName);
       strcpy(szFullFontPathName,DEFAULT_FONT_NAME);
    }

    LOGINFO(L_DEBUG,"*","Loading PRIVATE fonts");

    //Load tyhe font in question as a PRIVATE font.
    if (GpiLoadFonts(hAB,szFullFontPathName)==FALSE){
       SHOWERROR("GpiLoadFonts", EntryName);
    }


    LOGINFO(L_DEBUG,"*","Creating Logical PUBLIC and PRIVATE fonts and assign them IDs");

    if (EzQueryFonts(hPS,hWndClient,QF_PUBLIC |QF_PRIVATE,
                         &NumberOfFonts,&FontStorage)==FALSE){
        SHOWERROR("EzQueryFonts",EntryName);
    }

    //Create atmost 20 fonts.
    for (usIndex=0; (usIndex < 20) &&
                       usIndex<(USHORT)NumberOfFonts; usIndex++){

       //Setup conditions to create logical fonts.
        EzCreateLogicalFonts(hPS,hWndClient,MY_FONT_ID+usIndex,
                                    FontStorage[usIndex]);

    }
    ActualNumberOfFonts=usIndex;


    LOGINFO(L_DEBUG,"*","Changing Default line attributes.");
    if (GpiSetDefAttrs(hPS,PRIM_LINE,CHANGE_LINE,&LineBundle)==GPI_ERROR){
      SHOWERROR("GpiSetDefAttrs()",EntryName);
    }


    LOGINFO(L_DEBUG,"*","Changing Default character attributes.");
    if (GpiSetDefAttrs(hPS,PRIM_CHAR,CHANGE_CHAR,&CharBundle)==GPI_ERROR){
      SHOWERROR("GpiSetDefAttrs()",EntryName);
    }


    LOGINFO(L_DEBUG,"*","Changing Default marker attributes.");
    if (GpiSetDefAttrs(hPS,PRIM_MARKER,CHANGE_MARKER,&MarkerBundle)==GPI_ERROR){
      SHOWERROR("GpiSetDefAttrs()",EntryName);
    }


    LOGINFO(L_DEBUG,"*","Changing Default image attributes.");
    if (GpiSetDefAttrs(hPS,PRIM_IMAGE,CHANGE_IMAGE,&ImageBundle)==GPI_ERROR){
      SHOWERROR("GpiSetDefAttrs()",EntryName);
    }


    LOGINFO(L_DEBUG,"*","Changing Default ARC parameters.");
    if (GpiSetDefArcParams(hPS,&ArcParams)==FALSE){
      SHOWERROR("GpiSetDefArcParams()",EntryName);
    }


    LOGINFO(L_DEBUG,"*","Changing Default Viewing Limits.");
    if (GpiSetDefViewingLimits(hPS,&Rectl)==FALSE){
      SHOWERROR("GpiSetDefViewingLimits()",EntryName);
    }


    LOGINFO(L_DEBUG,"*","Creating an RGB  logical color table");
    if (GpiCreateLogColorTable(hPS,LCOL_RESET,LCOLF_RGB,
                              (LONG)NULL,(LONG)NULL,NULL)==FALSE){
      SHOWERROR("GpiCreateLogColorTable()",EntryName);
    }

    SETEXIT();

    SUBENTER(Calling the GreCopyDCLoadData,NULL);
    if (GreCopyDCLoadData(hDC1,LCID_RANGE_GPI,hDC)==FALSE){
       ErrorCount++;
       ERRSTATUS(TRUE,EntryName,"GreCopyDCLoadData failed.");
    }
    SUBEXIT();

    SETENTER("Comparing Device Context's attributes after using GreCopyDCLoadData",NULL);

    fOutcome=TRUE;  //Assume that things did work.

    // The next code segment will verify that all the default
    // primitives' attributes are set correctly into the
    // the target Device Context. Also will verify that all
    // loaded and created logical colors and fonts are properly
    // loaded.
    if ((ReturnValue=GpiQueryNumberSetIds(hPS1))==GPI_ALTERROR){
      SHOWERROR("GpiQueryNumberSetId()",EntryName);
      fOutcome=FALSE;
    }
    else {
      if (GpiQuerySetIds(hPS1,ReturnValue,alTypes,aszFontNames,
                        alFonts)==FALSE){
        SHOWERROR("GpiQuerySetId()",EntryName);
        LOGINFO(L_DEBUG,"*","Will not be able to test for created logical"
                            " since GpiQuerySetIds Failed.");
        fOutcome=FALSE;

      }
      else{
        for(usIndex=0,Count=0 ;usIndex < (USHORT)ReturnValue; usIndex++){
          if ((alTypes[usIndex]==LCIDT_FONT) &&
             (alFonts[usIndex]>=MY_FONT_ID)   &&
             (alFonts[usIndex]< (MY_FONT_ID+ActualNumberOfFonts))){
            Count++;
          }
        }

        fOutcome=(Count==ActualNumberOfFonts);
        sprintf(Message,"Logical fonts are  %s created",fOutcome==TRUE ? "" : "NOT");
        LOGINFO(L_DEBUG,"*",Message);

        if (fOutcome==FALSE)
          ErrorCount++;
      }
    }

    //do some clean ups....
    if (FontStorage != (PTEMPTYPE)NULL){
      if (GpiDeleteSetId(hPS1,LCID_ALL)==FALSE){
          SHOWERROR("GpiDeleteSetId()",EntryName);
       }

       if (GpiLoadFonts(hAB,szFullFontPathName)==FALSE){
         SHOWERROR("GpiUnLoadFonts", EntryName);
       }
       free(FontStorage);
    }

    if (GpiQueryDefAttrs(hPS1,PRIM_LINE,CHANGE_LINE,&LineBundle2)==FALSE){
      SHOWERROR("GpiQueryDefAttrs()",EntryName);
    }

    fOutcome=(BOOL)(memicmp(&LineBundle2,&LineBundle,sizeof(LineBundle))==0);
    sprintf(Message,"Queried and set Default line attributes are %s the same",
                     fOutcome==TRUE ? "" : "NOT");

    LOGINFO(L_DEBUG,"*",Message);
    if (fOutcome==FALSE)
      ErrorCount++;


    if (GpiQueryDefAttrs(hPS1,PRIM_CHAR,CHANGE_CHAR,&CharBundle2)==FALSE){
      SHOWERROR("GpiQueryDefAttrs()",EntryName);
    }

    fOutcome=(BOOL)(memicmp(&CharBundle2,&CharBundle,sizeof(CharBundle))==0);
    sprintf(Message,"Queried and set Default character attributes are %s the same",
                     fOutcome==TRUE ? "" : "NOT");

    LOGINFO(L_DEBUG,"*",Message);
    if (fOutcome==FALSE)
       ErrorCount++;


    if (GpiQueryDefAttrs(hPS1,PRIM_MARKER,CHANGE_MARKER,&MarkerBundle2)==GPI_ERROR){
      SHOWERROR("GpiQueryDefAttrs()",EntryName);
    }

    fOutcome=(BOOL)(memicmp(&MarkerBundle2,&MarkerBundle,sizeof(MarkerBundle))==0);
    sprintf(Message,"Queried and set Default Marker attributes are %s the same",
                     fOutcome==TRUE  ? "" : "NOT");

    LOGINFO(L_DEBUG,"*",Message);
    if (fOutcome==FALSE)
      ErrorCount++;


    if (GpiQueryDefAttrs(hPS1,PRIM_IMAGE,CHANGE_IMAGE,&ImageBundle2)==FALSE){
      SHOWERROR("GpiQueryDefAttrs()",EntryName);
    }

    fOutcome=(BOOL)(memicmp(&ImageBundle2,&ImageBundle,sizeof(ImageBundle))==0);
    sprintf(Message,"Queried and set Default Image attributes are %s the same",
                     fOutcome==TRUE  ? "" : "NOT");

    LOGINFO(L_DEBUG,"*",Message);
    if (fOutcome==FALSE)
      ErrorCount++;


    if (GpiQueryDefViewingLimits(hPS1,&Rectl2)==FALSE){
      SHOWERROR("GpiQueryDefViewingLimits()",EntryName);
    }


    fOutcome=(BOOL)(memicmp(&Rectl2,&Rectl,sizeof(Rectl))==0);
    sprintf(Message,"Queried and set Default Viewing limits are %s the same",
                     fOutcome==TRUE  ? "" : "NOT");

     LOGINFO(L_DEBUG,"*",Message);
     if (fOutcome==FALSE)
       ErrorCount++;


    if (GpiQueryDefArcParams(hPS1,&ArcParams2)==FALSE){
      SHOWERROR("GpiQueryDefArcParams()",EntryName);
    }

    fOutcome=(BOOL)(memicmp(&ArcParams2,&ArcParams,sizeof(ArcParams))==0);
    sprintf(Message,"Queried and set Default ARC parameters are %s the same",
                     fOutcome==TRUE  ? "" : "NOT");

    LOGINFO(L_DEBUG,"*",Message);
    if (fOutcome==FALSE)
       ErrorCount++;

    if ((Return=GpiQueryLogColorTable(hPS1,LCOLOPT_INDEX,
                              (LONG)NULL,(LONG)NULL,&ReturnValue))==QLCT_ERROR){
      SHOWERROR("GpiQueryLogColorTable()",EntryName);
    }

    fOutcome=(BOOL)(Return==QLCT_RGB );
    sprintf(Message,"Color Table is %s loaded",fOutcome == TRUE? "" : "NOT");

    LOGINFO(L_DEBUG,"*",Message);
    if (fOutcome==FALSE)
      ErrorCount++;

    SETEXIT();


    //The following code segment will create and attach bitmaps
    // to pattern IDs in order to verify that the function
    // GreCopyDCLoadData does indeed copy bitmaps fron the source
    // DC to the target DC.
    SETENTER("Creating Bitmaps and tag them to Pattern IDs",NULL);

    for (usIndex=0 ;usIndex < NUMBEROFELEMENTS(ahBM); usIndex++){

       //Setup conditions to create empty bitmap.
       if ((ahBM[usIndex]=GreCreateBitmap(hDC,&BitMapHeader,
                                CBM_INIT,abPat2,pBitMapInfo))==FALSE){
           CLEANUP("GreCreateBitmap",EntryName);
           return;
       }

       //Tag the bitmap with a pattern ID.
       if (GreSetBitmapID(hDC,ahBM[usIndex],MY_BITMAP_ID+usIndex)==FALSE){
          CLEANUP("GreSetBitmapID()",EntryName);
          return;
       }
    }
    SETEXIT();

    SUBENTER(Calling the GreCopyDCLoadData again.,NULL);
    //If this call fails , then the bitmaps are still attached
    // to the source hPS. So assume this call will work.
     TargethPS=hPS1;

    if (GreCopyDCLoadData(hDC1,LCID_RANGE_BOTH,hDC)==FALSE){
       ErrorCount++;
       ERRSTATUS(TRUE,EntryName,"GreCopyDCLoadData failed.");
       TargethPS=hPS;
    }

    SUBEXIT();

    SETENTER("Comparing Device Context's attributes after using GreCopyDCLoadData",NULL);
    fOutcome=TRUE;  //Assume that things did work.

    if ((ReturnValue=GpiQueryNumberSetIds(TargethPS))==GPI_ALTERROR){
      SHOWERROR("GpiQueryNumberSetId()",EntryName);
      fOutcome=FALSE;
    }
    else {
      if (GpiQuerySetIds(TargethPS,ReturnValue,alTypes,aszFontNames,
                        alFonts)==FALSE){
        SHOWERROR("GpiQuerySetId()",EntryName);
        LOGINFO(L_DEBUG,"*","Will not be able to test for Bitmap IDs"
                            " since GpiQuerySetIds Failed.");
        fOutcome=FALSE;

      }
      else{
        for(usIndex=0,Count=0 ;usIndex < (USHORT)ReturnValue; usIndex++){
          if ((alTypes[usIndex]==LCIDT_BITMAP) &&
             (alFonts[usIndex]>=MY_BITMAP_ID)   &&
             (alFonts[usIndex]< (MY_BITMAP_ID+(NUMBEROFELEMENTS(ahBM))))){
            Count++;
          }
        }

        fOutcome=(Count==(NUMBEROFELEMENTS(ahBM)));
        sprintf(Message,"Bitmap pattern IDs are  %s created",fOutcome==TRUE ? "" : "NOT");
        LOGINFO(L_DEBUG,"*",Message);

        if (fOutcome==FALSE)
          ErrorCount++;
      }
     }

     SETEXIT();

    /*=================================================================
      The purpose of next code segment is do some house cleaning
      before exiting this routine. the following is done:
      1. De-select any bitmaps that are selected into memory Device
         Contexts or/and Un-tag any bitmaps that are used as pattern.
      2. Delete any bitmaps created within this function.
      3. Delete any Presentation Spaces created within this function.
      4. Delate any Device Contexts created within this function.

         Note that these steps must be followed as presented above,
         otherwise; most GPIs used here will fail.
    ===================================================================*/

    if (GpiDeleteSetId(TargethPS,LCID_ALL)==FALSE){
         SHOWERROR("GpiDeleteSetId()",EntryName);
    }

    for(usIndex=0; usIndex<(NUMBEROFELEMENTS(ahBM));usIndex++){
      if (GreDeleteBitmap(ahBM[usIndex])==FALSE){
         SHOWERROR("GreDeleteBitmap()",EntryName);
      }
    }

    // dispose of presentation space. No longer needed.
    if (GpiDestroyPS(hPS1)==FALSE){
       CLEANUP("GpiDestroyPS()",EntryName);
       return ;
    }

    //Close the device context
    if (DevCloseDC(hDC1)==DEV_ERROR){
      CLEANUP("DevCloseDC()",EntryName);
      return ;
    }

    EXIT_MAIN (EntryName);
  }
}
