/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/*const char header=$header$*/

#define INCL_DEV                    //Include all DEV definitions
#define INCL_WIN                    //Include all WIN definitions
#define INCL_GREALL                 // include all Gre definitions
#define INCL_GPI                    //Include all Gpi definitions
#define INCL_ERRORS                 //Include all error definitions
#define INCL_DOSMEMMGR
#define INCL_DOSERRORS
#define INCL_DOSFILEMGR              // File Management
#define INCL_DOSMEMMGR               //Memory Management
#define INCL_DDIMISC
#define INCL_DDIBUNDLES

#include <string.h>
#include <stdlib.h>
#include <os2.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#include <pmddi.h>                  // gre - device driver includes
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "grebit2.h"                // bit map dll header file


/****************************************************************************
 *\\ddd
 * Routine Name: GreCreateBitmapRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreCreateBitmap().
 *     The syntax used to call the Gre is :
 *
 *      (HBITMAP) GreCreateBitmap(HDC               hDC,
 *                                PBITMAPINFOHEADER New,
 *                                LONG              Options,
 *                                PBYTE             abData,
 *                                PBITMAPINFO       InfoTable
 *                               )
 *     This Gre creates a bit map and returns the bit-map handle.
 *
 *     Bit-map handle/error indicator:
 *
 *        <>0              New bit-map handle
 *        FALSE            Error.
 *
 *     Possible returns from WinGetLastError:
 *
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INSUFFICIENT_MEMORY
 *        PMERR_INVALID_LENGTH_OR_COUNT
 *        PMERR_INV_INFO_TABLE
 *        PMERR_INV_USAGE_PARM
 *        PMERR_INV_BITMAP_DIMENSION
 *        PMERR_INV_SCAN_START
 *
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/16/91, PDVT ,Mike Abuzant
 *               -TestCase Design and conding.
 *               09/23/91 ,PDVT ,Mike Abuzant.
 *               -Produced PMERR_INSUFFICIENT_MEMORY error code.
 *               10/01/91 ,PDVT ,Mike Abuzant.
 *               -Allocated the array 'ahSel' instead of having it as
 *               - static array.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *        PMERR_INVALID_LENGTH_OR_COUNT
 *        PMERR_INV_INFO_TABLE
 *        PMERR_INV_USAGE_PARM
 *        PMERR_INV_BITMAP_DIMENSION
 *        PMERR_INSUFFICIENT_MEMORY
 *
 * Return Codes Not Tested Successfully Yet:
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INV_SCAN_START
 *
 * Subroutines Required:
 *      None.
 *
 * Limitations:
 *
 *  To produce the error code PMERR_INSUFFICIENT_MEMORY, a large file is created
 *  on 'C' drive to prevent OS/2 from swapping to the hard drive when we
 *  allocate all available memory. This big file is deleted after producing
 *  the error code in question, but should but deleted if something goes
 *  wrong.
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreCreateBitmap(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/

VOID APIENTRY GreCreateBitmapRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static  PCHAR    EntryName="GreCreateBitmapRc()";
  HBITMAP          hBMSuccess;
  BITMAPINFOHEADER BitMapHeader;      //bitmap info header
  PBITMAPINFO      pBitMapInfo;       //bitmap info structure
  BYTE             Temp_buffer[200];
  HDC              hDC;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER(EntryName,NULL);

    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
         CLEANUP("GpiQueryDevice()",EntryName);
         return;
    }

    SETEXIT();

   // Initalize the Bitmap information structure
   // first we need to  allocate memory to hold RGB array
   // since  BITMAPINFO structure is large enough to hold
   // one RGB structure.

    pBitMapInfo=(PBITMAPINFO) Temp_buffer;
    pBitMapInfo->cbFix=sizeof(BITMAPINFOHEADER); // This always =12
    pBitMapInfo->cx=SHAPE_WIDTH;                 //Width
    pBitMapInfo->cy=SHAPE_HEIGHT;                 //height
    pBitMapInfo->cPlanes=1;                      // one color plane
    pBitMapInfo->cBitCount=1;                    // 1-bit /pixel
    pBitMapInfo->argbColor[0].bBlue=
    pBitMapInfo->argbColor[0].bGreen=
    pBitMapInfo->argbColor[0].bRed=0;
    pBitMapInfo->argbColor[1].bBlue=
    pBitMapInfo->argbColor[1].bGreen=
    pBitMapInfo->argbColor[1].bRed=0xFF;

    BitMapHeader.cbFix=sizeof(BITMAPINFOHEADER); // This always =12
    BitMapHeader.cx=SHAPE_WIDTH;                 //Width
    BitMapHeader.cy=SHAPE_HEIGHT;                //height
    BitMapHeader.cPlanes=1;                      // one color plane
    BitMapHeader.cBitCount=1;                    // 1-bit /pixel



   /* ***************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say hDC=NULL ,to the function expecting
    // an error

    hBMSuccess=GreCreateBitmap((HDC)NULL,NULL,CBM_INIT,NULL,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    hBMSuccess=GreCreateBitmap((HDC)0x0ffffffL,NULL,CBM_INIT,NULL,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    hBMSuccess=GreCreateBitmap((HDC)-1l,NULL,CBM_INIT,NULL,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/

    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);               //Make this PS busy.
    hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,(ULONG)NULL,NULL,NULL);
    CHECKRC(hBMSuccess,PMERR_HDC_BUSY,NULL);
    (VOID)FREEPS;                       //Kill BUSY thread.
    SUBEXIT();

    /**************************************************************
    * The following Code will check for the PMERR_INV_USAGE_PARM  *
    * error code                                                  *
    **************************************************************/

    SUBENTER(PMERR_INV_USAGE_PARM ,"Test1");
    // Pass an invalid option ,say option=0xfffffL,to the function expecting
    // an error
    hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,-1L,NULL,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_USAGE_PARM ,"Test1");
    SUBEXIT();

    // Pass an invalid option ,say option=0x0fffffL,to the function expecting
    // an error
    SUBENTER(PMERR_INV_USAGE_PARM ,"Test2");
    hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,0x0fffffL,NULL,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_USAGE_PARM ,"Test2");
    SUBEXIT();

    /**************************************************************
    * The following Code will check for the PMERR_INV_INFO_TABLE  *
    * error code                                                  *
    **************************************************************/
    // set pBitMapInfo->cbFix=0 ( Structure size must be 12 Bytes )
     pBitMapInfo->cbFix=0;

    SUBENTER(PMERR_INV_INFO_TABLE,NULL);
    hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,CBM_INIT,abCross,pBitMapInfo);
    CHECKRC(hBMSuccess,PMERR_INV_INFO_TABLE,NULL);
    SUBEXIT();
    /*****************************************************************
    * The following Code will check for the PMERR_INV_LENGTH_OR_COUNT *
    * error code                                                      *
    ******************************************************************/
    // set pBitHeaderInfo->cbFix=0x444444L ( Structure size must be 12 Bytes )

    pBitMapInfo->cbFix=sizeof(BitMapHeader);
    BitMapHeader.cbFix=0x4444444L;

    SUBENTER(PMERR_INV_LENGTH_OR_COUNT," Test1");
    hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,CBM_INIT,abCross,pBitMapInfo);
    CHECKRC(hBMSuccess,PMERR_INV_LENGTH_OR_COUNT,"Test1");
    SUBEXIT();

    // set pBitHeaderInfo->cbFix=-1L ( Structure size must be 12 Bytes )

     BitMapHeader.cbFix=-1L;

     SUBENTER(PMERR_INV_LENGTH_OR_COUNT," Test2");
     hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,CBM_INIT,abCross,pBitMapInfo);
     CHECKRC(hBMSuccess,PMERR_INV_LENGTH_OR_COUNT,"Test2");
     SUBEXIT();

     BitMapHeader.cbFix=sizeof(BITMAPINFOHEADER);
    /*****************************************************************
    * The following Code will check for the PMERR_INV_BITMAP_DIMENSION *
    * error code                                                      *
    ******************************************************************/
    // set BitMapHeader.cx=meaningless value
     BitMapHeader.cx=(USHORT)65536;

     SUBENTER(PMERR_INV_BITMAP_DIMENSION," Test1");
     hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,CBM_INIT,abCross,pBitMapInfo);
     CHECKRC(hBMSuccess,PMERR_INV_BITMAP_DIMENSION,"Test2");
     SUBEXIT();

    // set BitMapHeader.cx=meaningless value
     BitMapHeader.cy=(USHORT)65536;
     BitMapHeader.cx=SHAPE_WIDTH;

     SUBENTER(PMERR_INV_BITMAP_DIMENSION," Test2");
     hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,CBM_INIT,abCross,pBitMapInfo);
     CHECKRC(hBMSuccess,PMERR_INV_BITMAP_DIMENSION,"Test2");
     SUBEXIT();

    // set BitMapHeader.cx * BitMapHeader.cy > 64k
     BitMapHeader.cy=2;
     BitMapHeader.cx=38000U;

     SUBENTER(PMERR_INV_BITMAP_DIMENSION," Test3");
     hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,(ULONG)NULL,NULL,NULL);
     CHECKRC(hBMSuccess,PMERR_INV_BITMAP_DIMENSION,"Test3");
     SUBEXIT();

    BitMapHeader.cx=SHAPE_WIDTH;                 //Width
    BitMapHeader.cy=SHAPE_HEIGHT;                //height

   /* ****************************************************************
    * The following Code will check for the PMERR_INSUFFICIENT_MEMORY *
    *  error Code                                                     *
    ******************************************************************/
    BitMapHeader.cx=
    pBitMapInfo->cx=2000;                 //Width
    BitMapHeader.cy=
    pBitMapInfo->cy=2000;                 //height

    SUBENTER(PMERR_INSUFFICIENT_MEMORY,NULL);

    SETENTER("Exhausting OS/2 memory..",NULL);

    if (InitNoMemoryTesting(CREATE_TESTING_ENV)==FALSE){
       LOGINFO(L_HDR,"*","Cannot create an enviroment to test\n"
                         " PMERR_INSUFFICIENT_MEMORY error code");
       SETEXIT();
    }
    else{
      SETEXIT();
      hBMSuccess=GreCreateBitmap(hDC,&BitMapHeader,(ULONG)NULL,NULL,NULL);

      SETENTER("Releasing OS/2 memory...",NULL);
      (VOID)InitNoMemoryTesting(CLEAN_TESTING_ENV);
      SETEXIT();

      CHECKRC(hBMSuccess,PMERR_INSUFFICIENT_MEMORY,NULL);

    }

    SUBEXIT();

    EXIT_MAIN(EntryName);

  }
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreDeleteBitmapRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreDeleteBitmap().
 *     The syntax used to call the Gre is :
 *
 *       (BOOL) GreDeleteBitmap(HBITMAP hBM
 *                             )
 *
 *     This Gre will delete the bitmap that has been created by
 *     GreCreateBitmap().
 *
 *     The return value is:
 *
 *        TRUE       Successful completion
 *        FALSE      Error occurred.
 *
 *     Possible returns from WinGetLastError:
 *
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INSUFFICIENT_MEMORY
 *        PMERR_INVALID_LENGTH_OR_COUNT
 *        PMERR_INV_HDC
 *        PMERR_INV_HBITMAP
 *        PMERR_BITMAP_IS_SELECTED
 *        PMERR_HBITMAP_BUSY
 *        PMERR_INV_INFO_TABLE
 *        PMERR_INV_CAN_START
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/16/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/16/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HBITMAP
 *        PMERR_BITMAP_IS_SELECTED
 *
 * Return Codes Not Tested Successfully Yet:
 *
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INSUFFICIENT_MEMORY
 *        PMERR_INVALID_LENGTH_OR_COUNT
 *        PMERR_INV_HDC
 *        PMERR_HBITMAP_BUSY
 *        PMERR_INV_INFO_TABLE
 *        PMERR_INV_CAN_START
 *
 * Subroutines Required:
 *        None.
 * Limitations:
 *        None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreDeleteBitmap(hDC...);
 *     if ( ulSuccess== GPI_ERROR) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeleteBitmapRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static  PCHAR  EntryName="GreDeleteBitmapRc()";
  BOOL     fSuccess;     //Holds return codes.
  HBITMAP  hBM;
  HDC      hDC;
  BITMAPINFOHEADER BitMapHeader;      //bitmap info header

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    BitMapHeader.cbFix=sizeof(BITMAPINFOHEADER); // This always =12
    BitMapHeader.cx=SHAPE_WIDTH;                 //Width
    BitMapHeader.cy=SHAPE_HEIGHT;                //height
    BitMapHeader.cPlanes=1;                      // one color plane
    BitMapHeader.cBitCount=1;                    // 1-bit /pixel

    SETENTER(EntryName,NULL);
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
         CLEANUP("GpiQueryDevice()",EntryName);
         return;
    }

    SETEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_HBITMAP error  *
    * code.                                                          *
    *****************************************************************/
    SUBENTER(PMERR_INV_HBITMAP,"Test1");
    // Pass an invalid HBITMAP ,say hBM=NULL ,to the function expecting
    // an error
    fSuccess=GreDeleteBitmap((HBITMAP)NULL);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HBITMAP,"Test2");
    // Pass an invalid HBITMAP ,say hBM=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreDeleteBitmap((HBITMAP)0x0ffffffL);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HBITMAP,"Test3");
    // Pass an invalid HBITMAP ,say hBM=-1L,to the function expecting
    // an error
    fSuccess=GreDeleteBitmap((HBITMAP)-1L);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test3");
    SUBEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_BITMAP_IS_SELECTED*
    * error Code                                                    *
    *****************************************************************/
    SUBENTER(PMERR_BITMAP_IS_SELECTED,NULL);

    SETENTER(EntryName, " Testing PMERR_BITMAP_IS_SELECTED");
    //Create A bitmap for testing purposes.

    if ((hBM=GreCreateBitmap(hDC,&BitMapHeader,(ULONG)NULL,NULL,NULL))==NULL){
        CLEANUP("GreCreatBitMap()",EntryName);
        return;
    }

    //Tag the bitmap with a pattern ID.
    if (GreSetBitmapID(hDC,hBM,MY_BITMAP_ID)==FALSE){
         CLEANUP("GreSetBitmapID()",EntryName);
         return;
    }

    //Use this bitmap as the default pattern.
    if (GpiSetPatternSet(hPS,MY_BITMAP_ID)==GPI_ERROR){
        CLEANUP("GpiSetPatternSet",EntryName);
        return;
    }

    SETEXIT();

    fSuccess=GreDeleteBitmap(hBM);
    CHECKRC(fSuccess,PMERR_BITMAP_IS_SELECTED,NULL);
    SUBEXIT();

    /**************************************************************
    * This next code segment prepares the bitmap for deletion.    *
    ***************************************************************/
    SETENTER(EntryName," Cleaning Up after testing");
    //Set the default pattern to system DEFAULT. You should this before
    //deleting the current bitmap pattern.
    if (GpiSetPatternSet(hPS,LCID_DEFAULT)==GPI_ERROR){
        CLEANUP("GpiSetPatternSet()",EntryName);
        return;
    }

    //Un-tag bitmap from  the pattern ID.
    if (GpiDeleteSetId(hPS,MY_BITMAP_ID)==FALSE){
         CLEANUP("GpiSetBitmapId()",EntryName);
         return;
    }

    //Release menory space occupied by this bitmap
    if (GpiDeleteBitmap(hBM)==FALSE){
         CLEANUP("GpiDeleteBitmap()",EntryName);
         return;
    }

    SETEXIT();
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreGetBitmapParametersRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreGetBitmapParameters().
 *     The syntax used to call the gre is :
 *
 *
 *     (BOOL) GreGetBitmapParameters(HBITMAP hBM,
 *                                     PBITMAPINFOHEADER BitmapInfo
 *                                    )
 *     This gre call returns information about a bit map identified by
 *     the bit-map handle. 'BitmapInfo' structure is filled with data
 *     for the specified bit map. The structure includes the elements
 *     (width, height, planes, bitcount) of a bit-map information table.
 *
 *     The return value is:
 *       TRUE       Successful completion
 *       FALSE      Error occurred.
 *
 *     Possible returns from WinGetLastError:
 *       PMERR_INV_HBITMAP
 *       PMERR_BITMAP_IS_SELECTED
 *       PMERR_HBITMAP_BUSY
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/16/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/16/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *       PMERR_INV_HBITMAP
 *
 * Return Codes Not Tested Successfully Yet:
 *       PMERR_HBITMAP_BUSY
 *       PMERR_BITMAP_IS_SELECTED
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *        None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the gre.
 *     ulSuccess= GreGetBitmapParameters(hDC...);
 *     if ( ulSuccess== gre_ERROR) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID APIENTRY GreGetBitmapParametersRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static  PCHAR  EntryName="GreGetBitmapParametersRc()";
  BOOL   fSuccess;                          //Holds return codes.
  BITMAPINFOHEADER BitMapHeader;      //bitmap info header
  HDC                    hDC;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);

    DeletePrinterOutput(EntryName);

    BitMapHeader.cbFix=sizeof(BITMAPINFOHEADER); // This always =12
    BitMapHeader.cx=SHAPE_WIDTH;                 //Width
    BitMapHeader.cy=SHAPE_HEIGHT;                //height
    BitMapHeader.cPlanes=1;                      // one color plane
    BitMapHeader.cBitCount=1;                    // 1-bit /pixel

    SETENTER(EntryName,NULL);
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
         CLEANUP("GpiQueryDevice()",EntryName);
         return;
    }


    SETEXIT();


    /*****************************************************************
    * The following Code will check for the PMERR_INV_HBITMAP error  *
    * code.                                                          *
    *****************************************************************/
    SUBENTER(PMERR_INV_HBITMAP,"Test1");
    // Pass an invalid HBITMAP ,say hBM=NULL ,to the function expecting
    // an error
    fSuccess=GreGetBitmapParameters((HBITMAP)NULL,&BitMapHeader);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test1");
    SUBEXIT();


    SUBENTER(PMERR_INV_HBITMAP,"Test2");
    // Pass an invalid HBITMAP ,say hBM=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreGetBitmapParameters((HBITMAP)0x0ffffffL,&BitMapHeader);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HBITMAP,"Test3");
    // Pass an invalid HBITMAP ,say hBM=-1L,to the function expecting
    // an error
    fSuccess=GreGetBitmapParameters((HBITMAP)-1L,&BitMapHeader);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test3");
    SUBEXIT();

    EXIT_MAIN(EntryName);

  }
  return;

}
/****************************************************************************
 *\\ddd
 * Routine Name: GreGetBitmapDimensionRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreGetBitmapDimension().
 *     The syntax used to call the Gre is :
 *
 *       (BOOL) GreGetBitmapDimension(HBITMAP hBM,
 *                                      PSIZEL  BitmapDimension
 *                                     )
 *     This Gre returns the width and height of a bit map, as specified
 *     on a previous GreSetBitmapDimension call. The width and height of
 *     the bit map in 0.1 millimeter units. If not set by
 *     GreSetBitmapDimension, zeros are returned.
 *
 *     The return value is:
 *        TRUE       Successful completion
 *        FALSE      Error occurred.
 *
 *     Possible returns from WinGetLastError:
 *        PMERR_INV_HBITMAP
 *        PMERR_HBITMAP_BUSY
 *        PMERR_BITMAP_IS_SELECTED
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/16/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/16/91, PDVT ,Mike Abuzant
 *               -TestCase Design and conding.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HBITMAP
 *
 * Return Codes Not Tested Successfully Yet:
 *        PMERR_HBITMAP_BUSY
 *        PMERR_BITMAP_IS_SELECTED
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *        None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreGetBitmapDimension(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID APIENTRY GreGetBitmapDimensionRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static  PCHAR  EntryName="GreGetBitmapDimensionRc()";
  BOOL   fSuccess;      //Holds return code
  SIZEL  sizel={0};     //Holds the dimension for the bitmap

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    /*****************************************************************
    * The following Code will check for the PMERR_INV_HBITMAP error  *
    * code.                                                          *
    *****************************************************************/
    SUBENTER(PMERR_INV_HBITMAP,"Test1");
    // Pass an invalid HBITMAP ,say hBM=NULL ,to the function expecting
    // an error
    fSuccess=GreGetBitmapDimension((HBITMAP)NULL,&sizel);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test1");
    SUBEXIT();


    SUBENTER(PMERR_INV_HBITMAP,"Test2");
    // Pass an invalid HBITMAP ,say hBM=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreGetBitmapDimension((HBITMAP)0x0ffffffL,&sizel);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test2");
    SUBEXIT();


    SUBENTER(PMERR_INV_HBITMAP,"Test3");
    // Pass an invalid HBITMAP ,say hBM=-1L,to the function expecting
    // an error
    fSuccess=GreGetBitmapDimension((HBITMAP)-1L,&sizel);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test3");
    SUBEXIT();
    EXIT_MAIN(EntryName);

  }
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetBitmapDimensionRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreSetBitmapDimension().
 *     The syntax used to call the Gre is :
 *
 *       (BOOL) GreSetBitmapDimension(HBITMAP hBM,
 *                                      PSIZEL  BitmapDimension
 *                                     )
 *     This Gre sets the width and height of a bit map. These values can
 *     read by a call to  GreGetBitmapDimension call. The width and height of
 *     the bit map in 0.1 millimeter units.
 *
 *     The return value is:
 *        TRUE       Successful completion
 *        FALSE      Error occurred.
 *
 *     Possible returns from WinGetLastError:
 *        PMERR_INV_HBITMAP
 *        PMERR_HBITMAP_BUSY
 *        PMERR_BITMAP_IS_SELECTED
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/16/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/16/91, PDVT ,Mike Abuzant
 *               -TestCase Design and conding.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HBITMAP
 *
 * Return Codes Not Tested Successfully Yet:
 *        PMERR_HBITMAP_BUSY
 *        PMERR_BITMAP_IS_SELECTED
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *        None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSetBitmapDimension(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetBitmapDimensionRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static  PCHAR  EntryName="GreSetBitmapDimensionRc()";
  BOOL   fSuccess;      //Holds return code
  SIZEL  sizel;         //Holds the dimension for the bitmap

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    /*****************************************************************
    * The following Code will check for the PMERR_INV_HBITMAP error  *
    * code.                                                          *
    *****************************************************************/
    SUBENTER(PMERR_INV_HBITMAP,"Test1");
    // Pass an invalid HBITMAP ,say hBM=NULL ,to the function expecting
    // an error
    fSuccess=GreSetBitmapDimension((HBITMAP)NULL,&sizel);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test1");
    SUBEXIT();


    SUBENTER(PMERR_INV_HBITMAP,"Test2");
    // Pass an invalid HBITMAP ,say hBM=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreSetBitmapDimension((HBITMAP)0x0ffffffL,&sizel);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test2");
    SUBEXIT();


    SUBENTER(PMERR_INV_HBITMAP,"Test3");
    // Pass an invalid HBITMAP ,say hBM=-1L,to the function expecting
    // an error
    fSuccess=GreSetBitmapDimension((HBITMAP)-1L,&sizel);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test3");
    SUBEXIT();
    EXIT_MAIN(EntryName);

  }
  return;
}

 /****************************************************************************
 *\\ddd
 * Routine Name: GreSetBitmapIdRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreSetBitmapId().
 *     The syntax used to call the Gre is :
 *
 *     BOOL  GreSetBitmapId(HPS     hPS,
 *                          HBITMAP hbm,
 *                          LONG    Lcid
 *                         )
 *
 *     This call tags a bit map with a local identifier, so that it can
 *     be used as a pattern set, containing a single member.
 *     The bit map must not be currently selected into a device context.
 *
 *     The return value is:
 *
 *       TRUE       Successful completion
 *       FALSE      Error occurred.
 *
 *     Possible returns from WinGetLastError:
 *
 *       PMERR_HDC_BUSY
 *       PMERR_INV_HDC
 *       PMERR_INV_BITMAP
 *       PMERR_BITMAP_IS_SELECTED
 *       PMERR_HBITMAP_BUSY
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/16/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/16/91, PDVT ,Mike Abuzant
 *               -TestCase Design and conding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *       PMERR_HDC_BUSY
 *       PMERR_INV_HDC
 *       PMERR_INV_BITMAP
 *       PMERR_BITMAP_IS_SELECTED
 *       PMERR_HBITMAP_BUSY
 *
 * Return Codes Not Tested Successfully Yet:
 *       PMERR_HBITMAP_BUSY
 *       PMERR_BITMAP_IS_SELECTED
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *        None at this time.
 *
 * Note:
 *   Two Return codes where tested even though they are not listed
 *   under this function. It seeded logical for this function to return
 *   the following return codes:
 *   above return codes.
 *
 *     PMERR_INV_SETID
 *     PMERR_SETID_IN_USE.
 *
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSetBitmapId(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetBitmapIdRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static  PCHAR  EntryName="GreSetBitmapIDRc()";
  BOOL     fSuccess;                  //Holds the return code
  BOOL     fBusyFailed;               //In case making HDc busy fails.
  HBITMAP  hBM;
  BYTE  Temp_buffer[100];
  BITMAPINFOHEADER BitMapHeader;      //bitmap info header
  PBITMAPINFO      pBitMapInfo;       //bitmap info structure
  HDC              hDC;
  HDC              hDC1;
  HPS              hPS1;
  HAB              hAB;
  SIZEL            sizel={0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    // Initalize the Bitmap information structure
    // first we need to  allocate memory to hold RGB array
    // since  BITMAPINFO structure is large enough to hold
    // one RGB structure.

     pBitMapInfo=(PBITMAPINFO)Temp_buffer;
     pBitMapInfo->cbFix=sizeof(BitMapHeader);   // This always =12
     pBitMapInfo->cPlanes=                      // one color plane
     pBitMapInfo->cBitCount=1;                  // 1-bit /pixel
     pBitMapInfo->argbColor[0].bBlue=
     pBitMapInfo->argbColor[0].bGreen=
     pBitMapInfo->argbColor[0].bRed=0;
     pBitMapInfo->argbColor[1].bBlue=
     pBitMapInfo->argbColor[1].bGreen=
     pBitMapInfo->argbColor[1].bRed=0xFF;

     BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
     BitMapHeader.cPlanes=                      // one color plane
     BitMapHeader.cBitCount=1;                  // 1-bit /pixel
     pBitMapInfo->cx=
     BitMapHeader.cx=PAT1_WIDTH;
     pBitMapInfo->cy=
     BitMapHeader.cy=PAT1_HEIGHT;


    SETENTER(EntryName,NULL);

    //Anchor Block is needed when creation presentation space.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
        CLEANUP("WinQueryAnchorBlock()",EntryName);
        return ;
    }

    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
         CLEANUP("GpiQueryDevice()",EntryName);
         return;
    }
    //Create the first Device Context.
    if ((hDC1=DevOpenDC(hAB,OD_MEMORY,"*",0L,(PDEVOPENDATA)NULL,
                        hDC))==DEV_ERROR){
       CLEANUP("DevOpenDC()",EntryName);
       return ;
    }
    // Create the first presentation space to associate with the
    // memory device context

    if ((hPS1=GpiCreatePS(hAB,hDC1,&sizel,PU_PELS |
                          GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR){
       CLEANUP("GpiCreatePS()",EntryName);
       return ;
    }

    // Create a bitmap to use it as pattern .
    if ((hBM=GreCreateBitmap(hDC,&BitMapHeader,
                           CBM_INIT,abPat1,
                           pBitMapInfo))==FALSE){
         CLEANUP("GreCreateBitmap()",EntryName);
         return ;
    }

    SETEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_BITMAP_IS_SELECTED *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_BITMAP_IS_SELECTED,NULL);

    //Select bitmap into the first Presentation Space and then try
    //to select the same bitmap into the second Presentation Space.
    if (GpiSetBitmap(hPS1,hBM)==HBM_ERROR){
       CLEANUP("GpiSetBitmap()",EntryName);
       return;
    }

    fSuccess=GreSetBitmapID(hDC,hBM,MY_BITMAP_ID);
    CHECKRC(fSuccess,PMERR_BITMAP_IS_SELECTED,NULL);
    SUBEXIT();


    //De-Select bitmap from PS

    if (GpiSetBitmap(hPS1,(HBITMAP)NULL)==HBM_ERROR){
       CLEANUP("GpiSetBitmap()",EntryName);
       return;
    }


    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreSetBitmapID((HDC)NULL,NULL,NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say hDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreSetBitmapID((HDC)0x0ffffffL,NULL,NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say hDC=-1L,to the function expecting
    // an error
    fSuccess=GreSetBitmapID((HDC)-1L,NULL,NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_HBITMAP error  *
    * code.                                                          *
    *****************************************************************/
    SUBENTER(PMERR_INV_HBITMAP,"Test1");
    // Pass an invalid HBITMAP ,say hBM=NULL ,to the function expecting
    // an error
    fSuccess=GreSetBitmapID(hDC,(HBITMAP)NULL,MY_BITMAP_ID);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HBITMAP,"Test2");
    // Pass an invalid HBITMAP ,say hBM=0x0ffffffL,to the function expecting
    // an error
    fSuccess=GreSetBitmapID(hDC,(HBITMAP)0x0ffffffL,2L);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HBITMAP,"Test3");
    // Pass an invalid HBITMAP ,say hBM=-1L,to the function expecting
    // an error
    fSuccess=GreSetBitmapID(hDC,(HBITMAP)-1L,2L);
    CHECKRC(fSuccess,PMERR_INV_HBITMAP,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/

    SUBENTER(PMERR_HDC_BUSY,NULL);


    (VOID)MAKEPSBUSY(hPS);   //Make This PS busy
    fBusyFailed=
    fSuccess=GreSetBitmapID(hDC,hBM,MY_BITMAP_ID);
    CHECKRC(fSuccess,PMERR_HDC_BUSY,NULL);
    SUBEXIT();
    (VOID)FREEPS;           //Kill the BUSY thread.

   /******************************************************************
    * The following Code will check for the PMERR_INV_SETID error Code*
    *******************************************************************/
    SUBENTER(PMERR_INV_SETID,"Test1");
    // Pass an invalid ID ,say ID=NULL ,to the function expecting
    // an error
    fSuccess=GreSetBitmapID(hDC,hBM,NULL);
    CHECKRC(fSuccess,PMERR_INV_SETID,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_SETID,"Test2");
    // Pass an invalid ID ,say ID=0x0fffL,to the function expecting
    // an error
    fSuccess=GreSetBitmapID(hDC,hBM,0xfffL);
    CHECKRC(fSuccess,PMERR_INV_SETID,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_SETID,"Test3");
    // Pass an invalid ID ,say ID=-1L,to the function expecting
    // an error
    fSuccess=GreSetBitmapID(hDC,hBM,-1L);
    CHECKRC(fSuccess,PMERR_INV_SETID,"Test3");
    SUBEXIT();

   /******************************************************************
    * The following Code will check for the PMERR_SETID_IN_USE error *
    * code                                                           *
    ******************************************************************/

    SUBENTER(PMERR_SETID_IN_USE,NULL);
    if (fBusyFailed==FALSE){  //already set.
      //Tag a pattern ID to this bitmap in order as a pattern
       if (GreSetBitmapID(hDC,hBM,MY_BITMAP_ID)==FALSE){
           CLEANUP("GreSetBitmapID()",EntryName);
           return;
       }
    }
    fSuccess=GreSetBitmapID(hDC,hBM,MY_BITMAP_ID);
    CHECKRC(fSuccess,PMERR_SETID_IN_USE,NULL);
    SUBEXIT();

    /**************************************************************
    * This next code segment prepares the bitmap for deletion.    *
    ***************************************************************/
    //Un-tag this Pattern ID.                                             Line to Go To
    if (GreDeleteSetId(hDC,MY_BITMAP_ID)==FALSE){
         CLEANUP("GreDeleteSetId()",EntryName);
         return;
    }

    //Release memory space occupied by this bitmap.
    if (GreDeleteBitmap(hBM)==FALSE){
         CLEANUP("GreDeleteBitmap()",EntryName);
         return;
    }

    // dispose of presentation space. No longer needed.
    if (GpiDestroyPS(hPS1)==FALSE){
         CLEANUP("GpiDestroyPS()",EntryName);
         return ;
    }

    //Close the device context
    if (DevCloseDC(hDC1)==DEV_ERROR){
      CLEANUP("DevCloseDC()",EntryName);
      return ;
    }

    EXIT_MAIN(EntryName);
  }
  return;
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryBitmapHandleRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreQueryBitmapHandle().
 *     The syntax used to call the Gre is :
 *
 *      HBITMAP GreQueryBitmapHandle(HDC hDC,
 *                                   LONG Lcid
 *                                   )
 *
 *      This call returns the handle of the bit map currently tagged
 *      with the specified local identifier (lcid).
 *      The return value is
 *        <>0              Bit-map handle
 *        GPI_ERROR        Error.
 *
 *      Possible returns from WinGetLastError:
 *
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *        PMERR_INV_SETID
 *        PMERR_BITMAP_NOT_SELECTED
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/17/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/17/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HPS
 *        PMERR_PS_BUSY
 *        PMERR_INV_SETID
 *        PMERR_BITMAP_NOT_SELECTED
 * Return Codes Not Tested Successfully Yet:
 *        None
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *        None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreQueryBitmapHandle(hDC...);
 *     if ( ulSuccess== GPI_ERROR) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryBitmapHandleRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static   PCHAR EntryName="GreQueryBitmapHandleRc";
  HBITMAP  hBMSuccess=(HBITMAP)NULL,  //Variable to hold Return code
           hBM=(HBITMAP)NULL;         //Bitmap Handle.
  BITMAPINFOHEADER BitMapHeader;      //bitmap info header
  HDC     hDC;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER(EntryName,NULL);

    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
         CLEANUP("GpiQueryDevice()",EntryName);
         return;
    }

    SETEXIT();

    BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
    BitMapHeader.cPlanes=                      // one color plane
    BitMapHeader.cBitCount=1;                  // 1-bit /pixel
    BitMapHeader.cx=PAT1_WIDTH;
    BitMapHeader.cy=PAT1_HEIGHT;

   /*****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    ****************************************************************/
    SUBENTER(PMERR_INV_HDC,"TEST1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    hBMSuccess=GreQueryBitmapHandle((HDC)NULL,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"TEST2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    hBMSuccess=GreQueryBitmapHandle((HDC)0x0ffffffL,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"TEST3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    hBMSuccess=GreQueryBitmapHandle((HDC)-1L,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_BITMAP_NOT_SELECTED*
    * error Code                                                     *
    *****************************************************************/
    SUBENTER(PMERR_BITMAP_NOT_SELECTED,NULL);
    hBMSuccess=GreQueryBitmapHandle(hDC,MY_BITMAP_ID);
    CHECKRC(hBMSuccess,PMERR_BITMAP_NOT_SELECTED,NULL);
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/
    SUBENTER(PMERR_HDC_BUSY,NULL);

    // Create a bitmap to use it as pattern .
    if ((hBM=GreCreateBitmap(hDC,&BitMapHeader,
                           !CBM_INIT,NULL,
                           NULL))==FALSE){
         CLEANUP("GreCreateBitmap()",EntryName);
         return ;
    }

    // Tag a pattern ID with this bitmap in order to use as pattern
    if (GreSetBitmapID(hDC,hBM,MY_BITMAP_ID)==FALSE){
        CLEANUP("GreSetBitmapID()",EntryName);
        return;
    }
    (VOID)MAKEPSBUSY(hPS);    //Make PS busy
    hBMSuccess=GreQueryBitmapHandle(hDC,MY_BITMAP_ID);
    CHECKRC(hBMSuccess,PMERR_HDC_BUSY,NULL);
    (VOID)FREEPS;             //Kill the BUSY thread.
    SUBEXIT();
    //Un-tag this Pattern ID.
    if (GreDeleteSetId(hDC,MY_BITMAP_ID)==FALSE){
         CLEANUP("GreDeleteBitmapId()",EntryName);
         return;
    }

    //Release memory space occupied by this bitmap.
    if (GreDeleteBitmap(hBM)==FALSE){
         CLEANUP("GreDeleteBitmap()",EntryName);
         return;
    }

   /* *****************************************************************
    * The following Code will check for the PMERR_INV_SETID error Code*
    *******************************************************************/
    SUBENTER(PMERR_INV_SETID,"TEST1");
    // Pass an invalid ID ,say ID=NULL ,to the function expecting
    // an error
    hBMSuccess=GreQueryBitmapHandle(hDC,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_SETID,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_SETID,"TEST2");
    // Pass an invalid ID ,say ID=0x0fffL,to the function expecting
    // an error
    hBMSuccess=GreQueryBitmapHandle(hDC,0xfffL);
    CHECKRC(hBMSuccess,PMERR_INV_SETID,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_SETID,"TEST3");
    // Pass an invalid ID ,say ID=-1L,to the function expecting
    // an error
    hBMSuccess=GreQueryBitmapHandle(hDC,-1L);
    CHECKRC(hBMSuccess,PMERR_INV_SETID,"Test3");
    SUBEXIT();

    EXIT_MAIN(EntryName);

  }
  return;
}


 /****************************************************************************
 *\\ddd
 * Routine Name: GreSelectBitmapRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreSelectBitmap().
 *     The syntax used to call the Gre is :
 *
 *     (HBITMAP) GreSelectBitmap(HDc  hDC,
 *                            HBITMAP hbm
 *                           )
 *     This Gre call sets a bit map as the currently selected bit map in a
 *     memory device context.A null handle causes the currently selected
 *     bit map, in the associated device, to be freed.
 *
 *     The return value is:
 *        NULL              Correct (null handle)
 *        HBM_ERROR         Error
 *        Otherwise         Old bit-map handle.
 *
 *     Possible returns from WinGetLastError:
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *        PMERR_INV_HBITMAP
 *        PMERR_BITMAP_IS_SELECTED
 *        PMERR_HBITMAP_BUSY
 *        PMERR_INV_IN_AREA
 *        PMERR_INV_IN_PATH
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INSUFFICIENT_MEMORY
 *        PMERR_INV_LENGHT_OR_COUNT
 *        PMERR_INV_RECT
 *        PMERR_INV_COORRDINATE
 *        PMERR_INV_HRGN
 *        PMERR_REGION_CONTOLR
 *        PMERR_INV_INFO_TABLE
 *        PMERR_BITMAP_DIMENSION
 *        PMERR_INV_SCAN_START
 *
 * Notes:
 *   The specified presentation space must be currently associated with
 *  a memory Device context. The Device context can represent a different
 *  physical device from the one that the bit map originally loaded or
 *  created, providing its format is convertible to one supported on the
 *  new device. This is ensured when one of the standard formats is being
 *  used.
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/17/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/17/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HDC
 *        PMERR_HDC_BUSY
 *        PMERR_INV_HBITMAP
 *        PMERR_BITMAP_IS_SELECTED
 *        PMERR_INV_IN_AREA
 *        PMERR_INV_IN_PATH
 *
 * Return Codes Not Tested Successfully Yet:
 *        PMERR_HBITMAP_BUSY
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INSUFFICIENT_MEMORY
 *        PMERR_INV_LENGHT_OR_COUNT
 *        PMERR_INV_RECT
 *        PMERR_INV_COORRDINATE
 *        PMERR_INV_HRGN
 *        PMERR_REGION_CONTOL
 *        PMERR_INV_INFO_TABLE
 *        PMERR_BITMAP_DIMENSION
 *        PMERR_INV_SCAN_START
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSelectBitmap(hDC...);
 *     if ( ulSuccess== GPI_ERROR) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSelectBitmapRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static  PCHAR  EntryName="GreSelectBitmapRc()";
  HBITMAP      hBMSuccess;          //Holds return code.
  HBITMAP      hBM;                 //Bitmap handle.
  HDC    hDC;
  HDC    hDC1;
  HDC    hDC2;
  HAB    hAB;
  HPS    hPS1;
  HPS    hPS2;
  SIZEL sizel={0};         //Holds PS dimensions.

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);
    SETENTER(EntryName,NULL);
    //Anchor Block is needed when creation presentation space.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
        CLEANUP("WinQueryAnchorBlock()",EntryName);
        return ;
    }

    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    /***********************************************************
    * This code segment is intended to create two Presentation *
    * Spaces and associate them with two Device Contexts. These*
    * two Presentation Spaces will be used to test the error   *
    * code PMERR_BITMAP_IS_SELECTED. The idea to set a bitmap  *
    * into one Presentation Space and then try to set the same *
    * bitmap into the second Presenatation Space. This will    *
    * produce the desired error code.                          *
    ***********************************************************/
    //Get a system bitmap to use in testing return codes.
    if ((hBM=WinGetSysBitmap(HWND_DESKTOP,SBMP_SYSMENU))==NULL){
        CLEANUP("WinGetSysBitmap()",EntryName);
        return;
    }

    //Create the first Device Context.
    if ((hDC1=DevOpenDC(hAB,OD_MEMORY,"*",0L,(PDEVOPENDATA)NULL,
                        hDC))==DEV_ERROR){
       CLEANUP("DevOpenDC()",EntryName);
       return ;
    }
    // Create the first presentation space to associate with the
    // memory device context

    if ((hPS1=GpiCreatePS(hAB,hDC1,&sizel,PU_PELS |
                          GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR){
       CLEANUP("GpiCreatePS()",EntryName);
       return ;
    }

    //Create the second Device Context.
    if ((hDC2=DevOpenDC(hAB,OD_MEMORY,"*",0L,(PDEVOPENDATA)NULL,
                        hDC))==DEV_ERROR){
       CLEANUP("DevOpenDC()",EntryName);
       return ;
    }

    // Create the second presentation space to associate with the
    // memory device context
    if ((hPS2=GpiCreatePS(hAB,hDC2,&sizel,PU_PELS |
                          GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR){
       CLEANUP("GpiCreatePS()",EntryName);
       return ;
    }

    SETEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    hBMSuccess=GreSelectBitmap((HDC)NULL,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    hBMSuccess=GreSelectBitmap((HDC)0x0ffffffL,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    hBMSuccess=GreSelectBitmap((HDC)-1L,NULL);
    CHECKRC(hBMSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_HBITMAP error  *
    * code. Note that NULL bitmap is valid.                          *
    *****************************************************************/
    SUBENTER(PMERR_INV_HBITMAP,"Test1");
    // Pass an invalid HBITMAP ,say hBM=1 ,to the function expecting
    // an error.
    hBMSuccess=GreSelectBitmap(hDC1,(HBITMAP)1);
    CHECKRC(hBMSuccess,PMERR_INV_HBITMAP,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HBITMAP,"Test2");
    // Pass an invalid HBITMAP ,say hBM=0x0ffffff,to the function expecting
    // an error
    hBMSuccess=GreSelectBitmap(hDC1,(HBITMAP)0x0fffffL);
    CHECKRC(hBMSuccess,PMERR_INV_HBITMAP,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HBITMAP,"Test3");
    // Pass an invalid HBITMAP ,say hBM=-1L,to the function expecting
    // an error
    hBMSuccess=GreSelectBitmap(hDC1,(HBITMAP)0xfffffL);
    CHECKRC(hBMSuccess,PMERR_INV_HBITMAP,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/
    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS1);            //Make this PS busy

    hBMSuccess=GreSelectBitmap(hDC1,hBM);
    //NULL is valid return code.
    if (hBMSuccess==NULL)
       CHECKRC(hBMSuccess+1,PMERR_HDC_BUSY,NULL);
    else
       CHECKRC(hBMSuccess,PMERR_HDC_BUSY,NULL);

    (VOID)FREEPS;                   //Kill BUSY thread.
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_BITMAP_IS_SELECTED *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_BITMAP_IS_SELECTED,NULL);

    //Select bitmap into the first Presentation Space and then try
    //to select the same bitmap into the second Presentation Space.
    if (GreSelectBitmap(hDC1,hBM)==HBM_ERROR){
       CLEANUP("GreSelectBitmap()",EntryName);
       return;
    }
    hBMSuccess=GreSelectBitmap(hDC2,hBM);
    //NULL is valid return code.
    if (hBMSuccess==NULL)
       CHECKRC(hBMSuccess+1,PMERR_BITMAP_IS_SELECTED,NULL);
    else
       CHECKRC(hBMSuccess,PMERR_BITMAP_IS_SELECTED,NULL);

    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_AREA        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_AREA,NULL);

    //Select bitmap into the first Presentation Space and then try
    //to select the bitmap into an open area brackets.

    SETENTER(EntryName," Opening AREA bracket");

    if (GpiBeginArea(hPS1,BA_WINDING)==FALSE){
       SHOWERROR("GpiBeginArea()",EntryName);
    }

    SETEXIT();

    hBMSuccess=GreSelectBitmap(hDC1,hBM);
    //NULL is valid return code.
    if (hBMSuccess==NULL)
       CHECKRC(hBMSuccess+1,PMERR_INV_IN_AREA,NULL);
    else
       CHECKRC(hBMSuccess,PMERR_INV_IN_AREA,NULL);

    SUBEXIT();

    //Close The Area

    SETENTER(EntryName," Closing AREA bracket");

    if (GpiEndArea(hPS1)==GPI_ERROR){
       SHOWERROR("GpiEndArea()",EntryName);
    }

    SETEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_PATH        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_PATH,NULL);

    //Select bitmap into the first Presentation Space and then try
    //to select the bitmap into an open Path brackets.

    SETENTER(EntryName," Opening Path bracket");

    if (GpiBeginPath(hPS1,1L)==FALSE){
       SHOWERROR("GpiBeginPath()",EntryName);
    }

    SETEXIT();

    hBMSuccess=GreSelectBitmap(hDC1,hBM);
    //NULL is valid return code.
    if (hBMSuccess==NULL)
       CHECKRC(hBMSuccess+1,PMERR_INV_IN_PATH,NULL);
    else
       CHECKRC(hBMSuccess,PMERR_INV_IN_PATH,NULL);

    SUBEXIT();

    //Close The Area

    SETENTER(EntryName," Closing Path bracket");

    if (GpiEndPath(hPS1)==GPI_ERROR){
       SHOWERROR("GpiEndPath()",EntryName);
    }

    SETEXIT();

    /*=================================================================
      The purpose of next code segment is do some house cleaning
      before exiting this routine. the following is done:
      1. De-select any bitmaps that are selected into memory Device
         Contexts or/and Un-tag any bitmaps that are used as pattern.
      2. Delete any bitmaps created within this function.
      3. Delete any Presentation Spaces created within this function.
      4. Delate any Device Contexts created within this function.

         Note that these steps must be followed as presented above,
         otherwise; most GPIs used here will fail.
    ===================================================================*/

    SETENTER(EntryName, "Cleaning up...");
    // De-select the bitmap handle. otherwise we get error when closing
    // device context
    if (GreSelectBitmap(hDC1,NULL)==HBM_ERROR){
         CLEANUP("GreSelectBitmap()",EntryName);
         return;
     }

    // Delete the bitmap handle.
    // device context
    if (GreDeleteBitmap(hBM)==FALSE){
         CLEANUP("GreDeleteBitmap()",EntryName);
         return;
     }

    // dispose of presentation space. No longer needed.
    if (GpiDestroyPS(hPS1)==FALSE){
         CLEANUP("GpiDestroyPS()",EntryName);
         return ;
    }

    //Close the device context
    if (DevCloseDC(hDC1)==DEV_ERROR){
      CLEANUP("DevCloseDC()",EntryName);
      return ;
    }
    // dispose of presentation space. No longer needed.
    if (GpiDestroyPS(hPS2)==FALSE){
         CLEANUP("GpiDestroyPS()",EntryName);
         return ;
    }

    //Close the  device context
    if (DevCloseDC(hDC2)==DEV_ERROR){
      CLEANUP("DevCloseDC()",EntryName);
      return ;
    }
    SETEXIT();
    EXIT_MAIN(EntryName);
  }
  return;
}
 /****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetCursorRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreDeviceSetCursor().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL) GreDeviceSetCursor(HDC  hDC,
 *                               PPOINTL  pHotspot,
 *                               HBITMAP hbm
 *                              )
 *    This Gre sets the cursor bit map that defines the cursor shape.
 *
 *     The return value is:
 *        TRUE              Correct
 *        FALSE              Error
 *
 *     Possible returns from WinGetLastError:
 *        PMERR_INV_HDC
 *        PMERR_INV_CURSOR_HBITMAP
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INV_COORRDINATE
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/17/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/17/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HDC
 *        PMERR_INV_CURSOR_HBITMAP
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *        PMERR_INV_COORRDINATE
 *
 * Return Codes Not Tested Successfully Yet:
 *      None
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreDeviceSetCursor(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreDeviceSetCursorRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){
  static  PCHAR  EntryName="GreDeviceSetCursorRc()";
  BOOL           fSuccess;          //Holds return code.
  HBITMAP        hBM;                 //Bitmap handle.
  HDC            hDC;
  POINTL         HotSpot={0,0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, (USHORT) WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SetEnter(hWndClient,EntryName,NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    //Get a system bitmap to use in testing return codes.
    if ((hBM=WinGetSysBitmap(HWND_DESKTOP,SBMP_SYSMENU))==NULL){
        CLEANUP("WinGetSysBitmap()",EntryName);
        return;
    }

    SETEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreDeviceSetCursor((HDC)NULL,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreDeviceSetCursor((HDC)0x0ffffff,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreDeviceSetCursor((HDC)-1L,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_CURSOR_BITMAP *
    * error code. Note that NULL bitmap is valid.                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_CURSOR_BITMAP,"Test1");
    // Pass an invalid HBITMAP ,say hBM=1 ,to the function expecting
    // an error.
    fSuccess=GreDeviceSetCursor(hDC,&HotSpot,(HBITMAP)1);
    CHECKRC(fSuccess,PMERR_INV_CURSOR_BITMAP,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_CURSOR_BITMAP,"Test2");
    // Pass an invalid HBITMAP ,say hBM=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreDeviceSetCursor(hDC,&HotSpot,(HBITMAP)0x0fffffL);
    CHECKRC(fSuccess,PMERR_INV_CURSOR_BITMAP,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_CURSOR_BITMAP,"Test3");
    // Pass an invalid HBITMAP ,say hBM=-1L,to the function expecting
    // an error
    fSuccess=GreDeviceSetCursor(hDC,&HotSpot,(HBITMAP)0xfffffL);
    CHECKRC(fSuccess,PMERR_INV_CURSOR_BITMAP,"Test3");
    SUBEXIT();

    /********************************************************************
    * The following Code will check for the PMERR_DEV_FUNC_NOT_INSTALLED *
    * error  code.                                                       *
    *********************************************************************/
//    SUBENTER(PMERR_DEV_FUNC_NOT_INSTALLED,NULL);

//    fSuccess=GreDeviceSetCursor(hDC,&HotSpot,hBM);
//    CHECKRC(fSuccess,PMERR_DEV_FUNC_NOT_INSTALLED,NULL);

//    SUBEXIT();

    /*************************************************************
    * The following Code will check for the PMERR_INV_COORDINATE *
    * error code.                                                *
    **************************************************************/
    SUBENTER(PMERR_INV_COORDINATE,"Test1");
    HotSpot.x=0x0fffffff;

    fSuccess=GreDeviceSetCursor(hDC,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_COORDINATE,"Test1");

    SUBEXIT();

    SUBENTER(PMERR_INV_COORDINATE,"Test2");
    HotSpot.x=0;
    HotSpot.y=0x0fffffff;

    fSuccess=GreDeviceSetCursor(hDC,&HotSpot,hBM);
    CHECKRC(fSuccess,PMERR_INV_COORDINATE,"Test2");

    SUBEXIT();


    /*=================================================================
      The purpose of next code segment is do some house cleaning
      before exiting this routine. the following is done:
      1. De-select any bitmaps that are selected into memory Device
         Contexts or/and Un-tag any bitmaps that are used as pattern.
      2. Delete any bitmaps created within this function.
      3. Delete any Presentation Spaces created within this function.
      4. Delate any Device Contexts created within this function.

         Note that these steps must be followed as presented above,
         otherwise; most GPIs used here will fail.
    ===================================================================*/

    SETENTER(EntryName, "Cleaning up...");

    // Delete the bitmap handle.
    // device context
    if (GreDeleteBitmap(hBM)==FALSE){
         CLEANUP("GreDeleteBitmap()",EntryName);
         return;
     }


    SETEXIT();
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSaveScreenBitsRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreSaveScreenBits().
 *     The syntax used to call the Gre is :
 *
 *     (ULONG) GreSaveScreenBits(HDC  hDC,
 *                               PRECTL  pRectl
 *                              )
 *                                                       Name of File to Edit
 *    This Gre saves a rectangle of screen bits
 *
 *     The return value is:
 *        TRUE               Successfull.
 *        FALSE              Error or rectangle is not saved
 *
 *     Possible returns from WinGetLastError:
 *        PMERR_INV_HDC
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/17/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/17/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *        PMERR_INV_HDC
 *        PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * Return Codes Not Tested Successfully Yet:
 *      None
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSaveScreenBits(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreSaveScreenBitsRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreSaveScreenBitsRc()";
  ULONG          fSuccess;          //Holds return code.
  HDC            hDC;
  RECTL          Rectl={0,0,10,10};
  LONG           LastErrorNumber;
  HAB            hAB;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER(EntryName,NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }
    //Anchor Block is needed when creation presentation space.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
        CLEANUP("WinQueryAnchorBlock()",EntryName);
        return ;
    }

    SETEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreSaveScreenBits((HDC)NULL,&Rectl);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreSaveScreenBits((HDC)0x0ffffffL,&Rectl);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreSaveScreenBits((HDC)-1L,&Rectl);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /********************************************************************
    * The following Code will check for the PMERR_DEV_FUNC_NOT_INSTALLED *
    * error  code.                                                       *
    *********************************************************************/
    SUBENTER(PMERR_DEV_FUNC_NOT_INSTALLED,NULL);

    //Cleanup the error stack.
    (VOID) WinGetLastError(hAB );

    fSuccess=GreSaveScreenBits(hDC,&Rectl);

    if ((LastErrorNumber=WinGetLastError(hAB))==0x00){
      LOGINFO(L_HDR,"*","GreSaveScreenBits() returned zero\n"
                        " Which means that the function does not\n"
                        " save the screen and the calling function\n"
                        " must save the screen."
                        );
    }
    else
      CHECKRC(fSuccess,PMERR_DEV_FUNC_NOT_INSTALLED,NULL);

    SUBEXIT();

    EXIT_MAIN(EntryName);
  }
  return;


}
/****************************************************************************
 *\\ddd
 * Routine Name: GreRestoreScreenBitsRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreRestoreScreenBits().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL) GreRestoreScreenBits(HDC  hDC,
 *                                  ULONG hsbits,
 *                                  PRECTL  pRectl,
 *                                  ULONG flOptions
 *                                 )
 *
 *    This Gre restores a rectangle of screen bits. This function can
 *    also free the handle of the saved bits.
 *
 *     The return value is:
 *        TRUE               Successfull.
 *        FALSE              Error or rectangle is not saved
 *
 *     Possible returns from WinGetLastError:
 *     PMERR_BASE_ERROR
 *     PMERR_INV_IN_AREA
 *     PMERR_INV_IN_PATH
 *     PMERR_HDC_BUSY
 *     PMERR_DEV_FUNC_NOT_INSTALLED
 *     PMERR_COORDINATE_OVERFLOW
 *     PMERR_BITMAP_NOT_SELECTED
 *     PMERR_INV_PRIMITIVE_TYPE
 *     PMERR_INV_SETID
 *     PMERR_INV_SET_CHAR_ATTR
 *     PMERR_INV_MARKER_CHAR_ATTR
 *     PMERR_INV_PATTERN_CHAR_ATTR
 *     PMERR_INV_MARKER_SYMBOL_ATTR
 *     PMERR_UNSUPPORTED_ATTR
 *     PMERR_INV_MIX_ATTR
 *     PMERR_INV_BACKGROUND_MIX_ATTR
 *     PMERR_INV_PATTREN_REF_PT_ATTR
 *     PMERR_INV_LINE_WIDTH_ATTR
 *     PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *     PMERR_INV_LINE_TYPE_ATTR
 *     PMERR_INV_LINE_JOIN_ATTR
 *     PMERR_INV_LINE_END_ATTR
 *     PMERR_INV_CHAR_MODE_ATTR
 *     PMERR_INV_CHAR_SHEAR_ATTR
 *     PMERR_INV_CHAR_DIRECTION_ATTR
 *     PMERR_UNSUPPORTED_ATTR_VALUE
 *     PMERR_INSUFFICIENT_MEMORY
 *     PMERR_INV_LENGTH_OR_COUNT
 *     PMERR_INV_CODEPAGE
 *     PMERR_EXCEEDS_MAX_SEG_LENGTH
 *     PMERR_INV_RECT
 *     PMERR_INV_COORDINATE
 *     PMERR_INV_HDC
 *     PMERR_INV_HRGN
 *     PMERR_INV_DC_DATA
 *     PMERR_INCOMPATIBLE_BITMAP
 *     PMERR_INV_REGION_CONTROL
 *     PMERR_INV_DRIVER_NAME
 *     PMERR_INV_BITBLT_STYLE
 *     PMERR_INV_BITBLT_MIX
 *     PMERR_INV_HBITMAP
 *     PMERR_INV_DC_TYPE
 *     PMERR_INV_COORD_SPACE
 *     PMERR_INV_ID
 *     PMERR_BITMAP_IS_SELECTED
 *     PMERR_HBITMAP_BUSY
 *     PMERR_INV_USAGE_PARM
 *     PMERR_INV_INFO_TABLE
 *     PMERR_INV_BITMAP_DIMENSION
 *     PMERR_INCORRECT_DC_TYPE
 *     PMERR_REALIZE_NOT_SUPPORTED
 *     PMERR_INV_COLOR_FORMAT
 *     PMERR_INV_COLOR_DATA
 *     PMERR_INV_COLOR_START_INDEX
 *     PMERR_INV_COLOR_OPTIONS
 *     PMERR_INV_COLOR_INDEX
 *     PMERR_INV_DRAW_BORDER_OPTION
 *     PMERR_INV_SCAN_START
 *     PMERR_INV_PICK_APERTURE_POSN
 *     PMERR_INV_PATTERN_SET_FONT
 *     PMERR_INV_COLOR_ATTR
 *     PMERR_INV_BACKGROUND_COL_ATTR
 *
 *
 * System Requirements:
 *
 * Revision Log:
 *               09/17/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/17/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *     PMERR_INV_HDC
 *     PMERR_HDC_BUSY
 *     PMERR_DEV_FUNC_NOT_INSTALLED
 *     PMERR_INV_IN_AREA
 *     PMERR_INV_IN_PATH
 *
 *
 * Return Codes Not Tested Successfully Yet:
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreRestoreScreenBits(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreRestoreScreenBitsRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreRestoreScreenBitsRc()";
  BOOL           fSuccess;          //Holds return code.
  HDC            hDC;
  RECTL          Rectl={100,200,150,300};
  ULONG          SavedBitsHandle;
  HAB            hAB;
  LONG           LastErrorNumber;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER(EntryName,NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    //Anchor Block is needed when creation presentation space.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
        CLEANUP("WinQueryAnchorBlock()",EntryName);
        return ;
    }

    SETEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/
    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);            //Make this PS busy

    fSuccess=GreRestoreScreenBits(hDC,NULL,&Rectl,NULL);
       CHECKRC(fSuccess,PMERR_HDC_BUSY,NULL);

    (VOID)FREEPS;                   //Kill BUSY thread.
    SUBEXIT();


    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreRestoreScreenBits((HDC)NULL,NULL,&Rectl,NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreRestoreScreenBits((HDC)0x0ffffffL,NULL,&Rectl,NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreRestoreScreenBits((HDC)-1L,NULL,&Rectl,NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /********************************************************************
    * The following Code will check for the PMERR_DEV_FUNC_NOT_INSTALLED *
    * error  code.                                                       *
    *********************************************************************/

    SUBENTER(PMERR_DEV_FUNC_NOT_INSTALLED,NULL);
     SETENTER(EntryName,"Saving Screen bits");

    //Cleanup the error stack.
    (VOID) WinGetLastError(hAB );

    SavedBitsHandle=GreSaveScreenBits(hDC,&Rectl);

    if ((LastErrorNumber=WinGetLastError(hAB))==0x00){
      LOGINFO(L_HDR,"*","GreSaveScreenBits() returned zero\n"
                        " Which means that the function does not\n"
                        " save the screen and the calling function\n"
                        " must save the screen."
                        );

    }

    SETEXIT();

    fSuccess=GreRestoreScreenBits(hDC,SavedBitsHandle,&Rectl,1);

    CHECKRC(fSuccess,PMERR_DEV_FUNC_NOT_INSTALLED,NULL);

    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_AREA        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_AREA,NULL);

    SETENTER(EntryName," Opening AREA bracket");

    if (GpiBeginArea(hPS,BA_WINDING)==FALSE){
       SHOWERROR("GpiBeginArea()",EntryName);
    }

    SETEXIT();
    fSuccess=GreRestoreScreenBits(hDC,SavedBitsHandle,&Rectl,1);

    CHECKRC(fSuccess,PMERR_INV_IN_AREA,NULL);

    //Close The Area

    SETENTER(EntryName," Closing AREA bracket");

    if (GpiEndArea(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndArea()",EntryName);
    }

    SETEXIT();

    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_PATH        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_PATH,NULL);

    SETENTER(EntryName," Opening PATH bracket");

    if (GpiBeginPath(hPS,1L)==FALSE){
       SHOWERROR("GpiBeginPath()",EntryName);
    }

    SETEXIT();
    fSuccess=GreRestoreScreenBits(hDC,SavedBitsHandle,&Rectl,1);

    CHECKRC(fSuccess,PMERR_INV_IN_PATH,NULL);

    //Close The Area
    SETENTER(EntryName," Closing PATH bracket");

    if (GpiEndPath(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndPath()",EntryName);
    }

    SETEXIT();

    SUBEXIT();

    EXIT_MAIN(EntryName);
  }
  return;


}
/****************************************************************************
 *\\ddd
 * Routine Name: GreCopyDCLoadDataRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreCopyDCLoadData().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL) GreCopyDCLoadData(HDC  hDC,
 *                                  ULONG flOptions,
 *                                  HDC    hDCSource
 *                                 )
 *
 *     This Gre copies the loaded fonts, bitmpas,color tables, and the
 *     default attributes from on DC to another. Device-dependant attributes
 *     such as pick aperture,character cells,and marker cells are only
 *     copied when they have been set by the application, otherwise the
 *     target device defaults are used.
 *
 *     The return value is:
 *        TRUE               Successfull.
 *        FALSE              Error
 *
 *     Possible returns from WinGetLastError:
 *         None.
 *
 * Revision Log:
 *               09/18/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               09/18/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *     PMERR_INV_HDC
 *     PMERR_HDC_BUSY
 *     PMERR_INV_IN_AREA
 *     PMERR_INV_IN_PATH
 *
 * Return Codes Not Tested Successfully Yet:
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   None at this time.
 *
 *
 * NOTE: This function has no documented return errors. If feel that this
 *       function should at least return the above listed error codes.
 *
 *  The function does not check if the source hDC in invalid.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreCopyDCLoadData(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreCopyDCLoadDataRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){
  static  PCHAR  EntryName="GreCopyDCLoadDataRc()";
  BOOL         fSuccess;          //Holds return code.
  HBITMAP      hBM;                 //Bitmap handle.
  HDC    hDC;
  HDC    hDC1;
  HAB    hAB;
  HPS    hPS1;
  SIZEL sizel={0};         //Holds PS dimensions.
  BITMAPINFOHEADER BitMapHeader;      //bitmap info header


  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);
    SETENTER(EntryName,NULL);

    BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
    BitMapHeader.cPlanes=                      // one color plane
    BitMapHeader.cBitCount=1;                  // 1-bit /pixel
    BitMapHeader.cx=PAT1_WIDTH;
    BitMapHeader.cy=PAT1_HEIGHT;


    //Anchor Block is needed when creation presentation space.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
        CLEANUP("WinQueryAnchorBlock()",EntryName);
        return ;
    }

    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    if ((hBM=GreCreateBitmap(hDC,&BitMapHeader,(ULONG)NULL,NULL,NULL))==FALSE){
        CLEANUP("GpiCreatBitMap()",EntryName);
        return;
    }


    //Create the first Device Context.
    if ((hDC1=DevOpenDC(hAB,OD_MEMORY,"*",0L,(PDEVOPENDATA)NULL,
                        hDC))==DEV_ERROR){
       CLEANUP("DevOpenDC()",EntryName);
       return ;
    }
    // Create the first presentation space to associate with the
    // memory device context

    if ((hPS1=GpiCreatePS(hAB,hDC1,&sizel,PU_PELS |
                          GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR){
       CLEANUP("GpiCreatePS()",EntryName);
       return ;
    }

    SETEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreCopyDCLoadData((HDC)NULL,LCID_RANGE_GPI,hDC);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffffL,to the function expecting
    // an error
    fSuccess=GreCopyDCLoadData((HDC)0x0fffffL,LCID_RANGE_GPI,hDC);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreCopyDCLoadData((HDC)0xffffffffL,LCID_RANGE_GPI,hDC);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();


    SUBENTER(PMERR_INV_HDC,"Test4");
    // Pass an invalid HDC ,say HDC=NULL,to the function expecting
    // an error
    fSuccess=GreCopyDCLoadData(hDC1,LCID_RANGE_GPI,(HDC)NULL);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test4");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/
    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);            //Make this PS busy

    fSuccess=GreCopyDCLoadData(hDC1,LCID_RANGE_GPI,hDC);
    CHECKRC(fSuccess,PMERR_HDC_BUSY,NULL);

    (VOID)FREEPS;                   //Kill BUSY thread.
    SUBEXIT();

   /******************************************************************
    * The following Code will check for the PMERR_SETID_IN_USE error *
    * code                                                           *
    ******************************************************************/

    SUBENTER(PMERR_SETID_IN_USE,NULL);

    SETENTER(EntryName," Setting UP Pattern IDs");

    //Tag the bitmap with a pattern ID.
    if (GreSetBitmapID(hDC,hBM,MY_BITMAP_ID)==FALSE){
         CLEANUP("GreSetBitmapID()",EntryName);
         return;
    }

    //Tag the bitmap with a pattern ID.
    if (GreSetBitmapID(hDC1,hBM,MY_BITMAP_ID)==FALSE){
         CLEANUP("GreSetBitmapID()",EntryName);
         return;
    }


     //Use this bitmap as the default pattern.
      if (GpiSetPatternSet(hPS,MY_BITMAP_ID)==GPI_ERROR){
         CLEANUP("GpiSetPatternSet",EntryName);
         return;
      }
      if (GpiSetPatternSet(hPS1,MY_BITMAP_ID)==GPI_ERROR){
         CLEANUP("GpiSetPatternSet",EntryName);
         return;
      }

     SETEXIT();

    fSuccess=GreCopyDCLoadData(hDC1,LCID_RANGE_GPI,hDC);

    CHECKRC(fSuccess,PMERR_SETID_IN_USE,NULL);
    SUBEXIT();


    /*=================================================================
      The purpose of next code segment is do some house cleaning
      before exiting this routine. the following is done:
      1. De-select any bitmaps that are selected into memory Device
         Contexts or/and Un-tag any bitmaps that are used as pattern.
      2. Delete any bitmaps created within this function.
      3. Delete any Presentation Spaces created within this function.
      4. Delate any Device Contexts created within this function.

         Note that these steps must be followed as presented above,
         otherwise; most GPIs used here will fail.
    ===================================================================*/

    SETENTER(EntryName, "Cleaning up...");


    //Set the default pattern to system DEFAULT. You should this before
    //deleting the current bitmap pattern.
    if (GpiSetPatternSet(hPS,LCID_DEFAULT)==GPI_ERROR){
        CLEANUP("GpiSetPatternSet()",EntryName);
        return;
    }

    //Set the default pattern to system DEFAULT. You should this before
    //deleting the current bitmap pattern.
    if (GpiSetPatternSet(hPS1,LCID_DEFAULT)==GPI_ERROR){
        CLEANUP("GpiSetPatternSet()",EntryName);
        return;
    }


    //Un-tag this Pattern ID.
    if (GreDeleteSetId(hDC,MY_BITMAP_ID)==FALSE){
         CLEANUP("GreDeleteSetId()",EntryName);
         return;
    }

    //Un-tag this Pattern ID.
    if (GreDeleteSetId(hDC1,MY_BITMAP_ID)==FALSE){
         CLEANUP("GreDeleteSetId()",EntryName);
         return;
    }

    //Release memory space occupied by this bitmap.
    if (GreDeleteBitmap(hBM)==FALSE){
         CLEANUP("GreDeleteBitmap()",EntryName);
         return;
    }


    // dispose of presentation space. No longer needed.
    if (GpiDestroyPS(hPS1)==FALSE){
         CLEANUP("GpiDestroyPS()",EntryName);
         return ;
    }

    //Close the device context
    if (DevCloseDC(hDC1)==DEV_ERROR){
      CLEANUP("DevCloseDC()",EntryName);
      return ;
    }

    SETEXIT();

    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetDefaultAttributesRc()
 *
 * Purpose:  This function exhaustively tests for most of the errors codes
 *           that may be returned by GreSetDefaultAttributes().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL) GreSetDefaultAttributes(HDC       hDC,
 *                                    ULONG    lPrimTypeType,
 *                                    ULONG    flAttribute,
 *                                    PBUNDLE   pAttrs
 *                                  )
 *    This Gre sets the specified default attributes in line,pattern,character,
 *    and maker bundles.
 *
 *     The return value is:
 *        TRUE              Success
 *        FALSE              Error
 *
 *     Possible returns from WinGetLastError:
 *
 *     PMERR_HDC_BUSY
 *     PMERR_DEV_FUNC_NOT_INSTALLED
 *     PMERR_COORDINATE_OVERFLOW
 *     PMERR_INV_PRIMITIVE_TYPE
 *     PMERR_INV_SETID
 *     PMERR_INV_SET_CHAR_ATTR
 *     PMERR_INV_MARKER_CHAR_ATTR
 *     PMERR_INV_PATTERN_CHAR_ATTR
 *     PMERR_INV_MARKER_SYMBOL_ATTR
 *     PMERR_UNSUPPORTED_ATTR
 *     PMERR_INV_MIX_ATTR
 *     PMERR_INV_BACKGROUND_MIX_ATTR
 *     PMERR_INV_PATTREN_REF_PT_ATTR
 *     PMERR_INV_LINE_WIDTH_ATTR
 *     PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *     PMERR_INV_LINE_TYPE_ATTR
 *     PMERR_INV_LINE_JOIN_ATTR
 *     PMERR_INV_LINE_END_ATTR
 *     PMERR_INV_CHAR_MODE_ATTR
 *     PMERR_INV_CHAR_SHEAR_ATTR
 *     PMERR_INV_CHAR_DIRECTION_ATTR
 *     PMERR_UNSUPPORTED_ATTR_VALUE
 *     PMERR_INSUFFICIENT_MEMORY
 *     PMERR_INV_LENGTH_OR_COUNT
 *     PMERR_INV_CODEPAGE
 *     PMERR_EXCEEDS_MAX_SEG_LENGTH
 *     PMERR_INV_HDC
 *     PMERR_INV_COORD_SPACE
 *     PMERR_INV_PATTERN_SET_FONT
 *     PMERR_INV_COLOR_ATTR
 *     PMERR_INV_BACKGROUND_COL_ATTR
 *
 *
 * System Requirements:
 *
 * Revision Log:
 *               10/05/91, PDVT ,Mike Abuzant
 *               -initial creation
 *               10/05/91, PDVT ,Mike Abuzant
 *               -TestCase Design and coding.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *
 *     PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *     PMERR_INV_LINE_WIDTH_ATTR
 *     PMERR_INV_IN_AREA
 *     PMERR_INV_IN_PATH
 *     PMERR_HDC_BUSY
 *     PMERR_INV_PRIMITIVE_TYPE
 *     PMERR_INV_SETID
 *     PMERR_UNSUPPORTED_ATTR
 *     PMERR_INV_MIX_ATTR
 *     PMERR_INV_BACKGROUND_MIX_ATTR
 *     PMERR_INV_LINE_TYPE_ATTR
 *     PMERR_INV_LINE_JOIN_ATTR
 *     PMERR_INV_LINE_END_ATTR
 *     PMERR_INV_CHAR_MODE_ATTR
 *     PMERR_INV_CHAR_DIRECTION_ATTR
 *     PMERR_INV_HDC
 *     PMERR_INV_COLOR_ATTR
 *     PMERR_INV_BACKGROUND_COL_ATTR
 *
 * Return Codes Not Tested Successfully Yet:
 *
 *     PMERR_DEV_FUNC_NOT_INSTALLED
 *     PMERR_COORDINATE_OVERFLOW
 *     PMERR_INV_SET_CHAR_ATTR
 *     PMERR_INV_MARKER_CHAR_ATTR
 *     PMERR_INV_PATTERN_CHAR_ATTR
 *     PMERR_INV_MARKER_SYMBOL_ATTR
 *     PMERR_UNSUPPORTED_ATTR
 *     PMERR_INV_MIX_ATTR
 *     PMERR_INV_BACKGROUND_MIX_ATTR
 *     PMERR_INV_PATTREN_REF_PT_ATTR
 *     PMERR_INV_CHAR_SHEAR_ATTR
 *     PMERR_UNSUPPORTED_ATTR_VALUE
 *     PMERR_INSUFFICIENT_MEMORY
 *     PMERR_INV_LENGTH_OR_COUNT
 *     PMERR_INV_CODEPAGE
 *     PMERR_EXCEEDS_MAX_SEG_LENGTH
 *     PMERR_INV_COORD_SPACE
 *     PMERR_INV_PATTERN_SET_FONT
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *   Almost half of the error codes listed above will not be returned by this
 *   function. The documenatation states that the GreSetDefaultAttribute()
 *   does not check validity of most of the given attributes instead,
 *   attributes checking is done when calling functions that make use of
 *   the attributes.
 *
 *  NOTE:
 *   Two return codes was tested even though it is not listed
 *   under this function. It seemed logical for this function to return
 *   the following return codes:
 *     PMERR_INV_IN_PATH
 *     PMERR_INV_IN_AREA
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSetDefaultAttributes(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetDefaultAttributesRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreSetDefaultAttributesRc()";
  BOOL           fSuccess;          //Holds return code.
  HDC            hDC;
  DIMAGEBUNDLE   ImageBundle={sizeof(ImageBundle),0};
  DLINEBUNDLE    LineBundle={sizeof(LineBundle),0};
  DCHARBUNDLE    CharBundle={sizeof(CharBundle),0};
  DMARKERBUNDLE  MarkerBundle={sizeof(MarkerBundle),0};
  DAREABUNDLE    AreaBundle={sizeof(AreaBundle),0};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    SETENTER("Query hDC associated with PS",NULL);
    //Get Device Context associated with PS passed from DTT.EXE
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
        CLEANUP("GpiQueryDevice()",EntryName);
        return ;
    }

    SETEXIT();

    LineBundle.lbnd.lColor=CLR_RED;
    /****************************************************************
    * The following Code will check for the PMERR_INV_HDC error Code*
    *****************************************************************/
    SUBENTER(PMERR_INV_HDC,"Test1");
    // Pass an invalid HDC ,say HDC=NULL ,to the function expecting
    // an error
    fSuccess=GreSetDefaultAttributes((HDC)NULL,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test2");
    // Pass an invalid HDC ,say HDC=0x0ffffff,to the function expecting
    // an error
    fSuccess=GreSetDefaultAttributes((HDC)0x0fffffff,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC,"Test3");
    // Pass an invalid HDC ,say HDC=-1L,to the function expecting
    // an error
    fSuccess=GreSetDefaultAttributes((HDC)0xfffffff,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_HDC,"Test3");
    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_HDC_BUSY error Code *
    *****************************************************************/

    SUBENTER(PMERR_HDC_BUSY,NULL);
    (VOID)MAKEPSBUSY(hPS);               //Make this PS busy.
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_HDC_BUSY,NULL);
    (VOID)FREEPS;                       //Kill BUSY thread.
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_SETID               *
    * error Code                                                          *
    ***********************************************************************/

    AreaBundle.abnd.ptlRefPoint.x=-12344;
    AreaBundle.abnd.ptlRefPoint.y=-12344;
    AreaBundle.cDefs=sizeof(AreaBundle.cDefs);
    AreaBundle.abnd.usSet=0x5666;
    SUBENTER(PMERR_INV_SETID,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_AREA,ABB_SET,&AreaBundle);
    CHECKRC(fSuccess,PMERR_INV_SETID,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_MODE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    AreaBundle.abnd.ptlRefPoint.x=-12344;
    AreaBundle.abnd.ptlRefPoint.y=-12344;
    SUBENTER(PMERR_INV_CHAR_MODE_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_AREA,ABB_REF_POINT,&AreaBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_MODE_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_MODE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.cbnd.usPrecision=0x1234;
    SUBENTER(PMERR_INV_CHAR_MODE_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_MODE,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_MODE_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_MARKER_SYMBOL_ATTR  *
    * error Code                                                          *
    ***********************************************************************/

    MarkerBundle.mbnd.usSymbol=0x7fff;
    SUBENTER(PMERR_INV_MARKER_SYMBOL_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_MARKER,MBB_SYMBOL,&MarkerBundle);
    CHECKRC(fSuccess,PMERR_INV_MARKER_SYMBOL_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_CHAR_DIRECTION_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.cbnd.usDirection=0x1234;
    SUBENTER(PMERR_INV_CHAR_DIRECTION_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_DIRECTION,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CHAR_DIRECTION_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_COLOR_ATTR          *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.cbnd.lColor=0x9fffffffL;
    SUBENTER(PMERR_INV_COLOR_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_COLOR,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_COLOR_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_BACKGROUND_COL_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.cbnd.lBackColor=0x0fffffffL;
    SUBENTER(PMERR_INV_BACKGROUND_COL_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_BACK_COLOR,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_COL_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_CODEPAGE            *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.cbnd.usPrecision=CM_MODE1;
    CharBundle.cdef.CodePage=0xffffu;
    CharBundle.cbnd.usSet=0xffffu;
    SUBENTER(PMERR_INV_CODEPAGE,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_SET,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_CODEPAGE,NULL);
    SUBEXIT();


    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_AREA        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_AREA,NULL);

    SETENTER(EntryName," Opening AREA bracket");

    if (GpiBeginArea(hPS,BA_WINDING)==FALSE){
       SHOWERROR("GpiBeginArea()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open area
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_IN_AREA,NULL);

    //Close The Area

    SETENTER(EntryName," Closing AREA bracket");

    if (GpiEndArea(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndArea()",EntryName);
    }
    SETEXIT();

    SUBEXIT();

    /*****************************************************************
    * The following Code will check for the PMERR_INV_IN_PATH        *
    * error  code.                                                   *
    *****************************************************************/
    SUBENTER(PMERR_INV_IN_PATH,NULL);

    SETENTER(EntryName," Opening Path bracket");
    if (GpiBeginPath(hPS,1L)==FALSE){
       SHOWERROR("GpiBeginPath()",EntryName);
    }
    SETEXIT();

    // Try to intialize the attributes while in an open path
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_IN_PATH,NULL);

    //Close The Area
    SETENTER(EntryName," Closing Path bracket");

    if (GpiEndPath(hPS)==GPI_ERROR){
       SHOWERROR("GpiEndPath()",EntryName);
    }
    SETEXIT();

    SUBEXIT();

    /****************************************************************
    * The following Code will check for the PMERR_UNSUPPORTED_ATTR  *
    * error Code                                                     *
    *****************************************************************/
    SUBENTER(PMERR_UNSUPPORTED_ATTR,"Test1");

    // Pass an invalid Primitive Type ,say 0xffffffff, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,0xffffffffL,&LineBundle);
    CHECKRC(fSuccess,PMERR_UNSUPPORTED_ATTR,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_UNSUPPORTED_ATTR,"Test2");

    // Pass an invalid Primitive Type ,say 0x0ffffffL, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,0x0fffffffL,&LineBundle);
    CHECKRC(fSuccess,PMERR_UNSUPPORTED_ATTR,"Test2");
    SUBEXIT();


    /****************************************************************
    * The following Code will check for the PMERR_INV_PRIMITIVE_TYPE *
    * error Code                                                     *
    *****************************************************************/
    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test1");

    // Pass an invalid Primitive Type ,say 0xffffffff, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,0xffffffff,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test1");
    SUBEXIT();

    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test2");

    // Pass an invalid Primitive Type ,say 0x00, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,(LONG)NULL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test2");
    SUBEXIT();

    SUBENTER(PMERR_INV_PRIMITIVE_TYPE,"Test3");

    // Pass an invalid Primitive Type ,say 0x0ffffffL, to
    // the function expecting an error

    fSuccess=GreSetDefaultAttributes(hDC,0x0fffffffL,LBB_COLOR,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_PRIMITIVE_TYPE,"Test3");
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_MIX_ATTR            *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.lbnd.usMixMode=0x1234;
    SUBENTER(PMERR_INV_MIX_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_MIX_MODE,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_MIX_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_BACKGROUND_MIX_ATTR *
    * error Code                                                          *
    ***********************************************************************/

    CharBundle.cbnd.usBackMixMode=0x1234;
    SUBENTER(PMERR_INV_BACKGROUND_MIX_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_CHAR,CBB_BACK_MIX_MODE,&CharBundle);
    CHECKRC(fSuccess,PMERR_INV_BACKGROUND_MIX_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_END_TYPE_ATTR       *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.lbnd.usEnd=0x1234;
    SUBENTER(PMERR_INV_LINE_END_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_END,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_END_ATTR,NULL);
    SUBEXIT();

    /**********************************************************************
    * The following Code will check for the PMERR_INV_JOIN_TYPE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.lbnd.usJoin=0x1234;
    SUBENTER(PMERR_INV_LINE_JOIN_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_JOIN,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_JOIN_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_LINE_TYPE_ATTR      *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.lbnd.usType=0x1234;
    SUBENTER(PMERR_INV_LINE_TYPE_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_TYPE,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_TYPE_ATTR,NULL);
    SUBEXIT();


    /**********************************************************************
    * The following Code will check for the PMERR_INV_GEOM_LINE_WIDTH_ATTR*
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.lbnd.lGeomWidth=MAKEFIXED(-1234,-7474);
    SUBENTER(PMERR_INV_GEOM_LINE_WIDTH_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_GEOM_WIDTH,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_GEOM_LINE_WIDTH_ATTR,NULL);
    SUBEXIT();



    /**********************************************************************
    * The following Code will check for the PMERR_INV_LINE_WIDTH_ATTR     *
    * error Code                                                          *
    ***********************************************************************/

    LineBundle.lbnd.fxWidth=-34l;
    SUBENTER(PMERR_INV_LINE_WIDTH_ATTR,NULL);
    fSuccess=GreSetDefaultAttributes(hDC,PRIM_LINE,LBB_WIDTH,&LineBundle);
    CHECKRC(fSuccess,PMERR_INV_LINE_WIDTH_ATTR,NULL);
    SUBEXIT();

    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreSetBitmapOwnerRc()
 *
 * Purpose:  This function exhaustively tests for most of errors codes
 *           that may be returned by GreSetBitmapOwner().
 *     The syntax used to call the Gre is :
 *
 *     (BOOL)GreSetBitmapOwner(HDC  hDC,
 *                             HBITMAP hBM
 *                            )
 *
 *
 * Revision Log:
 *               10/08/91, PDVT ,Mike Abuzant
 *               -initial creation
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Return Codes Successfully Tested:
 *      None.
 *
 * Return Codes Not Tested Successfully Yet:
 *
 * Subroutines Required:
 *        None.
 *
 * Limitations:
 *
 *   This entry point is not documented due to the fact that it is not
 *   documented.
 *
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *  //setup conditions to test all possible error codes
 *
 *   FOR ( Each possible error code) DO
 *     //Setup conditions to produce this error code,
 *     //then call the Gre.
 *     ulSuccess= GreSetBitmapOwner(hDC...);
 *     if ( ulSuccess== FALSE) {
 *       CALL  CHECKRETURNCODE(UlSuccess,PMERR_XXXX)
 *   ENDFOR
 *\\end
 ***************************************************************************/
VOID  APIENTRY GreSetBitmapOwnerRc(HPS hPS, HWND hWndClient, BOOL SelectionCall){

  static  PCHAR  EntryName="GreSetBitmapOwnerRc()";

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

    EXIT_MAIN(EntryName);
  }

  return;
}
