/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GREBNDS.C")
#pragma subtitle("IBM Confidential")

/*const char header=$header$*/

#define INCL_WIN
#define INCL_GPI
#define INCL_GREALL
#define INCL_ERRORS
#define INCL_DDIMISC
#define INCL_DEV

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <os2.h>
#include <pmddi.h>
#include "DTTDLLDF.h" // test case defines
#include "grebnds.h" /* gre bounds dll include file */
#include "std_util.h"

/****************************************************************************
 *\\ddd
 * Subtask Name: grebnds.dll
 *
 * Module name: grebnds.c
 *
 * Purpose: gre bounds dll test functions
 *
 * Revision Log:
 *               12/26/90, pdvt, markrichardson
 *                     - initial codeing
 *
 * SRS References: 6.11
 *
 * Contents:
 *     GreResetBoundsRc
 *     GreGetBoundsDataRc
 *     GreAccumulateBoundsRc
 *     InitTest
 *
 * Design Overview:
 *     The bounds gre interface calls produce no graphic output.  Each of
 *     the entry points for GreResetBounds, GreGetBoundsData, and GreAccumulateBounds
 *     test for certain return codes only.
 *
 * Limitations:
 *     There is no graphic output to the screen or the printer.
 *
 *\\end
****************************************************************************/
/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose:  Called by DTT to initially setup global data.
 *
 * System Requirements: none
 *
 * Revision Log:
 *               12/26/90, pdvt, markrichardson
 *                     - initial codeing
 *
 * Inputs:
 *       sel         - gloabal data for DTT
 *       pInitInfo   - global functions data
 *
 * Outputs:
 *       pInitInfo   - containing respective functions data
 *
 * Subroutines Required:
 *       InitTestSegVars - used by DTT
 *
 * Limitations:
 *       pInitInfo is of ENTRY_POINTS size as defined in DTTdlldf.h
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * set the number of entry points in this test case
 * set the testcase group name
 * for each function
 *     set the function address
 *     set the function name
 * end for
 *\\end
 ***************************************************************************/
VOID APIENTRY InitTest(PVOID sel, PTESTINITINFO pInitInfo)
  {
    pInitInfo->ntests = NTESTS;
    strcpy(pInitInfo->group_name,pGroupName);

    strcpy(pInitInfo->test_names[0],pTestName0);
    pInitInfo->test_addrs[0]=GreBoundsApp;

    strcpy(pInitInfo->test_names[1],pTestName1);
    pInitInfo->test_addrs[1]=GreResetBoundsRc;
    strcpy(pInitInfo->test_names[2],pTestName2);
    pInitInfo->test_addrs[2]=GreGetBoundsDataRc;
    strcpy(pInitInfo->test_names[3],pTestName3);
    pInitInfo->test_addrs[3]=GreAccumulateBoundsRc;

    InitTestSegVars(sel);
  }
/****************************************************************************
 *\\ddd
 * Routine Name: GreBoundsApp()
 *
 * Purpose:   Provide a working scenario using two or more of the gre functions
 *            tested in this testcase.
 *
 * System Requirements: 6.11
 *
 * Revision Log:
 *               12/26/90, pdvt, markrichardson
 *                     - initial codeing
 *
 * Inputs:
 *      hPS            - presentation space handle
 *      hWndClient     - window client handle
 *      SelectionCall  - selection parameter used by DTT
 *
 * Outputs:
 *      Test case success or failure entries in DTT_log.
 *
 * Subroutines Required:
 *      LOG_SIGNATURE
 *      CWRITELOG
 *      CHECKRETURNERROR
 *
 * Limitations:
 *      No screen or printer output.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *  reset the gpi bounds in model coordinates
 *  getboundsdata, returning null if bounds have been reset
 *  report the number of errors
 *\\end
 ***************************************************************************/
VOID APIENTRY GreBoundsApp(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  DEVOPENSTRUC   dop;         /* device open structure */
  DRIVDATA       driv;        /* sub structure to dop */
  HAB            hAB;         /* anchor block handle */
  HDC            printer_hdc; /* return code from DevOpenDC */
  LONG           lDCtype;     /* used by WinSendMessage */
  LONG           error_count; /* number of api failures */
  CHAR           out_string[257]; /* error message buffer */
  PRECTL         pBoundsData; /* pointer to returned structre defining bounds rectangle */
  BOOL           b_rc;        /* boolean return code for gre call */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreBoundsApp - Entry.\n");

        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCtype));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR ("DevOpenDC",TRUE,"GreBoundsApp");
           return;
        }
        /*
         * call reset bounds expecting good return\
         */
        b_rc = GreResetBounds(printer_hdc,RB_GPI);
        if (b_rc == TRUE) {
           pBoundsData = malloc(sizeof(RECTL));
           b_rc = GreGetBoundsData(printer_hdc,GBD_GPI,pBoundsData);
           if (b_rc == TRUE) {
              /*
               * if the bounds have been reset, GetBoundsData returns a null
               * pointer for pBoundsData
               * Presentation Driver Interfaces, Volume 2, Chapter 12, page 83
               */
               // Note from TSS...Page 83 states that a null boudary
               // is returned when the bounds have been reset.  But,
               // a null boundary is not a null pointer.  Rather, it
               // is defined (also on page 83) as a boundary whose
               // minimum boundary value is greater than its maximum
               // boundary value.  Hence,  I have change the following
               // *if* statement to check for this condition rather than
               // a null pointer as was previously done.  5/20/91

              if (pBoundsData->xRight <= pBoundsData->xLeft) {
                 CWRITELOG(L_LOTRACE,0,"$Bounds have been reset.\n");
                 CWRITELOG(L_LOTRACE,0,"$GreGetBoundsData returned null bounds as expected.\n");
              }
              else {
                   error_count += 1;
                   CWRITELOG(L_LOTRACE,0,"$Bounds have been reset.\n");
                   CWRITELOG(L_LOTRACE,0,"$GreGetBoundsData did not return as expected.\n");
                   CWRITELOG(L_LOTRACE,0,
"$GreGetBoundsData                                               -    FAIL.\n");
              }
           }
           else {
                 BADERROR ("GreBoundsApp",TRUE,"GreBoundsApp");
                 return;
           }
        }
        else {
              BADERROR ("GreBoundsApp",TRUE,"GreBoundsApp");
              return;
        }
        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreBoundsApp                                                   -    FAIL.\n");
        }
        else {
           CWRITELOG(L_HDR,0,
"$GreBoundsApp                                                   -    PASS.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ee+ GreBoundsApp - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreResetBoundsRc()
 *
 * Purpose:
 *
 * System Requirements: <paragraph references>
 *
 * Revision Log:
 *               12/26/90, pdvt, markrichardson
 *                     - initial codeing
 *
 * Inputs:
 *      hPS            - presentation space handle
 *      hWndClient     - window client handle
 *      SelectionCall  - selection parameter used by DTT
 *
 * Outputs:
 *      Test case success or failure entries in DTT_log.
 *
 * Subroutines Required:
 *      LOG_SIGNATURE
 *      CWRITELOG
 *      CHECKRETURNERROR
 *
 * Limitations:
 *      No screen or printer output.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *  setup conditions for an invalid device context
 *  call greresetbounds checking for PMERR_INV_HDC return code
 *  report number of errors
 *\\end
 ***************************************************************************/

VOID APIENTRY GreResetBoundsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  DEVOPENSTRUC   dop;         /* device open structure */
  DRIVDATA       driv;        /* sub structure to dop */
  HAB            hAB;         /* anchor block handle */
  HDC            printer_hdc; /* return code from DevOpenDC */
  LONG           lDCtype;     /* used by WinSendMessage */
  LONG           error_count; /* number of api failures */
  CHAR           out_string[257]; /* error message buffer */
  BOOL           b_rc;        /* boolean return code for gre call */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreResetBoundsRc - Entry.\n");
        /*
         * get the current anchor block
         * and printer device context
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCtype));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR ("DevOpenDC",TRUE,"GreResetBoundsRc");
           return;
        }

        /*
         * call the gre function with an invalid device context handle
         * expecting error return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreResetBoundsRc - Entry.\n");
        b_rc = GreResetBounds((HDC)0,0);
        if (b_rc == FALSE) {
           error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreResetBoundsRc - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG)NULL
                                      ,NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreResetBoundsRc                                               -    FAIL.\n");
        }
        else {
           CWRITELOG(L_HDR,0,
"$GreResetBoundsRc                                               -    PASS.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ee+ GreResetBoundsRc - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreGetBoundsRc()
 *
 * Purpose:
 *
 * System Requirements: <paragraph references>
 *
 * Revision Log:
 *               12/26/90, pdvt, markrichardson
 *                     - initial codeing
 *
 * Inputs:
 *      hPS            - presentation space handle
 *      hWndClient     - window client handle
 *      SelectionCall  - selection parameter used by DTT
 *
 * Outputs:
 *      Test case success or failure entries in DTT_log.
 *
 * Subroutines Required:
 *      LOG_SIGNATURE
 *      CWRITELOG
 *      CHECKRETURNERROR
 *
 * Limitations:
 *      No screen or printer output.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *  setup conditions for an invalid device context
 *  call gregetboundsdata checking for PMERR_INV_HDC return code
 *  report number of errors
 *\\end
 ***************************************************************************/
VOID APIENTRY GreGetBoundsDataRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  DEVOPENSTRUC   dop;         /* device open structure */
  DRIVDATA       driv;        /* sub structure to dop */
  HAB            hAB;         /* anchor block handle */
  HDC            printer_hdc; /* return code from DevOpenDC */
  LONG           lDCtype;     /* used by WinSendMessage */
  LONG           error_count; /* number of api failures */
  CHAR           out_string[257]; /* error message buffer */
  BOOL           b_rc;        /* boolean gre return code */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreGetBoundsDataRc - Entry.\n");

        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCtype));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR ("DevOpenDC",TRUE,"GreGetBoundsDataRc");
           return;
        }
        /*
         * call the gre function with an invalid device context handle
         * expecting error return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreGetBoundsDataRc - Entry.\n");
        b_rc = GreGetBoundsData((HDC)0,GBD_GPI,(PRECTL)0);
        if (b_rc == FALSE) {
           error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreGetBoundsDataRc - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG)NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreGetBoundsDataRc                                             -    FAIL.\n");
        }
        else {
           CWRITELOG(L_HDR,0,
"$GreGetBoundsDataRc                                             -    PASS.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ee+ GreGetBoundsDataRc - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreAccumulateBoundsRc()
 *
 * Purpose:
 *
 * System Requirements: <paragraph references>
 *
 * Revision Log:
 *               12/26/90, pdvt, markrichardson
 *                     - initial codeing
 *
 * Inputs:
 *      hPS            - presentation space handle
 *      hWndClient     - window client handle
 *      SelectionCall  - selection parameter used by DTT
 *
 * Outputs:
 *      Test case success or failure entries in DTT_log.
 *
 * Subroutines Required:
 *      LOG_SIGNATURE
 *      CWRITELOG
 *      CHECKRETURNERROR
 *
 * Limitations:
 *      No screen or printer output.
 *   Possible error codes returned by GreAccumulateBounds:
 *       PMERR_HDC_BUSY
 *       PMERR_DEV_FUNC_NOT_INSTALLED
 *       PMERR_COORDINATE_OVERFLOW
 *       PMERR_INV_LENGTH_OR_COUNT
 *       PMERR_INV_RECT
 *       PMERR_INV_HDC
 *       PMERR_INV_COORD_SPACE
 *   Error codes successfully tested for in this routine:
 *       PMERR_INV_HDC
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *  setup conditions for an invalid device context
 *  call greaccumulatebounds checking for PMERR_INV_HDC return code
 *  report number of errors
 *\\end
 ***************************************************************************/
VOID APIENTRY GreAccumulateBoundsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  DEVOPENSTRUC   dop;         /* device open structure */
  DRIVDATA       driv;        /* sub structure to dop */
  HAB            hAB;         /* anchor block handle */
  HDC            printer_hdc; /* return code from DevOpenDC */
  LONG           lDCtype;     /* used by WinSendMessage */
  LONG           error_count; /* number of api failures */
  CHAR           out_string[257]; /* error message buffer */
  LONG           rc;          /* boolean gre return code */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreAccumulateBoundsRc - Entry.\n");

        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCtype));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR ("DevOpenDC",TRUE,"GreAccumulateBoundsRc");
           return;
        }
        /*
         * call the gre function with an invalid device context handle
         * expecting error return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreAccumulateBoundsRc - Entry.\n");
        rc = GreAccumulateBounds((HDC)0,(PRECTL)0);
        if (rc == DEV_ERROR) {
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreAccumulateBoundsRc - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG)NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreAccumulateBoundsRc                                          -    FAIL.\n");
        }
        else {
           CWRITELOG(L_HDR,0,
"$GreAccumulateBoundsRc                                          -    PASS.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ee+ GreAccumulateBoundsRc - Exit.\n");
  }
}
