/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GREBTMP.C")
#pragma subtitle("IBM Confidential")

#define INCL_DEV      // include all DEV definitions
#define INCL_WIN
#define INCL_GPI      // include all Gpi definitions
#define INCL_GREALL   // include all Gre definitions
#define INCL_ERRORS   // include all Gre error definitions
#define INCL_DDIMISC
#define INCL_DEV
#include <string.h>
#include <stdlib.h>
#include <os2.h>
#include <stdio.h>
#include <math.h>
#include <pmddi.h>                  // gre - device driver includes
#include "DTTDLLDF.h"               // test case defines
#include "gre_grid.h"               // For grid drawing functions
#include "std_util.h"
#include "grebtmp.h"                // bit map dll header file

LONG ErrorCount;
CHAR achLogLine[100];
/****************************************************************************
 *\\ddd
 * Subtask Name: grebtmp.dll
 *
 * Module name: grebtmp.c
 *
 * Purpose: bit map dll test functions
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/13/90 ,PDVT ,Mike Abuzant
 *               -Test Case Design.
 *
 * SRS References: Printer Driver Verification Tool
 *
 * Contents:
 *         Grebtmp_app()                 // Application (Functionality)
 *         grebtmp_pre1()                 // precision test entry point.
 *         grebtmp_pre2()                 // precision test entry point.
 *         GreDeviceSelectBitmapExh()    //exhaustive GreDeviceSelectBitmap test
 *         GreDeviceDeleteBitmapExh()    //exhaustive GreDeviceDeleteBitmap test
 *         GreSetPelExh()                //exhaustive GreSetPel test
 *         GreBitbltExh()                //exhaustive GreBitblt test
 *         GreImageDataExh()             //exhaustive GreImageData test
 *         GreGetBitmapBitsExh()         //exhaustive GreGetBitmapBits test
 *         GreSetBitmapBitsExh()         //exhaustive GreSetBitmapBits test
 *         GreDrawBorderExh()            //exhaustive GreDrawBorder test
 *         GreDrawBorderRc()            //exhaustive GreDrawBorder test
 *         GreDeviceCreateBitmapRc()    //exhaustive GreDeviceCreateBitmap test
 *
 * Design Overview:
 *   This TestCase is designed to test Bitmap manipulation functions.These
 *   range form Creating,Deleting,and Selecting bitmaps to functions
 *   that manipulate the bitmaps. The GreDeviceCreateBitmap,GreDeviceSelectBitmap,
 *   GreDeviceDeleteBitmap,GreGetBitmapBits,GreSetBitmapBits,GreSetPel,
 *   GreGetPel,GreDrawBorder,GreImageData,and GreBitblt functions  have been
 *   tested. All tests include  a basic functionality test and an exhaustive
 *   (test all return codes at the lowest level) test. Only One function,
 *    GreDrawBorder, has been tested for  precision .
 *
 * Limitations:
 *    All hard-copy vector devices will fail on all bit-map operation since
 *  it is impossible to translate bitmaps to vectors.
 *     All the Gpi and Gre functions, other than the above, which are used in
 *  implementing this testcase, are assumed to be fully tested in other
 *  testcases.
 *
 *\\end
****************************************************************************/
/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose:  Initialize function name and address array for DTT.
 *
 * System Requirements:  None
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *
 * Inputs:
 *   sel          // passed through to the InitTestSegVars function
 *   init_info_p  // name and address array used by DTT interface
 *
 * Outputs:
 *   init_info_p  // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()    // used by the DTT interface
 *
 * Limitations:
 *   The size of the pInitInfo structure is defined in DTTdlldf.h
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *   for the number of test functions
 *       set function name
 *       set function address
 *   call init set function
 *\\end
 ***************************************************************************/
VOID APIENTRY InitTest(PVOID sel, PTESTINITINFO init_info_p)
  {
    init_info_p->ntests = NTESTS;
    strcpy(init_info_p->group_name,TestCaseName); //Testcase Name

    strcpy(init_info_p->test_names[0],pTestName1);
    init_info_p->test_addrs[0]=grebtmp_pre1;
    strcpy(init_info_p->test_names[1],pTestName2);
    init_info_p->test_addrs[1]=grebtmp_pre2;

    strcpy(init_info_p->test_names[2],pTestName4);
    init_info_p->test_addrs[2]=GreBitmapApp;

    strcpy(init_info_p->test_names[3],pTestName16);
    init_info_p->test_addrs[3]=GreSetPelExh;
    strcpy(init_info_p->test_names[4],pTestName17);
    init_info_p->test_addrs[4]= GreBitbltExh;
    strcpy(init_info_p->test_names[5],pTestName18);
    init_info_p->test_addrs[5]=GreImageDataExh;
    strcpy(init_info_p->test_names[6],pTestName19);
    init_info_p->test_addrs[6]=GreSetBitmapBitsExh;
    strcpy(init_info_p->test_names[7],pTestName20);
    init_info_p->test_addrs[7]= GreDrawBorderExh;

    strcpy(init_info_p->test_names[8],pTestName6);
    init_info_p->test_addrs[8]=GreSetPelRc;
    strcpy(init_info_p->test_names[9],pTestName7);
    init_info_p->test_addrs[9]=GreGetPelRc;
    strcpy(init_info_p->test_names[10],pTestName8);
    init_info_p->test_addrs[10]=GreDrawBorderRc;
    strcpy(init_info_p->test_names[11],pTestName9);
    init_info_p->test_addrs[11]=GreDeviceCreateBitmapRc;
    strcpy(init_info_p->test_names[12],pTestName10);
    init_info_p->test_addrs[12]=GreDeviceSelectBitmapRc;
    strcpy(init_info_p->test_names[13],pTestName11);
    init_info_p->test_addrs[13]=GreDeviceDeleteBitmapRc;
    strcpy(init_info_p->test_names[14],pTestName12);
    init_info_p->test_addrs[14]=GreImageDataRc;
    strcpy(init_info_p->test_names[15],pTestName13);
    init_info_p->test_addrs[15]=GreBitbltRc;
    strcpy(init_info_p->test_names[16],pTestName14);
    init_info_p->test_addrs[16]=GreSetBitmapBitsRc;
    strcpy(init_info_p->test_names[17],pTestName15);
    init_info_p->test_addrs[17]=GreGetBitmapBitsRc;

    InitTestSegVars(sel);
}
/****************************************************************************
 *\\ddd
 * Routine Name: grebtmp_pre1()
 *
 * Purpose:  This is an entry point for the DTT that causes Gre
 *               precision tests to be run.
 *
 * System Requirements:  Section 6.3.2
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/13/90 ,PDVT ,Mike Abuzant
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *    DTT_log       - error logging file
 *    bitmap images - screen and printed output
 *
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *  query the device context handle hDC
 *  draw the one inch grid for precision specifications
 *  draw image of cross at (5,6)
 *  draw image of line at (6,3)
 *  draw image of 1X1 box at (2,3)
 *\\end
 ***************************************************************************/
VOID APIENTRY grebtmp_pre1(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL      ptl;                 //Screen coordinate point
  SIZEL       sizel;               // size of bitmap to draw
  LONG        lIndex1;             // used as an index into bitmap array
  BYTE        abImage[300];        // used to store bitmap image
  HDC         hDC;
  CHAR        ErrorString[200];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
       LOG_SIGNATURE;
       TEST_SIGNATURE(0,0);
       CWRITELOG(L_LOTRACE, 0, "+ee+ GreBitPre1 - Entry.\n");
       set_delta(hPS);
       ErrorCount = 0L;

       if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  { /* Get  the default device context */
            BADERROR("GpiQueryDevice()",TRUE,"GrePre1");
            return;
       }
       // call the grid function to draw 1" X 1" grid cells.
       gre_grid(hPS,(LONG)INCH);
       memset (abImage,0xFF,sizeof(abImage));    //Clear the bitmap buffer to 1's
       if (GpiSetColor(hPS,CLR_GREEN)==GPI_ERROR) {   // Set foreground color to Green
       }
       sizel.cx=(LONG)CONVERTTOINCHES(hori_pels_per_m);
       sizel.cy=(LONG)CONVERTTOINCHES(vert_pels_per_m);
       ptl.x=2*delta_x;
       ptl.y=3*delta_y;
       MapPt(ptl);
       if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
       }
       // The third parameter will tell GreImage() where
       // to place the next line of the bitmap.
       for (lIndex1=0;lIndex1<=(sizel.cy);lIndex1++){
          if (GreImageData(hDC,abImage,sizel.cx+2L,-lIndex1)==GPI_ERROR) {
              BADERROR("GreImageData()",TRUE,"GrePre1");
              return;
          }
       }
       ptl.x=delta_x;
       ptl.y=5*delta_y/2;
       sprintf(ErrorString,"1\" X 1\" Image");
       MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);
       ptl.x=delta_x;
       ptl.y=9*delta_y/4;
       sprintf(ErrorString,"Located @ (2\" , 3\" ) ");
       MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);
       /*******************************************************
        * Draw a bitmap of a CROSS inside one of the grid boxes
        ******************************************************/
       memset (abImage,0x00,sizeof(abImage));  // clear bitmap buffer.
       // fill bitmap buffer with appropriate bit to form
       // a CROSS bitmap.
       abImage[(sizel.cx/16)]=(BYTE)(0x40>>(sizel.cx%16/2));
       if (GpiSetColor(hPS,CLR_RED)==GPI_ERROR){   // Set foreground color to RED
       }
       ptl.x=6*delta_x;
       ptl.y=5*delta_y;
       MapPt(ptl);
       if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
       }
       for (lIndex1=0;lIndex1<=(sizel.cy);lIndex1++){
           if (GreImageData(hDC,abImage,sizel.cx+1L,-lIndex1)==GPI_ERROR) {
                BADERROR("GreImageData()",TRUE,"GrePre1");
                return;
           }
       }
       // fill bitmap buffer with appropriate bit to form
       // a CROSS bitmap.
       memset(abImage,0xff,sizeof(abImage));
       ptl.x=6*delta_x;
       ptl.y=5*delta_y+(LONG)PIXELSTOUNITS(sizel.cx/2,delta_y,vert_pels_per_m);
       MapPt(ptl);
       if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
       }
       if (GreImageData(hDC,abImage,sizel.cx+1L,0L)==GPI_ERROR) {
           BADERROR("GreImageData()",TRUE,"GrePre1");
           return;
       }
       if (GpiSetColor(hPS,CLR_DEFAULT)==GPI_ERROR) {
       }
       ptl.x=5*delta_x;
       ptl.y=9*delta_y/2;
       sprintf(ErrorString,"Image Of a Cross  1\" X 1\"");
       MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);
       ptl.x=5*delta_x;
       ptl.y=17*delta_y/4;
       sprintf(ErrorString,"Located @ (5\",6\")" );
       MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);
       /*****************************************************************
        * Draw a bitmap of a Horizontal line inside one of the grid cells
        ****************************************************************/
       memset (abImage,0xff,sizeof(abImage));  // Clear bitmap buffer.
       ptl.x=6*delta_x;
       ptl.y=3*delta_y;
       MapPt(ptl);
       if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
       }
       if (GpiSetColor(hPS,CLR_BACKGROUND)==GPI_ERROR) {  // Set foreground color to
       }
       if (GreImageData(hDC,abImage,sizel.cx+1L,0L)==GPI_ERROR) {
          BADERROR("GreImageData()",TRUE,"GrePre1");
          return;
       }
       if (GpiSetColor(hPS,CLR_DEFAULT)==GPI_ERROR) {
       }
       ptl.x=4*delta_x;
       ptl.y=2*delta_y;
       sprintf(ErrorString,"Image Of a Line  1\" X 1 Pixel");
       MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);
       ptl.x=4*delta_x;
       ptl.y=7*delta_y/4;
       sprintf(ErrorString,"Located @ (6\",3\")" );
       MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);

       if (!ErrorCount) {
         CWRITELOG(L_HDR,0,
"$GreBitPre1                                                     -    PASS.\n");
       }
       else {
             sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
             CWRITELOG(L_HDR, 0, achLogLine);
             CWRITELOG(L_HDR, 0,
"$GreBitPre1                                                     -    FAIL.\n");
      }
        CWRITELOG(L_LOTRACE, 0, "+ex+ GreBitPre1 - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: grebtmp_pre2()
 *
 * Purpose:  This is an entry point for the DTT that causes Gre
 *               precision tests to be run.
 *
 * System Requirements:  Section 6.3.2
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/13/90 ,PDVT ,Mike Abuzant
 *               11/20/90 ,PDVT ,Mike Abuzant
 *               -initial file.
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *    DTT_log             - DTT error log file
 *
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY grebtmp_pre2(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   POINTL           ptl;               //Screen coordinate point
   SIZEL            sizel;             // size of bitmap to draw
   HDC              hDC;
   HBITMAP          hBM;
   HDC              hDC1;
   HPS              hPS1;
   HAB              hAB;
   BITMAPINFOHEADER BitMapHeader;      //bitmap info header
   PBITMAPINFO      pBitMapInfo;       //bitmap info structure
   POINTL           aptl[4];
   LONG             lDCType;
   DEVOPENSTRUC     dop;
   DRIVDATA         driv;
   BYTE             Temp_buffer[200];
   CHAR             ErrorString[200];
   LONG             xPixels;
   LONG             yPixels;
   static BYTE abCross[]={0x01,0x00,0x01,0x00,
                          0x01,0x00,0x01,0x00,
                          0x01,0x00,0x01,0x00,
                          0x01,0x00,0xff,0xff,
                          0x01,0x00,0x01,0x00,
                          0x01,0x00,0x01,0x00,
                          0x01,0x00,0x01,0x00,
                          0x01,0x00,0x01,0x00};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
       LOG_SIGNATURE;
       TEST_SIGNATURE(0,0);
       CWRITELOG(L_LOTRACE,0,"+ee+ GreBitPre2 Precision Test - Entry.\n");
       set_delta(hPS);
       ErrorCount = 0L;
       if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
            BADERROR("WinQueryAnchorBlock()",TRUE,"GreBtmpPre2");
            return;
       }
       if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {  /* Get  the default device context */
            BADERROR("GpiQueryDevice()",TRUE,"GreBtmpPre2");
            return;
       }
       // call the grid function to draw 100 X 100 grid cells.
       gre_grid(hPS,(LONG)INCH);
       // Initalize the Bitmap information structure
       // first we need to  allocate memory to hold RGB array
       // since  BITMAPINFO structure is large enough to hold
       // one RGB structure.
       pBitMapInfo=(PBITMAPINFO) Temp_buffer;
       pBitMapInfo->cbFix=sizeof(BITMAPINFOHEADER); // This always =12
       pBitMapInfo->cx=16;                          //Width
       pBitMapInfo->cy=16;                          //height
       pBitMapInfo->cPlanes=1;                      // one color plane
       pBitMapInfo->cBitCount=1;                    // 1-bit /pixel
       pBitMapInfo->argbColor[0].bBlue=
       pBitMapInfo->argbColor[0].bGreen=
       pBitMapInfo->argbColor[0].bRed=0;
       pBitMapInfo->argbColor[1].bBlue=
       pBitMapInfo->argbColor[1].bGreen=
       pBitMapInfo->argbColor[1].bRed=0xFF;
       BitMapHeader.cbFix=sizeof(BITMAPINFOHEADER); // This always =12
       BitMapHeader.cx=16;                          //Width
       BitMapHeader.cy=16;                          //height
       BitMapHeader.cPlanes=1;                      // one color plane
       BitMapHeader.cBitCount=1;                    // 1-bit /pixel
       sizel.cx=
       sizel.cy=0;
       // Open a Memory device context  to use for bitmap operations.
       dop.pdriv = &driv;
       WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
       if ((hDC1=DevOpenDC(hAB,OD_MEMORY,"*",6L,(PDEVOPENDATA)&dop,(HDC)NULL))==DEV_ERROR) {
          BADERROR("DevOpenDC()",TRUE,"GreBtmpPre2");
          return;
       }
       // Create a presentation space to associate with the
       // memory device context
       if ((hPS1=GpiCreatePS(hAB,hDC1,&sizel,PU_PELS |
                             GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR) {
           BADERROR("GpiCreatePS()",TRUE,"GreBtmpPre2");
           return;
       }
       // Create and obtain a bitmap handle to use with GreBitblt();
       if ((hBM=GreDeviceCreateBitmap(hDC1,&BitMapHeader,
                                  CBM_INIT,abCross,pBitMapInfo))==GPI_ERROR) {
           BADERROR("GreDeviceCreateBitmap()",TRUE,"GreBtmpPre2");
           return;
       }
       // The width of grid cells in pixels;
       xPixels=(LONG)CONVERTTOINCHES(hori_pels_per_m);
       // The height  of grid cells in pixels;
       yPixels=(LONG)CONVERTTOINCHES(vert_pels_per_m);
       // Perform a GreBitblt()  operation that will invert (ROP_DSTINVERT)
       // each  bitmap bits. and copy the bit map to the display
       // device passed to us by DTT.EXE
       aptl[0].x=2*xPixels+1;
       aptl[0].y=3*yPixels+2;
       aptl[1].x=3*(xPixels)+3;
       aptl[1].y=4*(yPixels)+3;
       if (GreBitblt(hDC,hDC1,2L,aptl,ROP_DSTINVERT,BBO_AND,NULL)==GPI_ERROR) {
            BADERROR("GreBitblt()",TRUE,"GreBtmpPre2");
            return;
        }
        sprintf(ErrorString,"Inverted Image Of Square Box 1\" X 1\"");
        ptl.x=delta_x;
        ptl.y=3*delta_y-delta_y/3;
        MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);

        sprintf(ErrorString,"Located @ (2\",3\")" );
        ptl.x=delta_x;
        ptl.y=3*delta_y-(2*delta_y/3);
        MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);

        // Perform a GreBitblt()  operation that will set all bits (ROP_ONE)
        // within the bitmaps to all ONE'S and copy the bit map to the display
        // device passed to us by DTT.EXE
        aptl[0].x=5*xPixels+3;
        aptl[0].y=5*yPixels+3;
        aptl[1].x=6*(xPixels)+5;
        aptl[1].y=6*(yPixels)+5;
        if (GreBitblt(hDC,hDC1,2L,aptl,ROP_ONE,BBO_AND,NULL)==GPI_ERROR) {
            BADERROR("GreBitblt()",TRUE,"GreBtmpPre2");
            return;
         }
         sprintf(ErrorString,"GreBitblt() with ROP_ONE");
         ptl.x=9*delta_x/2;
         ptl.y=(LONG)(9*delta_y/2);
         MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);

         sprintf(ErrorString,"On Square Box 1\" X 1\"");
         ptl.x=9*delta_x/2;
         ptl.y=(LONG)(17*delta_y/4);
         MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);

         sprintf(ErrorString,"Located @ (5\", 5\")");
         ptl.x=9*delta_x/2;
         ptl.y=(LONG)(4*delta_y);
         MapAndWriteString(hPS,ptl.x,ptl.y,ErrorString);

         // De-select the bitmap handle. otherwise we get error when closing
         // device context
         if (GreDeviceSelectBitmap(hDC1,NULL)==FALSE) {
            BADERROR("GreDeviceSelectBitmap()",TRUE,"GreBtmpPre2");
            return;
         }
         // Delete the bitmap handle.
         // device context
         if (GreDeviceDeleteBitmap(hDC1,hBM,NULL,!CBM_INIT)==FALSE) {
            BADERROR("GreDeviceDeleteBitmap()",TRUE,"GreBtmpPre2");
            return;
         }
         // dispose of presentation space. No longer needed.
         if (GpiDestroyPS(hPS1)==FALSE) {
            BADERROR("GpiDestroyPS()",TRUE,"GreBtmpPre2");
            return;
         }
         //Close the device context
         if (DevCloseDC(hDC1)==DEV_ERROR) {
             BADERROR("DevCloseDC()",TRUE,"GreBtmpPre2");
             return;
         }
         if (!ErrorCount) {
             CWRITELOG(L_HDR,0,
"$GreBitPre2                                                     -    PASS.\n");
      }
      else {
           sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
           CWRITELOG(L_HDR, 0, achLogLine);
           CWRITELOG(L_HDR, 0,
"$GreBitPre2                                                     -    FAIL.\n");
      }
      CWRITELOG(L_LOTRACE,0,"+ex+ GreBitPre2 Precision Test - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreBitmapApp()
 *
 * Purpose:  This function tests the basic functionality of the following
 *           gre functions:
 *     GreSetPel()
 *     GreGetPel()
 *
 * System Requirements: 6.3.2
 *
 * Revision Log:
 *               11/23/1990 ,PDVT , Mike Abuzant
 *               - TestCase coding  and design
 *               01/17/91, pdvt, markrichardson
 *               - redesign and rewrite on this runction eliminating
 *                 the previous spiral and trigonometric function calls
 *
 * Inputs:
 *    hps              // presentation space
 *    hWndClient       // client window
 *
 * Outputs:
 *    DTT_log          - DTT error log file
 *    graphic images   - to screen and printer device
 *
 * Subroutiynes Required:
 *  set_delta(hPS);
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * for three parabolas
 *  draw (+x,+y) part of the parabola,
 *  reflect this halv across origin to create complete parabola
 *  reflect whole parabola across mirror at y = 4*delta_y
 *\\end
 ***************************************************************************/
VOID APIENTRY GreBitmapApp(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG     x;
  LONG     y;
  HDC      hDC;
  POINTL   ptl;
  LONG     rc;
  LONG     x_origin;  /* x coordinate of parabola apex */
  LONG     y_origin;  /* y coordinate of parabola apex */
  LONG     y_mirror;  /* draw parabola upside down by reflecting vertically */

  if (SelectionCall) {
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      *pBasisX = 10000;
      *pBasisY = 10000;
  } else {
          LOG_SIGNATURE;
          TEST_SIGNATURE(0,0);
          CWRITELOG(L_LOTRACE,0,"+ee+ GreBitmapApp Application Test - Entry.\n");
          ErrorCount = 0L;
          hDC = set_delta(hPS);

          if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
             BADERROR("GpiQueryDevice()",TRUE,"GreBitmapApp");
             return;
          }
          if (GpiSetColor(hPS,CLR_RED)==GPI_ERROR) {
          }

          /*
           * let the parabola go through the origin
           * it is then simply reflecting half the parabola to get the
           * other half
           */
          x_origin = 4 * delta_x;
          y_origin = delta_y;
          y_mirror = 4 * delta_y;
          for (x = 1; x < 1500; x +=5) {
               y = (((x+2)*(x+2))+30)/900;
               ptl.x = x + x_origin;
               ptl.y = y + y_origin;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
               /*
                * get the original y value, i.e. unmap it
                * reflect the x value
                */
               ptl.y = y + y_origin;
               ptl.x = x_origin - x;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
               /*
                * to reflect the above parabola, simply
                * setup a mirror at 4 * delta_y and reflect each coordinate
                * vertically
                */
               ptl.y = y_mirror + (y_mirror - y);
               ptl.x = x + x_origin;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
               ptl.y = y_mirror + (y_mirror - y);
               ptl.x = x_origin - x;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
          }
          for (x = 1; x < 1386; x +=5) {
               y = (((x+2)*(x+2))+30)/600;
               ptl.x = x + x_origin;
               ptl.y = y + y_origin;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
               /*
                * get the original y value, i.e. unmap it
                * reflect the x value
                */
               ptl.y = y + y_origin;
               ptl.x = x_origin - x;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
               /*
                * to reflect the above parabola, simply
                * setup a mirror at 4 * delta_y and reflect each coordinate
                * as it is received
                */
               ptl.y = y_mirror + (y_mirror - y);
               ptl.x = x + x_origin;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
               ptl.y = y_mirror + (y_mirror - y);
               ptl.x = x_origin - x;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
          }
          for (x = 1; x < 1000; x +=5) {
               y = (((x+2)*(x+2))+30)/300;
               ptl.x = x + x_origin;
               ptl.y = y + y_origin;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
               /*
                * get the original y value, i.e. unmap it
                * reflect the x value
                */
               ptl.y = y + y_origin;
               ptl.x = x_origin - x;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
               /*
                * to reflect the above parabola, simply
                * setup a mirror at 4 * delta_y and reflect each coordinate
                * as it is received
                */
               ptl.y = y_mirror + (y_mirror - y);
               ptl.x = x + x_origin;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
               ptl.y = y_mirror + (y_mirror - y);
               ptl.x = x_origin - x;
               MapPt(ptl);
               rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
          }

       ptl.x = delta_x /4;
       ptl.y = delta_y - (delta_y / 4);
       MapAndWriteString(hPS,ptl.x,ptl.y,"GreSetPel exhaustively setting pixels.");

       if (!ErrorCount) {

           CWRITELOG(L_HDR,0,
"$GreBitmapApp                                                   -    PASS.\n");
       }
       else {
            sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
            CWRITELOG(L_HDR, 0, achLogLine);
            CWRITELOG(L_HDR, 0,
"$GreBitmapApp                                                   -    FAIL.\n");
       }
       CWRITELOG(L_LOTRACE,0,"+ex+ GreBitmapApp Application Test - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceCreateBitmapRc()
 *
 * Purpose:  This function Exhaustively tests for all errors codes returned
 *           by GreDeviceCreateBitmap(). The syntax used to call the Gre is :
 *           (LONG) GreDeviceCreateBitmap(
 *                       HDC               hdc,
 *                       PBITMAPINFOHEADER pInfoHd,
 *                       ULONG             flUsage,
 *                       PBYTE             pBitMap,
 *                       PBITMAPINFO       pInfo,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *   GreDeviceCreateBitmap() creates a bit map and obtains its handle.
 *   Errors That are returned by this function are :
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_HDC
 *   PMERR_INV_INFO_TABLE
 *   PMERR_INV_BITMAP_DIMENSION
 *   PMERR_INV_SCAN_START
 *   This function also checks for valid conditions.
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/06/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *
 * Limitations:
 *  Return Codes Successfully Tested:
 *      PMERR_INV_LENGTH_OR_COUNT
 *      PMERR_INV_HDC
 *      PMERR_INV_INFO_TABLE
 *      PMERR_INV_BITMAP_DIMENSION
 *  Return Codes Not Tested Yet:
 *          NONE
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceCreateBitmapRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC    hDC;                          //Handle printer device context.
   BITMAPINFOHEADER BitMapHeader;      //bitmap info header
   BYTE             BitMap[100]; //Array to hold bitmap data
   PBITMAPINFO      pBitMapInfo;       //bitmap info structure
   HBITMAP          ulSuccess;         //VAR  to hold  return code
   BYTE   Temp_buffer[1000];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreDeviceCreateBitmapRc - Entry.\n");
        ErrorCount = 0L;
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_HDC - Entry.\n");
        // Initalize the Bitmap information structure
        // first we need to  allocate memory to hold RGB array
        // since  BITMAPINFO structure is large enough to hold
        // one RGB structure.
        pBitMapInfo=(PBITMAPINFO)Temp_buffer;
        pBitMapInfo->cbFix=sizeof(BITMAPINFOHEADER); // This always =12
        pBitMapInfo->cx=16;                          //Width
        pBitMapInfo->cy=32;                          //height
        pBitMapInfo->cPlanes=1;                      // one color plane
        pBitMapInfo->cBitCount=1;                    // 1-bit /pixel
        pBitMapInfo->argbColor[0].bBlue=
        pBitMapInfo->argbColor[0].bGreen=
        pBitMapInfo->argbColor[0].bRed=0;
        pBitMapInfo->argbColor[1].bBlue=
        pBitMapInfo->argbColor[1].bGreen=
        pBitMapInfo->argbColor[1].bRed=0xFF;
        BitMapHeader.cbFix=sizeof(BITMAPINFOHEADER); // This always =12
        BitMapHeader.cx=16;                          //Width
        BitMapHeader.cy=32;                          //height
        BitMapHeader.cPlanes=1;                      // one color plane
        BitMapHeader.cBitCount=1;                    // 1-bit /pixel
        /*
         * The following Code will check for the PMERR_INV_HDC error Code
         */
        hDC= (HDC)1234;  // Set device context handle  invalid
        ulSuccess=GreDeviceCreateBitmap(hDC,&BitMapHeader,
                                       0L,BitMap,pBitMapInfo);
        ErrorCount += CHECKRETURNERROR(ulSuccess, "PMERR_INV_HDC ",
                                   SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_HDC - Exit.\n");
        /*
         * The following Code will check for the PMERR_INV_INFO_TABLE error Code
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_INFO_TABLE - Entry.\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){   /* Get  the default device context */
            BADERROR("GpiQueryDevice()",TRUE,"GreDeviceCreateBitmap");
            return;
        }
        // set pBitMapInfo->cbFix=0 ( Structure size must be 12 Bytes )
        pBitMapInfo->cbFix=0;
        ulSuccess=GreDeviceCreateBitmap(hDC,&BitMapHeader,
                                        CBM_INIT,BitMap,pBitMapInfo);
        ErrorCount += CHECKRETURNERROR(ulSuccess, " PMERR_INV_INFO_TABLE ",
                                  SEVERITY_ERROR, PMERR_INV_INFO_TABLE);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_INFO_TABLE - Exit.\n");
        /*
         * The following Code will check for the PMERR_INV_LENGTH_OR_COUNT
         * error Code
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_LENGTH_OR_COUNT - Entry.\n");
        // set pBitMapInfo->cbPlanes=meaningless value
                     //TSS was sizeof(BitMapHeader);
        pBitMapInfo->cbFix=sizeof(BITMAPINFO);

        BitMapHeader.cbFix=4444444L;
        ulSuccess=GreDeviceCreateBitmap(hDC,&BitMapHeader,
                              CBM_INIT,BitMap,pBitMapInfo);
        ErrorCount += CHECKRETURNERROR(ulSuccess, " PMERR_INV_LENGTH_OR_COUNT ",
                                   SEVERITY_ERROR,PMERR_INV_LENGTH_OR_COUNT );
        BitMapHeader.cbFix=sizeof(BitMapHeader);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_LENGTH_OR_COUNT - Exit.\n");
        /*
         * The following Code will check for the PMERR_INV_BITMAP_DIMENSION
         * error Code
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_BITMAP_DIMENSION - Entry.\n");
                     //TSS was sizeof(BitMapHeader);
        pBitMapInfo->cbFix=sizeof(BITMAPINFO);
        pBitMapInfo->cPlanes=1;
        // set BitMapHeader.cx=meaningless value
        BitMapHeader.cx=(USHORT)65536;
        ulSuccess=GreDeviceCreateBitmap(hDC,&BitMapHeader,
                              CBM_INIT,BitMap,pBitMapInfo);
        ErrorCount += CHECKRETURNERROR(ulSuccess, " PMERR_INV_BITMAP_DIMENSION item1 ",
                                   SEVERITY_ERROR,PMERR_INV_BITMAP_DIMENSION );
        BitMapHeader.cx=32;
        // set BitMapHeader.cy=meaningless value
        BitMapHeader.cy=(USHORT)65536;
        ulSuccess=GreDeviceCreateBitmap(hDC,&BitMapHeader,
                              CBM_INIT,BitMap,pBitMapInfo);
        ErrorCount += CHECKRETURNERROR(ulSuccess, " PMERR_INV_BITMAP_DIMENSION item2 ",
                                   SEVERITY_ERROR,PMERR_INV_BITMAP_DIMENSION );
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_BITMAP_DIMENSION - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreDeviceCreateBitmapRc                                        -    PASS.\n");
        }
        else {
             sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
             CWRITELOG(L_HDR, 0, achLogLine);
             CWRITELOG(L_HDR, 0,
"$GreDeviceCreateBitmapRc                                        -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceCreateBitmapRc - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSelectBitmapRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreDeviceSelectBitmap(). The syntax used to call the Gre is :
 *           (BOOL) GreDeviceSelectBitmap(
 *                       HDC               hdc,
 *                       ULONG             hbm ,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *   GreDeviceSelectBitmap() Inform the presentation driver that a new
 *    bit map is being selected into Dc .
 *   Errors That are returned by this function are :
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_INV_HDC
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/07/1990, pdvt ,Mike Abuzant
 *                -Test Case Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceSelectBitmapRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC        hDC;                          //Handle printer device context.
   HBITMAP    hBM;
   ULONG      uSuccess;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreDeviceSelectBitmapRc - Entry.\n");
        ErrorCount = 0L;
        /*
         * The following Code will check for the PMERR_INV_HDC error Code
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_HDC - Entry.\n");
        hDC=(HDC)7891;
        uSuccess=GreDeviceSelectBitmap(hDC,hBM);
        ErrorCount += CHECKRETURNERROR(uSuccess, " PMERR_INV_HDC ",SEVERITY_ERROR, PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_HDC - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /*
         * Return the proper log messages.
         */
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreDeviceSelectBitmapRc                                        -    PASS.\n");
        }
        else {
             sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
             CWRITELOG(L_HDR, 0, achLogLine);
             CWRITELOG(L_HDR, 0,
"$GreDeviceSelectBitmapRc                                        -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceSelectBitmapRc - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceDeleteBitmapRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreDeviceDeleteBitmap(). The syntax used to call the Gre is :
 *           (BOOL) GreDeviceDeleteBitmap(
 *                       HDC               hdc,          //Note: reserved
 *                       ULONG             hbm,
 *                       PDELETERETURN     pReturns,
 *                       ULONG             flOptions,
 *                       ULONG             pInstance,    // Note: reserved
 *                       ULONG             iFunction)
 *   GreDeviceDeleteBitmap() Destroys a bit map.
 *   Errors That are returned by this function are :
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_HDC
 *   PMERR_INV_INFO_TABLE
 *   PMERR_INV_SCAN_START
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/07/90,pdvt ,Mike Abuzant
 *                  -Test Case Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceDeleteBitmapRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC        hDC;              //Handle printer device context.
   BOOL       fSuccess;         //VAR  to hold  return code
   HBITMAP    hBM;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreDeviceDeleteBitmapRc - Entry.\n");
        ErrorCount = 0L;
        /*
         * The following Code will check for the PMERR_INV_HDC error Code
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_HDC - Entry.\n");
        hDC=(HDC)12456;
        fSuccess=GreDeviceDeleteBitmap(hDC,hBM, NULL ,NULL);
        ErrorCount += CHECKRETURNERROR(fSuccess, "PMERR_INV_HDC ", SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_HDC - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /*
         * Return the proper log messages.
         */
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreDeviceDeleteBitmapRc                                        -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreDeviceDeleteBitmapRc                                        -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceDeleteBitmapRc - Exit.\n");
       }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetPelRc()
 *
 * Purpose:  This function tests for all errors codes returned
 *           by GreSetPel(). The syntax used to call the Gre is :
 *           (LONG) GreSetPel(
 *                       HDC               hdc,
 *                       PPOINTL           pptPel,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *   GreSetPel() Sets a pel to the current line attribute,color,and mix.
 *               This function is subject to all usual clipping.
 *   Errors That are returned by this function are :
 *   PMERR_INV_IN_AREA
 *   PMERR_INV_IN_PATH
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_BITMAP_NOT_SELECTED
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_RECT
 *   PMERR_INV_COORDINATE
 *   PMERR_INV_HDC
 *   PMERR_INV_COORD_SPACE
 *   PMERR_INV_COLOR_DATA
 *   PMERR_INV_COLOR_INDEX
 *   PMERR_INV_PICK_APERTURE_POSN
 *   This function also checks for valid conditions.
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/07/90,pdvt ,Mike Abuzant
 *                  -Test Case Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *    CHECKRETURNERROR
 *    BADERROR
 *    CWRITELOG
 *    LOG_SIGNATURE
 *
 * Limitations:
 *    No error is returned when the point is clipped
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetPelRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   POINTL ptl;                          //Screen coordinate structure
   HDC    hDC;                          //Handle printer device context.
   LONG             lSuccess;
   HBITMAP          hBM=(HBITMAP)NULL ;
   BITMAPINFOHEADER BitMapHeader;      //bitmap info header

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreSetPelRc - Entry.\n");
        ErrorCount = 0L;
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_HDC - Entry.\n");
        hDC=(HDC)0;
        lSuccess=GreSetPel(hDC,&ptl);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_HDC - Exit.\n");


        /* ***************************************************************
         * The following Code will check for the PMERR_HDC_BUSY error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_HDC_BUSY - Entry.\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
             BADERROR("GpiQueryDevice()",TRUE,"GreSetPelRc");
             return;
        }

//TSS...create a bitmap to set.
     BitMapHeader.cbFix=sizeof(BitMapHeader);   // This always =12
     BitMapHeader.cx=120;                      //Width
     BitMapHeader.cy=72;                      //height
     BitMapHeader.cPlanes=1;                    // one color plane
     BitMapHeader.cBitCount=1;                  // 1-bit /pixel


     // Create and obtain a bitmap handle
     if ((hBM=GreDeviceCreateBitmap(hDC,&BitMapHeader, 0L,NULL,NULL))==GPI_ERROR) {
          REPORTSETUPERROR("GreDeviceCreateBitmap()");
          return;
     }


        ptl.x=1;
        ptl.y=1;

//TSS...added to force a busy state.

        if ( ! MAKEPSBUSY(hPS)) {
            CWRITELOG(L_FAIL,0,"GetBusy routine failed !\n");
        }

        lSuccess=GreSetPel(hDC,&ptl);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_HDC_BUSY ", SEVERITY_ERROR,PMERR_HDC_BUSY);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_HDC_BUSY - Exit.\n");

        FREEPS;

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /* Return the proper log messages.
        /********************************************************************/
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreSetPelRc                                                    -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreSetPelRc                                                    -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreSetPelRc - Exit.\n");
       }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetPelExh()
 *
 * Purpose:  This function c Exhaustively Test  the following
 *           gre functions:
 *           GreSetPel()
 *           GreGetPel()
 * System Requirements: 6.3.2
 *
 * Revision Log:
 *               11/23/1990 ,PDVT , Mike Abuzant
 *               - TestCase coding  and design
 *
 * Inputs:
 *    hps              // presentation space
 *    hWndClient       // client window
 *    SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *    NONE
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
\ ***************************************************************************/
VOID APIENTRY GreSetPelExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL   ptl;
  HDC      hDC;
  LONG     x_origin;
  LONG     y_origin;
  LONG     x;
  LONG     y;
  LONG     rc;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ GreSetPelExh Exhaustive Test - Entry.\n");
        ErrorCount = 0L;
        hDC = set_delta(hPS);

        x_origin = 4 * delta_x;
        y_origin = delta_y;
        for (x = 1; x < 1500; x +=5) {
             y = (((x+2)*(x+2))+30)/1000;
             ptl.x = x + x_origin;
             ptl.y = y + y_origin;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
             /*
              * get the original y value, i.e. unmap it
              * reflect the x value
              */
             ptl.y = y + y_origin;
             ptl.x = x_origin - x;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
        }
        for (x = 1; x < 1500; x +=5) {
             y = (((x+2)*(x+2))+30)/800;
             ptl.x = x + x_origin;
             ptl.y = y + y_origin;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
             /*
              * get the original y value, i.e. unmap it
              * reflect the x value
              */
             ptl.y = y + y_origin;
             ptl.x = x_origin - x;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
        }
        for (x = 1; x < 1500; x +=5) {
             y = (((x+2)*(x+2))+30)/600;
             ptl.x = x + x_origin;
             ptl.y = y + y_origin;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
             /*
              * get the original y value, i.e. unmap it
              * reflect the x value
              */
             ptl.y = y + y_origin;
             ptl.x = x_origin - x;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
        }
        for (x = 1; x < 1500; x +=5) {
             y = (((x+2)*(x+2))+30)/400;
             ptl.x = x + x_origin;
             ptl.y = y + y_origin;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
             /*
              * get the original y value, i.e. unmap it
              * reflect the x value
              */
             ptl.y = y + y_origin;
             ptl.x = x_origin - x;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
        }
        for (x = 1; x < 1500; x +=5) {
             y = (((x+2)*(x+2))+30)/200;
             ptl.x = x + x_origin;
             ptl.y = y + y_origin;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
             /*
              * get the original y value, i.e. unmap it
              * reflect the x value
              */
             ptl.y = y + y_origin;
             ptl.x = x_origin - x;
             MapPt(ptl);
             rc = GreSetPel(hDC,&ptl);
               if (rc == GPI_ERROR) {
                  ErrorCount += BADERROR("GreSetPel",FALSE,NULL);
               }
        }
        ptl.x = delta_x /4;
        ptl.y = delta_y - (delta_y /4);
        MapAndWriteString(hPS,ptl.x,ptl.y,"GreSetPelExh setting pixels on several parabolas.");
        /********************************************************************
        /* Return the proper log messages.
        /********************************************************************/
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreSetPelExh                                                   -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreSetPelExh                                                   -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreSetPelExh Exhaustive Test - Exit.\n");
       }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreGetPelRc()
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreGetPel(). The syntax used to call the Gre is :
 *           (LONG) GreGetPel(
 *                       HDC               hdc,
 *                       PPOINTL           pptPel,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *   GreGetPel() Returns the color of a pel at the position specified in
 *               world coordinates. The return value of this function
 *               is either the color index of the pel or it RGB value
 *               depending on the color mode of device.
 *   Errors That are returned by this function are :
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_BITMAP_NOT_SELECTED
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_RECT
 *   PMERR_INV_COORDINATE
 *   PMERR_INV_HDC
 *   PMERR_INV_COORD_SPACE
 *   PMERR_PEL_NOT_AVAILABLE
 *   PMERR_PEL_IS_CLIPPED
 *   This function also checks for valid conditions.
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/07/90,pdvt ,Mike Abuzant
 *                  -Test Case Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *             GreGetPel_test1()   //test for  all error codes that may
 *                                 // Be returned by GreSetPel().
 * Limitations:
 *  Return Codes Not Tested Yet:
 *  PMERR_HDC_BUSY
 *  PMERR_DEV_FUNC_NOT_INSTALLED
 *  PMERR_COORDINATE_OVERFLOW
 *  PMERR_BITMAP_NOT_SELECTED
 *  PMERR_INV_LENGTH_OR_COUNT
 *  PMERR_INV_RECT
 *  PMERR_INV_COORDINATE
 *  PMERR_INV_COORD_SPACE
 *  PMERR_PEL_NOT_AVAILABLE
 *  PMERR_PEL_IS_CLIPPED
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreGetPelRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   POINTL ptl;                          //Screen coordinate structure
   HDC    hDC;                          //Handle printer device context.
   HBITMAP          hBM=(HBITMAP)NULL ;
   LONG             lSuccess;
   HDC              hDC1;
   BITMAPINFOHEADER BitMapHeader;      //bitmap info header

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreGetPelRc - Entry.\n");
        ErrorCount = 0L;
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_HDC - Entry.\n");
        hDC=(HDC)0;
        ptl.x=200;
        ptl.y=200;
        lSuccess=GreGetPel(hDC,&ptl);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ", SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_HDC - Exit.\n");



        /* ***************************************************************
         * The following Code will check for the PMERR_HDC_BUSY error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_HDC_BUSY - Entry.\n");
        if ((hDC1=GpiQueryDevice(hPS))==HDC_ERROR) {   /* Get  the default device context */
             BADERROR("GpiQueryDevice()",TRUE,"GreGetPelRc");
             return;
        }

     BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
     BitMapHeader.cx=32;                  //Width
     BitMapHeader.cy=32;                //height
     BitMapHeader.cPlanes=1;                    // one color plane
     BitMapHeader.cBitCount=1;                  // 1-bit /pixel

     // Create and obtain a bitmap handle
     if ((hBM=GreDeviceCreateBitmap(hDC1,&BitMapHeader, 0L,NULL,NULL))==GPI_ERROR) {
          REPORTSETUPERROR("GreDeviceCreateBitmap()");
          return;
     }

          //tss was ptl.x = ptl.y = 0;
        ptl.x = 1;
        ptl.y = 1;

        if ( ! MAKEPSBUSY(hPS)) {
            CWRITELOG(L_FAIL,0,"GetBusy routine failed !\n");
        }

        lSuccess=GreGetPel(hDC1,&ptl);
        if (lSuccess==GPI_ERROR){
            ErrorCount += CHECKRETURNERROR(lSuccess, " PMERR_HDC_BUSY  ", SEVERITY_ERROR,PMERR_HDC_BUSY);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_HDC_BUSY - Exit.\n");
        FREEPS;

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /* Return the proper log messages.
        /********************************************************************/
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreGetPelRc                                                    -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreGetPelRc                                                    -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreGetPelRc - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreBitbltExh()
 *
 * Purpose:  This function exhaustively tests the functionality of
 *            The Gre function  GreBitblt().
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/08/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 *               11/08/90, pdvt ,Mike Abuzant
 *                  -Test Case Implemetation
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *            set_delta();
 * Limitations:
 *     Having no solid documentation ,i was led to believe that
 *    The GreBitblt() does not work properly when a bitmap handle is
 *    selected into either the Target Dc or the Source DC. Therefore;
 *    The GreBitblt() is not fully tested.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *    set_delta(hPS);
 *\\end
 ***************************************************************************/
VOID APIENTRY GreBitbltExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   POINTL ptl;                          //Screen coordinate point
   CHAR   out_string[257];
   LONG lIndex1;                        // used as an index into bitmap array
   HDC   hDC;
   POINTL aptl[4];
   static LONG abHello[]={0x00000000L,     // bitmap of the word HELLO
                         0x70445700L,
                         0x50445400L,
                         0x50447600L,
                         0x50445400L,
                         0x70775700L,
                         0x00000000L};
  INT cat_width=32;
  INT cat_height=32;
  LONG height;
  LONG width;
  LONG   xPixels;
  LONG   yPixels;
  static BYTE abCat[]={ 0x00,0xff,0xff,0x00,0x00,0xa2,0x45,0x00,
                        0x00,0xa2,0x45,0x00,0x00,0xa2,0x45,0xe0,
                        0x00,0xa2,0x45,0x10,0x00,0xa2,0x45,0x08,
                        0x00,0x9c,0x39,0x08,0x00,0xc0,0x03,0x04,

                        0x00,0x40,0x02,0x04,0x00,0x40,0x02,0x04,
                        0x00,0x40,0x02,0x04,0x00,0x20,0x04,0x04,
                        0x00,0x20,0x04,0x04,0x00,0x20,0x04,0x04,
                        0x00,0x10,0x08,0x04,0x00,0x10,0x08,0x08,

                        0x00,0x10,0x08,0x10,0x00,0x08,0x10,0x20,
                        0x00,0x08,0x10,0xc0,0x00,0x08,0x10,0x00,
                        0x00,0x07,0xe0,0x00,0x00,0x08,0x10,0x00,
                        0x00,0xfc,0x3f,0x00,0x00,0x09,0x90,0x00,

                        0x00,0xfc,0x3f,0x00,0x00,0x08,0x10,0x00,
                        0x00,0x1a,0x58,0x00,0x00,0x28,0x14,0x00,
                        0x00,0x48,0x12,0x00,0x00,0x8f,0xf1,0x00,
                        0x01,0x04,0x20,0x80,0x01,0xf8,0x1f,0x80};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ GreBitbltExh Exhaustive Test - Entry.\n");
        ErrorCount = 0L;
        set_delta(hPS);

    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){   /* Get  the default device context */
         BADERROR("GpiQueryDevice()",TRUE,"GreBitbltExh");
         return;
    }
    // Get device height
    if (DevQueryCaps(hDC,CAPS_HEIGHT,1L,&height)==FALSE) {
         BADERROR("DevQueryCaps()",TRUE,"GreBitbltExh");
         return;
    }
    // Get device width
    if (DevQueryCaps(hDC,CAPS_WIDTH,1L,&width)==FALSE) {
         BADERROR("DevQueryCaps()",TRUE,"GreBitbltExh");
         return;
    }
    // The width of grid cells in pixels;
    xPixels=(LONG)CONVERTTOINCHES(hori_pels_per_m);
    // The height  of grid cells in pixels;
    yPixels=(LONG)CONVERTTOINCHES(vert_pels_per_m);
    ptl.y=(LONG)(2*delta_y/3);
    ptl.x=0L;
    MapPt(ptl);
    if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
    }
    for (lIndex1=0;lIndex1<(sizeof(abCat)/sizeof(abCat[0])) ;lIndex1+=(cat_width/8)){
        if (GreImageData(hDC,&abCat[lIndex1],(LONG)cat_width, -(LONG)(lIndex1/8))==GPI_ERROR) {
            BADERROR("GreImageData()",TRUE,"GreBitbltExh");
            return;
        }
    }
      /* Setup Conditions to do bitmap manipulation by setting
       * the dimensions for the Target and Source rectangles.
       * Then use the GreBitblt() function with the above
       * settings. This segment will draw a bitmap of a cat
       * that will be drawn 7 times. Each time it will be
       * enlarged by one seventh the current DC width and in X-direction
       * and by on seventh the current Dc height in Y-direction.
      */
    // Source regtangle
    aptl[2].x=0L;
    aptl[2].y=(LONG)(2*yPixels/3);
    aptl[3].x=aptl[2].x+cat_width;
    aptl[3].y=aptl[2].y+cat_height;
    // Target regtangle
    aptl[0].x=cat_width;
    aptl[0].y=(LONG)(2*yPixels/3);
    aptl[1].x=xPixels+aptl[0].x;
    aptl[1].y=yPixels+aptl[0].y;
    for (lIndex1=0;lIndex1<=pg_size_x;lIndex1++){
         if (GreBitblt(hDC,hDC,4L,aptl,ROP_SRCCOPY,BBO_AND,NULL)==GPI_ERROR) {
             BADERROR("GreBitblt()",TRUE,"GreBitbltExh");
             return;
         }
         aptl[0].x=aptl[1].x;
         aptl[1].x=aptl[1].x+xPixels;
         aptl[1].y=aptl[1].y+yPixels;
    }
    ptl.x=(LONG)(pg_size_x-1)*delta_x;
    ptl.y=(LONG)(pg_size_y-1)*delta_y;
    MapPt(ptl);
    if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
    }
    for (lIndex1=0;lIndex1<(sizeof(abHello)/sizeof(abHello[0])) ;lIndex1++){
         if (GreImageData(hDC,&abHello[lIndex1],(LONG)sizeof(abHello[0])*8L,lIndex1)==GPI_ERROR) {
             BADERROR("GreImageData()",TRUE,"GreBitbltExh");
             return;
         }
    }
      /* Setup Conditions to do bitmap manipulation by setting
       * the dimensions for the Target and Source rectangles.
       * Then use the GreBitblt() function with the above
       * settings. This segment will draw a bitmap of the word 'Hello'
       * that will be drawn 7 times. Each time it will be
       * enlarged by one seventh the current DC width and in X-direction
       * and by on seventh the current Dc height in Y-direction.
      */
    // Source regtangle
    aptl[2].x=(LONG)CONVERTTOINCHES((pg_size_x-1)*hori_pels_per_m);
    aptl[3].y=(LONG)CONVERTTOINCHES((pg_size_y-1)*vert_pels_per_m);
    aptl[2].y=aptl[3].y-8L;
    aptl[3].x=sizeof(abHello[0])*8L+aptl[2].x;
    // Target regtangle
    aptl[1].x=aptl[2].x;
    aptl[0].x=aptl[1].x-xPixels;
    aptl[1].y=(LONG)CONVERTTOINCHES((pg_size_y-1)*vert_pels_per_m);
    aptl[0].y=(LONG)(aptl[1].y-yPixels/2);
    for (lIndex1=0;lIndex1<=pg_size_x ;lIndex1++){
         if (GreBitblt(hDC,hDC,4L,aptl,ROP_SRCCOPY,BBO_AND,NULL)==GPI_ERROR) {
             BADERROR("GreBitblt()",TRUE,"GreBitbltExh");
             return;
         }
         aptl[1].x=aptl[0].x;
         aptl[0].x=aptl[0].x-xPixels;
         aptl[0].y=(LONG)(aptl[0].y-yPixels/2);
    }
    sprintf(out_string,"Bitmap manipulation creating text and graphic images.");
    ptl.x = delta_x;
    ptl.y = 9 * delta_y + (delta_y / 2);
    MapAndWriteString(hPS,ptl.x,ptl.y,out_string);

        /*
         * Return the proper log messages.
         */
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreBitbltExh                                                   -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreBitbltExh                                                   -    FAIL.\n");
          }
          CWRITELOG(L_LOTRACE,0,"+ex+ GreBitbltExh Exhaustive Test - Exit.\n");
     }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreBitbltRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreBitblt(). The syntax used to call the Gre is :
 *           (LONG) GreBitblt(
 *                       HDC               hdc,
 *                       HDC               hdcSrc,
 *                       LONG              cPoint,
 *                       PPOINTL           paptPoints,
 *                       LONG              lRop,
 *                       ULONG             flOptions,
 *                       PBITBLTATTRS      pBattrs,
 *                       ULONG             pInstance,
 *                       ULONG             lFunction)
 *
 *   GreBitblt()   Modifies bit-map data at a target rectangle in the
 *                 current DC. The modifications can be either copying
 *                 a rectangle of data from a specified source DC to
 *                 the target or performing a raster operation on the
 *                 target.
 *   Errors That are returned by this function are :
 *   PMERR_BASE_ERROR
 *   PMERR_INV_IN_AREA
 *   PMERR_INV_IN_PATH
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_BITMAP_NOT_SELECTED
 *   PMERR_INV_PRIMITIVE_TYPE
 *   PMERR_INV_SETID
 *   PMERR_INV_SET_CHAR_ATTR
 *   PMERR_INV_MARKER_CHAR_ATTR
 *   PMERR_INV_PATTERN_CHAR_ATTR
 *   PMERR_INV_MARKER_SYMBOL_ATTR
 *   PMERR_UNSUPPORTED_ATTR
 *   PMERR_INV_MIX_ATTR
 *   PMERR_INV_BACKGROUND_MIX_ATTR
 *   PMERR_INV_PATTREN_REF_PT_ATTR
 *   PMERR_INV_LINE_WIDTH_ATTR
 *   PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *   PMERR_INV_LINE_TYPE_ATTR
 *   PMERR_INV_LINE_JOIN_ATTR
 *   PMERR_INV_LINE_END_ATTR
 *   PMERR_INV_CHAR_MODE_ATTR
 *   PMERR_INV_CHAR_SHEAR_ATTR
 *   PMERR_INV_CHAR_DIRECTION_ATTR
 *   PMERR_UNSUPPORTED_ATTR_VALUE
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_CODEPAGE
 *   PMERR_EXCEEDS_MAX_SEG_LENGTH
 *   PMERR_INV_RECT
 *   PMERR_INV_COORDINATE
 *   PMERR_INV_HDC
 *   PMERR_INV_HRGN
 *   PMERR_INV_DC_DATA
 *   PMERR_INCOMPATIBLE_BITMAP
 *   PMERR_INV_REGION_CONTROL
 *   PMERR_INV_DRIVER_NAME
 *   PMERR_INV_BITBLT_STYLE
 *   PMERR_INV_BITBLT_MIX
 *   PMERR_INV_HBITMAP
 *   PMERR_INV_DC_TYPE
 *   PMERR_INV_COORD_SPACE
 *   PMERR_INV_ID
 *   PMERR_BITMAP_IS_SELECTED
 *   PMERR_HBITMAP_BUSY
 *   PMERR_INV_USAGE_PARM
 *   PMERR_INV_INFO_TABLE
 *   PMERR_INV_BITMAP_DIMENSION
 *   PMERR_INCORRECT_DC_TYPE
 *   PMERR_REALIZE_NOT_SUPPORTED
 *   PMERR_INV_COLOR_FORMAT
 *   PMERR_INV_COLOR_DATA
 *   PMERR_INV_COLOR_START_INDEX
 *   PMERR_INV_COLOR_OPTIONS
 *   PMERR_INV_COLOR_INDEX
 *   PMERR_INV_SCAN_START
 *   PMERR_INV_PICK_APERTURE_POSN
 *   PMERR_INV_PATTERN_SET_FONT
 *   PMERR_HUGE_FONTS_NOT_SUPPORTED
 *   PMERR_INV_COLOR_ATTR
 *   PMERR_INV_BACKGROUND_COL_ATTR
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/08/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 * Limitations:
 *  Return Codes Successfully Tested:
 *      PMERR_INV_HDC
 *  Return Codes Not Tested Successfully Yet:
 *      PMERR_HUGE_FONTS_NOT_SUPPORTED
 *      PMERR_HDC_BUSY
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreBitbltRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC    hDC;                          //Handle printer device context.
   LONG             lSuccess;
   static POINTL           aptl[4]={{100,100},
                                    {200,200},
                                    {300,300},
                                    {400,400}};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreBitbltRc - Entry.\n");
        ErrorCount = 0L;
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_HDC - Entry.\n");
        hDC=(HDC)0;
        lSuccess=GreBitblt(hDC,NULL,2L,aptl,ROP_DSTINVERT,BBO_AND,NULL);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ", SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_HDC - Exit.\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_HDC_BUSY error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_HDC_BUSY - Entry.\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
            BADERROR("GpiQueryDevice()",TRUE,"GreBitbltRc");
            return;
        }
        if ( ! MAKEPSBUSY(hPS)) {
            CWRITELOG(L_FAIL,0,"GetBusy routine failed !\n");
        }
        lSuccess=GreBitblt(hDC,NULL,2L,aptl,ROP_DSTINVERT,BBO_AND,NULL);
        ErrorCount += CHECKRETURNERROR(lSuccess, " PMERR_HDC_BUSY  ", SEVERITY_ERROR,PMERR_HDC_BUSY);
        FREEPS;
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_HDC_BUSY - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /* Return the proper log messages.
        /********************************************************************/
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreBitbltRc                                                    -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreBitbltRc                                                    -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreBitbltRc - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreDrawBorderRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreDrawBorder(). The syntax used to call the Gre is :
 *           (BOOL) GreDrawBorder(
 *                       HDC               hdc,
 *                       PRECTL            prcFrame,
 *                       ULONG             cxBorder,
 *                       ULONG             cyBorder,
 *                       LONG              clrBorder,
 *                       LONG              clrInterior,
 *                       ULONG             flCmd,
 *                       ULONG             pInstance,
 *                       ULONG             lFunction)
 *   GreDrawBorder()  is a fast-frame function that draws an internal
 *                    in a rectangular frame. The interior can also
 *                    be drawn.
 *
 *   Errors That are returned by this function are :
 *   PMERR_BASE_ERROR
 *   PMERR_INV_IN_AREA
 *   PMERR_INV_IN_PATH
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_BITMAP_NOT_SELECTED
 *   PMERR_INV_PRIMITIVE_TYPE
 *   PMERR_INV_SETID
 *   PMERR_INV_SET_CHAR_ATTR
 *   PMERR_INV_MARKER_CHAR_ATTR
 *   PMERR_INV_PATTERN_CHAR_ATTR
 *   PMERR_INV_MARKER_SYMBOL_ATTR
 *   PMERR_UNSUPPORTED_ATTR
 *   PMERR_INV_MIX_ATTR
 *   PMERR_INV_BACKGROUND_MIX_ATTR
 *   PMERR_INV_PATTREN_REF_PT_ATTR
 *   PMERR_INV_LINE_WIDTH_ATTR
 *   PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *   PMERR_INV_LINE_TYPE_ATTR
 *   PMERR_INV_LINE_JOIN_ATTR
 *   PMERR_INV_LINE_END_ATTR
 *   PMERR_INV_CHAR_MODE_ATTR
 *   PMERR_INV_CHAR_SHEAR_ATTR
 *   PMERR_INV_CHAR_DIRECTION_ATTR
 *   PMERR_UNSUPPORTED_ATTR_VALUE
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_CODEPAGE
 *   PMERR_EXCEEDS_MAX_SEG_LENGTH
 *   PMERR_INV_RECT
 *   PMERR_INV_COORDINATE
 *   PMERR_INV_HDC
 *   PMERR_INV_HRGN
 *   PMERR_INV_DC_DATA
 *   PMERR_INCOMPATIBLE_BITMAP
 *   PMERR_INV_REGION_CONTROL
 *   PMERR_INV_DRIVER_NAME
 *   PMERR_INV_BITBLT_STYLE
 *   PMERR_INV_BITBLT_MIX
 *   PMERR_INV_HBITMAP
 *   PMERR_INV_DC_TYPE
 *   PMERR_INV_COORD_SPACE
 *   PMERR_INV_ID
 *   PMERR_BITMAP_IS_SELECTED
 *   PMERR_HBITMAP_BUSY
 *   PMERR_INV_USAGE_PARM
 *   PMERR_INV_INFO_TABLE
 *   PMERR_INV_BITMAP_DIMENSION
 *   PMERR_INCORRECT_DC_TYPE
 *   PMERR_REALIZE_NOT_SUPPORTED
 *   PMERR_INV_COLOR_FORMAT
 *   PMERR_INV_COLOR_DATA
 *   PMERR_INV_COLOR_START_INDEX
 *   PMERR_INV_COLOR_OPTIONS
 *   PMERR_INV_COLOR_INDEX
 *   PMERR_INV_DRAW_BORDER_OPTION
 *   PMERR_INV_SCAN_START
 *   PMERR_INV_PICK_APERTURE_POSN
 *   PMERR_INV_PATTERN_SET_FONT
 *   PMERR_INV_COLOR_ATTR
 *   PMERR_INV_BACKGROUND_COL_ATTR
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/06/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 *               11/28/90, pdvt ,Mike Abuzant
 *                  -Test Case Implementation
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *
 * Limitations:
 *   Return Codes Successfully Tested:
 *      PMERR_INV_HDC
 *   Return Codes Not Tested Successfully Yet:
 *      PMERR_HDC_BUSY
 *      PMERR_HUGE_FONTS_NOT_SUPPORTED
 *   This function is not supported by the print drivers. They should return
 *   Successful return code,However; Some of the print driver i have tested
 *   returned PMERR_DEVICE_DRIVER_ERROR2 instead of a successfull return.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDrawBorderRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
 static POINTL ptl={100,100};         //Screen coordinate point
 HDC   hDC;
 HDC   hDC1;
 LONG  lSuccess;
 HAB   hAB;
 RECTL rect;
  LONG lDCType;
  DEVOPENSTRUC   dop;
  DRIVDATA driv;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreDrawBorderRc - Entry.\n");
        ErrorCount = 0L;
        if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
             BADERROR("WinQueryAnchorBlock()",TRUE,"GreDrawBorderRc");
             return;
        }
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
/*
 * the following code is commented out because it
 * draws a border on the screen.
 * this entry point tests return codes only and should
 * produce no screen output
*/
//TSS...Put back in...I fail to see how this will create output.
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_HDC - Entry.\n");
        hDC=(HDC)0;
        rect.xLeft=100;
        rect.xRight=200;
        rect.yBottom=100;
        rect.yTop=200;
        lSuccess=GreDrawBorder(hDC,&rect,1L,1L,(COLOR)0L,(COLOR)0L,DB_AREAATTRS);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ", SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_HDC - Exit.\n");

        /* ***************************************************************
         * The following Code will check for the PMERR_HDC_BUSY error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_HDC_BUSY - Entry.\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {   /* Get  the default device context */
             BADERROR("GpiQueryDevice()",TRUE,"GreDrawBorderRc");
             return;
        }

        if ( ! MAKEPSBUSY(hPS)) {
            CWRITELOG(L_FAIL,0,"GetBusy routine failed !\n");
        }
        lSuccess=GreDrawBorder(hDC,&rect,3L,3L,(COLOR)0L,(COLOR)0L,DB_AREAATTRS);
        ErrorCount += CHECKRETURNERROR(lSuccess, " PMERR_HDC_BUSY  ", SEVERITY_ERROR,PMERR_HDC_BUSY);
        FREEPS;
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_HDC_BUSY - Exit.\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INCORRECT_DC_TYPE error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INCORRECT_DC_TYPE - Entry.\n");
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop), MPFROMP(&lDCType));
        hDC1= DevOpenDC(hAB,OD_METAFILE,"*",6L,(PDEVOPENDATA)&dop,NULL);

        lSuccess=GreDrawBorder(hDC1,&rect,3L,3L,(COLOR)0L,(COLOR)0L,DB_AREAATTRS);
        ErrorCount += CHECKRETURNERROR(lSuccess, " PMERR_INCORRECT_DC_TYPE  ", SEVERITY_ERROR,PMERR_INCORRECT_DC_TYPE);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INCORRECT_DC_TYPE - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /* Return the proper log messages.
        /********************************************************************/
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreDrawBorderRc                                                -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreDrawBorderRc                                                -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreDrawBorderRc - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 *
 * Routine Name: GreDrawBorderExh()
 *
 * Purpose:  This function exhaustively test the Gre function
 *           GreDrawBorder().
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               11/24/90, pdvt ,Mike Abuzant
 *                  -Test Case Design. and implementation
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *
 * Limitations:
 *   This function is not supported by the print drivers. They should return
 *   Successful return code,However; Some of the print driver i have tested
 *   returned PMERR_DEVICE_DRIVER_ERROR2 instead of a successfull return.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDrawBorderExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
 POINTL ptl;                          //Screen coordinate point
 HDC   hDC;
 static  INT color_index=0;           // color index
  static LONG  colors[]={ CLR_RED,     // Colors that will draw with
                           CLR_CYAN,
                           CLR_PINK,
                           CLR_BLUE,
                           CLR_BROWN,
                           CLR_YELLOW,
                           CLR_GREEN,
                           CLR_DARKCYAN,
                           CLR_DARKGREEN,
                           CLR_DARKBLUE};
  static INT pat_index=0;                      // pattern index
  LONG vthickness=0L;
  LONG hthickness=0L;
  BYTE  abImage[300];
  static LONG  pats[]={ PATSYM_DENSE8,
                        PATSYM_DIAG1,
                        PATSYM_DIAG4,
                        PATSYM_DENSE4,
                        PATSYM_VERT,
                        PATSYM_HORIZ,
                        };
   LONG lIndex1;                        // long Counter
   LONG  height;                         // height of device in pels
   LONG  width;                          // width of device in pels
   LONG  position=0L;
   RECTL  rect;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ GreDrawBorderExh Exhaustive Test - Entry.\n");
        ErrorCount = 0L;
        set_delta(hPS);

    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {   /* Get  the default device context */
         BADERROR("GpiQueryDevice()",TRUE,"GreDrawBorderExh");
         return;
    }
    if (DevQueryCaps(hDC,CAPS_HEIGHT,1L,&height)==FALSE) {
         BADERROR("DevQueryCaps()",TRUE,"GreDrawBorderExh");
         return;
    }
    // Get device width
    if (DevQueryCaps(hDC,CAPS_WIDTH,1L,&width)==FALSE) {
         BADERROR("DevQueryCaps()",TRUE,"GreDrawBorderExh");
         return;
     }
    /**************************************************************
     * This segment will draw Frames  have boarder of  the same
     * thickness using the Gre function GreDrawBorder()
     ***************************************************************/
    vthickness=
    hthickness=3;
    position=55;
    for (lIndex1=10 ;lIndex1<width ;lIndex1+=50){
        rect.xLeft=lIndex1;
        rect.xRight=lIndex1+47;
        rect.yBottom=position;
        rect.yTop=height/6+10;
        if (GreDrawBorder(hDC,&rect,hthickness,vthickness,(COLOR)0L,(COLOR)0L,DB_AREAATTRS)==FALSE) {
            BADERROR("GreDrawBorder()",TRUE,"GreDrawBorderExh");
            return;
        }
     }
    /**************************************************************
     * This segment will draw Frames  that have constant vertical thickness
     * but have a variable horizontal thickness using the Gre
     * function GreDrawBorder()
     ***************************************************************/
    vthickness=3;
    hthickness=2;
    position=(height/6+30);
    for (lIndex1=10 ;lIndex1<width ;lIndex1+=50){
        rect.xLeft=lIndex1;
        rect.xRight=lIndex1+47;
        rect.yBottom=position;
        rect.yTop=height*2/6;
        if (GreDrawBorder(hDC,&rect,(++hthickness)%15,vthickness,(COLOR)0L,(COLOR)0L,DB_AREAATTRS)==FALSE) {
            BADERROR("GreDrawBorder()",TRUE,"GreDrawBorderExh");
            return;
        }
     }
     sprintf(abImage,"Above Frames Have variable horizontal border Thickness " );
     ptl.x=(LONG)PIXELSTOUNITS(5,delta_x,hori_pels_per_m);
     ptl.y=(LONG)PIXELSTOUNITS(position-15L,delta_y,vert_pels_per_m) ;
     MapAndWriteString(hPS,ptl.x,ptl.y,abImage);
     /**************************************************************
     * This segment will draw Frames that have variable vertical thickness
     * but have constant horizontal thickness using the Gre
     * function GreDrawBorder()
     ***************************************************************/
     vthickness=3;
     hthickness=2;
     position=(height*2/6+30);
     for (lIndex1=10 ;lIndex1<width ;lIndex1+=50){
        rect.xLeft=lIndex1;
        rect.xRight=lIndex1+47;
        rect.yBottom=position;
        rect.yTop=height*3/6;
        if (GreDrawBorder(hDC,&rect,hthickness,(++vthickness)%15,
            (COLOR)0L,(COLOR)0L,DB_AREAATTRS)==FALSE) {
             BADERROR("GreDrawBorder()",TRUE,"GreDrawBorderExh");
             return;
        }
     }
     sprintf(abImage,"Above Frames Have variable vertical border Thickness " );
     ptl.x=(LONG)PIXELSTOUNITS(5,delta_x,hori_pels_per_m);
     ptl.y=(LONG)PIXELSTOUNITS(position-25L ,delta_y,vert_pels_per_m) ;
     MapAndWriteString(hPS,ptl.x,ptl.y,abImage);

     /**************************************************************
     * This segment will draw Frames  that have variable horizontal
     *  and variable vertical thickness.
     ***************************************************************/
     vthickness=2;
     hthickness=2;
     position=(height*3/6+30);
     for (lIndex1=10 ;lIndex1<width ;lIndex1+=50){
        rect.xLeft=lIndex1;
        rect.xRight=lIndex1+47;
        rect.yBottom=position;
        rect.yTop=height*4/6;
        if (GreDrawBorder(hDC,&rect,(++hthickness)%20,(++vthickness)%15,
            (COLOR)0L,(COLOR)0L,DB_AREAATTRS)==FALSE) {
            BADERROR("GreDrawBorder()",TRUE,"GreDrawBorderExh");
            return;
        }
     }
     sprintf(abImage,"Above Frames Have variable horizontal and variable vertical border Thickness " );
     ptl.x=(LONG)PIXELSTOUNITS(5,delta_x,hori_pels_per_m);
     ptl.y=(LONG)PIXELSTOUNITS(position-25L ,delta_y,vert_pels_per_m) ;
     MapAndWriteString(hPS,ptl.x,ptl.y,abImage);

     /**************************************************************
     * This segment will draw Frames  that have the border filled
     *  with the current pattern
     ***************************************************************/
     vthickness=
     hthickness=10;
     position=(height*4/6+30);

     color_index=0;
     pat_index=0;
     for (lIndex1=10 ;lIndex1<width ;lIndex1+=50){
         // Change the current  foreground color ...
         if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR) {
         }
         color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
         if (GpiSetPattern(hPS,pats[pat_index])==GPI_ERROR) {
         }
         pat_index=(++pat_index)%(sizeof(pats)/sizeof(pats[0]));
         rect.xLeft=lIndex1;
         rect.xRight=lIndex1+47;
         rect.yBottom=position;
         rect.yTop=height*5/6;
         if (GreDrawBorder(hDC,&rect,hthickness,vthickness,
             (COLOR)0L,(COLOR)0L,DB_AREAATTRS)==FALSE) {
              BADERROR("GreDrawBorder()",TRUE,"GreDrawBorderExh");
              return;
         }
     }
     sprintf(abImage,"Above Frames Have Filled border with current pattern");
     ptl.x=(LONG)PIXELSTOUNITS(5,delta_x,hori_pels_per_m);
     ptl.y=(LONG)PIXELSTOUNITS(position-25L ,delta_y,vert_pels_per_m) ;
     MapAndWriteString(hPS,ptl.x,ptl.y,abImage);

     /**************************************************************
     * This segment will draw Frames  that have the interior filled
     *  with the current pattern
     ***************************************************************/
     vthickness=
     hthickness=3;
     position=(height*5/6+15);
     color_index=0;
     pat_index=0;
     for (lIndex1=10 ;lIndex1<width ;lIndex1+=50){
          // Change the current  foreground color ...
          if (GpiSetBackColor(hPS,colors[color_index])==GPI_ERROR) {
          }
          color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
          if (GpiSetPattern(hPS,pats[pat_index])==GPI_ERROR) {
          }
          pat_index=(++pat_index)%(sizeof(pats)/sizeof(pats[0]));
          rect.xLeft=lIndex1;
          rect.xRight=lIndex1+47;
          rect.yBottom=position;
          rect.yTop=height;
          if (GreDrawBorder(hDC,&rect,hthickness,vthickness,(COLOR)0L,(COLOR)0L,DB_INTERIOR|DB_AREAATTRS)==FALSE) {
              BADERROR("GreDrawBorder()",TRUE,"GreDrawBorderExh");
              return;
          }
     }
     sprintf(abImage,"Above Frames Have Filled interior with current pattern");
     ptl.x=(LONG)PIXELSTOUNITS(5,delta_x,hori_pels_per_m);
     ptl.y=(LONG)PIXELSTOUNITS(position-13L ,delta_y,vert_pels_per_m) ;
     MapAndWriteString(hPS,ptl.x,ptl.y,abImage);

     // Set foreground color to  the System default color
     if (GpiSetColor(hPS,CLR_DEFAULT)==GPI_ERROR) {
     }
     // Set pattern to the System default pattern
     if (GpiSetPattern(hPS,PATSYM_DEFAULT)==GPI_ERROR) {
     }
     sprintf(abImage,"All Frames Will NOT be seen on ANY printer,"
                     " since print drivers don't support GreDrawBorder()");
     ptl.x=(LONG)PIXELSTOUNITS(5,delta_x,hori_pels_per_m);
     ptl.y=(LONG)PIXELSTOUNITS(60,delta_y,vert_pels_per_m) ;

        /*
         * Return the proper log messages.
         */
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreDrawBorderExh                                               -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreDrawBorderExh                                               -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreDrawBorderExh Exhaustive Test - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreImageDataRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreImageData(). The syntax used to call the Gre is :
 *           (LONG) GreImageData(
 *                       HDC               hdc,
 *                       PBYTE             pData,
 *                       LONG              cBits,
 *                       ULONG             offRow,
 *                       ULONG             pInstance,
 *                       ULONG             lFunction)
 *   GreImageData()   Draws a signal row of image data with one bit per pel
 *                    using the current image foreground and background color
 *                    and mix attributes.
 *   Errors That are returned by this function are :
 *   PMERR_INV_IN_AREA
 *   PMERR_INV_IN_PATH
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_BITMAP_NOT_SELECTED
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_RECT
 *   PMERR_INV_COORDINATE
 *   PMERR_INV_HDC
 *   PMERR_INV_REGION_CONTROL
 *   PMERR_INV_COORD_SPACE
 *   PMERR_INV_COLOR_DATA
 *   PMERR_INV_COLOR_INDEX
 *   PMERR_INV_SCAN_START
 *   PMERR_INV_PICK_APERTURE_POSN
 *   PMERR_INV_IMAGE_DATA_LENGTH
 *   This function also checks for valid conditions.
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/08/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreImageDataRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
 static POINTL ptl={100,100};         //Screen coordinate point
 BYTE  abImage[30];            // used to store bitmap image
 HDC   hDC;
 LONG  lSuccess;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreImageDataRc - Entry.\n");
        ErrorCount = 0L;
        // Call this function  to  Test For all error codes that
        // may be returned by GreImageData().
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INV_HDC - Entry.\n");
        hDC=(HDC)0;
        lSuccess=GreImageData(hDC,abImage,sizeof(abImage)*8L,0L);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ", SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INV_HDC - Exit.\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_HDC_BUSY error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_HDC_BUSY - Entry.\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
             BADERROR("GpiQueryDevice()",TRUE,"GreImageDataRc");
             return;
        }
        if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
        }
        memset(abImage,0xFF,sizeof(abImage));
        if ( ! MAKEPSBUSY(hPS)) {
            CWRITELOG(L_FAIL,0,"GetBusy routine failed !\n");
        }
        lSuccess=GreImageData(hDC,abImage,sizeof(abImage)*8L,0L);
        ErrorCount += CHECKRETURNERROR(lSuccess, " PMERR_HDC_BUSY  ", SEVERITY_ERROR,PMERR_HDC_BUSY);
        FREEPS;
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_HDC_BUSY - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /* Return the proper log messages.
        /********************************************************************/
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreImageDataRc                                                 -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreImageDataRc                                                 -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreImageDataRc - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreImageDataExh()
 *
 * Purpose:  This function exhaustively tests the functionalty of The Gre
 *           function GreImageData().
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/08/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 *               11/25/90, pdvt ,Mike Abuzant
 *                  -Test Case Implementation
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreImageDataExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
 POINTL ptl;                          //Screen coordinate point
 BYTE  abImage[IMAGESIZE];            // used to store bitmap image
 HDC   hDC;
 static INT color_index=0;                     // color index
  static LONG  colors[]={ CLR_RED,     // Colors that will draw with
                           CLR_CYAN,
                           CLR_PINK,
                           CLR_BLUE,
                           CLR_BROWN,
                           CLR_YELLOW,
                           CLR_GREEN,
                           CLR_DARKCYAN,
                           CLR_DARKGREEN,
                           CLR_DARKBLUE};
   LONG lIndex1;                        // long Counter
   LONG lIndex2;                        // long Counter
  LONG  height;                         // height of device in pels
  LONG  width;                          // width of device in pels
  INT shape_index=0;
  LONG  shift=0L;
  static BYTE abShape[]={0x3c,     // bitmap to be drawn
                         0x42,
                         0x81,
                         0x99,
                         0x99,
                         0x81,
                         0x42,
                         0x3c};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ GreImageDataExh Exhaustive Test - Entry.\n");
        ErrorCount = 0L;
        set_delta(hPS);

    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {   /* Get  the default device context */
         BADERROR("GpiQueryDevice()",TRUE,"GreImageDataExh");
         return;
    }
    // Get device height
    if (DevQueryCaps(hDC,CAPS_HEIGHT,1L,&height)==FALSE) {
         BADERROR("DevQueryCaps()",TRUE,"GreImageDataExh");
         return;
    }
    // Get device width
    if (DevQueryCaps(hDC,CAPS_WIDTH,1L,&width)==FALSE) {
         BADERROR("DevQueryCaps()",TRUE,"GreImageDataExh");
         return;
    }
    /**************************************************************
     * This segment will draw the image of a pattern
     * using the Gre function GreImageData().
    ****************************************************************/
    for (lIndex2=0;lIndex2<=10;lIndex2++){
         shape_index=0;
         ptl.x=delta_x/8;
         ptl.y=4*delta_y/3;
         MapPt(ptl);
         color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
         if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
         }
         for (lIndex1=0;(lIndex1<(height/3))||(shape_index!=0); lIndex1++){
             //Set up the image to draw
             memset(abImage,*(abShape+shape_index),(USHORT)(width/8));
             if (shape_index==0){
                if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR) {
                }
                color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
             }
             shape_index=(++shape_index)%(sizeof(abShape)/sizeof(abShape[0])) ;
             if (GreImageData(hDC,abImage,width,-lIndex1)==GPI_ERROR) {
                 BADERROR("GreImageData()",TRUE,"GreImageDataExh");
                 return;
             }
         }
    }
    if (GpiSetColor(hPS,CLR_DEFAULT)==GPI_ERROR) {
    }
    sprintf(abImage,"Above Images are drawn 10 Times starting at the same Location,");
    ptl.x=delta_x/8;
    ptl.y=delta_y;
    MapAndWriteString(hPS,ptl.x,ptl.y,abImage);
    sprintf(abImage,"therefore; It should looklike As if its been drawn Once.");
    ptl.x=delta_x/8;
    ptl.y=4*delta_y/5;
    MapAndWriteString(hPS,ptl.x,ptl.y,abImage);
    /**************************************************************
     * This segment will draw  lines the are 4 units vertically
     * a part and 4 units horizontally shifted.
    ****************************************************************/
    for (lIndex2=0;lIndex2<=10;lIndex2++){
        color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
        memset(abImage,0xFF,(USHORT)(width/8));
        ptl.y=MapY((pg_size_y-1)*delta_y);
        for (lIndex1=(height/3),shift=0;lIndex1>0;lIndex1-=4,shift+=4){
            //Set up the image to draw
            ptl.x=(LONG)MapX(delta_x/3+PIXELSTOUNITS(shift,delta_x,hori_pels_per_m));
            if (GpiMove(hPS,&ptl)==GPI_ERROR) {
            }
            if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR) {
            }
            color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
            if (GreImageData(hDC,abImage,30,shift)==GPI_ERROR) {
                 BADERROR("GreImageData()",TRUE,"GreImageDataExh");
                 return;
             }
        }
    }
    /**************************************************************
     * This segment will draw  lines their length is shortened
     *  by 4 units.
    ****************************************************************/
    for (lIndex2=0;lIndex2<=10;lIndex2++){
        memset(abImage,0xFF,(USHORT)(width/8));
        color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
        ptl.y=MapY((pg_size_y-1)*delta_y);
        for (lIndex1=(height*1/3),shift=0;lIndex1>0;lIndex1-=4,shift+=4){
             //Set up the image to draw
             ptl.x=(LONG)MapX((pg_size_x/2)*delta_x+PIXELSTOUNITS(shift,delta_x,hori_pels_per_m));
             if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
             }
             if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
             }
             if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR) {
             }
             color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
             if (GreImageData(hDC,abImage,(width/2)-(shift*2),shift)==GPI_ERROR) {
                 BADERROR("GreImageData()",TRUE,"GreImageDataExh");
                 return;
             }
        }
    }
    // Set foreground color to  the System default color
    if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR) {
    }

        /*
         * Return the proper log messages.
         */
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreImageDataExh                                                -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreImageDataExh                                                -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreImageDataExh Exhaustive Test - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreGetBitmapBitsRc()
 *
 * Purpose:  This function exhaustively tests for some errors codes returned
 *           by GreGetBitmapBits(). The syntax used to call the Gre is :
 *           (LONG) GreGetBitmapBits(
 *                       HDC               hdc,
 *                       HBITMAP           hbm,
 *                       LONG              iScanStart,
 *                       LONG              cScanCount,
 *                       PBYTE             pAddress,
 *                       PBITMAPINFO       pInfo,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *   GreGetBitmapBits() Transfers bit-map data from a bit map to application
 *                      storage.
 *   Errors That are returned by this function are :
 *   PMERR_INV_IN_AREA
 *   PMERR_INV_IN_PATH
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_BITMAP_NOT_SELECTED
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_HBITMAP
 *   PMERR_INV_INFO_TABLE
 *   PMERR_INCORRECT_DC_TYPE
 *   PMERR_INV_SCAN_START
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/06/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 *               11/28/90, pdvt ,Mike Abuzant
 *                  -Test Case Implementation
 *
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *
 * Limitations:
 *  Return Codes Not Tested Successfully Yet:
 *        NONE
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreGetBitmapBitsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
 BYTE  abImage[500];            // used to store bitmap image
 HDC   hDC;
 LONG  lSuccess;
 BITMAPINFO BitMapInfo;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreGetBitmapBitsRc - Entry.\n");
        ErrorCount = 0L;
        /* ***************************************************************
         * The following Code will check for the  error Code PMERR_INCORRECT_DC_TYPE
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INCORRECT_DC_TYPE - Entry.\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {   /* Get  the default device context */
             BADERROR("GpiQueryDevice()",TRUE,"GreGetBitmapBits");
             return;
        }

        BitMapInfo.cbFix = sizeof( BITMAPINFO );
        BitMapInfo.cx    = 100;
        BitMapInfo.cy    = 100;
        BitMapInfo.cPlanes = 1;
        BitMapInfo.cBitCount = 1;

                                      //TSS...pass in pointer to PBITMAPINFO instead of null
        lSuccess=GreGetBitmapBits(hDC,(HBITMAP)NULL,0L,2L,abImage,&BitMapInfo);
        ErrorCount += CHECKRETURNERROR(lSuccess, " PMERR_INCORRECT_DC_TYPE ",
                               SEVERITY_ERROR,PMERR_INCORRECT_DC_TYPE);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INCORRECT_DC_TYPE - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /* Return the proper log messages.
        /********************************************************************/
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreGetBitmapBitsRc                                             -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreGetBitmapBitsRc                                             -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreGetBitmapBitsRc - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetBitmapBitsExh()
 *
 * Purpose:  This function exhaustively tests the functionality
 *           of GreSetBitmapBits().
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/06/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 *               11/29/90, pdvt ,Mike Abuzant
 *                  -Test Case Implementation.
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient      -client window handle
 *        SelectionCall   -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 *            set_delta(hPS);
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetBitmapBitsExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{

   SIZEL            sizel;                         // size of bitmap to draw
   BYTE             abImage[IMAGESIZE];            // used to store bitmap image
   HDC              hDC;
   HAB              hAB;
   HBITMAP          hBM=(HBITMAP)NULL ;
   HDC              hDC1;
   HPS              hPS1;
   BYTE             Temp_buffer[100];
   BITMAPINFOHEADER BitMapHeader;      //bitmap info header
   PBITMAPINFO      pBitMapInfo;       //bitmap info structure
   POINTL           aptl[4];
   POINTL           ptl;
   static           INT color_index=0;                     // color index
   static LONG      colors[]={ CLR_RED,     // Colors that will draw with
                               CLR_CYAN,
                               CLR_PINK,
                               CLR_BLUE,
                               CLR_BROWN,
                               CLR_YELLOW,
                               CLR_GREEN,
                               CLR_DARKCYAN,
                               CLR_DARKGREEN,
                               CLR_DARKBLUE};
   LONG             lIndex1;                        // long Counter
   LONG             lIndex2;                        // long Counter
   LONG             lIndex3;                        // long Counter
   LONG              width;
   LONG              height;
#define CAT_WIDTH  32
#define CAT_HEIGHT 32
  static BYTE abCat[]={ 0x00,0xff,0xff,0x00,0x00,0xa2,0x45,0x00,
                        0x00,0xa2,0x45,0x00,0x00,0xa2,0x45,0xe0,
                        0x00,0xa2,0x45,0x10,0x00,0xa2,0x45,0x08,
                        0x00,0x9c,0x39,0x08,0x00,0xc0,0x03,0x04,

                        0x00,0x40,0x02,0x04,0x00,0x40,0x02,0x04,
                        0x00,0x40,0x02,0x04,0x00,0x20,0x04,0x04,
                        0x00,0x20,0x04,0x04,0x00,0x20,0x04,0x04,
                        0x00,0x10,0x08,0x04,0x00,0x10,0x08,0x08,

                        0x00,0x10,0x08,0x10,0x00,0x08,0x10,0x20,
                        0x00,0x08,0x10,0xc0,0x00,0x08,0x10,0x00,
                        0x00,0x07,0xe0,0x00,0x00,0x08,0x10,0x00,
                        0x00,0xfc,0x3f,0x00,0x00,0x09,0x90,0x00,

                        0x00,0xfc,0x3f,0x00,0x00,0x08,0x10,0x00,
                        0x00,0x1a,0x58,0x00,0x00,0x28,0x14,0x00,
                        0x00,0x48,0x12,0x00,0x00,0x8f,0xf1,0x00,
                        0x01,0x04,0x20,0x80,0x01,0xf8,0x1f,0x80};
#define  SHAPE_WIDTH  8
#define  SHAPE_HEIGHT 8
  static BYTE abShape[]={0x3c,0x00,0x00,0x00,
                         0x42,0x00,0x00,0x00,
                         0x81,0x00,0x00,0x00,
                         0x99,0x00,0x00,0x00,
                         0x99,0x00,0x00,0x00,
                         0x81,0x00,0x00,0x00,
                         0x42,0x00,0x00,0x00,
                         0x3c,0x00,0x00,0x00};
#define  CROSS_WIDTH  16
#define  CROSS_HEIGHT 16
    static BYTE abCross[]={0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,
                           0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,
                           0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,
                           0x01,0x00,0x00,0x00,0xff,0xff,0x00,0x00,
                           0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,
                           0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,
                           0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,
                           0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00};
  LONG lDCType;
  DEVOPENSTRUC   dop;
  DRIVDATA driv;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        TEST_PROGRESS ( 00, 15 );
        CWRITELOG(L_LOTRACE,0,"+ee+ GreSetBitmapBitsExh - Entry.\n");

  //TSS...8/1/91  added for clarity
        if( DeviceIsPrinter() ) {
           ptl.x = 0;
           ptl.y = delta_y * 4;
           MapPt( ptl );
           GpiCharStringAt( hPS, &ptl, 73L, "This test has no printed output due to limitations of GreSetBitmapBits()" );
           }

        ErrorCount = 0L;
        set_delta(hPS);

    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
         BADERROR("WinQueryAnchorBlock()",TRUE,"GreSetBitmapBits");
         return;
    }
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
         BADERROR("GpiQueryDevice()",TRUE,"GreSetBitmapBits");
         return;
     }
     // Initalize the Bitmap information structure
     // first we need to  allocate memory to hold RGB array
     // since  BITMAPINFO structure is large enough to hold
     // one RGB structure.
     /*
     pBitMapInfo=(PBITMAPINFO)Temp_buffer;
     */
     pBitMapInfo=malloc(sizeof(BITMAPINFO));
     pBitMapInfo->cbFix=sizeof(BitMapHeader);   // This always =12
     pBitMapInfo->cx=CROSS_WIDTH;                 //Width
     pBitMapInfo->cy=CROSS_HEIGHT;               //height
     pBitMapInfo->cPlanes=1;                      // one color plane
     pBitMapInfo->cBitCount=1;                    // 1-bit /pixel
     pBitMapInfo->argbColor[0].bBlue=
     pBitMapInfo->argbColor[0].bGreen=
     pBitMapInfo->argbColor[0].bRed=0;
     pBitMapInfo->argbColor[1].bBlue=
     pBitMapInfo->argbColor[1].bGreen=
     pBitMapInfo->argbColor[1].bRed=0xFF;
     BitMapHeader.cbFix=sizeof(BitMapHeader);    // This always =12
     BitMapHeader.cx=CAT_WIDTH;                  //Width
     BitMapHeader.cy=CAT_HEIGHT;                //height
     BitMapHeader.cPlanes=1;                    // one color plane
     BitMapHeader.cBitCount=1;                  // 1-bit /pixel
     sizel.cx=0;
     sizel.cy=0;
     // Open a Memory device context  to use for bitmap operations.
     dop.pdriv = &driv;
     WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop), MPFROMP(&lDCType));
     if ((hDC1=DevOpenDC(hAB,OD_MEMORY,"*",6L,(PDEVOPENDATA)&dop,
                        (HDC)NULL))==DEV_ERROR) {
         BADERROR("DevOpenDC()",TRUE,"GreSetBitmapBits");
         return;
     }
     // Create a presentation space to associate with the
     // memory device context
     if ((hPS1=GpiCreatePS(hAB,hDC1,&sizel,PU_PELS| GPIA_ASSOC|GPIT_MICRO))==GPI_ERROR) {
          REPORTSETUPERROR("GpiCreatePS()");
          return;
     }
     // Get device height
     if (DevQueryCaps(hDC1,CAPS_HEIGHT,1L,&height)==FALSE) {
          REPORTSETUPERROR("DevQueryCaps()");
          return;
     }
     // Get device width
     if (DevQueryCaps(hDC1,CAPS_WIDTH,1L,&width)==FALSE) {
          REPORTSETUPERROR("DevQueryCaps()");
          return;
     }
     // Create and obtain a bitmap handle to use with GreBitblt();
     if ((hBM=GreDeviceCreateBitmap(hDC1,&BitMapHeader, 0L,NULL,NULL))==GPI_ERROR) {
          REPORTSETUPERROR("GreDeviceCreateBitmap()");
          return;
     }
     //Set bitmap to use the memory device context .
     // Each bitmap created MUST belong to a device context.
     if (GreDeviceSelectBitmap(hDC1,hBM)==FALSE) {
          REPORTSETUPERROR("GreDeviceSelectBitmap()");
          return;
     }
     if (GreSetBitmapBits(hDC1,(HBITMAP)NULL,0L,CROSS_HEIGHT,abCross, pBitMapInfo)==GPI_ALTERROR) {
          REPORTSETUPERROR("GreSetBitmapBits()");
          return;
     }
     // Source regtangle
     for (lIndex3=0;lIndex3<5; lIndex3++){
          aptl[2].x=0;
          aptl[2].y=0;
          aptl[3].x=CROSS_WIDTH;
          aptl[3].y=CROSS_HEIGHT;
          for (lIndex1=50; lIndex1<(height/3) ;lIndex1+=CROSS_HEIGHT){
               // Set foreground color to a different color than default_color
               if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR) {
               }
               color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
               for (lIndex2=10; lIndex2<(width) ;lIndex2+=CROSS_WIDTH){
                    aptl[0].x=lIndex2;
                    aptl[0].y=lIndex1;
                    aptl[1].x=lIndex2+CROSS_WIDTH;
                    aptl[1].y=lIndex1+CROSS_HEIGHT;
                    if (GreBitblt(hDC,hDC1,3L,aptl,ROP_SRCCOPY,BBO_AND,NULL)==GPI_ERROR) {
                        REPORTSETUPERROR("GreBitblt()");
                        return;
                    }
                    if (GreGetBitmapBits(hDC1,(HBITMAP)NULL,0L,CROSS_HEIGHT,abImage, pBitMapInfo)==GPI_ALTERROR) {
                        REPORTSETUPERROR("GreGetBitmapBits()");
                        return;
                    }
                    if (memcmp(abImage,abCross,(1*CROSS_WIDTH +31)/32*1*4)!=0) {
                    }

               }
          }

          TEST_PROGRESS ( lIndex3 + 1, 15 );
     }
     pBitMapInfo->cx=SHAPE_WIDTH;                 //Width
     pBitMapInfo->cy=SHAPE_HEIGHT;               //height
     if (GreSetBitmapBits(hDC1,(HBITMAP)NULL,0L,SHAPE_HEIGHT,abShape, pBitMapInfo)==GPI_ALTERROR) {
          REPORTSETUPERROR("GreSetBitmapBits()");
          return;
     }
     for (lIndex3=0;lIndex3<5; lIndex3++){
          // Source regtangle
          aptl[2].x=0;
          aptl[2].y=0;
          aptl[3].x=SHAPE_WIDTH;
          aptl[3].y=SHAPE_HEIGHT;
          for (lIndex1=(height/3+10);lIndex1<(2*height/3-20) ;lIndex1+=SHAPE_HEIGHT){
               // Set foreground color to a different color than default_color
               if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR) {
               }
               color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
               for (lIndex2=10; lIndex2<(width) ;lIndex2+=SHAPE_WIDTH){
                    aptl[0].x=lIndex2;
                    aptl[0].y=lIndex1;
                    aptl[1].x=lIndex2+SHAPE_WIDTH;
                    aptl[1].y=lIndex1+SHAPE_HEIGHT;
                    if (GreBitblt(hDC,hDC1,3L,aptl,ROP_SRCCOPY,BBO_AND,NULL)==GPI_ERROR) {
                        REPORTSETUPERROR("GreBitblt()");
                        return;
                    }
                    /*
                    if (GreGetBitmapBits(hDC1,(HBITMAP)NULL,0L,SHAPE_HEIGHT,abImage, pBitMapInfo)==GPI_ALTERROR) {
                        REPORTSETUPERROR("GreGetBitmapBits()");
                        return;
                    }
                    if (memcmp(abImage,abShape,(1*SHAPE_WIDTH +31)/32*1*4)!=0) {
                    }
                    */
               }
          }

          TEST_PROGRESS ( lIndex3 + 6, 15 );
     }
     pBitMapInfo->cx=CAT_WIDTH;                 //Width
     pBitMapInfo->cy=CAT_HEIGHT;               //height
     if (GreSetBitmapBits(hDC1,(HBITMAP)NULL,0L,CAT_HEIGHT,abCat, pBitMapInfo)==GPI_ALTERROR) {
         REPORTSETUPERROR("GreSetBitmapBits()");
         return;
     }
     for (lIndex3=0;lIndex3<5; lIndex3++){
          // Source regtangle
          aptl[2].x=0;
          aptl[2].y=0;
          aptl[3].x=CAT_WIDTH;
          aptl[3].y=CAT_HEIGHT;
          for (lIndex1=(2*height/3); lIndex1<(height) ;lIndex1+=CAT_HEIGHT){
               // Set foreground color to a different color than default_color
               if (GpiSetColor(hPS,colors[color_index])==GPI_ERROR) {
               }
               color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
               for (lIndex2=10; lIndex2<(width) ;lIndex2+=CAT_WIDTH){
                    aptl[0].x=lIndex2;
                    aptl[0].y=lIndex1;
                    aptl[1].x=lIndex2+CAT_WIDTH;
                    aptl[1].y=lIndex1+CAT_HEIGHT;
                    if (GreBitblt(hDC,hDC1,3L,aptl,ROP_SRCCOPY,BBO_AND,NULL)==GPI_ERROR) {
                        REPORTSETUPERROR("GreBitblt()");
                        return;
                    }
                    if (GreGetBitmapBits(hDC1,(HBITMAP)NULL,0L,CAT_HEIGHT,abImage, pBitMapInfo)==GPI_ALTERROR) {
                        REPORTSETUPERROR("GreGetBitmapBits()");
                        return;
                    }
                    if (memcmp(abImage,abCat,(1*CAT_WIDTH +31)/32*1*4)!=0) {
                    }
               }
          }

          TEST_PROGRESS ( lIndex3 + 11, 15 );
     }


     if (GpiSetColor(hPS,CLR_DEFAULT)==GPI_ERROR) {
      }
      // De-select the bitmap handle. otherwise we get error when closing
      // device context
      if (GreDeviceSelectBitmap(hDC1,NULL)==FALSE) {
          REPORTSETUPERROR("GreDeviceSelectBitmap()");
          return;
      }
      // Delete the bitmap handle.
      // device context
      if (GreDeviceDeleteBitmap(hDC1,hBM,NULL,!CBM_INIT)==FALSE) {
          REPORTSETUPERROR("GreDeviceDeleteBitmap()");
          return;
      }
      // dispose of presentation space. No longer needed.
      if (GpiDestroyPS(hPS1)==FALSE) {
          REPORTSETUPERROR("GpiDestroyPS()");
          return;
      }
      //Close the device context
      if (DevCloseDC(hDC1)==DEV_ERROR) {
          REPORTSETUPERROR("DevCloseDC()");
          return;
      }

        /*
         * Return the proper log messages.
         */
        if (!ErrorCount) {
            CWRITELOG(L_HDR,0,
"$GreSetBitmapBitsExh                                            -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreSetBitmapBitsExh                                            -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreSetBitmapBitsExh - Exit.\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetBitmapBitsRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreSetBitmapBits(). The syntax used to call the Gre is :
 *           (LONG) GreSetBitmapBits(
 *                       HDC               hdc,
 *                       HBITMAP           hbm,
 *                       LONG              iScanStart,
 *                       LONG              cScanCount,
 *                       PBYTE             pAddress,
 *                       PBITMAPINFO       pInfo,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *   GreSetBitmapBits() Transfers bit-map data from application storage
 *                      into a specified bit map or DC.
 *   Errors That are returned by this function are :
 *   PMERR_INV_IN_AREA
 *   PMERR_INV_IN_PATH
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_BITMAP_NOT_SELECTED
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_HBITMAP
 *   PMERR_INV_INFO_TABLE
 *   PMERR_INCORRECT_DC_TYPE
 *   PMERR_INV_SCAN_START
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               11/06/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 *               11/29/90, pdvt ,Mike Abuzant
 *                  -Test Case Implementation
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *
 * Subroutines Required:
 * Limitations:
 *  Return Codes Successfully Tested:
 *      PMERR_INCORRECT_DC_TYPE
 *  Return Codes Not Tested Successfully Yet:
 *        NONE
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetBitmapBitsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
 BYTE  abImage[500];            // used to store bitmap image
 HDC   hDC;
 LONG  lSuccess;
 BITMAPINFO BitMapInfo;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreSetBitmapBitsRc - Entry.\n");
        ErrorCount = 0L;
        /* ***************************************************************
         * The following Code will check for the  error Code PMERR_INCORRECT_DC_TYPE
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ PMERR_INCORRECT_DC_TYPE - Entry.\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {   /* Get  the default device context */
             BADERROR("GpiQueryDevice()",TRUE,"GreSetBitmapBitsRc");
             return;
        }

        BitMapInfo.cbFix = sizeof( BITMAPINFO );
        BitMapInfo.cx    = 100;
        BitMapInfo.cy    = 100;
        BitMapInfo.cPlanes = 1;
        BitMapInfo.cBitCount = 1;

                                      //TSS...pass in pointer to PBITMAPINFO instead of null
        lSuccess=GreSetBitmapBits(hDC,(HBITMAP)NULL,0L,2L,abImage,&BitMapInfo);
        ErrorCount += CHECKRETURNERROR(lSuccess, " PMERR_INCORRECT_DC_TYPE ", SEVERITY_ERROR,PMERR_INCORRECT_DC_TYPE);
        CWRITELOG(L_LOTRACE,0,"+ix+ PMERR_INCORRECT_DC_TYPE - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /* Return the proper log messages.
        /********************************************************************/
        if (!ErrorCount) {
             CWRITELOG(L_HDR,0,
"$GreSetBitmapBitsRc                                             -    PASS.\n");
        }
        else {
              sprintf(achLogLine,"$ERRORCOUNT = %d.\n", ErrorCount);
              CWRITELOG(L_HDR, 0, achLogLine);
              CWRITELOG(L_HDR, 0,
"$GreSetBitmapBitsRc                                             -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreSetBitmapBitsRc - Exit.\n");
     }
}

