/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GRECOLR.C")
#pragma subtitle("IBM Confidential")

/*const char header=$header$*/

#define INCL_WIN
#define INCL_GPI
#define INCL_GREALL
#define INCL_ERRORS
#define INCL_DEV
#define INCL_GRE_COLORTABLE

#include <string.h>
#include <stdio.h>
#include <os2.h>
#include <pmddi.h>
#include "DTTDLLDF.h" // test case defines
#include "grecolr.h"  /* gre color dll include file */
#include "std_util.h"

/****************************************************************************
 *\\ddd
 * Subtask Name: pdvt
 *
 * Module name: grecolr.c
 *
 * Purpose: gre color dll test functions
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * SRS References: 6.10
 *
 * Contents:
 *     GreColorApp
 *     GreCreateLogColorTableRc
 *     GreQueryLogColorTableRc
 *     GreQueryColorDataRc
 *     GreQueryRealColorsRc
 *     GreQueryNearestColorRc
 *     GreQueryColorIndexRc
 *     GreQueryRGBColorRc
 *     GreRealizeColorTableRc
 *     GreUnRealizeColorTableRc
 *
 * Design Overview:  Test the GreColor program interface funtions for
 *                   respective error.
 *
 * Limitations:
 *   None.
 *
 *\\end
****************************************************************************/

/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose:  Initialize data structures for the DTT interface.
 *
 * System Requirements: None.
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    InitInfop     -   data structure for entry point names and addresses
 *    SelectionCall -   used by DTT
 *
 * Outputs:
 *    InitInfop     -   updated
 *
 * Subroutines Required:
 *    InitTestSegVars
 *
 * Limitations:
 *    None.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *  set number of testcase entries
 *  set group name
 *  for each testcase entrypoint
 *      set entrypoint name
 *      set entrypoint address
 *  call inittestsegvars
 *\\end
 ***************************************************************************/
VOID  APIENTRY InitTest(PVOID sel, PTESTINITINFO pInitInfo)
  {
    pInitInfo->ntests = NTESTS;
    strcpy(pInitInfo->group_name,pGroupName);
    strcpy(pInitInfo->test_names[0],pTestName0);
    pInitInfo->test_addrs[0]=GreColorApp;
    strcpy(pInitInfo->test_names[1],pTestName1);
    pInitInfo->test_addrs[1]=GreCreatLogColorTableRc;
    strcpy(pInitInfo->test_names[2],pTestName2);
    pInitInfo->test_addrs[2]=GreQueryLogColorTableRc;
    strcpy(pInitInfo->test_names[3],pTestName3);
    pInitInfo->test_addrs[3]=GreQueryColorDataRc;
    strcpy(pInitInfo->test_names[4],pTestName4);
    pInitInfo->test_addrs[4]=GreQueryRealColorsRc;
    strcpy(pInitInfo->test_names[5],pTestName5);
    pInitInfo->test_addrs[5]=GreQueryNearestColorRc;
    strcpy(pInitInfo->test_names[6],pTestName6);
    pInitInfo->test_addrs[6]=GreQueryColorIndexRc;
    strcpy(pInitInfo->test_names[7],pTestName7);
    pInitInfo->test_addrs[7]=GreQueryRGBColorRc;
    strcpy(pInitInfo->test_names[8],pTestName8);
    pInitInfo->test_addrs[8]=GreRealizeColorTableRc;
    strcpy(pInitInfo->test_names[9],pTestName9);
    pInitInfo->test_addrs[9]=GreUnRealizeColorTableRc;

    InitTestSegVars(sel);
  }
/****************************************************************************
 *\\ddd
 * Routine Name: GreColorApp()
 *
 * Purpose:  Show the GreColor interface functions working together in
 *           some sort of application scenario.  In this case, create a
 *           logical color table, realize the table, query the table and
 *           then unrealize the table.
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *   hPS           - presentation space handle
 *   hWndClient    - client window handle
 *   SelectionCall - boolean selection parameter
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *  create a logical color table
 *  realize the color table
 *  query the color table
 *  unrealize the color table
 *\\end
 ***************************************************************************/
VOID APIENTRY GreColorApp(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  BOOL         b_rc;            /* boolean return code */
  LONG         rc;              /* return code */
  ULONG        flOptions;       /* create options */
  LONG         lFormat;         /* format of entries in the table */
  LONG         iStart;          /* start index relevant for LCOLF_CONSECRGB */
  LONG         cCount;          /* number elements in application data area */
  PLONG        pData;           /* application data area */
  LONG         cArray;          /* number of elements supplied */
  PLONG        pArray;          /* pointer to array on return */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreColorApp - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC",TRUE,"GreColorApp");
           return;
        }

        /*
         * call GreCreateLogColorTable with good parameters expecting
         * pmerr_ok response
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ - GreColorApp - Entry.\n");
        flOptions = LCOL_RESET;
        lFormat   = LCOLF_INDRGB;
        iStart    = 0;
        cCount    = 0;
        pData     = 0;
        b_rc = GreCreateLogColorTable(printer_hdc,flOptions,lFormat,iStart,cCount,pData);
        if (b_rc == FALSE) {
           error_count += BADERROR("GreCreateLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ - GreColorApp - Exit.\n");
        /*
         * realize the newly created color table
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ - GreColorApp - Entry.\n");
        b_rc = GreRealizeColorTable(printer_hdc);
        if (b_rc == FALSE) {
           error_count += BADERROR("GreRealizeLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ - GreColorApp - Exit.\n");
        /*
         * call grequerylogcolortable with valid parameters expecting
         * pmerr_ok returned
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ - GreColorApp - Entry.\n");
        flOptions = LCOLOPT_INDEX;
        cArray    = 0L;
        pArray    = 0;
        iStart    = 0L;
        rc = GreQueryLogColorTable(printer_hdc,flOptions,iStart,cArray,pArray);
        if (rc == QLCT_ERROR) {
           error_count += BADERROR("GreQueryLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ - GreColorApp - Exit.\n");
        CWRITELOG(L_LOTRACE,0,"+ie+ - GreColorApp - Entry.\n");
        b_rc = GreUnrealizeColorTable(printer_hdc);
        if (b_rc == FALSE) {
           error_count += BADERROR("GreUnRealizeLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ - GreColorApp - Exit.\n");

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreColorApp                                                    -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreColorApp                                                    -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreColorApp - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreCreatLogColorTableRc()
 *
 * Purpose:  Test the GreCreateLogColorTable interface.  Test for not only
 *           successful return but invalid parameter options.
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    hPS           - presentation space handle
 *    hWndClient    - client window handle
 *    SelectionCall - selection parameter from DTT
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *   create the logical color table using default parameters
 *   create a color table with invalid options expecting failure
 *\\end
 ***************************************************************************/
VOID APIENTRY GreCreatLogColorTableRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  BOOL         b_rc;            /* boolean return code */
  ULONG        flOptions;       /* create options */
  LONG         lFormat;         /* format of entries in the table */
  LONG         iStart;          /* start index relevant for LCOLF_CONSECRGB */
  LONG         cCount;          /* number elements in application data area */
  PLONG        pData;           /* application data area */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreCreateLogColorTableRc - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC",TRUE,"GreCreateLogColorTableRc");
           return;
        }
        /*
         * call GreCreateLogColorTable with good parameters expecting
         * pmerr_ok response
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateLogColorTable - Entry.\n");
        flOptions = LCOL_RESET;
        lFormat   = LCOLF_INDRGB;
        iStart    = 0;
        cCount    = 0;
        pData     = 0;
        b_rc = GreCreateLogColorTable(printer_hdc,flOptions,lFormat,iStart,cCount,pData);
        if (b_rc == FALSE) {
           error_count += BADERROR("GreCreateLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateLogColorTable - Exit.\n");
        /*
         * call grecreatelogcolortable with invalid format option
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateLogColorTable - Entry.\n");
        flOptions = (ULONG)"none_defined";
        lFormat   = (ULONG)"none_defined";
        iStart    = 0L;
        cCount    = 0L;
        pData     = 0;
        b_rc = GreCreateLogColorTable(printer_hdc,flOptions,lFormat,iStart,cCount,pData);
        if (b_rc == FALSE)
           error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_COLOR_FORMAT",SEVERITY_ERROR,PMERR_INV_COLOR_FORMAT);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateLogColorTable - Exit.\n");

        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateLogColorTable - Entry.\n");
        b_rc = GreCreateLogColorTable((HDC)0,flOptions,lFormat,iStart,cCount,pData);
        if (b_rc == FALSE)
           error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateLogColorTable - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreCreateLogColorTable                                         -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreCreateLogColorTable                                         -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreCreateLogColorTableRc - Exit.\n");
  }
  return;
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryLogColorTableRc()
 *
 * Purpose:
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    hPS           - presentation space handle
 *    hWndClient    - client window handle
 *    SelectionCall - selection parameter from DTT
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *   create a logical color table and attempt to query this table using
 *       default parameters expecting success
 *   query the table with invalid parameters expecting failure
 *\\end
 ***************************************************************************/

VOID APIENTRY GreQueryLogColorTableRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  ULONG        flOptions;       /* query options */
  LONG         iStart;          /* starting index for stored data */
  LONG         cArray;          /* number of elements supplied */
  PLONG        pArray;          /* pointer to array on return */
  LONG         rc;              /* return code */
  BOOL         b_rc;            /* return code */
  LONG         cCount;          /* number elements in application data area */
  PLONG        pData;           /* application data area */
  LONG         lFormat;         /* format of entries in the table */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreQueryLogColorTableRc - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC",TRUE,"GreQueryLogColorTableRc");
           return;
        }
        /*
         * call GreCreateLogColorTable with good parameters expecting
         * pmerr_ok response
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryLogColorTable - Entry.\n");
        flOptions = LCOL_RESET;
        lFormat   = LCOLF_INDRGB;
        iStart    = 0;
        cCount    = 0;
        pData     = 0;
        b_rc = GreCreateLogColorTable(printer_hdc,flOptions,lFormat,iStart,cCount,pData);
        if (b_rc == FALSE) {
           error_count += BADERROR("GreQueryLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryLogColorTable - Exit.\n");
        /*
         * call grequerylogcolortable with valid parameters expecting
         * pmerr_ok returned
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryLogColorTable - Entry.\n");
        flOptions = LCOLOPT_INDEX;
        cArray    = 0L;
        pArray    = 0;
        iStart    = 0L;
        rc = GreQueryLogColorTable(printer_hdc,flOptions,iStart,cArray,pArray);
        if (rc == QLCT_ERROR) {
           error_count += BADERROR("GreQueryLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryLogColorTable - Exit.\n");
        /*
         * call grequerylogcolortable with invalid option, other than LCOLOPT_INDEX
         * it is invalid
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryLogColorTable - Entry.\n");
        rc = GreQueryLogColorTable(printer_hdc,(ULONG)62,0L,0L,0);
        if (rc == QLCT_ERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_COLOR_OPTIONS",SEVERITY_ERROR,PMERR_INV_COLOR_OPTIONS);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryLogColorTable - Exit.\n");

        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryLogColorTable - Entry.\n");
        rc = GreQueryLogColorTable((HDC)0,(ULONG)62,0L,0L,0);
        if (rc == QLCT_ERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryLogColorTable - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryLogColorTable                                          -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryLogColorTable                                          -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreQueryLogColorTableRc - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryColorDataRc()
 *
 * Purpose:
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    hPS           - presentation space handle
 *    hWndClient    - client window handle
 *    SelectionCall - selection parameter from DTT
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *   query the color data for the printer device expecting success
 *   query the color data with invalid parameters expecting failure
 *\\end
 ***************************************************************************/

VOID APIENTRY GreQueryColorDataRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  HDC          bad_hdc;
  BOOL         b_rc;            /* return code */
  LONG         cArray;          /* number of elements in Array */
  LONG         pArray;          /* pointer to Array on return */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreQueryColorDataRc - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC",TRUE,"GreQueryColorDataRc");
           return;
        }

        /*
         * call grequerycolordata with valid parameters expecting
         * pmerr_ok return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorData - Entry.\n");
        cArray = 0L;
                                             //TSS..pass address of cArray & pArray
        b_rc = GreQueryColorData(printer_hdc,cArray,&pArray);
        if (b_rc == FALSE)   {
           error_count += 1L;
           CWRITELOG(L_TRACE,0,"GreQueryColorDataRc - GreQueryColorData returned FALSE.        - FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorData - Exit.\n");
        /*
         * try grequerycolordata with a     hdc expecting error return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorData - Entry.\n");

                                         //TSS...changed 0L,0L to variables defined as 0L
                                         //      change 0 for hdc to bad_hdc
        bad_hdc = 0;
        b_rc = GreQueryColorData(&bad_hdc,cArray,&pArray);
        if (b_rc == FALSE)
           error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorData - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryColorData                                              -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryColorData                                              -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreQueryColorDataRc - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryRealColorsRc()
 *
 * Purpose:
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    hPS           - presentation space handle
 *    hWndClient    - client window handle
 *    SelectionCall - selection parameter from DTT
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *   query real colors for the printer device expecting success
 *   query real colors with invalid options expecting failure
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryRealColorsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  LONG         rc;              /* return code */
  ULONG        flOptions;       /* query real color options */
  LONG         iStart;          /* ordinal number of first color required */
  LONG         cArray;          /* elements in Array */
  PLONG        pArray;          /* data Array on return */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreQueryRealColorsRc - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC",TRUE,"GreRealColorsRc");
           return;
        }
        /*
         * call query with realized parameter expecting successful return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRealColors - Entry.\n");
        flOptions = LCOLOPT_REALIZED;
        iStart = 0;
        cArray = 0;
        pArray = 0;
        rc = GreQueryRealColors(printer_hdc,flOptions,iStart,cArray,pArray);
        if (rc == GPI_ALTERROR) {
           if (rc <= 0) {
              error_count += 1;
              CWRITELOG(L_TRACE,0,"$GreQueryRealColorsRc - Returned zero real colors unexpectedly. -    FAIL\n");
           }
           else error_count += CHECKRETURNERROR(rc,"QueryRealColors failed unexpectedly",SEVERITY_ERROR,PMERR_OK);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRealColors - Exit.\n");
        /*
         * call grequeryrealcolors with valid parameters expecting good
         * return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRealColors - Entry.\n");
        flOptions = LCOLOPT_INDEX;
        iStart = 0;
        cArray = 0;
        pArray = 0;
        rc = GreQueryRealColors(printer_hdc,flOptions,iStart,cArray,pArray);
        if (rc == GPI_ALTERROR) {
           if (rc <= 0)  {
              error_count += 1;
              CWRITELOG(L_TRACE,0,"$GreQueryRealColorsRc - Returned zero real colors unexpectedly. -    FAIL\n");
              if (pArray == 0) {
                 sprintf(out_string,"$GreQueryRealColorsRc - The Color Table is empty.               -    FAIL\n");
                 CWRITELOG(L_TRACE,0,out_string);
              }
           }
           else error_count += CHECKRETURNERROR(rc,"QueryRealColors failed unexpectedly",SEVERITY_ERROR,PMERR_OK);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRealColors - Exit.\n");
        /*
         * call grequeryrealcolors with a     options expecting error
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRealColors - Entry.\n");
        rc = GreQueryRealColors(printer_hdc,(ULONG)62,(ULONG)0,(ULONG)0,(LONG)0);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_COLOR_OPTIONS",SEVERITY_ERROR,PMERR_INV_COLOR_OPTIONS);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRealColors - Exit.\n");
        /*
         * call grequeryrealcolors with a     device con+ex+ expecting error
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRealColors - Entry.\n");
        rc = GreQueryRealColors((HDC)0L,(ULONG)0,(ULONG)0,(ULONG)0,(LONG)0);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRealColors - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryRealColors                                             -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryRealColors                                             -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreQueryRealColorsRc - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryNearestColorRc()
 *
 * Purpose:
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    hPS           - presentation space handle
 *    hWndClient    - client window handle
 *    SelectionCall - selection parameter from DTT
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *   query the nearest color to both -1 and 0xFFFFFFFF, neither are true colors
 *   query again with invalid options expecting failure
 *\\end
 ***************************************************************************/

VOID APIENTRY GreQueryNearestColorRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  ULONG        flOptions;       /* valid options parameter */
  LONG         rgbColorIn;      /* color required */
  LONG         rc;              /* return code */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreQueryNearestColorRc - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC",TRUE,"GreQueryNearestColorRc");
           return;
        }
        /*
         * call the function with valid parameters expecting good return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryNearestColor - Entry.\n");
        flOptions = LCOLOPT_REALIZED;
        rgbColorIn = 0L;
        rc = GreQueryNearestColor(printer_hdc,flOptions,rgbColorIn);
        if (rc == GPI_ALTERROR) {
           error_count += BADERROR("GreQueryNearestColor",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryNearestColor - Exit.\n");
        /*
         * call grequerynearestcolor with     color in expecting error
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryNearestColor - Entry.\n");
        flOptions = 0L;
        rgbColorIn = (LONG)-1;   /* invalid color in */
        rc = GreQueryNearestColor(printer_hdc,flOptions,rgbColorIn);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_RGBCOLOR",SEVERITY_ERROR,PMERR_INV_RGBCOLOR);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryNearestColor - Exit.\n");
        /*
         * call grequerynearestcolor with     options  expecting error
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryNearestColor - Entry.\n");
        flOptions = 0xFFFFFFFFL;
        rgbColorIn = (LONG)"no_color_in";
        rc = GreQueryNearestColor(printer_hdc,flOptions,rgbColorIn);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_COLOR_OPTIONS",SEVERITY_ERROR,PMERR_INV_COLOR_OPTIONS);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryNearestColor - Exit.\n");

        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryNearestColor - Entry.\n");
        rc = GreQueryNearestColor((HDC)0,flOptions,rgbColorIn);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryNearestColor - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryNearestColor                                           -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryNearestColor                                           -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreQueryNearestColorRc - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryColorIndexRc()
 *
 * Purpose:
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    hPS           - presentation space handle
 *    hWndClient    - client window handle
 *    SelectionCall - selection parameter from DTT
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *   query the index of a valid rgb color expecting success
 *   query for an invalid rgb color expecting failure
 *   query with invalid parameters expecting failure
 *\\end
 ***************************************************************************/

VOID APIENTRY GreQueryColorIndexRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  ULONG        flOptions;       /* valid options parameter */
  LONG         rgbColor;        /* RGB valued color */
  LONG         rc;              /* return code */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreQueryColorIndexRc - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC",TRUE,"GreQueryColorIndexRc");
           return;
        }
        /*
         * call querycolorindex with valid parameters expecting
         * good return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorIndex - Entry.\n");
        flOptions = LCOLOPT_REALIZED;
        rgbColor  = RGB_BLACK;
        rc = GreQueryColorIndex(printer_hdc,flOptions,rgbColor);
        if (rc == GPI_ALTERROR) {
           error_count += BADERROR("GreQueryColorIndex",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorIndex - Exit.\n");
        /*
         * call grequerycolorindex with invalid rgb color expecting error
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorIndex - Entry.\n");
        flOptions = LCOLOPT_REALIZED;
        rgbColor  = 0xFFFFFFFFL;   /* not defined as an rgb color */
        rc = GreQueryColorIndex(printer_hdc,flOptions,rgbColor);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_RGBCOLOR",SEVERITY_ERROR,PMERR_INV_RGBCOLOR);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorIndex - Exit.\n");
        /*
         * call grequerycolorindex with invalid options expecting error
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorIndex - Entry.\n");
        flOptions = 0xFFFFFFFFL;
        rgbColor  = RGB_BLACK;   /* not defined as an rgb color */
        rc = GreQueryColorIndex(printer_hdc,flOptions,rgbColor);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_COLOR_OPTIONS",SEVERITY_ERROR,PMERR_INV_COLOR_OPTIONS);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorIndex - Exit.\n");
        /*
         * call grequerycolorindex with invalid hdc expecting error
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorIndex - Entry.\n");
        flOptions = LCOLOPT_REALIZED;
        rgbColor  = RGB_BLACK;   /* not defined as an rgb color */
        rc = GreQueryColorIndex((HDC)0L,flOptions,rgbColor);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryColorIndex - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryColorIndex                                             -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryColorIndex                                             -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreQueryColorIndexRc - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryRGBColorRc()
 *
 * Purpose:
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    hPS           - presentation space handle
 *    hWndClient    - client window handle
 *    SelectionCall - selection parameter from DTT
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *   query a valid rgb color expecting success
 *   query both invalid colors and options expecting failure
 *\\end
 ***************************************************************************/

VOID APIENTRY GreQueryRGBColorRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  ULONG        flOptions;       /* valid options parameter */
  LONG         iColor;          /* color index */
  LONG         rc;              /* return code */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreQueryRGBColorRc - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC",TRUE,"GreQueryRGBColorRc");
           return;
        }
        /*
         * call grequeryrgbcolor with good parameters expecting good return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRGBColor - Entry.\n");
        flOptions = LCOLOPT_REALIZED;
        iColor    = 0L;
        rc = GreQueryRGBColor(printer_hdc,flOptions,iColor);
        if (rc == GPI_ALTERROR) {
           error_count += BADERROR("GreQueryRGBColor",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRGBColor - Exit.\n");
        /*
         * call grequeryrgbcolor with good parameters expecting good return
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRGBColor - Entry.\n");
        flOptions = LCOLOPT_INDEX;
        iColor    = 0L;
        rc = GreQueryRGBColor(printer_hdc,flOptions,iColor);
        if (rc == GPI_ALTERROR) {
           error_count += BADERROR("GreQueryRGBColor",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRGBColor - Exit.\n");
        /*
         * call grequeryrgbcolor with
         */
        flOptions = 0xFFFFFFFFL;
        iColor    = 0L;
        rc = GreQueryRGBColor(printer_hdc,flOptions,iColor);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_COLOR_OPTIONS",SEVERITY_ERROR,PMERR_INV_COLOR_OPTIONS);
        /*
         * call grequeryrgbcolor with
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRGBColor - Entry.\n");
        flOptions = LCOLOPT_REALIZED;
        iColor    = 0xFFFFFFFFL;
        rc = GreQueryRGBColor(printer_hdc,flOptions,iColor);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_COLOR_INDEX",SEVERITY_ERROR,PMERR_INV_COLOR_INDEX);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRGBColor - Exit.\n");

        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRGBColor - Entry.\n");
        rc = GreQueryRGBColor((HDC)0,flOptions,iColor);
        if (rc == GPI_ALTERROR)
           error_count += CHECKRETURNERROR(rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryRGBColor - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryRGBColor                                               -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryRGBColor                                               -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreQueryRGBColorRc - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreRealizeColorTableRc()
 *
 * Purpose:
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    hPS           - presentation space handle
 *    hWndClient    - client window handle
 *    SelectionCall - selection parameter from DTT
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *   create and realize a logical color table
 *\\end
 ***************************************************************************/

VOID APIENTRY GreRealizeColorTableRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  BOOL         b_rc;            /* boolean return code */
  ULONG        flOptions;       /* create options */
  LONG         lFormat;         /* format of entries in the table */
  LONG         iStart;          /* start index relevant for LCOLF_CONSECRGB */
  LONG         cCount;          /* number elements in application data area */
  PLONG        pData;           /* application data area */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreRealizeColorTableRc - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR ("DevOpenDC",TRUE,"GreRealizeColorTableRc");
           return;
        }
        /*
         * call GreCreateLogColorTable with good parameters expecting
         * pmerr_ok response
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateLogColorTable - Entry.\n");
        flOptions = LCOL_RESET;
        lFormat   = LCOLF_INDRGB;
        iStart    = 0;
        cCount    = 0;
        pData     = 0;
        b_rc = GreCreateLogColorTable(printer_hdc,flOptions,lFormat,iStart,cCount,pData);
        if (b_rc == FALSE) {
           error_count += BADERROR ("GreCreateLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateLogColorTable - Exit.\n");
        /*
         * realize the newly created color table
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreRealizeColorTable - Entry.\n");
        b_rc = GreRealizeColorTable(printer_hdc);
        if (b_rc == FALSE) {
           error_count += BADERROR ("GreCreateLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreRealizeColorTable - Exit.\n");
        /*
         * set the created color table to null and attempt another realize
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreRealizeColorTable - Entry.\n");
        pData = 0;
        b_rc = GreRealizeColorTable(printer_hdc);
        if (b_rc == FALSE)
           error_count += CHECKRETURNERROR(b_rc,"PMERR_COL_TABLE_NOT_REALIZABLE",SEVERITY_ERROR,PMERR_COL_TABLE_NOT_REALIZABLE);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreRealizeColorTable - Exit.\n");
        /*
         * check for invalid device con+ex+ handle
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreRealizeColorTable - Entry.\n");
        b_rc = GreRealizeColorTable((HDC)0L);
        if (b_rc == FALSE)
           error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreRealizeColorTable - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreRealizeColorTable                                           -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreRealizeColorTable                                           -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreRealizeColorTableRc - Exit.\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreUnrealizeColorTableRc()
 *
 * Purpose:
 *
 * System Requirements: 6.10.3
 *
 * Revision Log:
 *          12/12/90, pdvt, markrichardson
 *                  - initial codeing
 *
 * Inputs:
 *    hPS           - presentation space handle
 *    hWndClient    - client window handle
 *    SelectionCall - selection parameter from DTT
 *
 * Outputs:
 *   Messages to the DTT_log file.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   query the client window for it's anchor block
 *   get from DTT the device open structure for the printer device associated
 *       with this anchor block
 *   create, realize, and unrealize a logical color table
 *\\end
 ***************************************************************************/

VOID APIENTRY GreUnrealizeColorTableRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG         error_count;     /* api call failures */
  DEVOPENSTRUC dop;             /* device open info structure */
  DRIVDATA     driv;            /* structure for dop and DevOpenDC */
  LONG         lDCType;         /* used by WinSendMsg */
  HAB          hAB;             /* anchor block handle */
  HDC          printer_hdc;     /* printer device con+ex+ handle */
  BOOL         b_rc;            /* boolean return code */
  ULONG        flOptions;       /* create options */
  LONG         lFormat;         /* format of entries in the table */
  LONG         iStart;          /* start index relevant for LCOLF_CONSECRGB */
  LONG         cCount;          /* number elements in application data area */
  PLONG        pData;           /* application data area */
  CHAR         out_string[257]; /* buffer for error messages */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ - GreUnRealizeColorTableRc - Entry.\n");

        /*
         * get the device open structure for the printer device
         */
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR ("DevOpenDC",TRUE,"GreUnRealizeColorTableRc");
           return;
        }
        /*
         * unrealize the color table checking for ther not realized
         * error returned
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreUnRealizeColorTable - Entry.\n");
        b_rc = GreUnrealizeColorTable(printer_hdc);
        if (b_rc == FALSE)
           error_count += CHECKRETURNERROR(b_rc,"PMERR_COL_TABLE_NOT_REALIZED",SEVERITY_ERROR,PMERR_COL_TABLE_NOT_REALIZED);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreUnRealizeColorTable - Exit.\n");
        /*
         * call GreCreateLogColorTable with good parameters expecting
         * pmerr_ok response
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateColorTable - Entry.\n");
        flOptions = LCOL_RESET;
        lFormat   = LCOLF_INDRGB;
        iStart    = 0;
        cCount    = 0;
        pData     = 0;
        b_rc = GreCreateLogColorTable(printer_hdc,flOptions,lFormat,iStart,cCount,pData);
        if (b_rc == FALSE) {
           error_count += BADERROR ("GreCreateLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCreateColorTable - Exit.\n");
        /*
         * realize the newly created color table
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreRealizeColorTable - Entry.\n");
        b_rc = GreRealizeColorTable(printer_hdc);
        if (b_rc == FALSE) {
           error_count += BADERROR ("GreCreateLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreRealizeColorTable - Exit.\n");
        /*
         * now unrealize the color table
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreUnRealizeColorTable - Entry.\n");
        b_rc = GreUnrealizeColorTable(printer_hdc);
        if (b_rc == FALSE) {
           error_count += BADERROR ("GreCreateLogColorTable",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ie+ GreUnRealizeColorTable - Exit.\n");
        /*
         * check for invalid device con+ex+ error
         */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreUnRealizeColorTable - Entry.\n");
        b_rc = GreUnrealizeColorTable((HDC)0L);
        if (b_rc == FALSE)
           error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ie+ GreUnRealizeColorTable - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreUnRealizeColorTable                                         -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreUnRealizeColorTable                                         -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ - GreUnRealizeColorTableRc - Exit.\n");
  }
}
