/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GREDVCON.C")

#define INCL_DEV             // include all DEV definitions
#define INCL_WIN             // include Win definitions
#define INCL_GPI             // include all Gpi definitions
#define INCL_GREALL          // include all Gre definitions
#define INCL_ERRORS          // include all Gre error definitions
#define INCL_DOSPROCESS      // needed for DosSleep
#define INCL_DDIMISC         // needed for sfont defs.
#define INCL_SPLERRORS       // Spooler Errors
#define INCL_SHLERRORS       // Shell Errors

#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <pmddi.h>              // needed for GRE function definitions.
#include <stdio.h>
#include <limits.h>             // needed for LONG_MAX
#include "DTTDLLDF.h"           // test case defines.
#include "common1.h"
#include "common2.h"            // Graphics testing common files.
#include "gredvcon.h"           // Test Definition File.


/****************************************************************************
 *
 * Subtask Name: GreDvCon.dll
 *
 * Module name:  GreDvCon.c
 *
 * Purpose:      GreDvCon.dll C source code file
 *
 * Revision Log: 01/23/92,      Graphics Suite Extension, David McCombs
 *                               Initial Design.
 *
 * Documentation Reference:  Section 14.0
 *
 * Contents:     InitTest
 *               Add function names here.
 *
 * Design Overview:
 *
 *
 *
 * Limitations:
 *
 *
****************************************************************************/

/****************************************************************************
 *
 * Routine Name: InitTest()
 *
 * Purpose:  Initialize function names and address array.
 *
 * System Requirements:
 *
 * Revision Log: 01/01/92,      Graphics Suite Extension, Mike Abuzant
 *                              Initial Design.
 *
 * Inputs:
 *   sel                - passed through to the InitTestSegVars function
 *   init_info_p        - name and address array used by DTT interface
 *
 * Outputs:
 *   init_info_p        - returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()  - used by the DTT interface
 *
 * Limitations: None
 *
 *
 ***************************************************************************/
/****************************************************************************
 * Algorithm:
 * {
 *   Get the number of tests.
 *   for the number of tests {
 *                Get the test names;
 *                Get the test addresses;
 *                }
 *   call InitTestSegVars function
 *   }
 *
 ***************************************************************************/

VOID APIENTRY InitTest(PVOID Selector, PTESTINITINFO init_info_p)
{
  register USHORT usTestCounter;

  init_info_p->ntests = NTESTS;
  strcpy(init_info_p->group_name,TestCaseName); //Testcase Name

  for (usTestCounter=0 ;usTestCounter< NTESTS; ++usTestCounter)
  {

     init_info_p->test_addrs[usTestCounter]=
                               EntryPointList[usTestCounter].TestAddress;

     strcpy(init_info_p->test_names[usTestCounter],
                               EntryPointList[usTestCounter].TestName);

  }

  InitTestSegVars(Selector);
  return;

}

/***************************************************************************
 *
 * Routine Name: GreQueryDevResourceExh
 *
 * Purpose:       This entry point exhaustively tests the
 *                GreQueryDevResource  API.
 *
 *
 * System Requirements: Section 14.4.1
 *
 * Revision Log: 01/23/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    SETENTER()
 *    SETEXIT()
 *    SUBENTER()
 *    SUBEXIT()
 *    VerifyReturn()
 *
 * Limitations:
 *          Presently, does not check for valid selector to a font resource.
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *    Initialize structure with all valid parameters to GreQueryDevResource
 *    Setup{
        Get the Device Context handle.
        Write test info header.
         } // end setup
      Log Subenter(lType = RT_DISPLAYINFO)
      Initialize ltype comparator to RT_DISPLAY_INFO

 *
 * }
 *
 ***************************************************************************/

VOID APIENTRY GreQueryDevResourceExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
 {

      static PCHAR EntryName="GreQueryDevResourceExh"; // entry point name


      if (SelectionCall)
            WinSendMsg (hWndClient, (USHORT)WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      else
      {

             HDC   hdc;
             SHORT  sTestLoop, sRange;
             ULONG  ulReturn;
             ULONG ulCompType;
      static struct
           { ULONG  ulType;
             ULONG  ulID;
             PCHAR  szType;
             PCHAR  szID;
           } Params[] =
{
RT_DISPLAYINFO,   1L,             "RT_DISPLAYINFO",   "1L",
RT_POINTER,  SPTR_ARROW,          "RT_POINTER",       "SPTR_ARROW",
RT_POINTER,  SPTR_TEXT,           "RT_POINTER",       "SPTR_TEXT",
RT_POINTER,  SPTR_WAIT,           "RT_POINTER",       "SPTR_WAIT",
RT_POINTER,  SPTR_MOVE,           "RT_POINTER",       "SPTR_MOVE",
RT_POINTER,  SPTR_SIZENWSE,       "RT_POINTER",       "SPTR_SIZENWSE",
RT_POINTER,  SPTR_SIZENESW,       "RT_POINTER",       "SPTR_SIZENESW",
RT_POINTER,  SPTR_SIZEWE,         "RT_POINTER",       "SPTR_SIZEWE",
RT_POINTER,  SPTR_SIZENS,         "RT_POINTER",       "SPTR_SIZENS",
RT_POINTER,  SPTR_APPICON,        "RT_POINTER",       "SPTR_APPICON",
RT_POINTER,  SPTR_ICONINFORMATION,"RT_POINTER",       "SPTR_ICONINFORMATION",
RT_POINTER,  SPTR_ICONQUESTION,   "RT_POINTER",       "SPTR_ICONQUESTION",
RT_POINTER,  SPTR_ICONERROR,      "RT_POINTER",       "SPTR_ICONERROR",
RT_POINTER,  SPTR_ICONWARNING,    "RT_POINTER",       "SPTR_ICONWARNING",
RT_POINTER,  SPTR_ILLEGAL,        "RT_POINTER",       "SPTR_ILLEGAL",
RT_POINTER,  SPTR_MULTFILE,       "RT_POINTER",       "SPTR_MULTFILE",
RT_POINTER,  SPTR_PROGRAM,        "RT_POINTER",       "SPTR_PROGRAM",
RT_POINTER,  SPTR_FILE,           "RT_POINTER",       "SPTR_FILE",
RT_POINTER,  SPTR_FOLDER,         "RT_POINTER",       "SPTR_FOLDER",
RT_BITMAP,   SBMP_SYSMENU,        "RT_BITMAP",        "SBMP_SYSMENU",
RT_BITMAP,   SBMP_SBUPARROW,      "RT_BITMAP",        "SBMP_SBUPARROW",
RT_BITMAP,   SBMP_SBDNARROW,      "RT_BITMAP",        "SBMP_SBDNARROW",
RT_BITMAP,   SBMP_SBRGARROW,      "RT_BITMAP",        "SBMP_SBRGARROW",
RT_BITMAP,   SBMP_SBLFARROW,      "RT_BITMAP",        "SBMP_SBLFARROW",
/**************************************************************************
These are commented out since  their definitions can not be found.

RT_BITMAP,   SBMP_SBUPDBLARROW,   "RT_BITMAP",        "SBMP_SBUPDBLARROW",
RT_BITMAP,   SBMP_SBDNDBLARROW,   "RT_BITMAP",        "SBMP_SBDNDBLARROW",
RT_BITMAP,   SBMP_SBRGDBLARROW,   "RT_BITMAP",        "SBMP_SBRGDBLARROW",
RT_BITMAP,   SBMP_SBLFDBLARROW,   "RT_BITMAP",        "SBMP_SBLFDBLARROW",
**************************************************************************/
RT_BITMAP,   SBMP_MENUCHECK,      "RT_BITMAP",        "SBMP_MENUCHECK",
RT_BITMAP,   SBMP_CHECKBOXES,     "RT_BITMAP",        "SBMP_CHECKBOXES",
RT_BITMAP,   SBMP_BTNCORNERS,     "RT_BITMAP",        "SBMP_BTNCORNERS",
RT_BITMAP,   SBMP_MINBUTTON,      "RT_BITMAP",        "SBMP_MINBUTTON",
RT_BITMAP,   SBMP_MAXBUTTON,      "RT_BITMAP",        "SBMP_MAXBUTTON",
RT_BITMAP,   SBMP_RESTOREBUTTON,  "RT_BITMAP",        "SBMP_RESTOREBUTTON",
RT_BITMAP,   SBMP_DRIVE,          "RT_BITMAP",        "SBMP_DRIVE",
RT_BITMAP,   SBMP_CHILDSYSMENU,   "RT_BITMAP",        "SBMP_CHILDSYSMENU",
RT_BITMAP,   SBMP_FILE,           "RT_BITMAP",        "SBMP_FILE",
RT_BITMAP,   SBMP_FOLDER,         "RT_BITMAP",        "SBMP_FOLDER",
RT_BITMAP,   SBMP_PROGRAM,        "RT_BITMAP",        "SBMP_PROGRAM",
RT_BITMAP,   SBMP_TREEPLUS,       "RT_BITMAP",        "SBMP_TREEPLUS",
RT_BITMAP,   SBMP_TREEMINUS,      "RT_BITMAP",        "SBMP_TREEMINUS",
RT_BITMAP,   SBMP_MENUATTACHED,   "RT_BITMAP",        "SBMP_MENUATTACHED",
RT_BITMAP,   SBMP_SIZEBOX,        "RT_BITMAP",        "SBMP_SIZEBOX",
RT_FONT,     SFONT_RASTER,        "RT_FONT",          "SFONT_RASTER",
RT_FONT,     SFONT_OUTLINE,       "RT_FONT",          "SFONT_OUTLINE"
};
            sRange=sizeof(Params) / sizeof(Params[0]);

            ENTER_MAIN2(EntryName);

            SETENTER(EntryName,"Exhaustive Setup");

            if ((hdc = GpiQueryDevice(hPS)) == HDC_ERROR)

            {
               CLEANUP("GpiQueryDevice()", EntryName);
               return;
            }

            ulCompType = Params[0].ulType;

            LOGINFO(L_TRACE,"*","I/O Subsystems and Device Support Vol 2, pg 12-108, "
                                "indicates that non-display drivers are required to "
                                "support this function by at minimum returning NULL.  "
                                "Display drivers MUST support the GreQueryDevResource "
                                "function with appropriate selector returns.");

            SETEXIT();

            SUBENTER(lType, Params[0].szType);

            for (sTestLoop=0; sTestLoop < sRange; sTestLoop++)
            {
               // Start a new test item entry each time the Type changes..

               if (ulCompType != Params[sTestLoop].ulType)
                  {
                  SUBEXIT();
                  SUBENTER(lType, Params[sTestLoop].szType);
                  ulCompType =  Params[sTestLoop].ulType;
                  }

               ulReturn = (LONG)(GreQueryDevResource2(hdc,
                                                       Params[sTestLoop].ulType,
                                                       Params[sTestLoop].ulID));
               sprintf(LogText1,"ID: %s -",
                                 Params[sTestLoop].szID);
               sprintf(LogText2,"Selector: %d",
                                 ulReturn);

               if (ulReturn==GPI_ALTERROR)
                  ErrorCount++;
               else
                  VerifyReturn(hWndClient, &ErrorCount, ulReturn,
                        Params[sTestLoop].ulType,Params[sTestLoop].ulID);

               ERRSTATUS(TRUE, LogText1, LogText2 );

            }  // end of test loop

            SUBEXIT();

            EXIT_MAIN(EntryName);

            // since no output is generated, delete the page.

      }

      return;

 }

/*************************************************************************
*
*
*   Function used to verify that the returns from the
*   GreQueryDevResource function point to valid resources.
*
*
*************************************************************************/

VOID VerifyReturn(HWND hWndClient, PLONG ErrorCount,
                  PVOID Sel, ULONG ulType, ULONG ulID)
{
   DISPLAYINFO * Disp;  // to verify first element = 26 for RT_DISPLAYINFO

   static PVOID lAddress= NULLHANDLE;
          CHAR     LogBuf[35];
          PCHAR    Check;
          CHAR     Temp[2];

   if (Sel != NULLHANDLE)
   {
      lAddress = (ULONG *)Sel;
      switch ((int) ulType)
      {
         case (int) RT_DISPLAYINFO:

            Disp = (DISPLAYINFO *) lAddress;
            if (Disp->cb != 26 ) (*ErrorCount)++;
            sprintf(LogBuf,"Expected cb: %d - Got cb: %d",26 , Disp->cb );
            LOGINFO(L_DEBUG,"*",LogBuf);
            break;

         case (int) RT_POINTER:

            Check = (CHAR *) lAddress;
            Temp[0] = (*Check);
            Check++;
            Temp[1] = (*Check);
            if ( (Temp[0] == 'P' &&
                  Temp[1] == 'T') ||
                 (Temp[0] == 'B' &&
                  Temp[1] == 'A') )
            LOGINFO(L_DEBUG,"*","Proper Pointer Prefix Found");
            else
            (*ErrorCount)++;

            break;

         case (int) RT_BITMAP:


            Check = (CHAR *) lAddress;
            Temp[0] = (*Check);
            Check++;
            Temp[1] = (*Check);
            if ( (Temp[0] == 'B' &&
                  Temp[1] == 'M') ||
                 (Temp[0] == 'B' &&
                  Temp[1] == 'A') )
            LOGINFO(L_DEBUG,"*","Proper Bitmap Prefix Found");
            else
            (*ErrorCount)++;
            break;


      } // end switch

   } // end if

return;

}


/***************************************************************************
 *
 * Routine Name: GreOpenAndCloseExh
 *
 * Purpose:       This entry point exhaustively tests the
 *                GreOpenDC and GreCloseDC APIs.
 *
 *
 * System Requirements: Section 14.4.2
 *
 * Revision Log: 01/23/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    SETENTER()
 *    SETEXIT()
 *    SUBENTER()
 *    SUBEXIT()
 *    ERRSTATUS()
 *
 * Limitations:
 *          No output.  Will print to the print device whether
 *          the DISPLAY or PRINT option is selected.
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 * Get the current DEVOPENTSTUCT and DC TYPE.
 * Skip this entry point if DeviceIsDisplay.
 * Skip this entry point if DC TYPE is METAFILE or METAFILE_NO_QUERY
 * use ENTER_MAIN2()
 *
 *          SUBENTER( GreOpenDCs test)
 *          for ( 20 loops )
 *          {
 *           CALL GreOpenDC() with the currently set DEVOPENSTRUCT
 *             if (RESULT == DEV_ERROR)
 *             Increment the error count
 *           REPORT THE PASS/FAIL STATUS with ERRRSTATUS
 *          }
 *          SUBEXIT();
 *
 *          SUBENTER(Close DCs, EntryName);
 *          for ( 20 loops )
 *          {
 *             if (Corresponding Open Succeeded )
 *             {
 *             CALL GreClose()
 *               if (RESULT == FALSE)
 *               Increment the error count
 *             REPORT THE PASS/FAIL STATUS with ERRRSTATUS
 *             }
 *          }
 *          SUBEXIT();
 *
 * EXIT_MAIN();
 *
 * }
 *
 ***************************************************************************/


VOID APIENTRY GreOpenAndCloseDCExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
 {

      static PCHAR EntryName="GreOpenAndCloseDCExh"; // entry point name
             HDC     hReturn[30];       // Return Dev Context handle.
             LONG    lDCType;
             BOOL    bResult;
             USHORT  usLoop;
static       USHORT  usLoopMax=30;
             DEVOPENSTRUC   dop;
             DRIVDATA       driv;

      if (SelectionCall)
      {
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      }
      else
      {


                dop.pdriv = &driv;
              WinSendMsg(hWndClient,
                   WM_GET_DEVOPENSTRUC,
                   MPFROMP(&dop),      // pointer to dev open structure
                   MPFROMP(&lDCType)); // dc type like

            // The user should not be allowed to
            // select the display action, since the
            // display would not really be under test.

            if DeviceIsDisplay()
            {
               ABORTENTRY(EntryName,"Display");
               return;
            }


            // The user should not test OD_METAFILE or
            // OD_METAFILE_NOQUERY since those input
            // parameters are not valid with this function.

            if ( ( DeviceIsPrinter() ) &&
               ( lDCType ==  OD_METAFILE ||
                 lDCType == OD_METAFILE_NOQUERY) )
            {
               ABORTENTRY(EntryName,"OD_METAFILE");
               return;
            }

            ENTER_MAIN2(EntryName);

            // Exhaustive Testing.  Open DCs, then close them.

            SUBENTER(Open DCs, EntryName);
            for ( usLoop = 0; usLoop < usLoopMax; ++usLoop )
            {
               hReturn[usLoop]=
               (HDC)GreOpenDC((HDC) NULL, lDCType, "*", 9L,(PDEVOPENDATA) &dop);
               if (hReturn[usLoop] == DEV_ERROR)
                  ErrorCount++;
             sprintf(LogText1,"%d",usLoop+1);
             ERRSTATUS(TRUE, "GreOpenDC, DC No:",LogText1);
            }
            SUBEXIT();

            SUBENTER(Close DCs, EntryName);
            for ( usLoop=0; usLoop < usLoopMax; ++usLoop )
            {
               if (hReturn[usLoop] != DEV_ERROR)
               {
                  bResult=GreCloseDC(hReturn[usLoop]);
                  if (bResult == FALSE)
                   ErrorCount++;
             sprintf(LogText1,"%d",usLoop+1);
             ERRSTATUS(TRUE, "GreCloseDC, DC No:",LogText1);
               }
            }
            SUBEXIT();

            EXIT_MAIN(EntryName);

            // Allow the spooler 5 seconds to clear the print jobs.

            DosSleep(5000);
      }

      return;

 }


/***************************************************************************
 *
 * Routine Name: GreSetAndGetHandleExh
 *
 * Purpose:       This entry point exhaustively tests the
 *                GreSetHandle and GreGetHandle APIs.
 *
 *
 * System Requirements: Section 14.4.3
 *
 * Revision Log: 02/07/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN2()
 *    EXIT_MAIN()
 *    SETENTER()
 *    SETEXIT()
 *    SUBENTER()
 *    SUBEXIT()
 *    ERRSTATUS()
 *
 * Limitations: None.
 *
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      Use ENTERMAIN2 which distinguishes if device is display.
 *          (Calls EnterMain if display or EnterMain2 if printer.)
 *
 *      SUBENTER(Get Original Handles)
 *      Get the four orginal device handles using GreGetHandle.
 *      SUBEXIT
 *
 *      SUBENTER(Outer GreSetHandle and GreGetHandle 10 Iteration Test Loop)
 *      Loop 10 times {
 *        set random seed to sRandSeed + current loop number.
 *        loop 4 times {
 *          set a handle value using randL(0l,LONG_MAX).
 *          Call GreSetHandle to set handle to random handle value.
 *          } end of GreGetHandle Loop.
 *        re-set random seed to sRandSeed + current loop number.
 *        loop 4 times {
 *       Call GreGetHandle(get loop value)
 *          Verify return handle equals to correct (regenerated random) value.
 *          } end of GreGetHandle loop
 *       } end of outer test loop.
 *       SUBEXIT
 *
 *      SUBENTER(Re-set Original Handles)
 *      Set the four orginal device handles using GreSetHandle.
 *      SUBEXIT
 *
 *      EXIT_MAIN
 * }
 *
 ***************************************************************************/

VOID APIENTRY GreSetAndGetHandleExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
 {
      static PCHAR EntryName="GreSetAndGetHandleExh"; // entry point name
             HDC     hReturn;          // Return Dev Context handle.
             LONG    hSetHandle;       // Value for GreSetHandle.
             LONG    hGetHandle;       // Return from GreGetHandle.
             LONG    hOrigHandles[4];  // 4 original handles.
             BOOL    bResult;
             ULONG   ulLoop;
static       ULONG   ulLoopMax=4L;
static       SHORT   outermax=10;
             SHORT   outerloop;
static       SHORT   sRandSeed=10;

      if (SelectionCall)
      {
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      }
      else
      {

            // ENTER_MAIN2 insures nothing is output to the printer.

            ENTER_MAIN2(EntryName);

            // Query for the DC handle

            SETENTER(EntryName,"Query Device Context");

            if ((hReturn = GpiQueryDevice(hPS)) == HDC_ERROR)
            {
               CLEANUP("GpiQueryDevice()", EntryName);
               return;
            }

            SETEXIT();

            // Save original handles to restore them at end of test.
            // a "TRAP D" occurs on Close Doc. if this is not done.

            SUBENTER(Query Original Handles, EntryName);
               for ( ulLoop=0L; ulLoop < ulLoopMax; ++ulLoop )
               {
                  hOrigHandles[ulLoop]=GreGetHandle(hReturn, ulLoop);
                  if ( hOrigHandles[ulLoop] == GPI_ALTERROR )
                     ErrorCount++;
                  sprintf(LogText1,"iIndex: %li -",ulLoop+1);
                  sprintf(LogText2, "Queried hHandle: %li",hOrigHandles[ulLoop]);
                  ERRSTATUS(TRUE,LogText1, LogText2);
               }
            SUBEXIT();

            SUBENTER(10 x 4 Random Handles, EntryName);

            for (outerloop=0;outerloop < outermax; ++outerloop)
            {
               srand(sRandSeed+outerloop);  // set rand

               // Exhaustively Test Sets.

               sprintf(LogText1,"Test Pass %i of %i",outerloop+1, outermax);
               ERRSTATUS(FALSE,"GreSetHandle -",LogText1);

               for ( ulLoop=0L; ulLoop < ulLoopMax; ++ulLoop )
               {
                  hSetHandle=(randL(0L,LONG_MAX))/(outerloop+1L);
                  bResult=GreSetHandle( hReturn , hSetHandle , ulLoop );
                  if (bResult == FALSE)
                     ErrorCount++;
                  sprintf(LogText1,"iIndex: %li -",ulLoop+1);
                  sprintf(LogText2, "Set hHandle: %li",hSetHandle);
                  ERRSTATUS(TRUE,LogText1, LogText2);
               }

               srand(sRandSeed+outerloop);  // re-set rand for comparison
                                            // testing.

               // Exhaustively Test Gets.

               sprintf(LogText1,"Test Pass %i of %i",outerloop+1, outermax);
               ERRSTATUS(FALSE,"GreGetHandle -",LogText1);

               for ( ulLoop=0L; ulLoop < ulLoopMax; ++ulLoop )
               {
                  hGetHandle=GreGetHandle(hReturn, ulLoop);
                  if ( hGetHandle != (randL(0L,LONG_MAX))/(outerloop+1L) )
                     ErrorCount++;
                  sprintf(LogText1,"iIndex: %ld -",ulLoop+1);
                  sprintf(LogText2, "Queried hHandle: %ld",hGetHandle);
                  ERRSTATUS(TRUE,LogText1, LogText2);
               }

            } // end of outerloop

            SUBEXIT();

            // put the original handles back.

            SUBENTER(Reset Original Handles, EntryName);

               for ( ulLoop=0L; ulLoop < ulLoopMax; ++ulLoop )
               {
                  bResult=GreSetHandle( hReturn, hOrigHandles[ulLoop], ulLoop );
                  if (bResult == FALSE)
                     ErrorCount++;
                  sprintf(LogText1,"%li",ulLoop+1);
                  ERRSTATUS(FALSE,"Restoring hHandle No:", LogText1);
               }

            SUBEXIT();

            EXIT_MAIN(EntryName);

      }

      return;

 }


/***************************************************************************
 *
 * Routine Name: GreSetAndGetProcessControlExh
 *
 * Purpose:       This entry point exhaustively tests the
 *                GreSetProcessControl and GreGetProcessControl APIs.
 *
 * System Requirements: Section 14.4.4
 *
 * Revision Log: 02/07/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:       NONE
 *
 *
 * Subroutines Required:
 *                ENTER_MAIN2()
 *                EXIT_MAIN()
 *                SETENTER()
 *                SETEXIT()
 *                SUBENTER()
 *                SUBEXIT()
 *                ERRSTATUS()
 *                randL()
 *
 * Limitations:   NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      Use ENTERMAIN2 which distinguishes if device is display.
 *          (Calls EnterMain if display or EnterMain2 if printer.)
 *      SETENTER( Get DC )
 *      hdc=GpiQueryDevice()
 *      SETEXIT()
 *
 *      SUBENTER(Save Original Flags)
 *      flFlags=GreGetProcess()
 *      Verify function is not in error
 *      ERRSTATUS(original flags are 'flags');
 *      SUBEXIT
 *
 *      SUBENTER(Restore Original Flags)
 *      Success=GreSetProcess()
 *      Verify function is not in error
 *      ERRSTATUS(original flags are 'flags');
 *      SUBEXIT
 *
 *      SUBENTER(Verify Restore of Original Flags)
 *      flFlags=GreGetProcess()
 *      Verify function is not in error
 *      ERRSTATUS(original flags are 'flags');
 *      SUBEXIT
 *
 *      SUBENTER(30 Iteration Test Loop)
 *      Loop 30 times
 *      {
 *         1) Randomly set mask and flags in the range of 0L to
 *           ( PCTL_DRAW |
 *             PCTL_BOUND |
 *             PCTL_CORRELATE |
 *             PCTL_USERBOUNDS |
 *             PCTL_AREA ) = 1F.
 *          2) Call GreSetProcessControl with randomly set flags and mask.
 *          3) Verify return is not an error.
 *          4) Call GreGetProcessControl.
 *          5) Verify (return)&MASK == MASK&FLAGS of set.
 *      } end of test loop.
 *      SUBEXIT
 *
 *      SUBENTER(Restore Original Flags)
 *      success=GreSetProcess()
 *      ERRSTATUS(success);
 *      SUBEXIT
 *
 *      EXIT_MAIN
 * }
 *
 ***************************************************************************/

VOID APIENTRY GreSetAndGetProcessControlExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
 {
      static PCHAR EntryName="GreSetAndGetProcessControlExh"; // entry point name
             HDC     hdc;
             BOOL    bSuccess;
             LONG    flFlags,
                     flSaved;
             ULONG   flMask,
                     flSet;
             USHORT  usLoop;
static       USHORT  usLoopMax=30;
static       USHORT  sRandSeed=34;

      if (SelectionCall)
      {
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      }
      else
      {
            // ENTER_MAIN2 insures SAA compatabilty, and that
            //  nothing is output to the printer.

            ENTER_MAIN2(EntryName);

            // Query for the DC handle

            SETENTER(EntryName,"Query Device Context");

            if ((hdc = GpiQueryDevice(hPS)) == HDC_ERROR)
            {
               CLEANUP("GpiQueryDevice()", EntryName);
               return;
            }

            SETEXIT();

            // save original flags

            SUBENTER(Save Process Control Flags, EntryName);

            flSaved==GreGetProcessControl(hdc);
            if (flSaved==DCTL_ERROR) ErrorCount++;
            sprintf(LogText2, "FLAGS: 0x%lX",flSaved);
            ERRSTATUS(TRUE,"GreGetProcessControl", LogText2);

            SUBEXIT();

            // restore original flags

            SUBENTER(Restore Process Control Flags, EntryName);

            bSuccess=GreSetProcessControl(hdc, 0x1FL, flSaved);
            if (!bSuccess) ErrorCount++;
            sprintf(LogText2, "FLAGS: 0x%lX",flSaved);
            ERRSTATUS(TRUE,"GreSetProcessControl", LogText2);

            SUBEXIT();

            // look at previously restored flags to insure the defaults

            SUBENTER(Save Process Control Flags, EntryName);

            flSaved==GreGetProcessControl(hdc);
            if (flSaved==DCTL_ERROR) ErrorCount++;
            sprintf(LogText2, "FLAGS: 0x%lX",flSaved);
            ERRSTATUS(TRUE,"GreGetProcessControl", LogText2);

            SUBEXIT();

            // start a testing loop with random selection of mask and flags
            // in the range of 0L to
            // (PCTL_DRAW|PCTL_BOUND|PCTL_CORRELATE|PCTL_USERBOUNDS|PCTL_AREA)

            SUBENTER(Random Parameter Testing, EntryName);

            for ( usLoop=0; usLoop < usLoopMax; ++usLoop )
            {
               flSet=randL(0L,0x1FL);  // Range of ORed flags
               flMask=randL(0L,0x1FL); // Range of BIT fields
               bSuccess=GreSetProcessControl(hdc,flMask,flSet);
               if (!bSuccess) ErrorCount++;
               sprintf(LogText1, "MASK: 0x%lX -",flMask);
               sprintf(LogText2, "FLAGS: 0x%lX", flSet);
               ERRSTATUS(TRUE, LogText1, LogText2);
               sprintf(LogText1,"0x%lX",(flMask&flSet)); //store the AND result
               ERRSTATUS(FALSE,"MASK & FLAGS:",LogText1);//Report AND result
               flFlags = GreGetProcessControl(hdc);
               if ( flFlags == DCTL_ERROR)
                  ErrorCount++;
               else
                  {
                  flFlags &= flMask; //modify flFlag for valid log comparison.
                  if (flFlags != (long)(flSet & flMask) )
                     ErrorCount++;
                  }
               sprintf(LogText2, "FLAGS: 0x%lX",flFlags); //store Get result
               ERRSTATUS(TRUE,"GreGetProcessControl", LogText2);
            }

            SUBEXIT();

            // restore the default flags that were saved on entering.

            SUBENTER(Restore Process Control Flags, EntryName);

            bSuccess=GreSetProcessControl(hdc, 0x1FL, flSaved);
            if (!bSuccess) ErrorCount++;
            sprintf(LogText2, "FLAGS: 0x%lX",flSaved);
            ERRSTATUS(TRUE,"GreSetProcessControl", LogText2);

            SUBEXIT();

            EXIT_MAIN(EntryName);

      }

      return;

 }

/***************************************************************************
 *
 * Routine Name: GreQueryEngineVersionExh
 *
 * Purpose:       This entry point exhaustively tests the
 *                GreQueryEngineVersion API.
 *
 * System Requirements: Section 14.4.5
 *
 * Revision Log: 02/07/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:       NONE
 *
 *
 * Subroutines Required:
 *                ENTER_MAIN2()
 *                EXIT_MAIN()
 *                SETENTER()
 *                SETEXIT()
 *                SUBENTER()
 *                SUBEXIT()
 *                ERRSTATUS()
 *
 * Limitations:   NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      Use ENTERMAIN2 which distinguishes if device is display.
 *          (Calls EnterMain if display or EnterMain2 if printer.)
 *
 *      SUBENTER( Query Engine Version )
 *
 *      return=GreQueryEngineVersion()
 *      if return equals GPI_ALTERROR then increment ErrorCount.
 *      sprintf(logtext, return)
 *
 *      ERRSTATUS( Engine Version Number is, logtext);
 *
 *      SUBEXIT
 *
 *      EXIT_MAIN
 * }
 *
 ***************************************************************************/

VOID APIENTRY GreQueryEngineVersionExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
 {
      static PCHAR EntryName="GreQueryEngineVersionExh"; // entry point name
             LONG    lReturn;

      if (SelectionCall)
      {
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      }
      else
      {
            // ENTER_MAIN2 insures SAA compatabilty, and that
            //  nothing is output to the printer.

            ENTER_MAIN2(EntryName);

            // Call and validate the function under test.

            SUBENTER(Query Engine Version, EntryName);

            lReturn=GreQueryEngineVersion();
            if (lReturn == GPI_ALTERROR) ErrorCount++;
            sprintf(LogText2, "%ld",lReturn);
            ERRSTATUS(TRUE,"The Graphics Engine Version No:", LogText2);

            SUBEXIT();

            EXIT_MAIN(EntryName);

      }

      return;

 }



/****************************************************************************
 *
 *
 * Routine Name: GreQueryDevResourceRc
 *
 * Purpose:    This is an entry point for return code tests.
 *
 * System Requirements: Section 14.5.1
 *
 * Revision Log: 01/23/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    CHECKRC()
 *
 * Limitations:
 *          The only errors listed in presentation driver interfaces
 *          manual were PMERR_BASE_ERROR and PMERR_DEV_FUNC_NOT_INSTALLED.
 *          Neither of these are valid tests for the current situation, since
 *          They indicate a driver fault.  The only implementable test is
 *          PMERR_INV_HDC, which is complete.  The PMERR_HDC_BUSY test
 *          does not invoke the error and is commented out.
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *    Set an HDC type variable to -1L.
 *    ENTER_MAIN2()
 *    SUBENTER(PMERR_INV_HDC)
 *    Use that as the HDC value input to GreQueryDevDevice to
 *    solicit the PMERR_INV_HDC error.
 *    SUBEXIT
 *    EXIT_MAIN()
 * }
 *
 ***************************************************************************/

VOID APIENTRY GreQueryDevResourceRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{

      static PCHAR EntryName="GreQueryDevResourceRc"; // entry point name
             HDC     hdc,
                     InvalidHdc;
             BOOL    ReturnValue;
      LONG   result;


      if (SelectionCall)
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      else
      {

            InvalidHdc=(HDC)-1L;
            ENTER_MAIN2(EntryName);

            // Though the device context handle is not used
            // future releases of this entry point may require
            // this handle.

            SETENTER(EntryName,"Query the Device Context");

            if ((hdc = GpiQueryDevice(hPS)) == HDC_ERROR)
            {
               CLEANUP("GpiQueryDevice()", EntryName);
               return;
            }

            SETEXIT();

            // Test of Invalid HDC.

            SUBENTER(PMERR_INV_HDC, EntryName);

            result = (LONG) GreQueryDevResource2(InvalidHdc,RT_POINTER,SPTR_ARROW);
            ReturnValue = (result != GPI_ALTERROR);
            CHECKRC(ReturnValue, PMERR_INV_HDC, EntryName);

            SUBEXIT();

            // Test of Busy HDC.

            /*

            The test of HDC Busy is not in the return code list,
            This section of code would not invoke the PMERR_HDC_BUSY error.
            Since the documentation does not have this error in its RC list,
            and since it cannot be solicited, this test has been commented out.

            SUBENTER(PMERR_HDC_BUSY, EntryName);

            MAKEPSBUSY(hPS);
            result = (LONG) GreQueryDevResource2(hdc,RT_POINTER,SPTR_ARROW);
            ReturnValue = (result != GPI_ALTERROR);
            CHECKRC(ReturnValue, PMERR_HDC_BUSY, EntryName);
            FREEPS;

            SUBEXIT();

            */

            // End of GreQueryDevResource return code testing.

            EXIT_MAIN(EntryName);


      }
      return;
}

/****************************************************************************
 *
 *
 * Routine Name: GreOpenDCRc
 *
 * Purpose:    This entry point tests GreOpenDC return codes.
 *
 * System Requirements: Section 14.5.2
 *
 * Revision Log: 02/12/92,    Graphics Suite Extension, David McCombs.
 *                            Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    CHECKRC()
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *
 *  ABORTENTRY if device is Display.
 *  ENTER_MAIN2()
 *  SETENTER("Get Dev Open Structure, and current HDC")
 *  SETEXIT()
 *  if ( DEBUG option is TRUE )
 *  {
 *
 *   SUBENTER(PMERR_INV_HDC)
 *   Test uses a -1L HDC handle.
 *   SUBEXIT()
 *
 *   SUBENTER(PMERR_HDC_BUSY)
 *   Test Uses MAKEPSBUSY and FREEPS.
 *   SUBEXIT()
 *
 *  } end debug test.
 *
 *  SUBENTER(PMERR_INV_DRIVER_NAME, EntryName);
 *  Test uses dope.pszDriverName = NULL to invoke error;
 *  SUBEXIT()
 *
 *  if ( lDCType is OD_QUEUED ) // Goes to spooler in OD_QUEUED only.
 *  {
 *
 *   SUBENTER(PMERR_SPL_QUEUE_NOT_FOUND, EntryName);
 *   Test with dope.pszLogAddress set to NULL to invoke the error.
 *   SUBEXIT()
 *
 *   SUBENTER(PMERR_INVALID_PARM, EntryName);
 *   Test with Drive structure elements set to NULL.
 *   SUBEXIT()
 *
 *  } end OD_QUEUED test
 *
 *  SUBENTER(PMERR_INV_DC_TYPE, EntryName);
 *  Test with lType set to OD_METAFILE to invoke the error.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_INV_DC_DATA, EntryName);
 *  1rst Test of PMERR_INV_DC_DATA uses a "$" pszToken.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_INV_DC_DATA, EntryName);
 *  2nd Test of PMERR_INV_DC_DATA uses a NULL DEVOPENDATA structure.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_INV_LENGTH_OR_COUNT, EntryName);
 *  Test using 0xFFFFL as size of structure.
 *  SUBEXIT()
 *
 *  EXIT_MAIN()
 *
 * }
 *
 *
 ***************************************************************************/

VOID APIENTRY GreOpenDCRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{

     static PCHAR EntryName=
                     "GreOpenDCRc"; // entry point name
             HDC     hReturn;       // Return Dev  Context handle.
             HDC     hInvalid,
                     hdc;
             BOOL    ReturnValue;   // Boolian indication of success.
             hInvalid = (HDC)-1L;
      if (SelectionCall)
      {
             WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      }
      else
      {

            DEVOPENSTRUC   dop,
                           dope;
            DRIVDATA       driv,
                           drive;
            LONG    lDCType;

            // The user should not be allowed to
            // select the display device, since that is
            // not under test with this particular function.

            if DeviceIsDisplay()
            {
               ABORTENTRY(EntryName,"Display");
               return;
            }

            ENTER_MAIN2(EntryName);

            SETENTER(EntryName,"Get Dev Open Structure");
                dop.pdriv = &driv;
              WinSendMsg(hWndClient,
                   WM_GET_DEVOPENSTRUC,
                   MPFROMP(&dop),      // pointer to dev open structure
                   MPFROMP(&lDCType)); // dc type

            // Get current Device Context.

            if ((hdc = GpiQueryDevice(hPS)) == HDC_ERROR)
            {
               CLEANUP("GpiQueryDevice()", EntryName);
               return;
             }

            SETEXIT();

            if ( DEBUG )
            {

            // Test of Invalid HDC.

            SUBENTER(PMERR_INV_HDC, EntryName);

            ReturnValue =
            ((hReturn=(HDC)GreOpenDC((HDC) hInvalid, lDCType,
                        "*", 9L,(PDEVOPENDATA) &dop)) != DEV_ERROR);
            CHECKRC(ReturnValue, PMERR_INV_HDC, EntryName);
            if (ReturnValue)
            GreCloseDC(hReturn);

            SUBEXIT();


            // Test of Busy PS.

            SUBENTER(PMERR_HDC_BUSY, EntryName);

            MAKEPSBUSY(hPS);
            ReturnValue =
            ((hReturn=(HDC)GreOpenDC((HDC) hdc, lDCType,
                        "*", 9L,(PDEVOPENDATA) &dop)) != DEV_ERROR);
            FREEPS;
            CHECKRC(ReturnValue, PMERR_HDC_BUSY, EntryName);
            if (ReturnValue)
            GreCloseDC(hReturn);

            SUBEXIT();

            } // END DEBUG


            // Test of PMERR_INV_DRIVER_NAME

            SUBENTER(PMERR_INV_DRIVER_NAME, EntryName);

                dope.pdriv = &drive;
              WinSendMsg(hWndClient,
                   WM_GET_DEVOPENSTRUC,
                   MPFROMP(&dope),
                   MPFROMP(&lDCType));

            // dope.pszDriverName is under test, so make it null:
            dope.pszDriverName=(PSZ) NULL;

            ReturnValue =
            ((hReturn=(HDC)GreOpenDC((HDC) NULL, lDCType,
                     "*", 9L,(PDEVOPENDATA) &dope)) != DEV_ERROR);
            CHECKRC(ReturnValue, PMERR_INV_DRIVER_NAME, EntryName);
            if (ReturnValue)
               GreCloseDC(hReturn);

            SUBEXIT();

            // Test of PMERR_SPL_QUEUE_NOT_FOUND

            if ( lDCType == OD_QUEUED ) // Goes to spooler in OD_QUEUED only.
            {
                SUBENTER(PMERR_SPL_QUEUE_NOT_FOUND, EntryName);

                        dope.pdriv = &drive;
                  WinSendMsg(hWndClient,
                           WM_GET_DEVOPENSTRUC,
                           MPFROMP(&dope),
                           MPFROMP(&lDCType));

                // set dope.pszLogAddress to NULL to provoke the error
                dope.pszLogAddress=(PSZ) NULL;

                ReturnValue =
                ((hReturn=(HDC)GreOpenDC((HDC) NULL, lDCType,
                          "*", 9L,(PDEVOPENDATA) &dope)) != DEV_ERROR);

                CHECKRC(ReturnValue, PMERR_SPL_QUEUE_NOT_FOUND, EntryName);
                if (ReturnValue)
                   GreCloseDC(hReturn);

                SUBEXIT();

                // Test of PMERR_INVALID_PARM

                SUBENTER(PMERR_INVALID_PARM, EntryName);

                dope.pdriv = &driv;
                        WinSendMsg(hWndClient,
                  WM_GET_DEVOPENSTRUC,
                           MPFROMP(&dope),
                           MPFROMP(&lDCType));

                drive.cb=0L;
                drive.lVersion=0L;
                drive.abGeneralData[0]='\0'; // was 255
                dope.pdriv = &drive;

                ReturnValue =
                ((hReturn=(HDC)GreOpenDC((HDC) NULL, lDCType,
                          "*", 9L,(PDEVOPENDATA) &dope)) != DEV_ERROR);
                CHECKRC(ReturnValue, PMERR_INVALID_PARM, EntryName);
                if (ReturnValue)
                   GreCloseDC(hReturn);

                SUBEXIT();

            } // end of else OD_QUEUED


            // Test of PMERR_INV_DC_TYPE

            SUBENTER(PMERR_INV_DC_TYPE, EntryName);

            ReturnValue =
            ((hReturn=(HDC)GreOpenDC((HDC) hdc, OD_METAFILE,
                     "*", 9L,(PDEVOPENDATA) &dop)) != DEV_ERROR);
            CHECKRC(ReturnValue, PMERR_INV_DC_TYPE, EntryName);
            if (ReturnValue)
               GreCloseDC(hReturn);

            SUBEXIT();

            // Test of PMERR_INV_DC_DATA.
            // The validity of the invocation of this error
            // using the pszToken parameter is questonable, since
            // the function documentation states that this parameter
            // is ignored by the graphics engine.  Nevertheless, this
            // is the only way the error could be solicited.  This code
            // may need to be deleted if it is found that this
            // PMERR_INV_DC_DATA message is inappropriate for this
            // method of invocation.


            SUBENTER(PMERR_INV_DC_DATA, EntryName);
            sprintf(LogText1,"This error is solicited by entering a "
                             "pszToken value of \"$\" instead of a "
                              "\"*\".");
            LOGINFO(L_DEBUG,"*",LogText1);

            ReturnValue =
            ((hReturn=(HDC)GreOpenDC((HDC) NULL, lDCType,
                        "$", 9L,(PDEVOPENDATA) &dop)) != DEV_ERROR);
            CHECKRC(ReturnValue, PMERR_INV_DC_DATA, EntryName);
            if (ReturnValue)
               GreCloseDC(hReturn);

            SUBEXIT();


            // 2nd Test of PMERR_INV_DC_DATA using NULL DEVOPENDATA struct.

            SUBENTER(PMERR_INV_DC_DATA, EntryName);
            sprintf(LogText1,"This error solicition is attempted by entering a "
                             "pszData value of \"(PDEVOPENDATA) NULL\" instead "
                             "of a \"(PDEVOPENDATA) &dop\".");

            LOGINFO(L_DEBUG,"*",LogText1);

            ReturnValue =
            ((hReturn=(HDC)GreOpenDC((HDC) NULL, lDCType,
                        "*", 9L,(PDEVOPENDATA) NULL)) != DEV_ERROR);
            CHECKRC(ReturnValue, PMERR_INV_DC_DATA, EntryName);
            if (ReturnValue)
               GreCloseDC(hReturn);

            SUBEXIT();


            // Test of PMERR_INV_LENGTH_OR_COUNT

            SUBENTER(PMERR_INV_LENGTH_OR_COUNT, EntryName);

            ReturnValue =
            ((hReturn=(HDC)GreOpenDC((HDC) NULL, lDCType,
                        "*", 0xFFFFL,(PDEVOPENDATA) &dop)) != DEV_ERROR);
            CHECKRC(ReturnValue, PMERR_INV_LENGTH_OR_COUNT, EntryName);
            if (ReturnValue)
               GreCloseDC(hReturn);

            SUBEXIT();

            EXIT_MAIN(EntryName);
            DeletePrinterOutput(EntryName);
      }
      return;
}


/****************************************************************************
 *
 *
 * Routine Name: GreCloseDCRc
 *
 * Purpose:   This entry point tests GreCloseDC return codes.
 *
 * System Requirements: Section 14.5.3
 *
 * Revision Log: 02/13/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    CHECKRC()
 *    DevEscape()
 *    GpiBeginArea()
 *    GpiEndArea()
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *  ABORTENTRY if device is Display.
 *
 *  ENTER_MAIN2()
 *
 *  SETENTER("Get Dev Open Structure, and current HDC")
 *  SETEXIT()
 *
 *  SUBENTER(PMERR_INV_HDC)
 *  Test uses a -1L HDC handle.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_HDC_BUSY)
 *  Test Uses MAKEPSBUSY and FREEPS.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_ENDOC_NOT_ISSUED)
 *  Test by issuing a START_DOC with no subsequent END_DOC.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_INV_IN_PATH, EntryName);
 *  Test by issuing a GpiBeginPath with no subsequent GpiEndPath.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_INV_IN_AREA, EntryName);
 *  Test by issuing a GpiBeginArea with no subsequent GpiEndArea.
 *  SUBEXIT()
 *
 * }
 *
 ***************************************************************************/

VOID APIENTRY GreCloseDCRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{

     static PCHAR EntryName="GreCloseDCRc"; // entry point name
             HDC     hdc,
                     InvalidHdc,
                     hReturn;
             BOOL    ReturnValue;
             HAB     hAB;


      if (SelectionCall)
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      else
      {
            DEVOPENSTRUC   dop;
            DRIVDATA       driv;
            LONG    lDCType;
            ULONG   flOptions;
            HPS     hPs;
            SIZEL   sizel = { 0L, 0L };

            // The user should not be allowed to
            // select the display device, since that is
            // not under test with this particular function.

            if DeviceIsDisplay()
            {
               ABORTENTRY(EntryName,"Display");
               return;
            }

            ENTER_MAIN2(EntryName);

            SETENTER(EntryName,"Get Dev Open Structure");
                dop.pdriv = &driv;
              WinSendMsg(hWndClient,
                   WM_GET_DEVOPENSTRUC,
                   MPFROMP(&dop),      // pointer to dev open structure
                   MPFROMP(&lDCType)); // dc type

            // Get current Device Context.

            if ((hdc = GpiQueryDevice(hPS)) == HDC_ERROR)
            {
               CLEANUP("GpiQueryDevice", EntryName);
               return;
             }

            hAB=WinQueryAnchorBlock(hWndClient);

            SETEXIT();

            // Test of Invalid HDC.

            SUBENTER(PMERR_INV_HDC, EntryName);

            InvalidHdc=(HDC)-1L;
            ReturnValue=GreCloseDC(InvalidHdc);
            CHECKRC(ReturnValue, PMERR_INV_HDC, EntryName);

            SUBEXIT();

            // Test of Busy DC.

            SUBENTER(PMERR_HDC_BUSY, EntryName);

            if ((hReturn=(HDC)GreOpenDC((HDC) hdc, lDCType,
                        "*", 9L,(PDEVOPENDATA) &dop)) == DEV_ERROR)
               {
               CLEANUP("GreOpenDC", EntryName);
               return;
               }
            flOptions = GpiQueryPS(hPS, &sizel);
            flOptions ^= (flOptions & PS_NORESET);
            flOptions |= PS_ASSOCIATE;  // make sure associate flag is on

            // Issue a START_DOC.

            if (DevEscape(hReturn, DEVESC_STARTDOC,
                          0L ,NULL, NULL, NULL)==DEVESC_ERROR)
            {
               CLEANUP("DevEscape",EntryName);
               return;
            }

            // Create a presentation space to
            // associate with the Device Context.

            if ((hPs=GpiCreatePS(hAB,hReturn,
                                    &sizel,flOptions)) == GPI_ERROR)
            {
               CLEANUP("GpiCreatePS",EntryName);
               return;
            }

            // Busy DC in order to solicit error

            MAKEPSBUSY(hPs);

            // Call function under test in error prone environment

            ReturnValue=GreCloseDC(hReturn);

            // Free PS, and associated DC.

            FREEPS;

            // Check Return code.

            CHECKRC(ReturnValue, PMERR_HDC_BUSY, EntryName);

            // Attempt to clean up after the test.

            if (!ReturnValue)
            {

               // since a bit map was placed on PS, abort doc.

               if (DevEscape(hReturn, DEVESC_ABORTDOC,
                          0L ,NULL, NULL, NULL)==DEVESC_ERROR)
                  SHOWERROR("DevEscape",EntryName);


               if ( (flOptions & PS_TYPE) == GPIT_NORMAL )
               {
                  if (!GpiAssociate(hPs, NULLHANDLE))
                     SHOWERROR("GpiAssociate", EntryName);
               }

               if (!GpiDestroyPS(hPs))
                  SHOWERROR("GpiDestroyPS", EntryName);

               if (!GreCloseDC(hReturn))
               {
                  SHOWERROR("GreCloseDC", EntryName);
               }

            }  // end if (initial CloseDC failure)

            SUBEXIT();


            // Test of PMERR_ENDDOC_NOT_ISSUED.

            SUBENTER(PMERR_ENDOC_NOT_ISSUED, EntryName);

            if ((hReturn=(HDC)GreOpenDC((HDC) hdc, lDCType,
                        "*", 9L,(PDEVOPENDATA) &dop)) == DEV_ERROR)
            {
               CLEANUP("GreOpenDC", EntryName);
               return;
            }

            flOptions = GpiQueryPS(hPS, &sizel);
            flOptions ^= (flOptions & PS_NORESET);
            flOptions |= PS_ASSOCIATE;  // make sure associate flag is on

            // Issue a START_DOC.

            if (DevEscape(hReturn, DEVESC_STARTDOC,
                          0L ,NULL, NULL, NULL)==DEVESC_ERROR)
            {
               CLEANUP("DevEscape",EntryName);
               return;
            }


            // By calling the GreCloseDC function before issuing
            // a ENDDOC escape, the PMERR_ENDDOC_NOT_ISSUED
            // error should result.

            ReturnValue=GreCloseDC(hReturn);

            CHECKRC(ReturnValue, PMERR_ENDDOC_NOT_ISSUED, EntryName);

            // Clean Up After testing.

            if (!ReturnValue)
            {

               if (DevEscape(hReturn, DEVESC_ENDDOC,
                          0L ,NULL, NULL, NULL)==DEVESC_ERROR)
                  SHOWERROR("DevEscape",EntryName);

               if (!GreCloseDC(hReturn))
                  SHOWERROR("GreCloseDC", EntryName);

            }  // end if (initial CloseDC failure)

            SUBEXIT();


            // Test of Invalid Path.

            SUBENTER(PMERR_INV_IN_PATH, EntryName);

            if ((hReturn=(HDC)GreOpenDC((HDC) hdc, lDCType,
                        "*", 9L,(PDEVOPENDATA) &dop)) == DEV_ERROR)
            {
               CLEANUP("GreOpenDC", EntryName);
               return;
            }

            // Insure flag options are correct.

            flOptions = GpiQueryPS(hPS, &sizel);
            flOptions ^= (flOptions & PS_NORESET);
            flOptions |= PS_ASSOCIATE;  // make sure associate flag is on

            // Issue a START_DOC.

            if (DevEscape(hReturn, DEVESC_STARTDOC,
                          0L ,NULL, NULL, NULL)==DEVESC_ERROR)
            {
               CLEANUP("DevEscape",EntryName);
               return;
            }

            // Create a presentation space to
            // associate with the Device Context.

            if ((hPs=GpiCreatePS(hAB,hReturn,
                                    &sizel,flOptions)) == GPI_ERROR)
            {
               CLEANUP("GpiCreatePS",EntryName);
               return;
            }


            if (GpiBeginPath(hPS,1L)==FALSE)
               SHOWERROR("GpiBeginPath",EntryName);

            if (DevEscape(hReturn, DEVESC_ENDDOC,
                       0L ,NULL, NULL, NULL)==DEVESC_ERROR)
               SHOWERROR("DevEscape",EntryName);

            // Attempt to solicit return.

            ReturnValue=GreCloseDC(hReturn);

            if (GpiEndPath(hPS)==FALSE)
               SHOWERROR("GpiEndPath",EntryName);

            CHECKRC(ReturnValue, PMERR_INV_IN_PATH, EntryName);

            if (!ReturnValue)
            {


               if ( (flOptions & PS_TYPE) == GPIT_NORMAL )
               {
                  if (!GpiAssociate(hPs, NULLHANDLE))
                     SHOWERROR("GpiAssociate", EntryName);
               }

               if (!GpiDestroyPS(hPs))
                  SHOWERROR("GpiDestroyPS", EntryName);

               if (GreCloseDC(hReturn))
                  SHOWERROR("GreCloseDC", EntryName);

            }  // end if (initial CloseDC failure)

            SUBEXIT();


            // Test of Invalid In Area.

            SUBENTER(PMERR_INV_IN_AREA, EntryName);

            if ((hReturn=(HDC)GreOpenDC((HDC) hdc, lDCType,
                        "*", 9L,(PDEVOPENDATA) &dop)) == DEV_ERROR)
            {
               CLEANUP("GreOpenDC", EntryName);
               return;
            }

            flOptions = GpiQueryPS(hPS, &sizel);
            flOptions ^= (flOptions & PS_NORESET);
            flOptions |= PS_ASSOCIATE;  // make sure associate flag is on

            // Issue a START_DOC.

            if (DevEscape(hReturn, DEVESC_STARTDOC,
                          0L ,NULL, NULL, NULL)==DEVESC_ERROR)
            {
               CLEANUP("DevEscape",EntryName);
               return;
            }

            // Create a presentation space to
            // associate with the Device Context.

            if ((hPs=GpiCreatePS(hAB,hReturn,
                                    &sizel,flOptions)) == GPI_ERROR)
            {
               CLEANUP("GpiCreatePS",EntryName);
               return;
            }


            if (GpiBeginArea(hPS, BA_BOUNDARY | BA_WINDING )==GPI_ERROR)
               SHOWERROR("GpiEndArea",EntryName);

            if (DevEscape(hReturn, DEVESC_ENDDOC,
                        0L ,NULL, NULL, NULL)==DEVESC_ERROR)
               SHOWERROR("DevEscape",EntryName);

            // Atempt to solicit the PMERR_INV_IN_AREA error.

            ReturnValue=GreCloseDC(hReturn);

            CHECKRC(ReturnValue, PMERR_INV_IN_AREA, EntryName);

            if (GpiEndArea(hPS)==GPI_ERROR)
               SHOWERROR("GpiEndArea",EntryName);

            if (!ReturnValue)
            {

               if ( (flOptions & PS_TYPE) == GPIT_NORMAL )
               {
                  if (!GpiAssociate(hPs, NULLHANDLE))
                     SHOWERROR("GpiAssociate", EntryName);
               }

               if (!GpiDestroyPS(hPs))
                  SHOWERROR("GpiDestroyPS", EntryName);

               if (!GreCloseDC(hReturn))
                  SHOWERROR("GreCloseDC", EntryName);

            }  // end if (initial CloseDC failure)

            SUBEXIT();

            EXIT_MAIN(EntryName);

      }

      return;
}

/****************************************************************************
 *
 *
 * Routine Name: GreSetHandleRc
 *
 * Purpose:   This entry point tests GreSetHandle return codes.
 *
 * System Requirements: Section 14.5.5
 *
 * Revision Log: 02/10/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    CHECKRC()
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *
 *  ENTER_MAIN2()
 *
 *  SETENTER("Get current HDC")
 *  HDC = GpiQueryDevice.
 *  SETEXIT()
 *
 *  SUBENTER(PMERR_INV_HDC)
 *  Test uses a -1L HDC handle.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_HDC_BUSY)
 *  Test Uses MAKEPSBUSY and FREEPS.
 *  SUBEXIT()
 *

 * }
 *
 ***************************************************************************/

VOID APIENTRY GreSetHandleRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{

      static PCHAR EntryName="GreSetHandleRc"; // entry point name
             HDC     hdc,
                     InvalidHdc;
             BOOL    ReturnValue;    // Return from GreSetHandle is BOOLIAN
             ULONG    hSetHandle;     // Value for GreSetHandle
             ULONG    ulIndex;


      if (SelectionCall)
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      else
      {
            // initialize data

            ulIndex=0L;
            hSetHandle=300L;

            ENTER_MAIN2(EntryName);

            // Get Device Context

            SETENTER(EntryName,"Query Device Context");

            if ((hdc = GpiQueryDevice(hPS)) == HDC_ERROR)
            {
               CLEANUP("GpiQueryDevice()", EntryName);
               return;
            }

            SETEXIT();


            // Test of Invalid HDC.

            SUBENTER(PMERR_INV_HDC, EntryName);

            InvalidHdc=(HDC)-1L;

            ReturnValue = GreSetHandle( InvalidHdc , hSetHandle , ulIndex );

            CHECKRC(ReturnValue, PMERR_INV_HDC, EntryName);

            SUBEXIT();


            // Test of Busy HDC.

            SUBENTER(PMERR_HDC_BUSY, EntryName);

            MAKEPSBUSY(hPS);
            ReturnValue = GreSetHandle(hdc, hSetHandle, ulIndex);
            CHECKRC(ReturnValue, PMERR_HDC_BUSY, EntryName);
            FREEPS;

            SUBEXIT();


            // End of GreSetHandle return code testing.

            EXIT_MAIN(EntryName);

            DeletePrinterOutput(EntryName);

      }
      return;
}

/****************************************************************************
 *
 *
 * Routine Name: GreGetHandleRc
 *
 * Purpose:   This entry point tests GreGetHandle return codes.
 *
 * System Requirements: Section 14.5.4
 *
 * Revision Log: 02/14/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    CHECKRC()
 *    All other required subroutines are listed here.
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *
 *  ENTER_MAIN2()
 *
 *  SETENTER("Get current HDC")
 *  HDC = GpiQueryDevice(HPS).
 *  SETEXIT()
 *
 *  SUBENTER(PMERR_INV_HDC)
 *  Test uses a -1L HDC handle.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_HDC_BUSY)
 *  Test Uses MAKEPSBUSY and FREEPS.
 *  SUBEXIT()
 *
 * }
 *
 ***************************************************************************/


VOID APIENTRY GreGetHandleRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{

      static PCHAR EntryName="GreGetHandleRc"; // entry point name
             HDC     hdc,
                     InvalidHdc;
             ULONG    ReturnValue;    // Return from GreGetHandle
             ULONG    ulIndex;


      if (SelectionCall)
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      else
      {

            ulIndex=0L;

            ENTER_MAIN2(EntryName);

            // Get Device Context

            SETENTER(EntryName,"Query Device Context");

            if ((hdc = GpiQueryDevice(hPS)) == HDC_ERROR)
            {
               CLEANUP("GpiQueryDevice()", EntryName);
               return;
            }

            SETEXIT();

            // Test of Invalid HDC.

            SUBENTER(PMERR_INV_HDC, EntryName);

            InvalidHdc=(HDC)-1L;

            ReturnValue =
                  (GreGetHandle( InvalidHdc, ulIndex ) != GPI_ALTERROR);

            CHECKRC(ReturnValue, PMERR_INV_HDC, EntryName);

            SUBEXIT();

            // Test of Busy HDC.

            SUBENTER(PMERR_HDC_BUSY, EntryName);

            MAKEPSBUSY(hPS);
            ReturnValue =
                  (GreGetHandle(hdc, ulIndex) != GPI_ALTERROR);
            CHECKRC(ReturnValue, PMERR_HDC_BUSY, EntryName);
            FREEPS;

            SUBEXIT();

            // End of GreGetHandle return code testing.

            EXIT_MAIN(EntryName);

            DeletePrinterOutput(EntryName);

      }
      return;
}


/****************************************************************************
 *
 *
 * Routine Name: GreSetProcessControlRc
 *
 * Purpose:   This entry point tests GreSetProcessControl return codes.
 *
 * System Requirements: Section  14.5.5
 *
 * Revision Log: 02/14/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    CHECKRC()
 *    All other required subroutines are listed here.
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *
 *  ENTER_MAIN2()
 *
 *  SETENTER("Get current HDC")
 *  HDC = GpiQueryDevice(HPS).
 *  SETEXIT()
 *
 *  SUBENTER(PMERR_INV_HDC)
 *  Test uses a -1L HDC handle.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_HDC_BUSY)
 *  Test Uses MAKEPSBUSY and FREEPS.
 *  SUBEXIT()
 *
 * }
 *
 ***************************************************************************/


VOID APIENTRY GreSetProcessControlRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{

      static PCHAR EntryName="GreSetProcessControlRc"; // entry point name
             HDC     hdc,
                     InvalidHdc;
             BOOL    ReturnValue;    // Return from GreSetProcessControl
             ULONG   flMask,         // Masks for process flags.
                     flProcess;      // Process flags.


      if (SelectionCall)
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      else
      {

            flProcess=flMask=0x1FL;

            ENTER_MAIN2(EntryName);

            // Get Device Context

            SETENTER(EntryName,"Query Device Context");

            if ((hdc = GpiQueryDevice(hPS)) == HDC_ERROR)
            {
               CLEANUP("GpiQueryDevice()", EntryName);
               return;
            }

            SETEXIT();

            // Test of Invalid HDC.

            SUBENTER(PMERR_INV_HDC, EntryName);

            InvalidHdc=(HDC)-1L;

            ReturnValue = GreSetProcessControl(InvalidHdc, flMask, flProcess);

            CHECKRC(ReturnValue, PMERR_INV_HDC, EntryName);

            SUBEXIT();

            // Test of Busy HDC.

            SUBENTER(PMERR_HDC_BUSY, EntryName);

            MAKEPSBUSY(hPS);

            ReturnValue = GreSetProcessControl(hdc, flMask, flProcess);

            CHECKRC(ReturnValue, PMERR_HDC_BUSY, EntryName);

            FREEPS;

            SUBEXIT();

            // End of return code testing.

            EXIT_MAIN(EntryName);

            DeletePrinterOutput(EntryName);

      }
      return;
}


/****************************************************************************
 *
 *
 * Routine Name: GreGetProcessControlRc
 *
 * Purpose:   This entry point tests GreGetProcessControl return codes.
 *
 * System Requirements: Section 14.5.6
 *
 * Revision Log: 02/14/92,    Graphics Suite Extension, David McCombs
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    CHECKRC()
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *
 *  ENTER_MAIN2()
 *
 *  SETENTER("Get current HDC")
 *  HDC = GpiQueryDevice(HPS).
 *  SETEXIT()
 *
 *  SUBENTER(PMERR_INV_HDC)
 *  Test uses a -1L HDC handle.
 *  SUBEXIT()
 *
 *  SUBENTER(PMERR_HDC_BUSY)
 *  Test Uses MAKEPSBUSY and FREEPS.
 *  SUBEXIT()
 *
 * }
 *
 ***************************************************************************/


VOID APIENTRY GreGetProcessControlRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{

      static PCHAR EntryName="GreGetProcessControlRc"; // entry point name
             HDC     hdc,
                     InvalidHdc;
             LONG    ReturnValue;    // Return from GreGetProcessControl


      if (SelectionCall)
            WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
      else
      {

            ENTER_MAIN2(EntryName);

            // Get Device Context

            SETENTER(EntryName,"Query Device Context");

            if ((hdc = GpiQueryDevice(hPS)) == HDC_ERROR)
            {
               CLEANUP("GpiQueryDevice()", EntryName);
               return;
            }

            SETEXIT();

            // Test of Invalid HDC.

            SUBENTER(PMERR_INV_HDC, EntryName);

            InvalidHdc=(HDC)-1L;

            ReturnValue =
                  ( GreGetProcessControl(InvalidHdc ) != DCTL_ERROR);

            CHECKRC(ReturnValue, PMERR_INV_HDC, EntryName);

            SUBEXIT();

            // Test of Busy HDC.

            SUBENTER(PMERR_HDC_BUSY, EntryName);

            MAKEPSBUSY(hPS);

            ReturnValue =
                  ( GreGetProcessControl(hdc ) != DCTL_ERROR);
            CHECKRC(ReturnValue, PMERR_HDC_BUSY, EntryName);

            FREEPS;

            SUBEXIT();

            // End of return code testing.

            EXIT_MAIN(EntryName);

            DeletePrinterOutput(EntryName);

      }
      return;
}




/***************************************************************************
 *\\ddd
 * Routine Name: EnterMain2()
 *
 * Purpose:  This function call logs the test entry.
 *           It is to be used for entry points which do dev calls.
 *           It can also be used wher no output is wanted.
 *
 * System Requirements: None
 *
 * Revision Log: Graphics Engine Test Cases, David McCombs
 *               01/30/91
 *
 * Inputs:
 *   hPS            // presentation space
 *   hWndClient     // client window handle
 *   EntryPoint     // test case Entry point
 *   ErrorCount     // Error count.
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *   None at this time.
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *
 * Set the ErrorCount to zero.
 * Set the ERRORSTATUS static comparison register to ErrorCount.
 * sprintf the ENTRYSPEC format to the string buffer.
 * 'CWRITELOG' the string buffer at a L_LOTRACE log level.
 *
 * }
 *\\end
 ***************************************************************************/

VOID EnterMain2(HPS hPS, HWND hWndClient, CHAR * EntryPoint, PLONG ErrorCount)
 {
   CHAR  achLogLine[200];
   *ErrorCount=0L;
        ErrorStatus(hWndClient, *ErrorCount, FALSE, NULL, NULL);
   sprintf( achLogLine, ENTRY_SPEC, EntryPoint );
   CWRITELOG( L_LOTRACE, 0, achLogLine );
   return;
 }


