/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/*const char header=$header$*/

#define INCL_DOS
#define INCL_DEV      // include all DEV definitions
#define INCL_WIN
#define INCL_GPI      // include all Gpi definitions
#define INCL_ERRORS   // include all Gpi error definitions
#define INCL_WINP_SEI
#define INCL_WINHEAP
#define INCL_WINPOINTERS

#include <os2.h>

#define INCL_GREALL   // include all Gre definitions
#define INCL_DDIMISC
#define INCL_DDIBUNDLES

#include <pmddi.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <errno.h>
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "gregen2.h"

#pragma page()
/****************************************************************************
 *\\ddd
 * Subtask Name: GreGen2.dll
 *
 * Module name: GreGen2.c
 *
 * Purpose: dll test functions
 *
 * Revision Log:
 *        12/02/91 ,DTT, Ken McFadden
 *              -Initial Creation.
 *
 *
 *
 * SAD References:
 *
 * SRS References:
 *
 * Contents:  InitTest()
 *            GreDisjointLinesPre()
 *            GreSetColorCursorExh()  (in gregen2x.c)
 *            GreGetandSetPickWindowExh()  (in gregen2x.c)
 *            GreDisjointLinesExh()  (in gregen2x.c)
 *            GreSetColorCursorRc()  (in gregen2r.c)
 *            GreGetPickWindowRc()   (in gregen2r.c)
 *            GreSetPickWindowRc()   (in gregen2r.c)
 *            GreDisjointLinesRc()   (in gregen2r.c)
 *
 *
 * Design Overview:
 *
 * Limitations:
 *
 *
 *
 *\\end
 ****************************************************************************/
#pragma page()
/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose:  Initialize function names and address array for DTT.
 *
 * System Requirements:
 *
 * Revision Log:
 *
 *                                              10/24/90, pdvt, Mike Abuzant
 *               - initial file
 *
 * Inputs:
 *   sel          // passed through to the InitTestSegVars function
 *   init_info_p  // name and address array used by DTT interface
 *
 * Outputs:
 *   init_info_p  // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()    // used by the DTT interface
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
#pragma page()
/****************************************************************************
 *\\algorithm
 * {
 *        Get the number of tests.
 *   for the number of tests {
 *                      Get the test names;
 *                      Get the test addresses;
 *                      }
 *   call InitTestSegVars function
 *   }
 *\\end
 ***************************************************************************/
#pragma page()
VOID APIENTRY InitTest(PVOID Selector, PTESTINITINFO init_info_p)
{
  register USHORT usTestCounter;

  init_info_p->ntests = NTESTS;
  strcpy(init_info_p->group_name,TestCaseName); //Testcase Name

  for (usTestCounter=0 ;usTestCounter< NTESTS; ++usTestCounter){

     init_info_p->test_addrs[usTestCounter]=
                               EntryPointList[usTestCounter].TestAddress;
     strcpy(init_info_p->test_names[usTestCounter],
                               EntryPointList[usTestCounter].TestName);
  }

  InitTestSegVars(Selector);
  return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreDisjointLinesPre()
 *
 * Purpose: This is an entry point for the DTT that does a
 *          precision test of GreDisjointLines.
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 01/28/92 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              One inch precision grid
 *
 * Subroutines Required:
 *              WinSendMsg, ENTER_MAIN, GpiQueryDevice, SETENTER, SETEXIT, MapX, MapY,
 *    CLEANUP, GreDisjointLines, MapAndWriteString, EXIT_MAIN
 *
 * Limitations:
 *    OD_QUEUED/PM_Q_STD seems to work but there is no output to the device
 *    even though the function does not fail
 *
 *    Due to Mapping round-off error the right or top side lines may not
 *    be visible on devices that are an exact inch in dimension.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ENTER_MAIN();
 *              SETENTER();
 *              get the device context handle
 *              create the array of points to draw a one inch grid
 *                begin carefull to not exceed the page size on the top and
 *                right side boundaries.
 *              SETEXIT();
 *              call the function under test to output the grid
 *              EXIT_MAIN();
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreDisjointLinesPre(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PCHAR    szApiEntryName = "GreDisjointLinesPre";
  POINTL   aptl[100];
  INT      i, j;
  LONG     NumHPoints, NumVPoints, lLimit;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    ENTER_MAIN(szApiEntryName);

    SETENTER(szApiEntryName, "Initializations");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", szApiEntryName);
      return;
    }

    // no. of points to draw horizontal lines (0 thru pg_size_y)
    NumHPoints = 2 * (pg_size_y + 1);
    // check end points to insure they stay visible
    lLimit = MapX(pg_size_x * 100L);
    if (lLimit > pg_width - 1)
      lLimit = pg_width - 1;

    // generate points for horizontal lines
    for (i = 0; i < (INT)NumHPoints; i += 2)
    {
       aptl[i].x = 0L;
       aptl[i].y = MapY((i/2) * 100L);
       aptl[i+1].x = lLimit;
       aptl[i+1].y = aptl[i].y;
    } /* end for i */

    // check last line position, if on even inch size device Mapping may
    // cause last point to not be visible.
    if (aptl[NumHPoints - 2].y > pg_height - 1)
    {
      aptl[NumHPoints - 2].y = pg_height - 1;
      aptl[NumHPoints - 1].y = pg_height - 1;
    }

    // no. of points to draw vertical lines (0 thru pg_size_x)
    NumVPoints = 2 * (pg_size_x + 1);
    // check end points to insure they stay visible
    lLimit = MapY(pg_size_y * 100L);
    if (lLimit > pg_height - 1)
      lLimit = pg_height - 1;

    // generate points for vertical lines
    for (i = 0; i < (INT)NumVPoints; i += 2)
    {
       j = (INT)NumHPoints + i;
       aptl[j].x = MapX((i/2) * 100L);
       aptl[j].y = 0L;
       aptl[j+1].x = aptl[j].x;
       aptl[j+1].y = lLimit;
    } /* end for i */

    // check last line position, if on even inch size device Mapping may
    // cause last point to not be visible.
    if (aptl[NumHPoints + NumVPoints - 2].x > pg_width - 1)
    {
      aptl[NumHPoints + NumVPoints - 2].x = pg_width - 1;
      aptl[NumHPoints + NumVPoints - 1].x = pg_width - 1;
    }
    SETEXIT();

    // output the lines and log error on failure
    if (!GreDisjointLines(hDC, &aptl[0], NumHPoints + NumVPoints))
    {
      CLEANUP("GreDisjointLines", szApiEntryName);
      return;
    }

    MapAndWriteString( 100L, 110L,
                            "ONE INCH PRECISION GRID VIA GreDisjointLines");

    EXIT_MAIN(szApiEntryName);
  }
 return;
}
