/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/* const char header=$header$ */

#define INCL_DOS
#define INCL_DEV      // include all DEV definitions
#define INCL_WIN
#define INCL_GPI      // include all Gpi definitions
#define INCL_ERRORS   // include all Gpi error definitions
#define INCL_WINP_SEI
#define INCL_WINHEAP
#define INCL_WINPOINTERS

#include <os2.h>

#define INCL_GREALL
#define INCL_DDIMISC
#define INCL_DDIBUNDLES

#include <pmddi.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "gregen2r.h"

/****************************************************************************
 *
 * Subtask Name: GreGen2.dll
 *
 * Module name:  gregen2x.c
 *
 * Purpose:      module for GREGEN2 exhautive entry points
 *
 * Revision Log: 12/17/91,      Graphics Suite Extension, Ken McFadden
 *                               Initial Design.
 *               02/19/92       Updates to GreSetColorCursorExh, Thom Bain
 *
 * Documentation Reference:  Section
 *
 * Contents:     GreSetColorCursorRc
 *               GreGetPickWindowRc
 *               GreSetPickWindowRc
 *               GreDisjointLinesRc
 *
****************************************************************************/

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreSetColorCursorRc()
 *
 * Purpose: This is an entry point for DTT that does a
 *          return code test of GreSetColorCursor.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 01/17/92 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Device must not be a printer.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              if device = printer then ABORTENTRY();
 *              ENTER_MAIN();
 *              clear last PMERR;
 *              init pointerinfo;
 *
 *              call GreSetColorCursor using -1 for HDC;
 *              check return for PMERR_INV_HDC;
 *
 *              set xhotspot = 0xFFFFFFFF;
 *              call GreSetColorCursor;
 *              check return for PMERR_INV_COORDINATE;
 *              set xhotspot = NULL;
 *
 *              set hbmpointer = NULL;
 *              call GreSetColorCursor;
 *              check return for PMERR_INV_CURSOR_BITMAP;
 *
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreSetColorCursorRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
 PCHAR            szApiEntryName = "GreSetColorCursorRc";     // entry point name
 HDC              ThDC;
 ERRORID          err;                    // return variable
 POINTERINFO      PointerInfo;
 HAB              hAB;
 HPOINTER         hptr;
 HWND             hWndDeskTop;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else
     {
        if (DeviceIsPrinter())                            // abort if printer is selected
          {
             ABORTENTRY(szApiEntryName,"PRINT");
             return;
          }

        ENTER_MAIN(szApiEntryName);
        SETENTER(szApiEntryName, "Initializations");
        hDC = WinQueryWindowDC(hWndClient);
        ErrorCount = 0;
        if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE)
          {
             CLEANUP("WinQueryAnchorBlock()",szApiEntryName);
             return ;
          }
        WinGetLastError(hAB);

        // get the desktop window handle for pointer and cursor manipulation
        hWndDeskTop = WinQueryDesktopWindow(hAB, NULLHANDLE);

        // hide the system pointer
        if (!WinShowPointer(hWndDeskTop, FALSE))
          SHOWERROR("WinShowPointer", "Removing sytem pointer");

        // create a cusor
        if(!WinCreateCursor(hWndDeskTop, 100, 100, 0, 0, CURSOR_SOLID, NULL))
          SHOWERROR("WinCreateCursor", "Create");

        if (!WinShowCursor(hWndDeskTop, TRUE))
          SHOWERROR("WinShowCursor", "ON");

        hptr = WinQuerySysPointer(HWND_DESKTOP, SPTR_HANDICON, FALSE);
        if(!WinQueryPointerInfo(hptr, &PointerInfo))
        {
          SHOWERROR("WinQueryPointerInfo", "From Sys Pointer");
        }

        SETEXIT();

                                                             // check invalid device context handle return code.
        SUBENTER(PMERR_INV_HDC, "HDC = -1");
        ThDC = (HDC) -1;
        err = GreSetColorCursor(ThDC, &PointerInfo);
        CHECKRC(err, PMERR_INV_HDC, "GreSetColorCursor");
        SUBEXIT();

                                                             // check invalid coordinate.
        SUBENTER(PMERR_INV_COORDINATE, "xHotspot=0xFFFFFFFF");
        PointerInfo.xHotspot = 0xFFFFFFFF;
        err = GreSetColorCursor(hDC, &PointerInfo);
        CHECKRC(err, PMERR_INV_COORDINATE, "GreSetPickWindow");
        PointerInfo.xHotspot = NULLHANDLE;
        SUBEXIT();

                                                             // check invalid bitmap.
        SUBENTER(PMERR_INV_CURSOR_BITMAP,"NULL bitmap handle");
        PointerInfo.hbmPointer = NULLHANDLE;
        err = GreSetColorCursor(hDC, &PointerInfo);
        CHECKRC(err, PMERR_INV_CURSOR_BITMAP, "GreSetColorCursor");
        SUBEXIT();

        // hide the cursor
        if (!WinShowCursor(hWndDeskTop, FALSE))
          SHOWERROR("WinShowCursor", "OFF");

        // destroy the cursor
        if (!WinDestroyCursor(hWndDeskTop))
          SHOWERROR("WinDestroyCursor", NULL);

        if (!WinShowPointer(hWndDeskTop, TRUE))
          SHOWERROR("WinShowPointer", NULL);

        EXIT_MAIN(szApiEntryName);
     }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreGetPickWindowRc()
 *
 * Purpose: This is an entry point for DTT that does a
 *          return code test of GreGetPickWindow.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 01/17/92 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Device must not be a printer.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              if device = printer then ABORTENTRY();
 *              ENTER_MAIN();
 *              clear last PMERR;
 *
 *              call GreGetPickWindow using -1 for HDC;
 *              check return for PMERR_INV_HDC;
 *
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreGetPickWindowRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
 PCHAR            szApiEntryName = "GreGetPickWindowRc";    // entry point name
 HDC              ThDC;                   // device contexts used
 ERRORID          err;                    // return variable
 HAB              hAB;
 RECTL            PickWin;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else
     {
        if (DeviceIsPrinter())                            // abort if printer is selected
          {
             ABORTENTRY(szApiEntryName,"PRINT");
             return;
          }

        ENTER_MAIN(szApiEntryName);
        SETENTER(szApiEntryName, "Initializations");
        hDC = GpiQueryDevice(hPS);
        ErrorCount = 0;
        if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE)
          {
             CLEANUP("WinQueryAnchorBlock()",szApiEntryName);
             return ;
          }
        WinGetLastError(hAB);
        SETEXIT();
                                                             // check invalid device context handle return code.
        SUBENTER(PMERR_INV_HDC,"HDC = -1");
        ThDC = (HDC) -1;
        err = GreGetPickWindow(ThDC, &PickWin);
        CHECKRC(err, PMERR_INV_HDC, "GreGetPickWindow");
        SUBEXIT();

        EXIT_MAIN(szApiEntryName);
     }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreSetPickWindowRc()
 *
 * Purpose: This is an entry point for DTT that does a
 *          return code test of GreSetPickWindow.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 01/17/92 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              Device must not be a printer.
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              if device = printer then ABORTENTRY();
 *              ENTER_MAIN();
 *              clear last PMERR;
 *              init PickWin;
 *
 *              call GreSetPickWindow using -1 for HDC;
 *              check return for PMERR_INV_HDC;
 *
 *              make HDC busy;
 *              call GreSetPickWindow;
 *              check return for PMERR_HDC_BUSY;
 *              free hdc;
 *
 *              begin area;
 *              call GreSetPickWindow;
 *              check return for PMERR_INV_AREA;
 *              end area;
 *
 *              begin path;
 *              call GreSetPickWindow;
 *              check return for PMERR_INV_PATH;
 *              end path;
 *
 *              set xLeft = 0xFFFFFFFF;
 *              call GreSetPickWindow;
 *              check return for PMERR_COORDINATE_OVERFLOW;
 *              set xLeft = NULL;
 *
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreSetPickWindowRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR            szApiEntryName = "GreSetPickWindowRc";    // entry point name
static HDC              ThDC;                   // device contexts used
static ERRORID          err;                    // return variable
static HAB              hAB;
static RECTL            PickWin;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else
     {
        if (DeviceIsPrinter())                            // abort if printer is selected
          {
             ABORTENTRY(szApiEntryName,"PRINT");
             return;
          }

        ENTER_MAIN(szApiEntryName);
        SETENTER(szApiEntryName, "Initializations");
        hDC = GpiQueryDevice(hPS);
        ErrorCount = 0;
  if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE)
          {
             CLEANUP("WinQueryAnchorBlock()",szApiEntryName);
             return ;
          }
        WinGetLastError(hAB);

        PickWin.xLeft = 0L;
        PickWin.yBottom = 0L;
        PickWin.xRight = MapX(delta_x * pg_size_x);
        PickWin.yTop = MapY(delta_y * pg_size_y);
        SETEXIT();

                                                             // check invalid device context handle return code.
        SUBENTER(PMERR_INV_HDC, "HDC = -1");
        ThDC = (HDC) -1;
        err = GreSetPickWindow(ThDC, &PickWin);
        CHECKRC(err, PMERR_INV_HDC, "GreSetPickWindow");
        SUBEXIT();

                                                             // check device context busy return code.
        SUBENTER(PMERR_HDC_BUSY,NULL);
        MAKEPSBUSY(hPS);
        err = GreSetPickWindow(hDC, &PickWin);
        CHECKRC(err, PMERR_HDC_BUSY, "GreSetPickWindow");
        FREEPS;
        SUBEXIT();

        SUBENTER(PMERR_INV_IN_AREA, NULL);
        if (GpiBeginArea(hPS, BA_ALTERNATE) == GPI_ERROR)
          {
             sprintf(szErr,"GpiBeginArea Error = %x",WinGetLastError(hAB));
             LOGINFO(L_DEBUG,"*",szErr);
          }
        err = GreSetPickWindow(hDC, &PickWin);
        CHECKRC(err, PMERR_INV_IN_AREA, "GreSetPickWindow");
        if (GpiEndArea(hPS) == GPI_ERROR)
          {
             sprintf(szErr,"GpiEndArea Error = %x",WinGetLastError(hAB));
             LOGINFO(L_DEBUG,"*",szErr);
          }
        SUBEXIT();

        SUBENTER(PMERR_INV_IN_PATH, NULL);
        if (GpiBeginPath(hPS, 1L) == GPI_ERROR)
          {
             sprintf(szErr,"GpiBeginPath Error = %x",WinGetLastError(hAB));
             LOGINFO(L_DEBUG,"*",szErr);
          }
        err = GreSetPickWindow(hDC, &PickWin);
        CHECKRC(err, PMERR_INV_IN_PATH, "GreSetPickWindow");
        if (GpiEndPath(hPS) == GPI_ERROR)
          {
             sprintf(szErr,"GpiEndPath Error = %x",WinGetLastError(hAB));
             LOGINFO(L_DEBUG,"*",szErr);
          }
        SUBEXIT();

                                                             // check for coordinate overflow.
        SUBENTER(PMERR_COORDINATE_OVERFLOW, "xRight=134217728L");
        PickWin.xRight = 134217728L;
        err = GreSetPickWindow(hDC, &PickWin);
        CHECKRC(err, PMERR_COORDINATE_OVERFLOW, "GreSetPickWindow");
        SUBEXIT();

                                                             // check invalid position of pick aperture.
        SUBENTER(PMERR_INV_PICK_APERTURE_POSN, NULL);
        PickWin.xLeft = 0xFFFFFFFF;
        err = GreSetPickWindow(hDC, &PickWin);
        CHECKRC(err, PMERR_INV_PICK_APERTURE_POSN, "GreSetPickWindow");
        SUBEXIT();

        EXIT_MAIN(szApiEntryName);
     }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreDisjointLinesRc()
 *
 * Purpose: This is an entry point for DTT that does a
 *          return code test of GreDisjointLines.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 01/17/92 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE
 *
 * Subroutines Required:
 *              NONE
 *
 * Limitations:
 *              NONE
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ENTER_MAIN();
 *              clear last PMERR;
 *              init pointl structure;
 *
 *              call GreDisjointLines using -1 for HDC;
 *              check return for PMERR_INV_HDC;
 *
 *              call GreDisjointLines using 3 for COUNT;
 *              check return for PMERR_INV_LENGTH_OR_COUNT;
 *
 *              call MAKEPSBUSY
 *              call GreDIsjointLines using 4 valid points
 *              call FREEPS
 *              check return for PMERR_HDC_BUSY
 *
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreDisjointLinesRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
 PCHAR            szApiEntryName = "GreDisjointLinesRc";    // entry point name
 HDC              ThDC;                   // device contexts used
 ERRORID          err;                    // return variable
 HAB              hAB;
 POINTL           ptl[10];
// SHORT            i;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else
     {
        ENTER_MAIN(szApiEntryName);
        SETENTER(szApiEntryName, "Initializations");
        hDC = GpiQueryDevice(hPS);
        ErrorCount = 0;
  if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE)
          {
             CLEANUP("WinQueryAnchorBlock()",szApiEntryName);
             return ;
          }
        WinGetLastError(hAB);

        // line one is (0,0) to (0,pg_height)
        // line two is (pg_width, 0) to (pg_width, pg_height)
        // line three is (0,0) to (pg_width, 0)
        // line four is (0, pg_height, pg_width, pg_height)
        // line five is diagonal (0,0) to (pg_width, pg_height)
        ptl[0].x = ptl[0].y = ptl[1].x = ptl[2].y = ptl[4].x = ptl[4].y = 0L;
        ptl[5].y = ptl[6].x = ptl[8].x = ptl[8].y = 0L;
        ptl[1].y = ptl[2].x = ptl[3].x = ptl[5].x = ptl[7].x = pg_width - 1;
        ptl[9].x = pg_width - 1;
        ptl[3].y = ptl[6].y = ptl[7].y = ptl[9].y = pg_height - 1;
        SETEXIT();

                                                             // check invalid device context handle return code.
        SUBENTER(PMERR_INV_HDC, "HDC = -1");
        ThDC = (HDC) -1;
        err = GreDisjointLines(ThDC, &ptl, 2L);
        CHECKRC(err, PMERR_INV_HDC, "GreDisjointLines");
        SUBEXIT();

                                                             // check invalid count.
        SUBENTER(PMERR_INV_LENGTH_OR_COUNT, "Length = 3");
        err = GreDisjointLines(hDC, &ptl, 3L);
        CHECKRC(err, PMERR_INV_LENGTH_OR_COUNT, "GreDisjointLines");
        SUBEXIT();

        SUBENTER(PMERR_HDC_BUSY, NULL);
        MAKEPSBUSY(hPS);
        err = GreDisjointLines(hDC, &ptl, 20L);
        FREEPS;
        CHECKRC(err, PMERR_HDC_BUSY, "GreDisjointLines");
        SUBEXIT();

        SUBENTER(PMERR_COORDINATE_OVERFLOW, "coord=134217728L");
        ptl[0].x = 134217728L;
        err = GreDisjointLines(hDC, &ptl, 4L);
        CHECKRC(err, PMERR_COORDINATE_OVERFLOW, "GreDisjointLines");
        SUBEXIT();

        DeletePrinterOutput(szApiEntryName);
        EXIT_MAIN(szApiEntryName);
     }
   return;
}
