/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/* const char header=$header$ */

#define INCL_DOS
#define INCL_DEV      // include all DEV definitions
#define INCL_WIN
#define INCL_GPI      // include all Gpi definitions
#define INCL_ERRORS   // include all Gpi error definitions
#define INCL_WINP_SEI
#define INCL_WINHEAP
#define INCL_WINPOINTERS

#include <os2.h>

#define INCL_GREALL   // include all Gre definitions
#define INCL_DDIMISC
#define INCL_DDIBUNDLES

#include <pmddi.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <stdio.h>
#include <math.h>
#include "DTTDLLDF.h"               // test case defines
#include "common1.h"
#include "common2.h"
#include "gregen2x.h"

/****************************************************************************
 *
 * Subtask Name: GreGen2.dll
 *
 * Module name:  gregen2x.c
 *
 * Purpose:      module for GREGEN2 exhautive entry points
 *
 * Revision Log: 12/17/91,      Graphics Suite Extension, Ken McFadden
 *                               Initial Design.
 *               02/19/92       Updates to GreSetColorCursorExh, Thom Bain
 *
 * Documentation Reference:  Section
 *
 * Contents:     GreSetColorCursorExh
 *               GreGetandSetPickWindowExh
 *               GreDisjointLinesExh
 *
****************************************************************************/

static struct {
        SHORT sPtrId;
        CHAR *szPtrId;
        }
        ptrs[] = {
      SPTR_HANDICON, "SPTR_HANDICON",
      SPTR_QUESICON, "SPTR_QUESICON",
      SPTR_BANGICON, "SPTR_BANGICON",
      SPTR_NOTEICON, "SPTR_NOTEICON"
                } ;

static CHAR *szIconId[] = {
      "GUMBY",
      "TOMATO",
      "RUNDR11",
      "HOOK",
      "UNHOOK",
      "BRIEF",
      "CLOCK",
      "CM2",
      "NET",
      "INTRO",
      "HOUSE",
      "CALENDAR",
      "PEGBOARD"
   };

static struct {
        LONG lColorIndex;
        CHAR *szColorIndex;
        }
        clr[] = {
                CLR_RED,       "CLR_RED           ",
                CLR_DARKPINK,   "CLR_DARKPINK      ",
                CLR_PINK,      "CLR_PINK          ",
                CLR_WHITE,     "CLR_WHITE         ",
                CLR_YELLOW,     "CLR_YELLOW        ",
                CLR_CYAN,      "CLR_CYAN          ",
                CLR_DARKCYAN , "CLR_DARKCYAN      ",
                CLR_GREEN,     "CLR_GREEN         ",
                CLR_DARKGREEN, "CLR_DARKGREEN     ",
                CLR_DARKBLUE,   "CLR_DARKBLUE      ",
                CLR_BLUE,      "CLR_BLUE          ",
                CLR_PALEGRAY,   "CLR_PALEGRAY      ",
                CLR_DARKGRAY,   "CLR_DARKGRAY      ",
                CLR_BROWN,     "CLR_BROWN         "
                } ;     // 14 colors

static struct {
        LONG lBWIndex;
        CHAR *szBWIndex;
        }
        BW[] = {
                CLR_TRUE,   "CLR_TRUE  ",
                CLR_FALSE,  "CLR_FALSE "
                } ;     // 2 colors

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreSetColorCursorExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreSetColorCursor function
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 01/02/92 ,DTT, Ken McFadden
 *               02/19/92, Update GreSetColorCursorExh
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              4 system pointers and 13 icons in resource file in center of window
 *
 * Subroutines Required:
 *    WinSendMsg, DeviceIsPrinter, ABORTENTRY, ENTER_MAIN, SETENTER, SETEXIT,
 *    GpiSetMix, GpiSetBackMix, GpiQueryDevice, WinQueryAnchorBlock,
 *    WinGetLastError, WinQueryDesktopWindow, WinShowPointer, WinQueryWindow,
 *    WinQueryWindowPos, WinCreateCursor, WinSetPointerPos,
 *    WinQueryCursorInfo, GpiConvert, WorldWriteString, WinQuerySysPointer,
 *    WinQueryPointerInfo, sprintf, LOGINFO, ERRSTATUS, SHOWERROR,
 *    GreSetColorCursor, strcpy, strcat, WinShowCursor, DosLoadModule,
 *    WinLoadPointer, DosSleep, EXIT_MAIN
 *
 * Limitations:
 *    Environmental restriction to display devices only
 *
 *\\end
 *****************************************************************************/

/****************************************************************************
 *\\algorithm
 *
 *      if device is printer then ABORTENTRY
 *      setup
 *      {
 *         get the device context handle, anchor block handle and clear errors
 *         get the DESKTOP window handle
 *         hide the system pointer
 *         get DTT frame window handle (to get relative position of window
 *           on the screen for setting the cursor position)
 *         get the size and location of the frame (relative to its parent)
 *         calculate where the center of DTT window is in screen coords
 *         create a cursor there
 *         set DESKTOP pointer to there also
 *         calculate page coordinates to put text beside the pointer/icon
 *         write to screen the current pointer is the SYSTEM POINTER
 *      }
 *      process the four system  pointers that have compatible format for
 *        cursors (prevent PMERR_INV_CURSOR_BITMAP). Their IDs are stored
 *        static global array "ptrs"
 *      for each of the four
 *      {
 *        get the handle for the system pointer
 *        query for the pointer information (bitmap masks and color)
 *        if compatible format
 *        {
 *          query for the window location and size and calculate where the
 *            center is in screen coordinates
 *          move cursor/pointer to this location
 *          call the function under test to load the cursor with the bitmap
 *          query the cursor information to determine its size
 *          use window location and cursor size to calculate where to put
 *            descriptive text beside the pointer/icon
 *          make cursor visible
 *          wait 1/2 second
 *          make curor invisible
 *        }
 *        else
 *          log that incompatible format found
 *      }
 *      load the resource module handle for fetching icon from resource file
 *      process the 13 icons loaded in the DLL as resource
 *      for each of the thirteen
 *      {
 *        get the handle for the icon/pointer (load the icon)
 *        query for the pointer information (bitmap masks and color)
 *        if compatible format
 *        {
 *          query for the window location and size and calculate where the
 *            center is in screen coordinates
 *          move cursor/pointer to this location
 *          call the function under test to load the cursor with the bitmap
 *          query the cursor information to determine its size
 *          use window location and cursor size to calculate where to put
 *            descriptive text beside the pointer/icon
 *          make cursor visible
 *          wait 1/2 second
 *          make curor invisible
 *        }
 *        else
 *          log that incompatible format found
 *        Destroy the pointer in preparation for next to be loaded
 *      }
 *      cleanup - restore the system pointer and destroy the cursor
 *
 *\\end
 ***************************************************************************/

#pragma page()
VOID  APIENTRY GreSetColorCursorExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR            szApiEntryName = "GreSetColorCursorExh";   // entry point name
static POINTERINFO      ptrinfo;   // store info on pointer/cursor bitmap
static SHORT            i;         // general for loop control
static HPOINTER         hptr,      // handle for system pointers
                        hIcon;     // handle for icons loaded from DLL
       CHAR             Buffer[100];  // character buffer for logging
       HWND             hWndDeskTop, // handle for the DESKTOP =>HWND_DESKTOP
                        hWndFrame;   // handle for DTT frame
       HAB              hAB;       // anchor block handle
       HMODULE          hMOD;      // module handle for loading icons
       SWP              swp;       // struct to hold window/frame parameters
       SHORT            x, y;      // screen coords for center of DTT window
       POINTL           ptlText; // location to write ptr/icon descript
       CURSORINFO       csri;      // for queries to check & position text
       PCHAR            szBlanks = "                    ";
       CHAR             ModuleName [ 24 ];

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    if (DeviceIsPrinter())                            // abort if printer is selected
    {
      ABORTENTRY(szApiEntryName,"PRINT");
      return;
    }

    ENTER_MAIN(szApiEntryName);

    SETENTER(szApiEntryName, "Initializations");

    if (!GpiSetBackMix(hPS, BM_OVERPAINT))
      SHOWERROR("GpiSetBackMix", NULL);
    if (!GpiSetMix(hPS, FM_OVERPAINT))
      SHOWERROR("GpiSetMix", NULL);

    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", szApiEntryName);
      return;
    }

    //Get Anchor Block.
     if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",szApiEntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ErrorCount = 0;

    // get the desktop window handle for pointer and cursor manipulation
 hWndDeskTop = WinQueryDesktopWindow(hAB, NULLHANDLE);

    // hide the system pointer
    if (!WinShowPointer(hWndDeskTop, FALSE))
      SHOWERROR("WinShowPointer", "Removing sytem pointer");

    // get window handle for DTT frame (as WinQueryWindowPos returns
    // info relative to parent window, so hWndClient would not give
    // enough information to find the center of its window in screen coords.
    hWndFrame = WinQueryWindow(hWndClient, QW_PARENT);

    // calc. screen coordinates for center of DTT window.
    // get the DTT frame location, it will be relative to the desktop
    if (!WinQueryWindowPos(hWndFrame, &swp))
      SHOWERROR("WinQueryWindowPos", NULL);

    x = (SHORT)(swp.x + (swp.cx / 2));
    y = (SHORT)(swp.y + (swp.cy / 2));

    // create a cusor
    if(!WinCreateCursor(hWndDeskTop, x, y, 0, 0, CURSOR_SOLID, NULL))
      SHOWERROR("WinCreateCursor", "Create");

    // position the pointer/cursor in the middle of DTT window
    if (!WinSetPointerPos(hWndDeskTop, x, y))
      SHOWERROR("WinSetPointerPos", NULL);

    // get cursor size
    if (!WinQueryCursorInfo(hWndDeskTop, &csri))
      SHOWERROR("WinQueryCursorInfo", NULL);
    ptlText.x = (LONG)(swp.cx / 2 + csri.cx + 10);
    ptlText.y = (LONG)(swp.cy / 2 + csri.cy / 2);
    if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_PAGE, 1L, &ptlText))
      SHOWERROR("GpiConvert", NULL);
    ptlText.y -= syPosition;
    // write out description beside the cursor/icon
    WorldWriteString(ptlText.x, ptlText.y, "SYSTEM POINTER ");

    SETEXIT();

    TEST_PROGRESS ( 00, NUM_PTRS + NUM_ICONS );

    // process the four system pointers that have a compatible format
    // for color cursors (icon size bitmap)
    for (i = 0; i < NUM_PTRS; i++)
    {
      // get handle to a system pointer (icon size format)
      // previous testing reveal these four from list of 14 (see pmwin.h)
      hptr = WinQuerySysPointer(hWndDeskTop, ptrs[i].sPtrId, FALSE);
    if (hptr == NULLHANDLE)
      {
        sprintf(Buffer, "WinQuerySysPointer FAILED for %s",
                                                          ptrs[i].szPtrId);
        LOGINFO(L_TRACE, "*", Buffer);
        continue;
      }
      // get pointer bitmaps - masks and color info
      if (!WinQueryPointerInfo(hptr, &ptrinfo))
      {
        SHOWERROR("WinQueryPointerInfo", "From Sys Pointer");
      }

      // if this is an icon bitmap pointer
      if (ptrinfo.fPointer == FALSE)
      {
        // calc. screen coordinates for center of DTT window.
        // get the DTT frame location, it will be relative to the desktop
        if (!WinQueryWindowPos(hWndFrame, &swp))
          SHOWERROR("WinQueryWindowPos", NULL);

        x = (SHORT)(swp.x + (swp.cx / 2));
        y = (SHORT)(swp.y + (swp.cy / 2));

        // move current cusor
        if(!WinCreateCursor(hWndDeskTop, x, y, 0, 0, CURSOR_SETPOS, NULL))
          SHOWERROR("WinCreateCursor", "Moving");

        if (!WinSetPointerPos(hWndDeskTop, x, y))
          SHOWERROR("WinSetPointerPos", NULL);

        if (!GreSetColorCursor(hDC, &ptrinfo))
        {
          ErrorCount++;
        }
        ERRSTATUS(TRUE, "GreSetColorCursor", szApiEntryName);

        // get cursor size
        if (!WinQueryCursorInfo(hWndDeskTop, &csri))
          SHOWERROR("WinQueryCursorInfo", NULL);
        ptlText.x = (LONG)(swp.cx / 2 + csri.cx + 10);
        ptlText.y = (LONG)(swp.cy / 2 + csri.cy / 2);
        if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_PAGE, 1L, &ptlText))
          SHOWERROR("GpiConvert", NULL);
        ptlText.y -= syPosition;
        strcpy(Buffer, ptrs[i].szPtrId);
        strcat(Buffer, szBlanks);
        // write out description beside the cursor/icon
        WorldWriteString(ptlText.x, ptlText.y, Buffer);

        if (!WinShowCursor(hWndDeskTop, TRUE))
          SHOWERROR("WinShowCursor", "ON");

        DosSleep(500L);

        if (!WinShowCursor(hWndDeskTop, FALSE))
          SHOWERROR("WinShowCursor", "OFF");
      }
      else
      {
        LOGINFO(L_TRACE, "*", "Invalid Bitmap Format, must be ICON sized");
      }

      TEST_PROGRESS ( i + 1, NUM_PTRS + NUM_ICONS );
    }

    strcpy ( ModuleName, TestCaseName );
    strcat ( ModuleName, ".DLL" );

    //Ask OS/2 for the module handle for this DLL.
    if ( DosQueryModuleHandle ( ModuleName, & hMOD ) )
    {
       SHOWERROR("DosQueryModuleHandle",szApiEntryName);
       LOGINFO(L_TRACE, "*", "GREGEN2.DLL must be in current directory\n");
    }

    for (i = 0; i < NUM_ICONS; i++)
    {
      // attempt to load an icon
      hIcon = WinLoadPointer(HWND_DESKTOP, hMOD, ID_ICON1+i);
    if (hIcon == NULLHANDLE)
      {
        SHOWERROR("WinLoadPointer", NULL);
        continue;
      }

      if (!WinQueryPointerInfo(hIcon, &ptrinfo))
      {
        SHOWERROR("WinQueryPointerInfo", "From Icon Resource");
        continue;
      }

      // if this is an icon bitmap pointer
      if (ptrinfo.fPointer == FALSE)
      {

        if (!WinQueryWindowPos(hWndFrame, &swp))
          SHOWERROR("WinQueryWindowPos", NULL);

        x = (SHORT)(swp.x + (swp.cx / 2));
        y = (SHORT)(swp.y + (swp.cy / 2));

        // move current cusor
        if(!WinCreateCursor(hWndDeskTop, x, y, (SHORT)0, (SHORT)0,
                                                        CURSOR_SETPOS, NULL))
          SHOWERROR("WinCreateCursor", "Moving");

        if (!WinSetPointerPos(hWndDeskTop, x, y))
          SHOWERROR("WinSetPointerPos", NULL);

        if (!GreSetColorCursor(hDC, &ptrinfo))
        {
          ErrorCount++;
        }
        ERRSTATUS(TRUE, "GreSetColorCursor", szApiEntryName);

        // get cursor size
        if (!WinQueryCursorInfo(hWndDeskTop, &csri))
          SHOWERROR("WinQueryCursorInfo", NULL);
        ptlText.x = (LONG)(swp.cx / 2 + csri.cx + 10);
        ptlText.y = (LONG)(swp.cy / 2 + csri.cy / 2);
        if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_PAGE, 1L, &ptlText))
          SHOWERROR("GpiConvert", NULL);
        ptlText.y -= syPosition;
        strcpy(Buffer, szIconId[i]);
        strcat(Buffer, szBlanks);
        // write out description beside the cursor/icon
        WorldWriteString(ptlText.x, ptlText.y, Buffer);

        if (!WinShowCursor(hWndDeskTop, TRUE))
          SHOWERROR("WinShowCursor", "ON");

        DosSleep(500L);

        if (!WinShowCursor(hWndDeskTop, FALSE))
          SHOWERROR("WinShowCursor", "OFF");
      }
      else
      {
        LOGINFO(L_TRACE, "*", "Invalid Bitmap Format, must be ICON sized");
      }
      // destroy the icon in preparation to fetch next one
      if (!WinDestroyPointer(hIcon))
        SHOWERROR("WinDestroyPointer", NULL);

      TEST_PROGRESS ( i + NUM_PTRS + 1, NUM_PTRS + NUM_ICONS );
    }

    if (!WinQueryWindowPos(hWndFrame, &swp))
      SHOWERROR("WinQueryWindowPos", NULL);

    // get cursor size
    if (!WinQueryCursorInfo(hWndDeskTop, &csri))
      SHOWERROR("WinQueryCursorInfo", NULL);
    ptlText.x = (LONG)(swp.cx / 2 + csri.cx + 10);
    ptlText.y = (LONG)(swp.cy / 2 + csri.cy / 2);
    if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_PAGE, 1L, &ptlText))
      SHOWERROR("GpiConvert", NULL);
    ptlText.y -= syPosition;
    // write out description beside the cursor/icon
    WorldWriteString(ptlText.x, ptlText.y, "SYSTEM POINTER ");

    // destroy the cursor
    if (!WinDestroyCursor(hWndDeskTop))
      SHOWERROR("WinDestroyCursor", NULL);

    if (!WinShowPointer(hWndDeskTop, TRUE))
      SHOWERROR("WinShowPointer", NULL);

    EXIT_MAIN(szApiEntryName);
  }
  return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreSetandGetPickWindowExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreSetPickWindow and GreGetPickWindow.
 *
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 01/16/92 ,DTT, Ken McFadden
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              NONE except logging
 *
 * Subroutines Required:
 *              WinSendMsg, ENTER_MAIN, EXIT_MAIN, DeviceIsPrinter, ABORTENTRY,
 *    GpiQueryDevice, srand, MapX, MapY, SUBENTER, SUBEXIT, randL, sprintf,
 *    ERRSTATUS, LOGINFO, GreSetPickWindow, GreGetPickWindow, memcmp
 *
 * Limitations:
 *    output device must be a display
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *              ENTER_MAIN();
 *              ensure device is not a printer;
 *              set randomizer to pre-selected seed;
 *              loop from MINLOOP to MAXLOOP;
 *                      PickWin = random size;
 *                      call GreSetPickWindow();
 *                      call GreGetPickWindow();
 *                      check set pick window with the queried pick window;
 *              EXIT_MAIN();
 *      }
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreGetandSetPickWindowExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
static PCHAR            szApiEntryName = "GreGetandSetPickWindowExh";   // entry point name
static CHAR             szPickWin[50];
static RECTL            sPickWin, qPickWin;
static LONG             maxX, maxY;
register INT            lvar;                                 // Loop varable.

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else
     {
        if (DeviceIsPrinter())                            // abort if printer is selected
          {
             ABORTENTRY(szApiEntryName,"PRINT");
             return;
          }

        ENTER_MAIN(szApiEntryName);
        SETENTER(szApiEntryName, "Initializations");
        ErrorCount = 0;
        hDC = GpiQueryDevice(hPS);
        srand(seed);
        maxX = MapX(delta_x * pg_size_x);
        maxY = MapY(delta_y * pg_size_y);
        SETEXIT();

        SUBENTER("Main Test:", NULL);
        for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)                  // Enter main test loop.
          {
             sPickWin.xLeft = randL(0L, maxX);
             sPickWin.yBottom = randL(0L, maxY);
             sPickWin.xRight = randL(sPickWin.xLeft, maxX);
             sPickWin.yTop = randL(sPickWin.yBottom, maxY);
             sprintf(szPickWin,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                     sPickWin.xLeft,
                                                     sPickWin.yBottom,
                                                     sPickWin.xRight,
                                                     sPickWin.yTop);

             if (!GreSetPickWindow(hDC, &sPickWin))
               {
                  ErrorCount++;
                  ERRSTATUS(TRUE, "GreSetPickWindow:", NULL);
                  LOGINFO(L_DEBUG,"*",szPickWin);
               }

             if (!GreGetPickWindow(hDC, &qPickWin))
               {
                  ErrorCount++;
                  ERRSTATUS(TRUE, "GreGetPickWindow:", NULL);
               }
             else
                if(memcmp(&sPickWin, &qPickWin, sizeof(RECTL)))      // compare current parameters
                  {                                                  // with set parameters.
                     sprintf(szPickWin,"x = %ld, y = %ld, x = %ld, y = %ld",
                                                             qPickWin.xLeft,
                                                             qPickWin.yBottom,
                                                             qPickWin.xRight,
                                                             qPickWin.yTop);
                     ErrorCount++;
                     ERRSTATUS(TRUE,szDontMatch,szPickWin);
                  }
          }
        SUBEXIT();
        EXIT_MAIN(szApiEntryName);
     }
   return;
}

#pragma page()
/*****************************************************************************
 *\ddd
 * Routine Name: GreDisjointLinesExh()
 *
 * Purpose: This is an entry point for the DTT that does an
 *          exhaustive test of GreDisjointLines.
 *
 *
 * System Requirements:
 *              NONE
 *
 *
 * Revision Log: 01/28/92 ,DTT, Ken McFadden
 *               02/21/92, Udates (use exclude clip & adjust for DC origin)
 *                        and added preambles and comments.
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *              random disjoint lines in random colors (fewer lines and black and
 *    white for printers)
 *
 * Subroutines Required:
 *              WinSendMsg, ENTER_MAIN, SETENTER, GpiQueryDevice, GreQueryDCOrigin,
 *    DeviceIsPrinter, GpiExcludeClipRectangle, CLEANUP, srand, SETEXIT,
 *    SHOWERROR, sprintf, SUBENTER, GpiConvert, randI, GpiSetColor, randL,
 *    GreDisjointLines, ERRSTATUS, LOGINFO, SETEXIT, GpiSetClipRegion,
 *    GpiDestroyRegion, EXIT_MAIN
 *
 * Limitations:
 *    OD_QUEUED/PM_Q_STD seems to work but there is no output to the device
 *    even though the function does not fail
 *
 *    When ever the output device is a printer the minimum and maximum
 *    coordinate values "appear" to be -32000L and +32000L respectively.
 *    For the display the standard -32768L and +32767L apply. If allowed
 *    to use the display limits the function seems to "hang".
 *
 *\\end
 *****************************************************************************/
#pragma page()
/*****************************************************************************
 *\\algorithm
 * {
 *      if (SelectionCall)
 *              WinSendMsg ();
 *      else {
 *   setup
 *   {
 *     query for the device context
 *     initialize random seed
 *     exclude a clip rectangle to protect the SIGNATURE block
 *     determine the DC origin to help calc. ranges. (really only important
 *       to displays as the window can be resized and moved).
 *     establish min and max ranges based on device type (see limitations)
 *   }
 *   for each of three ranges
 *   {
 *     establish the range variables (MaxRangeX, MinRangeX, MaxRangeY, and
 *       MinRangeY) for random point coordinate generation in this range)
 *     for MAXLOOP times do
 *     {
 *       randomly establish the number of points/lines for this iteration
 *         (NOTE: max is 10 lines for printers and 50 for displays - this
 *         speeds up output to printers).
 *       randomlly generate enough points in this range.
 *       call the function under test to output the "disjoint" lines.
 *     }
 *   }
 *   cleanup - release and destroy the clip region.
 * }
 *\\end
 *****************************************************************************/
#pragma page()
VOID  APIENTRY GreDisjointLinesExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PCHAR    szApiEntryName = "GreDisjointLinesExh";
  POINTL   aptl[2 * MAX_LINES_DISPLAY], O;
  CHAR     szParm[50];
  RECTL    rect;
  HRGN     hrgn;
  INT      lvar, RangeLoop, sColor, i;
  LONG     MinRangeX, MinRangeY, MaxRangeX, MaxRangeY, NumPoints;
  LONG     MaxValue, MinValue;
  BOOL     bSuccess;
  USHORT   progress;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    ENTER_MAIN(szApiEntryName);

    SETENTER(szApiEntryName, "Initializations");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", szApiEntryName);
      return;
    }

    srand(seed);                    // Preset seed for randomization.

    if (!GreGetDCOrigin(hDC, &O))
      SHOWERROR("GreGetDCOrigin", NULL);

    // printers seem to have a smaller limit than the display
    if (DeviceIsPrinter())
    {
      MaxValue =  32000L;
      MinValue = -32000L;
    }
    else
    {
      MaxValue =  32767L;
      MinValue = -32768L;
    }

    TEST_PROGRESS ( 00, 3 * MAXLOOP );

    // create a clip region so output doesn't cover signature block
    rect.xLeft   = 0L;
    rect.yBottom = 0L;
    rect.xRight  = pg_width - 1;          // Device width
    rect.yTop    = MapY(70L);      // 0.7 inch
    if (GpiExcludeClipRectangle(hPS, &rect) == RGN_ERROR)
      SHOWERROR("GpiExcludeClipRectangle", NULL);
    SETEXIT();

    // for each of the three ranges run inner loop test
    for (RangeLoop = 0; RangeLoop < 3; RangeLoop++)
    {
      sprintf(szAddMessage,"Range %d",RangeLoop+1);
      SUBENTER(GreDisjointLinesExh, szAddMessage);

      // setup range control varialbles
      switch (RangeLoop) {
        case 0:  // maximum range, force landing on valid/legal device coord
          aptl[0].x = MinValue - O.x;
          aptl[0].y = MinValue - O.y;
          aptl[1].x =  MaxValue - O.x;
          aptl[1].y =  MaxValue - O.y;
          if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_PAGE, 2L, &aptl[0]))
            SHOWERROR("GpiConvert", NULL);
          if (aptl[0].x < MinValue) aptl[0].x = MinValue;
          if (aptl[0].y < MinValue) aptl[0].y = MinValue;
          if (aptl[1].x > MaxValue) aptl[1].x =  MaxValue;
          if (aptl[1].y > MaxValue) aptl[1].y =  MaxValue;
          MinRangeX = aptl[0].x;
          MinRangeY = aptl[0].y;
          MaxRangeX = aptl[1].x;
          MaxRangeY = aptl[1].y;
          break;
        case 1:  // middle range - plus/minus twice the device size
          MinRangeX = -2 * pg_width;
          MinRangeY = -2 * pg_height;
          MaxRangeX =  2 * pg_width;
          MaxRangeY =  2 * pg_height;
          break;
        case 2:  // concentrate on the visible device
          MinRangeX = 0L;;
          MinRangeY = 0L;
          MaxRangeX = pg_width - 1;
          MaxRangeY = pg_height - 1;
          break;
      } /* end switch */

      for (lvar=MINLOOP;lvar<=MAXLOOP;lvar++)       // Enter test loop.
      {
         // set number of points and pick a color. The MAX number of
         // disjoint lines is reduced for the printer and the color
         // is randomly picked between black and white. This is to avoid
         // cluttered output and speed the test up on printers.
         if (DeviceIsDisplay())
         {
           NumPoints = 2 * randI(0, MAX_LINES_DISPLAY);
           sColor = randI(0, 13);
           if (!GpiSetColor(hPS, clr[sColor].lColorIndex))
             SHOWERROR("GpiSetColor:", clr[sColor].szColorIndex);
         }
         else
         {
           NumPoints = 2 * randI(0, MAX_LINES_PRINTER);
           sColor = randI(0,1);
           if ( GpiSetColor(hPS, BW[sColor].lBWIndex) == FALSE)
             SHOWERROR("GpiSetColor:", BW[sColor].szBWIndex);
         }
         // generate random points
         for (i = 0; i < (INT)NumPoints; i++)
         {
            aptl[i].x = randL(MinRangeX, MaxRangeX);
            aptl[i].y = randL(MinRangeY, MaxRangeY);
         } /* end for i */

         // output the lines and log error and line points on failure
         bSuccess = GreDisjointLines(hDC, &aptl[0], NumPoints);
         if (!bSuccess)
         {
           ErrorCount++;
           ERRSTATUS(TRUE,"GreDisjointLines:", NULL);
           sprintf(szParm, "%d Lines", NumPoints/2);
           LOGINFO(L_TRACE, "*", szParm);
           for (i = 0; i < (INT)NumPoints; i += 2)
           {
             sprintf(szParm,"x=%ld, y=%ld to x=%ld, y=%ld",
                    aptl[i].x, aptl[i].y, aptl[i+1].x, aptl[i+1].y);
             LOGINFO(L_TRACE, "*", szParm);
           } /* for i */
         } /* if */

         progress = RangeLoop * MAXLOOP + lvar;

         TEST_PROGRESS ( progress, 3 * MAXLOOP );
      }
      SUBEXIT();
    }

    // cleanup - release and destroy the clip region
    GpiSetClipRegion(hPS,NULLHANDLE,&hrgn); //16KQ
    GpiDestroyRegion(hPS,hrgn);
    EXIT_MAIN(szApiEntryName);
  }
 return;
}
