/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GREGENFC.C")
#pragma subtitle("IBM Confidential")

/*const char header=$header$*/

#define INCL_GREALL
#define INCL_WIN
#define INCL_GPI
#define INCL_ERRORS
#define INCL_DEV
#include <os2.h>
#include <pmddi.h>
#include "DTTDLLDF.h" // test case defines
#include "gregenfc.h"  /* gre general functions include file */
#include "std_util.h"

/****************************************************************************
 *\\ddd
 * Subtask Name: pdvt
 *
 * Module name: gregenfc.c
 *
 * Purpose: This module is called by the DTT to test the GRE General
 *          functions.
 *
 * Revision Log: 10/31/90, PDVT, Gisselle Labault
 *       - created initial module template
 *
 * SRS References: Section 6.12
 *
 * Contents:    InitTest()
 *      GreLockDeviceExh()
 *      GreUnlockDeviceExh()
 *      GreErasePS()
 *      GreLockDevRC()
 *      GreUnlockDevRC()
 *      GreErasePSRC()
 *
 * Design Overview: GRE general functions test case includes GreLockDevice,
 *          GreUnlockDevice and GreErasePS. The test includes
 *          functionality and test error return codes related to the
 *          printer device driver.
 *
 * Limitations: none
 *
 *\\end
****************************************************************************/

/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose: Initialize function name and address array for DTT.
 *
 * System Requirements: None
 *
 * Revision Log: 10/31/90, PDVT, Gisselle Labault
 *       - initial function template
 *
 * Inputs:
 *   sel      // passed through to the InitTestSegVars function
 *   pInitInfo  // name and address array used by the DTT interface
 *
 * Outputs:
 *   pInitInfo  // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()  // used by the DTT interface
 *
 * Limitations:
 *   pInitInfo is of size ENTRYPOINTS defined in DTT.h and DTTdlldf.h (both)
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *   for all test functions in this test case {
 *  pInitInfo->function name = function name[i]
 *  pInitInfo->function address = function address[i]
 *   }
 *   call init set function
 * }
 *\\end
 ***************************************************************************/
VOID  APIENTRY InitTest(PVOID sel, PTESTINITINFO pInitInfo)
  {
    pInitInfo->ntests = NTESTS; /* set number of entry points */

    /* set each function name and address */
    strcpy(pInitInfo->group_name, "GRE General");
    strcpy(pInitInfo->test_names[0], "GreLockDeviceExh");
    pInitInfo->test_addrs[0]=GreLockDeviceExh;
    strcpy(pInitInfo->test_names[1], "GreUnlockDeviceExh");
    pInitInfo->test_addrs[1]=GreUnlockDeviceExh;
    strcpy(pInitInfo->test_names[2], "GreErasePSExh");
    pInitInfo->test_addrs[2]=GreErasePSExh;
    strcpy(pInitInfo->test_names[3], "GreLockDeviceRC");
    pInitInfo->test_addrs[3]=GreLockDeviceRC;
    strcpy(pInitInfo->test_names[4], "GreUnlockDeviceRC");
    pInitInfo->test_addrs[4]=GreUnlockDeviceRC;
    strcpy(pInitInfo->test_names[5], "GreErasePSRC");
    pInitInfo->test_addrs[5]=GreErasePSRC;

    InitTestSegVars(sel); /* this call used by the DTT interface */
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreLkDevExh()
 *
 * Purpose: This function tests functionality of GreLockDevice.
 *      The syntax used to call this function is:
 *      (BOOL) GreLockDevice(
 *         HDC      hdc,
 *         ULONG    pInstance,
 *         ULONG    lFunction)
 *
 *      GreLockDevice locks a device for use by a single thread.
 *
 * System Requirements: Section 6.12.3
 *
 * Revision Log: 10/31/90, PDVT, Gisselle Labault
 *       - created initial template
 *       1/2/91, PDVT, Gisselle Labault
 *       - testcase design
 *
 * Inputs:
 *   hPS        // presentation space
 *   hWndClient     // client window
 *   SelectionCall  // selection parameter
 *
 * Outputs: none
 *
 * Subroutines Required:
 *   LOG_SIGNATURE, CHECKRETURNERROR(),CWRITELOG(),
 *   WinSendMsg(), DevOpenDC(), WinQueryAnchorBlock()
 *
 * Limitations: none
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *   hAB=WinQueryAnchorBlock(hWndClient)
 *   WinSendMsg(hWndClient,...)
 *   printer_hdc = DevOpenDC(hAB,...,NULL)
 *   if (GreLockDevice(printer_hdc))
 *     GreUnlockDevice(printer_hdc)
 *   endif
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreLockDeviceExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC           printer_hdc;         /* device context handle */
  INT           gre_returned_value;
  INT           error_count;         /* contains number of test that fail */
  DEVOPENSTRUC  dop;                 /* device open info structure */
  DRIVDATA      driv;                /* structure for dop and DevOpenDC */
  LONG          lDCType;             /* used by WinSendMsg */
  HAB           hAB;                 /* anchor block handle */
  CHAR          str_msg[150];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreLockDevice - Entry\n");
    error_count = 0;
    hAB = WinQueryAnchorBlock(hWndClient);
    dop.pdriv = &driv;
    WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
    printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULLHANDLE);
    if (printer_hdc == DEV_ERROR) {
       BADERROR("DevOpenDC", TRUE, "GreLockDeviceExh");
       return;
    }
    if (!(gre_returned_value=GreLockDevice(printer_hdc)))
      error_count +=  CHECKRETURNERROR(gre_returned_value, "GreLockDevice", SEVERITY_ERROR, PMERR_OK);
    else
      if (!(gre_returned_value=GreUnlockDevice(printer_hdc)))
        error_count +=  CHECKRETURNERROR(gre_returned_value, "GreUnlockDevice", SEVERITY_ERROR, PMERR_OK);

    if (!error_count)
      CWRITELOG(L_HDR,0,
"$GreLockDevice                                                  -    PASS.\n");
    else {
         sprintf(str_msg, "$ErrorCount = %d.\n", error_count);
         CWRITELOG(L_HDR, 0, str_msg);
         CWRITELOG(L_HDR,0,
"$GreLockDevice                                                  -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GreLockDevice - Exit\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULLHANDLE
                                      ,NULL, NULL,NULL);
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreUlkDevExh()
 *
 * Purpose: This function tests functionality of GreUnlockDevice.
 *      The syntax used to call this function is:
 *      (BOOL) GreUnlockDevice(
 *         HDC      hdc,
 *         ULONG    pInstance,
 *         ULONG    lFunction)
 *
 *      GreUnlockDevice allows all pending screen input/output operations,
 *      blocked by GreLockDevice, to continue.
 *
 * System Requirements: Section 6.12.3
 *
 * Revision Log: 10/31/90, PDVT, Gisselle Labault
 *       - created initial template
 *       1/2/91, PDVT, Gisselle Labault
 *       - testcase design
 *
 * Inputs:
 *   hPS        // presentation space
 *   hWndClient     // client window
 *   SelectionCall  // selection parameter
 *
 * Outputs: none
 *
 * Subroutines Required:
 *   LOG_SIGNATURE, CHECKRETURNERROR(),CWRITELOG(),
 *   WinSendMsg(), DevOpenDC(), WinQueryAnchorBlock()
 *
 * Limitations: none
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *   hAB=WinQueryAnchorBlock(hWndClient)
 *   WinSendMsg(hWndClient,...)
 *   printer_hdc = DevOpenDC(hAB,...,NULL)
 *   if (GreLockDevice(printer_hdc))
 *     GreUnlockDevice(printer_hdc)
 *   endif
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreUnlockDeviceExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC           printer_hdc;         /* device context handle */
  INT           gre_returned_value;
  INT           error_count;         /* contains number of test that fail */
  DEVOPENSTRUC  dop;                 /* device open info structure */
  DRIVDATA      driv;                /* structure for dop and DevOpenDC */
  LONG          lDCType;             /* used by WinSendMsg */
  HAB           hAB;                 /* anchor block handle */
  CHAR          str_msg[150];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE, 0, "+ee+ GreUnlockDevice - Entry\n");
        error_count = 0;
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULLHANDLE);
        if (printer_hdc == DEV_ERROR) {
            BADERROR("DevOpenDC", TRUE, "GreUlkDevExh");
            return;
        }
        if (!(gre_returned_value=GreLockDevice(printer_hdc)))
           error_count +=  CHECKRETURNERROR(gre_returned_value, "GreLockDevice", SEVERITY_ERROR, PMERR_OK);
        else
            if (!(gre_returned_value=GreUnlockDevice(printer_hdc)))
               error_count +=  CHECKRETURNERROR(gre_returned_value, "GreUnlockDevice", SEVERITY_ERROR, PMERR_OK);

        if (!error_count)
           CWRITELOG(L_HDR,0,
"$GreUnlockDevice                                                -    PASS.\n");
        else {
             sprintf(str_msg, "$ErrorCount = %d.\n", error_count);
             CWRITELOG(L_HDR, 0, str_msg);
             CWRITELOG(L_HDR,0,
"$GreUnlockDevice                                                -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE, 0, "+ex+ GreUnlockDevice - Exit\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
         DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULLHANDLE
                                      ,NULL, NULL,NULL);
    }
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreErasePSExh()
 *
 * Purpose: This function tests functionality of GreErasePS.
 *      The syntax used to call this function is:
 *      (BOOL) GreErasePS(
 *         HDC      hdc,
 *         ULONG    pInstance,
 *         ULONG    lFunction)
 *
 *      GreErasePS resets the device context's presentation space to the
 *      background color(CLR_BACKGROUND).
 *
 * System Requirements: Section 6.12.3
 *
 * Revision Log: 10/31/90, PDVT, Gisselle Labault
 *       - created initial template
 *       12/29/90, PDVT, Gisselle Labault
 *       - testcase design
 *
 * Inputs:
 *   hPS        // presentation space
 *   hWndClient     // client window
 *   SelectionCall  // selection parameter
 *
 * Outputs: none
 *
 * Subroutines Required:
 *   LOG_SIGNATURE, CHECKRETURNERROR(),CWRITELOG(), WinSendMsg(),
 *   DevOpenDC(), WinQueryAnchorBlock(), GpiQueryDevice()
 *
 * Limitations: none
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 *   hAB=WinQueryAnchorBlock(hWndClient)
 *   WinSendMsg(hWndClient,...)
 *   printer_hdc = DevOpenDC(hAB,...,NULL)
 *   GreErasePS(printer_hdc)
 *\\end
 ***************************************************************************/

VOID APIENTRY GreErasePSExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC           printer_hdc;         /* device context handle */
  INT           gre_returned_value;
  INT           error_count;         /* contains number of test that fail */
  DEVOPENSTRUC  dop;                 /* device open info structure */
  DRIVDATA      driv;                /* structure for dop and DevOpenDC */
  LONG          lDCType;             /* used by WinSendMsg */
  HAB           hAB;                 /* anchor block handle */
  CHAR          str_msg[150];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE, 0, "+ee+ GreErasePS - Entry\n");
        error_count = 0;
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULLHANDLE);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC", TRUE, "GreErasePSExh");
           return;
        }
        gre_returned_value=GreErasePS(printer_hdc);
        if (!gre_returned_value)
           error_count +=  CHECKRETURNERROR(gre_returned_value, "GreErasePS", SEVERITY_ERROR, PMERR_OK);

        if (!error_count)
            CWRITELOG(L_HDR,0,
"$GreErasePS                                                     -    PASS.\n");
        else {
             sprintf(str_msg, "$ErrorCount = %d.\n", error_count);
             CWRITELOG(L_HDR, 0, str_msg);
             CWRITELOG(L_HDR,0,
"$GreErasePS                                                     -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE, 0, "+ex+ GreErasePS - Exit\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULLHANDLE
                                      ,NULL, NULL,NULL);

    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreLockDevRC
 *
 * Purpose: This an entry point for the DTT that executes
 *      the return code test.
 *
 * System Requirements: None
 *
 * Revision Log: 1/2/91, PDVT, Gisselle Labault
 *       - created initial template
 *       1/2/91, PDVT, Gisselle Labault
 *       - testcase design
 *
 * Inputs:
 *   hPS        // presentation space
 *   hWndClient     // client window
 *   SelectionCall  // selection parameter
 *
 * Outputs:
 *   none
 *
 * Subroutines Required:
 *   LOG_SIGNATURE, CHECKRETURNERROR(),CWRITELOG()
 *
 * Limitations:
 *  Return Code Successfully Tested
 *    PMERR_INV_HDC
 *
 *  Return Code Not Tested Successfully Yet
 *    PMERR_DEV_FUNC_NOT_INSTALLED
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *    // setup conditions to test PMERR_DEV_FUNC_NOT_INSTALLED
 *    // call gre expecting error code return
 *    if (GreLockDevice(hdc)
 *      log unexpected successful condition
 *    else
 *      post error condition
 *    endif
 *
 *    // setup conditions to test PMERR_INV_HDC
 *    // call gre expecting error code return
 *   GreLockDevice(invalid hdc)
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreLockDeviceRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  INT       gre_returned_value;
  INT       error_count;         /* contains number of test that fail */
  HDC       hdc;
  CHAR      str_msg[150];
  DEVOPENSTRUC  dop;                 /* device open info structure */
  DRIVDATA      driv;                /* structure for dop and DevOpenDC */
  LONG          lDCType;             /* used by WinSendMsg */
  HAB           hAB;                 /* anchor block handle */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE, 0, "+ee+ GreLockDevRC - Entry\n");
        error_count = 0;
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULLHANDLE);
        if (hdc == DEV_ERROR) {
           BADERROR("DevOpenDC", TRUE, "GreLockDeviceRC");
           return;
        }

        /****************************************************/
        /* Test GreLockDevice for return code PMERR_INV_HDC */
        /****************************************************/
        CWRITELOG(L_LOTRACE, 0, "+ie+ GreLockDevRC  - Entry\n");
        gre_returned_value=GreLockDevice((HDC)0);
        error_count +=  CHECKRETURNERROR(gre_returned_value, "GreLockDevice PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE, 0, "+ix+ GreLockDevRC  - Exit\n");

        /*******************************************************************/
        /* Test GreLockDevice for return code PMERR_DEV_FUNC_NOT_INSTALLED */
        /*******************************************************************/
        CWRITELOG(L_LOTRACE, 0, "+ie+ GreLockDevRC - Entry\n");
        gre_returned_value=GreLockDevice(hdc);

//TSS...This call is supported by all drivers, and as such, we should
//      not expect a failure here.  The logic has been changed to
//      account for this revelation.
        if (gre_returned_value == FALSE) {
          error_count += 1;
          BADERROR("GreLockDevice", TRUE, "GreLockDeviceRC");
        } /* endif */

//Taken out...see note above.
//        error_count +=  CHECKRETURNERROR(gre_returned_value, "GreLockDevice PMERR_DEV_FUNC_NOT_INSTALLED", SEVERITY_ERROR, PMERR_DEV_FUNC_NOT_INSTALLED);

        CWRITELOG(L_LOTRACE, 0, "+ix+ GreLockDevRC - Exit\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULLHANDLE
                                      ,NULL, NULL,NULL);

        if (!error_count)
           CWRITELOG(L_HDR,0,
"$GreLockDevRC                                                   -    PASS.\n");
        else {
              sprintf(str_msg, "$ErrorCount = %d.\n", error_count);
              CWRITELOG(L_HDR, 0, str_msg);
              CWRITELOG(L_HDR,0,
"$GreLockDevRC                                                   -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE, 0, "+ex+ GreLockDevRC - Exit\n");
     }
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreUnlockDevRC
 *
 * Purpose: This an entry point for the DTT that executes
 *      the return code test.
 *
 * System Requirements: None
 *
 * Revision Log: 10/31/90, PDVT, Gisselle Labault
 *       - created initial template
 *
 * Inputs:
 *   hPS        // presentation space
 *   hWndClient     // client window
 *   SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *   LOG_SIGNATURE, CHECKRETURNERROR(),CWRITELOG()
 *
 * Limitations:
 *  Return Code Successfully Tested
 *
 *  Return Code Not Tested Successfully Yet
 *    PMERR_DEV_FUNC_NOT_INSTALLED
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *    // setup conditions to test PMERR_DEV_FUNC_NOT_INSTALLED
 *    // call gre expecting error code return
 *    if (GreUnlockDevice(hdc)
 *      log unexpected successful condition
 *    else
 *      post error condition
 *    endif
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreUnlockDeviceRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL    ptl;
  INT       gre_returned_value;
  INT       error_count;       /* contains number of test that fail */
  HDC       hdc;
  CHAR      str_msg[150];
  DEVOPENSTRUC  dop;                 /* device open info structure */
  DRIVDATA      driv;                /* structure for dop and DevOpenDC */
  LONG          lDCType;             /* used by WinSendMsg */
  HAB           hAB;                 /* anchor block handle */

  if (SelectionCall) {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE, 0, "+ee+ GreUnlockDevRC - Entry\n");
        error_count = 0;
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULLHANDLE);
        if (hdc == DEV_ERROR) {
           BADERROR("DevOpenDC", TRUE, "GreErasePSExh");
           return;
        }

        /*********************************************************************/
        /* Test GreUnlockDevice for return code PMERR_DEV_FUNC_NOT_INSTALLED */
        /*********************************************************************/
        CWRITELOG(L_LOTRACE, 0, "+ie+ GreUnlockDevRC - Entry\n");
        gre_returned_value=GreUnlockDevice(hdc);
//TSS...This call is supported by all drivers, and as such, we should
//      not expect a failure here.  The logic has been changed to
//      account for this revelation.
        if (gre_returned_value == FALSE) {
          error_count += 1;
          BADERROR("GreUnlockDevice", TRUE, "GreUnlockDeviceRC");
        } /* endif */

//Taken out...see note above.
//        error_count +=  CHECKRETURNERROR(gre_returned_value, "GreUnlockDevice PMERR_DEV_FUNC_NOT_INSTALLED", SEVERITY_ERROR, PMERR_DEV_FUNC_NOT_INSTALLED);
        CWRITELOG(L_LOTRACE, 0, "+ix+ GreUnlockDevRC - Exit\n");

        CWRITELOG(L_LOTRACE, 0, "+ie+ GreUnlockDevRC - Entry\n");
        gre_returned_value=GreUnlockDevice((HDC)0);
        error_count +=  CHECKRETURNERROR(gre_returned_value,"PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE, 0, "+ix+ GreUnlockDevRC - Exit\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULLHANDLE
                                      ,NULL, NULL,NULL);

        if (!error_count)
           CWRITELOG(L_HDR,0,
"$GreUnlockDevRC                                                 -    PASS.\n");
        else {
             sprintf(str_msg, "$ErrorCount = %d.\n", error_count);
             CWRITELOG(L_HDR, 0, str_msg);
             CWRITELOG(L_HDR,0,
"$GreUnlockDevRC                                                 -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE, 0, "+ex+ GreUnlockDevRC - Exit\n");
   }
}

/****************************************************************************
 *\\ddd
 * Routine Name: GreErasePSRC
 *
 * Purpose: This an entry point for the DTT that executes
 *      the return code test.
 *
 * System Requirements: None
 *
 * Revision Log: 10/31/90, PDVT, Gisselle Labault
 *       - created initial template
 *       1/3/91, PDVT, Gisselle Labault
 *       - testcase design
 *
 * Inputs:
 *   hPS        // presentation space
 *   hWndClient     // client window
 *   SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *
 * Limitations:
 *  Return Code Successfully Tested
 *
 *  Return Code Not Tested Successfully Yet
 *    PMERR_HDC_BUSY
 *    PMERR_UNSUPPORTED_ATTR_VALUE
 *    PMERR_UNSUPPORTED_ATTR
 *    PMERR_INCOMPATIBLE_BITMAP
 *    PMERR_INCORRECT_DC_TYPE
 *    PMERR_REALIZE_NOT_SUPPORTED
 *    PMERR_HUGE_FONTS_NOT_SUPPORTED
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   hAB=WinQueryAnchorBlock(hWndClient)
 *   WinSendMsg(hWndClient,...)
 *   printer_hdc = DevOpenDC(hAB,...,NULL)
 *   // setup conditions to test PMERR_HDC_BUSY
 *   // setup conditions to test PMERR_UNSUPPORTED_ATTR_VALUE
 *   // setup conditions to test PMERR_UNSUPPORTED_ATTR
 *   // setup conditions to test PMERR_INCOMPATIBLE_BITMAP
 *   // setup conditions to test PMERR_INCORRECT_DC_TYPE
 *   // setup conditions to test PMERR_REALIZE_NOT_SUPPORTED
 *   // setup conditions to test PMERR_HUGE_FONTS_NOT_SUPPORTED
 *
 *   //for each return code, call the gre:
 *   GreErasePS(printer_hdc)
 *   Call CHECKRETURNERROR(..., expected error return code)
 *   log return condition
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreErasePSRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  INT           gre_returned_value;  /* value returned from GreErasePS */
  INT           error_count;         /* contains number of test that fail */
  HDC           printer_hdc;         /* device context handle */
  DEVOPENSTRUC  dop;                 /* device open info structure */
  DRIVDATA      driv;                /* structure for dop and DevOpenDC */
  LONG          lDCType;             /* used by WinSendMsg */
  LONG          set_bit_returned_value; /* value returned from GreSetBitmapBits */
  HAB           hAB;                 /* anchor block handle */
  BYTE          ptr_address[500];    /* pointer to receiving area */
                     /* for bit_map data */
  LINEBUNDLE    lbnd;                /* line attributes */
  CHARBUNDLE    char_bundle;         /* character attributes */
  BOOL          set_attrs_returned_value; /* value returned from GpiSetAttrs */
  CHAR          str_msg[150];

  if (SelectionCall) {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        TEST_SIGNATURE(0, 0);
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE, 0, "+ee+ GreErasePSRC - Entry\n");
        error_count = 0;
        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));

        /* get printer device context handle */
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULLHANDLE);
        if (printer_hdc == DEV_ERROR) {
            BADERROR("DevOpenDC", TRUE, "GreErasePSExh");
            return;
        }

        /**************************************************/
        /* Test GreErasePS for return code PMERR_HDC_BUSY */
        /**************************************************/
        // busy code start
        CWRITELOG(L_LOTRACE, 0, "+ie+ GreErasePSRC - Entry\n");
        if (MAKEPSBUSY(hPS))
            CWRITELOG(L_FAIL,0,"MAKEPSBUSY failed\n");
        gre_returned_value=GreErasePS(printer_hdc);
//TSS...This call should always return true by printer drivers and we should
//      not expect a failure here.  The logic has been changed to
//      account for this revelation.
        if (gre_returned_value == FALSE) {
          error_count += 1;
          BADERROR("GreErasePS", TRUE, "GreErasePSRC");
        } /* endif */

//Taken out...see note above.
//      error_count +=  CHECKRETURNERROR(gre_returned_value, "GreErasePS PMERR_HDC_BUSY", SEVERITY_ERROR, PMERR_HDC_BUSY);
        FREEPS;
        CWRITELOG(L_LOTRACE, 0, "+ix+ GreErasePSRC - Exit\n");

        /****************************************************************/
        /* Test GreErasePS for return code PMERR_UNSUPPORTED_ATTR_VALUE */
        /****************************************************************/
        CWRITELOG(L_LOTRACE, 0, "+ie+ GreErasePSRC - Entry\n");
        set_attrs_returned_value = GpiSetAttrs(hPS,
                                               PRIM_CHAR,
                                               LBB_COLOR | ABB_MIX_MODE | MBB_BOX |
                                               MBB_SYMBOL | LBB_TYPE | LBB_END |
                                               LBB_JOIN, LBB_COLOR | ABB_MIX_MODE |
                                               MBB_BOX | MBB_SYMBOL | LBB_TYPE | LBB_END | LBB_JOIN,
                                               &char_bundle);

         gre_returned_value=GreErasePS(printer_hdc);
//TSS...This call should always return true by printer drivers and we should
//      not expect a failure here.  The logic has been changed to
//      account for this revelation.
        if (gre_returned_value == FALSE) {
          error_count += 1;
          BADERROR("GreErasePS", TRUE, "GreErasePSRC");
        } /* endif */

//Taken out...see note above.
//         error_count +=  CHECKRETURNERROR(gre_returned_value, "GreErasePS PMERR_UNSUPPORTED_ATTR_VALUE", SEVERITY_ERROR, PMERR_UNSUPPORTED_ATTR_VALUE);
         CWRITELOG(L_LOTRACE, 0, "+ix+ GreErasePSRC - Exit\n");

         /**********************************************************/
         /* Test GreErasePS for return code PMERR_UNSUPPORTED_ATTR */
         /**********************************************************/

         CWRITELOG(L_LOTRACE, 0, "+ie+ GreErasePSRC - Entry\n");
         //  The following function call returns PMERR_UNSUPPORTED_ATTR
         //  this was done with the purpose of getting PMERR_UNSUPPORTED_ATTR
         //  when calling GreErasePS; but I couldn't get it. (I followed the
         //  explanation for PMERR_UNSUPPORTED_ATTR, Appendix A Volume 2
         //  Presentation Driver Interfaces).
         //
         // set_attrs_returned_value = GpiSetAttrs(hPS,
         //               PRIM_LINE,
         //               -999,
         //               -999,
         //               &lbnd);
         // error_count +=  CHECKRETURNERROR(set_attrs_returned_value, "GpiSetAttrs", SEVERITY_ERROR, PMERR_UNSUPPORTED_ATTR);

         set_attrs_returned_value = GpiSetAttrs(hPS,
                       PRIM_LINE,
                       -999L,
                       -999L,
                       &lbnd);
         gre_returned_value=GreErasePS(printer_hdc);

//TSS...This call should always return true by printer drivers and we should
//      not expect a failure here.  The logic has been changed to
//      account for this revelation.
        if (gre_returned_value == FALSE) {
          error_count += 1;
          BADERROR("GreErasePS", TRUE, "GreErasePSRC");
        } /* endif */

//Taken out...see note above.
//       error_count +=  CHECKRETURNERROR(gre_returned_value, "GreErasePS PMERR_UNSUPPORTED_ATTR", SEVERITY_ERROR, PMERR_UNSUPPORTED_ATTR);
         CWRITELOG(L_LOTRACE, 0, "+ix+ GreErasePSRC - Exit\n");

         /***********************************************************/
         /* Test GreErasePS for return code PMERR_INCORRECT_DC_TYPE */
         /***********************************************************/
         CWRITELOG(L_LOTRACE, 0, "+ie+ GreErasePSRC - Entry\n");
         /*
         set_bit_returned_value = GreSetBitmapBits(printer_hdc,(HBITMAP)NULL, 0L, 2L, ptr_address, NULL);
         */
         gre_returned_value=GreErasePS(printer_hdc);
//TSS...This call should always return true by printer drivers and we should
//      not expect a failure here.  The logic has been changed to
//      account for this revelation.
        if (gre_returned_value == FALSE) {
          error_count += 1;
          BADERROR("GreErasePS", TRUE, "GreErasePSRC");
        } /* endif */

//Taken out...see note above.
//       error_count +=  CHECKRETURNERROR(gre_returned_value, "GreErasePS PMERR_INCORRECT_DC_TYPE", SEVERITY_ERROR, PMERR_INCORRECT_DC_TYPE);
         CWRITELOG(L_LOTRACE, 0, "+ix+ GreErasePSRC - Exit\n");

         CWRITELOG(L_LOTRACE, 0, "+ie+ GreErasePSRC - Entry\n");
         gre_returned_value=GreErasePS((HDC)0);
         error_count +=  CHECKRETURNERROR(gre_returned_value, "PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
         CWRITELOG(L_LOTRACE, 0, "+ix+ GreErasePSRC - Exit\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULLHANDLE
                                      ,NULL, NULL,NULL);

         if (!error_count)
            CWRITELOG(L_HDR,0,
"$GreErasePSRC                                                   -    PASS.\n");
         else {
               sprintf(str_msg, "$ErrorCount = %d.\n", error_count);
               CWRITELOG(L_HDR, 0, str_msg);
               CWRITELOG(L_HDR,0,
"$GreErasePSRC                                                   -    FAIL.\n");
         }
         CWRITELOG(L_LOTRACE, 0, "+ex+ GreErasePSRC - Exit\n");
     }
}

