/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma  title("GRELINE.C")

/**************************************************************************
 *
 * SOURCE FILE NAME = GRELINE.C
 *
 * DESCRIPTIVE NAME = Gre line dll test functions.
 *
 *
 *      DISCLAIMER OF WARRANTIES.  The following enclosed code is
 *      sample code created by IBM Corporation. This sample code is not
 *      part of any standard or IBM product and is provided to you solely
 *      for  the purpose of assisting you in the development of your
 *      applications.  The code is provided "AS IS", without
 *      warranty of any kind.  IBM shall not be liable for any damages
 *      arising out of your use of the sample code, even if they have been
 *      advised of the possibility of such damages.
 *
 * VERSION = V1.1
 *
 * DATE      10/31/92
 *
 * DESCRIPTION  The GRELINE testcase involves testing of the following GPIs:
 *
 *                  GrePolyLine()
 *                  GreDrawLinesInPath() - this call is only valid in r206
 *                  GrePolyScanLine()
 *                  GreGetLineOrigin()
 *                  GreSetLineOrigin()
 *                  GreGetStyleRatio()
 *                  GreSetStyleRatio()
 *                  GreGetCurrentPosition()
 *                  GreSetCurrentPosition()
 *
 *              The individual testing of each GPI will include a functionality
 *              test and an exhaustive return codes test.  Along with the
 *              exhaustive tests, there will be an application routine and
 *              a precision routine.
 *
 * FUNCTIONS    InitTest()                 Initialize function name and address array.
 *              gre_line_pre()             Gre Line precision tests.
 *              GrePolyLineExh()           Perform functional testing of GrePolyLine.
 *              GreDrawLinesInPathExh()    Functional testing of
 *                                         GreDrawLinesInPath - only valid in r206
 *              GreLineOriginExh()         Functional testing of
 *                                         GreGetLineOrigin and
 *              GrePolyLineRC()            Exhaustive testing of
 *                                         GrePolyLine return codes.
 *              GreDrawLinesInPathRC()     Exhaustive testing of
 *                                         GreDrawLinesInPath return codes
 *              GrePolyShortLineRC()       Exhaustive testing of
 *                                         GrePolyShortLine return codes.
 *              GrePolyScanLineRC()        Exhaustive testing of
 *                                         GrePolyScanLine return codes.
 *              GreGetLineOriginRC()       Exhaustive testing of
 *                                         GreGetLineOrigin return codes.
 *              GreSetLineOriginRC()       Exhaustive testing of
 *                                         GreSetLineOrigin return codes.
 *              GreGetStyleRatioRC()       Exhaustive testing of
 *                                         GreGetStyleRatio return codes.
 *              GreSetStyleRatioRC()       Exhaustive testing of
 *                                         GreSetStyleRatio return codes.
 *              GreGetCurrentPositionRC()  Exhaustive testing of
 *                                         GreGetCurrentPosition return codes.
 *              GreSetCurrentPositionRC()  Exhaustive testing of
 *                                         GreSetCurrentPosition return codes.
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 * EXTERNAL REFERENCES
 *
 * EXTERNAL FUNCTIONS
 *
*/#define  INCL_32
#define  INCL_GPI                  /* all Gpi definitions                   */
#define  INCL_ERRORS               /* error definitions                     */
#define  INCL_GREALL               /* Gre definitions                       */
#define  INCL_DDIPATHS             /* path,line, and curve definitions      */
#define  INCL_DDIMISC              /* misc. definitions                     */
#define  INCL_DOS                  /* used for spiral image                 */
#define  INCL_DEV
#define  NUMPOINTS     500         /* number of points for spiral image     */
#define  NUMREV        7           /* number of spiral revolutions          */
#define  PI            3.14159     /* needed for spiral point calculations  */
#include <string.h>
#include <stdlib.h>
#include <os2.h>
#include <stdio.h>
#include "extras.h"                /* include file for LINE structure       */
#include <pmddi.h>
#include <math.h>
#include "DTTDLLDF.h"              /* test case defines                     */
#include "gre_grid.h"              /* grid definition include file          */
#include "std_util.h"              /* map and write functions               */
#include "greline.h"               /* gre line dll include file             */

/***************************************************************************
 *
 * FUNCTION NAME = InitTest()
 *
 * DESCRIPTION   = Initialize function name and address array for DTT
 *
 *
 * INPUT         = sel       - passed through to the InitTestSegVars function
 *                 pInitInfo - name and address array used by DTT interface
 * OUTPUT        =
 *                 InitInfo  - returned initialized with names and addresses
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 for all test functions int this test case
 *                 {
 *                    pInitInfo->function name = function name[i]
 *                    pInitInfo->function address = function address[i]
 *                 }
 *
 *                 call InitTestSegVars routine
 *
 ****************************************************************************/

VOID APIENTRY InitTest(PVOID sel,PTESTINITINFO pInitInfo)
{
  /*
  **  Set number of entry points and test group name
  */

  pInitInfo->ntests = NTESTS-1;        /* set number of entry points        */
  strcpy(pInitInfo->group_name, "GRE Line");

  /*
  **  Set each function name and address
  */

  strcpy(pInitInfo->test_names[0], test_name_0);
  pInitInfo->test_addrs[0] = gre_line_pre;

  strcpy(pInitInfo->test_names[1], test_name_1);
  pInitInfo->test_addrs[1] = GrePolyLineExh;

  strcpy(pInitInfo->test_names[2], test_name_2);
  pInitInfo->test_addrs[2] = GreDrawLinesInPathExh;

  strcpy(pInitInfo->test_names[3], test_name_3);
  pInitInfo->test_addrs[3] = GreLineOriginExh;

  strcpy(pInitInfo->test_names[4], test_name_4);
  pInitInfo->test_addrs[4] = GrePolyLineRC;

  strcpy(pInitInfo->test_names[5], test_name_5);
  pInitInfo->test_addrs[5] = GreDrawLinesInPathRC;

  strcpy(pInitInfo->test_names[6], test_name_6);
  pInitInfo->test_addrs[6] = GrePolyShortLineRC;

  strcpy(pInitInfo->test_names[7], test_name_7);
  pInitInfo->test_addrs[7] = GrePolyScanLineRC;

  strcpy(pInitInfo->test_names[8], test_name_8);
  pInitInfo->test_addrs[8] = GreGetLineOriginRC;

  strcpy(pInitInfo->test_names[9], test_name_9);
  pInitInfo->test_addrs[9] = GreSetLineOriginRC;

  strcpy(pInitInfo->test_names[10], test_name_10);
  pInitInfo->test_addrs[10] = GreGetStyleRatioRC;

  strcpy(pInitInfo->test_names[11], test_name_11);
  pInitInfo->test_addrs[11] = GreSetStyleRatioRC;

  strcpy(pInitInfo->test_names[12], test_name_12);
  pInitInfo->test_addrs[12] = GreGetCurrentPositionRC;

  strcpy(pInitInfo->test_names[13], test_name_13);
  pInitInfo->test_addrs[13] = GreSetCurrentPositionRC;

  InitTestSegVars(sel);            /* this call used by the DTT interface   */
 }

/***************************************************************************
 *
 * FUNCTION NAME = gre_line_pre()
 *
 * DESCRIPTION   = Gre Line precision tests.
 *
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 draw line from 0,3 to 7,10
 *                 draw line from 0,10 to 7,3
 *                 draw line from 0,6 to 4,10
 *                 draw line from 3,10 to 7,6
 *                 draw line from 0,7 to 4,3
 *                 draw line from 3,3 to 7,7
 *                 draw line from 0,9.5 to 7,9.5
 *                 draw line from 0,6.5 to 7,6.5
 *                 draw line from 0,3.5 to 7,3.5
 *                 draw line from 0,9 to 1,10
 *                 draw line from 6,10 to 7,9
 *                 draw line from 6,3 to 7,4
 *                 draw line from 0,4 to 1,3
 *                 draw line from 0.5,3 to 0.5,10
 *                 draw line from 3.5,3 to 3.5,10
 *                 draw line from 6.5,3 to 6.5,10
 *                 check four line intersections at coordinates
 *                       (0.5,3.5),(0.5,6.5),(0.5,9.5),(3.5,3.5),(3.5,6.5),
 *                       (3.5,9.5),(6.5,3.5),(6.5,6.5) and (6.5,9.5)
 *                 check two line intersections at coordinates
 *                       (2,7),(2,10),(5,7) and (5,10)
 *
 *
 ****************************************************************************/

VOID APIENTRY gre_line_pre(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL paptPoint[1];
  POINTL ptl;
  HDC Display_hdc;
  LONG lErrorCount;
  CHAR out_string[257];
  LONG rc;


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    lErrorCount = 0L;
    Display_hdc = set_delta(hPS);
    LOG_SIGNATURE;
    TEST_SIGNATURE(0, 0);
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreLinePre - Entry.\n");
    gre_grid(hPS, (LONG)INCH);

    /*
    ** draw a number of lines, all which intersect at various locations
    ** on the page
    */

    ptl.x = 0;
    ptl.y = 3*delta_y;
    paptPoint[0].x = 7*delta_x;
    paptPoint[0].y = 10*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GreMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 0;
    ptl.y = 10*delta_y;
    paptPoint[0].x = 7*delta_x;
    paptPoint[0].y = 3*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GreMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 0;
    ptl.y = 6*delta_y;
    paptPoint[0].x = 4*delta_x;
    paptPoint[0].y = 10*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GreMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePoly", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 0;
    ptl.y = 7*delta_y;
    paptPoint[0].x = 4*delta_x;
    paptPoint[0].y = 3*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GreMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 3*delta_x;
    ptl.y = 10*delta_y;
    paptPoint[0].x = 7*delta_x;
    paptPoint[0].y = 6*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GreMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 7*delta_x;
    ptl.y = 7*delta_y;
    paptPoint[0].x = 3*delta_x;
    paptPoint[0].y = 3*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GreMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = delta_x/2;
    ptl.y = 10*delta_y;
    paptPoint[0].x = delta_x/2;
    paptPoint[0].y = 3*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 7*delta_x/2;
    ptl.y = 10*delta_y;
    paptPoint[0].x = 7*delta_x/2;
    paptPoint[0].y = 3*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 13*delta_x/2;
    ptl.y = 10*delta_y;
    paptPoint[0].x = 13*delta_x/2;
    paptPoint[0].y = 3*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 0;
    ptl.y = 19*delta_y/2;
    paptPoint[0].x = 7*delta_x;
    paptPoint[0].y = 19*delta_y/2;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 0;
    ptl.y = 13*delta_y/2;
    paptPoint[0].x = 7*delta_x;
    paptPoint[0].y = 13*delta_y/2;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 0;
    ptl.y = 7*delta_y/2;
    paptPoint[0].x = 7*delta_x;
    paptPoint[0].y = 7*delta_y/2;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 0;
    ptl.y = 9*delta_y;
    paptPoint[0].x = delta_x;
    paptPoint[0].y = 10*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 6*delta_x;
    ptl.y = 10*delta_y;
    paptPoint[0].x = 7*delta_x;
    paptPoint[0].y = 9*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 6*delta_x;
    ptl.y = 3*delta_y;
    paptPoint[0].x = 7*delta_x;
    paptPoint[0].y = 4*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    ptl.x = 0;
    ptl.y = 4*delta_y;
    paptPoint[0].x = delta_x;
    paptPoint[0].y = 3*delta_y;
    MapPt(ptl);
    MapPt(paptPoint[0]);

    if (GpiMove(hPS, &ptl) == GPI_ERROR)
    {
      BADERROR("GpiMove", TRUE, "GreLinePre");
      return;
    }
    rc = GrePolyLine(Display_hdc, paptPoint, 1);

    if (rc == GPI_ERROR)
    {
      BADERROR("GrePolyLine", TRUE, "GreLinePre");
      return;
    }
    sprintf(out_string, "The top 7 x 7 grid units are used for this test.");
    MapAndWriteString(hPS, (delta_x/4), ((3*delta_y)-(delta_y/4)), out_string);
    sprintf(out_string,
       "Each corner square, each center square of each side, and the center");
    MapAndWriteString(hPS, (delta_x/4), ((3*delta_y)-(delta_y/2)), out_string);
    sprintf(out_string,
       "square should all have four intersecting lines at their centers.");
    MapAndWriteString(hPS, (delta_x/4), ((3*delta_y)-((delta_y/4)*3)),
       out_string);
    sprintf(out_string,
       "Coordinates (2,2), (2,5), (5,2), and (5,5) of the 7 x 7 grid");
    MapAndWriteString(hPS, (delta_x/4), ((2*delta_y)-(delta_y/4)), out_string);
    sprintf(out_string, "should have two intersecting diagonal lines.");
    MapAndWriteString(hPS, (delta_x/4), ((2*delta_y)-(delta_y/2)), out_string);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GreLinePre                                       "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GreLinePre                                       "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GreLinePre - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GrePolyLineExh()
 *
 * DESCRIPTION   = Perform functional testing of GrePolyLine,
 *                 GreGetCurrentPosition, and GreSetCurrentPosition.
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 set up array for poly line function
 *                 GrePolyLine(...); - draw spiral
 *                 check value received from GRE and log to DTT_LOG
 *                 BADERROR(...);
 *
 ****************************************************************************/

VOID APIENTRY GrePolyLineExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC hDC;
  LONG lErrorCount;
  CHAR out_string[100];
  double dAngle,dScale;
  PPOINTL pptl,pptPosition;
  SHORT sIndex;
  SHORT progress;


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    hDC = set_delta(hPS);
    lErrorCount = 0L;
    LOG_SIGNATURE;
    TEST_SIGNATURE(0, 0);
    TEST_PROGRESS(00, 2*NUMPOINTS);
    CWRITELOG(L_LOTRACE, 0, "+ee+ GrePolyLineExh - Entry.\n");

    /*
    **  Set up Array for Spiral Image
    */

    if(!DosAllocMem((PVOID)&pptl,NUMPOINTS * sizeof(POINTL),PAG_READ | PAG_WRITE | PAG_COMMIT)){

      for (sIndex = 0; sIndex < NUMPOINTS; sIndex++)
      {
        dAngle = sIndex *2*PI/(NUMPOINTS/NUMREV);
        dScale = 1-(double)sIndex/NUMPOINTS;
        pptl[sIndex].x = (LONG)((3000*PI)/2*(1+dScale *cos(dAngle)));
        pptl[sIndex].y = (LONG)((*pBasisY)/2*(1+dScale *sin(dAngle)));

        if (!((progress = sIndex+1)%50))
          TEST_PROGRESS(progress, 2*NUMPOINTS);
      }

      for (sIndex = 0; sIndex < NUMPOINTS; sIndex++)
      {
        MapPt(pptl[sIndex]);

        if (!((progress = sIndex+NUMPOINTS+1)%50))
          TEST_PROGRESS(progress, 2*NUMPOINTS);
      }

      /*
      **  Test GreSetCurrentPosition
      */

      CWRITELOG(L_LOTRACE, 0, "+ie+ GreSetCurrentPosition - Entry.\n");

      if (!GreSetCurrentPosition(hDC, pptl))
      {
        BADERROR("GreSetCurrentPosition", TRUE, "GrePolyLineExh");
        return;
      }
      CWRITELOG(L_LOTRACE, 0, "+ix+ GreSetCurrentPosition - Exit.\n");

      /*
      **  Test GreGetCurrentPosition
      */

      CWRITELOG(L_LOTRACE, 0, "+ie+ GreGetCurrentPosition - Entry.\n");
      pptPosition = malloc(sizeof(POINTL));

      if (!GreGetCurrentPosition(hDC, pptPosition))
      {
        BADERROR("GreGetCurrentPosition", TRUE, "GrePolyLineExh");
        return;
      }

      /*
      **  compare values received from GreGetCurrentPosition
      */

      if ((pptPosition->x != pptl->x) || (pptPosition->y != pptl->y))
      {
        CWRITELOG(L_TRACE, 0,
           "GreGetCurrentPosition Received Incorrect Values.\n");
        ++lErrorCount;
      }
      CWRITELOG(L_LOTRACE, 0, "+ix+ GreGetCurrentPosition - Exit.\n");

      /*
      ** Test GrePolyLine
      */

      CWRITELOG(L_LOTRACE, 0, "+ie+ GrePolyLine - Entry.\n");

      if (!GrePolyLine(hDC, pptl+1, NUMPOINTS-1L))
      {
        BADERROR("GrePolyLine", TRUE, "GrePolyLineExh");
        return;
      }
      CWRITELOG(L_LOTRACE, 0, "+ix+ GrePolyLine - Exit.\n");
      DosFreeMem((PVOID)pptl);
    }
    MapAndWriteString(hPS, (11*delta_x/2), (5*delta_y/4), "Tests: GrePolyLine"
       );
    MapAndWriteString(hPS, (11*delta_x/2), delta_y, "GreSetCurrentPosition");
    MapAndWriteString(hPS, (11*delta_x/2), (3*delta_y/4),
       "GreGetCurrentPosition");

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GrePolyLineExh                                   "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GrePolyLineExh                                   "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GrePolyLineExh - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GreDrawLinesInPathExh()
 *
 * DESCRIPTION   = Functional testing of GreDrawLinesInPath,
 *                 GreGetLineOrigin, GreSetLineOrigin, GreGetStyleRatio, and
 *                 GreSetStyleRatio.
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 Test GreSetStyle Ratio
 *                 initialize ratio values
 *                 GreSetStyleRatio(hDC,pRatio);
 *                 Test GreGetStyleRatio
 *                 GreGetStyleRatio(hDC,pRatio);
 *                 compare ratio return values
 *                 Report error if values are different.
 *
 ****************************************************************************/

VOID APIENTRY GreDrawLinesInPathExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL ptl;
  HDC hDC;
  PCURVE pPath;
  ULONG cLines;


  BYTE pRatio[2],pRatioComp[2];
  PBYTE ppRatio,ppRatioComp;


  CHAR out_string[100];
  LONG lErrorCount,rc=GPI_ERROR;
  ULONG lLineStyle,lSetStyle;
  PPOINTL pptCurPos,pptSetCurPos;
  PLINE plLine[20];
  USHORT i;
  ULONG ulLinesInPath;
  LONG xStart,yStart,xEnd,yEnd;

  ppRatio=pRatio;
  ppRatioComp=pRatioComp;

  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    hDC = set_delta(hPS);
    LOG_SIGNATURE;
    lErrorCount = 0;
    TEST_SIGNATURE(0, 0);
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreDrawLinesInPathExh - Entry.\n");

    /*
    **  Test GreSetStyleRatio
    */

    CWRITELOG(L_LOTRACE, 0, "+ie+ GreSetStyleRatio - Entry.\n");
    pRatio[0] = 85;
    pRatio[1] = 64;

    if ( !GreSetStyleRatio(hDC, ppRatio) )
    {
      BADERROR("GreSetStyleRatio", TRUE, "GreDrawLinesInPathExh");
      CWRITELOG(L_LOTRACE, 0,
         "$GreSetStyleRatio                          FAIL\n");
      return;
    }

    else
      CWRITELOG(L_LOTRACE, 0,
         "$GreSetStyleRatio                          PASS\n");
    CWRITELOG(L_LOTRACE, 0, "+ix+ GreSetStyleRatio - Exit.\n");

    /*
    **  Test GreGetStyleRatio
    */

    CWRITELOG(L_LOTRACE, 0, "+ie+ GreGetStyleRatio - Entry.\n");
    pRatioComp[0] = 0;
    pRatioComp[1] = 0;

    if (!GreGetStyleRatio(hDC, pRatioComp) )
    {
      BADERROR("GreGetStyleRatio", TRUE, "GreDrawLinesInPath");
      CWRITELOG(L_LOTRACE, 0,
         "$GreGetStyleRatio                          FAIL\n");
      return;
    }

    else
      CWRITELOG(L_LOTRACE, 0,
         "$GreGetStyleRatio                          PASS\n");
    CWRITELOG(L_LOTRACE, 0, "+ix+ GreGetStyleRatio - Exit.\n");

    if ((pRatio[0] != pRatioComp[0]) || (pRatio[1] != pRatioComp[1]))
    {
      CWRITELOG(L_LOTRACE, 0, "$ERROR!! GreGetStyleRatio did NOT return value"
         "s set by GreSetStyleRatio.\n");
      CWRITELOG(L_LOTRACE, 0,
        "$GreGet- /GreSetStyleRatio                             -    FAIL.\n");
      MapAndWriteString(hPS, (delta_x), (8*delta_y-delta_y/4),
    "ERROR!! GreGetStyleRatio did NOT return values set by GreSetStyleRatio.");
      ++lErrorCount;
    }

    else
    {
      CWRITELOG(L_LOTRACE, 0,
         "$GreGetStyleRatio returned values set by GreSetStyleRatio.\n");
      CWRITELOG(L_LOTRACE, 0,
        "$GreGet- /GreSetStyleRatio                             -    PASS.\n");
      MapAndWriteString(hPS, (delta_x), (8*delta_y-delta_y/4),
         "GreGetStyleRatio returned values set by GreSetStyleRatio");
      MapAndWriteString(hPS, (delta_x), (7*delta_y+delta_y/2),
         "GreSetStyleRatio and GreGetStyleRatio successfully tested");
    }
    sprintf(out_string, "SET ratio:  x = %hu, y = %hu", (USHORT)pRatio[1],
       (USHORT)pRatio[0]);
    MapAndWriteString(hPS, (delta_x), (7*delta_y+delta_y/4), out_string);
    sprintf(out_string, "GET ratio:  x = %hu, y = %hu", (USHORT)pRatioComp[1],
       (USHORT)pRatioComp[0]);
    MapAndWriteString(hPS, (delta_x), (7*delta_y), out_string);

    CWRITELOG(L_LOTRACE, 0, "+ie+ GreSetLineOrigin - Entry.\n");
    pptSetCurPos = (PPOINTL)malloc((size_t)sizeof(POINTL));
    lSetStyle = 1;
    pptSetCurPos->x = 2*delta_x;
    pptSetCurPos->y = 3*delta_y;

    if (!GreSetLineOrigin(hDC, pptSetCurPos, lSetStyle))
    {
      BADERROR("GreSetLineOrigin", TRUE, "GreDrawLinesInPathExh");
      CWRITELOG(L_LOTRACE, 0,
         "$GreSetLineOrigin                          FAIL\n");
      return;
    }

    else
      CWRITELOG(L_LOTRACE, 0,
         "$GreSetLineOrigin                          PASS\n");
    CWRITELOG(L_LOTRACE, 0, "+ix+ GreSetLineOrigin - Exit.\n");
    CWRITELOG(L_LOTRACE, 0, "+ie+ GreGetLineOrigin - Entry.\n");
    pptCurPos = (PPOINTL)malloc((size_t)sizeof(POINTL));

    if ((lLineStyle = GreGetLineOrigin(hDC, pptCurPos)) == GPI_ALTERROR)
    {
      BADERROR("GreGetLineOrigin", TRUE, "GreDrawLinesInPath");
      CWRITELOG(L_LOTRACE, 0,
         "$GreGetLineOrigin                          FAIL\n");
      return;
    }

    else
      CWRITELOG(L_LOTRACE, 0,
         "$GreGetLineOrigin                          PASS\n");
    CWRITELOG(L_LOTRACE, 0, "+ix+ GreGetLIneOrigin - Exit.\n");
    sprintf(out_string, "SET line style: %ld; SET position: x = %ld, y = %ld",
       lSetStyle, pptSetCurPos->x, pptSetCurPos->y);
    MapAndWriteString(hPS, (delta_x), (5*delta_y), out_string);
    sprintf(out_string, "GET line style: %ld; GET position: x = %ld, y = %ld",
       lLineStyle, pptCurPos->x, pptCurPos->y);
    MapAndWriteString(hPS, (delta_x), (5*delta_y-delta_y/4), out_string);

    if ((pptSetCurPos->x != pptCurPos->x) || (pptSetCurPos->y != pptCurPos->y)
       || (lSetStyle != lLineStyle))
    {
      CWRITELOG(L_LOTRACE, 0,
  "$ERROR! GreGetLineOrigin did NOT return values set by GreSetLineOrigin.\n");
      CWRITELOG(L_LOTRACE, 0,
         "$GreGet- /GreSetLineOrigin                          FAIL\n");
      MapAndWriteString(hPS, (delta_x), (5*delta_y+delta_y/2),
     "ERROR!! GreGetLineOrigin did NOT return values set by GreSetLineOrigin");
      ++lErrorCount;
    }

    else
    {
      CWRITELOG(L_LOTRACE, 0,
         "$GreGetLineOrigin returned values set by GreSetLineOrigin\n");
      CWRITELOG(L_LOTRACE, 0,
         "$GreGet- /GreSetLineOrigin                          PASS\n");
      MapAndWriteString(hPS, (delta_x), (5*delta_y+delta_y/2),
         "OK: GreGetLineOrigin returned values set by GreSetLineOrigin");
      MapAndWriteString(hPS, (delta_x), (5*delta_y+delta_y/4),
         "GreSetLineOrigin and GreGetLineOrigin successfully tested");
    }

    if (pptCurPos != NULL)
      free(pptCurPos);

    if (pptSetCurPos != NULL)
      free(pptSetCurPos);

    pPath = (PCURVE)malloc((size_t)sizeof(CURVE));
    pPath->bIdent = 0;
    pPath->bType = LINE_IDENTIFIER;
    pPath->usReserved = 0;
    pPath->flStyle = 0;
    pPath->fl = 0;
    pPath->pcvNext = (CURVE *)NULL;
    pPath->pcvPrev = (CURVE *)NULL;

    /*
    **  Set up an array of pointers to line structures and create a linked list
    */

    ulLinesInPath = 20;

    for (i = 0; i < ulLinesInPath; i++)
      plLine[i] = (PLINE)malloc((size_t)sizeof(LINE));
    xEnd = 25;
    yEnd = 150;

    for (i = 0; i < ulLinesInPath; i++)
    {
      xStart = xEnd;
      yStart = yEnd;
      xEnd += 25;
      yEnd = (yEnd == 175)?150:175;
      plLine[i]->bIdent = 0;
      plLine[i]->bType = LINE_IDENTIFIER;
      plLine[i]->flStyle = 0;

      if (i == (ulLinesInPath-1))
        plLine[i]->pcvNext = (CURVE NEAR *)NULL;

      else
        plLine[i]->pcvNext = (CURVE NEAR *)plLine[i+1];

      if (i == 0)
        plLine[0]->pcvPrev = (CURVE NEAR *)NULL;

      else
        plLine[i]->pcvPrev = (CURVE NEAR *)plLine[i-1];


      plLine[i]->ptlA.x = xStart;
      plLine[i]->ptlA.y = yStart;
      plLine[i]->ptlC.x = xEnd;
      plLine[i]->ptlC.y = yEnd;
      plLine[i]->ptfxA.x = xStart *FIXEDTOLONG;
      plLine[i]->ptfxA.y = yStart *FIXEDTOLONG;
      plLine[i]->ptfxC.x = xEnd *FIXEDTOLONG;
      plLine[i]->ptfxC.y = yEnd *FIXEDTOLONG;
    }

    /*
    **  By changing the btype parameter on these 2 lines, they should be
    **  omitted from the output zigzag
    */

    plLine[4]->bType = CURVE_IDENTIFIER;
    plLine[9]->bType = CURVE_IDENTIFIER;
  /* This call is only supported in R206 environment. */
  /*  rc = GreDrawLinesInPath(hDC, pPath, plLine[0], ulLinesInPath); */


    if (rc == GPI_ERROR)
    {
      BADERROR("GreDrawLinesInPath", TRUE, "GreDrawLinesInPath");
      CWRITELOG(L_LOTRACE, 0,
         "$GreDrawLinesInPath                          FAIL\n");
      MapAndWriteString(hPS, (delta_x), (4*delta_y),
         "GreDrawLinesInPath failed");
      lErrorCount++;
      return;
    }

    else
    {
      CWRITELOG(L_LOTRACE, 0,
         "$GreDrawLinesInPath                          PASS\n");
      MapAndWriteString(hPS, (delta_x), (4*delta_y),
         "GreDrawLinesInPath successfully tested");
      MapAndWriteString(hPS, (delta_x), (3*delta_y+delta_y/2),
         "Size and location of lines dependent on resolution");
    }

    for (i = 0; i < ulLinesInPath; i++)

      if (plLine[i] != NULL)
        free(plLine[i]);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GreDrawLinesInPathExh                            "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GreDrawLinesInPathExh                            "
         "              -    FAIL.\n");
    }
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GreLineOriginExh()
 *
 * DESCRIPTION   = Functional testing of GreGetLineOrigin and
 *                 GreSetLineOrigin.  NOTE:  This test is basically a
 *                 debugging test that is interested primarily in the line
 *                 style parameter of these 2 functions.  (However, the
 *                 current position setting is also tested.)
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *
 ****************************************************************************/

VOID APIENTRY GreLineOriginExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL ptl,ptlStartPoint,ptlEndPoint;
  HDC hDC;
  CHAR out_string[100];
  ULONG lErrorCount,lSetStyle,lGetStyle,y_decrement,lyPos;
  PPOINTL pptGetCurPos,pptSetCurPos;
  ULONG i;


  if (SelectionCall)
  {
    WinSendMsg(hWndClient,
               WM_DTHREAD_DOES_DEVCALLS,
               NULL,
               NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    hDC = set_delta(hPS);
    LOG_SIGNATURE;
    lErrorCount = 0;
    TEST_SIGNATURE(0, 0);
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreLineOriginExh - Entry.\n");
    pptSetCurPos = (PPOINTL)malloc((size_t)sizeof(POINTL));
    pptGetCurPos = (PPOINTL)malloc((size_t)sizeof(POINTL));
    pptSetCurPos->x = 2*delta_x;
    pptSetCurPos->y = 3*delta_y;
    MapAndWriteString(hPS, (delta_x), (10*delta_y),
       "NOTE: Line type set by GpiSetLineType()");

    for (i = 0; i < 12; i++)
    {
      GpiSetLineType(hPS,
                     i);
      lSetStyle = i;

      if ((lGetStyle = GreGetLineOrigin(hDC, pptGetCurPos)) == GPI_ALTERROR)
      {
        BADERROR("GreGetLineOrigin", TRUE, "GreLineOriginExh");
        lErrorCount++;
      }
      sprintf(out_string, "SET line style: %ld, GET line style: %ld (hex %lx)"
         , lSetStyle, lGetStyle, lGetStyle);

      /*
      **  Position for output string
      */

      y_decrement = i *delta_y/2;
      lyPos = 9*delta_y-y_decrement;
      MapAndWriteString(hPS, (delta_x), (lyPos), out_string);

      /*
      **  Position for output line
      */

      lyPos -= delta_y/4;
      ptlStartPoint.x = delta_x;
      ptlEndPoint.x = 6*delta_x;
      ptlEndPoint.y = ptlStartPoint.y = lyPos;
      MapPt(ptlStartPoint);
      MapPt(ptlEndPoint);
      GpiSetCurrentPosition(hPS,
                            &ptlStartPoint);
      GpiLine(hPS,
              &ptlEndPoint);
    }

    /*
    **  Now test to see if current position is set properly
    */

    pptSetCurPos->x = 2*delta_x;
    pptSetCurPos->y = delta_y+delta_y/2;
    MapPt(*pptSetCurPos);

    if (!GreSetLineOrigin(hDC, pptSetCurPos, lSetStyle))
    {
      BADERROR("GreSetLineOrigin", TRUE, "GreDrawLinesInPathExh");
      lErrorCount++;
    }
    strcpy(out_string, "And now to test the current position setting...");
    GpiCharString(hPS, (LONG)strlen(out_string), (PCH)out_string);
    pptSetCurPos->x = 2*delta_x;
    pptSetCurPos->y = delta_y;
    MapPt(*pptSetCurPos);

    if (!GreSetLineOrigin(hDC, pptSetCurPos, lSetStyle))
    {
      BADERROR("GreSetLineOrigin", TRUE, "GreDrawLinesInPathExh");
      lErrorCount++;
    }
    strcpy(out_string, "This string should begin at (2,1) inches.");
    GpiCharString(hPS, (LONG)strlen(out_string), (PCH)out_string);

    if (pptGetCurPos != NULL)
      free(pptGetCurPos);

    if (pptSetCurPos != NULL)
      free(pptSetCurPos);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0,
    "$GreLineOriginExh                                          -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0,
    "$GreLineOriginExh                                          -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ie+ GreLineOriginExh - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GrePolyLineRC()
 *
 * DESCRIPTION   = Exhaustive testing of GrePolyLine return codes.
 *
 *                 Return Codes Tested:
 *                    PMERR_INV_HDC
 *                    PMERR_HDC_BUSY
 *
 *                 Return Codes Not Tested:
 *                    PMERR_BASE_ERROR
 *                    PMERR_INV_IN_AREA
 *                    PMERR_PATH_UNKNOWN
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *                    PMERR_COORDINATE_OVERFLOW
 *                    PMERR_BITMAP_NOT_SELECTED
 *                    PMERR_INV_LENGTH_OR_COUNT
 *                    PMERR_INV_RECT
 *                    PMERR_INV_COORD_SPACE
 *                    PMERR_INV_COLOR_DATA
 *                    PMERR_INV_COLOR_INDEX
 *                    PMERR_INV_PICK_APERTURE_POSN
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 test for Invalid Device Context Handle  PMERR_INV_HDC
 *                 bad_hDC = meaningless value;
 *                 call function
 *                 return_code = GrePolyLine(bad_hDC,paptPoint,cPoints);
 *                 report error value to DTT_LOG
 *                 CHECKRETURNERROR ( return_code,
 *                                    "GrePolyLine PMERR_INV_HDC",
 *                                    SEVERITY_ERROR,
 *                                    PMERR_INV_HDC);
 *
 ****************************************************************************/

VOID APIENTRY GrePolyLineRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC bad_hDC,hDC;
  LONG return_code,cPoints;
  PPOINTL paptPoint, ptlBox;
  POINTL aptPoints[3];
  LONG lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    hDC = set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GrePolyLineRC - Entry.\n");

    /*
    **  Initialize parameters: to test an invalid parameter, all other
    **  parameters must be valid
    */

    aptPoints[0].x = 25;
    aptPoints[0].y = 25;
    aptPoints[1].x = 50;
    aptPoints[1].y = 50;
    aptPoints[2].x = 75;
    aptPoints[2].y = 75;
    cPoints = 3;

    /*
    **  Test the following Gre call for an invalid device context
    **  handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0, "+ie+ GrePolyLine PMERR_INV_HDC Test - Entry.\n");
    bad_hDC = 0;
    return_code = GrePolyLine(bad_hDC, aptPoints, cPoints);
    lErrorCount += CHECKRETURNERROR(return_code, "GrePolyLine PMERR_INV_HDC",
       SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0, "+ix+ GrePolyLine PMERR_INV_HDC Test - Exit.\n");

    /*
    **  Test for hdc_busy return code.
    */

    CWRITELOG(L_LOTRACE, 0, "+ie+ GrePolyLine PMERR_HDC_BUSY Test - Entry.\n");

    if(!DosAllocMem((PVOID)&ptlBox,NUMPOINTS * sizeof(POINTL),PAG_READ | PAG_WRITE | PAG_COMMIT)){

    /*
    **  initialize array for L-shape
    */

    ptlBox[0].x = MapX(5*delta_x);
    ptlBox[0].y = MapY(4*delta_y);
    ptlBox[1].x = MapX(5*delta_x);
    ptlBox[1].y = MapY(delta_y);
    ptlBox[2].x = MapX(7*delta_x);
    ptlBox[2].y = MapY(delta_y);
    ptlBox[3].x = MapX(7*delta_x);
    ptlBox[3].y = MapY(2*delta_y);
    ptlBox[4].x = MapX(6*delta_x);
    ptlBox[4].y = MapY(2*delta_y);
    ptlBox[5].x = MapX(6*delta_x);
    ptlBox[5].y = MapY(4*delta_y);
    ptlBox[6].x = MapX(5*delta_x);
    ptlBox[6].y = MapY(4*delta_y);
    GreSetCurrentPosition(hDC, ptlBox);

    /*
    **  Busy code starts
    */

    if (!MAKEPSBUSY(hPS))
    {
      CWRITELOG(L_FAIL, 0, "GetBusy failed\n");
    }

    /*
    **  modify the following two lines for your individual function
    */

    return_code = GrePolyLine(hDC, ptlBox, 6L);
    lErrorCount += CHECKRETURNERROR(return_code, "GrePolyLine PMERR_HDC_BUSY",
       SEVERITY_ERROR, PMERR_HDC_BUSY);

    DosFreeMem((PVOID)ptlBox);
    }
    FREEPS;


    /*
    **  Busy code ends
    */

    CWRITELOG(L_LOTRACE, 0, "+ix+ GrePolyLine PMERR_HDC_BUSY Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GrePolyLineRC                                    "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GrePolyLineRC                                    "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GrePolyLineRC - Exit.\n");

  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GreDrawLinesInPathRC()
 *
 * DESCRIPTION   = Exhaustive testing of GreDrawLinesInPath return codes
 *
 *                 Return Codes Tested:
 *                     PMERR_INV_HDC
 *
 *                 Return Codes Not Tested Due To Documentation Problems:
 *                    PMERR_HDC_BUSY
 *
 *                 Return Codes Not Tested:
 *                    PMERR_INV_IN_AREA
 *                    PMERR_INV_IN_PATH
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *                    PMERR_COORDINATE_OVERFLOW
 *                    PMERR_BITMAP_NOT_SELECTED
 *                    PMERR_INV_LENGTH_OR_COUNT
 *                    PMERR_INV_COORD_SPACE
 *                    PMERR_INV_COLOR_DATA
 *                    PMERR_INV_COLOR_INDEX
 *                    PMERR_INV_PICK_APERTURE_POSN
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 test for Invalid Device Context Handle  PMERR_INV_HDC
 *                 bad_hDC = meaningless value;
 *                 call function
 *                 return_code = GreDrawLinesInPath(bad_hDC,pPath,pLine,cLines);
 *                 report error value to DTT_LOG
 *                 CHECKRETURNERROR ( return_code,
 *                                    "GreDrawLinesInPath PMERR_INV_HDC",
 *                                    SEVERITY_ERROR,
 *                                    PMERR_INV_HDC );
 *
 ****************************************************************************/

VOID APIENTRY GreDrawLinesInPathRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC bad_hDC;
  LONG return_code = GPI_ERROR;
  PCURVE pPath;
  PLINE pLine;
  ULONG cLines;
  LONG lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreDrawLinesInPathRC - Entry.\n");

    /*
    **  To properly test an invalid parameter, all the other
    **  parameters must be valid
    */

    pPath = (PCURVE)malloc((size_t)sizeof(CURVE));
    pPath->bIdent = 0;
    pPath->bType = LINE_IDENTIFIER;
    pPath->usReserved = 0;
    pPath->flStyle = 0;
    pPath->fl = 0;
    pPath->pcvNext = (CURVE *)NULL;
    pPath->pcvPrev = (CURVE *)NULL;
    pLine = (PLINE)malloc((size_t)sizeof(LINE));
    pLine->bIdent = 0;
    pLine->bType = LINE_IDENTIFIER;
    pLine->flStyle = 0;
        pLine->pcvNext = (CURVE NEAR *)NULL;
    pLine->pcvPrev = (CURVE NEAR *)NULL;

    pLine->ptlA.x = 25;
    pLine->ptlA.y = 50;
    pLine->ptlC.x = 75;
    pLine->ptlC.y = 75;
    pLine->ptfxA.x = 25*FIXEDTOLONG;
    pLine->ptfxA.y = 50*FIXEDTOLONG;
    pLine->ptfxC.x = 75*FIXEDTOLONG;
    pLine->ptfxC.y = 75*FIXEDTOLONG;

    /*
    ** Test the following Gre call for an invalid device context
    ** handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GreDrawLinesInPath PMERR_INV_HDC Test - Entry.\n");
    bad_hDC = 0;
    /* This call is only supported in R206 environment. */
    /* return_code = GreDrawLinesInPath(bad_hDC, pPath, pLine, 1L);*/
    lErrorCount += CHECKRETURNERROR(return_code,
       "GreDrawLinesInPath PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GreDrawLinesInPath PMERR_INV_HDC Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (pPath != NULL)
      free(pPath);

    if (pLine != NULL)
      free(pLine);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GreDrawLinesInPathRC                             "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GreDrawLinesInPathRC                             "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GreDrawLinesInPathRC - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GrePolyShortLineRC()
 *
 * DESCRIPTION   = Exhaustive testing of GrePolyShortLine return codes.
 *
 *                 Return Codes Tested:
 *                    PMERR_INV_HDC
 *
 *                 Return Codes Not Tested Due To Documentation Problems:
 *                    PMERR_HDC_BUSY
 *
 *                 Return Codes Not Tested:
 *                    PMERR_INV_IN_AREA
 *                    PMERR_INV_IN_PATH
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *                    PMERR_COORDINATE_OVERFLOW
 *                    PMERR_BITMAP_NOT_SELECTED
 *                    PMERR_INV_LENGTH_OR_COUNT
 *                    PMERR_INV_COORD_SPACE
 *                    PMERR_INV_COLOR_DATA
 *                    PMERR_INV_COLOR_INDEX
 *                    PMERR_INV_PICK_APERTURE_POSN
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 test for Invalid Device Context Handle  PMERR_INV_HDC
 *                 bad_hDC = meaningless value;
 *                 call function
 *                 return_code = GrePolyShortLine(bad_hDC,pSline);
 *                 report error value to DTT_LOG
 *                 CHECKRETURNERROR ( return_code,
 *                                    "GrePolyShortLine PMERR_INV_HDC",
 *                                    SEVERITY_ERROR,
 *                                    PMERR_INV_HDC );
 *
 ****************************************************************************/

VOID APIENTRY GrePolyShortLineRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  HDC bad_hDC;
  PSHORTLINE pSline;
  LONG return_code,lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GrePolyShortLineRC - Entry.\n");

    /*
    **  Test the following Gre call for an invalid device context
    **  handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GrePolyShortLine PMERR_INV_HDC Test - Entry.\n");
    bad_hDC = 0;
    return_code = GrePolyShortLine(bad_hDC, pSline);
    lErrorCount += CHECKRETURNERROR(return_code,
       "GrePolyShortLine PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GrePolyShortLine PMERR_INV_HDC Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GrePolyShortLineRC                               \
              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GrePolyShortLineRC                               \
              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GrePolyShortLineRC - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GrePolyScanLineRC()
 *
 * DESCRIPTION   = Exhaustive testing of GrePolyScanLine return codes.
 *
 *                 Return Codes Tested:
 *                    PMERR_INV_HDC
 *
 *                 Return Codes Not Tested Due To Documentation Problems:
 *                    PMERR_HDC_BUSY
 *
 *                 Return Codes Not Tested:
 *                    PMERR_INV_IN_AREA
 *                    PMERR_INV_IN_PATH
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *                    PMERR_COORDINATE_OVERFLOW
 *                    PMERR_BITMAP_NOT_SELECTED
 *                    PMERR_INV_LENGTH_OR_COUNT
 *                    PMERR_INV_RECT
 *                    PMERR_INV_COORDINATE
 *                    PMERR_INV_REGION_CONTROL
 *                    PMERR_INV_COORD_SPACE
 *                    PMERR_INV_COLOR_DATA
 *                    PMERR_INV_COLOR_INDEX
 *                    PMERR_INV_PICK_APERTURE_POSN
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 setup conditions to test PMERR_INV_HDC error code
 *                 bad_hDC = meaningless value;
 *                             - to check for     return code
 *                 pass the     presentation space handle expecting an error
 *                 return code
 *                 return_code = GrePolyScanLine(bad_hDC,pScanData);
 *                 CHECKRETURNERROR(...);
 *
 ****************************************************************************/

VOID APIENTRY GrePolyScanLineRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PSCANDATA pScanData;
  LONG return_code;
  HDC bad_hDC;
  LONG lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GrePolyScanLineRC - Entry.\n");

    /*
    **  Test the following Gre call for an invalid device context
    **  handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GrePolyScanLine PMERR_INV_HDC Test - Entry.\n");
    bad_hDC = 0;
    return_code = GrePolyScanline(bad_hDC, pScanData);
    lErrorCount += CHECKRETURNERROR(return_code,
       "GrePolyScanLine PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GrePolyScanLine PMERR_INV_HDC Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GrePolyScanLineRC                                "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GrePolyScanLineRC                                "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GrePolyScanLineRC - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GreGetLineOriginRC()
 *
 * DESCRIPTION   = Exhaustive testing of GreGetLineOrigin return codes.
 *
 *                 Return Codes Tested:
 *                    PMERR_INV_HDC
 *
 *                 Return Codes Not Tested:
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 setup conditions to test PMERR_INV_HDC error code
 *                 bad_hDC = meaningless value; - to check for     return code
 *
 *                 pass the     presentation space handle expecting an error
 *                    return code
 *                 return_code = GreGetLineOrigin(bad_hDC,pptXY);
 *                 CHECKRETURNERROR(...);
 *
 ****************************************************************************/

VOID APIENTRY GreGetLineOriginRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PPOINTL pptXY;
  LONG return_code;
  HDC bad_hDC;
  LONG lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreGetLineOriginRC - Entry.\n");

    /*
    **  TO CORRECTLY TEST FOR AN INVALID PARAMETER, THE OTHER
    **  PARAMETERS MUST ALL BE VALID.
    */

    pptXY = (PPOINTL)malloc((size_t)sizeof(POINTL));

    /*
    **  Test the following Gre call for an invalid device context
    **  handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GreGetLineOrigin PMERR_INV_HDC Test - Entry.\n");
    bad_hDC = 0;
    return_code = GreGetLineOrigin(bad_hDC, pptXY);
    lErrorCount += CHECKRETURNERROR(return_code,
       "GreGetLineOrigin PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GreGetLineOrigin PMERR_INV_HDC Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (pptXY != NULL)
      free(pptXY);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GreGetLineOriginRC                               "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GreGetLineOriginRC                               "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GreGetLineOriginRC - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GreSetLineOriginRC()
 *
 * DESCRIPTION   = Exhaustive testing of GreSetLineOrigin return codes.
 *
 *                 Return Codes Tested:
 *                    PMERR_INV_HDC
 *
 *                 Return Codes Not Tested Due To Documenation Problems:
 *                    PMERR_HDC_BUSY
 *
 *                 Return Codes Not Tested:
 *                    PMERR_BASE_ERROR
 *                    PMERR_PATH_LIMIT_EXCEEDED
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *                    PMERR_COORDINATE_OVERFLOW
 *                    PMERR_INV_LENGTH_OR_COUNT
 *                    PMERR_INV_COORD_SPACE
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 setup conditions to test PMERR_INV_HDC error code
 *                 bad_hDC = meaningless value; - to check for     return code
 *                 pass the     presentation space handle expecting an error
 *                 return code
 *                 return_code = GreSetLineOriginRC(bad_hDC,pptXY,lStyle);
 *                 CHECKRETURNERROR(...);
 *
 ****************************************************************************/

VOID APIENTRY GreSetLineOriginRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PPOINTL pptXY;
  ULONG lStyle;
  LONG return_code;
  HDC bad_hDC;
  LONG lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreSetLineOriginRC - Entry.\n");

    /*
    **  To correctly test for an invalid parameter, all of the
    **  other parameters must be valid
    */

    pptXY = (PPOINTL)malloc((size_t)sizeof(POINTL));
    pptXY->x = 0;
    pptXY->y = 0;
    lStyle = 0;

    /*
    **  Test the following Gre call for an invalid device context
    **  handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GreSetLineOrigin PMERR_INV_HDC Test - Entry.\n");
    bad_hDC = 0;
    return_code = GreSetLineOrigin(bad_hDC, pptXY, lStyle);
    lErrorCount += CHECKRETURNERROR(return_code,
       "GreSetLineOrigin PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GreSetLineOrigin PMERR_INV_HDC Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (pptXY != NULL)
      free(pptXY);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GreSetLineOriginRC                               "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GreSetLineOriginRC                               "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GreSetLineOriginRC - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GreGetStyleRatioRC()
 *
 * DESCRIPTION   = Exhaustive testing of GreGetStyleRatio return codes.
 *
 *                 Return Codes Tested:
 *                    PMERR_INV_HDC
 *
 *                 Return Codes Not Tested:
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 setup conditions to test PMERR_INV_HDC error code
 *                 bad_hDC = meaningless value; - to check for     return code
 *
 *                 pass the     presentation space handle expecting an error
 *                 return code
 *                 return_code = GreGetStyleRatio(bad_hDC,pRatio);
 *                 CHECKRETURNERROR(...);
 *
 ****************************************************************************/

VOID APIENTRY GreGetStyleRatioRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PBYTE pRatio;
  LONG return_code;
  HDC bad_hDC;
  LONG lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreGetStyleRatioRC - Entry.\n");

    /*
    **  To correctly test for an invalid parameter, all of the
    **  other parameters must be valid
    */

    pRatio = (PBYTE)malloc((size_t)sizeof(BYTE));

    /*
    **  Test the following Gre call for an invalid device context
    **  handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GreGetStyleRatio PMERR_INV_HDC Test - Entry.\n");
    bad_hDC = 0;
    return_code = GreGetStyleRatio(bad_hDC, pRatio);
    lErrorCount += CHECKRETURNERROR(return_code,
       "GreGetStyleRatio PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GreGetStyleRatio PMERR_INV_HDC Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (pRatio != NULL)
      free(pRatio);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GreGetStyleRatioRC                               "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GreGetStyleRatioRC                               "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GreGetStyleRatioRC - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GreSetStyleRatioRC()
 *
 * DESCRIPTION   = Exhaustive testing of GreSetStyleRatio return codes.
 *
 *                 Return Codes Tested:
 *                    PMERR_INV_HDC
 *
 *                 Return Codes Not Tested:
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 setup conditions to test PMERR_INV_HDC error code
 *                 bad_hDC = meaningless value; - to check for     return code
 *
 *                 pass the     presentation space handle expecting an error
 *                 return code
 *                 return_code = GreSetStyleRatio(bad_hDC,pRatio);
 *                 CHECKRETURNERROR(...);
 *
 *
 ****************************************************************************/

VOID APIENTRY GreSetStyleRatioRC(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  unsigned char chRatio[2];
  LONG return_code;
  HDC bad_hDC;
  LONG lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreSetStyleRatioRC - Entry.\n");

    /*
    ** Test the following Gre call for an invalid device context
    ** handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GreSetStyleRatioRC PMERR_INV_HDC Test - Entry.\n");
    chRatio[0] = 0040;
    chRatio[1] = 0040;
    bad_hDC = 0;
    return_code = GreSetStyleRatio(bad_hDC, chRatio);
    lErrorCount += CHECKRETURNERROR(return_code,
       "GreSetStyleRatio PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GreSetStyleRatio PMERR_INV_HDC Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GreSetStyleRatioRC                               "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GreSetStyleRatioRC                               "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GreSetStyleRatioRC - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GreGetCurrentPositionRC()
 *
 * DESCRIPTION   = Exhaustive testing of GreGetCurrentPosition return codes.
 *
 *                 Return Codes Tested:
 *                    PMERR_INV_HDC
 *
 *                 Return Codes Not Tested:
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 setup conditions to test PMERR_INV_HDC error code
 *                 bad_hDC = meaningless value; - to check for     return code
 *
 *                 pass the     presentation space handle expecting an error
 *                 return code
 *                 return_code = GreGetCurrentPosition(bad_hDC,pptPosition);
 *                 CHECKRETURNERROR(...);
 *
 ****************************************************************************/

VOID APIENTRY GreGetCurrentPositionRC(HPS hPS,HWND hWndClient,BOOL
                                       SelectionCall)
{
  PPOINTL pptPosition;
  LONG return_code;
  HDC bad_hDC;
  LONG lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreGetCurrentPositionRC - Entry.\n");

    /*
    **  To correctly test for an invalid parameter, all of the
    **  other parameters must be valid
    */

    pptPosition = (PPOINTL)malloc((size_t)sizeof(POINTL));

    /*
    **  Test the following Gre call for an invalid device context
    **  handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GreGetCurrentPosition PMERR_INV_HDC Test - Entry.\n");
    bad_hDC = 0;
    return_code = GreGetCurrentPosition(bad_hDC, pptPosition);
    lErrorCount += CHECKRETURNERROR(return_code,
       "GreGetCurrentPosition PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GreGetCurrentPosition PMERR_INV_HDC Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (pptPosition != NULL)
      free(pptPosition);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GreGetCurrentPositionRC                          "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GreGetCurrentPositionRC                          "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GreGetCurrentPositionRC - Exit.\n");
  }
}

/***************************************************************************
 *
 * FUNCTION NAME = GreSetCurrentPositionRC()
 *
 * DESCRIPTION   = Exhaustive testing of GreSetCurrentPosition return codes.
 *
 *                 Return Codes Tested:
 *                    PMERR_INV_HDC
 *                    PMERR_HDC_BUSY
 *
 *                 Return Codes Not Tested:
 *                    PMERR_BASE_ERROR
 *                    PMERR_PATH_LIMIT_EXCEEDED
 *                    PMERR_DEV_FUNC_NOT_INSTALLED
 *                    PMERR_COORDINATE_OVERFLOW
 *                    PMERR_INV_LENGTH_OR_COUNT
 *                    PMERR_INV_COORD_SPACE
 *
 * INPUT         = hPS           - presentation space
 *                 hWndClient    - client window
 *                 SelectionCall - selection parameter
 *
 * OUTPUT        = Screen graphics and DTT_log data
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 setup conditions to test PMERR_INV_HDC error code
 *                 bad_hDC = meaningless value; - to check for     return code
 *
 *                 pass the     presentation space handle expecting an error
 *                      return code.
 *
 *                 return_code = GreSetCurrentPosition(bad_hDC,pptPosition);
 *                 CHECKRETURNERROR(...);
 *
 *
 ****************************************************************************/

VOID APIENTRY GreSetCurrentPositionRC(HPS hPS,HWND hWndClient,BOOL
                                       SelectionCall)
{
  PPOINTL pptPosition;
  LONG return_code;
  HDC bad_hDC,hDC;
  LONG lErrorCount;
  CHAR out_string[100];


  if (SelectionCall)
  {
    WinSendMsg(hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

    *pBasisX = 10000;
    *pBasisY = 10000;
  }

  else
  {
    hDC = set_delta(hPS);
    lErrorCount = 0;
    LOG_SIGNATURE;
    pptPosition = malloc(sizeof(POINTL));
    pptPosition->x = 0;
    pptPosition->y = 0;
    CWRITELOG(L_LOTRACE, 0, "+ee+ GreSetCurrentPositionRC - Entry.\n");

    /*
    **  Test the following Gre call for an invalid device context
    **  handle error.  The hDC passed is invalid if a 0 is passed.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GreSetCurrentPosition PMERR_INV_HDC Test - Entry.\n");
    bad_hDC = 0;
    return_code = GreSetCurrentPosition(bad_hDC, pptPosition);
    lErrorCount += CHECKRETURNERROR(return_code,
       "GreGetCurrentPosition PMERR_INV_HDC", SEVERITY_ERROR, PMERR_INV_HDC);
    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GreSetCurrentPosition PMERR_INV_HDC Test - Exit.\n");

    if (pptPosition != NULL)
      free(pptPosition);

    /*
    **  Test the following Gre call for a hdc_busy return code.
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ie+ GreSetCurrentPosition PMERR_HDC_BUSY Test - Entry.\n");

     if(!DosAllocMem((PVOID)&pptPosition,NUMPOINTS * sizeof(POINTL),PAG_READ | PAG_WRITE | PAG_COMMIT)){
    pptPosition[0].x = MapX(5*delta_x);
    pptPosition[0].y = MapY(4*delta_y);

    /*
    **  Busy code starts
    */

    if (!MAKEPSBUSY(hPS))
    {
      CWRITELOG(L_FAIL, 0, "GetBusy failed\n");
    }

    /*
    **  modify the following two lines for your individual function
    */

    return_code = GreSetCurrentPosition(hDC, pptPosition);
    lErrorCount += CHECKRETURNERROR(return_code,
       "GreSetCurrentPosition PMERR_HDC_BUSY", SEVERITY_ERROR, PMERR_HDC_BUSY);
    DosFreeMem((PVOID)pptPosition);
    }
    FREEPS;

    /*
    **  Busy code ends
    */

    CWRITELOG(L_LOTRACE, 0,
       "+ix+ GreSetCurrentPosition PMERR_HDC_BUSY Test - Exit.\n");

    /*
    **  Kill all output that is generated.
    **  Pass/Fail should be determined by the Log file.
    */

    DevEscape(GpiQueryDevice(hPS),
              DEVESC_ABORTDOC,
              (LONG)NULL,
              NULL,
              NULL,
              NULL);

    if (!lErrorCount)
    {
      CWRITELOG(L_HDR, 0, "$GreSetCurrentPositionRC                          "
         "              -    PASS.\n");
    }

    else
    {
      sprintf(out_string, "$ERRORCOUNT = %d.\n", lErrorCount);
      CWRITELOG(L_HDR, 0, out_string);
      CWRITELOG(L_HDR, 0, "$GreSetCurrentPositionRC                          "
         "              -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE, 0, "+ex+ GreSetCurrentPositionRC - Exit.\n");
  }
}

