/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GREMARK.C")
#pragma subtitle("IBM Confidential")

/*const char header=$header$*/
/***************************************************************************
 *\\ddd
 * Subtask Name:  PDVT
 *
 * Module name:   gremark.c
 *
 * Purpose:       This module is called by the DTT to test the GRE Marker
 *                function.
 *
 * Revision Log:  10/29/90, PDVT, Kim Cairns
 *                - Original module design
 *                12/04/90, PDVT, Kim Cairns
 *                - Code completion
 *                12/13/90, pdvt, markrichardson
 *                - trace level changes for DTT_log
 *
 * SRS References:  Section 6.6
 *
 * Contents:
 *               InitTest()
 *               gre_mark_pre()
 *               gre_mark_exh()
 *               gre_mark_rc()
 *               hdc_busy()
 *
 * Design Overview:
 *   There is one GRE Marker function, GrePolyMarker, which draws a sequence
 *   of markers at specified positions.  The API is tested for precision,
 *   basic functionality, and return codes affecting the printer driver.
 *
 * Limitations:
 *               None
 *
 *\\end
 ***************************************************************************/

#define INCL_DOS
#define INCL_DOSPROCESS

#define INCL_WIN
#define INCL_GPI
#define INCL_GREALL
#define INCL_DDIMISC
#define INCL_ERRORS
#define INCL_DEV
#include <string.h>
#include <os2.h>
#include <stdio.h>
#include <pmddi.h>    // gre include file
#include "DTTDLLDF.h" // test case defines
#include "gremark.h"  // gre marker dll header file
#include "gre_grid.h" // standard grid header file
#include "std_util.h"  /* screen comments */

#define LCID_MYCOUR    1L
#define LCID_MYHELV    2L
#define LCID_MYTIMES   3L

LONG    lErrorCount;
 /***************************************************************************
 *\\ddd
 * Routine name: InitTest()
 *
 * Purpose:  Initialize function name and address array for DTT.
 *
 * System Requirements:  None
 *
 * Revision Log:  10/29/90, PDVT, Kim Cairns
 *                - initial function design
 *                12/03/90, PDVT, Kim Cairns
 *                - added group_name
 *
 * Inputs:
 *             sel            - pass thru parameter for other function call
 *             pInitInfo      - pointer to info array
 *
 * Outputs:
 *             pInitInfo      - updated with all entry points names and addresses
 *
 * Subroutines Required:
 *             InitTestSegVars()
 *
 * Limitations:
 *             None
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *   for all test functions in this test case {
 *       pInitInfo->function name = function name[i]
 *       pInitInfo->function address = function address[i]
 *   }
 *   call init set function
 *\\end
 ***************************************************************************/
VOID APIENTRY InitTest(PVOID sel, PTESTINITINFO pInitInfo)
  {
    /**** Initialize total number of tests in test case ****/
    pInitInfo->ntests = NTESTS;

    /**** Load test names and entry point addresses ****/
    strcpy(pInitInfo->group_name,"GRE Marker");
    strcpy(pInitInfo->test_names[0],"GreMarkerPre");
    pInitInfo->test_addrs[0]=gre_mark_pre;
    strcpy(pInitInfo->test_names[1],"GreMarkerExh");
    pInitInfo->test_addrs[1]=gre_mark_exh;

    InitTestSegVars(sel);
  }
 /***************************************************************************
 *\\ddd
 * Routine name:  gre_mark_pre()
 *
 * Purpose:  This is an entry point for the DTT that causes GRE Marker
 *               precision tests to be run.
 *
 * System Requirements:  Section 5.1.2
 *
 * Revision Log:  10/29/90, PDVT, Kim Cairns
 *                - initial function design
 *
 * Inputs:
 *             hPS            - presentation space handle
 *             hWndClient     - client window handle
 *             SelectionCall  - selection indicator
 *
 * Outputs:
 *
 * Subroutines Required:
 *             gre_grid()      - standard grid function using GREs
 *
 * Limitations:
 *             None
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *  //basic function called here and measured against standard grid
 *  GrePolyMarker(hdc...);
 *
 *\\end
 ***************************************************************************/

VOID APIENTRY gre_mark_pre(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  CHAR         out_string[80];
  HDC          hDC;
  POINTL       ptl1;
  PPOINTL      paptPoint, pptPosition;
  LONG         cPoints;
  LONG         cHits;

  cPoints = 1;
  pAttrs = &mbnd;
  lErrorCount = 0L;  /* initialize the error count */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

     TEST_SIGNATURE(0,0);
     LOG_SIGNATURE;
     CWRITELOG(L_LOTRACE, 0, "+ee+ GreMarkerPre - Entry.\n");

     hDC = GpiQueryDevice(hPS);
     /*
      * Call grid to measure precision.
      */
     gre_grid(hPS, (LONG)INCH);
     /*
      * Plot markers on points of grid, using all available symbols.
      */
     ptl1.x = delta_x;
     ptl1.y = 2*delta_y;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 0;
     ptl1.y = 3*delta_y/2;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Default - X at (1,2)");

     ptl1.x = 3*delta_x;
     ptl1.y = 4*delta_y;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_DIAMOND;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 2*delta_x + 100;
     ptl1.y = 7*delta_y/2 + (delta_y * 1/5);
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Open Diamond at (3,4)");

     ptl1.x = 3*delta_x;
     ptl1.y = 2*delta_y;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_SQUARE;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 2*delta_x + 100;
     ptl1.y = 3*delta_y/2 + (delta_y * 1/5);
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Open Square at (3,2)");

     ptl1.x = 1*delta_x;
     ptl1.y = 4*delta_y;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_SIXPOINTSTAR;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 0;
     ptl1.y = 7*delta_y/2;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"6pt Star at (1,4)");

     ptl1.x = 1*delta_x;
     ptl1.y = 6*delta_y;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_EIGHTPOINTSTAR;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 0;
     ptl1.y = 11*delta_y/2;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"8pt Star at (1,6)");

     ptl1.x = 3*delta_x;
     ptl1.y = 6*delta_y;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_SOLIDDIAMOND;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 2*delta_x + 100;
     ptl1.y = 11*delta_y/2 + (delta_y * 1/5);
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Solid Diamond at (3,6)");

     ptl1.x = 5*delta_x;
     ptl1.y = 6*delta_y;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_SOLIDSQUARE;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 9*delta_x/2;
     ptl1.y = 11*delta_y/2;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Solid Square at (5,6)");

     ptl1.x = 7*delta_x/2;
     ptl1.y = 5*delta_y/2;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_DOT;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 7*delta_x/2;
     ptl1.y = 5*delta_y/2 + 200;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Dot at (3.5,2.5)");

     ptl1.x = 5*delta_x;
     ptl1.y = 4*delta_y;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_CROSS;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 9*delta_x/2;
     ptl1.y = 7*delta_y/2;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Explicit Cross at (5,4)");

     ptl1.x = 5*delta_x;
     ptl1.y = 2*delta_y;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_SMALLCIRCLE;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 9*delta_x/2;
     ptl1.y = 3*delta_y/2;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Circle at (5,2)");

     ptl1.x = 3*delta_x/2;
     ptl1.y = 9*delta_y/2;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_BLANK;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = delta_x * 0;
     ptl1.y = 9*delta_y/2 + 200;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Blank at (1.5,4.5)");

     ptl1.x = 7*delta_x/2;
     ptl1.y = 9*delta_y/2;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_DEFAULT;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = 7*delta_x/2;
     ptl1.y = 9*delta_y/2 + 200;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Explicit Default at (3.5,4.5)");

     ptl1.x = 3*delta_x/2;
     ptl1.y = 5*delta_y/2;
     MapPt(ptl1);
     pptPosition = &ptl1;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_PLUS;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.x = delta_x * 0;
     ptl1.y = 5*delta_y/2 + 200;
     MapAndWriteString(hPS,ptl1.x,ptl1.y,"Plus Sign at (1.5,2.5)");

     if (lErrorCount) {
       sprintf(out_string,"$ERRORCOUNT = %ld.\n",lErrorCount);
       CWRITELOG(L_HDR,0,out_string);
       CWRITELOG(L_HDR,0,
         "$GreMarkerPre                                                      - FAIL.\n");
     }
     else
       CWRITELOG(L_HDR,0,
         "$GreMarkerPre                                                      - PASS.\n");

     CWRITELOG(L_LOTRACE, 0, "+ex+ GreMarkerPre - Exit.\n");
  }
}
 /***************************************************************************
 *\\ddd
 * Routine name:  gre_mark_exh
 *
 * Purpose:  This is an entry point for the DTT that causes GRE Marker
 *               exhaustive tests to be run.
 *
 * System Requirements:  Section 5.1.2
 *
 * Revision Log:  10/29/90, PDVT, Kim Cairns
 *                - initial function design
 *
 * Inputs:
 *             hPS            - presentation space handle
 *             hWndClient     - client window handle
 *             SelectionCall  - selection indicator
 *
 * Outputs:
 *             None
 *
 * Subroutines Required:
 *             None
 *
 * Limitations:
 *             None
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *  //verify the current position is the center of the last marker
 *  rc = GrePolyMarker(hdc...);
 *  if (rc == GPI_OK) {
 *    log expected successful condition
 *  }
 *  else {
 *    log unexpected error condition
 *  }
 *
 * //set each unique marker parameter
 * //verify correct output
 *  rc = GrePolyMarker(hdc...);
 *  if (rc == GPI_OK) {
 *  ...
 *  }
 *
 *  //call gre with cPoints = NULL expecting success, but no output
 *  rc = GrePolyMarker(hdc...);
 *  if (rc == GPI_OK) {
 *  ...
 *  }
 *
 *\\end
 ***************************************************************************/

VOID APIENTRY gre_mark_exh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  CHAR         out_string[80];
  POINTL       ptl, ptl1, ptl2[7];
  HDC          hDC;
  PPOINTL      paptPoint, pptPosition;
  LONG         cPoints;
  LONG         cHits;

  pAttrs = &mbnd;
  lErrorCount = 0L;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
  }
  else {
     CWRITELOG(L_LOTRACE, 0, "+ee+ GreMarkerExh - Entry.\n");
     TEST_SIGNATURE(0,0);
     LOG_SIGNATURE;

     hDC = set_delta(hPS);
     cPoints = 2;
     /*
      * Test that current position is the center of the last marker.
      */
     ptl.x = delta_x/2;
     ptl.y = 4*delta_y;
     MapPt(ptl);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);

     ptl1.x = delta_x/2;
     ptl1.y = 3*delta_y;
     MapPt(ptl1);
     paptPoint = &ptl1;
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl1.y = MapY(2*delta_y);
     mbnd.usSymbol = MARKSYM_PLUS;
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl.x = delta_x;
     ptl.y = 4*delta_y;
     MapAndWriteString(hPS,ptl.x,ptl.y,"If the middle cross and plus");
     ptl.x = delta_x;
     ptl.y = 7*delta_y/2;
     MapAndWriteString(hPS,ptl.x,ptl.y,"are centered at the same point,");
     ptl.x = delta_x;
     ptl.y = 3*delta_y;
     MapAndWriteString(hPS,ptl.x,ptl.y,"the test proves that the");
     ptl.x = delta_x;
     ptl.y = 5*delta_y/2;
     MapAndWriteString(hPS,ptl.x,ptl.y,"current position is the");
     ptl.x = delta_x;
     ptl.y = 2*delta_y;
     MapAndWriteString(hPS,ptl.x,ptl.y,"center of the last marker.");

     cPoints = 8;
     ptl.x = delta_x/2;
     ptl.y = delta_y;
     MapPt(ptl);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     ptl2[0].x = 4*delta_x;
     ptl2[0].y = delta_y;
     MapPt(ptl2[0]);
     ptl2[1].x = 7*delta_x/2;
     ptl2[1].y = delta_y;
     MapPt(ptl2[1]);
     ptl2[2].x = 3*delta_x;
     ptl2[2].y = delta_y;
     MapPt(ptl2[2]);
     ptl2[3].x = 5*delta_x/2;
     ptl2[3].y = delta_y;
     MapPt(ptl2[3]);
     ptl2[4].x = 2*delta_x;
     ptl2[4].y = delta_y;
     MapPt(ptl2[4]);
     ptl2[5].x = 3*delta_x/2;
     ptl2[5].y = delta_y;
     MapPt(ptl2[5]);
     ptl2[6].x = delta_x;
     ptl2[6].y = delta_y;
     MapPt(ptl2[6]);
     paptPoint = ptl2;
     mbnd.usSymbol = MARKSYM_SOLIDDIAMOND;        // Set Symbol
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl.x = 9*delta_x/2;
     ptl.y = delta_y;
     MapAndWriteString(hPS,ptl.x,ptl.y,"These solid diamond markers");
     ptl.x = 9*delta_x/2;
     ptl.y = delta_y - 225;
     MapAndWriteString(hPS,ptl.x,ptl.y,"plotted with array.");
     /*
      * Test setting each attribute in MARKERBUNDLE structure.
      */
     ptl.x = delta_x;
     ptl.y = 7*delta_y + 150;
     MapAndWriteString(hPS,ptl.x,ptl.y,
          "Setting the following marker attributes will not affect printer output.");
     ptl.y = 7*delta_y;
     ptl.x = delta_x;
     MapAndWriteString(hPS,ptl.x,ptl.y,
      "Specifically, marker backgrounds will not appear on printer output.");

     cPoints = 1;
     ptl.x = delta_x/2;
     ptl.y = 6*delta_y;
     MapPt(ptl);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_SMALLCIRCLE;         // Set Symbol
     mbnd.lColor = CLR_GREEN;                            // Set Color
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL | MBB_COLOR, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl.y = MapY(11*delta_y/2);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_EIGHTPOINTSTAR;         // Set Symbol
     mbnd.lBackColor = CLR_PINK;                        // Set Background Color
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL | MBB_BACK_COLOR, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl.x = delta_x;
     ptl.y = 11*delta_y/2+75;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Eight point star");
     ptl.x = delta_x;
     ptl.y = 11*delta_y/2-150;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Set Background Color.");
     ptl.x = delta_x;
     ptl.y = 6*delta_y+75;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Small circle");
     ptl.x = delta_x;
     ptl.y = 6*delta_y-150;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Set Color");

     ptl.x = MapX(5*delta_x/2);
     ptl.y = MapY(5*delta_y);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_SIXPOINTSTAR;           // Set Symbol
     mbnd.usMixMode = FM_OVERPAINT;                  // Set Mix Mode
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL | MBB_MIX_MODE, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl.y = MapY(9*delta_y/2);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_SQUARE;                  // Set Symbol
     mbnd.usBackMixMode = FM_OVERPAINT;              // Set Background Mix Mode
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL | MBB_BACK_MIX_MODE, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl.x = 3*delta_x;
     ptl.y = 9*delta_y/2 + 75;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Square");
     ptl.x = 3*delta_x;
     ptl.y = 9*delta_y/2 - 150;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Set Background Mix Mode.");
     ptl.x = 3*delta_x;
     ptl.y = 5*delta_y + 75;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Six point star");
     ptl.x = 3*delta_x;
     ptl.y = 5*delta_y - 150;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Set Mix Mode.");


     ptl.x = MapX(9*delta_x/2);
     ptl.y = MapY(6*delta_y);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_DIAMOND;               // Set Symbol
     EzfQueryFonts(hPS);
     if (EzfCreateLogFont(hPS, LCID_MYCOUR, FONTFACE_COUR, FONTSIZE_10, 0))
        mbnd.usSet = LCID_MYCOUR;                         // Set Font
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL | MBB_SET, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl.y = MapY(11*delta_y/2);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_SOLIDSQUARE;            // Set Symbol
     mbnd.sizfxCell.cx = MAKEFIXED(10L, 0);      // Set Cell Dimensions
     mbnd.sizfxCell.cy = MAKEFIXED(10L, 0);
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL | MBB_BOX, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl.x = 5*delta_x;
     ptl.y = 11*delta_y/2 + 75;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Set Marker Font");
     ptl.x = 5*delta_x;
     ptl.y = 11*delta_y/2 - 150;
     MapAndWriteString(hPS,ptl.x,ptl.y,"to Courier.");
     ptl.x = 5*delta_x;
     ptl.y = 6*delta_y + 75;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Set Cell Dimensions");
     ptl.x = 5*delta_x;
     ptl.y = 6*delta_y - 150;
     MapAndWriteString(hPS,ptl.x,ptl.y,"using Courier font.");
     /*
      * Return attributes to default (except symbol)
      */

     ptl.x = MapX(5*delta_x);
     ptl.y = MapY(4*delta_y);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     ptl1.x = MapX(5*delta_x);
     mbnd.usSymbol = MARKSYM_DOT;                 // Set Symbol
     GreSetAttributes(hDC, PRIM_MARKER,
      MBB_COLOR | MBB_BACK_COLOR | MBB_MIX_MODE | MBB_BACK_MIX_MODE | MBB_SET,
      MBB_COLOR | MBB_BACK_COLOR | MBB_MIX_MODE | MBB_BACK_MIX_MODE | MBB_SET | MBB_SYMBOL,
      pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     /*
      * Verify BLANK marker.
      */
     ptl.y = MapY(3*delta_y);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     ptl1.x = MapX(1*delta_x);
     mbnd.usSymbol = MARKSYM_BLANK;               // Set Symbol
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     /*
      * Test cPoints=NULL returns successful, but produces no output.
      */

     cPoints = NULL;
     ptl.y = MapY(2*delta_y);
     pptPosition = &ptl;
     GreSetCurrentPosition(hDC, pptPosition);
     mbnd.usSymbol = MARKSYM_CROSS;               // Set Symbol
     GreSetAttributes(hDC, PRIM_MARKER, NULL, MBB_SYMBOL, pAttrs);
     cHits = GrePolyMarker(hDC, paptPoint, cPoints);
     if (cHits == GPI_ERROR) {
        lErrorCount += BADERROR("GrePolyMarker", FALSE, NULL);
     }
     ptl.x = 11*delta_x/2;
     ptl.y = 2*delta_y;
     MapAndWriteString(hPS,ptl.x,ptl.y,"<-cPoints=NULL");
     ptl.x = 11*delta_x/2;
     ptl.y = 2*delta_y - 225;
     MapAndWriteString(hPS,ptl.x,ptl.y,"no output expected.");
     ptl.x = 11*delta_x/2;
     ptl.y = 3*delta_y;
     MapAndWriteString(hPS,ptl.x,ptl.y,"<-BLANK marker,");
     ptl.x = 11*delta_x/2;
     ptl.y = 3*delta_y - 225;
     MapAndWriteString(hPS,ptl.x,ptl.y,"no output expected.");
     ptl.x = 11*delta_x/2;
     ptl.y = 4*delta_y+75;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Dot");
     ptl.x = 11*delta_x/2;
     ptl.y = 4*delta_y-150;
     MapAndWriteString(hPS,ptl.x,ptl.y,"Above attributes");
     ptl.x = 11*delta_x/2;
     ptl.y = 4*delta_y-375;
     MapAndWriteString(hPS,ptl.x,ptl.y,"set to default.");

    if (lErrorCount) {
       sprintf(out_string,"$ERRORCOUNT = %ld.\n",lErrorCount);
       CWRITELOG(L_HDR,0,out_string);
       CWRITELOG(L_HDR,0,
         "$GreMarkerExh                                                      - FAIL.\n");
    }
    else
       CWRITELOG(L_HDR,0,
         "$GreMarkerExh                                                      - PASS.\n");

     CWRITELOG(L_LOTRACE, 0, "+ex+ GreMarkerExh - Exit.\n");
 }
}
 /***************************************************************************
 *\\ddd
 * Routine name:  gre_mark_rc
 *
 * Purpose:  This is an entry point for the DTT that causes GRE Marker
 *               return code tests to be run.
 *
 * System Requirements:  Section 5.1.2
 *
 * Revision Log:  11/21/90, PDVT, Kim Cairns
 *                - initial function design
 *
 * Inputs:
 *             hPS            - presentation space handle
 *             hWndClient     - client window handle
 *             SelectionCall  - selection indicator
 *
 * Outputs:
 *             None
 *
 * Subroutines Required:
 *             hdc_busy()
 *
 * Limitations:
 *             This test only covers the PMERR_HDC_BUSY return code because
 *             the only others that apply to the printer driver have no
 *             documentation on how to produce.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *  call function to test PMERR_HDC_BUSY
 *
 *\\end
 ***************************************************************************/

VOID APIENTRY gre_mark_rc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  CHAR         out_string[80];
  HDC          hDC;

  lErrorCount = 0L;
  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
  }
  else {
     LOG_SIGNATURE;
     CWRITELOG(L_LOTRACE, 0, "+ee+ GreMarkerRC - Entry.\n");
     hDC = set_delta(hPS);
     hdc_busy(hPS, hDC, hWndClient);
     if (lErrorCount) {
        sprintf(out_string,"$ERRORCOUNT = %ld.\n",lErrorCount);
        CWRITELOG(L_HDR,0,out_string);
        CWRITELOG(L_HDR,0,
          "$GreMarkerRC                                                       - FAIL.\n");
     }
     else
        CWRITELOG(L_HDR,0,
         "$GreMarkerRC                                                       - PASS.\n");

     CWRITELOG(L_LOTRACE, 0, "+ex+ GreMarkerRC - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,NULL
                                      ,NULL, NULL,NULL);
  }
}
 /***************************************************************************
 *\\ddd
 * Routine name:  hdc_busy
 *
 * Purpose:  This is a function that sets up the conditions and tests
 *               GrePolyMarker for PMERR_HDC_BUSY.
 *
 * System Requirements:  none
 *
 * Revision Log:  11/25/90, PDVT, Kim Cairns
 *                - initial function design
 *
 * Inputs:
 *             hDC            - device context handle
 *             hWndClient     - client window handle
 *
 * Outputs:
 *             None
 *
 * Subroutines Required:
 *             None
 *
 * Limitations:
 *             None
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *  setup conditions to test PMERR_HDC_BUSY
 *  rc = GrePolyMarker(hDC...);
 *  CHECKRETURNERROR(rc, "text", SEVERITY_ERROR, PMERR_HDC_BUSY);
 *
 *\\end
 ***************************************************************************/

VOID hdc_busy(HPS hPS, HDC hDC, HWND hWndClient)
{
  POINTL       ptl;
  PPOINTL      paptPoint, pptPosition;
  LONG         cPoints;
  LONG         i, cHits;

   CWRITELOG(L_LOTRACE, 0, "+ie+ hdc_busy - Entry.\n");

   // MAKEPSBUSY setup
   ptl.y = delta_y;
   ptl.x = delta_x;
   MapPt(ptl);
   GpiMove (hPS, &ptl);
   ptl.y = 3*delta_y;
   ptl.x = 2*delta_x;
   MapPt(ptl);

   cPoints = 1L;
   paptPoint = &ptl;

   //    BUSY CODE STARTS
   if(!MAKEPSBUSY(hPS)){
     CWRITELOG(L_FAIL,0,"GetBusy failed\n");
   }
                                         //TSS...passed address of cPoints
   cHits = GrePolyMarker(hDC, paptPoint, &cPoints);
   lErrorCount += CHECKRETURNERROR(cHits, "PMERR_HDC_BUSY",
                                    SEVERITY_ERROR, PMERR_HDC_BUSY);

   FREEPS;
   //    BUSY CODE ENDS

   CWRITELOG(L_LOTRACE, 0, "+ix+ hdc_busy - Exit.\n");
}
