/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GREMDEV.C")

#define MAINSOURCEFILE

#define INCL_WIN
#define INCL_AVIO
#define INCL_GPI
#define INCL_GREALL
#define INCL_DEV
#define INCL_DDIMISC
#define INCL_DDIBUNDLES
#define INCL_ERRORS

#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <pmddi.h>
#include <pmddim.h>
#include <pmfont.h>
#include <limits.h>

#include "DTTDLLDF.h"                // test case defines
#include "common1.h"
#include "GREMDEV.h"                 // gre misc. dev dll include file

/*****************************************************************************
 *\\ddd
 * Subtask Name: GSE
 *
 * Module Name: MDEVEXH
 *
 * Purpose:
 *
 *
 * Revision Log: 12/12/91, GSE, Stan Logan
 *                 -Test Case Creation
 *
 *
 * Design Overview:
 *
 *    This testcase is designed to exhaustively test the Gre Miscellaneous
 *    Device functions that:
 *
 *    1. Set the font for AVIO use.
 *       - GreDeviceSetAVIOFont()
 *    2. Invalidate a visible region by setting HDC_IS_DIRTY flag.
 *       - GreDeviceInvalidateVisRegion()
 *    3. Set PM screen in background to give other processes priority.
 *       - GreDeath()
 *    4. Reverse GreDeath.
 *       - GreResurrection()
 *    5. Set the coordinates for mapping the DC to the device.
 *       - GreDeviceSetDCOrigin()
 *
 * Limitations:
 *    None.
 *
 *\\end
 *****************************************************************************/

/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose:  Initialize function names and address array.
 *
 * System Requirements:
 *
 * Revision Log:
 *
 *
 * Inputs:
 *   sel          // passed through to the InitTestSegVars function
 *   init_info_p  // name and address array used by DTT interface
 *
 * Outputs:
 *   init_info_p  // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()    // used by the DTT interface
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *        Get the number of tests.
 *   for the number of tests {
 *                      Get the test names;
 *                      Get the test addresses;
 *                      }
 *   call InitTestSegVars function
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY InitTest(PVOID Selector, PTESTINITINFO init_info_p)
{
  register USHORT usTestCounter;

  init_info_p->ntests = NTESTS;
  strcpy(init_info_p->group_name,TestCaseName); //Testcase Name

  for (usTestCounter=0 ;usTestCounter< NTESTS; ++usTestCounter){

     init_info_p->test_addrs[usTestCounter]=
                               EntryPointList[usTestCounter].TestAddress;
     strcpy(init_info_p->test_names[usTestCounter],
                               EntryPointList[usTestCounter].TestName);
  }

  InitTestSegVars(Selector);
  return;
}

/***************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetAVIOFontExh
 *
 * Purpose:  This is an entry point for the DTT that exhaustively tests
 *           the GreDeviceSetAVIOFont function call.
 *
 * System Requirements:
 *
 * Revision Log: 12/12/91 - Initial Creation, Stan Logan
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *     ENTER_MAIN
 *     EXIT_MAIN
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * Log setup
 *    Get the HDC
 *    Create an AVIO PS
 *    Associate the AVIO PS to the DC
 *    Initialize the font definition structure
 *    Initialize the font attributes structure
 * Log setup completion
 * Log subentry
 *    Call GreRealizeFont to attempt to retrieve the font definitiion structure
 * Log subexit
 * Log subentry
 *    Call GreDeviceSetAVIOFont in hopes that GreRealizeFont set up fmDef
 * Log subexit
 * Disassociate the AVIO PS from the DC
 * Destroy the AVIO PS
 * No output, so suppress printer output
 *
 *\\end
 ***************************************************************************/
/*
VOID APIENTRY GreDeviceSetAVIOFontExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static   PCHAR EntryName = "GreDeviceSetAVIOFontExh";
   HVPS           hVPS;
   FOCAFONT       fmDef;
   FATTRS         Fattrs;
   ULONG          ReturnValue;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

   else {
      ENTER_MAIN(EntryName);

// Log setup

      SETENTER(" ", "Check for valid HDC");

// Get the HDC

      if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR)
         SHOWERROR("GpiQueryDevice", EntryName);

// Create an AVIO PS

      VIOReturn = VioCreatePS(&hVPS, MAXROW, MAXCOL, 0, FORMAT_4BYTE, 0);

      if (VIOReturn != NO_ERROR)
      {
         sprintf(achLogLine, "Error creating AVIO PS: %ud", VIOReturn);

         LOGINFO(L_FAIL, "@", achLogLine);
      }

// Associate the AVIO PS to the DC

      VIOReturn = VioAssociate(hDC, hVPS);

      if (VIOReturn != NO_ERROR)
      {
         sprintf(achLogLine, "Error associating AVIO PS: %ud", VIOReturn);

         LOGINFO(L_FAIL, "@", achLogLine);
      }

// Initialize the font definition structure

      memset((VOID *)&fmDef, 0x00, sizeof(FOCAFONT));

// Initialize the font attributes structure

      memset((VOID *)&Fattrs, 0x00, sizeof(FATTRS));

// Log setup completion

      SETEXIT();

// Log subentry

      SUBENTER(Realizing font, NULL);

// Set up the font attributes structure

      Fattrs.usRecordLength = sizeof(FATTRS);
      Fattrs.lMatch = 0L;
      Fattrs.usCodePage = 850;
      Fattrs.lMaxBaselineExt = 8;
      Fattrs.lAveCharWidth = 8;
      strcpy(Fattrs.szFacename, "Courier");

// Call GreRealizeFont to attempt to retrieve the font definitiion structure

      ReturnValue = GreRealizeFont(hDC, REALIZE_ENGINE_FONT, &Fattrs, &fmDef);

      if ((ReturnValue == GPI_ALTERROR) || (ReturnValue == GPI_ERROR))
         SHOWERROR("GreRealizeFont", EntryName);

// Log subexit

      SUBEXIT();

// Log subentry

      SUBENTER(Creating font, NULL);

      strcpy(fmDef.fmMetrics.szFacename, "Courier");

// Call GreDeviceSetAVIOFont in hopes that GreRealizeFont set up fmDef

      if (!GreDeviceSetAVIOFont(hDC, &fmDef, LCID_AVIO_1))
         ErrorCount++;

      ERRSTATUS(TRUE, "GreDeviceSetAVIOFont", EntryName);

// Log subexit

      SUBEXIT();

// Disassociate the AVIO PS from the DC

      VIOReturn = VioAssociate(NULL, hVPS);

      if (VIOReturn != NO_ERROR)
      {
         sprintf(achLogLine, "Error disassociating AVIO PS: %ud", VIOReturn);

         LOGINFO(L_FAIL, "@", achLogLine);
      }

// Destroy the AVIO PS

      VIOReturn = VioDestroyPS(hVPS);

      if (VIOReturn != NO_ERROR)
      {
         sprintf(achLogLine, "Error destroying AVIO PS: %ud", VIOReturn);

         LOGINFO(L_FAIL, "@", achLogLine);
      }

// Exit entry point

      EXIT_MAIN(EntryName);

// No output, so suppress printer output

      DeletePrinterOutput(EntryName);

   }

   return;

}*/ /* End GreDeviceSetAVIOFontExh  */

/***************************************************************************
 *\\ddd
 * Routine Name: GreDeviceInvalidateVisRegionExh
 *
 * Purpose:  This is an entry point for the DTT that exhaustively tests
 *           the GreDeviceInvalidateVisRegion function call.
 *
 * System Requirements:
 *
 * Revision Log: 12/12/91 - Initial Creation, Stan Logan
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN
 *    EXIT_MAIN
 *    SETENTER
 *    SETEXIT
 *    SUBENTER
 *    SUBEXIT
 *    SHOWERROR
 *    ERRSTATUS
 *    DeletePrinterOutput
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * If DeviceIsPrinter, abort entry point
 * Log setup
 *    Get the HDC
 * Log setup completion
 * Log subentry
 *    Call GreDeviceInvalidateVisRegion
 * Log subexit
 * No output, so suppress printer output
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceInvalidateVisRegionExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static   PCHAR    EntryName = "GreDevInvalVisRegionExh";
            DC_BLOCK DCBlock;
   HDC    hDC;
 LONG xparm ;
         if (SelectionCall)
            WinSendMsg (hWndClient, (USHORT)WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

     else {

// If DeviceIsPrinter, abort entry point

      if (DeviceIsPrinter())
      {
         ABORTENTRY(EntryName, "Cannot go to printer.");
         return;
      }

      ENTER_MAIN(EntryName);

// Log setup

      SETENTER(" ", "Querying the DC instance data");

// Get the HDC

      if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR)
         SHOWERROR("GpiQueryDevice", EntryName);

            DCBlock.hdc = (ULONG)hDC;
      xparm = 0L ;
          xparm  =  GetDriverInfo(hDC, 0, hDC);
          DCBlock.hddc = (ULONG)xparm;

// Log setup completion

      SETEXIT();

// Log subentry

      SUBENTER(Invalidating Visible Region, NULL);

// Call GreDeviceInvalidateVisRegion

      if (!GreDeviceInvalidateVisRegion(hDC, 1L, &DCBlock))
         ErrorCount++;

      ERRSTATUS(TRUE, "GreDeviceInvalidateVisRegion()", EntryName);

// Log subexit

      SUBEXIT();

// Exit entry point

      EXIT_MAIN(EntryName);

// No output, so suppress printer output

      DeletePrinterOutput(EntryName);

         }

   return;

} /* End GreDeviceInvalidateVisRegionExh  */

/***************************************************************************
 *\\ddd
 * Routine Name: GreDeathAndResurrectionExh
 *
 * Purpose:  This is an entry point for the DTT that exhaustively tests
 *           the GreDeath and GreResurrection function calls.
 *
 * System Requirements:
 *
 * Revision Log: 12/12/91 - Initial Creation, Stan Logan
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *     ENTER_MAIN
 *     EXIT_MAIN
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * Log setup
 *    Get the anchor block
 *    Get the window handle to the Desktop
 *    Grab the handle to the Desktop DC
 * Log setup completion
 * Log subentry
 *    Call GreDeath
 * Log subexit
 * Log subentry
 *    Attempt to resurrect the PM screen
 * Log subexit
 * No output, so suppress printer output
 *
 *\\end
 ***************************************************************************/
/*
VOID APIENTRY GreDeathAndResurrectionExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static   PCHAR EntryName = "GreDeathAndResurrectionExh";
            HWND  hWnd = (HWND)NULL;
            HAB   hAB = (HAB)NULL;

   if (SelectionCall)
      WinSendMsg (hWndClient, (USHORT)WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

   else {
      ENTER_MAIN(EntryName);

// Log setup

      SETENTER(" ", "Querying Desktop HDC");

// Get the anchor block

      if ((hAB = WinQueryAnchorBlock(HWND_DESKTOP)) == NULL)
         SHOWERROR("WinQueryAnchorBlock()", EntryName);

// Get the window handle to the Desktop

      if ((hWnd = WinQueryDesktopWindow(hAB, NULL)) == NULL)
         SHOWERROR("WinQueryDesktopWindow()", EntryName);

// Grab the handle to the Desktop DC

      if ((hDC = WinQueryWindowDC(hWnd)) == NULL)
         SHOWERROR("WinQueryWindowDC()", EntryName);

// Log setup completion

      SETEXIT();

// Log subentry

      SUBENTER(Killing the PM screen, NULL);

// Call GreDeath

      if (!GreDeath(hDC))
      {
         ErrorCount++;
         ERRSTATUS(TRUE, "GreDeath()", EntryName);
      }

// Log subexit

      SUBEXIT();

// Log subentry

      SUBENTER(Resurrecting the PM screen (we hope), NULL);

// Attempt to resurrect the PM screen

      if (GreResurrection(hDC, 0L, (PULONG)NULL) == 0)
      {
         ErrorCount++;
         ERRSTATUS(TRUE, "GreResurrection()", EntryName);
      }

// Log subexit

      SUBEXIT();

// Exit entry point

      EXIT_MAIN(EntryName);

// No output, so suppress printer output

      DeletePrinterOutput(EntryName);

   }

   return;

}*/ /* End GreDeathAndResurrectionExh  */

/***************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetDCOriginExh
 *
 * Purpose:  This is an entry point for the DTT that exhaustively tests
 *           the GreDeviceSetDCOrigin function call.
 *
 * System Requirements:
 *
 * Revision Log: 12/12/91 - Initial Creation, Stan Logan
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN
 *    EXIT_MAIN
 *    SETENTER
 *    SETEXIT
 *    SUBENTER
 *    SUBEXIT
 *    SHOWERROR
 *    ERRSTATUS
 *    DeletePrinterOutput
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * Log setup
 *    Get the HDC
 * Log setup completion
 * Log subentry
 *    Begin loop to randomly set the DC Origin 100 times
 *       Set the origin to (0<=x<= 32K, 0<=y<=32K)
 *       Call GreDeviceSetDCOrigin
 * Log subexit
 * Log subentry
 *    Set the origin to (LONG_MAX, LONG_MAX)
 *    Call GreDeviceSetDCOrigin
 * Log subexit
 * Log subentry
 *    Set the origin to (LONG_MIN, LONG_MIN)
 *    Call GreDeviceSetDCOrigin
 * Log subexit
 * Log subentry
 *    Set the origin to (0, 0)
 *    Attempt to resset the DC origin
 * Log subexit
 * No output, so suppress printer output
 *
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceSetDCOriginExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static   PCHAR    EntryName = "GreDeviceSetDCOriginExh";
            HWND     hWnd = (HWND)NULL;
            HAB      hAB = (HAB)NULL;
            USHORT   DCLoop;

   if (SelectionCall)
      WinSendMsg (hWndClient, (USHORT)WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);

   else {
      ENTER_MAIN(EntryName);

// Log setup

      SETENTER(" ", "Setting up structures");

// Get the HDC

      if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR)
         SHOWERROR("GpiQueryDevice", EntryName);

// Log setup completion

      SETEXIT();

// Log subentry

      SUBENTER(Setting the DC Origin X and Y to random values 100 times, NULL);

// Begin loop to randomly set the DC Origin 100 times

      for (DCLoop = 0; DCLoop < 100; DCLoop++)
      {

// Set the origin to (0<=x<= 32K, 0<=y<=32K)

         ptl.x = MapX(randL(0, 32768));
         ptl.y = MapY(randL(0, 32768));

// Call GreDeviceSetDCOrigin

         if (!GreDeviceSetDCOrigin(hDC, &ptl))
         {
            ErrorCount++;
            break;
         }
      }

      ERRSTATUS(TRUE, "GreDeviceSetDCOrigin()", EntryName);

// Log subexit

      SUBEXIT();

// Log subentry

      SUBENTER(Setting the DC Origin X and Y to LONG_MAX, NULL);

// Set the origin to (LONG_MAX, LONG_MAX)

      ptl.x = LONG_MAX;
      ptl.y = LONG_MAX;

// Call GreDeviceSetDCOrigin

      if (!GreDeviceSetDCOrigin(hDC, &ptl))
         ErrorCount++;

      ERRSTATUS(TRUE, "GreDeviceSetDCOrigin()", EntryName);

// Log subexit

      SUBEXIT();

// Log subentry

      SUBENTER(Setting the DC Origin X and Y to LONG_MIN, NULL);

// Set the origin to (LONG_MIN, LONG_MIN)

      ptl.x = LONG_MIN;
      ptl.y = LONG_MIN;

// Call GreDeviceSetDCOrigin

      if (!GreDeviceSetDCOrigin(hDC, &ptl))
         ErrorCount++;

      ERRSTATUS(TRUE, "GreDeviceSetDCOrigin()", EntryName);

// Log subexit

      SUBEXIT();

// Log subentry

      SUBENTER(Resetting the DC Origin X and Y to 0, NULL);

// Set the origin to (0, 0)

      ptl.x = ptl.y = 0L;

// Attempt to resset the DC origin

      if (!GreDeviceSetDCOrigin(hDC, &ptl))
         ErrorCount++;

      ERRSTATUS(TRUE, "GreDeviceSetDCOrigin()", EntryName);

// Log subexit

      SUBEXIT();

// Exit entry point

      EXIT_MAIN(EntryName);

// No output, so suppress printer output

      DeletePrinterOutput(EntryName);

   }

   return;

} /* End GreDeviceSetDCOriginExh  */
