/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GREQURY.C")
#pragma subtitle("IBM Confidential")

/*const char header=$header$*/

#define INCL_DEV
#define INCL_WIN      // include all Win definitions
#define INCL_GPI      // include all Gpi definitions
#define INCL_GREALL   // include all Gre definitions
#define INCL_ERRORS   // include all Gre error definitions
#define INCL_DEV

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <os2.h>
#include <pmddi.h>    // gre - device driver includes
#include "DTTDLLDF.h" // test case defines
#include "grequry.h"  /* gre query dll include file */
#include "std_util.h"

/****************************************************************************
 *\\ddd
 * Subtask Name: PDVT
 *
 * Module name: grequry.c
 *
 * Purpose: Gre query dll test functions.
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *
 * SRS References: Printer Driver Verification Tool
 *
 * Contents:
 *   InitTest()
 *   GreQueryApp()
 *   GreQueryDeviceBitmapsRc()
 *   GreQueryHardcopyCapsRc()
 *   GreQueryDeviceCapsRc()
 *
 * Design Overview:
 *   GRE Query functions retrieve device capability information for
 *   the calling routines.  The GreQueryDeviceBitmaps, GreQueryHardcopyCaps,
 *   and GreQueryDeviceCaps functions are tested.  The tests include
 *   a basic functionality test and an exhaustive (test all return codes at
 *   the lowest level) test.
 *
 * Limitations:
 *   None.
 *
 *\\end
****************************************************************************/

/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose: Called by the DTT interface to initialize name and address
 *    arrays for the load of the dll
 *
 * System Requirements:
 *   None
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *
 * Inputs:
 *   sel          // passed through to the InitTestSegVars function
 *   pInitInfo    // name and address array used by DTT interface
 *
 * Outputs:
 *   pInitInfo    // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()    // used by the DTT interface
 *
 * Limitations:
 *   pInitInfo is of size ENTRYPOINTS defined in DTTdlldf.h
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *   for each function {
 *       set function name
 *       set function address
 *   }
 *   call inittestsegvars routine
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY InitTest(PVOID sel, PTESTINITINFO pInitInfo)
{
    pInitInfo->ntests = NTESTS;  /* set number of entry points */

    /* set the group name */
    strcpy(pInitInfo->group_name,pGroupName);

    /* set each function name and address */
    strcpy(pInitInfo->test_names[0],pTestName0);
    pInitInfo->test_addrs[0]=GreQueryApp;
    strcpy(pInitInfo->test_names[1],pTestName1);
    pInitInfo->test_addrs[1]=GreQueryHardcopyCapsRc;
    strcpy(pInitInfo->test_names[2],pTestName2);
    pInitInfo->test_addrs[2]=GreQueryDeviceCapsRc;
    strcpy(pInitInfo->test_names[3],pTestName3);
    pInitInfo->test_addrs[3]=GreQueryDeviceBitmapsRc;

    InitTestSegVars(sel); /* this call used by the DTT interface */
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryApp()
 *
 * Purpose:  This function tests the basic functionality of the following
 *     three gre functions:
 *               - GreQueryDeviceCaps
 *               - GreQueryHardCopyCaps
 *               - GreQueryDeviceBitmaps
 *
 * System Requirements: 6.8.2
 *
 * Revision Log:
 *               10/25/90, pdvt, mark richardson
 *               - initial file
 *
 * Inputs:
 *    hPS              // presentation space
 *    hWndClient       // client window
 *    SelectionCall    // selection parameter
 *
 * Outputs:
 *    None.
 *
 * Subroutines Required:
 *    LOG_SIGNATURE
 *    CHECKRETURNERROR
 *    CWRITELOG
 *
 * Limitations:
 *    None.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *   call each query routine
 *   expecting ok return
 *   if return is ok
 *      assume functionality is ok
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryApp(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  PLONG        out_data[DATA_SIZE]; /* bit map information array */
                     //tss i was here
  LONG         out_data_count=1;      /* number of elements in bit map info array */
  DEVOPENSTRUC dop;                 /* device open info structure */
  DRIVDATA     driv;                /* structure for dop and DevOpenDC */
  LONG         lDCType;             /* used by WinSendMsg */
  BOOL         b_rc;                /* boolean return code */
  SHORT        rc;                  /* return code */
  HDC          printer_hdc;         /* return from DevOpenDC */
  PHCINFO      info_p;              /* ptr to buffer of returned data */
  LONG         forms_count;         /* number of available or required forms */
  LONG         start_index;         /* structure array index */
  HAB          hAB;                 /* anchor block handle */
  LONG          error_count;         /* number of api failures */
  CHAR         out_string[80];     /* error count message */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        error_count = 0L;

        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreQueryApp - Entry.\n");

        hAB = WinQueryAnchorBlock(hWndClient);
        dop.pdriv = &driv;
        WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
        printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
        if (printer_hdc == DEV_ERROR) {
           BADERROR("DevOpenDC",TRUE,"GreQueryApp");
           return;
        }
       /*
        * Query the device bitmaps, if successful, the function should
        * work ok in a real application
        * same for hardcopy caps and device caps
        */
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryApp-DeviceBitmaps - Entry.\n");
        out_data_count = DATA_SIZE;
        b_rc = GreQueryDeviceBitmaps(printer_hdc,out_data,out_data_count);
        if (b_rc == FALSE) {
           error_count += BADERROR("GreQueryDeviceBitmaps",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryAppDeviceBitmaps - Exit.\n");

        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryApp-DeviceCaps Entry.\n");
        start_index = 0;   /* begin returning information at this element */
        b_rc = GreQueryDeviceCaps(printer_hdc,start_index,out_data,out_data_count);
        if (b_rc == FALSE) {
           error_count += BADERROR("GreQueryDeviceBitmaps",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryApp-DeviceCaps Exit.\n");

        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryApp-HardcopyCaps Entry.\n");
        start_index = 0;        /* structure for first form */
        forms_count = 0;        /* return number of forms supported */
        rc = GreQueryHardcopyCaps(printer_hdc,start_index,forms_count,info_p);
        if (rc == 0L) {
           error_count += BADERROR("GreQueryDeviceBitmaps",FALSE,NULL);
        }
        CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryApp-HardcopyCaps Exit.\n");

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryApp                                                    -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryApp                                                    -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryApp - Exit.\n");
       }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryDeviceBitmapsRc()
 *
 * Purpose:  This function exhaustively tests for all error codes
 *    returned by GreQueryDeviceBitmaps().  The syntax used to call the
 *    query routine is:
 *    (BOOL) GreQueryDeviceBitmaps(
 *           HDC      hdc,
 *           PLONG    paOutData,
 *           LONG     cOutData,
 *           ULONG    pInstance,
 *           ULONG    lFunction)
 *
 *    GreQueryDeviceBitmaps stores a list of bit-map formats supported by
 *    the device in the array addressed by paOutData.  Error codes for
 *    conditions that GreQueryDeviceBitmapsRc is expected to check are:
 *
 *    PMERR_DEV_FUNC_NOT_INSTALLED
 *    PMERR_INV_LENGTH_OR_COUNT.
 *
 *    This function also checks for valid conditions.
 *
 * System Requirements: 6.8.3
 *
 * Revision Log:
 *               10/25/90, pdvt, mark richardson
 *               - initial file
 *
 * Inputs:
 *    hPS              // presentation space
 *    hWndClient     // client window
 *    SelectionCall   // selection parameter
 *
 * Outputs:
 *    None.
 *
 * Subroutines Required:
 *    CWRITELOG
 *    CHECKRETURNERROR
 *    LOG_SIGNATURE
 *
 * Limitations:
 *   GreQueryDeviceBitmaps gets information about supported devices.  The
 *   function is tested with a valid device for functionality.  Error conditions
 *   are tested by passing the gre funtion invalid device descriptors.
 *
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *    // test one
 *    // setup conditions to test gre works as expected
 *    // get printer device con+ex+ information for the DevOpenDC call
 *    // call gre with all valid parameters expecting good return
 *    rc = GreQueryDeviceBitmaps(hdc...);
 *    if (rc == true) {
 *       log expected successful condition
 *    {
 *    else {
 *       log unexpected error condition
 *    }
 *
 *   // test two
 *   // setup conditions to test PMERR_DEV_FUNC_NOT_INSTALLED error code
 *   // call the gre with the     device con+ex+ handle expecting
 *   // error return
 *   rc = GreQueryDeviceBitmaps(hdc...)
 *   if (rc == false) {
 *      post error condition
 *      get error code
 *      if (error == PMERR_DEV_FUNC_NOT_INSTALLED) {
 *         log expected error condition
 *      }
 *      else {
 *         log unexpected error condition
 *      }
 *   }
 *   else {
 *      log unexpected successful condition
 *   }
 *
 *   // test three
 *   // setup conditions to test gre works as expected
 *   // get number of forms available by calling gre with the
 *   // count variable set to null
 *   rc = GreQueryDeviceBitmaps(hdc...cCount=NULL);
 *
 *   // call gre with invalid number of forms required by passing the
 *   // count variable set to more than there are forms available
 *   rc = GreQueryDeviceBitmaps(hdc...cCount=(number returned + 1);
 *   if (rc == false) {
 *      post error condition
 *      get error code
 *      if (error == PMERR_INV_LENGTH_OR_COUNT) {
 *         log expected error condition
 *      }
 *      else {
 *         log unexpected error condition
 *      }
 *   }
 *   else {
 *      log unexpected successful condition
 *   }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryDeviceBitmapsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC          printer_hdc;        /* default printer device con+ex+ handle */
   PLONG        out_data[DATA_SIZE];/* bit map information array */
   LONG         out_data_count;     /* number of elements in bit map info array */
   BOOL         b_rc;               /* boolean return code from query function */
   DEVOPENSTRUC dop;                /* device open info structure */
   DRIVDATA     driv;               /* structure for dop and DevOpenDC */
   LONG         lDCType;            /* used by WinSendMsg */
   HAB          hAB;                /* anchor block handle */
   LONG         error_count;         /* number of api failures */
   CHAR         out_string[80];     /* error count message */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreQueryDeviceBitmapsRc - Entry.\n");

        /************************************************************************
         * test one
         ************************************************************************/

         hAB = WinQueryAnchorBlock(hWndClient);
         dop.pdriv = &driv;
         WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
         printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
         if (printer_hdc == DEV_ERROR) {
            BADERROR("GreQueryDeviceBitmaps",TRUE,"GreQueryDeviceBitmapsRc");
            return;
         }

        /*
         * fill in parameters and call query device bitmaps
         */
         CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryDeviceBitmapsRc - Entry.\n");
         out_data_count = DATA_SIZE;
         b_rc = GreQueryDeviceBitmaps(printer_hdc,out_data,out_data_count);
         if (b_rc == FALSE) {
            error_count += BADERROR("GreQueryDeviceBitmaps",FALSE,NULL);
         }
         CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryDeviceBitmapsRc - Exit.\n");

         CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryDeviceBitmapsRc - Entry.\n");
         b_rc = GreQueryDeviceBitmaps(printer_hdc,out_data,out_data_count);
         if (b_rc == FALSE)
            error_count += CHECKRETURNERROR(b_rc,"PMERR_DEV_FUNC_NOT_INSTALLED", SEVERITY_ERROR,PMERR_DEV_FUNC_NOT_INSTALLED);
         CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryDeviceBitmapsRc - Exit.\n");

         CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryDeviceBitmapsRc - Entry.\n");
         out_data_count = 2L; /* 0 count will return number of supported forms */
         b_rc = GreQueryDeviceBitmaps(printer_hdc,out_data,out_data_count);
         if (b_rc == FALSE) {
            error_count += BADERROR("GreQueryDeviceBitmaps",FALSE,NULL);
         }
         else {                  //tss was out_data_count += 1;\/
               out_data_count = -1; /* now query more forms than are available */
               b_rc = GreQueryDeviceBitmaps(printer_hdc,out_data,out_data_count);
               if (b_rc == FALSE)
                  error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_LENGTH_OR_COUNT", SEVERITY_ERROR,PMERR_INV_LENGTH_OR_COUNT);
         }
         CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryDeviceBitmapsRc - Exit.\n");

         CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryDeviceBitmapsRc - Entry.\n");
         b_rc = GreQueryDeviceBitmaps((HDC)0,out_data,out_data_count);
         if (b_rc == FALSE)
            error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_HDC", SEVERITY_ERROR,PMERR_INV_HDC);
         CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryDeviceBitmapsRc - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryDeviceBitmaps                                          -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryDeviceBitmaps                                          -    PASS.\n");
        CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryDeviceBitmapsRc - Exit.\n");
       }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryHardcopyCapsRc()
 *
 * Purpose: This function exhaustively tests for all error codes
 *    returned by GreQueryHardcopyCaps().  The syntax used to call the
 *    query routine is:
 *
 *    (LONG) GreQueryHardcopyCaps(
 *           HDC     hdc,
 *           LONG    iStart,
 *           LONG    cCount,
 *           PHCINFO pInfo,
 *           ULONG   pInstance,
 *           ULONG   lFunction)
 *
 *    GreQueryHardcopyCaps stores information about the hard-copy capabilities
 *    of the device in the buffer addressed by pInfo.  The information is
 *    stored as a sequence of one or more HCINFO structures defining the
 *    hard-copy capabilities for one or more form codes.  Error codes for
 *    conditions that GreQueryHardcopyCpsRc is expected to check are:
 *
 *    PMERR_INV_LENGTH_OR_COUNT.
 *
 *    This functions also tests valid conditions.
 *
 * System Requirements: 6.8.3
 *
 * Revision Log:
 *               10/25/90, pdvt, mark richardson
 *               - initial file
 *
 * Inputs:
 *   hPS               // presentation space
 *   hWndClient      // client window
 *   hAB               // anchor block
 *   SelectionCall    // selection parameter
 *
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *   CWRITELOG
 *   LOG_SIGNATURE
 *   CHECKRETURNERROR
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
 /**************************************************************************
 *\\algorithm
 * {
 *   // get number of forms available by passing count varaible
 *   // set to null
 *   rc = GreQueryHardcopyCaps(hdc...cCount=NULL);
 *
 *   // call gre with invalid number of forms required by passing
 *   // count variable set to more forms than available
 *   rc = GreQueryHardcopyCaps(hdc...cCount=(number returned + 1);
 *   if (rc == false) {
 *      post error condition
 *      get error code
 *      if (error == PMERR_INV_LENGTH_OR_COUNT) {
 *         log expected error condition
 *      }
 *      else {
 *         log unexpected error condition
 *      }
 *   }
 *   else {
 *      log unexpected successful condition
 *   }
 *    // setup conditions to test gre works as expected
 *    hdc = DevOpenDC(hab...);
 *
 *    // call gre with good parameters expecting goo return
 *    rc = GreQueryHardcopyCaps(hdc...);
 *    if (rc == true) {
 *       log expected successful condition
 *    {
 *    else {
 *       log unexpected error condition
 *    }
 *    DevCloseDC(hdc);
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryHardcopyCapsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC          printer_hdc;  /* invalid device con+ex+ handle */
   LONG         start_index;  /* structure array index */
   LONG         forms_count;  /* number of available or required forms */
   PHCINFO      info_p;       /* ptr to buffer of returned data */
   LONG         rc;           /* return code from query function */
   DEVOPENSTRUC dop;          /* device open info structure */
   DRIVDATA     driv;         /* structure for dop and DevOpenDC */
   LONG         lDCType;      /* used by WinSendMsg */
   HAB          hAB;          /* anchor block handle */
   LONG         error_count;         /* number of api failures */
   CHAR         out_string[80];     /* error count message */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreQueryHardcopyCapsRc - Entry.\n");

        /***********************************************************************
         * test one
         ***********************************************************************/

         hAB = WinQueryAnchorBlock(hWndClient);
         dop.pdriv = &driv;
         WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
         printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
         if (printer_hdc == DEV_ERROR) {
            BADERROR("DevOpenDC",TRUE,"GreQueryHardcopyCapsRc");
            return;
         }

        /*
         * call the hardcopy caps query function expecting good return
         */
         CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryHardcopyCapsRc - Entry.\n");
         start_index = 0;        /* structure for first form */
         forms_count = 0;        /* return number of forms supported */
         rc = GreQueryHardcopyCaps(printer_hdc,start_index,forms_count,info_p);
         if (rc == 0L) {
            error_count += BADERROR("GreQueryHardCopyCapsRc",FALSE,NULL);
         }
         CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryHardcopyCapsRc - Exit.\n");


         CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryHardcopyCapsRc - Entry.\n");
         start_index = 0L;        /* structure for first form */
         forms_count = -1L;
         rc = GreQueryHardcopyCaps(printer_hdc,start_index,forms_count,info_p);
         if (rc == 0L)
            error_count += CHECKRETURNERROR(rc,"PMERR_INV_LENGTH_OR_COUNT",SEVERITY_ERROR, PMERR_INV_LENGTH_OR_COUNT);
         CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryHardcopyCapsRc - Exit.\n");

         CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryHardcopyCapsRc - Entry.\n");
         rc = GreQueryHardcopyCaps((HDC)0,start_index,&forms_count,info_p);
         if (rc == 0L) {
            error_count += CHECKRETURNERROR(rc,"PMERR_INV_HDC",SEVERITY_ERROR, PMERR_INV_HDC);
         }
         CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryHardcopyCapsRc - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryHardcopyCaps                                           -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryHardcopyCaps                                           -    PASS.\n");
         CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryHardcopyCapsRc - Exit.\n");
        }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryDeviceCapsRc()
 *
 * Purpose:  This function exhaustively tests for all error codes
 *    returned by GreQueryDeviceCaps().  The syntax used to call the query
 *    routine is:
 *
 *    (BOOL) GreQueryDeviceCaps(
 *           HDC      hdc,
 *           LONG     iIndex,
 *           PLONG    paOutData,
 *           LONG     cOutData,
 *           ULONG    pInstance,
 *           ULONG    lFunction)
 *
 *    GreQueryDeviceCaps returns device capability information in the array
 *    paOutData beginning at the index indicated by iIndex.  Error codes for
 *    conditions that GreQueryDeviceCapsRc is expected to check are:
 *
 *    PMERR_DEV_FUNC_NOT_INSTALLED
 *    PMERR_INF_QUERY_ELEMENT_NO
 *    PMERR_INV_LENGTH_OR_COUNT.
 *
 *    This function also checks for valid conditions.
 *
 *
 * System Requirements: 6.8.3
 *
 * Revision Log:
 *               10/25/90, pdvt, mark richardson
 *               - initial file
 *
 * Inputs:
 *   hPS               // presentation space
 *   hWndClient      // client window
 *   SelectionCall    // selection parameter
 *
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *   LOG_SIGNATURE
 *   CWRITELOG
 *   CHECKRETURNERROR
 *
 * Limitations:
 *   None.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *    // setup conditions to test gre works as expected
 *    hdc = DevOpenDC(hab...);
 *
 *    // call gre with good parameters expecting good return
 *    rc = GreQueryDeviceCaps(hdc...);
 *    if (rc == true) {
 *       log expected successful condition
 *    {
 *    else {
 *       log unexpected error condition
 *    }
 *    DevCloseDC(hdc);
 *   \\ setup conditions to test PMERR_DEV_FUNC_NOT_INSTALLED error code
 *   hab = meaningless value  \\ get     hab
 *   // call DevOpenDC with invalid anchor block expecting invalid
 *   // device con+ex+ handle returned
 *   hdc = DevOpenDC(hab...)
 *
 *   // call gre with invalid device con+ex+ handle expecting error return
 *   rc = GreQueryDeviceCaps(hdc...)
 *   if (rc == false) {
 *      post error condition
 *      get error code
 *      if (error == PMERR_DEV_FUNC_NOT_INSTALLED) {
 *         log expected error condition
 *      }
 *      else {
 *         log unexpected error condition
 *      }
 *   }
 *   else {
 *      log unexpected successful condition
 *   }
 *   // setup conditions to test gre works as expected
 *   hdc = DevOpenDC(hab...);
 *
 *   // call gre with invalid element number expecting error return
 *   rc = GreQueryDeviceCaps(hdc...iIndex=-1);
 *   if (rc == false) {
 *      post error condition
 *      get error code
 *      if (error == PMERR_QUERY_ELEMENT_NO) {
 *         log expected error condition
 *      }
 *      else {
 *         log unexpected error condition
 *      }
 *   }
 *   else {
 *      log unexpected successful condition
 *   }
 *   // setup conditions to test gre works as expected
 *   hdc = DevOpenDC(hab...);
 *
 *   // get number of forms available by calling gre with count
 *   // parameter set to null
 *   rc = GreQueryDeviceCaps(hdc...cCount=NULL);
 *
 *   // call gre with invalid number of forms required by passing
 *   // count variable set to more forms than are available
 *   rc = GreQueryDeviceCaps(hdc...cCount=(number returned + 1));
 *   if (rc == false) {
 *      post error condition
 *      get error code
 *      if (error == PMERR_INV_LENGTH_OR_COUNT) {
 *         log expected error condition
 *      }
 *      else {
 *         log unexpected error condition
 *      }
 *   }
 *   else {
 *      log unexpected successful condition
 *   }
 * }
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryDeviceCapsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC          printer_hdc;        /* printer device con+ex+ handle */
   BOOL         b_rc;               /* boolean return code for query function */
   LONG         start_index;        /* start element of bit map info array */
   PLONG        out_data[DATA_SIZE];/* bit map information array */
   LONG         out_data_count;     /* number of elements in bit map info array */
   DEVOPENSTRUC dop;                /* device open info structure */
   DRIVDATA     driv;               /* structure for dop and DevOpenDC */
   LONG         lDCType;            /* used by WinSendMsg */
   HAB          hAB;                /* anchor block handle */
   LONG         error_count;        /* number of api failures */
   CHAR         out_string[80];     /* error count message */
   INT          i;                  /* for loop counter */

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
        *pBasisX = 10000;
        *pBasisY = 10000;
  }
  else {
        error_count = 0L;
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreQueryDeviceCapsRc - Entry.\n");

        /************************************************************************
         * test one
         ************************************************************************/
         hAB = WinQueryAnchorBlock(hWndClient);
         dop.pdriv = &driv;
         WinSendMsg(hWndClient,WM_GET_DEVOPENSTRUC,MPFROMP(&dop),MPFROMP(&lDCType));
         printer_hdc = DevOpenDC(hAB,OD_QUEUED,"*",6L,(PDEVOPENDATA)&dop,NULL);
         if (printer_hdc == DEV_ERROR) {
            BADERROR("DevOpenDC",TRUE,"GreQueryDeviceCapsRc");
            return;
         }

        /*
         * call the query function for the printer dc expecting good return
         */
         CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryDeviceCapsRc - Entry.\n");
            //tss...temp to 1\/
         start_index = 1;
         out_data_count = 1;
         for (i = 0; i < DATA_SIZE; i++)
             out_data[i] = malloc(sizeof(LONG));
         b_rc = GreQueryDeviceCaps(printer_hdc,start_index,out_data,out_data_count);
         if (b_rc == FALSE)  {
            error_count += BADERROR("GreQueryDeviceCaps",FALSE,NULL);
         }
         CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryDeviceCapsRc - Exit.\n");

         CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryDeviceCapsRc - Entry.\n");
         b_rc = GreQueryDeviceCaps((HDC)0,start_index,out_data,out_data_count);
         if (b_rc == FALSE) {
            error_count += CHECKRETURNERROR(b_rc,"PMERR_INV_HDC",SEVERITY_ERROR,PMERR_INV_HDC);
         }
         CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryDeviceCapsRc - Exit.\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (error_count) {
           sprintf(out_string,"$Errorcount == %ld.\n",error_count);
           CWRITELOG(L_HDR,0,out_string);
           CWRITELOG(L_HDR,0,
"$GreQueryDeviceCaps                                             -    FAIL.\n");
        }
        else
           CWRITELOG(L_HDR,0,
"$GreQueryDeviceCaps                                             -    PASS.\n");
         CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryDeviceCapsRc - Exit.\n");
        }
}
