/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("GRETXTST.C")
#pragma subtitle("IBM Confidential")

/* const char header=$header$ */

#define INCL_DEV      // include all DEV definitions
#define INCL_WIN
#define INCL_GPI      // include all Gpi definitions
#define INCL_GREALL   // include all Gre definitions
#define INCL_ERRORS   // include all Gre error definitions
#define INCL_DDIMISC
#define INCL_DDIMISC2
#define INCL_FONTFILEFORMAT
#define  INCL_DDIBUNDLES
#include <string.h>
#include <stdlib.h>
#include <os2.h>
#include <stdio.h>
#include <math.h>
#include <pmddi.h>                  // gre - device driver includes
#include <malloc.h>
#include "DTTDLLDF.h"               // test case defines
#include "gretxtst.h"               // gre text dll include file
#include "std_util.h"

/*****************************************************************************
 *\\ddd
 * Subtask Name: PDVT
 *
 * Module Name: GRETXTST
 *
 * Purpose: This module is called by the DTT to test the Gre Text API's.
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial module template
 *               -12/13/90 ,PDVT ,Mike Abuzant
 *               -Test Case Design.
 *
 *               -12/17/90 ,PDVT ,Mike Abuzant
 *               -Deleted gre_text_pre(), gre_text_all()
 *               -Deleted 'hAB' parameters
 *               -Added 'Rc' function for every Gre function
 *               - 12/18/90 ,PDVT ,Mike Abuzant
 *               -TestCase Design
 *
 *
 * SRS References: Section 6.4
 *
 * Contents:
 *   InitTest()
 *   GreTextApp1()
 *   GreTextApp2()
 *   GreCharStringExh()
 *   GreCharStringPosExh()
 *   GreQueryTextBoxExh()
 *   GreQueryWidthTableExh()
 *   GreRealizeFontExh()
 *   GreDeviceQueryFontsExh()
 *   GreDeviceQueryFontAttributesExh()
 *   GreQueryCharPositionsExh()
 *   GreGetCodePageExh()
 *   GreSetCodePageExh()
 *   GreGetPairKerningTableExh()
 *
 *   GreCharStringRc()
 *   GreCharStringPosRc()
 *   GreQueryTextBoxRc()
 *   GreQueryWidthTableRc()
 *   GreRealizeFontRc()
 *   GreDeviceQueryFontsRc()
 *   GreDeviceQueryFontAttributesRc()
 *   GreQueryCharPositionsRc()
 *   GreSetCodePageRc()
 *   GreGetPairKerningTableRc()
 *
 * Design Overview:
 *    This testCase is designed to test some of the Gre text functions
 * That:
 *    1.Display character strings at current (x,y).
 *    2.Display display character strings at a given (x,y) coordinates.
 *    3.Query the fonts that are supported by the current display device.
 *    4.Query the Metrix of current font used with  current display device.
 *    5.Query the Kerning Pairs Information for the currently selected font.
 *    6.Query the character width table for the currently selected font.
 *    7.Setting and Querying the Code Page for the currently selected font.
 *    8.Load,realize,and delete fonts.
 *    9.Query a Box dimensions that inscribes a given character string.
 *
 *    This testCase includes application tests that are designed to test the
 *  behavier of these text function with other Gpi's and Gre's function.
 *
 *     Also, Testing includes an exhaustive and  return error codes
 *  functions for each and every Gre functions in this testCase.
 *
 * Limitations:
 *
 *    The Gre function GreRealizeFonts() Could not be tested since one
 *  of its parameters is FLAGS that their definitions can not be found
 *  any where
 *
 *\\end
 *****************************************************************************/
/*****************************************************************************
 *\ddd
 * Routine Name: InitTest
 *
 * Purpose: Initialize function name and address array for DTT.
 *
 * System Requirements: NONE
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *               -12/14/90 ,PDVT ,Mike Abuzant
 *               -Test Case Design.
 *
 * Inputs:
 *   sel           //passed through to the InitTestSegVars function
 *   init_info_p   // name and address array used by DTT interface
 *
 * Outputs:
 *   init_info_p   // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()    // used by the DTT interface
 *
 * Limitations:
 *   init_info_p is of size ENTRYPOINTS defined in DTT.h and DTTdlldf.h(both)
 *
 *\\end
 *****************************************************************************/
/*****************************************************************************
 *\\ algorithm
 *    for all test functions in this test case {
 *        init_info_p->function name = function name[i]
 *        init_info_p->function address = function address[i]
 *    set function address
 *    }
 *    call inittestsegvars routine
 *\\end
 *****************************************************************************/

VOID  APIENTRY InitTest(PVOID sel, PTESTINITINFO init_info_p)
  {
    init_info_p->ntests = NTESTS;  /* set number of entry points */
    strcpy(init_info_p->group_name,TestCaseName); //Testcase Name

    /*  Load test names and entry point addresses  */
    strcpy(init_info_p->test_names[0],testname1);
    init_info_p->test_addrs[0]=GreTextApp1;
    strcpy(init_info_p->test_names[1],testname2);
    init_info_p->test_addrs[1]=GreTextApp2;

    strcpy(init_info_p->test_names[2],testname13);
    init_info_p->test_addrs[2]=GreCharStringExh;
    strcpy(init_info_p->test_names[3],testname14);
    init_info_p->test_addrs[3]=GreCharStringPosExh;
    strcpy(init_info_p->test_names[4],testname15);
    init_info_p->test_addrs[4]=GreQueryTextBoxExh;
    strcpy(init_info_p->test_names[5],testname16);
    init_info_p->test_addrs[5]=GreQueryWidthTableExh;
    strcpy(init_info_p->test_names[6],testname18);
    init_info_p->test_addrs[6]=GreDeviceQueryFontsExh;
    strcpy(init_info_p->test_names[7],testname19);
    init_info_p->test_addrs[7]=GreDeviceQueryFontAttributesExh;
    strcpy(init_info_p->test_names[8],testname20);
    init_info_p->test_addrs[8]=GreQueryCharPositionsExh;
    strcpy(init_info_p->test_names[9],testname21);
    init_info_p->test_addrs[9]=GreGetCodePageExh;
    strcpy(init_info_p->test_names[10],testname22);
    init_info_p->test_addrs[10]=GreSetCodePageExh;
    strcpy(init_info_p->test_names[11],testname23);
    init_info_p->test_addrs[11]=GreGetPairKerningTableExh;
    strcpy(init_info_p->test_names[12],testname3);

    init_info_p->test_addrs[12]=GreCharStringRc;
    strcpy(init_info_p->test_names[13],testname4);
    init_info_p->test_addrs[13]=GreCharStringPosRc;
    strcpy(init_info_p->test_names[14],testname5);
    init_info_p->test_addrs[14]=GreQueryTextBoxRc;
    strcpy(init_info_p->test_names[15],testname6);
    init_info_p->test_addrs[15]=GreQueryWidthTableRc;
    strcpy(init_info_p->test_names[16],testname7);
    init_info_p->test_addrs[16]=GreRealizeFontRc;
    strcpy(init_info_p->test_names[17],testname8);
    init_info_p->test_addrs[17]=GreDeviceQueryFontsRc;
    strcpy(init_info_p->test_names[18],testname9);
    init_info_p->test_addrs[18]=GreDeviceQueryFontAttributesRc;
    strcpy(init_info_p->test_names[19],testname10);
    init_info_p->test_addrs[19]=GreQueryCharPositionsRc;
    strcpy(init_info_p->test_names[20],testname11);
    init_info_p->test_addrs[20]=GreSetCodePageRc;
    strcpy(init_info_p->test_names[21],testname12);
    init_info_p->test_addrs[21]=GreGetPairKerningTableRc;

    InitTestSegVars(sel);  /* this call used by the DTT interface */
 }
/***************************************************************************
 *\\ddd
 * Routine Name: GreTextApp1
 *
 * Purpose:  This is an entry point for the DTT that causes the Gre Text
 *           application test to be run.
 *
 * System Requirements: Section 6.4.2
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *               01/03/91, PDVT, Mike Abuzant
 *               - TestCase Design and Implemetation.
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta(hPS)
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *
 *   Change the character mode from default mode to CM_MODE3 since
 *   we need to change the character box size which in turn makes
 *   the characters larger. This can not be acomplished in either
 *   CM_MODE2 or CM_MODE1. Notice that the font chosen by the PM
 *   is an outlined (vector) font.
 *      GpiSetCharMode(hPS,CM_MODE3)..
 *    Get the default character box size.
 *      GpiQueryCharBox(hPS,&size)
 *    Increase the box default size by a factor and set as the default
 *      GpiSetCharBox(hPS,&size)
 *    //Change The character angle so the text is drawn from
 *    //Bottom of device to the top of it
 *        GpiSetCharAngle(hPS,&NewAngle);
 *    Setup Conditions to display a set character strings that will
 *    the entire width of the device. Each time through the process, only
 *    a portion of a string is displayed depending of some factors. Each
 *    string is inscribed by a box. The box dimensions are obtained by
 *    a call to GreQueryTextBox().
 *
 *    FOR (all string to display) DO
 *       //Query character string positions and
 *       //store them in the Array address by 'aptl'.
 *           GreQueryCharPositions(hDC,aptl....)
 *        //Query box coordinates that inscribes the character string
 *        //and store them in the Array address by 'aptl'.
 *           GreQueryTextBox(hDC,aptl....)
 *       Setup condition to draw the bounding parallelogram around
 *       the character string using the GpiPolyLine(). This will verify
 *       that the coordinates returned by GreQueryTextBox() is indeed
 *       correct.
 *       // Move to one of the bounding parallelogram corners
 *          GpiMove(hPS,&FirstPointOfBox)
 *       // Draw the rest of the bounding parallelogram.
 *          GpiPolyLine(hPS,4L,aptl+1).
 *       // Move to (x,y) coordinates  and display the displayable portion
 *       // of the string.
 *           GpiMove(hPS,&FirstPointOfBox)
 *           GreCharString(hDC,TheCharacterString...)
 *       //Adjust factors that determines which portion of string is
 *       // displayed.
 *       IF  (Only one character is displayed) THEN
 *            Increase number of displayable characters by a factor of two
 *       ELSE (IF the entire string is displayed) THEN
 *            Decrease number of displayable characters by a factor of two
 *   ENDDO
 *\\end
 ***************************************************************************/
VOID APIENTRY GreTextApp1(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL      ptl;
  LONG        SetupCount=0L;
  LONG        lIndex1,
              lIndex2;
  HDC         hDC;
  SIZEF       sizel;
  static      GRADIENTL  CharAngle={0,0};
  static      GRADIENTL  NewAngle ={0,45};
  POINTL      aBox[TXTBOX_COUNT];
  FONTMETRICS fm;
  LONG        lMiddle,
              lPosition,
              lStringLength,
              Incroment;
  POINTL      aptl[100];

  USHORT ProgressLimit;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
       LOG_SIGNATURE;
       TEST_SIGNATURE(0,0);
       CWRITELOG(L_LOTRACE,0,"+ee+ () GreTextApp1() Application Test - Entry Point\n");

       if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)   {
          BADERROR("GpiQueryDevice()",TRUE,"GreTextApp1");
          return;
       }

       set_delta(hPS);


       if (GpiSetCharMode(hPS,CM_MODE3)==FALSE) {
       }
       if (GpiQueryCharBox(hPS, &sizel) == FALSE) {
       }
       sizel.cx=(sizel.cx*6L)/4L;
       sizel.cy=(sizel.cy*6L)/4L;


       if (GpiSetCharBox(hPS, &sizel) == FALSE) {
       }


       if (GpiSetCharAngle(hPS,&NewAngle)==FALSE) {
       }

       if (GreQueryFontAttributes(hDC,(LONG)sizeof(fm),&fm )==FALSE) {
          BADERROR("GreQueryFontAttributes()",TRUE,"GreTextApp1");
          return;
       }
       strcpy(ErrorString,"GreCharString() and Other Text Functions.");
       lMiddle=(LONG)(strlen(ErrorString)/2L);
       Incroment=1L;
       lPosition=0L;
       ptl.x=(LONG)(delta_x/2L);
       ptl.y=delta_y;
       MapPt(ptl);

       lStringLength=(LONG)strlen(ErrorString);

       ProgressLimit = ( USHORT ) ( pg_size_x * delta_x );

       TEST_PROGRESS ( 00, ProgressLimit );

       for (lIndex1=0; UnMapX(ptl.x)<(pg_size_x*delta_x) ;lIndex1++){
           if(GreQueryCharPositions(hDC,&ptl,CHS_START_XY,
                                   (LONG)strlen(ErrorString),ErrorString,
                                    NULL,aptl)==FALSE) {
              BADERROR("GreQueryCharPositions()",TRUE,"GreTextApp1");
              return;
           }
           if(GreQueryTextBox(hDC,lStringLength,
                              ErrorString+lPosition,
                              TXTBOX_COUNT,aBox)==FALSE) {
              BADERROR("GreQueryCharPositions()",TRUE,"GreTextApp1");
              return;
           }
           SWAPCOORDINATE(aBox);
           for (lIndex2=0 ;lIndex2 < TXTBOX_COUNT ; lIndex2++) {
                aBox[lIndex2].x+=aptl[lPosition].x;
                aBox[lIndex2].y+=aptl[lPosition].y;
           }
           if (GpiMove(hPS,aBox)==GPI_ERROR) {
           }
           if (GpiPolyLine(hPS,4L,aBox+1)==GPI_ERROR) {
           }
           if (GpiMove(hPS,&aptl[lPosition])==GPI_ERROR) {
           }
           if(GreCharString(hDC,lStringLength,
                            ErrorString+lPosition)==GPI_ERROR) {
              BADERROR("GreCharSTring()",TRUE,"GreTextApp1");
              return;
           }

           TEST_PROGRESS ( ( USHORT ) UnMapX ( ptl . x ), ProgressLimit );

           if (lPosition==(lMiddle-1L))
              Incroment=-1L;
           else if (lPosition==0L)
                   Incroment=1L;
           lPosition+=Incroment;
           lStringLength-=(Incroment*2L);
           ptl.x += (LONG)abs((INT)(aBox[TXTBOX_TOPLEFT].x-aBox[TXTBOX_TOPRIGHT].x))+2L;
    }

    TEST_PROGRESS ( ProgressLimit, ProgressLimit );


    /********************************************************************
    /*       Return the proper log messages for Non-Test Gre's.         *
    /********************************************************************/
    if (!SetupCount)
       CWRITELOG(L_HDR,0,
"$GreTextApp1()                                                  -    PASS.\n");
    else{
        sprintf(ErrorString,"$ErrorCount = %ld.\n", SetupCount);
        CWRITELOG(L_HDR,0, ErrorString);
        CWRITELOG(L_HDR,0,
"$GreTextApp1()                                                  -    FAIL.\n");
    }
    CWRITELOG(L_LOTRACE,0,"+ex+ GreTextApp1() Application Test - Exit Point\n");
   }
 }
/***************************************************************************
 *\\ddd
 * Routine Name: GreTextApp2
 *
 * Purpose:  This is an entry point for the DTT that causes the Gre Text
 *           application test to be run.
 *
 * System Requirements: Section 6.4.2
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *               01/04/91, PDVT, Mike Abuzant
 *               - TestCase Design and Implemetation.
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta(hPS)
 *
 * Limitations:
 *
 *  The printer device driver does not work as expected  when the option
 *  'CHS_ATTR_INFO' is set and the selected font is an image font.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *
 *
 *    // Call \EzQueryFonts to retreive all device fonts supported
 *    // by the device. If no fonts are supported by the device, the
 *    // function will retrieve the fonts supported by the Graphic
 *    // Engine.
 *       fSuccess=EzQueryFonts(hDC,....)
 *    //Create a logical font.
 *       fSuccess=EzCreateLogicalFont(hPS,szFontName,FontAttributes...)
 *    // Make Logical font the current font.
 *        GpiSetCharSet(hPS,USER_FONT_LCID)
 *    Setup Conditions to display a set character strings that will
 *    the entire width of the device. Each time through the process, only
 *    a portion of a string is displayed depending of some factors. Every
 *    fourth string is inscribed by a box. The box dimensions are
 *    obtained by  a call to GreQueryTextBox().
 *    FOR (all string to display) DO
 *       //Query character string positions and
 *       //store them in the Array address by 'aptl'.
 *           GreQueryCharPositions(hDC,aptl....)
 *        //Query box coordinates that inscribes the character string
 *        //and store them in the Array address by 'aptl'.
 *           GreQueryTextBox(hDC,aptl....)
 *       // Move to (x,y) coordinates  and display the displayable portion
 *       // of the string.
 *           GpiMove(hPS,&FirstPointOfBox)
 *           GreCharStringPos(hDC,TheCharacterString...)
 *       For Every Fourth string, setup condition to draw the bounding
 *       parallelogram around the character string using the GpiPolyLine().
 *       This will verify that the coordinates returned by GreQueryTextBox()
 *       is indeed correct.
 *       // Move to one of the bounding parallelogram corners
 *          GpiMove(hPS,&FirstPointOfBox)
 *       // Draw the rest of the bounding parallelogram.
 *          GpiPolyLine(hPS,4L,aptl+1).
 *       //Adjust factors that determines which portion of string is
 *       // displayed.
 *       IF  (Only two characters are displayed) THEN
 *            Increase number of displayable characters by a factor of four
 *       ELSE (IF the entire string is displayed) THEN
 *            Decrease number of displayable characters by a factor of four
 *   ENDDO
 *       // Dispose of the current Logical font
 *          GpiSetCharSet(hPS,LCID_DEFALUT)
 *          GpiDeleteSetId(hPS,USER_FONT_LCID)
 *\\end
 ***************************************************************************/
VOID APIENTRY GreTextApp2(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL           ptl;
  LONG             SetupCount=0L;
  LONG             lIndex1,
                   lIndex2;
  HDC              hDC;
  POINTL           aBox[TXTBOX_COUNT];
  FONTMETRICS      fm;
  LONG             lMiddle,
                   lPosition,
                   lStringLength,
                   Incroment;
  POINTL           aptl[100];
  PFONTSELECTION   FontStorage=(PFONTSELECTION)NULL;
  LONG             FontCount=0L;
  BOOL             UserFont=FALSE;
  RECTL            rect;
  INT              color_index=0;           // color index
  static          CSP_INFO  csf= { sizeof(CSP_INFO),CLR_WHITE,CLR_BLACK};
  static LONG  colors[]={ CLR_RED,     // Colors that will draw with
                           CLR_CYAN,
                           CLR_PINK,
                           CLR_BLUE,
                           CLR_BROWN,
                           CLR_YELLOW,
                           CLR_GREEN,
                           CLR_DARKCYAN,
                           CLR_DARKGREEN,
                           CLR_DARKBLUE
                        };
#define  DEFAULTATTRIBUTES  CHS_OPAQUE|CHS_CLIP|CHS_ATTR_INFO|\
                            CHS_LEAVEPOS|CHS_START_XY

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ () GreTextApp2() Application Test - Entry Point\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)   {
           BADERROR("GpiQueryDevice()",TRUE,"GreTextApp2");
           return;
        }
        set_delta(hPS);
        if (EzQueryFonts(hPS,hWndClient,&FontCount,&FontStorage)==FALSE) {
           BADERROR("GpiQueryDevice()",TRUE,"GreTextApp2");
           return;
        }
        if (FontCount >0L) {
           if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                 FontStorage[0],0)){

              if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
              }
              UserFont=TRUE;
           }
           else {
                //Use default font
           }
        }
        if (GreQueryFontAttributes(hDC,(LONG)sizeof(fm),&fm )==FALSE) {
           BADERROR("GreQueryFontAttributes()",TRUE,"GreTextApp2");
           return;
        }
        strcpy(ErrorString," GreCharStringPos()'s Colors And Clipping Rectangles Plus Some Other Gre Text Functions.");
        lMiddle=(LONG)(strlen(ErrorString)/2L);
        Incroment=1L;
        lPosition=0L;
        ptl.x=(LONG)(delta_x/2L);
        ptl.y=delta_y;
        MapPt(ptl);
        lStringLength=(LONG)strlen(ErrorString);
        for (lIndex1=0; UnMapY(ptl.y)<(pg_size_y*delta_y) ;lIndex1++){
             if(GreQueryCharPositions(hDC,&ptl,CHS_START_XY,
                                     (LONG)strlen(ErrorString),ErrorString,
                                      NULL,aptl)==FALSE) {
                BADERROR("GreQueryCharPositions()",TRUE,"GreTextApp2");
                return;
             }
             if(GreQueryTextBox(hDC,lStringLength,
                                ErrorString+lPosition,
                                TXTBOX_COUNT,aBox)==FALSE) {
                BADERROR("GreQueryTextBox()",TRUE,"GreTextApp2");
                return;
             }
             for (lIndex2=0 ;lIndex2 < TXTBOX_COUNT ; lIndex2++) {
                  aBox[lIndex2].x+=aptl[lPosition].x;
                  aBox[lIndex2].y+=aptl[lPosition].y;
             }
             rect.xLeft=aptl[lPosition].x ;
             rect.yBottom=aBox[TXTBOX_BOTTOMLEFT].y;
             rect.xRight=aBox[TXTBOX_TOPRIGHT].x;
             rect.yTop=aBox[TXTBOX_TOPRIGHT].y;
             csf.lBackColor=colors[color_index];
             if(GreCharStringPos(hDC,&ptl,&rect,DEFAULTATTRIBUTES,
                                (LONG)strlen(ErrorString),ErrorString,
                                 NULL,&csf)==GPI_ERROR) {
                BADERROR("GreCharStringPos()",TRUE,"GreTextApp2");
                return;
             }
             if (!(lIndex1 & 3 )){
                 SWAPCOORDINATE(aBox);
                 if (GpiMove(hPS,aBox)==GPI_ERROR) {
                 }
                 if (GpiPolyLine(hPS,4L,aBox+1)==GPI_ERROR) {
                 }

             }
             if (lPosition==(lMiddle-2L))
                 Incroment=-2L;
             else if (lPosition==0L)
                      Incroment=2L;
             lPosition+=Incroment;
             lStringLength-=(Incroment*2L);
             ptl.y+=(LONG)(6L*fm.lMaxBaselineExt/5L);
             color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
             if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
             }
        }
        free(FontStorage);
        /********************************************************************
        /*       Return the proper log messages for Non-Test Gre's.         *
        /********************************************************************/

        if (!SetupCount)
           CWRITELOG(L_HDR,0,
"$GreTextApp2()                                                  -    PASS.\n");
        else{
            sprintf(ErrorString,"$ErrorCount = %ld.\n", SetupCount);
            CWRITELOG(L_HDR,0, ErrorString);
            CWRITELOG(L_HDR,0,
"$GreTextApp2()                                                  -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreTextApp2() Application Test - Exit Point\n");
   }
}
/***************************************************************************
 *\\ddd
 * Routine Name: GreCharStringExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreCharString
 *
 * System Requirements: Section 6.4.2
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *              12/21/90, PDVT, Mike Abuzant
 *               - TestCase Coding.
 *              12/30/90, PDVT, Mike Abuzant
 *               - TestCase Design.
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta(hPS)
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *   Change the character mode from default mode to CM_MODE3 since
 *   we need to change the character box size which in turn makes
 *   the characters larger. This can not be acomplished in either
 *   CM_MODE2 or CM_MODE1. Notice that the font chosen by the PM
 *   is an outlined (vector) font.
 *      GpiSetCharMode(hPS,CM_MODE3)..
 *    Get the default character box size.
 *      GpiQueryCharBox(hPS,&size)
 *    Increase the box default size by a factor and set as the default
 *      GpiSetCharBox(hPS,&size)
 *    Setup Conditions to display a character string with display dirction
 *    set to CHDIRN_BOTTOMTOP. This causes the string to be drawn from
 *    bottom of the display to the top of it.
 *       GpiSetCharDirection(hPS,CHDIRN_BOTTOMTOP)
 *       GreCharString(hDC,TheCharacterString...
 *    Setup Conditions to display a character string with display dirction
 *    set to CHDIRN_TOPBOTTOM. This causes the string to be drawn from
 *    top of the display to the bottom of it.
 *     GpiSetCharDirection(hPS,CHDIRN_TOPBOTTOM)
 *       GreCharString(hDC,TheCharacterString...
 *    Setup Conditions to display a sheard character string. The characters
 *    will be sheard in the negative X-direction. This acomplished by setting
 *    the character shear X-angle to a negative angle.
 *     GpiSetCharShear(hPS,&angle);
 *     GreCharString(hDC,TheCharacterString...)
 *    Setup Conditions to display a sheard character string. The characters
 *    will be sheard in the positive X-direction. This acomplished by setting
 *    the character shear X-angle to a positive angle.
 *     GpiSetCharShear(hPS,&angle);
 *     GreCharString(hDC,TheCharacterString...)
 *    Setup Conditions to display a set of character strings where
 *    each is drawn with a different character angle. This angle is
 *    defined as an (x,y) coordinates in such a way that the character string
 *    is drawn along the line that is formed from the (0,0) origin to the
 *    angle in question.
 *    FOR (all string to display) Do
 *        GpiSetCharAngle(hPS,&NewAngle);
 *        GreCharString(hDC,TheCharacterString...)
 *   ENDDO
 *\\end
 ***************************************************************************/
VOID APIENTRY GreCharStringExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL      ptl;
  LONG        SetupCount=0L;
  LONG        lIndex1;
  HDC         hDC;
  POINTL      angle;
  SIZEF       sizel;
  static      GRADIENTL  CharAngle={0,0};
  static      GRADIENTL aptl[]={
                               {0,0},{45,45},{0,45},
                               {-45,45},{-45,0},{-45,-45},
                               {0,-45},{45,-45}
                               };
#define  STRINGCOUNT  (LONG)(sizeof(aptl)/sizeof(aptl[0]))

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ () GreCharStringExh() Test - Entry Point\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)   {
            BADERROR("GpiQueryDevice()",TRUE,"GreCharStringExh");
            return;
        }
        set_delta(hPS);
        if (GpiSetCharMode(hPS,CM_MODE3)==FALSE) {
        }
        if (GpiQueryCharBox(hPS, &sizel) == FALSE) {
        }
        sizel.cx=(sizel.cx*4L)/3L;
        sizel.cy=(sizel.cy*6L)/4L;
        if (GpiSetCharBox(hPS, &sizel) == FALSE) {
        }
        strcpy(ErrorString,"Bottom to Top");
        if (GpiSetCharDirection(hPS,CHDIRN_BOTTOMTOP)==FALSE) {
        }
        ptl.x=delta_x;
        ptl.y=delta_y;
        MapPt(ptl);
        if (GpiMove(hPS,&ptl)==GPI_ERROR) {
        }
        if(GreCharString(hDC,(LONG)strlen(ErrorString),
                         ErrorString)==GPI_ERROR) {
           BADERROR("GreCharString()",TRUE,"GreCharStringExh");
           return;
        }
        strcpy(ErrorString,"Top to Bottom");
        if (GpiSetCharDirection(hPS,CHDIRN_TOPBOTTOM)==FALSE) {
        }
        ptl.x=(LONG)(delta_x/2L);
        ptl.y=(pg_size_y-1)*delta_y;
        MapPt(ptl);
        if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
        }
        if(GreCharString(hDC,(LONG)strlen(ErrorString),
                      ErrorString)==GPI_ERROR) {
           BADERROR("GreCharString()",TRUE,"GreCharStringExh");
           return;
        }
        if (GpiSetCharDirection(hPS,CHDIRN_LEFTRIGHT)==FALSE) {
        }
        ptl.x=(LONG)(3L*delta_x/2L);
        ptl.y=delta_y;
        MapPt(ptl);
        strcpy(ErrorString,"Sheared Text");
        if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
        }
        angle.x=-2L;
        angle.y=2L;
        if (GpiSetCharShear(hPS,&angle)==FALSE) {
        }
        if(GreCharString(hDC,(LONG)strlen(ErrorString),
                         ErrorString)==GPI_ERROR) {
           BADERROR("GreCharString()",TRUE,"GreCharStringExh");
            return;
        }
        angle.x=
        angle.y=2L;
        if (GpiSetCharShear(hPS,&angle)==FALSE) {
        }
        strcpy(ErrorString," Followed By An Other Sheared Text On The Same Line.");
        if(GreCharString(hDC,(LONG)strlen(ErrorString),
                         ErrorString)==GPI_ERROR) {
           BADERROR("GreCharString()",TRUE,"GreCharStringExh");
           return;
        }
        angle.x=0L;
        angle.y=1L;
        if (GpiSetCharShear(hPS,&angle)==FALSE) {
        }
        if (GpiSetCharAngle(hPS,&CharAngle)==FALSE) {
        }
        ptl.x=(LONG)(pg_size_x-4)*delta_x;
        ptl.y=(LONG)(3L*delta_y/2);
        MapPt(ptl);
        if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
        }
        strcpy(ErrorString,"   Hello World ");
        for (lIndex1=0; lIndex1<STRINGCOUNT ;lIndex1++){
             if (GpiSetCharAngle(hPS,aptl+lIndex1)==FALSE) {
             }
             if(GreCharString(hDC,(LONG)strlen(ErrorString),
                              ErrorString)==GPI_ERROR) {
                BADERROR("GreCharString()",TRUE,"GreCharStringExh");
                return;
             }
        }
        /********************************************************************
        /*       Return the proper log messages for Non-Test Gre's.         *
        /********************************************************************/
        if (!SetupCount)
            CWRITELOG(L_HDR,0,
"$GreCharStringExh()                                             -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", SetupCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreCharStringExh()                                             -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreCharStringExh() Test - Exit Point\n");
   }
}
/***************************************************************************
 *\\ddd
 * Routine Name: GreCharStringPosExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreCharStringPos
 *
 * System Requirements: Section 6.4.3
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *               01/02/91, PDVT, Mike Abuzant.
 *               - TestCase Design and Implementation.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta(hPS)
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *   Change the character mode from default mode to CM_MODE3 since
 *   we need to change the character box size which in turn makes
 *   the characters larger. This can not be acomplished in either
 *   CM_MODE2 or CM_MODE1. Notice that the font chosen by the PM
 *   is an outlined (vector) font.
 *      GpiSetCharMode(hPS,CM_MODE3)..
 *    Get the default character box size.
 *      GpiQueryCharBox(hPS,&DefaultSize)
 *    Increase the box default size by a factor and set as the default
 *      GpiSetCharBox(hPS,&size)
 *    Setup Conditions to display a character string with display dirction
 *    set to CHDIRN_BOTTOMTOP. This causes the string to be drawn from
 *    bottom of the display to the top of it.
 *       GpiSetCharDirection(hPS,CHDIRN_BOTTOMTOP)
 *       GreCharStringPos(hDC,TheCharacterString...
 *    Setup Conditions to display a character string with display dirction
 *    set to CHDIRN_TOPBOTTOM. This causes the string to be drawn from
 *    top of the display to the bottom of it.
 *     GpiSetCharDirection(hPS,CHDIRN_TOPBOTTOM)
 *       GreCharStringPos(hDC,TheCharacterString...
 *    Setup Conditions to display a sheard character string. The characters
 *    will be sheard in the negative X-direction. This acomplished by setting
 *    the character shear X-angle to a negative angle.
 *     GpiSetCharShear(hPS,&angle);
 *     GreCharStringPos(hDC,TheCharacterString...)
 *    Setup Conditions to display a sheard character string. The characters
 *    will be sheard in the positive X-direction. This acomplished by setting
 *    the character shear X-angle to a positive angle.
 *     GpiSetCharShear(hPS,&angle);
 *     GreCharStringPos(hDC,TheCharacterString...)
 *    Setup Conditions to display a set of character strings where
 *    each is drawn with a different character angle. This angle is
 *    defined as an (x,y) coordinates in such a way that the character string
 *    is drawn along the line that is formed from the (0,0) origin to the
 *    angle in question.
 *    FOR (all string to display) Do
 *        GpiSetCharAngle(hPS,&NewAngle);
 *        GreCharStringPos(hDC,TheCharacterString...)
 *   ENDDO
 *    //Set The character box size to its default
 *      GpiSetCharBox(hPS,&DefaultSize)
 *    //Set The character angle  to its default
 *        DefaultAngle.x=
 *        DefaultAngle.y=0L
 *        GpiSetCharAngle(hPS,&DefaultAngle);
 *    Setup Conditions to display a set of character strings where
 *    each is drawn with a different clipping rectangle and a different
 *    character background color.
 *    FOR (all string to display) Do
 *      //Query box coordinates that inscribes the character string
 *      //and store them in the Array address by 'aBox'.
 *           GreQueryTextBox(hDC,aBox....)
 *      //Shink the clipping rectangle by a fixed amount each time
 *      //through the loop and display the character string .
 *          GreCharStringPos(hDC,TheString,ClipRectangle...)
 *    ENDDO
 *\\end
 ***************************************************************************/
VOID APIENTRY GreCharStringPosExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  char        cBox[80];
  LONG        lTemp;
  POINTL      ptl;
  LONG        SetupCount=0L;
  LONG        lIndex1;
  HDC         hDC;
  POINTL      angle;
  static      GRADIENTL  CharAngle={0,0};
  SIZEF       sizel,sizel1;
  POINTL      aBox[TXTBOX_COUNT];
  static      RECTL       rect;
  FONTMETRICS fm;
  INT         color_index=0;           // color index
  static      CSP_INFO  csf= { sizeof(CSP_INFO),CLR_WHITE ,CLR_BLACK};
  static LONG  colors[]={ CLR_RED,     // Colors that will draw with
                           CLR_CYAN,
                           CLR_PINK,
                           CLR_BLUE,
                           CLR_BROWN,
                           CLR_YELLOW,
                           CLR_GREEN,
                           CLR_DARKCYAN,
                           CLR_DARKGREEN,
                           CLR_DARKBLUE};
  static GRADIENTL aptl[]={ {0,0},{45,45},{0,45},
                            {-45,45},{-45,0}};
#define  DEFAULTATTRIBUTES  CHS_OPAQUE|CHS_CLIP|CHS_ATTR_INFO|\
                            CHS_LEAVEPOS|CHS_START_XY
#define  STRINGCOUNT  (LONG)(sizeof(aptl)/sizeof(aptl[0]))

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ () GreCharStringPosExh() Test - Entry Point\n");

        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)   {
           BADERROR("GpiQueryDevice",TRUE,"GpiCharStringPosExh");
           return;
        }
        set_delta(hPS);

        if (GpiSetCharMode(hPS,CM_MODE3)==FALSE) {
            SetupCount += BADERROR("GreSetCharMode",FALSE,NULL);
        }
        if (GpiQueryCharBox(hPS, &sizel1) == FALSE) {
            SetupCount += BADERROR("GpiQueryCharBox",FALSE,NULL);
        }
        sizel.cx=(sizel1.cx*4L)/3L;
        sizel.cy=(sizel1.cy*6L)/4L;

        if (GpiSetCharBox(hPS, &sizel) == FALSE) {
            SetupCount += BADERROR("GpiSetCharBox",FALSE,NULL);
        }
        strcpy(ErrorString,"Bottom to Top");

        if (GpiSetCharDirection(hPS,CHDIRN_BOTTOMTOP)==FALSE) {
            SetupCount += BADERROR("GpiSetCharDirection",FALSE,NULL);
        }
        ptl.x=delta_x;
        ptl.y=delta_y;
        MapPt(ptl);

        if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY|CHS_LEAVEPOS,
                           (LONG)strlen(ErrorString),ErrorString,
                            NULL,NULL)==GPI_ERROR) {
            SetupCount += BADERROR("GreCharStringPos",FALSE,NULL);
        }
        strcpy(ErrorString,"Top to Bottom");

        if (GpiSetCharDirection(hPS,CHDIRN_TOPBOTTOM)==FALSE) {
            SetupCount += BADERROR("GpiSetCharDirection",FALSE,NULL);
        }
        ptl.x=(LONG)(delta_x/2L);
        ptl.y=(pg_size_y-1)*delta_y;
        MapPt(ptl);

        if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY|CHS_LEAVEPOS,
                           (LONG)strlen(ErrorString),ErrorString,
                            NULL,NULL)==GPI_ERROR) {
            SetupCount += BADERROR("GreCharStringPos",FALSE,NULL);
        }
        if (GpiSetCharDirection(hPS,CHDIRN_LEFTRIGHT)==FALSE) {
            SetupCount += BADERROR("GpiSetCharDirection",FALSE,NULL);
        }

        ptl.x=(LONG)(3L*delta_x/2L);
        ptl.y=delta_y;
        MapPt(ptl);
        strcpy(ErrorString,"Sheared Text");
        angle.x=-2L;
        angle.y=2L;

        if (GpiSetCharShear(hPS,&angle)==FALSE) {
            SetupCount += BADERROR("GpiSetCharShear",FALSE,NULL);
        }
        if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                           (LONG)strlen(ErrorString),ErrorString,
                            NULL,NULL)==GPI_ERROR) {
            SetupCount += BADERROR("GreCharStringPos",FALSE,NULL);
        }
        angle.x=
        angle.y=2L;

        if (GpiSetCharShear(hPS,&angle)==FALSE) {
            SetupCount += BADERROR("GpiSetCharShear",FALSE,NULL);
        }
        if(GreQueryTextBox(hDC,(LONG)strlen(ErrorString),ErrorString,
                           TXTBOX_COUNT,aBox)==FALSE) {
            SetupCount += BADERROR("GreQueryTextBox",FALSE,NULL);
        }
        ptl.x+=aBox[TXTBOX_CONCAT].x;
        if (GpiMove(hPS,&ptl)==GPI_ERROR) {
        }
        strcpy(ErrorString," Followed By Other Sheared Text On The Same Line.");
        if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY|CHS_LEAVEPOS,
                           (LONG)strlen(ErrorString),ErrorString,
                            NULL,NULL)==GPI_ERROR) {
            SetupCount += BADERROR("GreCharStringPos",FALSE,NULL);
        }
        angle.x=0L;
        angle.y=1L;
        if (GpiSetCharShear(hPS,&angle)==FALSE) {
            SetupCount += BADERROR("GreSetCharShear",FALSE,NULL);
        }
        ptl.x=4*delta_x;
        ptl.y=(LONG)(((double)pg_size_y-2.5)*delta_y);
        MapPt(ptl);
        strcpy(ErrorString,"   Hello World ");
        for (lIndex1=0; lIndex1<STRINGCOUNT;lIndex1++){
            if (GpiSetCharAngle(hPS,aptl+lIndex1)==FALSE) {
                SetupCount += BADERROR("GpiSetCharAngle",FALSE,NULL);
            }
            if(GreCharStringPos(hDC,&ptl,&rect,CHS_START_XY,
                               (LONG)strlen(ErrorString),ErrorString,
                                NULL,NULL)==GPI_ERROR) {
               SetupCount += BADERROR("GreCharStringPos",FALSE,NULL);
            }
        }
        if (GpiSetCharAngle(hPS,&CharAngle)==FALSE) {
            SetupCount += BADERROR("GpiSetCharAngle",FALSE,NULL);
        }
        ptl.x=(LONG)(3L*delta_x/2L);
        ptl.y=(LONG)(3L*delta_y/2L);
        MapPt(ptl);
        if (GpiSetCharBox(hPS, &sizel1) == FALSE) {
            SetupCount += BADERROR("GpiSetCharBox",FALSE,NULL);
        }
        if (GreQueryFontAttributes(hDC,(LONG)sizeof(fm),&fm )==FALSE) {
            SetupCount += BADERROR("GreQueryFontAttributes",FALSE,NULL);
        }
        strcpy(ErrorString," With GreCharStringPos(),We Can Specify Background Colors and A Clipping Rectangle.");
        for (lIndex1=0 ;UnMapY(ptl.y)< (pg_size_y-3)*delta_y;lIndex1++){
             if(GreQueryTextBox(hDC,(LONG)strlen(ErrorString),ErrorString,
                                TXTBOX_COUNT,aBox)==FALSE) {
                SetupCount += BADERROR("GreQueryTextBox",FALSE,NULL);
             }

             rect.xLeft=aBox[TXTBOX_BOTTOMLEFT].x+ptl.x+(3*fm.lAveCharWidth*lIndex1);
             rect.yBottom=aBox[TXTBOX_BOTTOMLEFT].y+ptl.y;
             rect.xRight=aBox[TXTBOX_TOPRIGHT].x-(3*fm.lAveCharWidth*lIndex1)+ptl.x;
             rect.yTop=aBox[TXTBOX_TOPRIGHT].y-lIndex1+ptl.y;
             rect.yTop=(rect.yTop<=rect.yBottom) ? (rect.yBottom+5L) :rect.yTop;
             csf.lBackColor=colors[color_index];

              // added by TSS...This test was incorrectly assigning
              // values to .xLeft that were greater than values assigned
              // to .xRight. The following code swaps the two if so.

             if( rect.xLeft > rect.xRight ) {
                lTemp = rect.xLeft;              // added by TSS...5/20/91
                rect.xLeft = rect.xRight;
                rect.xRight = lTemp;
             }

             sprintf(cBox, "left: %d\n", rect.xLeft );
             CWRITELOG(L_LOTRACE, 0, cBox );
             sprintf(cBox, "right: %d\n", rect.xRight );
             CWRITELOG(L_LOTRACE, 0, cBox );
             sprintf(cBox, "Top: %d\n", rect.yTop );
             CWRITELOG(L_LOTRACE, 0, cBox );
             sprintf(cBox, "Bottom: %d\n", rect.yBottom );
             CWRITELOG(L_LOTRACE, 0, cBox );

              // end add

             if(GreCharStringPos(hDC,&ptl,&rect,DEFAULTATTRIBUTES,
                                (LONG)strlen(ErrorString),ErrorString,
                                 NULL,&csf)==GPI_ERROR) {
                SetupCount += BADERROR("GreCharStringPos",FALSE,NULL);
             }
             ptl.y+=(LONG)(6L*fm.lMaxBaselineExt/5L);
             color_index=(++color_index)%(sizeof(colors)/sizeof(colors[0]));
         }
         /*
          *       Return the proper log messages for Non-Test Gre's.         *
          */

    if (!SetupCount)
       CWRITELOG(L_HDR,0,
"$GreCharStringPosExh()                                          -    PASS.\n");
    else{
      sprintf(ErrorString, "$ErrorCount = %ld.\n", SetupCount);
      CWRITELOG(L_HDR,0, ErrorString);
      CWRITELOG(L_HDR,0,
"$GreCharStringPosExh()                                          -    FAIL.\n");
    }

    CWRITELOG(L_LOTRACE,0,"+ex+ GreCharStringPosExh() Test - Exit Point\n");

   }
}
/***************************************************************************
 *\\ddd
 * Routine Name: GreQueryTextBoxExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreQueryTextBox
 *
 * System Requirements: Section 6.4.3
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *               12/19/90, PDVT, Mike Abuzant
 *               - TestCase Coding.
 *               12/30/90, PDVT, Mike Abuzant
 *               - TestCase Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta()
 *  EzQueryFonts()
 *  EzCreatLogicalFonts()
 *
 * Limitations:
 *
 *   For some fonts, This function does not return  the correct
 *   box coordinates  especially when the character direction is
 *   set  to anything other than CHDIRN_LEFTRIGHT .
 *   This is true for the display and printer device drivers.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *     Query fonts supported by either current device or Graphic Engine
 *     and store them in the array address by 'FontStorage'.
 *     fSuccess=EzQueryFonts(hPs,&FontStorage,&FontCount....)
 *     WHILE (there still enough room in PS to display character strings ) DO
 *        //Create a logical font.
 *           fSuccess=EzCreateLogicalFont(hPS,szFontName,FontAttributes...)
 *        // Make Logical font the current font.
 *           GpiSetCharSet(hPS,USER_FONT_LCID)
 *       //Setup conditions to display a string at the current (x,y) .
 *          GreCharStringPos(hDC,String...);
 *        //Query box coordinates that inscribes the character string
 *        //and store them in the Array address by 'aptl'.
 *           GreQueryTextBox(hDC,aptl....)
 *       Setup condition to draw the bounding parallelogram around
 *       the character string using the GpiPolyLine(). This will verify
 *       that the coordinates returned by GreQueryTextBox() is indeed
 *       correct.
 *       // Move to one of the bounding parallelogram corners
 *          GpiMove(hPS,&FirstPointOfBox)
 *       // Draw the rest of the bounding parallelogram.
 *          GpiPolyLine(hPS,4L,apyl+1).
 *       // Dispose of the current Logical font
 *          GpiSetCharSet(hPS,LCID_DEFALUT)
 *          GpiDeleteSetId(hPS,USER_FONT_LCID)
 *    ENDDOWHILE
 *    Use the above algorithm to draw three character strings, but this time
 *    with a character direction of CHDIRN_BOTTOMTOP instead of a character
 *    direction of CHDIRN_LEFTRIGHT.
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryTextBoxExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL           ptl;
  PFONTSELECTION   FontStorage=(PFONTSELECTION)NULL;
  LONG             FontCount=0L;
  LONG             SetupCount=0L;
  LONG             lIndex1,
                   lIndex2;
  HDC              hDC;
  POINTL           aptl[TXTBOX_COUNT];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
       LOG_SIGNATURE;
       TEST_SIGNATURE(0,0);
       set_delta(hPS);
       CWRITELOG(L_LOTRACE,0,"+ee+ () GreQueryTextBoxExh() Test - Entry Point\n");
       if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
           BADERROR("GpiQueryDevice()",TRUE,"GreQueryTextBoxExh");
           return;
       }
       if (EzQueryFonts(hPS,hWndClient,&FontCount,&FontStorage)==FALSE) {
           BADERROR("EzQueryFonts()",TRUE,"GreQueryTextBoxExh");
           return;
       }
       ptl.x=delta_x*3L;
       ptl.y=(LONG)(3L*delta_y/4L);
       MapPt(ptl);
       FontCount=(FontCount >(pg_size_y-1L) ) ? (pg_size_y-1L) : FontCount;
       for (lIndex1=0; lIndex1<FontCount ;lIndex1++){
           if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                    FontStorage[lIndex1],afsSel[lIndex1%4])){

               strcpy(ErrorString,"Text Must Be Within Box.");
               if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
               }
               if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                                  (LONG)strlen(ErrorString),ErrorString,
                                   NULL,NULL)==GPI_ERROR) {
                  BADERROR("GreCharStringPos()",TRUE,"GreQueryTextBoxExh");
                  return;
               }
               if(GreQueryTextBox(hDC,(LONG)strlen(ErrorString),ErrorString,
                                  TXTBOX_COUNT,aptl)==FALSE) {
                  BADERROR("GreQueryTextBox()",TRUE,"GreQueryTextBoxExh");
                  return;
               }
               for (lIndex2=0 ;lIndex2 < TXTBOX_COUNT ; lIndex2++){
                     aptl[lIndex2].x+=ptl.x;
                     aptl[lIndex2].y+=ptl.y;
               }
               SWAPCOORDINATE(aptl);
               if (GpiMove(hPS,aptl)==GPI_ERROR) {
               }
               if (GpiPolyLine(hPS,4L,aptl+1)==GPI_ERROR) {
               }
               ptl.y=UnMapY(ptl.y);
               ptl.y+=delta_y;
               ptl.y=MapY(ptl.y);
               if (GpiSetCharSet(hPS,LCID_DEFAULT)==FALSE) {
               }
               if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
               }
            }
        }
        ptl.x=(LONG)(delta_x/2L);
        ptl.y=(LONG)(2L*delta_y/3L);
        MapPt(ptl);
        for (lIndex1=0; lIndex1<3 ;lIndex1++){
            if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                     FontStorage[lIndex1],afsSel[lIndex1%4])){
               if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
               }
               if (GpiSetCharDirection(hPS,CHDIRN_BOTTOMTOP)==FALSE) {
               }
               if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                                  (LONG)strlen(ErrorString),ErrorString,
                                   NULL,NULL)==GPI_ERROR) {
                  BADERROR("GreCharStringPos()",TRUE,"GreQueryTextBoxExh");
                  return;
               }
               if(GreQueryTextBox(hDC,(LONG)strlen(ErrorString),ErrorString,
                                  TXTBOX_COUNT,aptl)==FALSE) {
                 BADERROR("GreQueryTextBox()",TRUE,"GreQueryTextBoxExh");
                 return;
               }
               for (lIndex2=0;lIndex2 <TXTBOX_COUNT;lIndex2++){
                    aptl[lIndex2].x+=ptl.x;
                    aptl[lIndex2].y+=ptl.y;
               }
               SWAPCOORDINATE(aptl);
               if ( GpiMove(hPS,aptl)==GPI_ERROR) {
               }
               if ( GpiPolyLine(hPS,4L,aptl+1)==GPI_ERROR) {
               }
               ptl.x=UnMapX(ptl.x);
               ptl.x+=delta_x;
               ptl.x=MapX(ptl.x);
               if (GpiSetCharSet(hPS,LCID_DEFAULT)==FALSE) {
               }
               if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
               }
            }
        }
        /********************************************************************
        /*       Return the proper log messages for Non-Test Gre's.
        /********************************************************************/
        free(FontStorage);

        if (!SetupCount)
            CWRITELOG(L_HDR,0,
"$GreQueryTextBoxExh()                                           -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", SetupCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreQueryTextBoxExh()                                           -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryTextBoxExh() Test - Exit Point\n");
 }
}
 /***************************************************************************
 *\\ddd
 * Routine Name: GreQueryWidthTableExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreQueryWidthTable
 *
 * System Requirements: Section 6.4.3
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *               12/19/90, PDVT, Mike Abuzant
 *               - TestCase Coding.
 *               12/29/90, PDVT, Mike Abuzant
 *               - TestCase Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta()
 *  EzQueryFonts()
 *  EzCreateLogicalFonts()
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *     Query fonts supported by either current device or Graphic Engine
 *     and store them in the array address by 'FontStorage'.
 *     fSuccess=EzQueryFonts(hPs,&FontStorage,&FontCount....)
 *     FOR (All the fonts Supported ) Do
 *      //Create a logical font.
 *       fSuccess=EzCreateLogicalFont(hPS,szFontName,FontAttributes...)
 *     // Make Logical font the current font.
 *        GpiSetCharSet(hPS,USER_FONT_LCID)
 *     //Query current font metrix.
 *         GreQueryFontAttributes(hDC,&fm...)
 *     //Query The width table for selected font and store them
 *     //in the Array address by aWidth.
 *         GreQueryWidthTable(hDC,aWidth....)
 *     //Setup conditions to display a string at the current (x,y) .
 *        GreCharStringPos(hDC,ErrorString...);
 *     Setup condition to BOLD every other character in the string
 *     by using the characters width table to calculate the (x,y)
 *     position of the addressed character. This will verify that
 *     width table we queried earlier isindeed correct.
 *     for( the length of thw string) DO
 *       calculate the position of every other character.
 *       Display the character @ calculated (x,y)
 *          GreCharStringPos(hDC,ErrorString...);
 *     ENDDO (string length)
 *     // Dispose of the current Logical font
 *        GpiSetCharSet(hPS,LCID_DEFALUT)
 *        GpiDeleteSetId(hPS,USER_FONT_LCID)
 *  ENDDO   (all fonts supported)
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryWidthTableExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL           ptl;
  LONG             SetupCount=0L;
  HDC              hDC;
  USHORT           Index,
                   yChar;
  LONG             aWidth[256];
  FONTMETRICS      fm;
  PFONTSELECTION   FontStorage=(PFONTSELECTION)NULL;
  LONG             FontCount=0L;
  LONG             lFirstChar=(LONG)0;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+  GreQueryWidthTableExh() Test - Entry Point\n");
        set_delta(hPS);
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
             BADERROR("GpiQueryDevice()",TRUE,"GreQueryWidthTableExh");
             return;
        }
        if (EzQueryFonts(hPS,hWndClient,&FontCount,&FontStorage)==FALSE) {
             BADERROR("EzQueryFonts()",TRUE,"GreQueryWidthTableExh");
             return;
        }
        ptl.x=(LONG)(delta_x/2L);
        ptl.y=(LONG)(2L*delta_y/3L);
        MapPt(ptl);
        if (GreQueryFontAttributes(hDC,(LONG)sizeof(fm),&fm )==FALSE) {
            BADERROR("GreQueryFontAttributes()",TRUE,"GreQueryWidthTableExh");
            return;
        }

        TEST_PROGRESS ( 00, ( USHORT ) FontCount );

        yChar=(USHORT)(fm.lMaxBaselineExt);
        for (Index=0; Index<(USHORT)FontCount;Index++){
             if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                      FontStorage[Index],afsSel[Index%4])){
                if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
                }
                if (GreQueryFontAttributes(hDC,(LONG)sizeof(fm),&fm )==FALSE) {
                    BADERROR("GreQueryFontAttributes()",TRUE,"GreQueryWidthTableExh");
                    break;
                }
                if ((GreQueryWidthTable(hDC,lFirstChar,255L,aWidth))==FALSE) {
                     BADERROR("GreQueryWidthTable()",TRUE,"GreQueryWidthTableExh");
                     break;;
                }
                if (GpiSetCharSet(hPS,LCID_DEFAULT)==FALSE) {
                }
                if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
                }
                sprintf(ErrorString,"%s : Width Of The Character  'W' is  %ld ",
                        fm.szFacename,aWidth[(INT)'W']);
                if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                                   (LONG)strlen(ErrorString),ErrorString,
                                    NULL,NULL)==GPI_ERROR) {
                   BADERROR("GreCharStringPos()",TRUE,"GreQueryWidthTableExh");
                   break;;
                }
                ptl.y+=(LONG)(3L*yChar/2L);
            }

            TEST_PROGRESS ( Index + 1, ( USHORT ) FontCount );
        }

        /********************************************************************
        /*              Return the proper log messages for Non-Test Gre's.
        /********************************************************************/
        free(FontStorage);

        if (!SetupCount)
            CWRITELOG(L_HDR,0,
"$GreQueryWidthTableExh()                                        -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", SetupCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreQueryWidthTableExh()                                        -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryWidthTableExh() Test - Exit Point\n");
   }
}
 /***************************************************************************
 *\\ddd
 * Routine Name: GreRealizeFontExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreRealizeFont
 *
 * System Requirements: Section 6.4.3
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta(hPS)
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *\\end
 ***************************************************************************/
VOID APIENTRY GreRealizeFontExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL ptl;
  LONG   ErrorCount;
  CHAR   ErrorString[257];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ GreRealizeFontExh - Entry.\n");
        ErrorCount = 0L;
        set_delta(hPS);
        if (ErrorCount) {
           sprintf(ErrorString,"$ErrorCount == %d.\n",ErrorCount);
           CWRITELOG(L_HDR,0, ErrorString);
           CWRITELOG(L_HDR,0,
"$GreRealizeFontExh                                              -    FAIL.\n");
        }
        else {
           CWRITELOG(L_HDR,0,
"$GreRealizeFontExh                                              -    PASS.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreRealizeFontExh - Exit.\n");
  }
}
/***************************************************************************
 *\\ddd
 * Routine Name: GreDeviceQueryFontsExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreDeviceQueryFonts()
 *
 * System Requirements: Section 6.4.3
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *               12/24/90, PDVT, Mike Abuzant
 *               - TestCase Design and implementation.
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta(hPS)
 *
 * Limitations:
 *
 *  The font metrics contain a large a mount of data that can't be
 *  displayed on current device all at once,so i choose to display the
 *  face name of all the fonts that are supported by the current device.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    //Get the Device Context associated with Presentation Spac.
 *      hDC=GpiQueryDevice(hPS) ....
 *    //Query the number of device fonts supported by the device
 *    // by passing a value of zero for the number of the fonts
 *    // requested to return information for..
 *      NumberOfDeviceFontsAvailable=GreDeviceQueryFont(hDC,....)
 *    // Print a message to inform the user that no device fonts
 *      are supported by current device and exit to DTT.
 *    //Allocate enough memory to hold information for all device fonts
 *      supported by the device.
 *      FontStorage=malloc(NumberOfDeviceFontsAvailable*size(FONTMETRICS ))..
 *    //Query All information of all device fonts supported by the device
 *     GreDeviceQueryFont(hDC,NumberOfDeviceFontsAvailable,....)
 *     IF (Device does not support any fonts) BEGIN
 *         display a message that says :
 *            "device does not support any fonts"
 *     ENDIF
 *    ELSE
 *     For (all the fonts supported by device ) DO
 *       //Print the font names on current display device
 *     ENDDO;
 *    ENDELSE;
 *     Return the proper log messages for Non-Test Gre's. and Gpi's by
 *     printing the number of function that failed.
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceQueryFontsExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL        ptl;
  HDC           hDC;
  FONTMETRICS  *pfm;
  void        huge *pmem;
  FONTMETRICS   fm;
  LONG          Requested=0L;
  LONG          ychar;
  SHORT         Index;
  LONG          SetupCount=0L;
  LONG          FontCount=0L;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        TEST_SIGNATURE(0,0);
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+  GreDeviceQueryFontsExh() Test - Entry Point\n");
        set_delta(hPS);
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
             BADERROR("GpiQueryDevice()",TRUE,"GreDeviceQueryFontsExh");
             return;
        }
        Requested=0L;
        if ((FontCount=GreDeviceQueryFonts(hDC,QF_PUBLIC,NULL,
                                     &fm,sizeof(fm),&Requested))==GPI_ALTERROR){
            BADERROR("GreQueryDeviceFonts()",TRUE,"GreDeviceQueryFontsExh");
            return;
        }

 //TSS...limit FontCount to 314...1 segment or less of mem needed
 //       if( FontCount > 314 ) FontCount = 314;

        if (FontCount==0){
            ptl.x =(LONG)(delta_x*2);
            ptl.y =(LONG)(pg_size_y/2L)*delta_y;
            MapPt(ptl);
            sprintf(ErrorString,"Total Device Fonts Supported By This Device= %ld ",FontCount);
            if (GpiMove(hPS,&ptl)==GPI_ERROR) {
            }
            if(GreCharString(hDC,(LONG)strlen(ErrorString),ErrorString)==GPI_ERROR) {
               BADERROR("GreCharString()",TRUE,"GreDeviceQueryFontsExh");
               return;
            }
        }
        else {
             /*******************************************************************
              * Display all the fonts names that are supported by
              * This device.
              *******************************************************************/
//             if ((pfm=(PFONTMETRICS)malloc((SHORT)FontCount*sizeof(FONTMETRICS)))==NULL){
//                  BADERROR("malloc()",TRUE,"GreDeviceQueryFontsExh");
//                  return ;
//             }



    if ((  pfm=(PFONTMETRICS) malloc ((SHORT)FontCount *sizeof(FONTMETRICS)))
                                                       ==NULLHANDLE )  {
         BADERROR("malloc()",TRUE,"GreDeviceQueryFontsExh");
         return;
    }



             if ((GreDeviceQueryFonts(hDC,QF_PUBLIC,NULL,
                        pfm,(LONG)sizeof(FONTMETRICS),&FontCount))==GPI_ALTERROR){
                 BADERROR("GreDeviceQueryFonts()",TRUE,"GreDeviceQueryFontsExh");
                 free(pfm);
                 return;
             }
             ptl.x=(LONG)(delta_x/2L);
             ptl.y=delta_y;
             MapPt(ptl);
             if (GreQueryFontAttributes(hDC,(LONG)sizeof(fm),&fm )==FALSE) {
                 BADERROR("GreQueryFontAttributes()",TRUE,"GreDeviceQueryFontsExh");
                 return;
             }
             ychar=(LONG)(fm.lMaxBaselineExt);
             for (Index = 0; Index< (SHORT)FontCount-1 ; Index+=2){

//TSS...increment Index if on segment boarder.
    if( (Index == 315) || (Index == 630) || (Index == 945) || (Index == 1260) )
         Index += 1;

                  sprintf(ErrorString,"%33s.   %33s.",pfm[Index].szFacename,
                                                      pfm[Index+1].szFacename);
                  if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                                     (LONG)strlen(ErrorString),ErrorString,
                                      NULL,NULL)==GPI_ERROR) {
                     BADERROR("GreCharStringPos()",TRUE,"GreDeviceQueryFontsExh");
                     return;
                  }
                  ptl.y+=(ychar+5L);
             }
             free(pfm); // 16KQ
        }
        /********************************************************************
        /*              Return the proper log messages for Non-Test Gre's.  *
        /********************************************************************/
        if (!SetupCount)
            CWRITELOG(L_HDR,0,
"$GreDeviceQueryFontsExh()                                       -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", SetupCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreDeviceQueryFontsExh()                                       -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceQueryFontsExh() Test -  Exit Point\n");
    }
}
/***************************************************************************
 *\\ddd
 * Routine Name: GreDeviceQueryFontAttributesExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreDeviceQueryFontAttributes()
 *
 * System Requirements: Section 6.4.3
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *
 *               12/24/90, PDVT, Mike Abuzant
 *               - TestCase Design and implementation.
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta(hPS)
 *
 * Limitations:
 *  The font metrics contain a large a mount of data that can't be
 *  displayed on current device all at once,so i choose to display the
 *  face name of the currently selected font instead of displaying all
 *  the information contained in the font metrics.
 *
 *   This Does not work correctly for display driver and sometimes
 *   it traps. For print drivers, this function works correctly.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    //Get the Device Context associated with Presentation Spac.
 *      hDC=GpiQueryDevice(hPS) ....
 *    // Call EzQueryFonts to retreive all device fonts supported
 *    // by the device. If no fonts are supported by the device, the
 *    // function will retrieve the fonts supported by the Graphic
 *    // Engine.
 *      fSuccess=EzQueryFonts(hDC,....)
 *    //Query All information of current font in order to determine the
 *    //character height of the font. This used for displaying purposes.
 *     GreDeviceQueryFontAttributes(hDC,&fm,....)
 *     For (all the fonts that been retreived... ) DO
 *       //Realize next logical font.
 *         EzCreateLogicalFonts(hPS,.....)
 *       //if succeeded in realizing the font Do
 *       // Make this font the current font
 *          GpiSetCharSet(hPS,USER_LCID...);
 *       //Query All information of current font.
 *         GreDeviceQueryFontAttributes(hDC,&fm,....)
 *       //  Display the face name of realized font and the face
 *       //  name of the current font. They should be the same.
 *      //   Set current font to the default font...
 *           GpiSetCharSet(hPS,LCID_DEFAULT...);
 *      // Delete the Realized font. No longer needed ..
 *          GpiDeleteSetId(hPS,USER_LCID ...)
 *       ENDIF
 *   ENDDO;
 *     Return the proper log messages for Non-Test Gre's. and Gpi's by
 *     printing the number of function that failed.
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceQueryFontAttributesExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL        ptl;
  HDC           hDC;
  FONTMETRICS   fm;
  LONG          Requested=0L;
  LONG          ychar;
  SHORT         Index;
  LONG          SetupCount=0L;
  LONG          FontCount=0L;
  PFONTSELECTION     FontStorage=(PFONTSELECTION)NULL;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        TEST_SIGNATURE(0,0);
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+  GreDeviceQueryFontAttributesExh() Test - Entry Point\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
             BADERROR("GpiQueryDevice()",TRUE,"GreDeviceQueryFontAttributesExh");
             return;
        }
        set_delta(hPS);
        if (EzQueryFonts(hPS,hWndClient,&FontCount,&FontStorage)==FALSE) {
            BADERROR("EzQueryFonts()",TRUE,"GreDeviceQueryFontAttributesExh");
            return;
        }
        ptl.x=(LONG)(delta_x/2L);
        ptl.y=delta_y;
        MapPt(ptl);
        if (GreDeviceQueryFontAttributes(hDC,(LONG)sizeof(FONTMETRICS), &fm)==FALSE) {
           BADERROR("GreDeviceQueryFontAttributes()",TRUE,"GreDeviceQueryFontAttributesExh");
           return;
        }
        ychar=(LONG)(fm.lMaxBaselineExt);
        for (Index=0; Index<(USHORT)FontCount;Index++){
             if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                      FontStorage[Index],afsSel[Index%4])){
                if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
                }
                if (GreDeviceQueryFontAttributes(hDC,(LONG)sizeof(FONTMETRICS),
                                          &fm)==FALSE) {
                    BADERROR("GreDeviceQueryFontAttributes()",TRUE,"GreDeviceQueryFontAttributesExh");
                    return;
                }
                sprintf(ErrorString,"Selected Font : %s .",fm.szFacename);
                if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                                   (LONG)strlen(ErrorString),ErrorString,
                                    NULL,NULL)==GPI_ERROR) {
                   BADERROR("GreCharStringPos()",TRUE,"GreDeviceQueryFontAttributesExh");
                   return;
                }
                if (GpiSetCharSet(hPS,LCID_DEFAULT)==FALSE) {
                }
                if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
                }
                ptl.y+=(LONG)(3L*ychar/2L);
            }
        }
        free(FontStorage);
        /********************************************************************
        /*              Return the proper log messages for Non-Test Gre's.  *
        /********************************************************************/
        if (!SetupCount)
            CWRITELOG(L_HDR,0,
"$GreDeviceQueryFontAttributesExh()                              -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorString = %ld.\n", SetupCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreDeviceQueryFontAttributesExh()                              -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreDeviceQueryFontAttributesExh() Test - Exit Point\n");
    }
}
/***************************************************************************
 *\\ddd
 * Routine Name: GreQueryCharPositionsExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreQueryCharPositions
 *
 * System Requirements: Section 6.4.3
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *
 *               12/19/90, PDVT, Mike Abuzant
 *               - TestCase  Implementation
 *               12/31/90, PDVT, Mike Abuzant
 *               - TestCase  Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta()
 *  EzQueryFonts()
 *  EzCreateLogicalFonts()
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *     Query fonts supported by either current device or Graphic Engine
 *     and store them in the array address by 'FontStorage'.
 *     fSuccess=EzQueryFonts(hPs,&FontStorage,&FontCount....)
 *     WHILE (there still enough room in PS to display character strings ) DO
 *        //Create a logical font.
 *           fSuccess=EzCreateLogicalFont(hPS,szFontName,FontAttributes...)
 *        // Make Logical font the current font.
 *           GpiSetCharSet(hPS,USER_FONT_LCID)
 *       //Setup conditions to display a string at the current (x,y) .
 *          GreCharStringPos(hDC,String...);
 *        //Query character string positions and
 *        //store them in the Array address by 'aptl'.
 *           GreQueryCharPositions(hDC,aptl....)
 *       Setup conditions to UNDERLINE a word of the previously
 *       displayed character string. If the word is UnderLined Correctly,
 *       it will be an indication that GreQueryCharPositions() does indeed
 *       work.
 *        //Make the current font an UnderLined font.
 *           fSuccess=EzCreateLogicalFont(hPS,szFontName,UNDERLINE_FONT...)
 *        // Make Logical font the current font.
 *           GpiSetCharSet(hPS,USER_FONT_LCID)
 *       //Setup conditions to display the word at the (x,y) coordinate
 *       // of its first Character.
 *          GreCharStringPos(hDC,TheWord...);
 *       // Dispose of the current Logical font
 *          GpiSetCharSet(hPS,LCID_DEFALUT)
 *          GpiDeleteSetId(hPS,USER_FONT_LCID)
 *    ENDDOWHILE
 *    Use the above algorithm to draw three character strings, but this time
 *    with a character direction of CHDIRN_BOTTOMTOP instead of a character
 *    direction of CHDIRN_LEFTRIGHT.
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryCharPositionsExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL           ptl;
  PFONTSELECTION   FontStorage=(PFONTSELECTION )NULL;
  LONG             FontCount=0L;
  LONG             SetupCount=0L;
  LONG             lIndex1;
  FONTMETRICS      fm;
  HDC              hDC;
  POINTL           aptl[100];
  CHAR             sOut[256];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 100000;
    *pBasisY = 100000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ () GreQueryCharPositionsExh() Test - Entry Point\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
           BADERROR("GpiQueryDevice()",TRUE,"GreQueryCharPositionsExh");
           return;
        }
        set_delta(hPS);
        if (EzQueryFonts(hPS,hWndClient,&FontCount,&FontStorage)==FALSE) {
           BADERROR("GpiQueryDevice()",TRUE,"GreQueryCharPositionsExh");
           return;
        }
        ptl.x=delta_x*2;
        ptl.y=delta_y;
        MapPt(ptl);


        TEST_PROGRESS ( 00, ( USHORT ) FontCount + 3 );

        for (lIndex1=0; lIndex1<FontCount ;lIndex1++){
       sprintf( sOut, "    FontStorage[%ld] (%s) was placed at y=%ld", lIndex1,
                              FontStorage[lIndex1].Fontname, ptl.y );
       CWRITELOG(L_LOTRACE,0,sOut);
             if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                      FontStorage[lIndex1],afsSel[lIndex1%3])){
        //TSS         strcpy(ErrorString,"    This Word is UnderScored");
       sprintf( ErrorString, "    FontStorage[%ld] (%s) was placed at y=%ld", lIndex1,
                              FontStorage[lIndex1].Fontname, ptl.y );
                 if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
                 }
                 if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                            (LONG)strlen(ErrorString),ErrorString,
                            NULL,NULL)==GPI_ERROR) {
                    BADERROR("GreCharStringPos()",TRUE,"GreQueryCharPositionsExh");
                    break;
                 }
                 if(GreQueryCharPositions(hDC,&ptl,CHS_START_XY,
                                         (LONG)strlen(ErrorString),ErrorString,
                                          NULL,aptl)==FALSE) {
                    BADERROR("GreQueryCharPositions()",TRUE,"GreQueryCharPositionsExh");
                    break;;
                 }
                 if (GpiSetCharSet(hPS,LCID_DEFAULT)==FALSE) {
                 }
                 if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
                 }
                 if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                          FontStorage[lIndex1],
                                          afsSel[lIndex1%3]|FATTR_SEL_UNDERSCORE)){
                     if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
                     }
                     if(GreCharStringPos(hDC,aptl+4,NULL,CHS_START_XY,
                                         4L,ErrorString+4,
                                         NULL,NULL)==GPI_ERROR) {
                        BADERROR("GreCharStringPos()",TRUE,"GreQueryCharPositionsExh");
                        break;
                     }
                 }
                 if (GreQueryFontAttributes(hDC,(LONG)sizeof(fm),&fm )==FALSE) {
                    BADERROR("GreQueryFontAttributes()",TRUE,"GreQueryCharPositionsExh");
                    break;
                 }
                 ptl.y+=(LONG)((LONG)fm.lMaxBaselineExt*3L/2L);
                 if (GpiSetCharSet(hPS,LCID_DEFAULT)==FALSE) {
                 }
                 if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
                 }
            }

            TEST_PROGRESS ( ( USHORT ) lIndex1 + 1, ( USHORT ) FontCount + 3 );
        }
        ptl.x=0L;
        ptl.y=delta_y;
        MapPt(ptl);
        for (lIndex1=0; lIndex1<3 ;lIndex1++){
             if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                      FontStorage[lIndex1],afsSel[lIndex1%2])){
                 strcpy(ErrorString,"    This Word is UnderScored");
                 if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
                 }
                 if (GreQueryFontAttributes(hDC,(LONG)sizeof(fm),&fm )==FALSE) {
                     BADERROR("GreQueryFontAttributes()",TRUE,"GreQueryCharPositionsExh");
                     break;
                 }
                 ptl.x+=(LONG)(2L*(LONG)fm.lAveCharWidth);
                 if (GpiSetCharDirection(hPS,CHDIRN_BOTTOMTOP)==FALSE) {
                 }
                 if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                                    (LONG)strlen(ErrorString),ErrorString,
                                     NULL,NULL)==GPI_ERROR) {
                    BADERROR("GreCharStringPos()",TRUE,"GreQueryCharPositionsExh");
                    break;
                 }
                 if(GreQueryCharPositions(hDC,&ptl,CHS_START_XY,
                                         (LONG)strlen(ErrorString),ErrorString,
                                          NULL,aptl)==FALSE) {
                    BADERROR("GreQueryCharPositions()",TRUE,"GreQueryCharPositionsExh");
                    break;
                 }
                 if (GpiSetCharSet(hPS,LCID_DEFAULT)==FALSE) {
                 }
                 if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
                 }
                 if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                          FontStorage[lIndex1],
                                          afsSel[lIndex1%2]|FATTR_SEL_UNDERSCORE)){
                     if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
                     }
                     if (GpiSetCharDirection(hPS,CHDIRN_BOTTOMTOP)==FALSE) {
                     }
                     if(GreCharStringPos(hDC,aptl+4,NULL,CHS_START_XY,
                                         4L,ErrorString+4,
                                         NULL,NULL)==GPI_ERROR) {
                        BADERROR("GreCharStringPos()",TRUE,"GreQueryCharPositionsExh");
                        break;
                     }
                 }
                 if (GpiSetCharSet(hPS,LCID_DEFAULT)==FALSE) {
                 }
                 if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
                 }
            }

            TEST_PROGRESS ( ( ( USHORT ) lIndex1 + FontCount + 1 ),
                              ( USHORT ) FontCount );
        }
        free(FontStorage);
        /********************************************************************
        /*       Return the proper log messages for Non-Test Gre's.
        /********************************************************************/

        if (!SetupCount)
            CWRITELOG(L_HDR,0,
"$GreQueryCharPositionsExh()                                     -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", SetupCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreQueryCharPositionsExh()                                     -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryCharPositionsExh() Test - Exit Point\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreGetCodePageExh()
 *
 * Purpose: This function tests the functionality of the Gre function
 *           GreGetCodePage(). The syntax used to call the Gre is :
 *
 *          (LONG) GreGetCodePage(
 *                      (HDC)     hdc,
 *                      (ULONG)   pInstance,
 *                      (ULONG)   lFunction)
 *
 *           GreGetCodePage() returns the current code page.  This is the
 *           default code page.  This function applies to the default font,
 *           not the currently selected font, which you can determine with
 *           a call to GreQueryFontAttributes.
 * Errors that are returned by this function are :
 *
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * System Requirements: 6.4.3
 *
 * Revision Log:
 *               12/14/90, pdvt ,Darin C. Hafer
 *               -Test Case Design.
 *               12/24/90, PDVT , Mike Abuzant
 *               -Test Case Implemetation
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *        NONE
 *
 * Return Codes Successfully Tested:
 *    NONE
 * Return Codes Not Tested Yet:
 *    NONE
 * Subroutines Required:
 *  set_delta(hPS)
 *
 * Limitations:
 *
 *  This function will be tested more in the function  GreGetCodePageExh()
 *  since both of them go together.
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *   // Get CodePage of the current font.
 *      GreGetCodePage(hDC).
 *   Display the CodePAge retured by GreGetCodePage().
 *\\end
 ***************************************************************************/
VOID APIENTRY GreGetCodePageExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  LONG     SetupCount=0L;
  HDC      hDC;
  LONG     CodePage;
  POINTL   ptl;

  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS,NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ () GreGetCodePageExh() Test - Entry Point\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
           BADERROR("GpiQueryDevice()",TRUE,"GreGetCodePageExh");
           return;
        }
        set_delta(hPS);
        ptl.x=(LONG)(delta_x/4L);
        ptl.y=delta_y;
        MapPt(ptl);
        if ((CodePage=GreGetCodePage(hDC))==GPI_ERROR) {
           BADERROR("GreGetCodePage()",TRUE,"GreGetCodePageExh");
           return;
        }
        sprintf(ErrorString,"GreGetCodePage() Says CodePage=%ld.",CodePage);
        if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                           (LONG)strlen(ErrorString),ErrorString,
                            NULL,NULL)==GPI_ERROR) {
          BADERROR("GreCharStringPos()",TRUE,"GreGetCodePageExh");
          return;
        }
        /********************************************************************
        /*              Return the proper log messages for Non-Test Gre's.
        /********************************************************************/

        if (!SetupCount)
            CWRITELOG(L_HDR,0,
"$GreGetCodePageExh()                                            -    PASS.\n");
        else{
             sprintf(ErrorString,"$$$SETUPCOUNT = %ld.\n", SetupCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreGetCodePageExh()                                            -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ () GreGetCodePageExh() Test - Exit Point\n");
     }
}
/***************************************************************************
 *\\ddd
 * Routine Name: GreSetCodePageExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreSetCodePage
 *
 * System Requirements: Section 6.4.3
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *               12/18/90, PDVT ,Mike M. Abuzant
 *               -Test Case implemetation
 *               12/31/90, PDVT ,Mike M. Abuzant
 *               -Test Case Design
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta(hPS)
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *    //Query the Anchor block.
 *     hAB=WinQueryAnchorBlock(hWndClient)
 *    //Query the number of the code pages suppoted by the display
 *       TotalAvailable=WinQueryCpList(hAB,TotalReturned..)
 *    //Allocate enough memory to hold all the code pages that are
 *      supported by display drive.
 *      aCode=malloc(TotalAvailable...)
 *    //Query Info. for all code pages.
 *       TotalAvailable=WinQueryCpList(hAB,TotalReturned..)
 *    FOR (All the code pages supported by display driver) DO
 *       //Set a new  code page .
 *         GreSetCodePage(hDC,&NewCodePage)
 *       //Get current code page
 *         CurrentCodePageGreSetCodePage(hDC)
 *       //Setup conditiuons to display the current code page.
 *         sprintf(ErrorString,"current codepage=....",...)
 *         GreCharStringPos(hDC,ErrorString,....)
 *    ENDDO
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetCodePageExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL     ptl;
  LONG       SetupCount=0L;
  HDC        hDC;
  HAB        hAB;
  USHORT     TotalReturned=0,
             Index,
             yChar,
             TotalAvailable;
  PULONG    aCodePage;
  LONG       CodePage;
  FONTMETRICS fm;
  static USHORT PrintaCodePage[]={437,850,437,850,
                                  437,850,437,850,
                                  437,850,437,850};

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ () GreSetCodePageExh() Test - Entry Point\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
           BADERROR("GpiQueryDevice()",TRUE,"GreSetCodePageExh");
           return;
        }
        set_delta(hPS);
        if ((hAB=WinQueryAnchorBlock(hWndClient))==NULL){
            BADERROR("GpiQueryDevice()",TRUE,"GreSetCodePageExh");
            return;     // Can't proceed without Anchor block
        }
        if (alDevCaps[CAPS_TECHNOLOGY] == 2){
           if ((TotalAvailable=WinQueryCpList(hAB,TotalReturned,NULL))==0) {
              BADERROR("WinQueryCpList()",TRUE,"GreSetCodePageExh");
              return;
           }
           if (TotalAvailable==0L) {
               BADERROR("WinQueryCpList()",TRUE,"GreSetCodePageExh");
               return;
           }
        }
        else {
              TotalAvailable=12;   // For printers set the total pages to 12
                                   //Since there is no way of knowing how many
                                   // code pages the printers support
        }
        if ((aCodePage=(PULONG)malloc((USHORT)TotalAvailable* sizeof(USHORT)))==NULL) {
           BADERROR("WinQueryCpList()",TRUE,"GreSetCodePageExh");
           return;
        }
        if (alDevCaps[CAPS_TECHNOLOGY] == 2){
           if (WinQueryCpList(hAB,TotalAvailable,aCodePage)==0) {
               BADERROR("WinQueryCpList()",TRUE,"GreSetCodePageExh");
               return;
           }
        }
        else {
             memcpy(aCodePage,PrintaCodePage,sizeof(PrintaCodePage));
        }
        ptl.x=(LONG)(delta_x/4L);
        ptl.y=delta_y;
        MapPt(ptl);
        if (GreQueryFontAttributes(hDC,(LONG)sizeof(fm),&fm )==FALSE) {
           BADERROR("GreQueryFontAttributes()",TRUE,"GreSetCodePageExh");
           return;
        }
        yChar=(USHORT)(fm.lMaxBaselineExt);
        for (Index=0;Index<TotalAvailable-1 ;Index++){
            if (GreSetCodePage(hDC,(LONG)aCodePage[Index])==FALSE) {
               BADERROR("GreSetCodePage()",TRUE,"GreSetCodePageExh");
               return;
            }
            if ((CodePage=GreGetCodePage(hDC))==GPI_ERROR) {
               BADERROR("GreGetCodePage()",TRUE,"GreSetCodePageExh");
               return;
            }
            if ((USHORT)CodePage!=aCodePage[Index]){
               BADERROR("GreGetCodePage()",TRUE,"GreSetCodePageExh");
               return;
            }
            sprintf(ErrorString,"GreGetCodePage() Says CodePage=%ld. Now Changing"
                                " CodePage to  %d",CodePage,aCodePage[Index+1]);
            if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                               (LONG)strlen(ErrorString),ErrorString,
                                NULL,NULL)==GPI_ERROR) {
               BADERROR("GreCharStringPos()",TRUE,"GreSetCodePageExh");
               return;
            }
            ptl.y+=(LONG)(3L*yChar/2L);
        }
        free(aCodePage);
        /********************************************************************
        /*              Return the proper log messages for Non-Test Gre's.
        /********************************************************************/

       if (!SetupCount)
           CWRITELOG(L_HDR,0,
"$GreSetCodePageExh()                                            -    PASS.\n");
       else{
            sprintf(ErrorString,"$ErrorCount = %ld.\n", SetupCount);
            CWRITELOG(L_HDR,0, ErrorString);
            CWRITELOG(L_HDR,0,
"$GreSetCodePageExh()                                            -    FAIL.\n");
       }
       CWRITELOG(L_LOTRACE,0,"+ex+ () GreSetCodePageExh() Test - Exit Point\n");
    }
}
/***************************************************************************
 *\\ddd
 * Routine Name: GreGetPairKerningTableExh
 *
 * Purpose:  This is an entry point for the DTT that causes exhaustive
 *           testing of GreGetPairKerningTable
 *
 * System Requirements: Section 6.4.3
 *
 * Revision Log: 10/19/90, PDVT, Gisselle Labault
 *               - created initial template
 *               12/18/90, PDVT ,Mike M. Abuzant
 *               -TestCase implemetation.
 *               12/29/90, PDVT ,Mike M. Abuzant
 *               -TestCase Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *
 * Subroutines Required:
 *  set_delta()
 *  EzQueryFonts()
 *  EzCreateLogicalFonts()
 *
 * Limitations:
 *  for print drivers, the actual count of kerning pairs does not
 *  agree with the count returned by the function GreGetPairKerningTable().
 *  The actual Kerning Pairs count is a field in the font metrix.
 *  In other word, if a font does not support any kerning, the function
 *  GreGetPairKerningTable() returns a value of one instead of zero.
 *  For Display drivers, The function works correctly.
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *     Query fonts supported by either current device or Graphic Engine
 *     and store them in the array address by 'FontStorage'.
 *     fSuccess=EzQueryFonts(hPs,&FontStorage,&FontCount....)
 *     FOR (All the fonts Supported ) Do
 *      //Create a logical font.
 *       fSuccess=EzCreateLogicalFont(hPS,szFontName,FontAttributes...)
 *     // Make Logical font the current font.
 *        GpiSetCharSet(hPS,USER_FONT_LCID)
 *     //Query current font metrix.
 *         GreQueryFontAttributes(hDC,&fm...)
 *     //Retrieve the actual Kerning Pairs count
 *       ActualKerningCount=fm.sKerningPairs;
 *     //Query The kerning pairs information for selected font and store them
 *     //in the Array address by aKern.
 *        ExpectedKerningCount=GreGetPairKerningTable(hDC,aKern,ActualKerningCount)
 *     // Dispose of the current Logical font
 *        GpiSetCharSet(hPS,LCID_DEFALUT)
 *        GpiDeleteSetId(hPS,USER_FONT_LCID)
 *     if (kerning is supported)
 *        Setup conditions to print all the kerning pairs for current
 *        font plus the KerningAmount for each Kerning Pairs.
 *     ENDIF
 *     else (if kerning is not supported)
 *       Display the font name plus the string "kerning is not supported "
 *          GreCharStringPos(hDC,ErrorString...);
 *     ENDELSE
 *  ENDDO   (all fonts supported)
 *\\end
 ***************************************************************************/
VOID APIENTRY GreGetPairKerningTableExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  POINTL ptl;
  LONG       SetupCount=0L;
  HDC        hDC;
  CHAR       String1[400];
  CHAR       String2[400];
  LONG       Index,
             Index1,
             yChar;
  KERNINGPAIRS  aKern[30];

  FONTMETRICS fm;
  PFONTSELECTION   FontStorage=(PFONTSELECTION)NULL;
  LONG        FontCount=0L;
  LONG        ExpectedKernCount=0L;
  LONG        ActualKernCount=0L;
  LONG        TempKernCount=0L;
  LONG        Inc;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        TEST_SIGNATURE(0,0);
        CWRITELOG(L_LOTRACE,0,"+ee+ () GreGetPairKerningTableExh() Test - Entry Point\n");
        set_delta(hPS);
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
           BADERROR("GpiQueryDevice()",TRUE,"GreGetPairKerningTableExh");
           return;
        }
        if (EzQueryFonts(hPS,hWndClient,&FontCount,&FontStorage)==FALSE) {
           BADERROR("GpiQueryDevice()",TRUE,"GreGetPairKerningTableExh");
           return;
        }
        ptl.x=(LONG)(delta_x/2);
        ptl.y=delta_y;
        MapPt(ptl);
        if (GreQueryFontAttributes(hDC,(LONG)sizeof(FONTMETRICS),&fm)==FALSE) {
           BADERROR("GreQueryFontAttributes()",TRUE,"GreGetPairKerningTableExh");
           return;
        }
        yChar=(LONG)(fm.lMaxBaselineExt);
        Inc=(LONG)(FontCount/21L);
        Inc = (Inc==0L) ? (1L) : (Inc);
        for (Index=0; Index< FontCount ; Index+=Inc){
             if (EzCreateLogicalFonts(hPS,hWndClient,USER_LCID,
                                      FontStorage[Index],afsSel[Index%4])){
                if (GpiSetCharSet(hPS,USER_LCID)==FALSE) {
                }
                if (GreQueryFontAttributes(hDC,(LONG)sizeof(FONTMETRICS),&fm)==FALSE) {
                    BADERROR("GreQueryFontAttributes()",TRUE,"GreGetPairKerningTableExh");
                    return;
                }
                ActualKernCount=(LONG)fm.sKerningPairs;
                TempKernCount=(ActualKernCount>20L) ? (20L) : ActualKernCount;
                if ((ExpectedKernCount=GreGetPairKerningTable(hDC,TempKernCount,
                                 aKern))==GPI_ALTERROR) {
                   BADERROR("GreGetPairKerningTable()",TRUE,"GreGetPairKerningTableExh");
                   return;
                }
                if (GpiSetCharSet(hPS,LCID_DEFAULT)==FALSE) {
                }
                if (GpiDeleteSetId(hPS,USER_LCID)==FALSE) {
                }
                if (ActualKernCount > 0L){
                   strcpy(String1,fm.szFacename);
                   strcpy(String2,"KerningPairAmount:");
                   for (Index1=0;Index1<ExpectedKernCount;Index1++){
                        sprintf(&String1[32+Index1*5],"%c%c   ",(CHAR)aKern[Index1].sFirstChar,
                               (CHAR)aKern[Index1].sSecondChar);
                        sprintf(&String2[32+Index1*5],"%+3d ",aKern[Index1].lKerningAmount);
                   }
                   if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                                      (LONG)strlen(String2),String2,
                                       NULL,NULL)==GPI_ERROR) {
                     BADERROR("GreCharStringPos()",TRUE,"GreGetPairKerningTableExh");
                     return;
                   }
                   ptl.y+=(LONG)(3L*yChar/2L);
                   if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                                      (LONG)strlen(String1),String1,
                                       NULL,NULL)==GPI_ERROR) {
                      BADERROR("GreCharStringPos()",TRUE,"GreGetPairKerningTableExh");
                      return;
                   }
                   ptl.y+=(LONG)(3L*yChar/2L);
                }
                else {
                     sprintf(ErrorString,"%s:Kerning Not Supported. Actual #=%ld Expected #=%ld ",
                                          fm.szFacename,ActualKernCount,ExpectedKernCount);
                     if(GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,
                                        (LONG)strlen(ErrorString),ErrorString,
                                         NULL,NULL)==GPI_ERROR) {
                       BADERROR("GreCharStringPos()",TRUE,"GreGetPairKerningTableExh");
                       return;
                     }
                     ptl.y+=(LONG)(3L*yChar/2L);
                }
             }
        }
        free(FontStorage);
        /********************************************************************
        /*              Return the proper log messages for Non-Test Gre's.
        /********************************************************************/

        if (!SetupCount)
            CWRITELOG(L_HDR,0,
"$GreGetPairKerningTableExh()                                    -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", SetupCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreGetPairKerningTableExh()                                    -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ () GreGetPairKerningTableExh() Test - Exit Point\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreCharStringRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreCharString(). The syntax used to call the Gre is :
 *
 *           (LONG) GreCharString(
 *                       HDC               hdc,
 *                       LONG              cChars,
 *                       PCHR              pchString,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *
 *   GreCharString() Draws a character string at the current (x,y) position.
 *                   Upon completion, the current (x,y) postion is the start
 *                   point for the character cell immediatly after the last
 *                   character in the string.
 *
 *   Errors that are returned by this function are :
 *
 *   PMERR_ALREADY_IN_PATH
 *   PMERR_BASE_ERROR
 *   PMERR_BITMAP_NOT_SELECTED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_EXCEEDS_MAX_SEG_LENGTH
 *   PMERR_FONT_AND_MODE_MISMATCH
 *   PMERR_HDC_BUSY
 *   PMERR_HRGN_BUSY
 *   PMERR_HUGE_FONTS_NOT_SUPPORTED
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_BACKGROUND_COL_ATTR
 *   PMERR_INV_BACKGROUND_MIX_ATTR
 *   PMERR_INV_CHAR_ANGLE_ATTR
 *   PMERR_INV_CHAR_MODE_ATTR
 *   PMERR_INV_CHAR_SHEAR_ATTR
 *   PMERR_INV_CHAR_DIRECTION_ATTR
 *   PMERR_INV_SET_CHAR_ATTR
 *   PMERR_INV_CLIP_PATH_OPTIONS
 *   PMERR_INV_CODEPAGE
 *   PMERR_INV_COLOR_ATTR
 *   PMERR_INV_COLOR_DATA
 *   PMERR_INV_COLOR_INDEX
 *   PMERR_INV_COLOR_OPTIONS
 *   PMERR_INV_COLOR_INDEX
 *   PMERR_INV_COORD_SPACE
 *   PMERR_INV_COORDINATE
 *   PMERR_INV_DC_TYPE
 *   PMERR_INV_END_PATH_OPTIONS
 *   PMERR_INV_FILL_PATH_OPTIONS
 *   PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *   PMERR_INV_HDC
 *   PMERR_INV_HRGN
 *   PMERR_INV_ID
 *   PMERR_INV_IN_AREA
 *   PMERR_INV_IN_PATH
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_LINE_WIDTH_ATTR
 *   PMERR_INV_LINE_TYPE_ATTR
 *   PMERR_INV_LINE_JOIN_ATTR
 *   PMERR_INV_LINE_END_ATTR
 *   PMERR_INV_MARKER_SET_ATTR
 *   PMERR_INV_MARKER_SYMBOL_ATTR
 *   PMERR_INV_MATRIX_ELEMENT
 *   PMERR_INV_MIX_ATTR
 *   PMERR_INV_PATH_ID
 *   PMERR_INV_PATTREN_REF_PT_ATTR
 *   PMERR_INV_PATTERN_SET_FONT
 *   PMERR_INV_PATTERN_SET_ATTR
 *   PMERR_INV_PICK_APERTURE_POSN
 *   PMERR_INV_PRIMITIVE_TYPE
 *   PMERR_INV_RECT
 *   PMERR_INV_REGION_CONTROL
 *   PMERR_INV_SETID
 *   PMERR_INV_TRANSFORM_TYPE
 *   PMERR_NOT_IN_PATH
 *   PMERR_PATH_LIMIT_EXCEEDED
 *   PMERR_PATH_UNKNOWN
 *   PMERR_REGION_IS_CLIP_REGION
 *   PMERR_UNSUPPORTED_ATTR
 *   PMERR_UNSUPPORTED_ATTR_VALUE
 *
 * System Requirements: 6.4.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               12/14/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 *               12/16/90, pdvt ,Mike Abuzant
 *                  -Test Case Implemetation
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *   NONE
 *
 *  Return Codes Successfully Tested:
 *  PMERR_INV_HDC
 *  PMERR_FONT_AND_MODE_MISMATCH
 *
 *  Return Codes Not Tested Yet:
 *  PMERR_HDC_BUSY
 *  PMERR_HUGE_FONTS_NOT_SUPPORTED
 *
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * //setup conditions to test   PMERR_HDC_BUSY
 * //setup conditions to test   PMERR_INV_HDC
 * //setup conditions to test   PMERR_FONT_AND_MODE_MISMATCH
 * //setup conditions to test   PMERR_HUGE_FONTS_NOT_SUPPORTED
 *  //for each return code, call the Gre:
 *  lSuccess = GreCharString(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *\\end
 ***************************************************************************/
VOID APIENTRY GreCharStringRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   POINTL ptl;                          //Screen coordinate structure
   HDC    hDC;                          //Handle printer device context.
   LONG             ErrorCount=0L;
   LONG             SetupCount=0L;
   LONG             lSuccess;
   PFONTSELECTION   FontStorage=(PFONTSELECTION)NULL;
   LONG        FontCount=0L;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        set_delta(hPS);
        CWRITELOG(L_LOTRACE,0,"+ee+  GreCharStringRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCharString() PMERR_INV_HDC Test - Entry Point\n");
        // Pass an invalid HDC ,say hDC=NULL ,to the function expecting an error
        hDC=(HDC)NULL;
        ptl.x=delta_x;
        ptl.y=(LONG)(pg_size_y/2)*delta_y;
        MapPt(ptl);
        if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
        }
        lSuccess=GreCharString(hDC,7L,"Testing");
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ",
                               SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ GreCharString()  PMERR_INV_HDC Test - Exit Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_HDC_BUSY error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCharString() PMERR_HDC_BUSY Test - Entry Point\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
            BADERROR("GpiQueryDevice()",TRUE,"GreCharStringRc");
            return;
        }
        strcpy(ErrorString,"This Message Will Not Be Seen When Testing"
                           " PMERR_HDC_BUSY  Succeeds ");
        if (MAKEPSBUSY(hPS))
           CWRITELOG(L_FAIL,0,"MAKEPSBUSY failed.\n");
        lSuccess=GreCharString(hDC,(LONG)strlen(ErrorString),ErrorString);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_HDC_BUSY ",
                               SEVERITY_ERROR,PMERR_HDC_BUSY);
        FREEPS;
        CWRITELOG(L_LOTRACE,0,"+ix+ GreCharString()  PMERR_HDC_BUSY Test - Exit Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_FONT_AND_MODE_MISMATCH
         *  error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCharString() PMERR_FONT_AND_MODE_MISMATCH Test - Entry Point\n");
        ptl.x=delta_x;
        ptl.y=(LONG)(pg_size_y/3)*delta_y;
        MapPt(ptl);
        if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
        }
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)   {
            BADERROR("GpiQueryDevice()",TRUE,"GreCharStringRc");
            return;
        }
        if (EzQueryFonts(hPS,hWndClient,&FontCount,&FontStorage)==FALSE) {
            BADERROR("GpiQueryDevice()",TRUE,"GreCharStringRc");
            return;
        }
        EzCreateLogicalFonts(hPS,hWndClient,USER_LCID, FontStorage[0],afsSel[2]);
        if (GpiSetCharSet(hPS,USER_LCID)==FALSE)           {
        }
        if (GpiSetCharMode(hPS,CM_MODE3)==FALSE) {
        }
        strcpy(ErrorString,"Testing PMERR_FONT_AND_MODE_MISMATCH Error");
        lSuccess=GreCharString(hDC,(LONG)strlen(ErrorString),ErrorString);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_FONT_AND_MODE_MISMATCH ",
                                       SEVERITY_ERROR,PMERR_FONT_AND_MODE_MISMATCH);
        CWRITELOG(L_LOTRACE,0,"+ix+ GreCharString()  PMERR_FONT_AND_MODE_MISMATCH Test - Exit Point\n");
        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/
        free(FontStorage);

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        if (!ErrorCount)
            CWRITELOG(L_HDR,0,
"$GreCharStringRc()                                              -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", ErrorCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreCharStringRc()                                              -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreCharStringRc()  Test - Exit Point\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreCharStringPosRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreCharStringPos(). The syntax used to call the Gre is :
 *
 *           (LONG) GreCharStringPos(
 *                       HDC               hdc,
 *                       PPOINTL           pptStart,
 *                       PRECTL            prcRect,
 *                       ULONG             flOptions,
 *                       LONG              cChars,
 *                       PCHR              pchString,
 *                       PLONG             padx,
 *                       PCSP_INFO         pAttr,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *
 *   GreCharStringPos() Draws a character string at the current (x,y) position
 *                      or from a position specified.
 *
 *   Errors that are returned by this function are :
 *
 *   PMERR_ALREADY_IN_PATH
 *   PMERR_BASE_ERROR
 *   PMERR_ALREADY_IN_AREA
 *   PMERR_INV_PATH_ID
 *   PMERR_INV_END_PATH_OPTIONS
 *   PMERR_NOT_IN_PATH
 *   PMERR_PATH_LIMIT_EXCEEDED
 *   PMERR_INV_FILL_PATH_OPTIONS
 *   PMERR_INV_PATH
 *   PMERR_PATH_UNKNOWN
 *   PMERR_INV_CLIP_PATH_OPTIONS
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_BITMAP_NOT_SELECTED
 *   PMERR_INV_MATRIX_ELEMENT
 *   PMERR_INV_SETID
 *   PMERR_INV_SET_CHAR_ATTR
 *   PMERR_INV_MARKER_SET_ATTR
 *   PMERR_INV_MARKER_SYMBOL_ATTR
 *   PMERR_UNSUPPORTED_ATTR
 *   PMERR_INV_MIX_ATTR
 *   PMERR_INV_BACKGROUND_MIX_ATTR
 *   PMERR_INV_PATTREN_REF_PT_ATTR
 *   PMERR_INV_LINE_WIDTH_ATTR
 *   PMERR_INV_GEOM_LINE_WIDTH_ATTR
 *   PMERR_INV_LINE_TYPE_ATTR
 *   PMERR_INV_LINE_JOIN_ATTR
 *   PMERR_INV_LINE_END_ATTR
 *   PMERR_INV_CHAR_MODE_ATTR
 *   PMERR_INV_CHAR_SHEAR_ATTR
 *   PMERR_INV_CHAR_DIRECTION_ATTR
 *   PMERR_UNSUPPORTED_ATTR_VALUE
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_CODEPAGE
 *   PMERR_EXCEEDS_MAX_SEG_LENGTH
 *   PMERR_INV_RECT
 *   PMERR_INV_COORDINATE
 *   PMERR_INV_HDC
 *   PMERR_INV_HRGN
 *   PMERR_HRGN_BUSY
 *   PMERR_INV_CHAR_ANGLE_ATTR
 *   PMERR_INV_TRANSFORM_TYPE
 *   PMERR_REGION_IS_CLIP_REGION
 *   PMERR_INV_REGION_CONTROL
 *   PMERR_INV_DC_TYPE
 *   PMERR_INV_CHAR_POS_OPTIONS
 *   PMERR_FONT_AND_MODE_MISMATCH
 *   PMERR_INV_COORD_SPACE
 *   PMERR_INV_ID
 *   PMERR_INV_COLOR_DATA
 *   PMERR_INV_COLOR_INDEX
 *   PMERR_INV_PICK_APERTURE_POSN
 *   PMERR_INV_PATTERN_SET_FONT
 *   PMERR_HUGE_FONTS_NOT_SUPPORTED
 *   PMERR_INV_COLOR_ATTR
 *   PMERR_INV_BACKGROUND_COL_ATTR
 *
 * System Requirements: 6.4.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               12/14/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 *               12/16/90, pdvt ,Mike Abuzant
 *                  -Test Case Implemetation
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *   NONE
 *
 *  Return Codes Successfully Tested:
 *
 *      PMERR_INV_HDC
 *      PMERR_FONT_AND_MODE_MISMATCH
 *
 *  Return Codes Not Tested Yet:
 *
 *   PMERR_HDC_BUSY
 *   PMERR_HUGE_FONTS_NOT_SUPPORTED
 *
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 * //setup conditions to test   PMERR_HDC_BUSY
 * //setup conditions to test   PMERR_INV_HDC
 * //setup conditions to test   PMERR_FONT_AND_MODE_MISMATCH
 * //setup conditions to test   PMERR_HUGE_FONTS_NOT_SUPPORTED
 *  //for each return code, call the Gre:
 *  lSuccess = GreCharStringPos(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *\\end
 ***************************************************************************/
VOID APIENTRY GreCharStringPosRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   POINTL           ptl;                    //Screen coordinate structure
   HDC              hDC;                   //Handle printer device context.
   LONG             ErrorCount=0L;
   LONG             SetupCount=0L;
   LONG             lSuccess;
   PFONTSELECTION   FontStorage=(PFONTSELECTION)NULL;
   LONG             FontCount=0L;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        set_delta(hPS);
        CWRITELOG(L_LOTRACE,0,"+ee+  GreCharStringPosRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCharStringPos() PMERR_INV_HDC Test - Entry Point\n");
        // Pass an invalid HDC ,say hDc=NULL ,to the function expecting an error
        hDC=(HDC)NULL;
        ptl.x=delta_x;
        ptl.y=(LONG)(pg_size_y/2L)*delta_y;
        MapPt(ptl);
        if ( GpiMove(hPS,&ptl)==GPI_ERROR) {
        }
        lSuccess=GreCharStringPos(hDC,NULL,NULL,NULL,7L,"Testing",NULL,NULL);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ",
                                       SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ GreCharStringPos()  PMERR_INV_HDC Test - Exit Point\n");
        /*****************************************************************
        * The following Code will check for the PMERR_HDC_BUSY error Code
        *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCharStringPos() PMERR_HDC_BUSY Test - Entry Point\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
           BADERROR("GpiQueryDevice()",TRUE,"GreCharStringPosRc");
           return;
        }
        strcpy(ErrorString,"This Message Will Not Be Seen When Testing"
                           " PMERR_HDC_BUSY  Succeeds ");
        if (MAKEPSBUSY(hPS))
           CWRITELOG(L_FAIL,0,"MAKEPSBUSY failed.\n");
        lSuccess=GreCharStringPos(hDC,NULL,NULL,NULL,strlen(ErrorString),
                                  ErrorString,NULL,NULL);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_HDC_BUSY ",
                                       SEVERITY_ERROR,PMERR_HDC_BUSY);
        FREEPS;
        CWRITELOG(L_LOTRACE,0,"+ix+ MAKEPSBUSY  GreCharStringPos()  PMERR_HDC_BUSY Test - Exit Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_FONT_AND_MODE_MISMATCH
         *  error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreCharStringPos() PMERR_FONT_AND_MODE_MISMATCH Test - Entry Point\n");
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)   {
           BADERROR("GpiQueryDevice()",TRUE,"GreCharStringPosRc");
           return;
        }
        if (EzQueryFonts(hPS,hWndClient,&FontCount,&FontStorage)==FALSE) {
           BADERROR("GpiQueryDevice()",TRUE,"GreCharStringPosRc");
           return;
        }
        EzCreateLogicalFonts(hPS,hWndClient,USER_LCID, FontStorage[0],afsSel[2]);
        if (GpiSetCharSet(hPS,USER_LCID)==FALSE)           {
        }
        if (GpiSetCharMode(hPS,CM_MODE3)==FALSE) {
        }
        strcpy(ErrorString,"Testing PMERR_FONT_AND_MODE_MISMATCH Error");
        lSuccess=GreCharStringPos(hDC,&ptl,NULL,CHS_START_XY,(LONG)strlen(ErrorString),
                                  ErrorString,NULL,NULL);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_FONT_AND_MODE_MISMATCH ",
                                        SEVERITY_ERROR,PMERR_FONT_AND_MODE_MISMATCH);
        CWRITELOG(L_LOTRACE,0,"+ix+ GreCharStringPos()  PMERR_FONT_AND_MODE_MISMATCH Test - Exit Point\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/
        if (!ErrorCount)
           CWRITELOG(L_HDR,0,
"$GreCharStringPosRc()                                           -    PASS.\n");
        else{
             sprintf(ErrorString,"ErrorCountERRORCOUNT = %ld.\n", ErrorCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreCharStringPosRc()                                           -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreCharStringPosRc()  Test - Exit Point\n");
   }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryTextBoxRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreQueryTextBox(). The syntax used to call the Gre is :
 *
 *           (LONG) GreQueryTextBox(
 *                       HDC               hdc,
 *                       LONG              cChars,
 *                       PCHR              pchString,
 *                       ULONG             cptPosition,
 *                       PPOINTL           paptPosition,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *
 *   GreQueryTextBox() Processes a character string as if it were Drawn.
 *       The handling function stores the coordinates of the current text
 *       box, relative to the current (x,y) position,as an array at
 *       the location indicated by paptPostion. The first four coordinates
 *       pairs idintify the bounding parallelogram for the given character
 *       string. the fifth coordinate pair is the (x,y) position of the
 *       starting point for the next character position after the string
 *       is drawn.
 *   Errors that are returned by this function are :
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_INV_MATRIX_ELEMENT
 *   PMERR_INV_SETID
 *   PMERR_INV_CHAR_MODE_ATTR
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_CODEPAGE
 *   PMERR_EXCEEDS_MAX_SEG_LENGTH
 *   PMERR_INV_HDC
 *   PMERR_INV_CHAR_ANGLE_ATTR
 *   PMERR_INV_TRANSFORM_TYPE
 *   PMERR_INV_COORD_SPACE
 *
 * System Requirements: 6.4.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               12/14/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 *               12/17/90, pdvt ,Mike Abuzant
 *                  -Test Case Implemetation
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *   NONE
 *
 *  Return Codes Successfully Tested:
 *        PMERR_HDC_BUSY
 *
 *  Return Codes Not Tested Yet:
 *
 *        PMERR_INV_HDC
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 * //setup conditions to test   PMERR_HDC_BUSY
 * //setup conditions to test   PMERR_INV_HDC
 *  //for each return code, call the Gre:
 *  lSuccess = GreQueryTextBox(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryTextBoxRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC    hDC;                          //Handle printer device context.
   LONG             ErrorCount=0L;
   LONG             SetupCount=0L;
   BOOL             fSuccess;
   POINTL           aptl[5];
  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+  GreQueryTextBoxRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryTextBox() PMERR_INV_HDC Test - Entry Point\n");
        // Pass an invalid HDC ,say hDc=NULL ,to the function expecting an error
        hDC=(HDC)NULL;
        fSuccess=GreQueryTextBox(hDC,7l,"Testing",5L,aptl);
        ErrorCount += CHECKRETURNERROR(fSuccess, "PMERR_INV_HDC ",
                                       SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryTextBox()  PMERR_INV_HDC Test - Exit Point\n");
        /*****************************************************************
        * The following Code will check for the PMERR_HDC_BUSY error Code
        *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryTextBox() PMERR_HDC_BUSY Test - Entry Point\n");
/*****
 *****
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
           BADERROR("GpiQueryDevice()",TRUE,"GreCharStringPosRc");
           return;
        }
        strcpy(ErrorString,"This Message Is Used For Testing"
                           " PMERR_HDC_BUSY ");
        fSuccess=GreQueryTextBox(hDC,strlen(ErrorString),ErrorString,5L,aptl);
        ErrorCount += CHECKRETURNERROR(fSuccess, "PMERR_HDC_BUSY ",
                                       SEVERITY_ERROR,PMERR_HDC_BUSY);
 *****
  ****/
        CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryTextBox()  PMERR_HDC_BUSY Test - Exit Point\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/

        if (!ErrorCount)
           CWRITELOG(L_HDR,0,
"$GreQueryTextBoxRc()                                            -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", ErrorCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreQueryTextBoxRc()                                            -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryTextBoxRc()  Test - Exit Point\n");
    }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryCharPositionsRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreQueryCharPositions(). The syntax used to call the Gre is :
 *
 *           (LONG) GreQueryCharPositions(
 *                       HDC               hdc,
 *                       PPOINTL           pptStart,
 *                       PRECTL            prcRect,
 *                       ULONG             flOptions,
 *                       LONG              cChars,
 *                       PCHR              pchString,
 *                       PLONG             padx,
 *                       PPOINTL           paptXY,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *
 *   GreQueryCharPositions() Stores at the location addresses by paptXY, an array
 *           of world coordinates identifying the start points at which
 *           device is to place each character of a given string. This function
 *           is required for the managements of the device fonts,in CM_MODE2
 *           only. When the presentation driver has no device fonts, the
 *           handling function must post PMERR_DEV_FUNC_NOT_INSTALLED.
 *
 *   Errors that are returned by this function are :
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_INV_MATRIX_ELEMENT
 *   PMERR_INV_SETID
 *   PMERR_INV_CHAR_MODE_ATTR
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_CODEPAGE
 *   PMERR_EXCEEDS_MAX_SEG_LENGTH
 *   PMERR_INV_HDC
 *   PMERR_INV_CHAR_ANGLE_ATTR
 *   PMERR_INV_TRANSFORM_TYPE
 *   PMERR_INV_COORD_SPACE
 *   PMERR_INV_CHAR_POS_OPTIONS
 *
 * System Requirements: 6.4.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               12/14/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *   NONE
 *
 *  Return Codes Successfully Tested:
 *
 *      PMERR_INV_HDC
 *
 *  Return Codes Not Tested Yet:
 *
 *      PMERR_HDC_BUSY
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 * //setup conditions to test   PMERR_HDC_BUSY
 * //setup conditions to test   PMERR_INV_HDC
 *  //for each return code, call the Gre:
 *  lSuccess = GreQueryCharPositions(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryCharPositionsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   POINTL ptl;                          //Screen coordinate structure
   HDC    hDC;                          //Handle printer device context.
   LONG             ErrorCount=0L;
   LONG             SetupCount=0L;
   LONG             lSuccess;
   POINTL           aptl[100];

  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS , NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+  GreQueryCharPositionsRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryCharPositions() PMERR_INV_HDC Test - Entry Point\n");
        // Pass an invalid HDC ,say hDc=NULL ,to the function expecting an error
        hDC=(HDC)NULL;
        lSuccess=GreQueryCharPositions(hDC,NULL,NULL,7L,"Testing",NULL,aptl);
        ErrorCount+= CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ", SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryCharPositions()  PMERR_INV_HDC Test - Exit Point\n");
        /*****************************************************************
        * The following Code will check for the PMERR_HDC_BUSY error Code
        *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryCharPositions() PMERR_HDC_BUSY Test - Entry Point\n");
/*****
 *****
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)  {
           BADERROR("GpiQueryDevice()",TRUE,"GreQueryCharPositions");
           return;
        }
        strcpy(ErrorString,"This Message Is Used For Testing PMERR_HDC_BUSY ");
        lSuccess=GreQueryCharPositions(hDC,NULL,NULL,(LONG)strlen(ErrorString),
                                       ErrorString,NULL,aptl);
        ErrorCount+=CHECKRETURNERROR(lSuccess, "PMERR_HDC_BUSY ",
                                     SEVERITY_ERROR,PMERR_HDC_BUSY);
 *****
 ****/
        CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryCharPositions()  PMERR_HDC_BUSY Test - Exit Point\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/
        if (!ErrorCount)
            CWRITELOG(L_HDR,0,
"$GreQueryCharPositionsRc()                                      -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", ErrorCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreQueryCharPositionsRc()                                      -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryCharPositionsRc()  Test - Exit Point\n");
     }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreSetCodePageRc()
 *
 * Purpose: This function exhaustively tests for all errors codes returned
 *          by GreSetCodePage(). The syntax used to call the Gre is :
 *
 *          (BOOL) GreSetCodePage(
 *                        (HDC)   hdc,
 *                        (ULONG) lCodePage,
 *                        (ULONG) pInstance,
 *                        (ULONG) lFunction)
 *
 *          GreSetCodePage() sets the current code page for characters written
 *          with default font.  The default is the font that the system uses
 *          when the cbnd.usSet attrubute is 0000H.  When the default font is
 *          not is use, lCodePage is saved until required.  When a DC is
 *          initialized, the codepage is set to the default codepage obtained
 *          from WinQueryProcessCp.
 * Errors that are returned by this function are :
 *
 *   PMERR_INV_IN_AREA
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_CODEPAGE
 *   PMERR_EXCEEDS_MAX_SEG_LENGTH
 *   PMERR_INV_HDC
 *
 * System Requirements: 6.4.3
 *
 * Revision Log:
 *               12/14/90, pdvt ,Darin C. Hafer
 *               -Test Case Design.
 *               12/17/90, pdvt ,Mike Abuzant
 *                  -Test Case Implemetation
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *        NONE
 *
 * Return Codes Successfully Tested:
 *   PMERR_INV_HDC
 *
 * Return Codes Not Tested Yet:
 *    NONE
 *
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 * //setup conditions to test   PMERR_INV_HDC
 *  //for each return code, call the Gre:
 *  lSuccess = GreSetCodePageRc(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *\\end
 ***************************************************************************/
VOID APIENTRY GreSetCodePageRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC    hDC;                          //Handle printer device context.
   LONG             ErrorCount=0L;
   LONG             SetupCount=0L;
   LONG             fSuccess;

  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS , NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+ GreSetCodePageRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+  GreSetCodePage() PMERR_INV_HDC Test - Entry Point\n");
        hDC=(HDC)NULL;
        fSuccess=GreSetCodePage(hDC,437L);
        ErrorCount += CHECKRETURNERROR(fSuccess, "PMERR_INV_HDC ",
                                       SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+  GreSetCodePage()  PMERR_INV_HDC Test - Exit Point\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/
        if (!ErrorCount)
           CWRITELOG(L_HDR,0,
"$GreSetCodePageRc()                                             -    PASS.\n");
        else{
            sprintf(ErrorString,"$ErrorCount = %ld.\n", ErrorCount);
            CWRITELOG(L_HDR,0, ErrorString);
            CWRITELOG(L_HDR,0,
"$GreSetCodePageRc()                                             -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+  GreSetCodePageRc()  Test - Exit Point\n");
  }
}
/***************************************************************************
 *\\ddd
 * Routine Name: GreGetPairKerningTableRc()
 *
 * Purpose: This function exhaustively tests for all errors codes returned
 *          by GreGetPairKerningTable(). The syntax used to call the Gre is :
 *
 *           (LONG) GreGetPairKerningTable(
 *                       HDC               hdc,
 *                       LONG              cKernPairs,
 *                       PKERNPAIRS        pKernPairs,
 *                       ULONG             pInstance,
 *                       ULONG             lFunction)
 *
 *   GreGetPairKerningTable() stores the kerning pairs of the current font to
 *   the buffer addressed by pKernPairs.  The handling routine must transform
 *   all kerning-pair coordinates from device to world coordinates before handling
 *   the information to the calling routine, this can be done using GreConvert.
 *   When it is unable to do this, because the transform matrix is singular, it must
 *   log PMERR_COORDINATE_OVERFLOW.
 *
 * Errors that are returned by this function are :
 *
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_INV_SETID
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_CODEPAGE
 *   PMERR_EXCEEDS_MAX_SEG_LENGTH
 *   PMERR_INV_HDC
 *   PMERR_INV_COORD_SPACE
 *
 * System Requirements:   6.4.3
 *
 * Revision Log:
 *               12/14/90, pdvt , Darin C. Hafer
 *               -Test Case Design.
 *               12/17/90, pdvt ,Mike Abuzant
 *                  -Test Case Implemetation
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *        NONE
 *
 *  Return Codes Successfully Tested:
 *   PMERR_INV_HDC
 *  Return Codes Not Tested Yet:
 *   PMERR_HDC_BUSY
 *
 *  Subroutines Required:
 *
 *  Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 * //setup conditions to test   PMERR_HDC_BUSY
 * //setup conditions to test   PMERR_INV_HDC
 *  //for each return code, call the Gre:
 *  lSuccess = GreGetPairKerningTableRc(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *\\end
 ***************************************************************************/
VOID APIENTRY GreGetPairKerningTableRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC    hDC;                          //Handle printer device context.
   LONG             ErrorCount=0L;
   LONG             SetupCount=0L;
   LONG             lSuccess;
   KERNINGPAIRS     KernPairs;
  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS , NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+  GreGetPairKerningTableRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+  GreGetPairKerningTable() PMERR_INV_HDC Test - Entry Point\n");
        hDC=(HDC)NULL;
        lSuccess=GreGetPairKerningTable(hDC,1L,&KernPairs);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ",
                                       SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ GreGetPairKerningTable()  PMERR_INV_HDC Test - Exit Point\n");
        /*****************************************************************
        * The following Code will check for the PMERR_HDC_BUSY error Code
        *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreGetPairKerningTable() PMERR_HDC_BUSY Test - Entry Point\n");
/*****
 *****
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
           BADERROR("GpiQueryDevice()",TRUE,"GreGetPairKerningTableRc");
           return;
        }
        lSuccess=GreGetPairKerningTable(hDC,1L,&KernPairs);
        ErrorCount+=CHECKRETURNERROR(lSuccess, "PMERR_HDC_BUSY ",
                                     SEVERITY_ERROR,PMERR_HDC_BUSY);
 ****
 *****/
        CWRITELOG(L_LOTRACE,0,"+ix+ GreGetPairKerningTable()  PMERR_HDC_BUSY Test - Exit Point\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/

        if (!ErrorCount)
           CWRITELOG(L_HDR,0,
"$GreGetPairKerningTableRc()                                     -    PASS.\n");
        else{
            sprintf(ErrorString,"$ErrorCount = %ld.\n", ErrorCount);
            CWRITELOG(L_HDR,0, ErrorString);
            CWRITELOG(L_HDR,0,
"$GreGetPairKerningTableRc()                                     -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+  GreGetPairKerningTableRc()  Test - Exit Point\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceQueryFontAttributesRc()
 *
 * Purpose: This function exhaustively tests for all errors codes returned
 *          by GreDeviceQueryFontAttributes(). The syntax used to call the Gre is :
 *
 *          (BOOL) GreDeviceQueryFontAttributes(
 *                           (ULONG)       hdc,
 *                           (ULONG)       cbMerrics,
 *                           (ULONG FAR *) pfmMetrics,
 *                           (ULONG)       pInstance,
 *                           (ULONG)       lFunction)
 *
 *  GreDeviceQueryFontAttributes() returns, at the location addressed by pfmMetrics,
 *  the metrics of the current font.
 *
 * Errors that are returned by this function are :
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_INV_HDC
 *
 * System Requirements: 6.4.3
 *
 * Revision Log:
 *               12/14/90, pdvT, Darin C. Hafer
 *                -Test Case Design.
 *               12/20/90, pdvt ,Mike Abuzant
 *                  -Test Case Implemetation
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *        NONE
 *
 * Return Codes Successfully Tested:
 *  PMERR_INV_HDC
 *
 * Return Codes Not Tested Yet:
 *  None
 *
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 * //setup conditions to test   PMERR_INV_HDC
 *  //for each return code, call the Gre:
 *  lSuccess = GreDeviceQueryFontAttributesRc(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceQueryFontAttributesRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC              hDC;                         //Handle printer device context.
   LONG             ErrorCount=0L;
   BOOL             fSuccess;

  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+  GreDeviceQueryFontAttributesRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+  GreDeviceQueryFontAttributes() PMERR_INV_HDC Test - Entry Point\n");
        // Pass an invalid HDC ,say hDC=NULL ,to the function expecting an error
        hDC=(HDC)NULL;
        fSuccess=GreDeviceQueryFontAttributes(hDC,NULL,NULL);
        ErrorCount += CHECKRETURNERROR(fSuccess, "PMERR_INV_HDC ",
                                       SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+  GreDeviceQueryFontAttributes()  PMERR_INV_HDC Test - Exit Point\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/
        if (!ErrorCount)
           CWRITELOG(L_HDR,0,
"$GreDeviceQueryFontAttributesRc()                               -    PASS.\n");
        else{
             sprintf(ErrorString,"$ERRORCOUNT = %ld.\n", ErrorCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreDeviceQueryFontAttributesRc()                               -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+  GreDeviceQueryFontAttributesRc()  Test - Exit Point\n");
     }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreDeviceQueryFontsRc()
 *
 * Purpose: This function exhaustively tests for all errors codes returned
 *          by GreDeviceQueryFonts(). The syntax used to call the Gre is :
 *
 *          (LONG) GreDeviceQueryFonts(
 *                 (ULONG)       hdc,
 *                 (ULONG)       flOptions,
 *                 (ULONG FAR *) pszFaceName,
 *                 (ULONG FAR *) paMetrics,
 *                 (ULONG)       cMetricLen,
 *                 (ULONG)       pcFontCount,
 *                 (ULONG)       pInstance,
 *                 (ULONG)       lFunction)
 *
 *          GreDeviceQueryFonts() returns details of fonts matching the face
 *          name addressed by pszFaceName.
 *
 * Errors that are returned by this function are :
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_INV_HDC
 *
 * System Requirements:  6.4.3
 *
 * Revision Log:
 *               12/14/90, pdvt ,Darin C. Hafer
 *               -Test Case Design.
 *               12/20/90, pdvt ,Mike Abuzant
 *                  -Test Case Implemetation
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *        NONE
 *
 * Return Codes Successfully Tested:
 *   PMERR_INV_HDC
 *
 * Return Codes Not Tested Yet:
 *   NONE
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 * //setup conditions to test   PMERR_INV_HDC
 *  //for each return code, call the Gre:
 *  lSuccess = GreDeviceQueryFontsRc(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *\\end
 ***************************************************************************/
VOID APIENTRY GreDeviceQueryFontsRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC              hDC;                   //Handle printer device context.
   LONG             ErrorCount=0L;
   LONG             lSuccess;
   ULONG            cFontCount;
  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+  GreDeviceQueryFontsRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+  GreDeviceQueryFonts() PMERR_INV_HDC Test - Entry Point\n");
        hDC=(HDC)NULL;
        cFontCount=0L;
        lSuccess= GreDeviceQueryFonts(hDC,NULL,NULL,NULL,NULL,&cFontCount);
        ErrorCount += CHECKRETURNERROR(lSuccess, "PMERR_INV_HDC ",
                                       SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+  GreDeviceQueryFonts()  PMERR_INV_HDC Test - Exit Point\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/
        if (!ErrorCount)
            CWRITELOG(L_HDR,0,
"$GreDeviceQueryFontsRc()                                        -    PASS.\n");
        else{
             sprintf(ErrorString,"$ERRORCOUNT = %ld.\n", ErrorCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreDeviceQueryFontsRc()                                        -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+  GreDeviceQueryFontsRc()  Test - Exit Point\n");
  }
}
/****************************************************************************
 *\\ddd
 * Routine Name: GreQueryWidthTableRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreQueryWidthTable(). The syntax used to call the Gre is :
 *
 *           (BOOL)   GreQueryWidthTable(
 *                       HDC               hdc,
 *                       LONG              lFirstChar,
 *                       LONG              cWidthTabel,
 *                       PLONG             paWidthTabel,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *
 *   GreQueryWidthTable() Stores at the location addresses by paWidthTable,
 *     an array of world coordinates representing the width-table information
 *     of the currently selected font. The handling routine must use
 *     GreConver() to transform the width-table information from device
 *     to world coordinates.
 *
 *   Errors that are returned by this function are :
 *
 *   PMERR_COORDINATE_OVERFLOW
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_EXCEEDS_MAX_SEG_LENGTH
 *   PMERR_HDC_BUSY
 *   PMERR_INSUFFICIENT_MEMORY
 *   PMERR_INV_CHAR_MODE_ATTR
 *   PMERR_INV_CODEPAGE
 *   PMERR_INV_COORD_SPACE
 *   PMERR_INV_FIRST_CHAR
 *   PMERR_INV_HDC
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_MATRIX_ELEMENT
 *   PMERR_INV_SETID
 *   PMERR_INV_TRANSFORM_TYPE
 *
 * System Requirements: 6.4.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               12/17/90, pdvt ,Mike Abuzant
 *                  -Test Case Design
 *               12/19/90, pdvt ,Mike Abuzant
 *                  -Test Case Implementation.
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *   NONE
 *
 *  Return Codes Successfully Tested:
 *
 *  PMERR_INV_HDC
 *
 *  Return Codes Not Tested Yet:
 *
 *  PMERR_HDC_BUSY
 *
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 * //setup conditions to test   PMERR_HDC_BUSY
 * //setup conditions to test   PMERR_INV_HDC
 *  //for each return code, call the Gre:
 *  lSuccess = GreQueryWidthTable(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *\\end
 ***************************************************************************/
VOID APIENTRY GreQueryWidthTableRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   HDC              hDC;             //Handle current device context.
   LONG             ErrorCount=0L;
   LONG             SetupCount=0L;
   BOOL             fSuccess;
   LONG             FirstChar=(LONG)0;
   LONG             aWidthTable[10];

  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+  GreQueryWidthTableRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryWidthTable() PMERR_INV_HDC Test - Entry Point\n");
        // Pass an invalid HDC ,say hDC=NULL ,to the function expecting an error
        hDC=(HDC)NULL;
        FirstChar=32L;
        fSuccess=GreQueryWidthTable(hDC,FirstChar,2L,aWidthTable);
        ErrorCount += CHECKRETURNERROR(fSuccess, "PMERR_INV_HDC ",
                                       SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryWidthTable()  PMERR_INV_HDC Test - Exit Point\n");
        /*****************************************************************
        * The following Code will check for the PMERR_HDC_BUSY error Code
        *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreQueryWidthTable() PMERR_HDC_BUSY Test - Entry Point\n");
/*****
*****
        if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR) {
           BADERROR("GpiQueryDevice()",TRUE,"GreQueryWidthTableRc");
           return;
        }
        fSuccess=GreQueryWidthTable(hDC,FirstChar,2L,aWidthTable);
        ErrorCount += CHECKRETURNERROR(fSuccess, "PMERR_HDC_BUSY ",
                                       SEVERITY_ERROR,PMERR_HDC_BUSY);
 *****
 *****/
        CWRITELOG(L_LOTRACE,0,"+ix+ GreQueryWidthTable()  PMERR_HDC_BUSY Test - Exit Point\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/

        if (!ErrorCount)
            CWRITELOG(L_HDR,0,
"$GreQueryWidthTableRc()                                         -    PASS.\n");
        else{
             sprintf(ErrorString,"$ErrorCount = %ld.\n", ErrorCount);
             CWRITELOG(L_HDR,0, ErrorString);
             CWRITELOG(L_HDR,0,
"$GreQueryWidthTableRc()                                         -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreQueryWidthTableRc - Exit.\n");
    }
 }
/****************************************************************************
 *\\ddd
 * Routine Name: GreRealizeFontRc()
 *
 * Purpose:  This function exhaustively tests for all errors codes returned
 *           by GreRealizeFont(). The syntax used to call the Gre is :
 *
 *
 *           (ULONG) GreRealizeFont(
 *                       HDC               hdc,
 *                       ULONG             cmdCommand,
 *                       PFATTR            pLogFont,
 *                       (ULONG FAR *)     pFont,
 *                       ULONG             pInstance,
 *                       ULONG             iFunction)
 *
 *   GreRealizeFont()  Requests the drive to Realize or Delete a font.
 *
 *   Errors that are returned by this function are :
 *
 *   PMERR_HDC_BUSY
 *   PMERR_DEV_FUNC_NOT_INSTALLED
 *   PMERR_INV_LENGTH_OR_COUNT
 *   PMERR_INV_HDC
 *   PMERR_INV_COORD_SPACE
 *   PMERR_COORDINATE_OVERFLOW
 *
 * System Requirements: 6.3.3
 *
 * Revision Log:
 *               10/24/90, pdvt, mark richardson
 *               - initial file
 *               12/14/90, pdvt ,Mike Abuzant
 *                  -Test Case Design.
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *   NONE
 *
 *  Return Codes Successfully Tested:
 *
 *    PMERR_INV_HDC
 *
 *  Return Codes Not Tested Yet:
 *
 *    PMERR_HDC_BUSY
 *
 * Subroutines Required:
 *
 * Limitations:
 *
 *\\end
 ***************************************************************************/

/***************************************************************************
 *\\algorithm
 *
 * //setup conditions to test   PMERR_HDC_BUSY
 * //setup conditions to test   PMERR_INV_HDC
 *
 *  //for each return code, call the Gre:
 *  lSuccess = GreRealizeFont(hDC...);
 *  if (lSuccess == GPI_ERROR) {
 *    post error condition
 *    get error code
 *    if (error == PMERR_xxxxx)
 *      log expected error condition
 *    else
 *      log unexpected error condition
 *  }
 *  else
 *    log unexpected successful condition
 *
 *
 *\\end
 ***************************************************************************/

VOID APIENTRY GreRealizeFontRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{

   HDC              hDC;                 //Handle printer device context.
   LONG             ErrorCount=0L;
   LONG             SetupCount=0L;
   ULONG            ulSuccess;

  if (SelectionCall) {
    WinSendMsg (hWndClient,WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
        LOG_SIGNATURE;
        CWRITELOG(L_LOTRACE,0,"+ee+  GreRealizeFontRc()  Test - Entry Point\n");
        /* ***************************************************************
         * The following Code will check for the PMERR_INV_HDC error Code
         *****************************************************************/
        CWRITELOG(L_LOTRACE,0,"+ie+ GreRealizeFont() PMERR_INV_HDC Test - Entry Point\n");
        // Pass an invalid HDC ,say hDC=NULL ,to the function expecting an error
        hDC=(HDC)NULL;
        ulSuccess=GreRealizeFont(hDC,NULL,NULL,NULL);
        ErrorCount += CHECKRETURNERROR(ulSuccess, "PMERR_INV_HDC ",
                                       SEVERITY_ERROR,PMERR_INV_HDC);
        CWRITELOG(L_LOTRACE,0,"+ix+ GreRealizeFont()  PMERR_INV_HDC Test - Exit Point\n");

    // **************************************************
    // TSS...added to kill all output that is generated.*
    // Pass/Fail should be determined by the Log file.  *
    // Changes made on 6/28/91                          *
    DevEscape(GpiQueryDevice(hPS),DEVESC_ABORTDOC,(LONG) NULL,
                                      NULL, NULL,NULL);

        /********************************************************************
        /*              Return the proper log messages for Test Gre's.
        /********************************************************************/

        if (!ErrorCount)
           CWRITELOG(L_HDR,0,
"$GreRealizeFontRc()                                             -    PASS.\n");
        else{
            sprintf(ErrorString,"$ErrorCount = %ld.\n", ErrorCount);
            CWRITELOG(L_HDR,0, ErrorString);
            CWRITELOG(L_HDR,0,
"$GreRealizeFontRc()                                             -    FAIL.\n");
        }
        CWRITELOG(L_LOTRACE,0,"+ex+ GreRealizeFontRc()  Test - Exit Point\n");
  }
}
/***************************************************************************
 *\\ddd
 * Routine Name: EzQueyFont
 *
 * Purpose:  The purpose of this function is to Query and store all the
 *           fonts supported By current device. If none is supported,
 *           The Graphic Engine fonts are retreived.
 *
 * System Requirements: NONE
 *
 * Revision Log: 12/18/90, PDVT, Mike Abuzant
 *               - Function  implementation.
 *
 *               12/24/90, PDVT, Mike Abuzant
 *               - Function Design
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    Count          //Number of font found
 *    FontStorage    // Array of fonts names and match numbers.
 *
 * Outputs:
 *    1. All fonts names and match numbers are returned in the dynamically
 *      allocated array  'FontStorage'.
 *    2. The number of fonts found are returned in 'Count'.
 *
 * Subroutines Required:
 *  None
 *
 * Limitations:
 *
 *
 *\\end
 ***************************************************************************/

/***************************************************************************
 *\\algorithm
 *
 *    // Get the Device Context associated with Presentation Spac.
 *      hDC=GpiQueryDevice(hPS) ....
 *
 *    // Get the current device Character horizontal resl
 *       DevQueryCaps(hDC,CAPS_HORIZONTAL_FONT_RES,1L,&Hori_res)
 *
 *    // Get the current device Character vertical  resl.
 *        DevQueryCaps(hDC,CAPS_VERTICAL_FONT_RES,1L,&Vert_res)
 *
 *
 *     Setup condition to query number of fonts supported by current device
 *     by passing a value of zero in the parameter 'Requested'.
 *
 *       NumberOfDeviceFonts=GreDeviceQueryFonts(hDC,&(Rerquested=0),...
 *
 *     if the current device does not support any fonts,then query
 *     the number of fonts that are supported by the Graphic Engin.
 *
 *         NumberOfDeviceFonts=GreQueryFonts(hDC,&(Rerquested=0),...
 *
 *     Allocate enough storage hold the all the metrics of all fonts.
 *     and return with an error if could not allocate storage.
 *
 *         pfm=malloc(NumberOfDeviceFonts*sizeof(fm));
 *
 *     Query all font metrics supported by current device, or
 *     query all font metrics supported by Graphic Engine if
 *     current device support no fonts.
 *
 *        if ( No device fonts are supported ) Do
 *
 *           GreQueryFonts(hDC,&NumberOfDeviceFonts,...
 *        else ( Query Device fonts)
 *
 *           GreDeviceQueryFonts(hDC,&NumberOfDeviceFonts,...
 *        ENDIF
 *
 *
 *     //Find all fonts that are displayable at the current device
 *     //resolution. (fonts that look good at this device resolution).
 *
 *     FOR (All the fonts queried ) Do
 *
 *       IF ( a font horizontal and vertical resolution are the equal
 *            to the current device horizontal and vertical character
 *            resolution )
 *
 *          //Allocate storage to new font
 *
 *       incroment the font counter.
 *       store the font facename and match number in the array 'FontStorage'
 *
 *       ENDIF
 *
 *     ENDDO
 *
 *\\end
 ***************************************************************************/
BOOL EzQueryFonts(HPS hPS, HWND hWndClient, PLONG Count ,PPFONTSELECTION  FontStorage)
{
  HDC           hDC;
  FONTMETRICS  *pfm;
  void        huge *pmem;
  LONG          Hori_res,
                Vert_res,
                Requested=0L,
                Number;
  SHORT         Index;
  BOOL          DeviceFont=FALSE;
  FONTMETRICS   fm;

   //Get device context associated of  Presentation Space.
   if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
       BADERROR("GpiQueryDevice()",TRUE,"EzQueryFonts");
       return FALSE;
   }
   //Get the device horizontal character resolution.
   if (DevQueryCaps(hDC,CAPS_HORIZONTAL_FONT_RES,1L,&Hori_res)==FALSE){
        BADERROR("DevQueryCaps()",TRUE,"EzQueryFonts");
        return FALSE;
   }
   //Get the device vertical character resolution.
   if (DevQueryCaps(hDC,CAPS_VERTICAL_FONT_RES,1L,&Vert_res)==FALSE){
        BADERROR("DevQueryCaps()",TRUE,"EzQueryFonts");
        return FALSE;
   }
   //Get Total number of fonts supported by the current device
   Requested=0L;

   if ((Number=GreDeviceQueryFonts(hDC,QF_PUBLIC,NULL,
                 &fm,(LONG)sizeof(fm),&Requested))==GPI_ALTERROR){
       BADERROR("GreDeviceQueryFonts()",TRUE,"EzQueryFonts");
       return(FALSE);
   }

   if (Number==0L){   //No fonts are supported by current device
                      //Get total number of fonts in supported by
                      //Graphic Engine.
      CWRITELOG(L_HDR,0,"Device Has No Built in Fonts.\n");
      Requested=0L;
      if ((Number=GreQueryFonts(hDC,QF_PUBLIC,NULL,
                        NULL,NULL,&Requested))==GPI_ALTERROR){
            BADERROR("GreDeviceQueryFonts()",TRUE,"EzQueryFonts");
            return FALSE;
      }
   }
   else {                   //Device Supports some fonts

      CWRITELOG(L_HDR,0,"Retreiving Device  Built in Fonts.\n");
      DeviceFont=TRUE;
   }

   //TSS...Limit Number to 314
//   if( Number > 314 ) Number = 314;

//   if ((pfm=(PFONTMETRICS)malloc((SHORT)Number*sizeof(FONTMETRICS)))==NULL){
//       BADERROR("malloc()",TRUE,"EzQueryFonts");
//       return FALSE;
//   }



    if ((  pfm=(PFONTMETRICS)malloc ((SHORT) Number *sizeof(FONTMETRICS)))
                                                       ==NULLHANDLE )  {

         BADERROR("malloc()",TRUE,"GreDeviceQueryFontsExh");
         return FALSE;
    }


   if (DeviceFont){   // Device supports fonts. Get  their metrics

      if ((GreDeviceQueryFonts(hDC,QF_PUBLIC,NULL,
                     pfm,(LONG)sizeof(FONTMETRICS),&Number))==GPI_ALTERROR){
           BADERROR("GreDeviceQueryFonts()",TRUE,"EzQueryFonts");
           free(pfm); // 16KQ
           return(FALSE);
      }
   }

   else{            //Device supports no fonts, Get Graphic Engine fonts
                    //instead.
     if ((GreQueryFonts(hDC,QF_PUBLIC,NULL,
                     pfm,(LONG)sizeof(FONTMETRICS),&Number))==GPI_ALTERROR){
          BADERROR("GreDeviceQueryFonts()",TRUE,"EzQueryFonts");
           free(pfm); // 16KQ
          return(FALSE);
     }
   }

  *Count=0;

  *FontStorage=(PFONTSELECTION)NULL;

  for(Index = 0; Index< (SHORT)Number ; Index++){


        if ((*FontStorage)==NULL){
           if ((*FontStorage=(PFONTSELECTION)malloc(1*sizeof(FontSelection)))==NULL){
              BADERROR("malloc()",TRUE,"EzQueryFonts");
           free(pfm); // 16KQ
              return FALSE;
           }
        }
        else{
           if ((*FontStorage=(PFONTSELECTION)realloc(*FontStorage,
                                        (USHORT)(*Count+1)*sizeof(FontSelection)))==NULL){
              BADERROR("malloc()",TRUE,"EzQueryFonts");
           free(pfm); // 16KQ
              return FALSE;
           }
        }


//TSS...Use Index-1 if on segment boarder...9/25/91
    if( (Index == 315) || (Index == 630) || (Index == 945) || (Index == 1260) ) {
        strcpy((*FontStorage+*Count)->Fontname,pfm[Index-1].szFacename);
        (*FontStorage+*Count)->lMatch=pfm[Index-1].lMatch ;
       }
    else {
        strcpy((*FontStorage+*Count)->Fontname,pfm[Index].szFacename);
        (*FontStorage+*Count)->lMatch=pfm[Index].lMatch ;
       }

        (*Count)++;
  }

           free(pfm); // 16KQ
  return TRUE;
}
/***************************************************************************
 *\\ddd
 * Routine Name: EzCreateLogicalFont()
 *
 * Purpose:  The purpose of this function is to realize a logical font
 *          of a known face name and font matching number.
 *
 * System Requirements: NONE
 *
 * Revision Log: 12/18/90, PDVT, Mike Abuzant
 *               - Function   implementation.
 *
 *               12/24/90, PDVT, Mike Abuzant
 *               - Function Design
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    Lcid           //Local indentifier
 *    fsSelection    // Kind of font needed (Italic ,Bold ...)
 *    FontStorage    // Font name and match number.
 *
 *
 * Outputs:
 *    A new logical font associated with a local identifier (lcid).
 *
 * Subroutines Required:
 *  None
 *
 * Limitations:
 *     None
 *
 *\\end
 ***************************************************************************/

/***************************************************************************
 *\\algorithm
 *
 *    //Get the Device Context associated with Presentation Spac.
 *      hDC=GpiQueryDevice(hPS) ....
 *
 *    Store the logical font face name, match number,and selection type
 *    into the structure 'fat' that will be passed as a parameter to the
 *    Gre function GreCreateLogicalFont().
 *
 *    //Create the desired logical font
 *
 *      GreCreateLogicalFont(hDC,lcid,pchName,&fat);
 *
 *     return TRUE if the logical font has been created or
 *            FALSE otherwise.
 *\\end
 ***************************************************************************/

LONG EzCreateLogicalFonts(HPS hPS, HWND hWndClient,LONG lcid ,
                          FontSelection FontStorage,USHORT fsSelection)
{

  static           FATTRS   Atter;
  STR8             chName;
  HDC              hDC;
  LONG             ReturnCode=FALSE;

  if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR)
       BADERROR("GpiQueryDevice()",TRUE,"EzCreateFonts");
  else {
    Atter.usRecordLength=sizeof(Atter);
    Atter.fsSelection=fsSelection;
    Atter.lMatch=FontStorage.lMatch;
    Atter.fsType=FATTR_TYPE_KERNING;
    strcpy(Atter.szFacename,FontStorage.Fontname);
    strcpy(chName,"ABUZANT");

    if ((ReturnCode=GreCreateLogicalFont(hDC,lcid,chName,&Atter))==GPI_ERROR)
         BADERROR("GpiCreateLogicalFont()",TRUE,"EzCreateFonts");

  }
  return ReturnCode;
}


