/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/**************************************************************************
 *
 * SOURCE FILE NAME = gre_grid.c
 *
 * DESCRIPTIVE NAME = Standard Grid function for use in GRE precision tests.
 *
 *
 *      DISCLAIMER OF WARRANTIES.  The following enclosed code is
 *      sample code created by IBM Corporation. This sample code is not
 *      part of any standard or IBM product and is provided to you solely
 *      for  the purpose of assisting you in the development of your
 *      applications.  The code is provided "AS IS", without
 *      warranty of any kind.  IBM shall not be liable for any damages
 *      arising out of your use of the sample code, even if they have been
 *      advised of the possibility of such damages.
 *
 * VERSION = V1.1
 *
 * DATE      10/31/92
 *
 * DESCRIPTION  Standard Grid function for use in GRE precision tests.
 *
 *              Design Overview:
 *
 *                This module is designed to be linked with any GRE DLL
 *                requiring the drawing of a standard grid to measure
 *                precision.
 *
 * FUNCTIONS    gre_grid()   Generic grid creation routine
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 * EXTERNAL REFERENCES
 *
 * EXTERNAL FUNCTIONS
 *
*/
#define  INCL_WIN
#define  INCL_GREALL
#define  INCL_DDIMISC
#define  INCL_ERRORS
#include <string.h>
#include <os2.h>
#include <pmbitmap.h>
#include <pmddi.h>
#include "DTTDLLDF.h"
#include "gre_grid.h"
#include "std_util.h"

/***************************************************************************
 *
 * FUNCTION NAME = gre_grid()
 *
 * DESCRIPTION   = Generic grid creation routine, called with a parameter
 *                 for scale of grid.
 *
 * INPUT         = hPS                 - presentation space handle
 *                 scale               - scale of grid, suggested values:
 *                                       HALFINCH - 1/2 inch squares
 *                                       INCH     - 1 inch squares
 *                                       TWOINCH  - 2 inch squares
 *                                       (These are defined in gre_grid.h)
 *
 * OUTPUT        = None
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 call set_delta function to set current pels per inch and
 *                 page size.
 *
 *                 for vertical lines:
 *                   Map starting point of line
 *                   Use GreSetCurrentPosition to set starting point of line
 *                   Map ending point of line
 *                   Draw line using GrePolyLine
 *
 *                 for horizontal lines:
 *                   Map starting point of line
 *                   Use GreSetCurrentPosition to set starting point of line
 *                   Map ending point of line
 *                   Draw line using GrePolyLine
 *
 *
 *
 ****************************************************************************/

VOID EXPENTRY gre_grid(HPS hPS,LONG scale)
{
  HDC hDC;
  POINTL ptPoint1,ptPoint2;
  PPOINTL pptPosition;
  PPOINTL paptPoint;
  LONG cPoints;

  /*
  **  This draws the grid used by most of the GRE Precision tests.
  */

  cPoints = 1;

  /*
  **  This function sets delta_x, delta_y, pg_size_x, pg_size_y and returns hDC
  */

  hDC = set_delta(hPS);

  /*
  **  Draw first vertical line on left of page
  */

  ptPoint1.x = 0L;
  ptPoint1.y = 0L;                     /* starting point at (0,0)           */
  MapPt(ptPoint1);
  pptPosition = &ptPoint1;
  GreSetCurrentPosition(hDC, pptPosition);
  ptPoint2.x = 0L;
  ptPoint2.y = pg_size_y *delta_y;     /* vertical pels per page            */
  MapPt(ptPoint2);
  paptPoint = &ptPoint2;
  GrePolyLine(hDC, paptPoint, cPoints);


  /*
  **  Make even width for grid with two inch squares
  */

  if (scale == TWOINCH)
  {
    if ((pg_size_x%2) != 0)
      pg_size_x = pg_size_x-1;
  }

  /*
  **  Draw the rest of the vertical lines.
  */

  for (ptPoint1.x = delta_x *scale/2; ptPoint1.x <= (pg_size_x *delta_x);
     ptPoint1.x += delta_x *scale/2)
  {
    ptPoint2.x = ptPoint1.x;
    ptPoint2.y = 0L;
    MapPt(ptPoint2);
    pptPosition = &ptPoint2;
    GreSetCurrentPosition(hDC, pptPosition);
    ptPoint2.y = MapY(pg_size_y *delta_y);
    paptPoint = &ptPoint2;
    GrePolyLine(hDC, paptPoint, cPoints);
  }

  /*
  **  Draw the first horizontal line at bottom of page
  */

  ptPoint1.x = 0L;
  ptPoint1.y = 0L;                     /* starting point at (0,0)           */
  MapPt(ptPoint1);
  pptPosition = &ptPoint1;
  GreSetCurrentPosition(hDC, pptPosition);
  ptPoint2.x = pg_size_x *delta_x;     /* horizontal pels per page          */
  ptPoint2.y = 0L;
  MapPt(ptPoint2);
  paptPoint = &ptPoint2;
  GrePolyLine(hDC, paptPoint, cPoints);


  /*
  **  Draw the rest of the horizontal lines.
  */

  for (ptPoint1.y = delta_y *scale/2; ptPoint1.y <= (pg_size_y *delta_y);
     ptPoint1.y += delta_y *scale/2)
  {
    ptPoint2.x = 0L;
    ptPoint2.y = ptPoint1.y;
    MapPt(ptPoint2);
    pptPosition = &ptPoint2;
    GreSetCurrentPosition(hDC, pptPosition);
    ptPoint2.x = MapX(pg_size_x *delta_x);
    paptPoint = &ptPoint2;
    GrePolyLine(hDC, paptPoint, cPoints);
  }
}

