/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("FONTRC.C")

#define INCL_WIN
#define INCL_AVIO
#define INCL_GPI
#define INCL_GREALL
#define INCL_DEV
#define INCL_DDIMISC
#define INCL_DDIBUNDLES
#define INCL_ERRORS

#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <pmddi.h>
#include <pmddim.h>
#include <pmfont.h>
#include <limits.h>

#include "DTTDLLDF.h"                // test case defines
#include "common1.h"
#include "GREMDEV.h"                 // gre text dll include file

/*****************************************************************************
 *\\ddd
 * Subtask Name: GSE
 *
 * Module Name: MDEVRC
 *
 * Purpose:
 *
 *
 * Revision Log: 12/12/91, GSE, Stan Logan
 *                 -Test Case Completion
 *
 * Design Overview:
 *
 *    This testcase is designed to test the return codes of the Gre misc dev
 *    functions that:
 *
 *    1. Set the font for AVIO use.
 *       - GreDeviceSetAVIOFont()
 *    2. Invalidate a visible region by setting HDC_IS_DIRTY flag.
 *       - GreDeviceInvalidateVisRegion()
 *    3. Set PM screen in background to give other processes priority.
 *       - GreDeath()
 *    4. Reverse GreDeath.
 *       - GreResurrection()
 *    5. Set the coordinates for mapping the DC to the device.
 *       - GreDeviceSetDCOrigin()
 *
 * Limitations:
 *    None.
 *
 *\\end
 *****************************************************************************/



/***************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetAVIOFontRc
 *
 * Purpose: This is an entry point for the DTT that tests the
 *          Return Codes for GreDeviceSetAVIOFont.
 *
 * Documented errors that are returned by this function are :
 *
 *      PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * Errors that are tested are :
 *
 *      PMERR_INV_HDC                   PMERR_INV_SETID
 *      PMERR_FONT_NOT_LOADED           PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * System Requirements:
 *
 * Revision Log: 12/12/91, GSE, Stan Logan
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *
 *  ENTER_MAIN
 *  EXIT_MAIN
 *  SETENTER
 *  SETEXIT
 *  SUBENTER
 *  SUBEXIT
 *  CHECKRC
 *  LOGINFO
 *  SHOWERROR
 *  DeletePrinterOutput
 *
 * Limitations:
 *
 *  None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * Log setup
 *    Get the HDC
 *    Create an AVIO PS
 *    Associate the AVIO PS to the DC
 *    Initialize the font definition structure
 * Log setup completion
 * If DeviceIsDisplay
 *    Log subentry
 *       Attempt to solicit PMERR_INV_HDC
 *    Log subexit
 *    Log subentry
 *       Attempt to solicit PMERR_FONT_NOT_LOADED
 *    Log subexit
 *    Log subentry
 *       Attempt to solicit PMERR_INV_SETID with setid = -1
 *    Log subexit
 * Else DeviceIsPrinter
 *    Log subentry
 *       Attempt to solicit PMERR_DEV_FUNC_NOT_INSTALLED
 *    Log subexit
 * Disassociate the AVIO PS from the DC
 * Destroy the AVIO PS
 * Return code test, so suppress printer output
 *
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreDeviceSetAVIOFontRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static PCHAR   EntryName = "GreDeviceSetAVIOFontRc";
   HDC            thDC = (HDC)-1;
   HVPS           hVPS;
   FOCAFONT       fmDef;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else
   {
      ENTER_MAIN(EntryName);

// Log setup

      SETENTER(" ", "Check for valid HDC");

// Get the HDC

      if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR)
         SHOWERROR("GpiQueryDevice", EntryName);

// Create an AVIO PS

      VIOReturn = VioCreatePS(&hVPS, MAXROW, MAXCOL, 0, FORMAT_CGA, 0);

      if (VIOReturn != NO_ERROR)
      {
         sprintf(achLogLine, "Error creating AVIO PS: %ud", VIOReturn);

         LOGINFO(L_FAIL, "@", achLogLine);
      }

// Associate the AVIO PS to the DC

      VIOReturn = VioAssociate(hDC, hVPS);

      if (VIOReturn != NO_ERROR)
      {
         sprintf(achLogLine, "Error creating AVIO PS: %ud", VIOReturn);

         LOGINFO(L_FAIL, "@", achLogLine);
      }

// Initialize the font definition structure

      memset((VOID *)&fmDef, 0x00, sizeof(FOCAFONT));

// Log setup completion

      SETEXIT();

      if (DeviceIsDisplay())
      {

   /**************************************************************
   * The following Code will check for PMERR_INV_HDC
   **************************************************************/

// Log subentry

      SUBENTER(PMERR_INV_HDC, EntryName);

// Attempt to solicit PMERR_INV_HDC

      lSuccess = (GreDeviceSetAVIOFont(thDC, (PFOCAFONT)NULL, -2L) != FALSE);

      CHECKRC(lSuccess, PMERR_INV_HDC, EntryName);

// Log subexit

      SUBEXIT();

   /**************************************************************
   * The following Code will check for PMERR_FONT_NOT_LOADED
   **************************************************************/

// Log subentry

      SUBENTER(PMERR_FONT_NOT_LOADED, EntryName);

      memset((VOID *)&fmDef, 0xff, sizeof(FOCAFONT));

// Attempt to solicit PMERR_FONT_NOT_LOADED

      lSuccess = (GreDeviceSetAVIOFont(hDC, &fmDef, -2L) != FALSE);

      CHECKRC(lSuccess, PMERR_FONT_NOT_LOADED, EntryName);

// Log subexit

      SUBEXIT();

   /**************************************************************
   * The following Code will check for PMERR_INV_SETID
   **************************************************************/

// Log subentry

      SUBENTER(PMERR_INV_SETID, EntryName);

// Attempt to solicit PMERR_INV_SETID with setid = -1

      lSuccess = (GreDeviceSetAVIOFont(hDC, (PFOCAFONT)NULL, -1L) != FALSE);

      CHECKRC(lSuccess, PMERR_INV_SETID, EntryName);

// Log subexit

      SUBEXIT();

      }
      else
      {

   /**************************************************************
   * The following Code will check for PMERR_DEV_FUNC_NOT_INSTALLED
   **************************************************************/

// Log subentry

      SUBENTER(PMERR_DEV_FUNC_NOT_INSTALLED, EntryName);

      memset((VOID *)&fmDef, 0xff, sizeof(FOCAFONT));

// Attempt to solicit PMERR_DEV_FUNC_NOT_INSTALLED

      lSuccess = (GreDeviceSetAVIOFont(hDC, &fmDef, -2L) != FALSE);

      CHECKRC(lSuccess, PMERR_DEV_FUNC_NOT_INSTALLED, EntryName);

// Log subexit

      SUBEXIT();

      }

// Disassociate the AVIO PS from the DC

      VIOReturn = VioAssociate(NULLHANDLE, hVPS);

      if (VIOReturn != NO_ERROR)
      {
         sprintf(achLogLine, "Error creating AVIO PS: %ud", VIOReturn);

         LOGINFO(L_FAIL, "@", achLogLine);
      }

// Destroy the AVIO PS

      VIOReturn = VioDestroyPS(hVPS);

      if (VIOReturn != NO_ERROR)
      {
         sprintf(achLogLine, "Error creating AVIO PS: %ud", VIOReturn);

         LOGINFO(L_FAIL, "@", achLogLine);
      }

// Exit entry point

      EXIT_MAIN(EntryName);

// Return code test, so suppress printer output

      DeletePrinterOutput(EntryName);

   }
}

/***************************************************************************
 *\\ddd
 * Routine Name: GreDeviceInvalidateVisRegionRc
 *
 * Purpose: This is an entry point for the DTT that tests the
 *          Return Codes for GreDeviceInvalidateVisRegion.
 *
 * Documented errors that are returned by this function are :
 *
 *      PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * Errors that are tested are :
 *
 *      PMERR_INV_HDC
 *
 * System Requirements:
 *
 * Revision Log: 12/12/91, GSE, Stan Logan
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *
 *  ENTER_MAIN
 *  EXIT_MAIN
 *  SETENTER
 *  SETEXIT
 *  SUBENTER
 *  SUBEXIT
 *  CHECKRC
 *  SHOWERROR
 *  DeletePrinterOutput
 *
 * Limitations:
 *
 *  None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * Log setup
 *    Get the HDC
 *    Set up the DC_BLOCK structure
 * Log setup completion
 * Log subentry
 *    Attempt to solicit PMERR_INV_HDC
 * Log subexit
 * Return code test, so suppress printer output
 *
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreDeviceInvalidateVisRegionRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static PCHAR   EntryName = "GreDevInvalVisRegionRc";
   HDC            thDC = (HDC)-1;
   DC_BLOCK       DCBlock;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else
   {
      ENTER_MAIN(EntryName);

// Log setup

      SETENTER(" ", "Check for valid HDC");

// Get the HDC

      if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR)
         SHOWERROR("GpiQueryDevice", EntryName);

// Set up the DC_BLOCK structure

      DCBlock.hdc = (ULONG)hDC;
      DCBlock.hddc = GetDDC();

// Log setup completion

      SETEXIT();

   /**************************************************************
   * The following Code will check for PMERR_INV_HDC
   **************************************************************/

// Log subentry

      SUBENTER(PMERR_INV_HDC, EntryName);

// Attempt to solicit PMERR_INV_HDC

      bSuccess = (GreDeviceInvalidateVisRegion(thDC, 0L, &DCBlock) != FALSE);

      CHECKRC(bSuccess, PMERR_INV_HDC, EntryName);

// Log subexit

      SUBEXIT();

// Exit entry point

      EXIT_MAIN(EntryName);

// Return code test, so suppress printer output

      DeletePrinterOutput(EntryName);

   }
}

/***************************************************************************
 *\\ddd
 * Routine Name: GreDeathRc
 *
 * Purpose: This is an entry point for the DTT that tests the
 *          Return Codes for GreDeath.
 *
 * Documented errors that are returned by this function are :
 *
 *      PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * Errors that are tested are :
 *
 *      PMERR_INV_HDC
 *
 * System Requirements:
 *
 * Revision Log: 12/12/91, GSE, Stan Logan
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *
 *  ENTER_MAIN
 *  EXIT_MAIN
 *  SUBENTER
 *  SUBEXIT
 *  CHECKRC
 *  DeletePrinterOutput
 *
 * Limitations:
 *
 *  None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * Log subentry
 *    Attempt to solicit PMERR_INV_HDC
 * Log subexit
 * Return code test, so suppress printer output
 *
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreDeathRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static PCHAR   EntryName = "GreDeathRc";
   HDC            thDC = (HDC)-1;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else
   {
      ENTER_MAIN(EntryName);

   /**************************************************************
   * The following Code will check for PMERR_INV_HDC
   **************************************************************/

// Log subentry

      SUBENTER(PMERR_INV_HDC, EntryName);

// Attempt to solicit PMERR_INV_HDC

      bSuccess = (GreDeath(thDC) != FALSE);

      CHECKRC(bSuccess, PMERR_INV_HDC, EntryName);

// Log subexit

      SUBEXIT();

// Exit entry point

      EXIT_MAIN(EntryName);

// Return code test, so suppress printer output

      DeletePrinterOutput(EntryName);

   }
}

/***************************************************************************
 *\\ddd
 * Routine Name: GreResurrectionRc
 *
 * Purpose: This is an entry point for the DTT that tests the
 *          Return Codes for GreResurrection.
 *
 * Documented errors that are returned by this function are :
 *
 *      PMERR_INV_HDC                   PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * Errors that are tested are :
 *
 *      PMERR_INV_HDC
 *
 * System Requirements:
 *
 * Revision Log: 12/12/91, GSE, Stan Logan
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *
 *  ENTER_MAIN
 *  EXIT_MAIN
 *  SUBENTER
 *  SUBEXIT
 *  CHECKRC
 *  DeletePrinterOutput
 *
 * Limitations:
 *
 *  None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * Log subentry
 *    Attempt to solicit PMERR_INV_HDC
 * Log subexit
 * Return code test, so suppress printer output
 *
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreResurrectionRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static PCHAR   EntryName = "GreResurrectionRc";
   HDC            thDC = (HDC)-1;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else
   {
      ENTER_MAIN(EntryName);

   /**************************************************************
   * The following Code will check for PMERR_INV_HDC
   **************************************************************/

// Log subentry

      SUBENTER(PMERR_INV_HDC, EntryName);

// Attempt to solicit PMERR_INV_HDC

      bSuccess = (GreResurrection(thDC, 0L, (PBYTE)NULL) != FALSE);

      CHECKRC(bSuccess, PMERR_INV_HDC, EntryName);

// Log subexit

      SUBEXIT();

// Exit entry point

      EXIT_MAIN(EntryName);

// Return code test, so suppress printer output

      DeletePrinterOutput(EntryName);

   }
}

/***************************************************************************
 *\\ddd
 * Routine Name: GreDeviceSetDCOriginRc
 *
 * Purpose: This is an entry point for the DTT that tests the
 *          Return Codes for GreDeviceSetDCOrigin.
 *
 * Documented errors that are returned by this function are :
 *
 *      PMERR_INV_HDC                   PMERR_DEV_FUNC_NOT_INSTALLED
 *
 * Errors that are tested are :
 *
 *      PMERR_INV_HDC
 *
 * System Requirements:
 *
 * Revision Log: 12/12/91, GSE, Stan Logan
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *
 *  ENTER_MAIN
 *  EXIT_MAIN
 *  SUBENTER
 *  SUBEXIT
 *  CHECKRC
 *  DeletePrinterOutput
 *
 * Limitations:
 *
 *  None.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 * Log subentry
 *    Attempt to solicit PMERR_INV_HDC
 * Log subexit
 * Return code test, so suppress printer output
 *
 *\\end
 ***************************************************************************/

VOID  APIENTRY GreDeviceSetDCOriginRc(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static PCHAR   EntryName="GreDeviceSetDCOriginRc";
   HDC            thDC = (HDC)-1;

   if (SelectionCall)
      WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
   else {
      ENTER_MAIN(EntryName);

   /**************************************************************
   * The following Code will check for PMERR_INV_HDC
   **************************************************************/

// Log subentry

      SUBENTER(PMERR_INV_HDC, EntryName);

      ptl.x = ptl.y = 0L;

// Attempt to solicit PMERR_INV_HDC

      bSuccess = (GreDeviceSetDCOrigin(thDC, &ptl) != FALSE);

      CHECKRC(bSuccess, PMERR_INV_HDC, EntryName);

// Log subexit

      SUBEXIT();

// Exit entry point

      EXIT_MAIN(EntryName);

// Return code test, so suppress printer output

      DeletePrinterOutput(EntryName);

   }
}
