/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma  title("STD_UTIL.C")

/**************************************************************************
 *
 * SOURCE FILE NAME = STDUTIL2.C
 *
 * DESCRIPTIVE NAME = Standard test case routines two.
 *
 *
 *      DISCLAIMER OF WARRANTIES.  The following enclosed code is
 *      sample code created by IBM Corporation. This sample code is not
 *      part of any standard or IBM product and is provided to you solely
 *      for  the purpose of assisting you in the development of your
 *      applications.  The code is provided "AS IS", without
 *      warranty of any kind.  IBM shall not be liable for any damages
 *      arising out of your use of the sample code, even if they have been
 *      advised of the possibility of such damages.
 *
 * VERSION = V1.1
 *
 * DATE      10/31/92
 *
 * DESCRIPTION  General purpose test case dll functions.
 *
 * FUNCTIONS    set_delta()              Initialization routin.
 *              EzfQueryFonts()          Query available fonts.
 *              EzfCreateLogicalFonts()  Create a logical font.
 *              MapArrayOfPoints()       Map coordinate points
 *                                       to display coordinates.
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 * EXTERNAL REFERENCES
 *
 * EXTERNAL FUNCTIONS
 *
*/
#define  INCL_WIN
#define  INCL_DEV
#define  INCL_GPI
#define  INCL_ERRORS
#include <string.h>
#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include "DTTDLLDF.h"
#include "std_util.h"
#define  LCID_FIXED10  1L
static SHORT sFontSize[6] =
{
  80, 100, 120, 140, 180, 240
} ;

static CHAR *szFacename[4] =
{
  "System Proportional", "Courier", "Helv", "Tms Rmn"
} ;

static LONG alMatch[4][6];
static LONG MyMatch;
static LONG MyAveCharWidth;
static CHAR MyFacename[FACESIZE];

/***************************************************************************
 *
 * FUNCTION NAME = set_delta()
 *
 * DESCRIPTION   = This is the initialization routine that sets the following
 *                 global variables for use in formatting output to the proper
 *                 screen location.  Sets:
 *                      *pBasisX       - horizontal units per page
 *                      *pBasisY       - vertical units per page
 *                      width_in_pels  - device width
 *                      height_in_pels - device height
 *                      hori_pels_per_m- horizontal resolution
 *                      vert_pels_per_m- vertical resolution
 *                      delta_x        - units per horizontal inch
 *                      delta_y        - units per vertical inch
 *                      pg_size_x      - horizontal page size in smallest whole inch
 *                      pg_size_y      - vertical page size in smallest whole inch
 *                      line_space     - std amount of vertical space between lines
 *                                        of text
 *
 * INPUT         = hPS - presentation space handle
 *
 * OUTPUT        = hDC - device context handle
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 set *pBasisX and *pBasisY = 10000
 *                 use DevQueryCaps() to get:
 *                   width_in_pels
 *                   height_in_pels
 *                   hori_pels_per_m
 *                   vert_pels_per_m
 *                 and calculate:
 *                   delta_x
 *                   delta_y
 *                   pg_size_x
 *                   pg_size_y
 *                   line_space
 *
 ****************************************************************************/

HDC set_delta(HPS hPS)
{
  LONG alArray[12],buf[2];
  HDC hDC;

  *pBasisX = 10000;                    /* horizontal units per page         */
  *pBasisY = 10000;                    /* vertical units per page           */
  hDC = GpiQueryDevice(hPS);
  DevQueryCaps(hDC, 4L, 6L, alArray);
  width_in_pels = alArray[0];          /* device width                      */
  height_in_pels = alArray[1];         /* device height                     */
  buf[0] = alArray[2];                 /* we don't need these two           */
  buf[1] = alArray[3];
  hori_pels_per_m = alArray[4];        /* horizontal resolution             */
  vert_pels_per_m = alArray[5];        /* vertical resolution               */
  delta_x = (254*hori_pels_per_m)/width_in_pels;
  delta_y = (254*vert_pels_per_m)/height_in_pels; /* page size in inches    */

  /*
  **  Device is a display don't count on font resolution being accurate
  */

  DevQueryCaps(hDC,
               CAPS_TECHNOLOGY,
               1L,
               alArray);

  if (alArray[0] == CAPS_TECH_RASTER_DISPLAY)
  {
    pg_size_x = (width_in_pels *10000)/(hori_pels_per_m *254);
    pg_size_y = (height_in_pels *10000)/(vert_pels_per_m *254);
  }

  else
  {
    DevQueryCaps(hDC, CAPS_HORIZONTAL_FONT_RES, 2L, alArray);
    pg_size_x = width_in_pels/alArray[0];
    pg_size_y = height_in_pels/alArray[1];
  }
  line_space = delta_y *1/5;
  return (hDC);
}

/***************************************************************************
 *
 * FUNCTION NAME = EzfQueryFonts()
 *
 * DESCRIPTION   = This is a function to query available fonts
 *
 * INPUT         = hPS - presentation space handle
 *
 * OUTPUT        = TRUE if useable font(s) available
 *                 FALSE if all font(s) too large
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 for fonts System Proportional, Courier, Times Roman, Helvetica
 *                   query font information
 *                   return FALSE if all fonts too large
 *                      else return TRUE
 *
 ****************************************************************************/

BOOL EzfQueryFonts(HPS hps)
{
  FONTMETRICS huge *pfm;
  void huge *pmem;
  HDC hdc;
  LONG lHorzRes,lVertRes,lRequestFonts,lNumberFonts;
  SHORT sIndex,sFace,sSize;
  SHORT sTemp;
  USHORT i,j;

  hdc = GpiQueryDevice(hps);
  DevQueryCaps(hdc, CAPS_HORIZONTAL_FONT_RES, 1L, &lHorzRes);
  DevQueryCaps(hdc, CAPS_VERTICAL_FONT_RES, 1L, &lVertRes);

  /*
  ** intializing the two dimensional array of alMatch[4][6]
  */

  for (i = 0; i < 4; i++)

    for (j = 0; j < 6; j++)

      alMatch[4][6] = 0;


  for (sFace = 0; sFace < 4; sFace++)
  {
    lRequestFonts = 0;
    lNumberFonts = GpiQueryFonts(hps,
                                 QF_PUBLIC,
                                 szFacename[sFace],
                                 & lRequestFonts,
                                 0L,
                                 NULL);

    if (lNumberFonts == 0)
      continue;

    if (lNumberFonts *sizeof(FONTMETRICS) >= 65536L)
      return  FALSE;


     pfm = malloc ((SHORT) lNumberFonts*sizeof(FONTMETRICS));


    if (pfm == NULL)
      return  FALSE;

    GpiQueryFonts(hps,
                  QF_PUBLIC,
                  szFacename[sFace],
                  &lNumberFonts,
                  (LONG)sizeof(FONTMETRICS),
                  pfm);

    for (sIndex = 0; sIndex < (SHORT)lNumberFonts; sIndex++)

      /*
      **  Avoid when font crosses segment.
      */

      if ((sIndex == 315) || (sIndex == 630) || (sIndex == 945)
        || (sIndex == 1260))

        sTemp = sIndex-1;

      else

        sTemp = sIndex;

    if (pfm[sTemp].sXDeviceRes == (SHORT)lHorzRes
       && pfm[sTemp].sYDeviceRes == (SHORT)lVertRes)
    {

      for (sSize = 0; sSize < 6; sSize++)

        if (pfm[sTemp].sNominalPointSize == sFontSize[sSize])
          break;

      if (sSize != 6)
        alMatch[sFace][sSize] = pfm[sTemp].lMatch;
    }
    free(pfm); // 16kq
  }
  return  TRUE;
}

/***************************************************************************
 *
 * FUNCTION NAME = EzfCreateLogicalFonts()
 *
 * DESCRIPTION   = Create a logical font
 *
 *
 * INPUT         = hPS            - presentation space handle
 *                 lcid           - font id
 *                 idFace         - face name of desired font
 *                 idSize         - size of desired font
 *                 fsSelection    - other desired font criterion
 *
 * OUTPUT        = 0 if criteria out of range
 *                 The lcid of the selected font
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 return FALSE if criteria out of range
 *                 GpiCreateLogFont for desired criteria and return lcid
 *
 ****************************************************************************/

LONG EzfCreateLogFont(HPS hps,LONG lcid,USHORT idFace,USHORT idSize,USHORT
                       fsSelection)
{
  static FATTRS fat;


  if (idFace > 3 || idSize > 5 || alMatch[idFace][idSize] == 0)
    return  FALSE;

  fat.usRecordLength = sizeof fat;
  fat.fsSelection = fsSelection;
  fat.lMatch = alMatch[idFace][idSize];

  /*
  **  Initialize code page
  */

  if (fat.lMatch == 0)
  {
    fat.usCodePage = 850;
  }
  strcpy(fat.szFacename, szFacename[idFace]);

  return  GpiCreateLogFont(hps, NULL, lcid, &fat);
}


/***************************************************************************
 *
 * FUNCTION NAME = MapArrayOfPoints()
 *
 * DESCRIPTION   = Map coordinate points to display coordinates
 *
 *
 * INPUT         = Number of array points
 *                 Array points to be mapped
 *
 * OUTPUT        = None
 *
 * RETURN-NORMAL =
 * RETURN-ERROR  =
 *
 * ALGORITHM
 *                 For each point,
 *                    call MapPt
 *
 ****************************************************************************/

VOID APIENTRY MapArrayOfPoints(int num_pts,POINTL aptl[])
{
  int i;


  for (i = 0; i < num_pts; i++)
    MapPt(aptl[i]);
}

