/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT (C) Microsoft Corporation, 1989                                 */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = DTTLOG.C
 *
 * DESCRIPTIVE NAME =  Display Test Tool
 *
 *
 *
 * VERSION = V2.0
 *
 * DATE
 *
 * DESCRIPTION :
 *
 *   Purpose: Provides an alternative logging method for DTT--ie, instead
 *            of using the Testlog logging method.
 *
 *
 *
 * FUNCTIONS  : dtt_openlog
 *              dtt_closelog
 *
 *
 *
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 * EXTERNAL REFERENCES
 *
 *
 *
 * EXTERNAL FUNCTIONS
 *
*/


#include "DTT.h"
#define  DTTLOG
#include "DTTLog.h"

/***************************************************************************
 *
 * FUNCTION NAME =   dtt_openlog
 *
 *
 * DESCRIPTION   =
 *
 *   Purpose:  Open the logging file and print out the header
 *             to the log file.
 *
 *   Function Calls:  NONE
 *
 *
 *
 * INPUT = NONE
 *
 *
 *
 *
 * OUTPUT = NONE
 *
 *
 * RETURN-NORMAL = VOID
 *
 * RETURN-ERROR  = NONE
 *
 ****************************************************************************/

VOID dtt_WriteHeader(VOID)
{
  DosGetDateTime(&DateTime);    /* get the date and time to enter into log  */

   /*
   ** Print out the header to the log file
   */

  fprintf(logfile,
          "Start of DTT run.\n");
  fprintf(logfile,
          "%02d/%02d/%02d    %02d:%02d\n",
          DateTime.month,
          DateTime.day,
          DateTime.year,
          DateTime.hours,
          DateTime.minutes);
  fprintf(logfile,
          "=== START ===\n");
  return ;
}

/***************************************************************************
 *
 * FUNCTION NAME =   dtt_RecordLogLevel
 *
 *
 * DESCRIPTION   =
 *
 *   Purpose:
 *
 *
 *   Function Calls:  NONE
 *
 *
 *
 * INPUT = NONE
 *
 *
 *
 *
 * OUTPUT = NONE
 *
 *
 * RETURN-NORMAL = VOID
 *
 * RETURN-ERROR  = NONE
 *
 ****************************************************************************/

VOID dtt_RecordLogLevel(VOID)
{
  CHAR szLogLevel[20];

  DosGetDateTime(&DateTime);    /* get the date and time to enter into log */

   /*
   ** find out the log level for entry into the log file header
   */

  switch (dttloglevel)
  {
    case  L_HDR :
      strcpy(szLogLevel,
             "L_HDR");
      break;
    case  L_ABORT :
      strcpy(szLogLevel,
             "L_ABORT");
      break;
    case  L_FAIL :
      strcpy(szLogLevel,
             "L_FAIL");
      break;
    case  L_WARN :
      strcpy(szLogLevel,
             "L_WARN");
      break;
    case  L_TRACE :
      strcpy(szLogLevel,
             "L_TRACE");
      break;
    case  L_LOTRACE :
      strcpy(szLogLevel,
             "L_LOTRACE");
      break;
    case  L_DEBUG :
      strcpy(szLogLevel,
             "L_DEBUG");
      break;
    default  :                /* should never get here, but just in case.  */
      strcpy(szLogLevel,
             "L_HDR");
      break;
  }
  fprintf(logfile,
          "------------Current Logging Level ------------------\n");
  fprintf(logfile,
          "Logging Level = %d (%s)\n",
          dttloglevel,
          szLogLevel);
  return ;
}

/***************************************************************************
 *
 * FUNCTION NAME =   dtt_openlog
 *
 *
 * DESCRIPTION   =
 *
 *   Purpose:
 *
 *
 *   Function Calls:  NONE
 *
 *
 *
 * INPUT = CHAR *Filename
 *
 *
 *
 *
 * OUTPUT = NONE
 *
 *
 * RETURN-NORMAL = VOID
 *
 * RETURN-ERROR  = NONE
 *
 ****************************************************************************/

VOID dtt_openlog(CHAR *filename)
{


  if ((logfile = fopen(filename,
                       "a+")) == NULL)
  {
    PostErrorMessage(hWndClient,
                     IDS_ERR_OPENFILE);
  }
  return ;
}                                      /* end dtt_openlog                  */

/***************************************************************************
 *
 * FUNCTION NAME =   dtt_closelog
 *
 *
 * DESCRIPTION   =
 *
 *   Purpose:  Close the logging file and print out the trailer
 *             to the log file.
 *
 *   Function Calls:  NONE
 *
 *
 *
 * INPUT = CHAR *Filename
 *
 *
 *
 *
 * OUTPUT = NONE
 *
 *
 * RETURN-NORMAL = VOID
 *
 * RETURN-ERROR  = NONE
 *
 ****************************************************************************/

VOID dtt_closelog()
{
  DosGetDateTime(&DateTime);    /* get the date and time to enter into log  */
  fprintf(logfile,
          "===  END  ===\n");
  fprintf(logfile,
          "%02d:%02d\n",
          DateTime.hours,
          DateTime.minutes);
  fprintf(logfile,
          "End of DTT run.\n");
  fclose(logfile);
  logfile = (FILE *)NULL;
}                                      /* end dtt_closelog                  */
