/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT (C) Microsoft Corporation, 1989                                 */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = DTTSCRPT.C
 *
 * DESCRIPTIVE NAME = Display Test Tool
 *
 *
 *
 * VERSION = V2.0
 *
 * DATE
 *
 * DESCRIPTION :
 *
 *   Purpose: Read commands from a script file and send commands to DTT
 *            window procedure.
 *
 *
 *
 * FUNCTIONS  :  startScriptThread
 *               ScriptThread
 *               GetScriptCmd
 *
 *
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 * EXTERNAL REFERENCES
 *
 *
 *
 * EXTERNAL FUNCTIONS
 *
*/

#include "DTT.h"
#include "DTTscrpt.h"

/***************************************************************************
 *
 * FUNCTION NAME = StartScriptThread
 *
 *
 * DESCRIPTION   =
 *
 *   Purpose:  Startup the script thread.
 *
 *
 *   Function Calls:  ScriptThtread
 *
 *
 *
 * INPUT = NONE
 *
 *
 *
 *
 * OUTPUT = NONE
 *
 *
 * RETURN-NORMAL = Thread Id
 *
 * RETURN-ERROR  = -1
 *
 ****************************************************************************/

TID startScriptThread(VOID)
{
  TID tid;                             /* thread id                         */


  /*
  ** start the thread running
  */

  tid = _beginthread(ScriptThread,
                     NULL,
                     STACKSIZE,
                     NULL);
  return  tid;
}

/***************************************************************************
 *
 * FUNCTION NAME = ScriptThread
 *
 *
 * DESCRIPTION   =
 *
 *   Purpose:  Read commands from a script file, interpret the
 *             the commands and issue messages to the DTT
 *             window procedure to execute the commands.
 *
 *   Function Calls:  GetScriptCmd
 *
 *
 *
 * INPUT = NONE
 *
 *
 *
 *
 * OUTPUT = NONE
 *
 *
 * RETURN-NORMAL = VOID
 *
 * RETURN-ERROR  = NONE
 *
 ****************************************************************************/

VOID _cdecl ScriptThread(PVOID Dummy)
{


    FILE *scriptfile;
    HFILE scripthandle;
    APIRET rc;
    ULONG FType;
    ULONG Attr;
    USHORT PipeRead;
    ULONG BytesWritten;
    static CHAR szLine[85];
    static CHAR szLogString[120];
    CHAR *result;
    SCRIPTCMD scriptcmd;
    USHORT NumberOfTestCases;
    USHORT usIndex;
    PVOID junk = (PVOID)Dummy;

    ULONG ulPostCnt;

  /*
  ** wait for initialization to finish before preceeding
  */

    DosWaitEventSem(ulDTTRamSem,
                    SEM_INDEFINITE_WAIT);


    if ((hABScriptThread = WinInitialize((USHORT)NULL)) == (HAB)NULL)
    {
      WinPostMsg(hWndClient,
                 WM_TEST_LOG,
                 MPFROM2SHORT(L_HDR,
                              0),
                 MPFROMLONG("DTTSCRPT: Error from WinInitialize\n"));

    /*
    ** terminate Dtt
    */

      WinPostMsg(hWndClient,
                 WM_CLOSE,
                 0L,
                 0L);

    /*
    ** _endthread
    */

      DosExit(0,
              0);
    }

    if ((hMQScriptThread = WinCreateMsgQueue(hABScriptThread,
                                             0)) == NULL)
    {
      WinPostMsg(hWndClient,
                 WM_TEST_LOG,
                 MPFROM2SHORT(L_HDR,
                              0),
                 MPFROMLONG("DTTSCRPT: Error from WinCreateMsgQueue\n"));

    /*
    ** terminate Dtt
    */

      WinPostMsg(hWndClient,
                 WM_CLOSE,
                 0L,
                 0L);

    /*
    ** _endthread
    */

      DosExit(0,
              0);
    }
    DosSetPrty(PRTYS_THREAD,
               PRTYC_NOCHANGE,
               -1,
               (USHORT)NULL);

    if ((scriptfile = fopen(szScriptFileName,
                            "r+")) == NULL)
    {

    /*
    ** log an error while opening the script file
    */

      WinPostMsg(hWndClient,
                 WM_TEST_LOG,
                 MPFROM2SHORT(L_HDR,
                              0),
                 MPFROMLONG("Error Opening Script File\n"));

    /*
    ** terminate Dtt
    */

      WinPostMsg(hWndClient,
                 WM_CLOSE,
                 0L,
                 0L);

    /*
    **_endthread;
    */

      DosExit(0,
              0);
    }

/*
** Added for automation
*/

    setvbuf(scriptfile,
            0,
            _IONBF,
            0);
    scripthandle = fileno(scriptfile);
    //rc = DosQHandType(scripthandle,
    rc = DosQueryHType(scripthandle,
                      &FType,
                      &Attr);
    PipeRead = (((FType == HANDTYPE_PIPE) || (FType == (HANDTYPE_PIPE|
       HANDTYPE_NETWORK)))?1:0);

/*
** End of addition
*/

  /*
  ** This code will enable use to run entry points from different
  ** Testcases.
  */

    NumberOfTestCases = SHORT1FROMMR(WinSendMsg(WinWindowFromID
       (hwndTestCasesDlg,
        DLG_GROUPLB),
                                                LM_QUERYITEMCOUNT,
                                                NULL,
                                                NULL));

    for (usIndex = 0; usIndex < NumberOfTestCases; )
    {
      WinSendMsg(WinWindowFromID(hwndTestCasesDlg,
                                 DLG_GROUPLB),
                 LM_SETTOPINDEX,
                 MPFROMSHORT(usIndex),
                 NULL);
      WinSendMsg(WinWindowFromID(hwndTestCasesDlg,
                                 DLG_GROUPLB),
                 LM_SELECTITEM,
                 MPFROMSHORT(usIndex++),
                 MPFROMSHORT(TRUE));
    }

    while (result = fgets(szLine,
                          80,
                          scriptfile))
    {

 /*
 ** Notify server that message received only if it is a pipe
 ** Added for automation
 */

      if (PipeRead)
        DosWrite(scripthandle,
                 "OK",
                 3,
                 &BytesWritten);
      GetScriptCmd(szLine,
                   &scriptcmd);

      switch (scriptcmd.msg)
      {
        case  WM_CHOOSE_DDRIVER :
          WinSendMsg(hWndClient,
                     WM_CHOOSE_QUEUE,
                     MPFROMP(apPrinters[scriptcmd.usDDriver]->aszQueueName),
                     NULL);
          WinSendMsg(hWndClient,
                     WM_CHOOSE_PORT,
                     MPFROMP(apPrinters[scriptcmd.usDDriver]->aszPortName),
                     NULL);
          WinSendMsg(hWndClient,
                     WM_CHOOSE_DDRIVER,
                     MPFROMP(apPrinters[scriptcmd.usDDriver]->aszDDName),
                     MPFROMP(apPrinters[scriptcmd.usDDriver]->aszDeviceName));
          WinSendMsg(hWndClient,
                     WM_CHOOSE_PRINTERNAME,
                     MPFROMP(apPrinters[scriptcmd.usDDriver]->aszPrinterName),
                     NULL);
          break;
        case  WM_CHOOSE_QPROCESS :
          WinSendMsg(hWndClient,
                     WM_CHOOSE_QPROCESS,
                     MPFROMP(apQProcNames[scriptcmd.usQProc]),
                     NULL);
          break;
        case  WM_CHOOSE_DCTYPE :
          WinSendMsg(hWndClient,
                     WM_CHOOSE_DCTYPE,
                     MPFROMLONG(scriptcmd.lDCType),
                     NULL);
          break;
        case  WM_CHOOSE_DATATYPE :
          WinSendMsg(hWndClient,
                     WM_CHOOSE_DATATYPE,
                     MPFROMP(scriptcmd.szDataType),
                     NULL);
          break;
        case  WM_CHOOSE_PSTYPE :
          WinSendMsg(hWndClient,
                     WM_CHOOSE_PSTYPE,
                     MPFROMLONG(scriptcmd.ulPSType),
                     NULL);
          break;
        case  WM_CHOOSE_PSUNITS :
          WinSendMsg(hWndClient,
                     WM_CHOOSE_PSUNITS,
                     MPFROMLONG(scriptcmd.ulPSUnits),
                     NULL);
          break;
        case  WM_CHOOSE_TESTCASE :
          break;
        case  IDM_ACTIONS_DISPLAY :

             DosResetEventSem(ulDTTRamSem,
                              &ulPostCnt);


           /*
           ** call to send output to the screen
           */

          WinPostQueueMsg(hMQPrintThread,
                          WM_TEST_DISPLAY,
                          pSelectedTestCase,
                          NULL);

           /*
           ** wait till were done
           */

          DosWaitEventSem(ulDTTRamSem,
                          SEM_INDEFINITE_WAIT);

          break;
        case  IDM_EXIT :

           /*
           ** terminate Dtt
           */

          fclose(scriptfile);
          WinPostQueueMsg(hMQPrintThread,
                          WM_CANCEL_PRINT_THREAD,
                          NULL,
                          NULL);
          break;
        case  SCR_BLANK_LINE :
        case  SCR_COMMENT :

           /*
           ** ignore blank lines and comments in the script file
           */

          break;
        case  SCR_DELAY :
          DosSleep(DELAYLENGTH);
          break;
        default  :

           /*
           ** log an error while reading the script file
           */

          strcpy(szLogString,
                 "Invalid command in Script File: \"");
          strcat(szLogString,
                 szLine);
          (*strstr(szLogString,
                   "\n")) = (CHAR)NULL;
          strcat(szLogString,
                 "\"\n");
          WinPostMsg(hWndClient,
                     WM_TEST_LOG,
                     MPFROM2SHORT(L_HDR,
                                  0),
                     MPFROMLONG(szLogString));

           /*
           ** terminate Dtt
           */

          WinPostQueueMsg(hMQPrintThread,
                          WM_CANCEL_PRINT_THREAD,
                          NULL,
                          NULL);
          break;
      }                                /* end switch                      */

      if ((scriptcmd.msg != SCR_BLANK_LINE) && (scriptcmd.msg != SCR_COMMENT))
      {

      /*
      ** Log the script commands to the log file at the DEBUG level
      */

        strcpy(szLogString,
               "Script Command: \"");
        strcat(szLogString,
               szLine);
        (*strstr(szLogString,
                 "\n")) = (CHAR)NULL;
        strcat(szLogString,
               "\"\n");
        WinSendMsg(hWndClient,
                   WM_TEST_LOG,
                   MPFROM2SHORT(L_TRACE,
                                0),
                   MPFROMLONG(szLogString));

      }                                /* end if                         */

    }                                  /* end while                      */

    DosPostEventSem(ulDTTRamSem);


  /*
  ** if an error occured during reading, then log the error
  */

    if (ferror(scriptfile))
    {

    /*
    ** log an error while reading the script file
    */

      WinSendMsg(hWndClient,
                 WM_TEST_LOG,
                 MPFROM2SHORT(L_HDR,
                              0),
                 MPFROMLONG("Error reading Script File\n"));
    }

  /*
  ** terminate Dtt
  */

    WinPostQueueMsg(hMQPrintThread,
                    WM_CANCEL_PRINT_THREAD,
                    NULL,
                    NULL);
    fclose(scriptfile);
  }                                    /* end ScriptThread                 */

/***************************************************************************
 *
 * FUNCTION NAME = GetScriptCmd
 *
 *
 * DESCRIPTION   =
 *
 *   Purpose:  Takes as input a line from the script file and
 *             a pointer to a script command structure.  The
 *             line is parsed for script commands and the
 *             script command struc is filled in accordingly
 *
 *
 *   Function Calls:  GetScriptCmd
 *
 *
 *
 * INPUT = PCHAR
 *         PSCRIPTCMD
 *
 *
 *
 * OUTPUT = NONE
 *
 *
 * RETURN-NORMAL = VOID
 *
 * RETURN-ERROR  = NONE
 *
 ****************************************************************************/

  VOID GetScriptCmd(PCHAR      line,
                    PSCRIPTCMD pscriptcmd)
  {
    INT n;
    SHORT i;
    CHAR cmd[20];
    CHAR parm[60];
    SHORT usSelect = LIT_FIRST;

  /*
  ** scan the line of input for a command and a parameter
  */

    n = sscanf(line,
               "%s %s",
               cmd,
               parm);

    switch (n)
    {
      case  SCR_CMD :                  /* command with no parameter         */

        if (strcmpi(cmd,
                    "PRINT") == 0)
        {
          pscriptcmd->msg = IDM_ACTIONS_PRINT;
        }

        else
        {

          if (strcmpi(cmd,
                      "DISPLAY") == 0)
          {
            pscriptcmd->msg = IDM_ACTIONS_DISPLAY;
          }

          else
          {

            if (strcmpi(cmd,
                        "EXIT") == 0)
            {
              pscriptcmd->msg = IDM_EXIT;
            }

            else
            {

              if (strcmpi(cmd,
                          "DELAY") == 0)
              {
                pscriptcmd->msg = SCR_DELAY;
              }

              else
              {

                if (cmd[0] == '*')
                {
                  pscriptcmd->msg = SCR_COMMENT;
                }

                else
                {
                  pscriptcmd->msg = SCR_BAD_CMD;
                }
              }
            }
          }
        }
        break;
      case  SCR_CMD_PARM :             /* command with parameter            */

        if (cmd[0] == '*')
        {
          pscriptcmd->msg = SCR_COMMENT;
        }

        else
        {
             /*
             ** QPROCESS parse
             */

          if (strcmpi(cmd,
                      "QPROCESS") == 0)
          {
                /*
                ** setup as     cmd until parm is verified
                */

            pscriptcmd->msg = SCR_BAD_CMD;

                /*
                ** search for the QProcessor
                ** need to fix this so we don't loop after
                ** we find the test case!
                */

            for (i = 0; i < NQProcessors; i++)
            {

              if (strcmpi(parm,
                          apQProcNames[i]) == 0)
              {
                pscriptcmd->msg = WM_CHOOSE_QPROCESS;
                pscriptcmd->usQProc = i;
                i = NQProcessors;      /* terminate loop                   */
              }
            }
          }

          else
          {
               /*
               ** DCTYPE parse
               */

            if (strcmpi(cmd,
                        "DCTYPE") == 0)
            {

              if (strcmpi(parm,
                          "OD_QUEUED") == 0)
              {
                pscriptcmd->msg = WM_CHOOSE_DCTYPE;
                pscriptcmd->lDCType = OD_QUEUED;
              }

              else
              {

                if (strcmpi(parm,
                            "OD_DIRECT") == 0)
                {
                  pscriptcmd->msg = WM_CHOOSE_DCTYPE;
                  pscriptcmd->lDCType = OD_DIRECT;
                }

                else
                {

                  if (strcmpi(parm,
                              "OD_INFO") == 0)
                  {
                    pscriptcmd->msg = WM_CHOOSE_DCTYPE;
                    pscriptcmd->lDCType = OD_INFO;
                  }

                  else
                  {

                    if (strcmpi(parm,
                                "OD_METAFILE") == 0)
                    {
                      pscriptcmd->msg = WM_CHOOSE_DCTYPE;
                      pscriptcmd->lDCType = OD_METAFILE;
                    }

                    else
                    {

                      if (strcmpi(parm,
                                  "OD_METAFILE_NOQUERY") == 0)
                      {
                        pscriptcmd->msg = WM_CHOOSE_DCTYPE;
                        pscriptcmd->lDCType = OD_METAFILE_NOQUERY;
                      }

                      else
                      {

                        if (strcmpi(parm,
                                    "OD_MEMORY") == 0)
                        {
                          pscriptcmd->msg = WM_CHOOSE_DCTYPE;
                          pscriptcmd->lDCType = OD_MEMORY;
                        }

                        else
                        {
                          pscriptcmd->msg = SCR_BAD_CMD;
                        }
                      }
                    }
                  }
                }
              }
            }

            else
            {
                 /*
                 ** DATATYPE parse
                 */

              if (strcmpi(cmd,
                          "DATATYPE") == 0)
              {

                if (strcmpi(parm,
                            "PM_Q_STD") == 0)
                {
                  pscriptcmd->msg = WM_CHOOSE_DATATYPE;
                  strcpy(pscriptcmd->szDataType,
                         "PM_Q_STD");
                }

                else
                {

                  if (strcmpi(parm,
                              "PM_Q_RAW") == 0)
                  {
                    pscriptcmd->msg = WM_CHOOSE_DATATYPE;
                    strcpy(pscriptcmd->szDataType,
                           "PM_Q_RAW");
                  }

                  else
                  {
                    pscriptcmd->msg = SCR_BAD_CMD;
                  }
                }
              }

              else
              {
                   /*
                   ** PSTYPE parse
                   */

                if (strcmpi(cmd,
                            "PSTYPE") == 0)
                {

                  if (strcmpi(parm,
                              "GPIT_MICRO") == 0)
                  {
                    pscriptcmd->msg = WM_CHOOSE_PSTYPE;
                    pscriptcmd->ulPSType = GPIT_MICRO;
                  }

                  else
                  {

                    if (strcmpi(parm,
                                "GPIT_NORMAL") == 0)
                    {
                      pscriptcmd->msg = WM_CHOOSE_PSTYPE;
                      pscriptcmd->ulPSType = GPIT_NORMAL;
                    }

                    else
                    {
                      pscriptcmd->msg = SCR_BAD_CMD;
                    }
                  }
                }

                else
                {
                     /*
                     ** PSUNITS parse
                     */

                  if (strcmpi(cmd,
                              "PSUNITS") == 0)
                  {

                    if (strcmpi(parm,
                                "PU_ARBITRARY") == 0)
                    {
                      pscriptcmd->msg = WM_CHOOSE_PSUNITS;
                      pscriptcmd->ulPSUnits = PU_ARBITRARY;
                    }

                    else
                    {

                      if (strcmpi(parm,
                                  "PU_PELS") == 0)
                      {
                        pscriptcmd->msg = WM_CHOOSE_PSUNITS;
                        pscriptcmd->ulPSUnits = PU_PELS;
                      }

                      else
                      {

                        if (strcmpi(parm,
                                    "PU_LOMETRIC") == 0)
                        {
                          pscriptcmd->msg = WM_CHOOSE_PSUNITS;
                          pscriptcmd->ulPSUnits = PU_LOMETRIC;
                        }

                        else
                        {

                          if (strcmpi(parm,
                                      "PU_HIMETRIC") == 0)
                          {
                            pscriptcmd->msg = WM_CHOOSE_PSUNITS;
                            pscriptcmd->ulPSUnits = PU_HIMETRIC;
                          }

                          else
                          {

                            if (strcmpi(parm,
                                        "PU_LOENGLISH") == 0)
                            {
                              pscriptcmd->msg = WM_CHOOSE_PSUNITS;
                              pscriptcmd->ulPSUnits = PU_LOENGLISH;
                            }

                            else
                            {

                              if (strcmpi(parm,
                                          "PU_HIENGLISH") == 0)
                              {
                                pscriptcmd->msg = WM_CHOOSE_PSUNITS;
                                pscriptcmd->ulPSUnits = PU_HIENGLISH;
                              }

                              else
                              {

                                if (strcmpi(parm,
                                            "PU_TWIPS") == 0)
                                {
                                  pscriptcmd->msg = WM_CHOOSE_PSUNITS;
                                  pscriptcmd->ulPSUnits = PU_TWIPS;
                                }

                                else
                                {
                                  pscriptcmd->msg = SCR_BAD_CMD;
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }

                  else
                  {
                       /*
                       ** CHOOSE test case parse
                       */

                    if (strcmpi(cmd,
                                "CHOOSE") == 0)
                    {
                          /*
                          ** setup as     cmd until parm is verified
                          */

                      pscriptcmd->msg = SCR_BAD_CMD;

                          /*
                          ** search for the test case
                          */

                      usSelect = SHORT1FROMMR(WinSendMsg(WinWindowFromID
                         (hwndTestCasesDlg,
                          DLG_TESTSLB),
                                                         LM_SEARCHSTRING,
                                                         MPFROM2SHORT((SHORT)
                                                         NULL,
                                                                     LIT_FIRST
                                                                      ),
                                                         MPFROMP(parm)));

                      if (usSelect != LIT_NONE)
                      {
                        pscriptcmd->msg = WM_CHOOSE_TESTCASE;
                        pscriptcmd->usTestCase = usSelect;
                        pSelectedTestCase = (PENTRYPOINT)WinSendMsg
                           (WinWindowFromID(hwndTestCasesDlg,
                                            DLG_TESTSLB),
                            LM_QUERYITEMHANDLE,
                            MPFROMSHORT(usSelect),
                            NULL);
                      }
                    }

                    else
                    {
                         /*
                         ** PRINTER_NAME parse
                         */

                      if (strcmpi(cmd,
                                  "PRINTER_NAME") == 0)
                      {

                            /*
                            ** setup as     cmd until parm is verified
                            */

                        pscriptcmd->msg = SCR_BAD_CMD;

                            /*
                            ** search for the printer name as defined in
                            ** print manager
                            ** need to fix this so we don't loop after
                            ** we find the printer name!
                            */

                        for (i = 0; i < NPrinters; i++)
                        {

                          if (strcmpi(parm,
                                      apPrinters[i]->aszPrinterName) == 0)
                          {
                            pscriptcmd->msg = WM_CHOOSE_DDRIVER;
                            pscriptcmd->usDDriver = i;
                            i = NPrinters; /* terminate loop              */
                          }
                        }
                      }

                      else
                      {
                           /*
                           ** PRINTER_QUEUE parse
                           */

                        if (strcmpi(cmd,
                                    "PRINTER_QUEUE") == 0)
                        {
                              /*
                              ** setup as     cmd until parm is verified
                              */

                          pscriptcmd->msg = SCR_BAD_CMD;

                              /*
                              ** search for the print queue and choose the
                              ** 1st printer we find
                              */

                          for (i = 0; i < NPrinters; i++)
                          {

                            if (strcmpi(parm,
                                        apPrinters[i]->aszQueueName) == 0)
                            {
                              pscriptcmd->msg = WM_CHOOSE_DDRIVER;
                              pscriptcmd->usDDriver = i;
                              i = NPrinters; /* terminate loop              */
                            }
                          }
                        }

                        else
                        {
                             /*
                             ** PRINTER_PORT parse
                             */

                          if (strcmpi(cmd,
                                      "PRINTER_PORT") == 0)
                          {

                               /*
                               ** setup as     cmd until parm is verified
                               */

                            pscriptcmd->msg = SCR_BAD_CMD;

                               /*
                               ** search for the printer port and choose the
                               ** 1st printer associated with it
                               */

                            for (i = 0; i < NPrinters; i++)
                            {

                              if (strcmpi(parm,
                                          apPrinters[i]->aszPortName) == 0)
                              {
                                pscriptcmd->msg = WM_CHOOSE_DDRIVER;
                                pscriptcmd->usDDriver = i;
                                i = NPrinters; /* terminate loop          */
                              }
                            }
                          }

                          else
                          {

                               /*
                               **     SCRIPT COMMAND
                               */

                            pscriptcmd->msg = SCR_BAD_CMD;
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
        break;
      case  SCR_NO_CMD :               /* the line was empty                */

        pscriptcmd->msg = SCR_BLANK_LINE;
        break;
      default  :

           /*
           ** we shouldn't get here, but just in case...
           */

        pscriptcmd->msg = SCR_BAD_CMD;
        break;
    }                                  /* end switch                        */
  }                                    /* GetScriptCmd                      */

