/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
/****************************************************************************/
/*                                                                          */
/*      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is         */
/*      sample code created by IBM Corporation. This sample code is not     */
/*      part of any standard or IBM product and is provided to you solely   */
/*      for  the purpose of assisting you in the development of your        */
/*      presentation drivers.  The code is provided "AS IS", without        */
/*      warranty of any kind.  IBM shall not be liable for any damages      */
/*      arising out of your use of the sample code, even if they have been  */
/*      advised of the possibility of such damages.                         */
/*                                                                          */
/****************************************************************************/
/****************************************************************************/
/*                                                                          */
/* PROGRAM NAME   : Font Test Print Queue Access Functions                  */
/* AUTHOR         : Matthew F. Rutkowski                                    */
/* FILENAME       : FTQUEUE.C                                               */
/* DATE WRITTEN   : 02-29-95                                                */
/* DESCRIPTION    : This module contains functions that help access         */
/*                  OS/2 Print Queue information.                           */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* Modification/Change log                                                  */
/*--------------------------------------------------------------------------*/
/* TAG - DATE - [COMPANY] - AUTHOR - DESCRIPTION                            */
/*--------------------------------------------------------------------------*/
/*                                                                          */
/****************************************************************************/

#define INCL_DEV
#define INCL_DOSFILEMGR
#define INCL_DOSMEMMGR
#define INCL_DOSPROCESS
#define INCL_ERRORS
#define INCL_GPI
#define INCL_GPIERRORS
#define INCL_SPL
#define INCL_SPLDOSPRINT
#define INCL_WIN

#include <os2.h>
#include <stdlib.h>
#include <stdio.h>

#include "ft32.h"
#include "ftdlg.h"

USHORT FillQueNameListBox( HAB, HWND, PQUEUEDATA );
BOOL   GetDefaultQueueData( HAB, PQUEUEDATA, PPRINTERDATA );
APIRET GetQueueData( HAB, PQUEUEDATA, PPRINTERDATA, BOOL );

/****************************************************************************/
/* PROCEDURE NAME : GetDefaultQueueData                                     */
/* AUTHOR         : MFR                                                     */
/* DATE WRITTEN   : 05/04/93                                                */
/* DESCRIPTION    : If the user never selects a queue using the "Select     */
/*                  Queue..." dialog this routine will be called to         */
/*                  search the OS2.INI file for the system default queue.   */
/*                                                                          */
/*                  Control Flow:                                           */
/*                  (a) Query OS2.INI for default queue information.        */
/*                                                                          */
/* PARAMETERS:      (HAB)  hab - anchor block handle for error processing   */
/*                                                                          */
/* RETURN VALUES:   (BOOL) ErrorOccurred == FALSE - Successful completion   */
/*                                       == TRUE  - Failure error occurred  */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* CHANGE/MODIFICATION LOG :                                                */
/*--------------------------------------------------------------------------*/
/*                                                                          */
/****************************************************************************/
BOOL GetDefaultQueueData( HAB hab, PQUEUEDATA pqd, PPRINTERDATA prd )
{
  /*------------------------------------------------------------------------*/
  /* LOCAL VARIABLES                                                        */
  /*------------------------------------------------------------------------*/
  CHAR          szBuffer[MAXSTR];      // Buffer for profile string
  BOOL          ErrorOccurred = FALSE; // Error assertion return code
  ULONG         ulReturn;              // Size to allocate for SplQueryQueue

  /*------------------------------------------------------------------------*/
  /* BEGIN CODE                                                             */
  /*------------------------------------------------------------------------*/

  /*-------------------------------------------------*/
  /* QUEUE INFO                                      */
  /*-------------------------------------------------*/

  /* Get default printer name since none selected */
  ulReturn = PrfQueryProfileString( HINI_USER, "PM_SPOOLER", "PRINTER",
                                    ";", szBuffer, (ULONG)sizeof(szBuffer));

  ErrorOccurred = PMassertp( ulReturn == 0, hab,
                             ERR_PRFQRYSTRING_PRINTER , FALSE );

  // Able to query the printer entry
  if( !ErrorOccurred )
  {
    StrToken( prd->szPrinterName,  szBuffer, ';', 1 );

    /* Get default printer name since none selected */
    ulReturn = PrfQueryProfileString( HINI_USER, "PM_SPOOLER", "QUEUE",
                                      ";", szBuffer, (ULONG)sizeof(szBuffer));

    ErrorOccurred = PMassertp( ulReturn == 0, hab,
                              ERR_PRFQRYSTRING_QUEUE , FALSE );

    // Able to query the queue entry
    if( !ErrorOccurred )
    {

      StrToken( pqd->QueueName, szBuffer, ';', 1 );

    } /* end if */

  } /* end if */

  return( ErrorOccurred );

} /* GetDefaultQueueData */


/****************************************************************************/
/* PROCEDURE NAME : GetQueueData                                            */
/* AUTHOR         : MFR                                                     */
/* DATE WRITTEN   : 12/20/92                                                */
/* DESCRIPTION    : Called whenever we are going to access a device on a    */
/*                  a queue (printing, queries etc) to retrieve all details */
/*                  about that queue (drivername, printername etc.).        */
/*                                                                          */
/*                  Control Flow:                                           */
/*                  (a) call GetDefaultQueueData(); if no queue selected    */
/*                  (b) Check for server information                        */
/*                  (c) call SplQueryQueue(); to get driver data from queue */
/*                                                                          */
/* PARAMETERS:      (HAB)  hab - anchor block handle for error processing   */
/*                                                                          */
/* RETURN VALUES:   (BOOL) ErrorOccurred == FALSE - Successful completion   */
/*                                       == TRUE  - Failure error occurred  */
/*                                                                          */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* CHANGE/MODIFICATION LOG :                                                */
/*--------------------------------------------------------------------------*/
/*                                                                          */
/****************************************************************************/
APIRET GetQueueData( HAB hab, PQUEUEDATA pqd, PPRINTERDATA prd, BOOL bResetDrivData )
{
  /*------------------------------------------------------------------------*/
  /* LOCAL VARIABLES                                                        */
  /*------------------------------------------------------------------------*/
  APIRET        apiret = NO_ERROR;     // Error assertion return code
  PBYTE         pbuf;                  // pointer to printer queue info
  PPRQINFO3     prq;                   // pointer to PRQINFO3 struct
  CHAR          szDDL[MAXSTR];         // Buffer for driver name
  CHAR          szQL[MAXSTR];          // Buffer of Queue names
  CHAR          szServer[MAXSTR];      // Buffer for Server name
  SPLERR        SplErr;                // Return code for DosPrintQGetInfo
  ULONG         ulSize;                // Size to allocate for SplQueryQueue
  BOOL          bError = FALSE;

  /*------------------------------------------------------------------------*/
  /* BEGIN CODE                                                             */
  /*------------------------------------------------------------------------*/

  /*-------------------------------------------------*/
  /* QUEUE INFO                                      */
  /*-------------------------------------------------*/
  szServer[0] = '\0';

  /* Check to see if a printer queue was selected */
  if( pqd->QueueName[0] == 0 )
  {
    bError = GetDefaultQueueData( hab, pqd, prd );

    // validate queue name even if Prf queries work
    if( bError = (pqd->QueueName[0] == 0) )
    {
       PMassertp( TRUE, hab, ERR_NO_DEFAULT_QUEUE, FALSE );

    } /* end if */

  } /* end if */

  if( !bError )
  {
    /* Check to see if its a network queue */
    if( pqd->QueueName[0] != '\\' )
    {
      strcpy( szQL, pqd->QueueName );
    } /* end if */
    else
    {
      szServer[0] = '\\';
      szServer[1] = '\\';
      StrToken( szServer+2, pqd->QueueName, '\\', 3 );
      StrToken( szQL, pqd->QueueName, '\\', 4 );
    } /* end else */

    // TBD - validate queue still exists - user may have deleted it.

    /* Get the Queue info */
    if( SplErr = SplQueryQueue( szServer, szQL, 3L, (PVOID)NULL, 0L, &ulSize ))
    {
      if( DosAllocMem( (PPVOID)&pbuf, ulSize, PAG_READ |
                       PAG_WRITE | PAG_COMMIT ) == NO_ERROR )
      {
         if( SplQueryQueue( szServer, szQL, 3L, (PBYTE)pbuf,
                            ulSize, &ulSize) == NO_ERROR )
         {
           // Format the data into PRQINFO3 structure
           prq = (PPRQINFO3)pbuf;

           // Get Queue Description for the queue name
           strcpy( pqd->QueueDesc, prq->pszComment );

           // Get default driver for queue
           strcpy( szDDL, prq->pszDriverName );
           StrToken( prd->szDriverName, szDDL, '.', 1 );
           StrToken( prd->szDeviceName, szDDL, '.', 2 );

           // Get all printers assigned to queue
           StrToken( prd->szPrinterName, prq->pszPrinters, ',', 1 );

           // if we have previously allocated a copy of Driver Data
           if( pqd->pDrivData )
           {
             // if Driver Data is a different size than what was there before OR
             // we were asked to reset the driver data to a known state
             if( (pqd->pDrivData->cb != prq->pDriverData->cb) || bResetDrivData )
             {
               // free old drivdata memory
               DosFreeMem( pqd->pDrivData );
               pqd->pDrivData = NULL;

             } /* end if */

           } /* end if */

           // If no DRIVDATA allocate and copy it from spooler queue info strucs
           if( pqd->pDrivData == NULL )
           {
             DosAllocMem( (PPVOID)&(pqd->pDrivData),
                           prq->pDriverData->cb,
                           PAG_READ | PAG_WRITE | PAG_COMMIT );

             memcpy( pqd->pDrivData, prq->pDriverData, prq->pDriverData->cb );
           } /* end if */

         } /* end if */
         else
         {
           bError = PMassertp( TRUE, hab,(PSZ)ERR_SPLQUERYQUEUE, FALSE );

         } /* end else */

         // Free up memory from PRQINFO3 struct
         DosFreeMem( pbuf );

      } /* end if */
      else
      {
        bError = PMassertp( TRUE, hab, (PSZ)ERR_DOSALLOCMEM, FALSE );

      } /* end else */

    } /* end if */
    else
    {
      bError = PMassertp( TRUE, hab, (PSZ)ERR_SPLQUERYQUEUE, FALSE );

    } /* end else */

  } /* end if */

  if(bError)
  {
    apiret = ERROR_QUE_NOT_INITIALIZED;
  } /* end if */

  return( apiret );

} /* GetQueueData */


/****************************************************************************/
/* PROCEDURE NAME : FillQueNameListBox                                      */
/* AUTHOR         : Matt R.                                                 */
/* DATE WRITTEN   : 5/6/92                                                  */
/* DESCRIPTION    : Fills a listbox in the Queue dialog with all available  */
/*                  queues found in the system (LOCAL and REMOTE).          */
/*                                                                          */
/*                  Control Flow:                                           */
/*                  (a) call SplEnumQueue(); to enumerate all queues        */
/*                      visible from this workstation.                      */
/*                                                                          */
/*                  (b) Display each queue's comment (desktop name) and     */
/*                      internal queue name in a listbox.                   */
/*                                                                          */
/*                  (c) loop through all queue names and highlite the       */
/*                      one the user selected previously or the system      */
/*                      default queue.                                      */
/*                                                                          */
/*                                                                          */
/* PARAMETERS:      (HAB)    hab  - caller's anchor block                   */
/*                  (HWND)   hwnd - caller's window handle                  */
/*                                                                          */
/* RETURN VALUES:   (USHORT) ulQueuesReturned - Number of queues found      */
/*                                                                          */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* CHANGE/MODIFICATION LOG :                                                */
/*--------------------------------------------------------------------------*/
/*                                                                          */
/* NOTE: change return value to ULONG                                       */
/*                                                                          */
/****************************************************************************/
USHORT FillQueNameListBox( HAB hab, HWND hwnd, PQUEUEDATA pqd )
{
  /*------------------------------------------------------------------------*/
  /* LOCAL VARIABLES                                                        */
  /*------------------------------------------------------------------------*/
  PBYTE     pbPrqInfoBuf;                   // Buffer for PRQINFO3
  BOOL      MatchFound = FALSE;             // Internal queue name match found
  CHAR      pszListboxQueName[MAXSTR];      // Listbox Queue Name buffer
  CHAR      pszInternalQueName[MAXSTR];     // Internal Queue Name buffer
  CHAR      pszDefaultQueName[MAXSTR];      // OS/2 Default Queue Name buffer
  CHAR      pszPrfBuffer[MAXSTR];           // Profile String buffer
  PPRQINFO3 pPrqInfo3;                      // Pointer PRQINFO3 structs
  SPLERR    rc;                             // Spooler Return Code
  ULONG     ulCurrentQueue;                 // Loop control for current queue
  ULONG     ulBufSize;                      // size of pszPrqInfoBuf
  ULONG     ulQueuesReturned = 0;           // # Qs returned by DosPrintQEnum
  ULONG     ulQueuesTotal    = 0;           // # Qs available by DosPrintQEnum
  ULONG     ulNeeded         = 0;           // Memory needed for Queue info

  /*------------------------------------------------------------------------*/
  /* BEGIN CODE                                                             */
  /*------------------------------------------------------------------------*/

  // Clear out the listbox
  WinSendDlgItemMsg( hwnd, IDCB_PRINT_QUEUES, LM_DELETEALL, NULL, NULL );

  /*--------------------------------------------------------*/
  /* Find out how much to allocate for Queue Info           */
  /*--------------------------------------------------------*/
  rc = SplEnumQueue( (PSZ)NULL, 3L, pbPrqInfoBuf, 0L, &ulQueuesReturned,
                     &ulQueuesTotal, &ulNeeded, NULL);

  // If memory needed (i.e. queues exist and memory allocation works
  if( ulNeeded &&
      DosAllocMem( (PPVOID)(&pbPrqInfoBuf), ulNeeded,
                   PAG_READ | PAG_WRITE | PAG_COMMIT ) == NO_ERROR )
  {
    // We have allocated what we needed so set that as our buffer size
    ulBufSize = ulNeeded;

    /*------------------------------------------------------*/
    /* Enumerate all local or aliases of remote queues      */
    /*------------------------------------------------------*/
    if( rc = SplEnumQueue( (PSZ)NULL, 3L, pbPrqInfoBuf, ulBufSize,
                            &ulQueuesReturned, &ulQueuesTotal,
                            &ulNeeded, (PVOID)NULL ) == NO_ERROR )
    {
      /*------------------------------------------------------*/
      /* Display all Queue names found to listbox             */
      /*------------------------------------------------------*/
      if ( ulQueuesReturned )
      {
        HWND hwndLB = WinWindowFromID( hwnd, IDCB_PRINT_QUEUES );

        ulCurrentQueue = 0;

        /* Get pointer to begining of first PRQINFO3 struct */
        pPrqInfo3 = (PPRQINFO3)pbPrqInfoBuf;

        /* while there are still queues in the PrqInfoBuf */
        while ( ulCurrentQueue < ulQueuesReturned )
        {
          /*----------------------------------------------------*/
          /* Format queue name displayed in listbox             */
          /* to show first the name seen under icon then        */
          /* actual internal queue name                         */
          /*----------------------------------------------------*/
          sprintf( pszListboxQueName, "%s %c%s%c",
                   pPrqInfo3->pszComment,
                   LEAD_QUEUE_TOKEN,
                   pPrqInfo3->pszName,
                   TRAIL_QUEUE_TOKEN );

          /* Insert Hybrid Queue name into end of listbox */
          WinInsertLboxItem( hwndLB, LIT_END, pszListboxQueName );

          /* increment current queue */
          ulCurrentQueue++;

          /* increment to next PRQINFO3 struct */
          pPrqInfo3++;

        } /* end while */

        /*------------------------------------------------------*/
        /* Highlite queue selected by user as FT's default      */
        /*------------------------------------------------------*/
        if( pqd->QueueName[0] != '\0' )
        {
           ulCurrentQueue = 0;

           /* reset PRQINFO3 buffer pointer */
           pPrqInfo3 = (PPRQINFO3)pbPrqInfoBuf;

           while( ulCurrentQueue < ulQueuesReturned )
           {
              /* Get the internal queue name from PRQINFO3 struct */
              strcpy( pszInternalQueName, pPrqInfo3->pszName );

              /* If we find an internal queue name match */
              if( strcmp( pqd->QueueName, pszInternalQueName ) == 0)
              {
                 WinSendDlgItemMsg( hwnd,
                                    IDCB_PRINT_QUEUES,
                                    LM_SELECTITEM,
                                    MPFROMSHORT(ulCurrentQueue),
                                    MPFROMSHORT((SHORT)TRUE));

                 /* Indicate match found */
                 MatchFound = TRUE;

                 /* Quit looping */
                 break;

              } /* end if */

              /* increment current queue */
              ulCurrentQueue++;

              /* increment to next PRQINFO3 struct */
              pPrqInfo3++;

           } /* end while */

        } /* end if */

        /*------------------------------------------------------*/
        /* If no FT default use OS/2's default queue            */
        /*------------------------------------------------------*/
        if( !MatchFound )
        {
          PrfQueryProfileString( HINI_USER, "PM_SPOOLER", "QUEUE", NULL,
                                 pszPrfBuffer, MAXSTR );

          StrToken( pszDefaultQueName, pszPrfBuffer, ';', 1 );

          if ( pszDefaultQueName[0] != '\0' )
          {
             ulCurrentQueue = 0;

             /* reset PRQINFO3 buffer pointer */
             pPrqInfo3 = (PPRQINFO3)pbPrqInfoBuf;

             while( ulCurrentQueue < ulQueuesReturned )
             {
                /* Get the internal queue name from PRQINFO3 struct */
                strcpy( pszInternalQueName, pPrqInfo3->pszName );

                if( strcmp( pszDefaultQueName, pszInternalQueName ) == 0)
                {
                   WinSendDlgItemMsg( hwnd,
                                      IDCB_PRINT_QUEUES,
                                      LM_SELECTITEM,
                                      MPFROMSHORT(ulCurrentQueue),
                                      MPFROMSHORT((SHORT)TRUE));

                   /* Get out of loop we found a match */
                   break;

                } /* end if */

                /* increment current queue */
                ulCurrentQueue++;

                /* increment to next PRQINFO3 struct */
                pPrqInfo3++;

             } /* end while */

          } /* end if */

        } /* end if */

      } /* end if */

    } /* end if */
    else
    {
        PMassertp( TRUE, hab, (PSZ)ERR_SPLENUMQUEUE, FALSE );

    } /* end else */

    // Free up the queue info buffer
    DosFreeMem( pbPrqInfoBuf );

  } /* end if */
  else
  {
    if(ulNeeded)
      PMassertp( TRUE, hab, (PSZ)ERR_DOSALLOCMEM, FALSE );

  } /* end else */

  return( ulQueuesReturned );

} /* end FillQueNameListBox */

