/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/

/*
**   DISCLAIMER OF WARRANTIES.  The following enclosed code is
**   sample code created by IBM Corporation. This sample code is not
**   part of any standard or IBM product and is provided to you solely
**   for  the purpose of assisting you in the development of your
**   applications.  The code is provided "AS IS", without
**   warranty of any kind.  IBM shall not be liable for any damages
**   arising out of your use of the sample code, even if they have been
**   advised of the possibility of such damages.
*/

/**********************************************************************/
/* program to display the current hardware palette                    */
/**********************************************************************/
#define PROGNAME "H/W Palette Display"

#define INCL_WIN
#define INCL_PM
#define INCL_DOS
#include <os2.h>

#include <stdio.h>
#include "paldisp.h"

/**********************************************************************/
/* function prototypes                                                */
/**********************************************************************/
PFNTHREAD SecondThread(VOID);
MRESULT EXPENTRY ClientWndProc(HWND,USHORT,MPARAM,MPARAM);
MRESULT EXPENTRY DlgProc(HWND,USHORT,MPARAM,MPARAM);

VOID PaintWindow (HWND);
VOID CheckMenuItem (HWND, SHORT, BOOL);
VOID EnableMenuItem (HWND, SHORT, BOOL);

/**********************************************************************/
/* global variables and structures                                    */
/**********************************************************************/
HAB     hab;
HWND    hwndClient;

USHORT  xsize,ysize;
LONG   screencolours;
LONG   bpp;

HDC     hdc;

/**********************************************************************/
/* event semaphore handle                                             */
/**********************************************************************/
HEV     hev;
ULONG   dummy;

/**********************************************************************/
/* 2.0 palette manager globals                                        */
/**********************************************************************/
HPAL    explicit_hpal;
RGB2    explicitpalette[256];

INT main (VOID)
{
    static  CHAR szClassName[] = "Palette";
    static  CHAR szApplicatName[] = PROGNAME;
    HMQ     hmq;
    HWND    hwndFrame;
    QMSG    qmsg;
    ULONG   fcf       = FCF_SIZEBORDER |
                        FCF_TITLEBAR |
                        FCF_SYSMENU |
                        FCF_MINMAX |
                        FCF_MENU   |
                        FCF_SHELLPOSITION;
    USHORT  i;

    hab = WinInitialize(0);
    hmq = WinCreateMsgQueue(hab,0);

    WinRegisterClass (hab,
                      szClassName,
                      (PFNWP)ClientWndProc,
                      CS_SIZEREDRAW,
                      0);

    hwndFrame = WinCreateStdWindow(HWND_DESKTOP,
                                   WS_VISIBLE,
                                   &fcf,
                                   szClassName,
                                   szApplicatName,
                                   WS_VISIBLE,
                                   NULLHANDLE,
                                   ID_RESOURCE,
                                   &hwndClient);

    /******************************************************************/
    /* find how many bpp the driver supports                          */
    /******************************************************************/
    DevQueryCaps(hdc, CAPS_COLOR_BITCOUNT,
                 1L,&bpp);

    /******************************************************************/
    /* Create a DC for the Client window.                             */
    /******************************************************************/
    hdc = WinOpenWindowDC(hwndClient);

    /******************************************************************/
    /* palettes are global objects so create one now                  */
    /******************************************************************/
    DevQueryCaps(hdc, CAPS_COLORS, 1L, &screencolours);

    /**********************************************************/
    /* create a new palette of the right size                 */
    /**********************************************************/
    for (i=0; i<(USHORT)screencolours; i++)
    {
        explicitpalette[i].fcOptions = PC_EXPLICIT;
        explicitpalette[i].bRed   = 0;
        explicitpalette[i].bGreen = 0;
        explicitpalette[i].bBlue  = (BYTE)i;
    }

    explicit_hpal = GpiCreatePalette(
        hab,
        LCOL_PURECOLOR,    /* purecolor, or override def cols */
        LCOLF_CONSECRGB,   /* only allowed format !           */
        screencolours,     /* number of entries               */
        (PULONG)explicitpalette );

    while (WinGetMsg(hab, &qmsg, NULLHANDLE, 0, 0))
    {
        WinDispatchMsg(hab,&qmsg);
    }

    /******************************************************************/
    /* destory the global palette                                     */
    /******************************************************************/
    GpiDeletePalette( explicit_hpal );

    WinDestroyWindow(hwndFrame);
    WinDestroyMsgQueue(hmq);
    WinTerminate(hab);

    return 0;
}

MRESULT EXPENTRY ClientWndProc(HWND hwnd, USHORT msg,
                                 MPARAM mp1, MPARAM mp2)
{
    switch (msg)
    {
        case WM_SIZE :
             xsize = LOUSHORT (mp2) ;
             ysize = HIUSHORT (mp2) ;
             break ;

        case WM_COMMAND:
            switch (LOUSHORT(mp1))
            {
                case ABOUT:
                    WinDlgBox( HWND_DESKTOP
                              , hwnd
                              , (PFNWP) DlgProc
                              , NULLHANDLE
                              , IDD_ABOUT
                              , NULL
                              );
                    break;

                case PAINT:
                    /**************************************************/
                    /* force a repaint - used for testing purposes    */
                    /**************************************************/
                    WinInvalidateRect(hwnd, NULL, FALSE);

                default:
                    break;
            }
        break;

        case WM_REALIZEPALETTE:
            /**********************************************************/
            /* We do not need to do any thing when asked to realize   */
            /* the palette because we are only using explicit entries.*/
            /* But we should process the message because otherwise    */
            /* the default message handler will try to realize the    */
            /* default palette.                                       */
            /**********************************************************/
            break;

        case WM_PAINT:
            PaintWindow (hwnd);
            break;

        default:
            return WinDefWindowProc (hwnd, msg, mp1, mp2);
     }
     return 0L;
}


MRESULT EXPENTRY DlgProc (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
{
switch(msg)
   {
   case WM_COMMAND:
        switch (LOUSHORT(mp1))
        {
           case DID_OK:
           case DID_CANCEL:
                WinDismissDlg(hwnd, TRUE);
                return 0;
        }
        break;
   }
return WinDefDlgProc( hwnd, msg, mp1, mp2 );
}

VOID CheckMenuItem (HWND hwnd, SHORT iMenuItem, BOOL bCheck)
{
    HWND hwndParent = WinQueryWindow(hwnd, QW_PARENT);
    HWND hwndMenu   = WinWindowFromID (hwndParent, (ULONG)FID_MENU);

    WinSendMsg (hwndMenu, MM_SETITEMATTR,
        (MPARAM)(MAKEULONG (iMenuItem, TRUE)),
        (MPARAM)(MAKEULONG (MIA_CHECKED, bCheck ? MIA_CHECKED : 0))) ;
}

VOID EnableMenuItem (HWND hwnd, SHORT iMenuItem, BOOL bEnable)
{
    HWND hwndParent = WinQueryWindow(hwnd, QW_PARENT);
    HWND hwndMenu   = WinWindowFromID (hwndParent, (ULONG)FID_MENU);

    WinSendMsg (hwndMenu, MM_SETITEMATTR,
         (MPARAM)(MAKEULONG (iMenuItem, TRUE)),
         (MPARAM)(MAKEULONG (MIA_DISABLED, bEnable ? 0 : MIA_DISABLED))) ;
}

VOID PaintWindow (HWND hwnd)
{
    HPS         hps;
    ULONG       changed;
    USHORT      Box;
    RECTL       rect;

    hps = WinBeginPaint(hwnd, NULLHANDLE, NULL);
    /******************************************************************/
    /* select the palette                                             */
    /******************************************************************/
    GpiSelectPalette( hps, explicit_hpal );

    /******************************************************************/
    /* realize the palette                                            */
    /******************************************************************/
    WinRealizePalette( hwnd, hps, &changed );

    /******************************************************************/
    /* clear the window                                               */
    /******************************************************************/
    GpiErase(hps);


    /**********************************************************/
    /* now draw one bar for each of the colours in the        */
    /* devices hardware palette                               */
    /**********************************************************/
    for (Box = 0;
         Box < (USHORT)screencolours;
         Box++)
    {
        /******************************************************/
        /* draw a box in this colour                          */
        /******************************************************/
        rect.xLeft   = Box * xsize/screencolours;
        rect.yBottom = 0;
        rect.xRight  = (Box+1) * xsize/screencolours;
        rect.yTop    = ysize;
        WinFillRect (hps, (PRECTL)&rect, Box);
    }

    /**********************************************************/
    /* deselect the palette                                   */
    /**********************************************************/
    GpiSelectPalette( hps, 0L );

    WinEndPaint(hps);
}

