/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
/*********************** START OF SPECIFICATIONS *******************************
*
* SOURCE FILE NAME: PMADDE.C
*
* DESCRIPTIVE NAME:Presentation Manager Driven Audio Device Driver Exerciser.
*
* FUNCTION: Exercise Multimedia Audio Stream Handler.
*
* NOTES:
*
* EXTERNAL ENTRY POINTS: None.
*
* INTERNAL ENTRY POINTS: None.
*
* EXTERNAL REFERENCES (system):
*
* APIs CALLED:
* ------------
*   WinInitialize
*   WinCreateMsgQueue
*   WinRegisterClass
*   WinCreateHelpInstance
*   WinDestroyHelpInstance
*   WinCreateStdWindow
*   WinGetLastError
*   WinShowWindow
*   WinAssociateHelpInstance
*   WinGetMsg
*   WinDispatchMsg
*   WinReleasePS
*   WinDestroyHelpInstance
*   WinDestroyWindow
*   WinDestroyMsgQueue
*   WinTerminate
*   WinRealizePalette
*   GpiCreatePS
*   WinQueryWindowRect
*   WinPostMsg
*   WinWindowFromID
*   WinInvalidateRect
*   WinDlgBox
*   WinSendMsg
*   WinBeginPaint
*   WinFillRect
*   WinEndPaint
*   WinDefWindowProc
*   WinDismissDlg
*   WinDefDlgProc
*   DosExit
*   WinAlarm
*   WinMessageBox
*
*
* WHAT YOU NEED TO COMPILE THIS PROGRAM:
* --------------------------------------
*
* REQUIRED FILES:
* ---------------
*
*   PMADDE.C      - Source code
*   PMADSPI.C     - PMADDE SPI Calls module
*   PMADLEX.C     - PMADDE lexical Analyzer
*   PMADDE.H      - PMADDE Data Structure definitions
*   PMADEXT.H     - External definitions header file
*   PMADGLOB.H    - Global definitions header file
*   PMADLEX.H     - Lexical definitions header file
*   PMADDE.IPF    - Documentation file for this program
*   PMADDE.DLG    - Dialog file
*   PMADDE.RC     - Resource file
*   PMADDEV.RC    - Device Specific Resource
*   PMADDE.MAK    - Make file for this program
*
*   OS2.H          - Presentation Manager include file
*   STDLIB.H       - Standard library function declarations
*   OS2ME.H        - MultiMedia System include
*
* REQUIRED LIBRARIES:
* -------------------
*
*   OS2386.LIB     - OS/2 API Library
*   MMPM2.LIB      - MMPM/2 API Library
*
* REQUIRED PROGRAMS:
* ------------------
*
*   IBM C Set/2 Compiler (icc.exe    )
*   IBM Linker           (link386.exe)
*   Resource Compiler    (rc.exe     )
*   Help Compiler        (ipfc.exe   )
*
*
*/

#define INCL_DOS                       /* DosCall Definitions include     */
#define INCL_PM                        /* Presentation Manager include    */
#define INCL_WIN                       /* OS/2 2.x Win  include           */
#define INCL_DOSSEMAPHORES             /* OS/2 2.x Semaphores             */
#include <os2.h>                       /* OS/2 2.x System include         */
#include <stdio.h>
#include <stdlib.h>                    /* C Standard library include      */
#include <string.h>                    /* C String Functions include      */
#include <stdarg.h>                    /* C Standard Arguments include    */
#include <os2me.h>                     /* Multimedia System include       */
#include <audio.h>                     /* Multimedia Audio Driver include */
#include "pmadde.h"                    /* Component Private Definitions   */
#include "pmadglob.h"                  /* Component Global Definitions    */

CHAR szMainWin[]="MainWin";            /* Main window class title         */
CHAR szActiveHndlrs[]="ActiveHndlrs";  /* Active handlers class title     */
CHAR szInstldHndlrs[]="InstldHndlrs";  /* Installed handlers class title  */
CHAR szMonitor[]="EventMonitor";       /* Event monitor window class title*/
CHAR szCmndMonitor[]="CmndMonitor";    /* Cmnd nonitor window class title */
CHAR szTitle[]="Audio Device Driver Exerciser V1.0"; /* Application title */

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    main
*
* ARGUMENTS:  argc, argv, envp.
*
* RETURN:     None.
*
* DESCRIPTION: PMADDE main entry point.
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
INT main(LONG argc,PSZ argv[])
{
   QMSG   qmsg;                /* Message Queue                */
   HMQ    hmq;                 /* Handle to Queue              */
   HPS    hps;                 /* Handle to Presentation Space */
   int    i;                   /* Local Loop Index             */
   RECTL  rectl;               /* Recatngle Structure          */
   static ULONG flCreate;      /* Window Creation Flags        */
   APIRET rc;                  /* Return Code                  */

   /* initialize global variables */

   ipt = 0;
   iptComMle = 0;
   iptEveMle = 0;
   ulNumHand = 20;
   HandlerCount = 0;
   StreamCount = 0;
   fScriptMode = FALSE ;
   memset (&fm,0,sizeof(FONTMETRICS)); /* clear the fontmetrics structure */
   sACPAOpened = 0;                    /* flag for driver open or close   */

   hab = WinInitialize((USHORT) NULL); /* Intialize PM */
   hmq = WinCreateMsgQueue(hab,0);     /* Create default size message queue */

   /* Register pmadde main window proc */

   WinRegisterClass( hab,
                     szMainWin,
                     (PFNWP) PMADDEWinProc,
                     CS_SIZEREDRAW,
                     0 );

   /* Register active handlers window proc */

    WinRegisterClass( hab,
                       szActiveHndlrs,
                       (PFNWP) ActiveHndlrsWinProc,
                       CS_SIZEREDRAW,
                       0 );

    /* Register command monitor window proc */

    WinRegisterClass( hab,
                      szCmndMonitor,
                      (PFNWP) CmndMonitorWinProc,
                      CS_SIZEREDRAW,
                      0 );

    /* Register event monitor window proc */

    WinRegisterClass( hab,
                      szMonitor,
                      (PFNWP) EventMonitorWinProc,
                      CS_SIZEREDRAW,
                      0 );

   /* Set main window creation flags */

   flCreate = FCF_SIZEBORDER | FCF_MENU  | FCF_MINMAX |
              FCF_SYSMENU | FCF_TITLEBAR | FCF_TASKLIST |
              FCF_ICON | FCF_SHELLPOSITION;

    /* Initialize Global Data Structures */

    Initialize();

    /*
    ** Command line arguments are accepted. Two arguments accepted
    ** are the script file name and the logname. Both these file names
    ** should be provided.
    */

    if (argc > 1) {
       if (argc != 3)
          mprintf ("PMADDE ScriptFile LogFile \n");
       if (argc == 3)
       {
         strcpy (szScriptFile,argv[1]);
         strcpy (szLogFileName,argv[2]);
         memset (&FileInfoStr,0,sizeof(FileInfoStr));

         rc = DosQueryPathInfo (szScriptFile,
                                0x0001,
                                &FileInfoStr,
                                sizeof(FileInfoStr));
         /*
         ** If the script file doesnot exist, create a script file
         ** and also the log file.  The actions should be built into
         ** the script file so that it can be run the next time.
         */

         if (rc)
         {
             OpenFile(szScriptFile,&hfileScript);
             WriteScriptFile(INIT);
         }
         else
            fScriptMode = TRUE ;

         OpenFile(szLogFileName, &hLogFile);

       } /* argc == 3 */
    }  /* argc > 1 */

    /* Create the main window */

    hwnd1 = WinCreateStdWindow( HWND_DESKTOP, WS_VISIBLE, &flCreate,
                                szMainWin, szTitle, 0L, NULLHANDLE,
                                ID_MAINWIN, (PHWND) & hwndc1         );

    if ( !hwnd1 )
       mprintf ("Main Window Creation Error");

    /* Query the co-ordinates for the desktop window */

    WinQueryWindowRect ( HWND_DESKTOP,&rectl );

    /* Set the window position */
    if ( hwndHelpInstance )
      WinAssociateHelpInstance( hwndHelpInstance, hwnd1 );

    /*
    ** If we are in script mode enable that interface to
    ** run the current script file specified in the command
    ** line argument.
    */

    if (fScriptMode)
    {
        WinPostMsg(hwnd1, WM_COMMAND,MPFROMSHORT(IDM_SCRIPT), MPVOID);
    }

    /* Message Loop */

    while ( WinGetMsg(hab,(PQMSG) &qmsg,(HWND) NULL,0,0))
    WinDispatchMsg(hab,(PQMSG) &qmsg );

    Terminate (hmq);  /* Terminate Application */
    return(0);
}  /* of main */

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: PMADDEWinProc()
*
* ARGUMENTS: Standard OS/2 PM Win Proc arguments
*
* RETURN:
*
* DESCRIPTION: This is the main window that interfaces with the user.
* it routes user choices to various internal routines to perform
* specific data streaming tasks.
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:  GetStrmHndlrDlgProc()
*                                CreateStrmDlgProc(),
*                                DestroyStrmDlgProc(),
*                                AssociateStrmDlgProc(),
*                                SeekStrmDlgProc(),
*                                StartStrmDlgProc(),
*                                StopStrmDlgProc(),
*                                InstallProtoDlgProc(),
*                                InstldHndlrsWinProc(),
*                                ActiveHndlrsWinProc(),
*                                EventMonitorWinProc(),
*                                CmndMonitorWinProc().
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY PMADDEWinProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
   LONG   lPresParam;                    /* Presenation Parameter */
   HPS    hps;                           /* Presentation Space    */
   RECTL  rectl, rctlSize;               /* Client rectangles     */
   int    i ;                            /* loop index            */
   ULONG  flCreate1;                     /* Window Creation flags */
   APIRET RetCode;                       /* API return code       */
   char szFullFile[CCHMAXPATH] ="*.LOG"; /* Filename holder       */

   switch( msg )
   {

     case WM_HELP:
         if (hwndHelpInstance)
             DisplayHelpPanel(OPEN);
        break;

     case WM_COMMAND:
      switch ( SHORT1FROMMP( mp1 ) )
      {

        case IDM_GETHANDLER:                  /* Get Handler */
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) GetStrmHndlrDlgProc,
                         hModPMADDE, ID_GETHNDLRDLG, NULL );
             break;

        case IDM_CREATE:                      /* Create   */
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) CreateStrmDlgProc, NULLHANDLE,
                         ID_CREATEDLG, NULL );
             break;

        case IDM_ASSOCIATE:                   /* Associate */
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) AssociateStrmDlgProc,
                         hModPMADDE, ID_ASSOCDLG, NULL );
             break;

        case IDM_START:                       /* Start */
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP)StartStrmDlgProc, hModPMADDE,
                         ID_STARTDLG, NULL );
             break;

        case IDM_STOP:                        /* Stop */
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP)StopStrmDlgProc, hModPMADDE,
                         ID_STOPDLG, NULL );
             break;

        case IDM_INSTPROT:                    /* Install Protocol */
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) InstallProtoDlgProc, hModPMADDE,
                         ID_INSTPROTDLG, NULL );
             break;

        case IDM_DESTROY:                     /* Destroy */
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) DestroyStrmDlgProc, hModPMADDE,
                         ID_DESTROYDLG, NULL );
             break;

        case IDM_SEEK:                        /* Seek */
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) SeekStrmDlgProc, hModPMADDE,
                         ID_SEEKDLG, NULL );
             break;

        case IDM_TIME:                        /* Query Time */

             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) QryStrmTimeDlgProc, NULLHANDLE,
                         ID_QRYTIMEDLG, NULL);
             break;

        case IDM_IHANDLERS:                   /* Query Installed handlers */
             WinDlgBox( HWND_DESKTOP, hwnd,
                        (PFNWP) InstalHandlersDlgProc, NULLHANDLE,
                        ID_INSTHNDLRSDLG, (PVOID) NULL );
             break;

        case IDM_SCRIPT:                      /* Script file */
             RetCode = 0 ;
             if (!fScriptMode)
                 RetCode = OpenFileDlg (hwnd,"PMADDE: Open Script File",
                                       szScriptFile,"*.SCR");
             if (!RetCode)
             {
                 RetCode = _beginthread (ReadNExecScriptFile,NULL,8192,NULL);
             }
             break;


        case IDM_OS2SHELL:                    /* OS/2 Shell */
             OpenShell();
             break;


        case IDM_DEVNAME:                     /* Query Installed Audio Devices*/
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) DevNamesDlgProc, hModPMADDE,
                         ID_DEVNAMESDLG, NULL );
             break;

        case IDM_LOGFILE:                     /*  Create a Log file */
             RetCode = OpenFileDlg (hwnd,"PMADDE: Create Log File",
                                    szLogFileName,"*.LOG");
             if (!RetCode)
                 OpenFile(szLogFileName, &hLogFile);
             break;

        case IDM_DEVSTATUS:                   /* Query Device Status  */

             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) DevStatDlgProc, hModPMADDE,
                         ID_DEVSTATDLG, NULL );

             break;
        case IDM_DISPSTATUS:                 /* Display Status       */
             WinDlgBox ( HWND_DESKTOP, hwnd,
                         (PFNWP) DevStatDispDlgProc, hModPMADDE,
                         ID_DEVSTATDISPDLG, NULL );
             break;

        case IDM_HELP_INDEX:
             if ( hwndHelpInstance )
               WinSendMsg(hwndHelpInstance, HM_HELP_INDEX, MPVOID, MPVOID);
             break;

        case IDM_HELP_GEN:                     /* General Help */
             if ( hwndHelpInstance )
               WinSendMsg(hwndHelpInstance, HM_EXT_HELP, MPVOID, MPVOID);
             break;

        case IDM_HELP_KEYS:                    /* Keys Help */
             if ( hwndHelpInstance )
                 DisplayHelpPanel(KEYSHELP);
             break;

        case IDM_HELP_USING:                   /* Using Help */
             if ( hwndHelpInstance )
               WinSendMsg(hwndHelpInstance, HM_DISPLAY_HELP, MPVOID, MPVOID);
             break;

        case IDM_HELP_PRODINFO:                /* Product Information */
             WinDlgBox( HWND_DESKTOP, hwnd,
                        (PFNWP) AboutDlgProc, NULLHANDLE,
                        IDM_PRODUCTINFO, (PVOID)NULL );
             WinInvalidateRect (hwnd,&rctlSize,FALSE);
             break;
      }
      break;

     case WM_CREATE:

          flCreate1  = FCF_DLGBORDER | FCF_TITLEBAR |
                       FCF_SYSMENU | FCF_ICON | FCF_MINMAX ;

          /* Create event monitor window */
          hwndActWinFrm = WinCreateStdWindow( hwnd, (ULONG)NULL,
                                              &flCreate1,
                                              szActiveHndlrs,
"Active Handlers                                                 Active Streams",
                                              (ULONG)NULL,
                                              (HMODULE)0L,
                                              ID_ACTWIN,
                                              (PHWND) &hwndClActWin );

         if ( hwndActWinFrm == NULLHANDLE )
             mprintf ("Active Handlers Window Creation Error\n");

         /* Get Character dimensions */
         hps = WinGetPS (hwnd);
         GpiQueryFontMetrics (hps,(LONG)sizeof(fm),&fm);

         /* Set font metrics */
         fm.fsType = FM_TYPE_FIXED;
         fm.fsDefn = FM_DEFN_GENERIC;
         strcpy (fm.szFacename, "System Monospaced");

         cxChar = (SHORT)fm.lAveCharWidth;
         cyChar = (SHORT)fm.lMaxBaselineExt;

         /* Create event monitor window */
         hwndEveWinFrm = WinCreateStdWindow( hwnd,
                                             (ULONG)NULL,
                                             &flCreate1,
                                             szMonitor,
                                             "Event Monitor",
                                             (ULONG)NULL,
                                             (HMODULE)0L,
                                             ID_EVEWIN,
                                             (PHWND) &hwndClEveWin );

         if ( hwndEveWinFrm == NULLHANDLE )
             mprintf ("Event Monitor Creation Error\n");

         /* Create command monitor window    */
         hwndComWinFrm = WinCreateStdWindow( hwnd,
                                             (ULONG)NULL,
                                             &flCreate1,
                                             szCmndMonitor,
                                             "Command Monitor",
                                             (ULONG)NULL,
                                             (HMODULE)0L,
                                             ID_COMWIN,
                                             (PHWND) &hwndClComWin );

         if ( hwndComWinFrm == NULLHANDLE )
             mprintf ("Command Monitor Creation Error\n");
        /*
        ** Create Static text controls that define the various
        ** Presentation fields.
        */

         CreateStaticText (hwnd);
         break;

     case WM_SIZE:
          WinQueryWindowRect ( hwndc1, &rectl );

         /*   Set the window position  of the frames making them visible */

          WinSetWindowPos( hwndActWinFrm, HWND_TOP,
                           (SHORT)rectl.xLeft, (SHORT)rectl.yTop/2,
                           (SHORT)rectl.xRight, (SHORT)rectl.yTop/2,
                           SWP_MOVE|SWP_SIZE | SWP_ACTIVATE |SWP_SHOW );

          WinSetWindowPos( hwndEveWinFrm, HWND_TOP,
                           0, 0,
                           (SHORT)rectl.xRight/3, (SHORT)rectl.yTop/2,
                           SWP_MOVE|SWP_SIZE | SWP_ACTIVATE |SWP_SHOW );

          WinSetWindowPos( hwndComWinFrm, HWND_BOTTOM,
                          (SHORT)rectl.xRight/3, 0,
                          (SHORT)(rectl.xRight - rectl.xRight/3),
                          (SHORT)rectl.yTop/2,
                          SWP_MOVE|SWP_SIZE | SWP_ACTIVATE |SWP_SHOW );

          /*
          ** Set Presentation Parameters
          */

          WinSetPresParam( hwndActLst, PP_FONTNAMESIZE,
                           (ULONG) strlen("10.System Monospaced")+1,
                           (PVOID) "10.System Monospaced");
        break;

     case WM_ERASEBACKGROUND:
          return (MRESULT)1;
          break;

     case WM_PAINT:
        /* Default code for WM_PAINT */
        hps = WinBeginPaint( hwnd, (HPS) NULL, (PRECTL) &rectl );
        WinFillRect( hps, (PRECTL) &rectl, SYSCLR_WINDOW );
        WinEndPaint( hps );
        break;

     default:
        return(WinDefWindowProc(hwnd,msg,mp1,mp2));
  }
  return(FALSE);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    ActiveHndlrsWinProc
*
* ARGUMENTS:   HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:      None
*
* DESCRIPTION: Active Handlers Window Procedure
*
* OS/2 API's USED: None
*
* C FUNCTION CALLS: memset.
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY ActiveHndlrsWinProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
   LONG lPresParam;                   /* Presntation Parameter        */
   HPS hps;                           /* Handle to Presentation Space */
   RECTL rectl;                       /* Dimensions of window         */
   static cxClient,cyClient;          /* Client dimensions            */
   static USHORT usxloc,usHeight;     /* Client dimensions            */
   static USHORT usListWidth;         /* Client dimensions            */
   static USHORT usyloc,usWidth,usWidth1,usxloc1; /* Client Dimens    */

   /*
   ** Initially when the controls are created they are set to not be
   ** visible. We size them appropriately here and also use
   ** WinSetWindowPos () API to position as well as size them.
   ** The Character dimensons are used for placement of static
   ** text. The default font used is "System monospaced".
   */

   switch(msg)
   {
     case WM_SIZE:

          WinQueryWindowRect (hwnd,&rectl);

          usxloc = rectl.xLeft;
          usyloc = (rectl.yTop - (cyChar*2));
          usxloc1 = rectl.xRight;
          usWidth = ((cxChar * strlen (DISPCAT1))-(3*cxChar));
          usHeight = cyChar*2;
          SetPosition (STX_);

          usxloc+=usWidth+cxChar/2;
          usWidth = ((cxChar * strlen (DISPCAT2))-(3*cxChar));
          SetPosition (STX_1);

          usxloc+=usWidth;
          usWidth = ((cxChar * strlen (DISPCAT3))-(4*cxChar));
          SetPosition (STX_2);

          usxloc+=usWidth+cxChar/3;
          usWidth = ((cxChar * strlen (DISPCAT4))-(5*cxChar));
          usListWidth =usxloc +usWidth+cyChar/2;
          SetPosition (STX_3);

          usxloc+=usWidth+5/2*cxChar;
          usWidth = ((cxChar * strlen (DISPCAT5))-(4*cxChar));
          SetPosition (STX_4);

          usxloc+=usWidth+cxChar/2;
          usWidth = ((cxChar * strlen (DISPCAT6))-(4*cxChar));
          SetPosition (STX_5);

          usxloc+=usWidth+4*cxChar;
          usWidth = ((cxChar * strlen (DISPCAT7))-(4*cxChar));
          SetPosition (STX_6);

          usxloc+=usWidth+4*cxChar;
          usWidth = ((cxChar * strlen (DISPCAT8))-(4*cxChar));
          SetPosition (STX_7);

          usxloc+=usWidth+cxChar/2;
          usWidth = ((cxChar * strlen (DISPCAT9))-(4*cxChar));
          SetPosition (STX_8);

          usxloc+=usWidth+cxChar/2;
          usWidth = ((cxChar * strlen (DISPCAT10))-(3*cxChar));
          SetPosition (STX_9);

          /* Size the clients accordingly */
          usxloc = usyloc = 0;
          usWidth = usListWidth ;
          usHeight = (SHORT)rectl.yTop -cyChar*2 ;
          SetPosition (ID_ACTLIST);

          usxloc = usListWidth;
          usyloc = 0;
          usWidth = (SHORT)(rectl.xRight-usListWidth);
          usHeight = (SHORT)rectl.yTop -cyChar*2;
          SetPosition (FID_CLIENT);

          /*
          ** Set Presentation Parameters
          */
          WinSetPresParam( WinWindowFromID(hwnd,FID_CLIENT ),
                           PP_FONTNAMESIZE,
                           (ULONG) strlen("10.System Monospaced")+1,
                           (PVOID) "10.System Monospaced" );
          break;

     case WM_CREATE:
         CreateFontFixed(hwnd);
         break;

     case WM_ERASEBACKGROUND:
          return (MRESULT)1;
          break;

     case WM_PAINT:
         hps = WinBeginPaint( hwnd, (HPS) NULL, (PRECTL) &rectl );
         WinFillRect( hps, (PRECTL) &rectl, SYSCLR_WINDOW );
         WinEndPaint( hps );
         break;

    default:
        return((MRESULT)WinDefWindowProc(hwnd,msg,mp1,mp2));
       break;
  }
  return((MRESULT)FALSE);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   EventMonitorWinProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:     None
*
* DESCRIPTION: Event Monitor Window Procedure
*
* OS/2 API's USED: None.
*
* C FUNCTION CALLS: None.
*
* INTERNAL FUNCTION REFERENCES:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY EventMonitorWinProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)

{

   LONG lPresParam;                     /* Presntation Parameter        */
   HPS hps;                             /* Handle to Presentation Space */
   RECTL rectl;                         /* Dimensions of window         */
   USHORT usxloc,usHeight;              /* Client dimensions            */
   USHORT usyloc,usWidth;               /* Client dimensions            */

   /*
   ** Initially when the controls are created they are set to not be
   ** visible. We size them appropriately here and also use
   ** WinSetWindowPos () API to position as well as size them.
   ** The Character dimensons are used for placement of static
   ** text. The default font used is "System monospaced".
   */

   switch(msg)
   {
     case WM_SIZE:
       WinQueryWindowRect (hwnd,&rectl);

       usxloc   = rectl.xLeft;
       usyloc   = (rectl.yTop - (cyChar*2));
       usWidth  = cxChar * strlen (DISPCAT11);
       usHeight = cyChar*2;

       SetPosition (STX_10);

       usxloc+=usWidth+cyChar;
       usWidth = cxChar * strlen (DISPCAT12);

       SetPosition (STX_11);

       /* Size the clients accordingly */
       usxloc = 0;
       usyloc = 0;
       usWidth = (SHORT)(rectl.xRight);
       usHeight = (SHORT)rectl.yTop -cyChar*2;

       SetPosition (FID_CLIENT);

       /*
       ** Set Presentation Parameters
       */

       WinSetPresParam( WinWindowFromID(hwnd,FID_CLIENT ),
                        PP_FONTNAMESIZE,
                        (ULONG) strlen("10.System Monospaced")+1,
                        (PVOID) "10.System Monospaced" );
      break;

    case WM_CREATE:
      CreateFontFixed(hwnd);
      break;

    case WM_ERASEBACKGROUND:
      return (MRESULT)1;
      break;

    case WM_PAINT:
        hps = WinBeginPaint( hwnd, (HPS) NULL, (PRECTL) &rectl );
        WinFillRect( hps, (PRECTL) &rectl, SYSCLR_WINDOW );
        WinEndPaint( hps );
        break;

    default:
        return ((MRESULT)WinDefWindowProc(hwnd,msg,mp1,mp2));
  }
  return(FALSE);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   CmndMonitorWinProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:     None
*
* DESCRIPTION: Command Monitor Window  Procedure
*
* OS/2 API's USED: None
*
* C FUNCTION CALLS: memset
*
* INTERNAL FUNCTION REFERENCES: None
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY CmndMonitorWinProc( HWND hwnd, ULONG msg, MPARAM mp1,
                                     MPARAM mp2)
{                                    /* Presntation Parameter        */
  LONG lPresParam;                   /* Handle to Presentation Space */
  HPS hps;                           /* Dimensions of window         */
  RECTL rectl;                       /* Client dimensions            */
  USHORT usxloc,usHeight;            /* Client dimensions            */
  USHORT usyloc,usWidth;

   /*
   ** Initially when the controls are created they are set to not be
   ** visible. We size them appropriately here and also use
   ** WinSetWindowPos () API to position as well as size them.
   ** The Character dimensons are used for placement of static
   ** text. The default font used is "System monospaced".
   */


  switch(msg)
  {
    case WM_SIZE:
      WinQueryWindowRect (hwnd,&rectl);

      /* Set up positions and size */

      usxloc = rectl.xLeft;
      usyloc = (rectl.yTop - (cyChar*2));
      usWidth = cxChar * strlen (DISPCAT13);
      usHeight = cyChar*2;

      SetPosition (STX_12);

      usxloc+=usWidth+3*cyChar;
      usWidth = cxChar * strlen (DISPCAT11);

      SetPosition (STX_17);

      usxloc+=usWidth+cxChar;
      usWidth = cxChar * strlen (DISPCAT14);

      /*
      ** Size the clients accordingly. These are static
      ** text controls.
      */
      SetPosition (STX_13);

      usxloc+=usWidth;
      usWidth = cxChar * strlen (DISPCAT15);

      /*
      ** Size the clients accordingly. These are static
      ** text controls.
      */
      SetPosition (STX_14);

      /* Size the clients accordingly */

      usxloc = 0;
      usyloc = 0;
      usWidth = (SHORT)(rectl.xRight);
      usHeight = (SHORT)rectl.yTop -cyChar*2;

      SetPosition (FID_CLIENT);

      /*
      ** Set Presentation Parameters
      */

      WinSetPresParam( WinWindowFromID(hwnd,FID_CLIENT ),
                       PP_FONTNAMESIZE,
                       (ULONG) strlen("10.System Monospaced")+1,
                       (PVOID) "10.System Monospaced" );
      break;

    case WM_ERASEBACKGROUND:
      return (MRESULT)1;
      break;

    case WM_CONTROL:
      break;

    case WM_CREATE:
      CreateFontFixed(hwnd);
      break;

    case WM_PAINT:
      /* Default code for WM_PAINT */
      hps = WinBeginPaint( hwnd, (HPS) NULL, (PRECTL) &rectl );
      WinFillRect( hps, (PRECTL) &rectl, SYSCLR_WINDOW);
      WinEndPaint( hps );
      break;

    default:
      return((MRESULT)WinDefWindowProc(hwnd,msg,mp1,mp2));
  }
  return(FALSE);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   InstallProtoDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
*
* RETURN:     None
*
* DESCRIPTION: InstallProtoDlgProc gets all the information provided by the user
*              and installs the protocol.
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY InstallProtoDlgProc( HWND hwnd, ULONG msg, MPARAM mp1,
                                      MPARAM mp2)
{
  LONG     lPresParam;           /* Presentation Parameter        */
  SPCB     spcb;                 /* Stream Protocol Control Block */
  SPCBKEY  spcbkey;              /* SPCB Key                      */
  SHORT    sDataIndex;           /* Index of Selected data type   */
  char     szTemp1[9];           /* Temporary Storage             */
  ULONG    TargetId;             /* MMPM/2 Handler ID             */
  RC       rc;                   /* Return Code                   */
  ULONG    LstId;                /* Data types list box ID        */
  PSPCBKEY pspcbkey;             /* Pointer to SPCBKEY struct     */

  switch(msg)
  {
    case WM_CONTROL:

       if ( SHORT2FROMMP ( mp1 ) ==  LN_ENTER )
             WinSendMsg ( hwnd, WM_COMMAND, MPFROMSHORT ( PB_OK ),
              MPFROM2SHORT ( CMDSRC_PUSHBUTTON, TRUE ) );
      break;
    case WM_COMMAND:
     switch (SHORT1FROMMP(mp1))
     {
       case PB_HELP:
         if ( hwndHelpInstance )
           DisplayHelpPanel(INSTALL);
           break;

       case DID_CANCEL:
         /* Dismiss the dialog box */
         WinDismissDlg (hwnd,DID_CANCEL);
         break;

       case PB_OK:
          /*
          ** Obtain the data index of the selected data type
          ** from the list box.
          */

          sDataIndex = SHORT1FROMMR ( WinSendDlgItemMsg( hwnd,
                                      LI_PROTOCOL,
                                      LM_QUERYSELECTION,
                                      (MPARAM)LIT_FIRST,
                                      (MPARAM) 0L ) );

          /* No selection Inform the user */

          if(sDataIndex == LIT_NONE)
          {
            mprintf ("Stream Data Type not selected\n");
            break;
          }

          /*
          ** Use the data index above obtained to find out what the
          ** SPCB key items are. A String Data type is displayed in the
          ** list box, but this has to be decoded into a data type and
          ** data sub type. The function in this call format returns a
          ** pointer to the SPCB which has the data type and data sub type
          ** set.
          */

          pspcbkey =(PSPCBKEY) ListInsertStreamDataTypes( hwnd,LI_PROTOCOL,
                                          MPFROM2SHORT(QUERY_ITEM,sDataIndex));

          /* Set up SPCB Key */

          spcbkey.ulDataType = pspcbkey->ulDataType;
          spcbkey.ulDataSubType = pspcbkey->ulDataSubType;
          spcb.spcbkey.ulIntKey = spcbkey.ulIntKey=0;

          /*
          ** Query The Target MMPM/2 Handler ID which uses and
          ** understands the protocol we wish to install.
          */

          WinQueryDlgItemText (hwnd, EF_TARGETID, 3, szTemp1);
          CheckString (szTemp1);
          TargetId = atol(szTemp1);

          /* Call The Worker */

          if(  rc = GetProtocol ( TargetId, (PSPCBKEY)&spcbkey, (PSPCB)&spcb))
          {
            mprintf ("SpiGetProtocol Failed. Make sure you entered AUDIOSH$ stream handler's target id.\n");
            break;
          }

          /*
          ** Query The user for protocol details like
          ** buffer sizes, minimum number of buffers to start the
          ** source handler, Min buff for target, and so on.
          */

          WinQueryDlgItemText (hwnd, EF_BUFFERSIZE, 8, szTemp1);
          CheckString (szTemp1);
          spcb.ulBufSize = atol ( szTemp1);

          WinQueryDlgItemText (hwnd, EF_MAXBUFF, 3, szTemp1);
          CheckString (szTemp1);
          spcb.ulMaxBuf = atol ( szTemp1);

          WinQueryDlgItemText (hwnd, EF_MINBUFF, 3, szTemp1);
          CheckString (szTemp1);
          spcb.ulMinBuf = atol ( szTemp1);

          WinQueryDlgItemText (hwnd, EF_SRCMINBUFF, 3, szTemp1);
          CheckString (szTemp1);
          spcb.ulSrcStart = atol ( szTemp1);

          WinQueryDlgItemText (hwnd, EF_TGTMINBUF, 3, szTemp1);
          CheckString (szTemp1);
          spcb.ulTgtStart = atol ( szTemp1);

          WinQueryDlgItemText (hwnd, EF_SPCBKEY, 3, szTemp1);
          CheckString (szTemp1);
          spcbkey.ulIntKey = spcb.spcbkey.ulIntKey = atol ( szTemp1);

          /* Call The worker */

          if(rc = InstallProtocol ( TargetId, (PSPCBKEY)&spcbkey, (PSPCB)&spcb))
          {
            mprintf ("SpiInstallProtocol Failed\n");
            break;
          }
          else
               PrintCmndInfo (INSTALL, 0,rc);

          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,PB_OK);
        break;
     }
     break;

   case WM_INITDLG:

            /* Max no. of chars for EF_TARGETID is 32 */

     WinSendDlgItemMsg(hwnd, EF_TARGETID,
                       EM_SETTEXTLIMIT, MPFROMSHORT(32),0L);

           /* Max no. of chars for EF_BUFFERSIZE is 32 */

     WinSendDlgItemMsg(hwnd, EF_BUFFERSIZE,
                       EM_SETTEXTLIMIT, MPFROMSHORT(32),0L);

          /* Max no. of chars for EF_MAXBUFF is 32 */

     WinSendDlgItemMsg(hwnd, EF_MAXBUFF,
                       EM_SETTEXTLIMIT, MPFROMSHORT(32),0L);

          /* Max no. of chars for EF_MINBUFF is 32 */

     WinSendDlgItemMsg(hwnd, EF_MINBUFF,
                       EM_SETTEXTLIMIT, MPFROMSHORT(32),0L);

         /* Max no. of chars for EF_SRCMINBUFF is 8 */

     WinSendDlgItemMsg(hwnd, EF_SRCMINBUFF,
                       EM_SETTEXTLIMIT, MPFROMSHORT(8), 0L );

         /* Max no. of chars for EF_TGTMINBUF is 8 */

     WinSendDlgItemMsg( hwnd, EF_TGTMINBUF,
                       EM_SETTEXTLIMIT, MPFROMSHORT(8), 0L );

         /* Max no. of chars for EF_SPCBKEY is 8 */

     WinSendDlgItemMsg(hwnd, EF_SPCBKEY,
                       EM_SETTEXTLIMIT, MPFROMSHORT(8), 0L );

     /*
     ** Fix the device specific information. At Application initialization
     ** time QueryInstalledDevices () function is called to obtain
     ** Multimedia system configuration information. A Resource ID
     ** Exists for all installed audio devices, that tells us what
     ** the capabilities of the device are (PMADDEVS.rc). We call
     ** the function FixDeviceData () to set up the resource id for
     ** the current device in selection.
     */

     rc = FixDeviceData ();

     /* Insert The Data Types supported on this Audio Device */

     ListInsertStreamDataTypes( hwnd,LI_PROTOCOL,
                                MPFROM2SHORT(INSERT_ITEM,sDataIndex));
     break;

     case WM_PAINT:
        /* Execute default processing */
        return( WinDefDlgProc( hwnd,msg, mp1, mp2 ) );
        break;
     default:
        return( WinDefDlgProc( hwnd,msg,mp1,mp2 ) );
  }
  return(FALSE);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   CreateStrmDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:     None
*
* DESCRIPTION: This dialog Procedure gets all the information from the user
*              and creates a stream for the given stream handlers.
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY CreateStrmDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  SHORT sDataIndex;                   /* Data Item Index     */
  RC    RetCode;                      /* API Return Code     */
  char  szTemp1[3];                   /* Temporary Storage   */
  int   i,k;                          /* Loop Indices        */
  ULONG ulLstId;                      /* List box ID         */
  BOOL  fHandlesObtained = FALSE;     /* Local error flag    */
  BOOL  fhidSrcValid = FALSE;         /* Local error flag    */

  switch(msg)
  {

    case WM_CONTROL:

      if ( SHORT2FROMMP ( mp1 ) ==  LN_ENTER )
             WinSendMsg ( hwnd, WM_COMMAND, MPFROMSHORT ( PB_OK ),
                          MPFROM2SHORT ( CMDSRC_PUSHBUTTON, TRUE ) );
      break;

    case WM_COMMAND:
      switch (SHORT1FROMMP(mp1))
      {
        case PB_HELP:
          if (hwndHelpInstance)
             DisplayHelpPanel(CREATE);
          break;

        case PB_OK:
          sDataIndex = SHORT1FROMMR ( WinSendDlgItemMsg( hwnd,
                                                         LI_STRMDATATYPES,
                                                         LM_QUERYSELECTION,
                                                         (MPARAM)LIT_FIRST,
                                                         (MPARAM) 0L ) );


          /*
          ** Nothing selected from the list, signal error
          */

          if(sDataIndex == LIT_NONE)
          {
            mprintf ("Stream Data Type not selected\n");
            break;
          }

          aStreamData[StreamCount].ulmode = sDataIndex;
          WinQueryDlgItemText (hwnd, EF_SRCHID, 3, szTemp1);
          CheckString (szTemp1);
          aStreamData[StreamCount].hidSrc = atol(szTemp1);

          WinQueryDlgItemText (hwnd, EF_HIDTGT, 3, szTemp1);
          CheckString (szTemp1);
          aStreamData[StreamCount].hidTgt = atol(szTemp1);

          for (i = 0; i < HandlerCount; i++)
          {
            if (aActiveHandlers[i].hidSrc == aStreamData[StreamCount].hidSrc)
            {
              for (k = 0; k < HandlerCount; k++)
              {
                if (aActiveHandlers[k].hidTgt == aStreamData[StreamCount].hidTgt)
                {
                  fHandlesObtained = TRUE;
                  break;
                } /* Inner If */
              }   /* Inner for */
            }   /* Outer If */
          }/* Outer for */

          if (!fHandlesObtained)
          {
            mprintf ("Invalid Handler IDs, %d %d\n",
                      aStreamData[StreamCount].hidSrc,
                      aStreamData[StreamCount].hidTgt  );
            break;
          }
          /*
          ** Call the Worker
          */

          RetCode = CreateStream();

          /*
          ** Update Display with command information and return
          ** codes. The Stream handle is set to 0 at create time.
          */

          PrintCmndInfo (CREATE, 0,RetCode);
          if (!RetCode)
          {
             WriteScriptFile (CREATE);
             PrintActiveInfo (hwnd);
          }
          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,PB_OK);
          break;

        case DID_CANCEL:
          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,DID_CANCEL);
          break;
      }
      break;
      case WM_INITDLG:
        /* Max no. of chars for EF_SRCHID is 32 */
        WinSendDlgItemMsg( hwnd, EF_SRCHID, EM_SETTEXTLIMIT,
                           MPFROMSHORT(3), 0L );

        /* Max no. of chars for EF_HIDTGT is 32 */
        WinSendDlgItemMsg( hwnd, EF_HIDTGT, EM_SETTEXTLIMIT,
                           MPFROMSHORT(3), 0L );

        RetCode = FixDeviceData ();
        ListInsertStreamDataTypes( hwnd,LI_STRMDATATYPES,
                                   MPFROM2SHORT(INSERT_ITEM,sDataIndex));
        break;

      case WM_PAINT:
        /* Execute default processing */
        return( WinDefDlgProc( hwnd, msg, mp1, mp2) );
        break;
      default:
        return(WinDefDlgProc(hwnd,msg,mp1,mp2));
   }
   return(FALSE);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  GetStrmHndlrDlgProc
*
* ARGUMENTS: HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:    None
*
* DESCRIPTION: This function gets the Stream Handlers selected by the user.
*
* OS/2 API's USED: WinSendDlgItemMsg()
*                  WinSendMsg()
*                  WinDismmisDlg()
*                  WinDefDlgProc()
*
*
* C FUNCTION CALLS: strcmp()
*                   strtok()
*                   strcat()
*                   strcpy()
*                   strncat()
*
* INTERNAL FUNCTION REFERENCES: CheckString()
*                               GetHandler()
*                               LongToString()
*                               DisplayHelpPanel Macro
*                               WriteScriptFile()
*                               ListInsertInstalledHandler()
*
* GLOBAL VARIABLES REFERENCED: HandlerCount
*
*****************************END OF SPECIFICATIONS ***************************/

MRESULT EXPENTRY GetStrmHndlrDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{

   SHORT sDataIndex;                /* Data item Index    */
   RC    ReturnCode;                /* API return Code    */
   HPS   hps;                       /* Presentation Space */
   char  TempBuffer[4096];          /* Temporary Storage  */
   char  TokenSep[] = " \t";        /* Token Separotor    */
   PCHAR pchWord;                   /* Local pointer      */
   PSZ   pszTemp;                   /* Temporary pointer  */
   SHORT i,res,j;                   /* Loop Indicies      */
   ULONG ulCount;                   /* Multi Select Count */

   switch(msg)
   {
     case WM_COMMAND:

       switch (SHORT1FROMMP(mp1))
       {
         case PB_HELP:
           if (hwndHelpInstance)
             DisplayHelpPanel(GETHANDLER);
             break;

         case PB_OK:
           ulCount = 0L;
           sDataIndex = SHORT1FROMMR( WinSendDlgItemMsg( hwnd,
                                                         LI_INSTHNDLR,
                                                         LM_QUERYSELECTION,
                                                         (MPARAM)LIT_FIRST,
                                                         (MPARAM) 0L ) );

           while (sDataIndex != LIT_NONE)
           {
             ulCount++;
             sDataIndex = SHORT1FROMMR( WinSendDlgItemMsg( hwnd, LI_INSTHNDLR,
                                             LM_QUERYSELECTION,
                                             (MPARAM) sDataIndex,
                                             (MPARAM) 0L ) ) ;
           } /* While */
           if (ulCount == 0)
           {

             mprintf ("Atleast two handlers have to be selected to create a stream\n");
             break;
           } /* No Selection */

           sDataIndex = LIT_FIRST;
           for (j = 0; j < ulCount; ++j)
           {
              sDataIndex = SHORT1FROMMR( WinSendDlgItemMsg( hwnd, LI_INSTHNDLR,
                                              LM_QUERYSELECTION,
                                              (MPARAM) sDataIndex,
                                               (MPARAM) 0L ) );

              WinSendDlgItemMsg( hwnd, LI_INSTHNDLR, LM_QUERYITEMTEXT,
                                 MPFROM2SHORT(sDataIndex,80), TempBuffer );

              pchWord = strtok (TempBuffer,TokenSep);

              CheckString(pchWord);

              strcpy (aActiveHandlers[HandlerCount].nszHandlerName,pchWord);

              pchWord = strtok (NULL,TokenSep);
              CheckString(pchWord);
              strcpy (aActiveHandlers[HandlerCount].nszHandlerClass,pchWord);

              for ( i=0;i < HandlerCount; i++ )
                  if(strcmp ( aActiveHandlers[i].nszHandlerName,
                            aActiveHandlers[HandlerCount].nszHandlerName)== 0 )
              res =1;
              if( res != 1 )
              {
                /* Call The Worker        */

                ReturnCode = GetStreamHandler();
               /*
               ** Update Display with command information and return
               ** codes.
               */

                PrintCmndInfo(GETHANDLER,0,ReturnCode);

                if (!ReturnCode)
                {
                  strcpy (TempBuffer,
                          aActiveHandlers[HandlerCount -1].nszHandlerName);

                  strncat(TempBuffer,hspace, 9-strlen(TempBuffer));

                  strcat (TempBuffer,
                          aActiveHandlers[HandlerCount -1].nszHandlerClass);

                  strncat(TempBuffer,hspace, 20-strlen(TempBuffer));

                  pszTemp = LongToString(aActiveHandlers[HandlerCount-1].hidSrc);

                  strcat(TempBuffer,pszTemp);

                  strncat(TempBuffer,hspace, 25-strlen(TempBuffer));

                  pszTemp = LongToString (aActiveHandlers[HandlerCount-1].hidTgt);

                  strcat(TempBuffer,pszTemp);

                  WinSendMsg ( hwndActLst, (ULONG)LM_INSERTITEM,
                               MPFROMSHORT(LIT_SORTASCENDING),
                               MPFROMP(TempBuffer) );

                  WriteScriptFile (GETHANDLER);
                }
              }
              res =0;
           }

           WinDismissDlg (hwnd,PB_OK);
           break;

        case DID_CANCEL:
          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,DID_CANCEL);
          break;
       }
       break;

       case WM_INITDLG:
         ListInsertInstalledHandlers(hwnd,LI_INSTHNDLR);
         break;

       case WM_PAINT:
         /* Execute default processing */
         return(WinDefDlgProc(hwnd, msg, mp1, mp2) );
         break;
       default:
         return(WinDefDlgProc(hwnd,msg,mp1,mp2));
    }
    return(FALSE);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   InstalHandlersDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:     None
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY InstalHandlersDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{

  switch(msg)
  {

    case WM_COMMAND:
      switch (SHORT1FROMMP(mp1))
      {
        case PB_HELP:
          if (hwndHelpInstance)
             DisplayHelpPanel(HANDLERS);
          break;

        case PB_OK:
          WinDismissDlg (hwnd,PB_OK);
            break;

        case DID_CANCEL:
          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,DID_CANCEL);
          break;
      }
      break;
    case WM_INITDLG:
      ListInsertInstalledHandlers(hwnd,LI_INHNDLRS);
      break;

    case WM_PAINT:
      /* Execute default processing */
      return(WinDefDlgProc(hwnd, msg, mp1, mp2) );
      break;
    default:
      return(WinDefDlgProc(hwnd,msg,mp1,mp2));
      break;
  }
  return(FALSE);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   AssociateStrmDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY AssociateStrmDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
RC           RetCode;                   /* API Return Code   */
CHAR         szTemp2[2];                /* Temporary Storage */
static CHAR  szTemp[FILENAME_SIZE];     /* Temporary Storage */
USHORT       i;                         /* Loop Index        */
WNDPARAMS    EfWndParams;               /* Control Structure */
BOOL         fFoundStream = FALSE;      /* Local Flag        */
BOOL         fFileOpened  = FALSE;      /* Local Flag        */


    switch(msg)
    {

        case WM_HELP:
             if (hwndHelpInstance)
                 DisplayHelpPanel(FILE);
             break;

        case WM_COMMAND:
            switch (SHORT1FROMMP(mp1))
            {
                case PB_OPEN:

                     RetCode = OpenFileDlg (hwnd,
                                           "PMADDE: Associate Stream",
                                            szTemp, "*.WAV");

                     if (RetCode)
                         break;
                     CheckString (szTemp);

                     memset (&EfWndParams,0,sizeof(WNDPARAMS));
                     EfWndParams.fsStatus = WPM_TEXT;
                     EfWndParams.cchText = strlen(szTemp);

                     EfWndParams.pszText = szTemp;
                     WinSendMsg (WinWindowFromID(hwnd,IDC_AUDIOFILE),
                                 WM_SETWINDOWPARAMS,
                                 MPFROMP(&EfWndParams),
                                 MPFROMSHORT(0));

                     break;

                case PB_OK:
                     WinQueryWindowText (WinWindowFromID(hwnd,EF_STREAMNUM),
                                         sizeof(szTemp2),
                                         szTemp2);
                     CheckString (szTemp2);


                     WinQueryWindowText (WinWindowFromID(hwnd,IDC_AUDIOFILE),
                                         sizeof(szTemp),
                                         szTemp);

                     CheckString (szTemp);

                     ulIndex = atol(szTemp2);

                     for (i = 0; i < StreamCount; i++ )
                     {
                         if (aStreamData[i].hStream == ulIndex)
                         {
                             fFoundStream = TRUE;
                             ulIndex = i;
                             break;
                         }
                     }

                     if (!fFoundStream)
                     {
                           mprintf ("Stream Not Found %d \n",ulIndex);
                           WinDismissDlg (hwnd,PB_OK);
                           break;
                     }
                     strcpy (aStreamData[ulIndex].nszFileName,szTemp);

                     RetCode = AssociateStream();
                    /*
                    ** Update Display with command information and return
                    ** codes.
                    */
                     PrintCmndInfo ( ASSOCIATE,ulIndex+1, RetCode);
                     if (!RetCode){
                        WriteScriptFile (ASSOCIATE);
                        PrintActiveInfo (hwnd);
                     }
                     /* Dismiss the dialog box */
                     WinDismissDlg (hwnd,PB_OK);
                break;

                case DID_CANCEL:

                    /* Dismiss the dialog box */
                    WinDismissDlg (hwnd,DID_CANCEL);
                break;

               case PB_HELP:

                 if (hwndHelpInstance)
                 {
                   DisplayHelpPanel(ASSOCIATE);
                 }
               break;
            }
            break;
        case WM_CONTROL:
            break;
        case WM_INITDLG:

             /* Max no. of chars for EF_STREAMNUM is 4 */
             WinSendDlgItemMsg(hwnd,
                               EF_STREAMNUM,
                               EM_SETTEXTLIMIT,
                               MPFROMSHORT(4),
                               0L);

             WinSendDlgItemMsg(hwnd,
                               IDC_AUDIOFILE,
                               EM_SETTEXTLIMIT,
                               MPFROMSHORT(FILENAME_SIZE),
                               0L);


            break;
        case WM_PAINT:

            /* Execute default processing */
            return(WinDefDlgProc(hwnd,
                                 msg,
                                 mp1,
                                 mp2)
                                );
            break;
        default:
            return(WinDefDlgProc(hwnd,msg,mp1,mp2));
    }
    return(FALSE);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   StartStrmDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY StartStrmDlgProc( HWND hwnd, ULONG msg, MPARAM mp1,
                                   MPARAM mp2)
{
  RC     RetCode;                     /* API Return Code         */
  CHAR   szTemp[10];                  /* Temporary Storage space */
  USHORT i;                           /* Loop Index              */
  BOOL   fFoundStream = FALSE;        /* Stream Found Flag       */
  ULONG  ulCount;                     /* Semaphore Count         */

  switch(msg)
  {
    case WM_COMMAND:
      switch (SHORT1FROMMP(mp1))
      {
        case PB_START:
          WinSendDlgItemMsg (hwnd, PB_START,
                             BM_SETHILITE, MPFROMSHORT(1), 0L);
          break;

        case PB_PREROLL:
          WinSendDlgItemMsg (hwnd, PB_PREROLL,
                             BM_SETHILITE, MPFROMSHORT(1), 0L);
          break;

        case PB_OK:
          WinQueryWindowText ( WinWindowFromID(hwnd,EF_STRMNUM),
                               sizeof(szTemp), szTemp );

          ulIndex = atol(szTemp) ;
          /*
          ** loop thro to find the stream data entry
          ** corresponding to this stream.
          */
          for (i = 0; i < StreamCount; i++ )
          {
              if (aStreamData[i].hStream == ulIndex)
              {
                  fFoundStream = TRUE;
                  ulIndex = i;
                  break;
              }
          }

          if (!fFoundStream)
          {
              mprintf ("Stream Not Found %d \n", ulIndex);
              WinDismissDlg (hwnd, PB_OK);
              break;
          }

          aStreamData[ulIndex].hStream = atol(szTemp);
          aStreamData[ulIndex].ulStartFlag = SPI_START_STREAM;
          if ( WinSendDlgItemMsg ( hwnd, PB_PREROLL,
                                   BM_QUERYCHECK, MPFROMSHORT(1), 0L ) )
          {
             aStreamData[ulIndex].ulStartFlag = SPI_START_PREROLL;
          }
          DosResetEventSem (aStreamData[ulIndex].hevThrdSem,&ulCount);
          RetCode =  _beginthread (StartStream,NULL,8192,NULL);
          if (RetCode != -1)
          {
             /*
             **  Wait for the thread we just started to
             **   indicate that everything is allright so
             **   that we can update stream state.
             */

            DosWaitEventSem (aStreamData[ulIndex].hevThrdSem, (ULONG)-1);

           /*
           ** Update Display with command information and return
           ** codes.
           */
            PrintCmndInfo ( PLAY, aStreamData[ulIndex].hStream,
                            aStreamData[ulIndex].RetCode );

            /* Change The State of the stream */

            strcpy(aStreamData[ulIndex].nszState, "STREAMING");

            PrintActiveInfo (hwnd);

            /*
            ** The Script interface if enabled at application invocation
            ** time logs all user choices into a script file. The function
            ** below writes the current command into the script file
            */

            WriteScriptFile (START);
          }
          else
           mprintf ("Start Thread Creation Error\n");
           WinDismissDlg (hwnd, PB_OK);
           break;

       case DID_CANCEL:
               WinDismissDlg (hwnd, DID_CANCEL);
              break;

       case PB_HELP:
         if (hwndHelpInstance)
           DisplayHelpPanel(START);
           break;
      }
      break;

      case WM_INITDLG:
        /* Max no. of chars for EF_STRMNUM is 8 */

        WinSendDlgItemMsg( hwnd, EF_STRMNUM, EM_SETTEXTLIMIT,
                           MPFROMSHORT(3), 0L );

        WinSendDlgItemMsg ( hwnd, PB_START,
                            BM_SETHILITE, MPFROMSHORT(1), 0L );
        break;
      case WM_PAINT:
        /* Execute default processing */
        return(WinDefDlgProc(hwnd, msg, mp1, mp2) );
        break;
      default:
        return(WinDefDlgProc(hwnd,msg,mp1,mp2));
   }
   return(FALSE);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   StopStrmDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY StopStrmDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{

  RC    RetCode;                          /* API Return Code         */
  CHAR  szTemp[10];                       /* Temporary Storage space */
  USHORT i;                               /* Loop Index              */
  BOOL  fFoundStream = FALSE;             /* Stream Found Flag       */

  switch( msg )
  {
    case WM_COMMAND:
      switch ( SHORT1FROMMP( mp1 ) )
      {
        case PB_PAUSE:
          WinSendDlgItemMsg( hwnd, PB_PAUSE, BM_SETHILITE, MPFROMSHORT(1), 0L );
                 break;

        case PB_FLUSH:
          WinSendDlgItemMsg ( hwnd, PB_FLUSH, BM_SETHILITE, MPFROMSHORT(1), 0L );
            break;

         case PB_DISCARD:
           WinSendDlgItemMsg( hwnd, PB_DISCARD, BM_SETHILITE, MPFROMSHORT(1), 0L );
           break;

         case PB_OK:
           WinQueryWindowText ( WinWindowFromID(hwnd,EF_STRMNUM),
                                sizeof(szTemp), szTemp );

           ulIndex = atol(szTemp);
          /*
          ** loop thro to find the stream data entry
          ** corresponding to this stream.
          */

          for (i = 0; i < StreamCount; i++ )
          {
              if (aStreamData[i].hStream == ulIndex)
              {
                  fFoundStream = TRUE;
                  ulIndex = i;
                  break;
              }
          }

          if (!fFoundStream)
          {
              mprintf ("Stream Not Found %d \n",ulIndex);
              WinDismissDlg (hwnd, PB_OK);
              break;
          }

           aStreamData[ulIndex].hStream = atol(szTemp);
           aStreamData[ulIndex].ulStopFlag = SPI_STOP_STREAM;

           if ( WinSendDlgItemMsg ( hwnd,PB_FLUSH,BM_QUERYCHECK,
                                    MPFROMSHORT(1),0L ) )
           {
              aStreamData[ulIndex].ulStopFlag = SPI_STOP_FLUSH;
           }
           else
           if ( WinSendDlgItemMsg ( hwnd,PB_DISCARD,BM_QUERYCHECK,
                                    MPFROMSHORT(1),0L ) )
           {
             aStreamData[ulIndex].ulStopFlag = SPI_STOP_DISCARD;
           }
           RetCode = StopStream();
          /*
          ** Update Display with command information and return
          ** codes.
          */
           PrintCmndInfo (STOP,ulIndex, RetCode);
           if (!RetCode)
           {
             PrintActiveInfo (hwnd);
             WriteScriptFile (STOP);
           }
           WinDismissDlg (hwnd, PB_OK);
           break;

         case DID_CANCEL:
           WinDismissDlg (hwnd,DID_CANCEL);
           break;

         case PB_HELP:
           if (hwndHelpInstance)
             DisplayHelpPanel(STOP);
             break;
      }
      break;

   case WM_INITDLG:
     /* Max no. of chars for EF_STRMNUM is 3 */
     WinSendDlgItemMsg( hwnd, EF_STRMNUM, EM_SETTEXTLIMIT, MPFROMSHORT(3), 0L );

     /* Set Pause as the default */
     WinSendDlgItemMsg ( hwnd, PB_PAUSE, BM_SETHILITE, MPFROMSHORT(1), 0L );
     break;
   case WM_PAINT:
     /* Execute default processing */
     return(WinDefDlgProc(hwnd, msg, mp1, mp2) );
     break;
   default:
     return(WinDefDlgProc(hwnd,msg,mp1,mp2));
  }
  return(FALSE);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   SeekStrmDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY SeekStrmDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  RC     RetCode;                    /* API Return Code         */
  CHAR   szTemp[10];                 /* Temporary Storage space */
  USHORT i;                          /* Loop Index              */
  BOOL   fFoundStream = FALSE;       /* Stream Found Flag       */

  switch(msg)
  {
    case WM_COMMAND:
      switch(SHORT1FROMMP(mp1))
      {
        case PB_TIME:
          if (WinSendDlgItemMsg ( hwnd,PB_TIME,BM_QUERYCHECK,
                                  MPFROMSHORT(1),0L ) )
          {
             WinSendDlgItemMsg ( hwnd,PB_TIME,BM_SETHILITE, MPFROMSHORT(1),0L );
          }
          break;

        case PB_BYTES:
          if ( WinSendDlgItemMsg ( hwnd,PB_BYTES,BM_QUERYCHECK,
                                  MPFROMSHORT(1),0L ) )
          {
            WinSendDlgItemMsg ( hwnd,PB_BYTES,BM_SETHILITE, MPFROMSHORT(1),0L );
          }
          break;

        case PB_ABSOLUTE:
          if ( WinSendDlgItemMsg ( hwnd,PB_ABSOLUTE,BM_QUERYCHECK,
                                   MPFROMSHORT(1),0L ) )
          {
            WinSendDlgItemMsg ( hwnd,PB_ABSOLUTE,BM_SETHILITE, MPFROMSHORT(1),0L );
            aStreamData[ulIndex].ulSeekFlag |= SPI_SEEK_ABSOLUTE;
          }
          break;

        case PB_RELATIVE:
          if ( WinSendDlgItemMsg ( hwnd,PB_RELATIVE,BM_QUERYCHECK,
                                   MPFROMSHORT(1),0L ) )
          {
            WinSendDlgItemMsg ( hwnd,PB_RELATIVE,BM_SETHILITE, MPFROMSHORT(1),0L );
            aStreamData[ulIndex].ulSeekFlag |= SPI_SEEK_RELATIVE;
          }
          break;

        case PB_END:
          if ( WinSendDlgItemMsg ( hwnd,PB_END,BM_QUERYCHECK,
                                   MPFROMSHORT(1),0L ) )
          {
            WinSendDlgItemMsg ( hwnd,PB_END,BM_SETHILITE, MPFROMSHORT(1),0L );
            aStreamData[ulIndex].ulSeekFlag |= SPI_SEEK_FROMEND;
          }
          break;

        case PB_OK:
          WinQueryWindowText( WinWindowFromID(hwnd,EF_STRMNUM),
                              sizeof(szTemp), szTemp );
          CheckString (szTemp);
          ulIndex = atol(szTemp);

          /*
          ** loop thro to find the stream data entry
          ** corresponding to this stream.
          */

          for (i = 0; i < StreamCount; i++ )
          {
              if (aStreamData[i].hStream == ulIndex)
              {
                  fFoundStream = TRUE;
                  ulIndex = i;
                  break;
              }
          }
          /* Print Stream not found error and return */

          if (!fFoundStream)
          {
              mprintf ("Stream Not Found %d\n",ulIndex);
              WinDismissDlg (hwnd, PB_OK);
              break;
          }

          /*
          ** Query the Dlg items to determine the seek mode
          ** and value and so on.
          */

          /* default to MMTIME units */

          aStreamData[ulIndex].ulSeekFlag = SPI_SEEK_MMTIME;

          if (WinSendDlgItemMsg ( hwnd,PB_TIME,BM_QUERYCHECK,
                                  MPFROMSHORT(1),0L ) )
          {
             aStreamData[ulIndex].ulSeekFlag = SPI_SEEK_MMTIME;
          }

          if ( WinSendDlgItemMsg ( hwnd,PB_BYTES,BM_QUERYCHECK,
                                  MPFROMSHORT(1),0L ) )
          {
            aStreamData[ulIndex].ulSeekFlag = SPI_SEEK_BYTES;
          }

          /* Default to Absolute seeks */

          aStreamData[ulIndex].ulSeekFlag |= SPI_SEEK_ABSOLUTE;

          if ( WinSendDlgItemMsg ( hwnd,PB_ABSOLUTE,BM_QUERYCHECK,
                                   MPFROMSHORT(1),0L ) )
          {
            aStreamData[ulIndex].ulSeekFlag |= SPI_SEEK_ABSOLUTE;
          }

          if ( WinSendDlgItemMsg ( hwnd,PB_RELATIVE,BM_QUERYCHECK,
                                   MPFROMSHORT(1),0L ) )
          {
            aStreamData[ulIndex].ulSeekFlag |= SPI_SEEK_RELATIVE;
          }

          if ( WinSendDlgItemMsg ( hwnd,PB_END,BM_QUERYCHECK,
                                   MPFROMSHORT(1),0L ) )
          {
            aStreamData[ulIndex].ulSeekFlag |= SPI_SEEK_FROMEND;
          }

          aStreamData[ulIndex].hStream = atol(szTemp);

          memset(szTemp,'\0',sizeof(szTemp));

          WinQueryWindowText ( WinWindowFromID(hwnd,EF_SEEKPOS),
                               sizeof(szTemp), szTemp );
          CheckString (szTemp);

          aStreamData[ulIndex].ulSeekPoint = atol(szTemp);

          /* Call The worker */

          RetCode = SeekStream();

          /*
          ** Update Display with command information and return
          ** codes.
          */

          PrintCmndInfo (SEEK,ulIndex, RetCode);

          if (!RetCode)
          {
            PrintActiveInfo (hwnd);
            WriteScriptFile (SEEK);
          }

          WinDismissDlg (hwnd, PB_OK);
          break;

        case DID_CANCEL:
          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,DID_CANCEL);
          break;

        case PB_HELP:
          if (hwndHelpInstance)
             DisplayHelpPanel(SEEK);
          break;
      }

   case WM_CONTROL:
     break;

   case WM_INITDLG:

     /* Max no. of chars for EF_STRMNUM is 3 */

     WinSendDlgItemMsg( hwnd, EF_STRMNUM,
                        EM_SETTEXTLIMIT, MPFROMSHORT(3), 0L );

     /* Max no. of chars for EF_SEEKPOS is 8 */

     WinSendDlgItemMsg( hwnd, EF_SEEKPOS,
                        EM_SETTEXTLIMIT, MPFROMSHORT(8), 0L );
     break;

   case WM_PAINT:
     /* Execute default processing */
     return( WinDefDlgProc( hwnd, msg, mp1, mp2) );
     break;
   default:
     return(WinDefDlgProc(hwnd,msg,mp1,mp2));
  }
  return(FALSE);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   DevStatDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY DevStatDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{

  RC     RetCode;                   /* API Return Code   */
  CHAR   szTemp[10];                /* Temporary Storage */
  USHORT i;                         /* Loop Index        */
  BOOL   fFoundStream = FALSE;      /* Local error flag  */

  switch(msg)
  {
    case WM_COMMAND:
      switch (SHORT1FROMMP(mp1))
      {
        case PB_HELP:
          if (hwndHelpInstance)
             DisplayHelpPanel(STATUS);
          break;

        case PB_OK:
          WinQueryWindowText ( WinWindowFromID(hwnd,EF_STREAMNUM),
                               sizeof(szTemp), szTemp );

          CheckString (szTemp);
          ulIndex = atol(szTemp);
          /*
          ** loop thro to find the stream data entry
          ** corresponding to this stream.
          */

          for (i = 0; i < StreamCount; i++ )
          {
              if (aStreamData[i].hStream == ulIndex)
              {
                  fFoundStream = TRUE;
                  ulIndex = i;
                  break;
              }
          }

          if (!fFoundStream)
          {
              mprintf ("Stream Not Found %d\n",ulIndex);
              WinDismissDlg (hwnd, PB_OK);
              break;
          }

          aStreamData[ulIndex].hStream = atol(szTemp);

          /* call the worker */

          RetCode = AudioStatus (ulIndex);

          if (!RetCode)
              WinSendMsg (hwnd1, WM_COMMAND,
                          MPFROMSHORT(IDM_DISPSTATUS),(MPARAM)0);
          else
              mprintf ("Not Supported \n");

          WinDismissDlg (hwnd, PB_OK);

          break;

        case DID_CANCEL:
          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,DID_CANCEL);
          break;
       }
       break;

    case WM_CONTROL:
      break;

    case WM_INITDLG:
      /* Max no. of chars for EF_STREAMNUM_1 is 8 */
      WinSendDlgItemMsg( hwnd, EF_STREAMNUM,
                        EM_SETTEXTLIMIT, MPFROMSHORT(8), 0L );
      break;

    case WM_PAINT:
      /* Execute default processing */
      return(WinDefDlgProc(hwnd, msg, mp1, mp2) );
      break;
    default:
      return(WinDefDlgProc(hwnd,msg,mp1,mp2));
  }
  return(FALSE);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   DestroyStrmDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY DestroyStrmDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{

  RC      RetCode;                 /* API Return Code   */
  CHAR    szTemp[10];              /* Temporary Storage */
  USHORT  i;                       /* Loop Index        */
  HSTREAM hStream;                 /* Stream Handle     */
  BOOL    fFoundStream = FALSE;    /* Local error flag  */

  switch(msg)
  {
    case WM_COMMAND:
      switch (SHORT1FROMMP(mp1))
      {
        case PB_HELP:
          if (hwndHelpInstance)
             DisplayHelpPanel(DESTROY);
          break;

        case PB_OK:
          WinQueryWindowText ( WinWindowFromID(hwnd,EF_STREAMNUM),
                               sizeof(szTemp), szTemp );

          CheckString (szTemp);
          ulIndex = atol(szTemp);
          /*
          ** loop thro to find the stream data entry
          ** corresponding to this stream.
          */

          for (i = 0; i < StreamCount; i++ )
          {
              if (aStreamData[i].hStream == ulIndex)
              {
                  fFoundStream = TRUE;
                  ulIndex = i;
                  break;
              }
          }

          if (!fFoundStream)
          {
              mprintf ("Stream Not Found %d\n",ulIndex);
              WinDismissDlg (hwnd, PB_OK);
              break;
          }

          aStreamData[ulIndex].hStream =hStream = atol(szTemp);

          /* call the worker */

          RetCode = DestroyStream();

          /* Update Display with command information and return
             codes.
           */
          PrintCmndInfo (DESTROY, hStream,RetCode);

          if (!RetCode)
          {
            PrintActiveInfo (hwnd);
            WriteScriptFile (DESTROY);
          }
          WinDismissDlg (hwnd, PB_OK);
          break;

        case DID_CANCEL:
          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,DID_CANCEL);
          break;
       }
       break;

    case WM_CONTROL:
      break;

    case WM_INITDLG:
      /* Max no. of chars for EF_STREAMNUM_1 is 8 */
      WinSendDlgItemMsg( hwnd, EF_STREAMNUM,
                        EM_SETTEXTLIMIT, MPFROMSHORT(8), 0L );
      break;

    case WM_PAINT:
      /* Execute default processing */
      return(WinDefDlgProc(hwnd, msg, mp1, mp2) );
      break;
    default:
      return(WinDefDlgProc(hwnd,msg,mp1,mp2));
  }
  return(FALSE);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   DevNamesDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY DevNamesDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{

  SHORT sDataIndex;              /* Data Item Index   */
  SHORT k,i = 0;                 /* Loop Indicies     */
  RC    ReturnCode;              /* API Return Code   */
  static char  buffer[512];      /* Temporary Storage */
  static char space[] = "               ";/* Text Allignment */
  static char tokensep[]=" \t";  /* Token Separotor   */
  PCHAR  pchWord;                /* Temporary pointer */


    switch(msg)
    {

        case WM_CONTROL:

           if ( SHORT2FROMMP ( mp1 ) ==  LN_ENTER )
                WinSendMsg ( hwnd, WM_COMMAND, MPFROMSHORT ( PB_OK ),
                          MPFROM2SHORT ( CMDSRC_PUSHBUTTON, TRUE ) );
          break;
        case WM_COMMAND:
            switch (SHORT1FROMMP(mp1))
            {
              case PB_OK:

                 sDataIndex = SHORT1FROMMR (WinSendDlgItemMsg(hwnd,
                                                         LI_DEVNAMES,
                                                         LM_QUERYSELECTION,
                                                         (MPARAM)LIT_FIRST,
                                                         (MPARAM) 0L));
                if (sDataIndex == LIT_NONE) {
                    mprintf ("No Device Selected \n");
                    break;
                }
                WinSendDlgItemMsg( hwnd,
                                   LI_DEVNAMES,
                                   LM_QUERYITEMTEXT,
                                   MPFROM2SHORT(sDataIndex,512),
                                   MPFROMP(buffer));
                pchWord = strtok (buffer,tokensep);
                pchWord = strtok (NULL,tokensep);

                if (pchWord != NULL) {
                   /*
                   ** Update The AudioDCB field to reflect the
                   ** selected Physical device name. Look in the
                   ** Physical Device Data Block to find the right
                   ** Block and update resource information.
                   */

                    for (i = 0; i < MAXSTREAMS; i++)
                    {
                        if ((strnicmp (PhysicalDevData[i].szPhysicalDevName,
                                    pchWord,7)) == 0)
                        {
                             for (k = StreamCount; k < MAXSTREAMS ; k++ )
                             {
                                  aStreamData[k].ulResID = PhysicalDevData[i].ulResourceId;
                                  strcpy (aStreamData[k].AudioDCB.szDevName,
                                          PhysicalDevData[i].szPhysicalDevName);
                             }
                        }
                    }
                } /* Null pchWord */
                sACPAOpened = 0;

                WriteScriptFile (DEVNAME);
                /* Dismiss the dialog box */

                WinDismissDlg (hwnd,PB_OK);
                break;

                case DID_CANCEL:

                    /* Dismiss the dialog box */
                    WinDismissDlg (hwnd,DID_CANCEL);
                break;

              case PB_HELP:

                 if (hwndHelpInstance)
                 {
                   DisplayHelpPanel(DEVNAME);
                 }
               break;
            }
            break;
        case WM_INITDLG:

             /* Max no. of chars for Device Names list is 512 */
             WinSendDlgItemMsg(hwnd,
                               LI_DEVNAMES,
                               EM_SETTEXTLIMIT,
                               MPFROMSHORT(512),
                               0L);


             /* Query Installed devices and initialize */

             for (k = 0; k < 20 ; k++ )
             {
                 if (PhysicalDevData[k].ulResourceId != 0)
                 {
                     strcpy (buffer,PhysicalDevData[k].szLogDevName);

                     strncat (buffer, space,
                              20-strlen(PhysicalDevData[k].szLogDevName));

                     strcat (buffer,PhysicalDevData[k].szPhysicalDevName);

                     strncat (buffer, space,
                              15-strlen(PhysicalDevData[k].szPhysicalDevName));

                     strcat (buffer,PhysicalDevData[k].szProductInfo);

                     InsertListItem (LI_DEVNAMES,LIT_SORTASCENDING,buffer);
                 }
                 else
                   break;
             }
             break;

        case WM_PAINT:

            /* Execute default processing */
            return(WinDefDlgProc(hwnd,
                                 msg,
                                 mp1,
                                 mp2)
                                );
            break;
        default:
            return(WinDefDlgProc(hwnd,msg,mp1,mp2));
    }
    return(FALSE);
}


/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   DevStatDispDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY DevStatDispDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{

  SHORT        sDataIndex;      /* Data Item Index   */
  RC           ReturnCode;      /* API return Code   */
  static char  buffer[125];     /* Temporary buffer  */
  PSZ          pszTemp;         /* Temporary storage */

  switch(msg)
  {
    case WM_COMMAND:
      switch (SHORT1FROMMP(mp1))
      {
        case PB_OK:
          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,PB_OK);
          break;

        case DID_CANCEL:
          /* Dismiss the dialog box */
          WinDismissDlg (hwnd,DID_CANCEL);
          break;

        case PB_HELP:
          if (hwndHelpInstance)
            DisplayHelpPanel(STATUS);
            break;
      }
      break;

    case WM_CONTROL:
      break;

    case WM_INITDLG:
      memset (buffer,0,sizeof(buffer));

      pszTemp = LongToString (rAudioStatus.lSRate);

      CheckString (pszTemp);

      strcpy (buffer,pszTemp);
      /*
      ** Insert the item into the list box.
      */
      InsertListItem (LI_DEVSTAT,LIT_END,buffer);

      /* Convert it to the right data type */

      pszTemp = LongToString (rAudioStatus.lBitsPerSRate);

      CheckString (pszTemp);
      strcpy (buffer,pszTemp);
      InsertListItem (LI_DEVSTAT,LIT_END,buffer);

      pszTemp = LongToString (rAudioStatus.lBsize);

      CheckString (pszTemp);
      strcpy (buffer,pszTemp);
      InsertListItem (LI_DEVSTAT,LIT_END,buffer);

      pszTemp = LongToString ((LONG)rAudioStatus.sMode);

      CheckString (pszTemp);
      strcpy (buffer,pszTemp);
      InsertListItem (LI_DEVSTAT,LIT_END,buffer);

      pszTemp = LongToString ((LONG)rAudioStatus.sChannels);
      CheckString (pszTemp);
      strcpy (buffer,pszTemp);
      InsertListItem (LI_DEVSTAT,LIT_END,buffer);

      strcpy (buffer,aStreamData[ulIndex].AudioDCB.szDevName);
      InsertListItem (LI_DEVSTAT,LIT_END,buffer);

      pszTemp = LongToString (rAudioStatus.ulOperation);
      CheckString (pszTemp);
      strcpy (buffer,pszTemp);
      InsertListItem (LI_DEVSTAT,LIT_END,buffer);

      pszTemp = LongToString (aStreamData[ulIndex].hDevHandle);
      CheckString (pszTemp);
      strcpy (buffer,pszTemp);
      InsertListItem (LI_DEVSTAT,LIT_END,buffer);
      break;

   case WM_PAINT:
     /* Execute default processing */
     return(WinDefDlgProc(hwnd, msg, mp1, mp2) );
     break;

   default:
     return(WinDefDlgProc(hwnd,msg,mp1,mp2));
 }
 return(FALSE);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   QryStrmTimeDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT EXPENTRY QryStrmTimeDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  RC        RetCode;                /* API Return Code   */
  CHAR      szTemp[10];             /* Temporary Storage */
  CHAR      buffer[115];            /* Temporary buffer  */
  USHORT    i;                      /* Loop Index        */
  WNDPARAMS EfWndParams;            /* Entry Field cntrl */
  BOOL      fFoundStream = FALSE;   /* Local Flag        */

  switch(msg)
  {
    case WM_COMMAND:
      switch (SHORT1FROMMP(mp1))
      {
        case PB_OK:

          WinQueryWindowText ( WinWindowFromID(hwnd,EF_STREAMNUM),
                               sizeof(szTemp), szTemp );

           ulIndex = atol(szTemp);

          /*
          ** loop thro to find the stream data entry
          **   corresponding to this stream.
          */

          for (i = 0; i < StreamCount; i++ )
          {
              if (aStreamData[i].hStream == ulIndex)
              {
                  fFoundStream = TRUE;
                  ulIndex = i;
                  break;
              }
          }
          if (!fFoundStream)
          {
              mprintf ("Stream Not Found %d\n", ulIndex);
              WinDismissDlg (hwnd, PB_OK);
              break;
          }
          aStreamData[ulIndex].hStream = atol (szTemp);

          /* Call the worker  */

          RetCode = GetStreamTime();
          if (!RetCode)
          {
              strcpy (buffer,"Current Stream Time (MMTIME)  is ");
              strcat (buffer,LongToString(mmtimeCurrTime));

              WinMessageBox( HWND_DESKTOP,       /* parent window handle    */
                             hwnd,               /* owner window handle     */
                             buffer,             /* pointer to message text */
                             "Stream Time",      /* pointer to title text   */
                             MSG_BOX_ID,         /* message box identifier  */
                             MB_OK | MB_INFORMATION);/* message box style   */
          }
          /*
          ** Update Display with command information and return
          ** codes.
          */
          PrintCmndInfo (QRYTIME,ulIndex, RetCode);
        break;

        case DID_CANCEL:
          WinDismissDlg (hwnd, PB_OK);
          break;

        case PB_HELP:
          if (hwndHelpInstance)
            DisplayHelpPanel(TIME);
            break;
      }
      break;

   case WM_INITDLG:
     /* Max no. of chars for EF_STREAMNUM is 3 */


     WinSendDlgItemMsg(hwnd, EF_STREAMNUM, EM_SETTEXTLIMIT,
                       MPFROMSHORT(3), 0L);
     break;

   case WM_PAINT:
     /* Execute default processing */

     return(WinDefDlgProc(hwnd, msg, mp1, mp2) );
     break;

   default:
     return(WinDefDlgProc(hwnd,msg,mp1,mp2));
  }
  return(FALSE);
}
