/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
/*********************** START OF SPECIFICATIONS *******************************
*
* SOURCE FILE NAME: PMADDE.H
*
* DESCRIPTIVE NAME: Presentation Manager Audio Device Driver Exerciser.
*
*
* FUNCTION: Component Global Data Structures, and Constant definitions
*
*
*/

/* PMADDE application constant definitions  */

#define FILENAME_SIZE           129 /* Maximum File Name              */
#define MAXSTREAMS              20  /* Maximum Number of data streams */
#define MAXHANDLERS             10  /* Max Number of Stream Handlers  */
#define DEVICECHARS             9   /* Max Chars in Device Name       */
#define HM_RESOURCEID           0   /* Help Resource ID               */
#define MAX_SYSINFO_RETURN      512 /* Maximum sysinfo return size    */
#define NUMDATATYPES            67  /* Data Types known to the system */
#define MAXDEVICES              20  /* Max Number of physical devices */

/* PMADDE Internal data structure definitions  */

/*
** Stream Data Structure is a block that contains pertinent information
** necesaary to manage Data Streams. This is a per unit data item tag.
** A Global array of MAXSTREAMS elements is created that fixes the
** maximum number of streams this application can handle per process.
** Note that the memory allocation is static and sizeof (STREAMDATA) *
** MAXSTREAMS amount of memory is set asid statically.
*/
typedef struct {
   SZ  nszState[10];                /* Streaming State                */
   SZ  nszDataType[11];             /* Stream Data Type               */
   SZ  nszFileName[FILENAME_SIZE];  /* Stream Data File               */
   SZ  nszReserved[8];              /* Reserved Space                 */
   HID hidSrc;                      /* Stream Source ID               */
   HID hidTgt;                      /* Stream Target ID               */
   HSTREAM  hStream;                /* Handle to this Stream          */
   HEVENT   EventHandle;            /* Handle to Stream events        */
   ULONG    ulOperation;            /* Playback or Record Mode        */
   ULONG    ulmode;                 /* Hardware Mode (PCM,etc)        */
   ULONG    ulStartFlag;            /* Stream Start Flags             */
   ULONG    ulStopFlag;             /* Stream Stop Flags              */
   ULONG    ulSeekFlag;             /* Stream Seek Flags              */
   ULONG    ulSeekPoint;            /* Stream Seek Value              */
   PSPCBKEY pSpcbKey;               /* Stream Protocol Block          */
   PIMPL_EVCB  pEventBlock;         /* Implicit Event Block           */
   DCB_AUDIOSH AudioDCB;            /* Audio Device Control Block     */
   ACB         AssCtlBlk;           /* Associate Control Block        */
   HFILE       hDevHandle;          /* Physical Device handle         */
   ULONG       ulCntrlField;        /* Internal Control Filed         */
   HMMIO       hmmio;               /* MMIO file handle               */
   HEV         hevThrdSem;          /* Thread Control Event Semaphore */
   RC          RetCode;             /* Return Code                    */
   PVOID       pAudDevData;         /* Audio Device Data Table        */
   ULONG       ulResID;             /* Resource ID Burnt in           */
   ULONG       ulDeviceID;          /* Device ID                      */
   ULONG       ulNumDataTypes;      /* Known Number of data types     */
   PVOID       pDevDataTable;       /* Device data table pointer      */
   PVOID       pDevDTblStrt;        /* Device data table strt ptr     */
   PVOID       pDataTypes;          /* String Data types              */
}STREAMDATA;

/*
** Active handlers information block is defined to contain
** The MMPM/2 Stream handler name, the stream handler class,
** The handler IDs as source and Target.
*/
typedef struct {
   SZ       nszHandlerName[DEVICECHARS]; /* MMPM2 Stream Handler Name */
   SZ       nszHandlerClass[20];         /* Stream Handler Class      */
   HID      hidSrc;                      /* Stream Handler Source ID  */
   HID      hidTgt;                      /* Stream Handler Target ID  */
}ACTIVEHANDLERS;

#pragma pack(1)
typedef struct _class_data
  {
   ULONG      ulDataType;           /* PCM, ADPCM, etc..       */
   ULONG      ulSubType;            /* Freq, bits/sam,..       */
   USHORT     sSRate;               /* 11,22,44 Khz            */
   USHORT     sBitsPerSRate;        /* 8 or 16 bits/sam        */
   USHORT     usChannels;           /* Mono or Stereo          */
   }CLASS_DATA;
#pragma pack()

/* Known stream data types */
struct _datatypes
  {
   ULONG      ulDataType;           /* PCM, ADPCM, etc..       */
   ULONG      ulSubType;            /* Freq, bits/sam,..       */
   USHORT     sSRate ;              /* Sampling Freq           */
   SZ         szDataType[80];       /* Data Type String        */
   };

/* Device Specific data table */

typedef struct _deviceData
  {
   SZ     szPhysicalDevName[DEVICECHARS];/* Physical device name     */
   SZ     szProductInfo[MAX_PRODINFO];   /* Product Information      */
   SZ     szLogDevName[30];              /* Logical Device Name      */
   ULONG  ulResourceId;                  /* Device Data Table ID     */
  }PHYSDEV_DATA;

/* FIle Information structure for DosQueryPathInfo() */
struct FileInfo {
    USHORT  DateCreate;                  /* Creation Date      */
    USHORT  TimeCreate;                  /* Creation Time      */
    USHORT  DateAccess;                  /* Access Date        */
    USHORT  TimeAccess;                  /* Access Time        */
    USHORT  DateWrite;                   /* Last Write Date    */
    USHORT  TimeWrite;                   /* Last Write Time    */
    LONG    cbEOF;                       /* EOF                */
    LONG    cbAlloc;                     /* Allocation bytes   */
    ULONG   attr;                        /* File Attributes    */
}FileInfoStr;


SZ      szLogFileName[128];              /* Log file Name holder      */
HFILE   hLogFile;                        /* Log file handle           */

/* PMADDE Internal function prototypes  */

FNWP   PMADDEWinProc          ;     /* Main Window Procedure               */
FNWP   GetStrmHndlrDlgProc    ;     /* Get Stream Handler Dialog Procedure */
FNWP   StartStrmDlgProc       ;     /* Start stream Dialog Procedure       */
FNWP   StopStrmDlgProc        ;     /* Stop stream Dialog Procedure        */
FNWP   SeekStrmDlgProc        ;     /* Seek stream Dialog Procedure        */
FNWP   DestroyStrmDlgProc     ;     /* Destroy stream Dialog Procedure     */
FNWP   QryStrmTimeDlgProc     ;     /* Stream Time Dialog Procedure        */
FNWP   AssociateStrmDlgProc   ;     /* Assoc stream Dialog Procedure       */
FNWP   DevNamesDlgProc        ;     /* Device Names Dialog Procedure       */
FNWP   DevStatDlgProc         ;     /* Device status Dialog Procedure      */
FNWP   DevStatDispDlgProc     ;     /* Device status Dialog Procedure      */
FNWP   CreateStrmDlgProc      ;     /* Create stream Dialog Procedure      */
FNWP   InstallProtoDlgProc    ;     /* Install Protocol Dialog Procedure   */
FNWP   InstalHandlersDlgProc  ;     /* Installed Handlers Dialog Procedure */
FNWP   ActiveHndlrsWinProc    ;     /* Active Handlers Window Procedure    */
FNWP   EventMonitorWinProc    ;     /* Event Monitor Window Procedure      */
FNWP   CmndMonitorWinProc     ;     /* Command Monitor Window Procedure    */
FNWP   AboutDlgProc           ;     /* Product Info Dialog Procedure       */
VOID   Initialize()           ;     /* PMADDE Application Initialization   */


VOID   PrintActiveInfo(HWND hwnd);     /* Print Active Window Information */
VOID   PrintCmndInfo(USHORT,ULONG,RC); /* Print Cmnd Window Information   */
VOID   PrintEventInfo();               /* Print Event Window Information  */
PVOID  ListInsertStreamDataTypes (HWND, ULONG, MPARAM ); /* Insert Item     */
VOID   ListInsertInstalledHandlers(HWND,ULONG);          /* Update List     */
VOID   ListInsertProtocolDataTypes (HWND );              /* Update List     */
VOID   Terminate (HMQ );          /* Terminate Application */
PSZ    LongToString (LONG );      /* Conversion helper     */
LONG   mprintf(PSZ pszFmt, ...);  /* Print messages        */


RC     OpenFileDlg (HWND,PSZ,PSZ ,PSZ );   /* Open File Dialog     */
APIRET OpenFile(PSZ , ULONG* );            /* Open File            */
APIRET WriteFile(HFILE,PSZ  );             /* Write File           */
APIRET Write2File(HFILE,PSZ );             /* Write File           */
BOOL   SetFont( HWND, ULONG, PSZ );        /* Set Font             */
VOID   CreateStaticText (HWND );           /* Create Text controls */
RC     WriteScriptFile(USHORT );           /* Write Sctipt token   */
VOID   OpenShell();                        /* Fork Shell           */
VOID   CreateFontFixed (HWND );            /* Create Fixed font    */

RC     GetStreamHandler();                      /* Get Stream Handler Worker */
RC     CreateStream();                          /* Create Stream Worker      */
RC     GetProtocol(ULONG, PSPCBKEY, PSPCB);     /* Get Protocol              */
RC     InstallProtocol(ULONG,  PSPCBKEY, PSPCB);/* Install Protocol          */
RC     QuitProg();                              /* Quit Program              */
RC     DestroyStream();                         /* Destroy Stream Worker     */
RC     AssociateStream();                       /* Associate Stream Worker   */
RC     SeekStream();                            /* Seek Stream Worker        */
RC     StopStream();                            /* Stop Stream Worker        */
RC     GetStreamTime();                         /* Get Stream Time Worker    */
LONG   AudioStatus(ULONG);                      /* Audio Device Status       */
void _Optlink StartStream();                    /* Start Stream Worker       */
BOOL   CheckUserSelection (HWND);               /* Query User Choices        */
BOOL QueryInstalledDevices () ;                 /* Query Installed Devices   */
APIRET FixDeviceData () ;                       /* Fix Device Specific Info  */
void _Optlink  ReadNExecScriptFile();           /* Read N Exec Script File  */

/* PMADDE Macro definitions  */

#define  InsertListItem(LboxID,     \
                        InsertOrder,\
                        Text)       \
         (WinSendDlgItemMsg (hwnd,                      \
                             LboxID,                    \
                             LM_INSERTITEM,             \
                             MPFROMSHORT(InsertOrder),  \
                             MPFROMP(Text)))

#define  InitMLE(hwnd,     \
                 MLEBuffer) \
         (WinSendMsg (hwnd,                \
                      MLM_SETIMPORTEXPORT, \
                      MPFROMP(MLEBuffer),  \
                      MPFROMSHORT((USHORT)sizeof(MLEBuffer))))

#define  SetMLEFormat(hwnd)         \
         (WinSendMsg (hwnd,         \
                      MLM_FORMAT,   \
                      MPFROMSHORT(MLFIE_NOTRANS),\
                      MPFROMSHORT(0)))



#define ImportMLE(hwnd,            \
                  pt,              \
                  MLEBuffer)       \
        (WinSendMsg (hwnd,         \
                     MLM_IMPORT,   \
                     MPFROMP(&pt), \
                     MPFROMSHORT((USHORT)strlen(MLEBuffer))))


#define DeleteMLE(hwnd,                       \
                  MLEBuffer)                  \
        (WinSendMsg (hwnd,                    \
                     MLM_DELETE,              \
                     MPFROMLONG(ipt),         \
                     MPFROMLONG((ULONG)sizeof(MLEBuffer))))

#define DisableMLERefresh(hwnd)         \
        (WinSendMsg (hwnd,              \
                     MLM_DISABLEREFRESH,\
                     (MPARAM)0,         \
                     (MPARAM)0))

#define EnableMLERefresh(hwnd)          \
        (WinSendMsg (hwnd,              \
                     MLM_ENABLEREFRESH, \
                     (MPARAM)0,         \
                     (MPARAM)0))        \


#define DisplayHelpPanel(idPanel)                \
        ( WinSendMsg ( hwndHelpInstance,         \
                       HM_DISPLAY_HELP,          \
                       MPFROM2SHORT(idPanel, NULL), \
                       MPFROMSHORT(HM_RESOURCEID)))

#define SetPosition(idWin)                      \
        (WinSetWindowPos ( WinWindowFromID(hwnd,idWin), HWND_TOP,    \
                          (SHORT)usxloc, (SHORT)usyloc,              \
                          (SHORT)usWidth, (SHORT)usHeight,           \
                          SWP_MOVE | SWP_SIZE | SWP_ACTIVATE |SWP_SHOW ))

#define CreateTxt(hwnd,text,idWin)                                   \
        (WinCreateWindow( hwnd, WC_STATIC, text,                      \
                         SS_TEXT | DT_WORDBREAK | DT_TOP | DT_LEFT,   \
                         (SHORT)usxloc, (SHORT)usyloc, (SHORT)usWidth,\
                         (SHORT)usHeight, hwnd, HWND_TOP,             \
                         idWin, 0L, NULL ))



/* PMADDE PM constant definitions  */

#define CREATE                  1                      /* Stream Creation */
#define ASSOCIATE               2                      /* Stream Assoc    */
#define START                   3                      /* Stream Start    */
#define STOP                    4                      /* Stream Stop     */
#define SEEK                    5                      /* Stream Seek     */
#define GETHANDLER              6                      /* Get Handler     */
#define INSTALL                 7                      /* Install Proto   */
#define PLAY                    8                      /* Play Stream     */
#define DESTROY                 9                      /* Destroy Stream  */
#define INIT                    10                     /* Initialize      */
#define DEVNAME                 11                     /* Device Name     */
#define QRYTIME                 12                     /* Query Time      */
#define EXIT                    13                     /* Exit            */
#define PAUSE                   14                     /* PAUSE Stream    */
#define STATUS                  15                     /* STATUS Stream   */
#define TIME                    16                     /* TIME Stream     */
#define FILE                    17                     /* Help File       */
#define STREAM                  18
#define OPTIONS                 19
#define INDEXHELP               64
#define GENERALHELP             20
#define USINGHELP               21
#define HELPPRODINFO            22
#define GENHLP                  29
#define KEYSHELP                23
#define ABOUT                   24
#define VIEWS                   25
#define SCRIPT                  26
#define HANDLERS                27
#define LOGFILE                 28
#define HELP_HOWTO              32
#define OPEN                    30
#define HOWTO_SUPP              31
#define ID_ACTWIN               47                     /* Active Window   */
#define ID_COMWIN               48                     /* Command Window  */
#define ID_EVEWIN               49                     /* Event Window    */
#define ID_ASSOCDLG             50                     /* Assoc Dialog    */
#define ID_QRYTIMEDLG           51                     /* Strm Time Dlg   */
#define ID_SEEKDLG              52                     /* Seek Dialog     */
#define ID_STOPDLG              53                     /* Stop Dialog     */
#define ID_STARTDLG             54                     /* Start Dialog    */
#define ID_DESTROYDLG           55                     /* Destroy Dialog  */
#define ID_CREATEDLG            56                     /* Create Dialog   */
#define ID_GETHNDLRDLG          57                     /* Get Hndlr Dlg   */
#define ID_INSTPROTDLG          58                     /* Instl Proto Dlg */
#define ID_INSTLDHNDLRS         59                     /* Instl Hndlrs    */
#define ID_INSTHNDLRSDLG        60                     /* Instl Hndlr Dlg */
#define ID_DEVNAMESDLG          61                     /* Device Names    */
#define ID_DEVSTATDLG           62                     /* Device Stat Dlg */
#define ID_DEVSTATDISPDLG       63                     /* Dev Stat Disp   */
#define IDM_PRODUCTINFO          106                    /* Product Info    */
#define IDM_DISPSTATUS          933                    /* Menu Status     */
#define IDR_BITMAP              108                    /* Bitmap          */
#define IDC_BITMAP              109                    /* Bitmap          */
#define IDC_AUDIOFILE            200                    /*  E              */
#define EF_BUFFERSIZE           201                    /*  N              */
#define EF_GETHNDLR             202                    /*  T              */
#define EF_HIDTGT               203                    /*  R              */
#define EF_MAXBUFF              204                    /*  Y              */
#define EF_MINBUFF              205                    /*                 */
#define EF_SEEKPOS              206                    /*  F              */
#define EF_SPCBKEY              207                    /*  I              */
#define EF_SRCHID               208                    /*  E              */
#define EF_SRCMINBUFF           210                    /*  L              */
#define EF_STREAMNUM            211                    /*  D              */
#define EF_STRMNUM              213                    /*                 */
#define EF_TGTHID               214                    /*  C              */
#define EF_TGTMINBUF            215                    /*  O              */
#define EF_TARGETID             216                    /*  N              */
#define EF_STRMTIME             217                    /*  STANTS         */
#define IDM_HELP                300                    /* Help Menu       */
#define IDM_HELP_GEN            301                    /* General help    */
#define IDM_HELP_KEYS           302                    /* Keys Help       */
#define IDM_HELP_PRODINFO       303                    /* Product Info    */
#define IDM_HELP_USING          304                    /* Using Help      */
#define IDM_EXECSCRIPT          334                    /* Menu Script     */
#define IDM_HELP_INDEX          335                    /* Help Index      */
#define ID_MAINWIN              001                    /* Main Window     */
#define ID_DDK                  002                    /* DDK Icon ID     */
#define ID_ACTLIST              003                    /* List box ID     */
#define IDM_OS2SHELL            306                    /* Menu Os2 shell  */
#define IDM_ASSOCIATE           307                    /* Menu Associate  */
#define IDM_CREATE              308                    /* Menu Create     */
#define IDM_DESTROY             309                    /* Menu Destroy    */
#define IDM_EXIT                310                    /* Menu Exit       */
#define IDM_GETHANDLER          311                    /* Menu Get handler*/
#define IDM_INSTPROT            312                    /* Menu Protocol   */
#define IDM_SEEK                313                    /* Menu Seek       */
#define IDM_START               314                    /* Menu Start      */
#define IDM_STOP                315                    /* Menu Stop       */
#define IDM_TIME                316                    /* Menu Time       */
#define IDM_INSTHNDLRS          317                    /* Menu View Hndlrs*/
#define IDM_OPTIONS             318                    /* Menu Options    */
#define IDM_DEVNAME             319                    /* Menu Device name*/
#define IDM_DEVSTATUS           320                    /* Menu Status     */
#define IDM_LOGFILE             321                    /* Menu Log File   */
#define ID_STRMCMND             322                    /* Stream Cmnd     */
#define IDM_IHANDLERS           323                    /* Menu Handlers   */
#define IDM_SCRIPT              324                    /* Menu Script     */
#define LI_PROTOCOL             400                    /* Protocol List ID */
#define LI_STRMDATATYPES        401                    /* DataType LID     */
#define LI_INSTHNDLR            403                    /* Hndlrs LID       */
#define LI_INHNDLRS             404                    /* Hndlrs LID       */
#define LI_DEVNAMES             405                    /* Devices LID      */
#define LI_ACTIVEHNDLRS         406                    /* Active Hndlrs LID*/
#define LI_DEVSTAT              407                    /* Device Status LID*/
#define PB_BYTES                500                    /* Bytes Button     */
#define PB_DISCARD              502                    /* Discard Button   */
#define PB_END                  503                    /* End Button       */
#define PB_FLUSH                504                    /* Flush Button     */
#define PB_HELP                 505                    /* Help Button      */
#define PB_OK                   506                    /* OK Button        */
#define PB_PREROLL              507                    /* Preroll Button   */
#define PB_RELATIVE             508                    /* Relative Button  */
#define PB_START                509                    /* Start Button     */
#define PB_TIME                 510                    /* Time Button      */
#define PB_PAUSE                511                    /* Pause Button     */
#define PB_ABSOLUTE             512                    /* Absolute Button  */
#define PB_OPEN                 513                    /* Open Button      */
#define STX_                    600                    /* Static Text ID  */
#define STX_1                   601                    /* Static Text ID  */
#define STX_10                  602                    /* Static Text ID  */
#define STX_11                  603                    /* Static Text ID  */
#define STX_12                  604                    /* Static Text ID  */
#define STX_13                  605                    /* Static Text ID  */
#define STX_14                  606                    /* Static Text ID  */
#define STX_15                  607                    /* Static Text ID  */
#define STX_16                  608                    /* Static Text ID  */
#define STX_17                  609                    /* Static Text ID  */
#define STX_2                   630                    /* Static Text ID  */
#define STX_3                   610                    /* Static Text ID  */
#define STX_4                   611                    /* Static Text ID  */
#define STX_5                   612                    /* Static Text ID  */
#define STX_6                   613                    /* Static Text ID  */
#define STX_7                   614                    /* Static Text ID  */
#define STX_8                   615                    /* Static Text ID  */
#define STX_9                   616                    /* Static Text ID  */
#define STX_21                  666                    /* Static Text ID  */
#define STX_AUDIOFILENAME       617                    /* Static Text ID  */
#define STX_DATATYPE            618                    /* Static Text ID  */
#define STX_SEEKPOS             619                    /* Static Text ID  */
#define STX_SRCHID              620                    /* Static Text ID  */
#define STX_STREAMNUM           621                    /* Static Text ID  */
#define STX_TGTHID              622                    /* Static Text ID  */
#define MSG_BOX_ID              623                    /* Msg Box ID      */
#define MLE_ACTWIN              624                    /* MLE ID          */
#define MLE_COMWIN              625                    /* MLE ID          */
#define MLE_EVEWIN              626                    /* MLE ID          */
#define STX_DD                  628                    /* Static Text ID  */
#define QUERY_ITEM              629                    /* Flag Query Item */
#define INSERT_ITEM             630                    /* Flag Insert Itm */

/* PMADDE Help Manager  definitions */

#define  MAINHELPTABLE          700    /* Main Help Table ID    */
#define  HELP_SUBTABLE_STREAM   701    /* Stream Help Table ID  */
#define  HELP_SUBTABLE_OPTIONS  702    /* Options Help table ID */
#define  HELP_MAIN              703
/* PMADDE Display category static string definitions */

#define  DISPCAT1               "Handler Name"         /* static HN       */
#define  DISPCAT2               "Handler Class"        /* static HC       */
#define  DISPCAT3               "Source     ID"           /* static SID      */
#define  DISPCAT4               "Target     ID"           /* static TID      */
#define  DISPCAT5               "Stream Device"        /* static SD       */
#define  DISPCAT6               "Stream Number"        /* static SN       */
#define  DISPCAT7               "Stream State"         /* static SS       */
#define  DISPCAT8               "Stream Source"        /* static SS       */
#define  DISPCAT9               "Stream Target"        /* static ST       */
#define  DISPCAT10              "Stream Data"          /* static SD       */
#define  DISPCAT11              "Stream Number"        /* static SN       */
#define  DISPCAT12              "Stream Event"         /* static SE       */
#define  DISPCAT13              "Stream Command"       /* static SC       */
#define  DISPCAT14              "Command Status"       /* static CS       */
#define  DISPCAT15              "Return Code"          /* static RC       */

/* PMADDE Application Default definitions */

#define DEFAULT_AUDIO_DEVICE      "AUDIO1$"            /* Def Aud Device  */
#define DEFAULT_SCRIPT_FILE       "PMADDE.SCR"         /* Defalt Scr File */
#define DEFAULT_LOG_FILE          "PMADDE.LOG"         /* Defalt Log File */
#define DEFAULT_FONT              "10.System Monospaced"/* Default Font    */
#define hspace  "             "

/* Known Audio Devices   */

#define MAUDIO                  1      /* IBM M Audio Card   */
#define SB                      2      /* Sound Blaster      */
#define SB_PRO                  3      /* Sound Blaster      */
#define SB_16                   4      /* Sound Blaster      */
#define PAS_16                  5      /* Pro Audio Spectrum */
#define FOXGLOVE                6      /* Fox Glove          */
#define SPEAKER                 7      /* Speaker Device     */
