/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/* The following IBM OS/2 2.1 source code is provided to you solely for     */
/* the purpose of assisting you in your development of OS/2 2.x device      */
/* drivers. You may use this code in accordance with the IBM License        */
/* Agreement provided in the IBM Device Driver Source Kit for OS/2. This    */
/* Copyright statement may not be removed.                                  */
/*                                                                          */
/****************************************************************************/
/********************** START OF SPECIFICATIONS ****************************
*
* SOURCE FILE NAME: PMADSPI.C
*
* DESCRIPTIVE NAME: Multimedia Device Driver Exerciser
*
*
* STATUS: OS/2 Release 2.00
*
* FUNCTION: This module contains Spi calls which will call the Audio SH
*
* NOTES:
*    DEPENDENCIES: NONE
*    RESTRICTIONS: NONE
*
* ENTRY POINTS:
*
* EXTERNAL REFERENCES:
*
*/
/*
** Set up defines to include the necessary definitions from the OS/2
** include files.
*/
#define INCL_BASE
#define INCL_DOS
#define INCL_PM
#define INCL_MMIO

/* C includes */

#include <os2.h>                         /* OS/2 2.x System include         */
#include <stdlib.h>                      /* C Standard library include      */
#include <stdio.h>                       /* C Standard IO include           */
#include <string.h>                      /* C String Functions include      */
#include <os2me.h>                       /* Multimedia System include       */
#include <audio.h>                       /* Multimedia Audio Driver include */
#include "pmadde.h"                      /* Component Private Definitions   */
#include "pmadspi.h"                     /* Component Private Definitions   */
#include "pmadext.h"                     /* Component external Definitions  */

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  EventRtn
*
* ARGUMENTS: PEVCB pevcb
*
* RETURN:    None
*
* DESCRIPTION:  Master Stream Event Routine.
*
*     This routine receives asynchronous SSM event notifications when
*     the event is received, the event semaphore is posted which will
*     wake up the MCD thread(s) blocked on this semaphore.
*
* OS/2 API's USED: DosPostEventSem
*
* C FUNCTION CALLS: Strcpy
*
* INTERNAL FUNCTION REFERENCES: PrintEventInfo
*
* GLOBAL VARIABLES REFERENCED: aStreamData, PEVCB
*
* GLOBAL VARIABLES MODIFIED  :
*
*****************************END OF SPECIFICATIONS ***************************/
RC EventRtn(PEVCB pevcb)
{
    RC        rc ;                       /* Return Code        */
    USHORT    i ;                        /* Loop Index         */
    USHORT    usIndex ;                  /* Stream Data Index  */
    HSTREAM   hstream ;                  /* Stream Handle      */
    HID       hid;                       /* Handler ID         */
    ULONG     ulStatus ;                 /* Status             */
    ULONG     ulType ;                   /* Event Type         */
    ULONG     ulSubType ;                /* Event Subtype      */
    BOOL      fFoundStream ;             /* Found Stream Flag  */
    HAB       habStartThread ;           /* HAB for this thread*/
    HMQ       hmqStartThread ;           /* HMQ for this thread*/

    if (pevcb != (PEVCB)NULL)
    {
        hstream = pevcb->hstream;
        hid = pevcb->ulEventParm1;
        ulStatus = pevcb->ulStatus;
        ulType = pevcb->ulType;
        ulSubType = pevcb->ulSubType;
    }
    else
       return (0);

   /* Intialize PM and obtain anchor block for current thread */

    habStartThread = WinInitialize((USHORT) NULL);

  /* Create queue to receive messsages  on this thread */

   hmqStartThread = WinCreateMsgQueue(habStartThread,0);

  /* Locate the stream in our Stream Data Structure */

    usIndex = 0 ;
    fFoundStream = FALSE ;
    for (i = 0 ; i < StreamCount; i++ )
    {
         if (aStreamData[i].hStream == hstream)
         {
            fFoundStream = TRUE;
            usIndex = i;
            break;
         }
    }

    if (!fFoundStream)
    {
       PrintEventInfo("Event Proc Error: Stream Not Found", hstream);
       return (0);
    }

  switch(ulType)
  {
    case EVENT_IMPLICIT_TYPE:
      switch(ulSubType)
      {
        case EVENT_EOS:

           DosPostEventSem (aStreamData[usIndex].hevThrdSem);
           aulMastFlag[usIndex] = 1;
           PrintEventInfo("EVENT_EOS", hstream);
           strcpy(aStreamData[usIndex].nszState, "EOS");
           PrintActiveInfo (hwndActWinFrm);
           break;

        case EVENT_ERROR:

           DosPostEventSem (aStreamData[usIndex].hevThrdSem);
           aulMErrorFlag[usIndex] = ulStatus;
           PrintEventInfo("EVENT_ERROR", hstream);
           strcpy(aStreamData[usIndex].nszState, "ERROR");
           PrintActiveInfo (hwndActWinFrm);
           break;

        case EVENT_STREAM_STOPPED:

           DosPostEventSem (aStreamData[usIndex].hevThrdSem);
           aulMStopFlag[usIndex] = 1;
           PrintEventInfo("EVENT_STREAM_STOPED", hstream);
           strcpy(aStreamData[usIndex].nszState, "STOPPED  ");
           PrintActiveInfo (hwndActWinFrm);
           break;

        case EVENT_SYNC_PREROLLED:
           aulMPrerollFlag[usIndex] = 1;

           rc = SpiStartStream (aStreamData[usIndex].hStream, SPI_START_STREAM);
           break;

             /* Playlist parser hits the MESSAGE COMMAND. */
        case EVENT_PLAYLISTMESSAGE:
          aulMPlayFlag[usIndex] = 1;
          PrintEventInfo("EVENT_PLAYLISTMESSAGE", hstream);
          strcpy(aStreamData[usIndex].nszState, "PLAYLSTMSG");
          PrintActiveInfo (hwndActWinFrm);
          break;

             /* Start occured   */
        case SH0EVENT_STARTSRC:
          if (fDisplayStartEvents)
          {
             PrintEventInfo("SHOEVENT_STARTSRC", hstream);
             strcpy(aStreamData[usIndex].nszState, "STARTSRC");
             PrintActiveInfo (hwndActWinFrm);
          }
          break;

             /* Start occured  */
        case SH0EVENT_STARTTGT:
          if (fDisplayStartEvents)
          {
             PrintEventInfo("SHOEVENT_STARTTGT", hstream);
             strcpy(aStreamData[usIndex].nszState, "STARTTGT");
             PrintActiveInfo (hwndActWinFrm);
          }
          break;

             /* Get full error occured */
        case SH0EVENT_GETFULL_ERR:
          if (fDisplayNotifyEvents)
          {
             PrintEventInfo("SHOEVENT_GETFULL_ERR", hstream);
             strcpy(aStreamData[usIndex].nszState, "GETFULL_ERR");
             PrintActiveInfo (hwndActWinFrm);
          }
          break;

             /* Get empty error occured */
        case SH0EVENT_GETEMPTY_ERR:
          if (fDisplayNotifyEvents)
          {
             PrintEventInfo("SHOEVENT_GETEMPTY_ERR", hstream);
             strcpy(aStreamData[usIndex].nszState, "GETEMPTY_ERR");
             PrintActiveInfo (hwndActWinFrm);

          }
          break;

             /* Get empty request */
        case SSMEVENT_GETEMPTY:
          if (fDisplayNotifyEvents)
          {
             PrintEventInfo("SSMEVENT_GETEMPTY", hstream);
             strcpy(aStreamData[usIndex].nszState, "GETEMPTY");
             PrintActiveInfo (hwndActWinFrm);

          }
          break;

             /* Get full request */
        case SSMEVENT_GETFULL:
          if (fDisplayNotifyEvents)
          {
             PrintEventInfo("SSMEVENT_GETFULL", hstream);
             strcpy(aStreamData[usIndex].nszState, "GETFULL");
             PrintActiveInfo (hwndActWinFrm);

          }
          break;

             /* Returnempty request */
        case SSMEVENT_RETURNEMPTY:
          if (fDisplayNotifyEvents)
          {
             PrintEventInfo("SSMEVENT_RETURNEMPTY", hstream);
             strcpy(aStreamData[usIndex].nszState, "RETURNEMPTY");
             PrintActiveInfo (hwndActWinFrm);
          }
          break;

             /* ReturnFull request */
        case SSMEVENT_RETURNFULL:
          if (fDisplayNotifyEvents)
          {
             PrintEventInfo("SSMEVENT_RETURNFULL", hstream);
             strcpy(aStreamData[usIndex].nszState, "RETURNFULL");
             PrintActiveInfo (hwndActWinFrm);
          }
          break;

             /* Start signal  */
        case SSMEVENT_STARTTGT_SINGL:
          if (fDisplayStartEvents)
          {
             PrintEventInfo("SSMEVENT_STARTTGT_SINGL", hstream);
             strcpy(aStreamData[usIndex].nszState, "STARTTGT_SINGL");
             PrintActiveInfo (hwndActWinFrm);
          }
          break;

             /* SSMEVENT_STARTTGT_SYNC OCCURED */
        case SSMEVENT_STARTTGT_SYNC:
          if (fDisplayStartEvents)
          {
             PrintEventInfo("SSMEVENT_STARTTGT_SYNC", hstream);
             strcpy(aStreamData[usIndex].nszState, "STARTTGT_SYNC");
             PrintActiveInfo (hwndActWinFrm);
          }
          break;

        case SSMEVENT_STARTSRC_WAIT:
          if (fDisplayStartEvents)
          {
             PrintEventInfo("SSMEVENT_STARTSRC_WAIT", hstream);
             strcpy(aStreamData[usIndex].nszState, "STARTSRC_WAIT");
             PrintActiveInfo (hwndActWinFrm);
          }
          break;

        case SSMEVENT_SMH_NOTIFY_ERROR:  /* SMH_NOTIFY error occurred!*/
          if (fDisplayNotifyEvents)
          {
             PrintEventInfo("SSMEVENT_SMH_NOTIFY_ERROR", hstream);
             strcpy(aStreamData[usIndex].nszState, "SMH_NOTIFY_ERROR");
             PrintActiveInfo (hwndActWinFrm);
          }
          break;

        default:
          break;

      }
      break;

   case EVENT_CUE_TIME:
     aulMCueTimeFlag[usIndex] = 1;
     PrintEventInfo("EVENT_CUE_TIME", hstream);
     break;

   case EVENT_CUE_DATA:
     aulMCueDataFlag[usIndex] = 1;
     PrintEventInfo("EVENT_CUE_DATA", hstream);
     break;

   case EVENT_SYNCOVERRUN:
     aulMOverrunFlag[usIndex] = 1;
     PrintEventInfo("EVENT_SYNCOVERRUN", hstream);
     break;

   default:
     break;
  }

  /* Destroy the message queue */
  WinDestroyMsgQueue( hmqStartThread );

  /* Terminate and release resources */
  WinTerminate( habStartThread );

  return(0);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  Open_Driver
*
* ARGUMENTS: None
*
* RETURN:    None
*
* INPUT :
*
* DESCRIPTION: Verifies that ACPA PDD has been loaded and INITs Driver
*
* OS/2 API's USED: DosOpen
*
* C FUNCTION CALLS:
*
*****************************END OF SPECIFICATIONS ***************************/
LONG Open_Driver()
{
  LONG   rc;               /* API Return Code */
  ULONG  ulAction;         /* Open Action     */
  ULONG  ulFileSize;       /* File Size       */

       /*
       ** Verify that PDD is installed
       */

   ulFileSize = 0;

   rc = DosOpen( aStreamData[StreamCount].AudioDCB.szDevName,
                 &(aStreamData[StreamCount].hDevHandle),
                 &ulAction,
                 ulFileSize,
                 FILE_NORMAL,
                 FILE_OPEN,
                 OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYNONE |
                 OPEN_FLAGS_FAIL_ON_ERROR | OPEN_FLAGS_NOINHERIT ,
                 0 );

   if (!rc)
      sACPAOpened ++;                       /* set as driver opened */
   return(rc);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  AudioInit()
*
* ARGUMENTS: Index, and Mode
*
* RETURN:
*
* DESCRIPTION: Re-inits the device
*
* OS/2 API's USED: DosDevIOCtl()
*
*****************************END OF SPECIFICATIONS ***************************/
LONG AudioInit(ULONG Index,ULONG ulMode)
{
   ULONG            lParmLengthInOut;         /* Parameter Length */
   ULONG            lDataLengthInOut;         /* Data Length      */
   CHAR             abParmList[128];          /* Parm Size        */
   MCI_AUDIO_INIT   rAudioInit;               /* Init Structure   */
   LONG             rc;                       /* Return Code      */
   CLASS_DATA   *pDeviceData;                 /* Device Data      */

   pDeviceData = (CLASS_DATA *)(aStreamData[Index].pDevDataTable) ;

   if (pDeviceData == NULL)
      mprintf ("Null Device Data\n");

   /* Offset the pointer to the right data type */
   pDeviceData += ulMode;

   memset(&rAudioInit,0,sizeof(rAudioInit));
   switch ( pDeviceData->ulDataType)
   {
     case DATATYPE_WAVEFORM:
       rAudioInit.sMode = PCM ;
       break;

     case DATATYPE_ADPCM_AVC:
       rAudioInit.sMode = ADPCM ;
       break;

     case DATATYPE_MULAW:
       rAudioInit.sMode = MU_LAW ;
       break;

     case DATATYPE_ALAW:
       rAudioInit.sMode = A_LAW ;
       break;

     case DATATYPE_MIDI:
       rAudioInit.sMode = MIDI ;
       break;

     case DATATYPE_SPV2:
       rAudioInit.sMode = SPV2 ;
       break;

     case DATATYPE_CDXA_AUDIO:
       rAudioInit.sMode = ADPCMXA ;
       break;

     case DATATYPE_CT_ADPCM:
       rAudioInit.sMode = DATATYPE_CT_ADPCM ;
       break;
   }

   /*
   ** Set up the Audio Init Data Structure to reflect the
   ** Instance attributes. Note that the Device data
   ** pointer will already be offset to point at current
   ** device info
   */
   rAudioInit.lSRate = pDeviceData->sSRate;               /* Sampling Freq */
   rAudioInit.lBitsPerSRate = pDeviceData->sBitsPerSRate; /* BitsPerSample */
   rAudioInit.lBsize = 0;                                 /* Block Size    */
   rAudioInit.ulOperation = aStreamData[Index].ulOperation;/* Play or Recd */
   rAudioInit.sChannels =  pDeviceData->usChannels;        /* mono,stereo  */
   rAudioInit.ulFlags = FIXED | VOLUME | BALANCE |         /* Flags        */
                        TREBLE | BASS;
   rAudioInit.sDeviceID = (SHORT)aStreamData[Index].ulDeviceID ;/* ID      */
   rAudioInit.sReturnCode = 0;
   rAudioInit.sSlotNumber = 0;
   rAudioInit.pvReserved= NULL;                            /* SysFileNum    */
   rAudioInit.ulVersionLevel = CURRENT_VERSION ;           /* Revision      */


   if ( rAudioInit.sMode == PCM && rAudioInit.lBitsPerSRate == 16 )
        rAudioInit.ulFlags |= TWOS_COMPLEMENT;
   else
        rAudioInit.ulFlags &= ~TWOS_COMPLEMENT;

   lParmLengthInOut = 0L;
   lDataLengthInOut = (ULONG) sizeof(MCI_AUDIO_INIT);

   rc = DosDevIOCtl( aStreamData[Index].hDevHandle,
                     AUDIO_IOCTL_CAT,
                     AUDIO_INIT,
                     abParmList,
                     128L,
                     &lParmLengthInOut,
                     &rAudioInit,
                     (LONG) sizeof(MCI_AUDIO_INIT),
                     &lDataLengthInOut );

/*
**  Load DSP code if needed
*/

    ulSysFileNum = (ULONG)rAudioInit.pvReserved;

    aStreamData[Index].AudioDCB.ulSysFileNum = ulSysFileNum;

    if (!rc && (rAudioInit.ulFlags & LOAD_CODE))
        rc = LoadDSP(aStreamData[Index].hDevHandle, &rAudioInit);

    if (!rc)
        rc = AudioChange(aStreamData[Index].hDevHandle);

    return(rc);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  AudioStatus
*
* ARGUMENTS: ULONG Index
*
* RETURN:
*
* DESCRIPTION: Reports Audio Device status
*
* OS/2 API's USED: DosDevIOCtl
*
*****************************END OF SPECIFICATIONS ***************************/
LONG AudioStatus (ULONG Index)
{
   ULONG       lParmLengthInOut;      /* Parameter Length */
   ULONG       lDataLengthInOut;      /* Data Length      */
   CHAR        abParmList[128];       /* Parm Size        */
   LONG        rc = 0;                /* Return Code      */
   MCI_AUDIO_STATUS AudioStatus;

   memset(&AudioStatus,0,sizeof(AudioStatus));

   lParmLengthInOut = 0L;
   lDataLengthInOut = (ULONG) sizeof(MCI_AUDIO_STATUS);

   rc = DosDevIOCtl( aStreamData[Index].hDevHandle,
                     AUDIO_IOCTL_CAT,
                     AUDIO_STATUS,
                     abParmList,
                     128L,
                     &lParmLengthInOut,
                     &AudioStatus,
                     (LONG) sizeof(MCI_AUDIO_STATUS),
                     &lDataLengthInOut );

  rAudioStatus = AudioStatus ;
  rAudioStatus.ulOperation =aStreamData[Index].ulOperation;

   return(rc);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   AudioChange()
*
* ARGUMENTS:  HFILE hFile
*
* RETURN:
*
* DESCRIPTION: Setups the device
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:   NONE
*
*
*     AudioControl           AudioChange               TrackInfo              MidiInfo OR PCMInfo
*   Ŀ   > Ŀ     > Ŀ   > Ŀ
*   IOCTLRequest         pDevInfo16        MasterVol            Tempo        
*   pRequestInfo16    ...                     DitherPerc           CPQN         
*   Position              ...                     Reserved[3]          MidiSwitches 
*   RC                    pModeInfo16  Ŀ             
*             
*
*
*****************************END OF SPECIFICATIONS ***************************/
LONG AudioChange(HFILE hFile)
{
   ULONG                 lDataLengthInOut;      /* Data length       */
   MCI_AUDIO_CONTROL     rAudioControl;         /* Control Struct    */
   MCI_AUDIO_CHANGE      rAudioChange;          /* Change Struct     */
   LPMCI_AUDIO_CHANGE    prAudioChange32;       /* Change struct ptr */
   LPMCI_AUDIO_CHANGE    prAudioChange16;       /* Change struct ptr */
   MCI_TRACK_INFO        rTrackInfo;            /* Track info struct */
   LPMCI_TRACK_INFO      prTrackInfo16;         /* Track info ptr    */
   LPMCI_TRACK_INFO      prTrackInfo32;         /* Track info ptr    */
   LONG                  rc;                    /* API return Code   */

   rAudioChange.lVolume = 0x7fffffff;
   rAudioChange.lVolumeDelay = 0;
   rAudioChange.lBalance = 0x3fffffff;
   rAudioChange.lBalanceDelay = 0;
   rAudioChange.lTreble = 0x3fffffff;
   rAudioChange.lBass = 0x3fffffff;
   rAudioChange.lPitch = 0;
   rAudioChange.lMonitor = MONITOR_UNCOMPRESSED;

   Set_IO_Ports(&rAudioChange);
   lDataLengthInOut = (ULONG) sizeof(MCI_AUDIO_CONTROL);

   rAudioControl.usIOCtlRequest = AUDIO_CHANGE;
   rAudioControl.ulPosition = 0;
   rAudioControl.sReturnCode = 0;

   rTrackInfo.usDitherPct = 0;
   rTrackInfo.usMasterVolume = ( USHORT ) 0x7fff;

        /* Convert pointers to 16 bit*/

   prAudioChange32 = &rAudioChange;
   prAudioChange16 = ( PVOID ) MAKEULONG( LOUSHORT(prAudioChange32),
                               (HIUSHORT(prAudioChange32) << 3) + 7);
   rAudioControl.pbRequestInfo = prAudioChange16;

   prTrackInfo32 = &rTrackInfo;
   prTrackInfo16 = ( PVOID ) MAKEULONG ( LOUSHORT(prTrackInfo32),
                                        (HIUSHORT(prTrackInfo32) << 3) + 7);
   rAudioChange.pvDevInfo = prTrackInfo16;

/*
** This was used for Microsoft complier, but not C Set/2
** prAudioChange16 = MAKEP((HIUSHORT(prAudioChange32) << 3) + 7,
**                          LOUSHORT(prAudioChange32));
*/
     rc = DosDevIOCtl( hFile,
                       AUDIO_IOCTL_CAT,
                       AUDIO_CONTROL,
                       0,
                       0L,
                       0L,
                       &rAudioControl,
                       (LONG) sizeof(MCI_AUDIO_CONTROL),
                       &lDataLengthInOut );
    return(rc);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  Set_IO_Ports
*
* ARGUMENTS: LPMCI_AUDIO_CHANGE    prAudioChange
*
* RETURN:    None
*
* DESCRIPTION:
*
*
*****************************END OF SPECIFICATIONS ***************************/
LONG Set_IO_Ports(LPMCI_AUDIO_CHANGE prAudioChange)
{
   USHORT i;     /* Loop Index  */

   prAudioChange->lInput = INPUTS_LISTED;
   prAudioChange->lOutput = OUTPUTS_LISTED;

   prAudioChange->rInputList[0].ulDevType = STEREO_LINE_INPUT;
   prAudioChange->rInputList[0].ulDevNum = DEVICE_1;

   prAudioChange->rOutputList[0].ulDevType = SPEAKER_OUTPUT;
   prAudioChange->rOutputList[0].ulDevNum = DEVICE_1;

   for (i = 1;i < 8;i++)
   {
      prAudioChange->rInputList[i].ulDevType = NULL_INPUT;
      prAudioChange->rInputList[i].ulDevNum = NULL_INPUT;

      prAudioChange->rOutputList[i].ulDevType = NULL_OUTPUT;
      prAudioChange->rOutputList[i].ulDevNum = NULL_OUTPUT;
   }

   return(0L);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  LoadDSP()
*
* ARGUMENTS: HFILE hFile, LPMCI_AUDIO_INIT prAudioInit
*
* RETURN:
*
* DESCRIPTION: Loads DSP module
*
* OS/2 API's USED: DosOpen, DosClose, DosQueryFileInfo, DosAllocSharedMem
*                  DosDevIOCtl, DosFreeMem, DosRead, DosQueryCurrentDisk
*
* C FUNCTION CALLS: Strcpy, Strcat
*
*****************************END OF SPECIFICATIONS ***************************/
LONG LoadDSP(HFILE hFile, LPMCI_AUDIO_INIT prAudioInit)
{
   LONG         rc;                       /* API return Code   */
   LONG         lError = MCIERR_SUCCESS;  /* Local Error Flag  */
   HFILE        hDSPFile;                 /* File Handle       */
   ULONG        lParmLengthInOut;         /* Paramter length   */
   ULONG        lDataLengthInOut;         /* Data Length       */
   ULONG        ulAction;                 /* Action Taken      */
   ULONG        ulDSPFileSize;            /* DSP File Size     */
   ULONG        ulBytesRead;              /* Bytes Read        */
   ULONG        ulCurrentDrive;           /* Current Drive     */
   ULONG        ulDriveMap;               /* Drive Map         */
   USHORT       i;                        /* Index             */
   CHAR         bDriveNum;                /* Drive Number      */
   USHORT       usDSPFound = MCI_FALSE;   /* Local Flag        */
   SHORT        sDSPFileOpen;             /* Local Flag        */
   CHAR         szDSPFile[261];           /* DSP File Name     */
   FILESTATUS   rFileInfo;                /* File Info         */
   ULONG        *pulDSPBase32;            /* Pointer           */
   ULONG        *pulDSPBase16;            /* Pointer           */
   CHAR         abParmList[128];          /* Paramter Packet   */
   MCI_AUDIO_LOAD rAudioLoad;             /* Audio Load Struct */

       /* Get drive map */

   rc = DosQueryCurrentDisk(&ulCurrentDrive, &ulDriveMap);

       /* If DosQueryCurrentDisk() fails, assume drive C: is only drive*/

   if (rc)
     ulDriveMap = 0x0004L;

   /* Find and open DSP file*/
    bDriveNum = 2;  /* Start with drive C:*/

   while (bDriveNum < 26 && !usDSPFound)
   {
     if (!(ulDriveMap & (0x01 << bDriveNum)))
     {
        bDriveNum++;
        continue;
     }
     for (i = 0;i < VSD_NUM_DSP_PATHS;i++)
     {
       strcpy(szDSPFile,"A:");
       szDSPFile[0] = (CHAR) ('A' + bDriveNum);
       strcat(szDSPFile,abDSPPath[i]);
       strcat(szDSPFile,prAudioInit->abLoadPath);
       rc = DosOpen( szDSPFile,
                     &hDSPFile,
                     &ulAction,
                     0L,
                     FILE_NORMAL,
                     FILE_OPEN,
                     OPEN_ACCESS_READONLY | OPEN_SHARE_DENYNONE,
                     0 );
      if (!rc)
      {
        usDSPFound = MCI_TRUE;
        break;
      }
    }

    bDriveNum++;
   }

   if (!usDSPFound)
     lError = MCIERR_CANNOT_LOAD_DSP_MOD;

   if (lError)
   {
       DosClose(hFile);
       sDSPFileOpen = MCI_FALSE;
   }
   else
       sDSPFileOpen = MCI_TRUE;

       /* Get DSP File Size*/
   if (!lError)
   {
     rc = DosQueryFileInfo( hDSPFile,
                            1L,
                            &rFileInfo,
                            sizeof(FILESTATUS) );

     lError = rc;

     if (lError)
        DosClose(hFile);

     ulDSPFileSize = rFileInfo.cbFile;
   }

       /* Allocate Buffer to Store DSP Code*/

   if (!lError && ulDSPFileSize > 0)
   {
     rc = DosAllocSharedMem((PVOID)&pulDSPBase32,
                             NULL,
                             ulDSPFileSize,
                             PAG_COMMIT | OBJ_GIVEABLE |
                             PAG_READ | PAG_WRITE);

     lError = rc;

     if (lError)
         DosClose(hFile);
   }

        /* Read DSP File into Buffer*/

   if (!lError && ulDSPFileSize > 0)
   {
      rc = DosRead(hDSPFile,
                   pulDSPBase32,
                   ulDSPFileSize,
                   &ulBytesRead);

      lError = rc;
      if (lError)
          DosClose(hFile);
   }

        /* Convert 32 bit address to 16 bit address*/

/*  pulDSPBase16 = MAKEP((HIUSHORT(pulDSPBase32) << 3) + 7,
                        LOUSHORT(pulDSPBase32));  */

    pulDSPBase16 = ( PULONG ) MAKEULONG( LOUSHORT( pulDSPBase32  ),
                               (HIUSHORT( pulDSPBase32 ) << 3) + 7);

        /* Call AUDIO_LOAD IOCtl*/

  if (!lError && ulDSPFileSize > 0)
  {
    rAudioLoad.pbBuffer = (CHAR FAR *) pulDSPBase16;
    rAudioLoad.ulSize = (USHORT) ulBytesRead;
    rAudioLoad.ulFlags = LOAD_START | LOAD_END;

    lParmLengthInOut = 0L;
    lDataLengthInOut = (LONG) sizeof(MCI_AUDIO_LOAD);

    rc = DosDevIOCtl(hFile,
                     AUDIO_IOCTL_CAT,
                     AUDIO_LOAD,
                     abParmList,
                     128L,
                     &lParmLengthInOut,
                     &rAudioLoad,
                     (LONG) sizeof(MCI_AUDIO_LOAD),
                     &lDataLengthInOut);

   lError = rc;
   if (lError)
       DosClose(hFile);
   DosFreeMem(pulDSPBase32);
  }
        /* Close DSP File*/
  if (sDSPFileOpen)
     DosClose(hDSPFile);

  return(lError);

}
/************************ START OF SPECIFICATIONS **************************
*
* SUBROUTINE NAME: Close_Driver
*
* FUNCTION: Closes Audio device
*
* INPUT:  lParam1       - pointer to AMP_INSTANCE structure
*
* OUTPUT: returns MCIERR_SUCCESS if successful, otherwise it returns an
*         MCI error code.
*                                                                                                                                                  *
*************************** END OF SPECIFICATIONS *************************/

LONG Close_Driver(HFILE hFile)
{
   LONG rc;         /* API Return Code */

   sACPAOpened = 0;
   rc = DosClose(hFile);
   return(rc);
}
/****************************************************************************
*
* FUNCTION: GetStreamHandler()
*
* ARGUMENTS: None
*
* RETURN: RC
*
* DESCRIPTION: This call gets a Stream Handler for Source & Target operations
*              The file system stream handler is usually be the Source.
*              The audio stream handler is usually be the target.
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:  SpiGetHandler
*
* GLOBAL VARIABLES REFERENCED: HandlerCount
*
* GLOBAL VARIABLES MODIFIED: HandlerCount
*
*****************************END OF SPECIFICATIONS ***************************/
RC GetStreamHandler()
{
  int   i;        /* Loop Index   */
  int res;        /* Result       */

  for (i=0;i<HandlerCount;i++)
  {
    if ((res = strcmp (aActiveHandlers[i].nszHandlerName,
        aActiveHandlers[HandlerCount].nszHandlerName)) == 0)
    return (0);
  }
  rc = SpiGetHandler(  aActiveHandlers[HandlerCount].nszHandlerName,
                       &aActiveHandlers[HandlerCount].hidSrc,
                       &aActiveHandlers[HandlerCount].hidTgt);

  if (!rc)
      HandlerCount++;   /* Increment Handler Count */

  return(rc);

}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: GetProtocol()
*
* ARGUMENTS: NOne
*
* RETURN:    None
*
* DESCRIPTION: This function allows to Get stream protocol in the
*              stream handler.  It also generates an internal Stream
*              Protocol Control Block (SPCB) key, which is used to
*              differentiate between multiple SPCBs of the same data
*              stream type.
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES: SpiGetProtocol
*
*****************************END OF SPECIFICATIONS ***************************/
RC GetProtocol(ULONG TargetId, PSPCBKEY TempSPCBKey, PSPCB TempSPCB)
{
RC   rc ; /* API Return Code */

    rc = SpiGetProtocol (TargetId, TempSPCBKey, TempSPCB);
    return (rc);

}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: InstallProtocol()
*
* ARGUMENTS: NOne
*
* RETURN:    None
*
* DESCRIPTION:
*              It generates an internal Stream
*              Protocol Control Block (SPCB) key, which is used to
*              differentiate between multiple SPCBs of the same data
*              stream type.
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:  SpiInstallProtocol
*
*****************************END OF SPECIFICATIONS ***************************/

RC InstallProtocol(ULONG TargetId, PSPCBKEY TempSPCBKey, PSPCB TempSPCB)
{
  RC    rc;  /* API Return Code */

  rc = SpiInstallProtocol (TargetId,
                           TempSPCBKey,
                           TempSPCB,
                           SPI_INSTALL_PROTOCOL);
  return(rc);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: CreateStream()
*
* ARGUMENTS:
*
* RETURN:
*
* DESCRIPTION: This Function creates a data stream.
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES: Open_Driver()
*                               AudioInit ()
*
* GLOBAL VARIABLES REFERENCED: StreamCount
*
* GLOBAL VARIABLES MODIFIED: StreamCount
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/

RC CreateStream()
{

    USHORT   Index;          /* Loop Index   */
    SPCBKEY  spcbkey;        /* SPCB Key     */
    CLASS_DATA *pClassData;  /* Device Data  */

    if (aStreamData[StreamCount].pDevDataTable == NULL)
        return (100);

    pClassData = (CLASS_DATA *)(aStreamData[StreamCount].pDevDataTable);

    memset (&spcbkey,0,sizeof(SPCBKEY));

    /*
    ** Move the device data table pointer to the location
    ** where we can pick up the current data type info.
    ** The offset of the pointer is given by mode
    */
    pClassData += aStreamData[StreamCount].ulmode;

    /* Set up SPCB */

    spcbkey.ulDataType =  pClassData->ulDataType ;
    spcbkey.ulDataSubType = pClassData->ulSubType ;

    /* Set up the SPCB pointer */

    aStreamData[StreamCount].pSpcbKey = (PSPCBKEY)&spcbkey;

    /* Set up the Event routine pointer */

    aStreamData[StreamCount].pEventBlock = (PIMPL_EVCB)&Events;
    /*
    ** Determine if this will be a playback or a record
    ** stream. Assuming this is an Audio stream we can
    ** determine this based on handler ID.
    */

    Index = 0;
    while (stricmp(aActiveHandlers[Index].nszHandlerName, "AUDIOSH$"))
            Index++;
    if (aActiveHandlers[Index].hidSrc == aStreamData[StreamCount].hidSrc)
            aStreamData[StreamCount].ulOperation = OPERATION_RECORD;
    else
            aStreamData[StreamCount].ulOperation = OPERATION_PLAY;

    /* Open and Init the Audio Card               */

    rc = Open_Driver();

    AudioInit(StreamCount,aStreamData[StreamCount].ulmode);

    /* Setup Audio Device control Block */

    aStreamData[StreamCount].AudioDCB.ulDCBLen = sizeof(DCB_AUDIOSH);

    rc = SpiCreateStream (aStreamData[StreamCount].hidSrc,            /* Src */
                          aStreamData[StreamCount].hidTgt,            /* Tgt */
                          aStreamData[StreamCount].pSpcbKey,          /* SPCB*/
                          (PDCB)&aStreamData[StreamCount].AudioDCB,   /* DCB */               // 08/18
                          (PDCB)&aStreamData[StreamCount].AudioDCB,   /* DCB */               // 08/18
                          aStreamData[StreamCount].pEventBlock,       /*EVCB */
                          (PEVFN)EventRtn,                            /*PFN  */
                          (ULONG)0,                                   /* ??  */
                          &(aStreamData[StreamCount].hStream),        /*hstr */
                          &(aStreamData[StreamCount].EventHandle));   /*evnt */
    if (!rc)
    {
      strcpy(aStreamData[StreamCount].nszState, "CREATED  ");

      /*
      ** Create Event semaphore to control any play
      ** threads we may spawn.
      */

     rc = DosCreateEventSem ((ULONG)NULL,
                             (PHEV)&(aStreamData[StreamCount].hevThrdSem),
                             DC_SEM_SHARED,
                             FALSE);
    if (rc)
    {
       /* Destroy the stream created and return */
       DestroyStream (StreamCount);
       return (rc);

    }  /* Semaphore created */

    StreamCount++; /* Increment Stream count */
 } /* Good Stream Create */
 return(rc);

}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: AssociateStream()
*
* ARGUMENTS:
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
RC AssociateStream()
{
    PCHAR   pstr;                    /* Temporary String     */
    UCHAR   szTemp[FILENAME_SIZE];   /* Temporary File Name  */
    APIRET  rc;                      /* Return Code          */

    memset (&FileInfoStr,0,sizeof(FileInfoStr));

    strcpy (szTemp,aStreamData[ulIndex].nszFileName);

    /*
    ** Find out if the file exists. This is done using DosQPthInfo.
    ** If We get an return code from the API that would almost certainly
    ** mean that file does not exist. We Use the MMIO interface to create
    ** the file for us. This implies a recording stream going from Audio
    ** Stream handler as Source to File System stream handler as target.
    ** If No return code, then we know that file exists and we just open
    ** it in a shareable mode.
    */

    rc = DosQueryPathInfo (szTemp,
                           0x0001,
                           &FileInfoStr,
                           sizeof(FileInfoStr));
    if (rc != 0)
    {
      aStreamData[ulIndex].hmmio = mmioOpen (aStreamData[ulIndex].nszFileName,
                                             0,
                                             MMIO_READWRITE|
                                             MMIO_CREATE|
                                             MMIO_DENYNONE );

      if (aStreamData[ulIndex].hmmio == (ULONG)NULL)
         return(5000);
    }
    else
    {
      aStreamData[ulIndex].hmmio = mmioOpen (aStreamData[ulIndex].nszFileName,
                                             0,
                                             MMIO_READWRITE|
                                             MMIO_DENYNONE);
      if (aStreamData[ulIndex].hmmio == (ULONG)NULL)
         return(5001);
    }

    /* Fill up the ACB for this stream */

    aStreamData[ulIndex].AssCtlBlk.ulObjType = ACBTYPE_MMIO;
    aStreamData[ulIndex].AssCtlBlk.ulParm1 = aStreamData[ulIndex].hmmio;
    aStreamData[ulIndex].AssCtlBlk.ulACBLen = sizeof(ACB_MMIO);

    if (aStreamData[ulIndex].ulOperation == OPERATION_PLAY)

        rc = SpiAssociate(aStreamData[ulIndex].hStream,
                          aStreamData[ulIndex].hidSrc,
                          &(aStreamData[ulIndex].AssCtlBlk));

    if (aStreamData[ulIndex].ulOperation == OPERATION_RECORD)

        rc = SpiAssociate(aStreamData[ulIndex].hStream,
                          aStreamData[ulIndex].hidTgt,
                          &(aStreamData[ulIndex].AssCtlBlk));
    if (!rc)
        strcpy(aStreamData[ulIndex].nszState, "ASSOCIATED");

    return(rc);

}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: StartStream()
*
* ARGUMENTS:
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
VOID _Optlink StartStream()
{
    ULONG   ulCount = 0;

    /* Re-init ACPA card */

    rc = AudioInit(ulIndex,aStreamData[ulIndex].ulmode);
    aStreamData[ulIndex].RetCode = SpiStartStream (aStreamData[ulIndex].hStream,
                                               aStreamData[ulIndex].ulStartFlag );

     /*
     ** Signal our creator that we started the stream, and that
     ** the return code is in the instance structure
     */

     DosPostEventSem (aStreamData[ulIndex].hevThrdSem);

     /*
     ** Lets Block ourselves for an infinite period.
     ** We get awakened asynchronously on events.
     */
     /*
     if (!aStreamData[ulIndex].RetCode)
     {
       aStreamData[ulIndex].ulCntrlField |= PLAY ;
       DosResetEventSem (aStreamData[ulIndex].hevThrdSem,&ulCount);
       DosWaitEventSem (aStreamData[ulIndex].hevThrdSem, (ULONG)-1);
     }
       */
    /* Change the control field state to not playing */

    aStreamData[ulIndex].ulCntrlField &= ~PLAY ;

}
/*************************************************************************
*
* FUNCTION: StopStream()
*
* ARGUMENTS:
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/

RC StopStream()
{
    RC rc ;   /* Return Code */

    rc = SpiStopStream (aStreamData[ulIndex].hStream,
                        aStreamData[ulIndex].ulStopFlag);
    if (!rc)
        strcpy(aStreamData[ulIndex].nszState, "STOPPED  ");

    return(rc);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: DestroyStream()
*
* ARGUMENTS:
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED: StreamCount
*
* GLOBAL VARIABLES MODIFIED: StreamCount
*
*****************************END OF SPECIFICATIONS ***************************/
RC DestroyStream()
{
    rc = SpiDestroyStream (aStreamData[ulIndex].hStream);
    if (!rc)
    {
       memset (&aStreamData[ulIndex],0,sizeof(STREAMDATA));
       aStreamData[ulIndex].hStream = -1;

       /* Leave the defaults on */

       strcpy (aStreamData[ulIndex].AudioDCB.szDevName,
               PhysicalDevData[0].szPhysicalDevName);

       aStreamData[ulIndex].ulResID =  PhysicalDevData[0].ulResourceId ;
       StreamCount--;
    }
    return(rc);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: GetStreamTime()
*
* ARGUMENTS:
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
RC GetStreamTime()
{
    rc = SpiGetTime (aStreamData[ulIndex].hStream, &mmtimeCurrTime);
    return(rc);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: SeekStream()
*
* ARGUMENTS:
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
RC SeekStream()
{
    rc = SpiSeekStream ( aStreamData[ulIndex].hStream,
                         aStreamData[ulIndex].ulSeekFlag,
                         aStreamData[ulIndex].ulSeekPoint  );


    if (!rc)
        strcpy(aStreamData[ulIndex].nszState,
               LongToString(aStreamData[ulIndex].ulSeekPoint));

    return(rc);
}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: QuitProg()
*
* ARGUMENTS:
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED:
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES:
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
* NOTES:
*
* SIDE EFFECTS:
*
*****************************END OF SPECIFICATIONS ***************************/
RC QuitProg()
{
    /* Destroy all streams before exiting and closing ACPA*/

    for (ulIndex=0; ulIndex < MAXSTREAMS; ulIndex++)
    {
         if (aStreamData[ulIndex].hStream != -1)
              SpiDestroyStream (aStreamData[ulIndex].hStream);

         if (aStreamData[ulIndex].hDevHandle != 0)
             rc = Close_Driver(aStreamData[ulIndex].hDevHandle);
         if (rc)
         {
             return(rc);
         } /* Error Code */
    } /* For loop */
} /* QuitProg */
