/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
/*********************** START OF SPECIFICATIONS *******************************
*
* SOURCE FILE NAME: PMADSUBS.C
*
* DESCRIPTION:PMADDE helper routines.
*
* FUNCTION: Exercise Multimedia Audio Stream Handler.
*
*
* NOTES:
*
*
* EXTERNAL ENTRY POINTS: None.
*
*
* INTERNAL ENTRY POINTS: None.
*
*
* EXTERNAL REFERENCES (system):
*
*
*
*/
#define INCL_WIN                        /* OS/2 2.x Win include            */
#define INCL_PM                         /* Presentation Manager include    */
#define INCL_DOS                        /* OS/2 2.x DOS API include        */
#define INCL_DOSSEMAPHORES              /* OS/2 2.x Semaphores             */
#include <os2.h>                        /* OS/2 2.x System include         */
#include <stdio.h>                      /* C Standard IO include           */
#include <stdlib.h>                     /* C Standard library include      */
#include <string.h>                     /* C String Functions include      */
#include <stdarg.h>                     /* C Standard Arguments include    */
#include <os2me.h>                      /* Multimedia System include       */
#include <audio.h>                      /* Multimedia Audio Driver include */
#include "pmadde.h"                     /* Component Private Definitions   */
#include "pmadext.h"                    /* Component external Definitions  */

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  PrintActiveInfo
*
* ARGUMENTS: HWND hwnd
*
* RETURN:    None
*
* DESCRIPTION: This routine prints active stream data.
*
* OS/2 API's USED: None
*
* C FUNCTION CALLS: Strcat, Strncat.
*
*****************************END OF SPECIFICATIONS ***************************/
VOID PrintActiveInfo (HWND hwnd)
{
   PSZ   pszTemp;                /* Temporary Storage  */
   int   i;                      /* Loop Index         */
   char  ActWinMLEbuffer[2048];  /* MLE Buffer         */

  /*
  ** Active Streams Information may be updated by different threads
  ** depending on asynchronous events. To serialize acces We use
  ** The Print Semaphore.
  */


   DosRequestMutexSem (hmtxPrntSem, -1);

   /* Turn off refresh so that we dont see flickers */

   DisableMLERefresh(hwndActMLE);

   /* Delete the current text in the MLE buffers */

   DeleteMLE (hwndActMLE,ActWinMLEbuffer);

   ipt = 0;

   /* Initialize MLE for the current transaction */

   InitMLE (hwndActMLE,ActWinMLEbuffer);

   /* Set font sizes and associated metrics */

   CreateFontFixed(hwnd);

     /*
     **  We loop through our internal stream data structure till current
     **  stream count. We build the buffer to be imported into the MLE
     **  by a series of string concatenations. At the end of each iteration
     **  the MLE buffer is imported. The Offsets are magic numbers that
     **  ensure string allignments at all resolutions.
     */

   for (i=0;i < StreamCount ;i++)
   {

     /* Initialize the buffer to 0 */
     memset (ActWinMLEbuffer,0,sizeof(ActWinMLEbuffer));

     strcat(ActWinMLEbuffer,aStreamData[i].AudioDCB.szDevName);
     if (aStreamData[i].hStream != -1)
     {
         pszTemp = LongToString (aStreamData[i].hStream);
         strncat (ActWinMLEbuffer,hspace,11-strlen(ActWinMLEbuffer));
         strcat(ActWinMLEbuffer,pszTemp);

         strncat (ActWinMLEbuffer,hspace,19-strlen(ActWinMLEbuffer));
         strcat(ActWinMLEbuffer,aStreamData[i].nszState);

         strncat (ActWinMLEbuffer,hspace,33-strlen(ActWinMLEbuffer));
         pszTemp = LongToString (aStreamData[i].hidSrc);
         strcat (ActWinMLEbuffer,pszTemp);

         strncat (ActWinMLEbuffer,hspace,40-strlen(ActWinMLEbuffer));
         pszTemp = LongToString (aStreamData[i].hidTgt);
         strcat (ActWinMLEbuffer,pszTemp);

         strncat (ActWinMLEbuffer,hspace,46-strlen(ActWinMLEbuffer));
         strcat(ActWinMLEbuffer,aStreamData[i].nszFileName);

         /* Add a new line */

         strcat (ActWinMLEbuffer,"\n");

         /* Get the text into the MLE buffer */

         ImportMLE(hwndActMLE,ipt,  ActWinMLEbuffer);
     } /* Valid Hstream */
   } /* Iteration Complete */
   EnableMLERefresh (hwndActMLE);
   ipt = 0;

   /*
   ** Give up the Print Semaphore
   */

   DosReleaseMutexSem (hmtxPrntSem);
}

/*****************************START OF SPECIFICATIONS ***************************
*
* FUNCTION : PrintCmndInfo
*
* ARGUMENTS: USHORT usCmnd,  ULONG ulStreamNum,  RC rc.
*
* RETURN:    None
*
* DESCRIPTION: This routine prints the status of the command, and return code.
*
* OS/2 API's USED: None
*
* C FUNCTION CALLS: Strcat, Strncat.
*
*****************************END OF SPECIFICATIONS ***************************/
VOID PrintCmndInfo (USHORT usCmnd,ULONG ulStreamNum, RC rc)
{
   int  i;                     /* Loop Index         */
   HPS  hps;                   /* Presentation Space */
   PSZ  pszTemp;               /* Temporary Store    */
   SZ   ComWinMLEbuffer[126];  /* MLE buffer         */


   DisableMLERefresh(hwndComMLE);

   memset (ComWinMLEbuffer,0,sizeof(ComWinMLEbuffer));

   switch (usCmnd)
   {
      case GETHANDLER:
        strcat (ComWinMLEbuffer,"SpiGetHandler   ");
        break;

      case CREATE:
        strcpy (ComWinMLEbuffer,"SpiCreateStream ");
        break;

     case ASSOCIATE:
       strcpy (ComWinMLEbuffer,"SpiAssociate");
       break;

     case STOP:
       strcpy (ComWinMLEbuffer,"SpiStopStream");
       break;

     case SEEK:
       strcpy (ComWinMLEbuffer,"SpiSeekStream");
       break;

     case PLAY:
       strcpy (ComWinMLEbuffer,"SpiStartStream");
       break;

     case DESTROY:
       strcpy (ComWinMLEbuffer,"SpiDestroyStream");
       break;

     case QRYTIME:
       strcpy (ComWinMLEbuffer,"SpiGetTime");
       break;

     case INSTALL:
       strcpy (ComWinMLEbuffer,"SpiInstallProtocol");
       break;

     case INIT:
       strcpy (ComWinMLEbuffer,"SpiEnumerateHandlers");
       break;

     default:
       strcpy (ComWinMLEbuffer,"Not Done");
       break;

   } /* endswitch */

   pszTemp = LongToString (ulStreamNum);
   strncat (ComWinMLEbuffer,hspace,20-strlen(ComWinMLEbuffer));
   strcat (ComWinMLEbuffer,pszTemp);

   strncat (ComWinMLEbuffer,hspace,30-strlen(ComWinMLEbuffer));

   if (rc)
        strcat (ComWinMLEbuffer,"Error");
   else
        strcat (ComWinMLEbuffer,"Successful");

   pszTemp = LongToString (rc);
   strncat (ComWinMLEbuffer,hspace,43-strlen(ComWinMLEbuffer));
   strcat (ComWinMLEbuffer,pszTemp);

   strcat (ComWinMLEbuffer,"\n");

   /* Import the text into the MLE buffer */

   InitMLE (hwndComMLE,ComWinMLEbuffer);

   /* Import the text into MLE */

   ImportMLE(hwndComMLE,iptComMle,ComWinMLEbuffer);

   /* Turn Refresh back on again */

   EnableMLERefresh (hwndComMLE);

   /* Writ to Log File */

   Write2File (hLogFile,ComWinMLEbuffer);

 }
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  PrintEventInfo - Prints the events occured.
*
* ARGUMENTS: pszMsg, Value
*
* RETURN:    None
*
* DESCRIPTION: This function prints the events occured.
*
* OS/2 API's USED: None
*
* C FUNCTION CALLS: Memset, Strcpy
*
* INTERNAL FUNCTION REFERENCES: Write2File
*
*****************************END OF SPECIFICATIONS ***************************/
VOID PrintEventInfo (PSZ pszMsg, ULONG value)
{
   SZ EveWinMLEbuffer[126];     /* MLE buffer      */

   DisableMLERefresh(hwndEveMLE);
   memset (EveWinMLEbuffer,0,sizeof(EveWinMLEbuffer));

   strcpy (EveWinMLEbuffer,LongToString(value));
   strncat (EveWinMLEbuffer,hspace,7-strlen(EveWinMLEbuffer));
   strcat (EveWinMLEbuffer,pszMsg);
   strcat (EveWinMLEbuffer,"\n");

   InitMLE (hwndEveMLE,EveWinMLEbuffer);

   ImportMLE(hwndEveMLE,iptEveMle,EveWinMLEbuffer);

   EnableMLERefresh(hwndEveMLE);

    /* Write to the log file */
   Write2File (hLogFile,EveWinMLEbuffer);

}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:   AboutDlgProc
*
* ARGUMENTS:  HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2
*
* RETURN: None
*
* DESCRIPTION: This routine brings up the dialog about the Audi Excerciser Tool
*
* OS/2 API's USED: None
*
*
*****************************END OF SPECIFICATIONS ***************************/
MRESULT AboutDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    switch( msg )
    {
      case WM_COMMAND:
           WinDismissDlg( hwnd, TRUE );
          break;

       default:
          return (WinDefDlgProc( hwnd, msg, mp1, mp2 ));
    }
    return (MRESULT)(FALSE);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: ListInsertInstalledHandlers.
*
* ARGUMENTS: hwnd. The handle of Install Installed Handlers Dialog box.
*
* RETURN: None.
*
* DESCRIPTION: Update the Install Protocol Dialogs Possible Datatypes
* list box. After the dialog is created this function is called to
* fill up the list box with possible datatypes.
*
* OS/2 API's USED: WinSendDlgItemMsg().
*
* C FUNCTION CALLS: None.
*
* INTERNAL FUNCTION REFERENCES: InserListItem Macro
*
* GLOBAL VARIABLES REFERENCED: None.
*
* GLOBAL VARIABLES MODIFIED: None.
*
*****************************END OF SPECIFICATIONS ***************************/
VOID ListInsertInstalledHandlers(HWND hwnd, ULONG ulListID)
{
    ULONG i=0;                             /* Loop Index        */
    static char buffer[70];                /* Temporary Storage */
    static char TempSpace[] = "         "; /* Text allignment   */

    for (i=0; i < ulNumHand; i++)
    {
      memset(buffer,'\0',sizeof(buffer));
      strcpy (buffer,aHand[i].szHandlerName);
      strncat(buffer,TempSpace,(8-strlen(aHand[i].szHandlerName)));
      strcat (buffer,TempSpace);
      strcat(buffer,aHand[i].szHandlerClass);
      InsertListItem (ulListID,LIT_SORTASCENDING,buffer);
    } /* Number of installed stream handlers */
} /* Return to caller */

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: ListInsertStreamDataTypes
*
* ARGUMENTS: hwnd. The Handle of Stream Creation Dialog box.
*
* RETURN: None.
*
* DESCRIPTION: Update The Stream creation Dialogs Possible Datatypes
* list box. After the dialog is created this function is called to
* fill up the list box with possible stream datatypes.
*
* OS/2 API's USED: WinSendDlgItemMsg ().
*
* C FUNCTION CALLS: None.
*
* INTERNAL FUNCTION REFERENCES: None.
*
*****************************END OF SPECIFICATIONS ***************************/
PVOID ListInsertStreamDataTypes (HWND hwnd, ULONG LstId, MPARAM ulDataIndex)
{
   char    buffer[60];                /* PSZ Data Type storage */
   PSZ     pszTemp;                   /* Temporary Storage     */
   ULONG   i;                         /* Loop Index            */
   USHORT  j;                         /* Loop Index            */
   APIRET  ReturnCode;                /* API Return Code       */
   SPCBKEY spcbkey;                   /* SPCB Key              */
   USHORT  Index =0 ;                 /* Index                 */
   USHORT  usType = 0;                /* Request Type          */
   BOOL    fFoundDataType = FALSE;    /* Local Flag            */

struct _datatypes DataTypes[] = {
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_8M16,2000,    "PCM 16 bit 2 KHz Mono"                      ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_8M16,8000,    "PCM 16 bit 8 KHz  Mono"                     ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_1M16,11025,   "PCM 16 bit 11 KHz  Mono"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_2M16,22050,   "PCM 16 bit 22 KHz  Mono"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4M16,44100,   "PCM 16 bit 44 KHz  Mono"                    ,
DATATYPE_MIDI       ,  SUBTYPE_NONE,        0,   "MIDI "                                      ,
DATATYPE_ADPCM_AVC  ,  ADPCM_AVC_VOICE, 11025,   "ADPCM AVC Voice. 16 bit 11 KHz  Mono"       ,
DATATYPE_ADPCM_AVC  ,  ADPCM_AVC_MUSIC, 22050,   "ADPCM AVC Music. 16 bit 22 KHz  Mono"       ,
DATATYPE_ADPCM_AVC  ,  ADPCM_AVC_STEREO,22050,   "ADPCM AVC Stereo. 16 bit 22 KHz  Stereo"    ,
DATATYPE_ADPCM_AVC  ,  ADPCM_AVC_HQ,    44100,   "ADPCM AVC High Quality. 16 bit 44 KHz  Mono",
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_8M08,2000,    "PCM 8 bit 2 KHz Mono"                       ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_8M08,8000,    "PCM 8 bit 8 KHz  Mono"                      ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_1M08,11025,   "PCM 8 bit 11 KHz  Mono"                     ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_2M08,22050,   "PCM 8 bit 22 KHz  Mono"                     ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4M08,44100,   "PCM 8 bit 44 KHz  Mono"                     ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4M08,50000,   "PCM 8 bit 50 KHz  Mono"                     ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_8S08,2000,    "PCM 8 bit 2 KHz Stereo"                     ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_8S08,8000,    "PCM 8 bit 8 KHz  Stereo"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_1S08,11025,   "PCM 8 bit 11 KHz Stereo"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_2S08,22050,   "PCM 8 bit 22 KHz Stereo"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4S08,44100,   "PCM 8 bit 44 KHz Stereo"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4S08,50000,   "PCM 8 bit 50 KHz Stereo"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_8S16,2000,    "PCM 16 bit 2 KHz Stereo"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_8S16,8000,    "PCM 16 bit 8 KHz Stereo"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_1S16,11025,   "PCM 16 bit 11 KHz Stereo"                   ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_2S16,22050,   "PCM 16 bit 22 KHz Stereo"                   ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4S16,44100,   "PCM 16 bit 44 KHz Stereo"                   ,
DATATYPE_MULAW      ,  MULAW_8B8KS,     8000,    "Mulaw 8 bit 8 KHz Stereo"                   ,
DATATYPE_MULAW      ,  MULAW_8B11KS,    11025,   "Mulaw 8 bit 11 KHz Stereo"                  ,
DATATYPE_MULAW      ,  MULAW_8B22KS,    22050,   "Mulaw 8 bit 22 KHz Stereo"                  ,
DATATYPE_MULAW      ,  MULAW_8B8KM,     8000,    "Mulaw 8 bit 8 KHz  Mono"                    ,
DATATYPE_MULAW      ,  MULAW_8B11KM,    11025,   "Mulaw 8 bit 11 KHz  Mono"                   ,
DATATYPE_MULAW      ,  MULAW_8B22KM,    22050,   "Mulaw 8 bit 22 KHz  Mono"                   ,
DATATYPE_MULAW      ,  MULAW_8B44KM,    44100,   "Mulaw 8 bit 44 KHz  Mono"                   ,
DATATYPE_ALAW       ,  ALAW_8B8KS,      8000,    "Alaw 8 bit 8 KHz  Stereo"                   ,
DATATYPE_ALAW       ,  ALAW_8B11KS,     11025,   "Alaw 8 bit 11 KHz Stereo"                   ,
DATATYPE_ALAW       ,  ALAW_8B22KS,     22050,   "Alaw 8 bit 22 KHz Stereo"                   ,
DATATYPE_ALAW       ,  ALAW_8B44KS,     44100,   "Alaw 8 bit 44 KHz Stereo"                   ,
DATATYPE_ALAW       ,  ALAW_8B8KM,      8000,    "Alaw 8 bit 8 KHz  Mono"                     ,
DATATYPE_ALAW       ,  ALAW_8B11KM,     11025,   "Alaw 8 bit 11 KHz  Mono"                    ,
DATATYPE_ALAW       ,  ALAW_8B22KM,     22050,   "Alaw 8 bit 22 KHz  Mono"                    ,
DATATYPE_ALAW       ,  ALAW_8B44KM,     44100,   "Alaw 8 bit 44 KHz  Mono"                    ,
DATATYPE_SPV2       ,  SPV2_BPCM,       14700,   "SPV2 BPCM. 16 bit 14.7 KHz  Mono"           ,
DATATYPE_SPV2       ,  SPV2_PCM,        14700,   "SPV2 PCM. 16 bit 14.7 KHz  Mono"            ,
DATATYPE_SPV2       ,  SPV2_NONE,       0,       "SPV2 None. 0 bit 0 KHz  Mono"               ,
DATATYPE_CDXA_AUDIO ,  CDXA_LEVELC_MONO,18900,   "CDXA Level C. 16 bit 18.9 KHz  Mono"        ,
DATATYPE_CDXA_AUDIO ,  CDXA_LEVELB_MONO,37800,   "CDXA Level B. 16 bit 37.8 KHz  Mono"        ,
DATATYPE_CDXA_AUDIO ,  CDXA_LEVELC,     18900,   "CDXA Level C. 16 bit 18.9 KHz  Stereo"      ,
DATATYPE_CDXA_AUDIO ,  CDXA_LEVELB,     37800,   "CDXA Level B. 16 bit 37.8 KHz  Stereo"      ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4M08,50000,  "PCM 8 bit 50 KHz  Mono"                     ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4S08,50000,  "PCM 8 bit 50 KHz Stereo"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4M16,50000,  "PCM 16 bit 50 KHz  Mono"                    ,
DATATYPE_WAVEFORM   ,  WAVE_FORMAT_4S16,50000,  "PCM 16 bit 50 KHz Stereo"                   ,
DATATYPE_MULAW      ,  MULAW_8B44KS,    44100,  "Mulaw 8 bit 44 KHz  Stereo"                 ,
DATATYPE_MULAW      ,  ALAW_8B44KS,     44100,  "Alaw 8 bit 44 KHz  Stereo"                  ,
DATATYPE_CT_ADPCM   ,  CT_ADPCM_16B8KS, 8000,  "CT_ADPCM 16 bit 8KHz Stereo"                 ,
DATATYPE_CT_ADPCM   ,  CT_ADPCM_16B11KS,11025,  "CT_ADPCM 16 bit 11KHz Stereo"               ,
DATATYPE_CT_ADPCM   ,  CT_ADPCM_16B22KS,22050,  "CT_ADPCM 16 bit 22KHz Stereo"               ,
DATATYPE_CT_ADPCM   ,  CT_ADPCM_16B44KS,44100,  "CT_ADPCM 16 bit 44KHz Stereo"               ,
DATATYPE_CT_ADPCM   ,  CT_ADPCM_16B8KM, 8000,  "CT_ADPCM 16 bit 8KHz Mono"                   ,
DATATYPE_CT_ADPCM   ,  CT_ADPCM_16B11KM,11025,  "CT_ADPCM 16 bit 8KHz Mono"                  ,
DATATYPE_CT_ADPCM   ,  CT_ADPCM_16B22KM,22050,  "CT_ADPCM 16 bit 8KHz Mono"                  ,
DATATYPE_CT_ADPCM   ,  CT_ADPCM_16B44KM,44100,  "CT_ADPCM 16 bit 8KHz Mono"                  ,
DATATYPE_WAVEFORM,   WAVE_FORMAT_4M16,  48000,  "PCM 16 bit 44 KHz Mono"                     ,
DATATYPE_WAVEFORM,   WAVE_FORMAT_4M08,  48000,  "PCM 8 bit 44 KHz Mono"                      ,
DATATYPE_WAVEFORM,   WAVE_FORMAT_4S08,  48000,  "PCM 8  bit 44 KHz Stereo"                   ,
DATATYPE_WAVEFORM,   WAVE_FORMAT_4S16,  48000,  "PCM 16 bit 44 KHz Stereo"
};

   if ( LstId == LI_STRMDATATYPES)
   {
     CLASS_DATA *pDevDataTable = (CLASS_DATA *)aStreamData[StreamCount].pDevDataTable ;

     if (pDevDataTable == NULL)
     {
         mprintf ("ListInsertStrm: Null Device Data\n");
         return (PVOID)(NULL);
     }


     for (i = 0; i < aStreamData[StreamCount].ulNumDataTypes ; i++)
     {
       for (j = 0 ; j < NUMDATATYPES; j++)
       {
           fFoundDataType = FALSE ;
           if ( ( pDevDataTable->ulDataType == DataTypes[j].ulDataType ) &&
                ( pDevDataTable->ulSubType ==  DataTypes[j].ulSubType  ) &&
                ( pDevDataTable->sSRate ==  DataTypes[j].sSRate        )   )
           {

                  InsertListItem ( LstId,
                                   LIT_END,
                                   DataTypes[j].szDataType );

                  fFoundDataType = TRUE;
                  break;

           }

       } /* Number of known system datatypes */

       if (!fFoundDataType)
           mprintf ("DataType Not found %d %d %d \n",
                    pDevDataTable->ulDataType,
                    pDevDataTable->ulSubType,
                    pDevDataTable->sSRate);

      pDevDataTable++;   /* Increment the devtable pointer */

     } /* Number of known data types to this instance */
   }   /* This was the Stream Data types list box */
   else
   {
     usType = SHORT1FROMMP (ulDataIndex);
     if(usType == INSERT_ITEM)
     {
       /*
       ** We come here if we just want to insert all known
       ** data types to the system. This is used in the case of
       **  Install protocol dialog
       */
       for ( i= 0; i< 56; i++)
        InsertListItem ( LstId, LIT_END, DataTypes[i].szDataType );
     }
     else
     {
        /*
        ** This execution path is taken when we have a user
        ** supplied index into all the known system datatypes
        ** and based on that index we want to retrieve the
        ** stream data type and sub type. Also used in the
        ** case of stream protocol dialog.
        */

        if(usType == QUERY_ITEM)
        {
          Index = SHORT2FROMMP (ulDataIndex);
          spcbkey.ulDataType = DataTypes[Index].ulDataType;
          spcbkey.ulDataSubType= DataTypes[Index].ulSubType;
          return((PVOID)(&spcbkey));
        } /* Query Item */
     } /* Not Insert Item */
   }  /* Not the Stream data types list */
}  /* End of function */
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION: Initialize()
*
* ARGUMENTS:    None.
*
* RETURN:       None.
*
* DESCRIPTION:
*
* OS/2 API's USED: WinCreateHelpInstance()
*                  WinDestroyHelpInstance()
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES: None.
*
* GLOBAL VARIABLES REFERENCED: helpinit data structure.
*                              hwndHelpInstance.
*
* GLOBAL VARIABLES MODIFIED: helpinit data structure.
*                            hwndHelpInstance.
*
*****************************END OF SPECIFICATIONS ***************************/
VOID Initialize()
{
    int    i;            /* Loop Index  */
    APIRET rc;           /* Return Code */

    /*
    **  Fill in the Audio Device Control Block physical device name
    **  with the default audio device to start with. This would normally
    **  be AUDIO1$.
    */

    /* Obtain System Multimedia Hardware Setup Information */

    QueryInstalledDevices ();

    /*
    **  Fill in the Audio Device Control Block physical device name
    **  with the default audio device to start with. This would normally
    **  be AUDIO1$. Also default to the first audio device installed.
    */

    for (i = 0; i < MAXSTREAMS ; i++)
    {
        strcpy (aStreamData[i].AudioDCB.szDevName,
                PhysicalDevData[0].szPhysicalDevName);

        aStreamData[i].ulResID =  PhysicalDevData[0].ulResourceId ;
    }

    rc = FixDeviceData ();

    /*
    ** Obtain a list of Installed handlers from the MMPM/2
    **   Sync Stream Manager (SSM).
    */

    SpiEnumerateHandlers((PHAND)&aHand, &ulNumHand);

    /* Initialize our default log and script file names */

    strcpy (szScriptFile,DEFAULT_SCRIPT_FILE);
    strcpy (szLogFileName,DEFAULT_LOG_FILE);

    /* Initialize the default Font used for presentation */

    strcpy (szDefaultFont,DEFAULT_FONT);

    /* Initilaize the Help Manager structures */

    helpinit.cb                       = sizeof(HELPINIT);
    helpinit.ulReturnCode             = 0LU;
    helpinit.pszTutorialName          = (PSZ)NULL;
    helpinit.phtHelpTable      = (PHELPTABLE)MAKELONG(MAINHELPTABLE, 0xFFFF);
    helpinit.hmodAccelActionBarModule = NULLHANDLE;
    helpinit.idAccelTable             = 0;
    helpinit.idActionBar              = 0;
    helpinit.pszHelpWindowTitle       = "Audio Device Driver Exerciser 1.0";
    helpinit.hmodHelpTableModule      = NULLHANDLE;
    helpinit.fShowPanelId             = CMIC_HIDE_PANEL_ID;
    helpinit.pszHelpLibraryName       = "PMADDE.HLP";

    if((hwndHelpInstance = WinCreateHelpInstance(hab, &helpinit))==NULLHANDLE)
    {
       mprintf ("Help Creation Error\n");
    }
    else
    if (helpinit.ulReturnCode)
    {
      mprintf ("Help Creation Error\n");
      WinDestroyHelpInstance(hwndHelpInstance);
      hwndHelpInstance = 0;
    }
    /* Create Print Active Information mutual exclusion semaphore */

    DosCreateMutexSem (NULL,
                      (PHEV) &hmtxPrntSem,
                       DC_SEM_SHARED,
                       FALSE);

}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    OpenFileDlg
*
* ARGUMENTS:   None
*
* RETURN:      None.
*
* DESCRIPTION: Open File Dialog.
*
* OS/2 API's USED: WinFileDlg().
*
* C FUNCTION CALLS:
*
*
* INTERNAL FUNCTION REFERENCES: None.
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
*****************************END OF SPECIFICATIONS ***************************/
RC OpenFileDlg (HWND hwndOwner,PSZ pszTitle,PSZ FileName, PSZ FullFile)
{
   FILEDLG  pfdFileDlg;                  /* File Dialog Structure */
   HWND     hwndFileDlg;                 /* Handle to the above   */

   /* Initialize the structure to 0 */

   memset (&pfdFileDlg,0,sizeof(FILEDLG));

   pfdFileDlg.cbSize   = sizeof(FILEDLG);
   pfdFileDlg.fl       = FDS_HELPBUTTON | FDS_CENTER | FDS_OPEN_DIALOG ;
   pfdFileDlg.pszTitle = pszTitle;

   strcpy (pfdFileDlg.szFullFile,FullFile);

   hwndFileDlg = WinFileDlg (HWND_DESKTOP,hwndOwner,&pfdFileDlg);

   if (hwndFileDlg == NULLHANDLE)
   {
       mprintf ("File Dialog Creation Error\n");
       return (TRUE);
   }

   if (hwndFileDlg && (pfdFileDlg.lReturn == DID_OK))
   {
       CheckString (pfdFileDlg.szFullFile);
       strcpy (FileName,pfdFileDlg.szFullFile);
       return (FALSE);
   }

   if (hwndFileDlg && (pfdFileDlg.lReturn == DID_CANCEL))
   {
       return (TRUE);
   }
   return (FALSE);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION      Terminate
*
* ARGUMENTS:    None.
*
* RETURN:       None.
*
* DESCRIPTION: Terminate the application and release resourves.
*
* OS/2 API's USED: WinDestroyWindow()
*                  WinDestroyHelpInstance()
*                  WinDestroyMsgQueue()
*                  WinTerminate()
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES: None.
*
* GLOBAL VARIABLES REFERENCED: hab,hwnd1,hmq,
*                              hwndHelpInstance.
*
* GLOBAL VARIABLES MODIFIED: hab,hwnd1,hmq,
*                            hwndHelpInstance.
*
*****************************END OF SPECIFICATIONS ***************************/
VOID Terminate (HMQ hmq)
{
    USHORT i;    /* Loop Index */

    /*
    ** Write Exit token if script is enabled
    */
    WriteScriptFile (EXIT);


    QuitProg();

    /* Destroy the window */
    WinDestroyWindow(hwnd1);

    /* Destroy the Help instance */
    if (hwndHelpInstance)
       WinDestroyHelpInstance (hwndHelpInstance);

    /* Destroy the message queue */
    WinDestroyMsgQueue( hmq );

    /* Terminate and release resources */
    WinTerminate( hab );

}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION     LongToString
*
* ARGUMENTS:   Long .
*
* RETURN:      String .
*
* DESCRIPTION: Conversion routine.
*
* OS/2 API's USED: None.
*
* C FUNCTION CALLS:     _ltoa()
*                       malloc()
*
* INTERNAL FUNCTION REFERENCES: None.
*
* GLOBAL VARIABLES REFERENCED:
*
* GLOBAL VARIABLES MODIFIED:
*
*****************************END OF SPECIFICATIONS ***************************/
PSZ LongToString(LONG l)
{
    PSZ   pszReturn;     /* Temporary Storage */
    CHAR  cString[34];   /* Buffer            */

    pszReturn = _ltoa(l, cString, 10);
    return ((PSZ)cString);

}  /* LongToString */

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    OpenFile
*
* ARGUMENTS:   FileName and File Handle pointer
*
* RETURN:      API Return Code.
*
* DESCRIPTION: Open File.
*
* OS/2 API's USED: DosOpen.
*
*****************************END OF SPECIFICATIONS ***************************/
APIRET OpenFile(PSZ szFileName,ULONG *pFileHandle)
{
   ULONG    ulAction;                 /* Action to be taken */
   APIRET   RetCode;                  /* API return code    */

   RetCode = DosOpen (szFileName,
                      pFileHandle,
                      &ulAction,
                      0,
                      FILE_NORMAL,
                      OPEN_ACTION_OPEN_IF_EXISTS|
                      OPEN_ACTION_CREATE_IF_NEW,
                      OPEN_FLAGS_WRITE_THROUGH | OPEN_FLAGS_NO_CACHE|
                      OPEN_FLAGS_SEQUENTIAL | OPEN_SHARE_DENYNONE|
                      OPEN_ACCESS_READWRITE,
                      0);

   return (RetCode);
}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    WriteFile
*
* ARGUMENTS:   None
*
* RETURN:      None.
*
* DESCRIPTION: Write to Log File for command logging.
*
* OS/2 API's USED: DosWrite.
*
* C FUNCTION CALLS:
*
* INTERNAL FUNCTION REFERENCES: None.
*
*****************************END OF SPECIFICATIONS ***************************/

APIRET WriteFile(HFILE FileHandle,PSZ buffer)
{
   ULONG    ulBytesWritten;          /* Bytes Written    */
   APIRET   ReturnCode;              /* API Return Code  */
   CHAR     chEOL[2] = "\r\n";       /* End of Line      */

   if (FileHandle == (HFILE)NULL)
       return(1) ;

   ReturnCode = DosWrite(FileHandle,buffer,strlen(buffer),&ulBytesWritten);

   ReturnCode = DosWrite(FileHandle,chEOL,2,&ulBytesWritten);

   return (ReturnCode);

}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    mprintf
*
* ARGUMENTS:   Error Message String.
*
* RETURN:      None.
*
* DESCRIPTION: Write to Log File for command logging.
*
* OS/2 API's USED:   WinAlarm()
*                    WinMessageBox().
*
* C FUNCTION CALLS:  malloc()
*                     va_start();
*
*
* INTERNAL FUNCTION REFERENCES: None.
*
*****************************END OF SPECIFICATIONS ***************************/

LONG mprintf(PSZ format, ...)
{
   va_list pArg;              /* Variable Argument list  */
   SZ     buffer[513];        /* Temporary String buffer */
   LONG    lCount;            /* Character count         */

   /* Beep to alert the user */

   WinAlarm( HWND_DESKTOP, WA_ERROR );


   va_start(pArg, format);

   lCount = vsprintf(buffer, format, pArg);

   WinMessageBox( HWND_DESKTOP,       /* parent window handle    */
                  HWND_DESKTOP,       /* owner window handle     */
                  buffer,             /* pointer to message text */
                  "PMADDE",            /* pointer to title text   */
                  MSG_BOX_ID,         /* message box identifier  */
                  MB_OK | MB_ERROR ); /* message box style       */

   return lCount;
}   /*  end of mprintf()  */

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    Write2File
*
* ARGUMENTS:   HFILE FileHandle,   PSZ buffer
*
* RETURN:      RC
*
* DESCRIPTION: This function gets Date and Time and write it to the log file
*
* OS/2 API's USED: DosGetDateTime, DosWrite
*
* C FUNCTION CALLS: Sprintf
*
**************************************************************************/
APIRET Write2File(HFILE FileHandle,PSZ buffer)
{
   ULONG    ulBytesWritten;        /* Bytes Written    */
   APIRET   ReturnCode;            /* API return Code  */
   CHAR     chEOL[2] = "\r\n";     /* CR-LF            */
   CHAR     szTimeDate[32];        /* Time Date Array  */
   DATETIME dt;                    /* Date Time Struct */

   ReturnCode = DosGetDateTime(&dt);
   if (ReturnCode)
   {
       strcpy (buffer, "DosGetDateTime failure:");
       DosWrite (FileHandle,buffer, strlen(buffer),&ulBytesWritten);
       return (ReturnCode);
   }

   sprintf (szTimeDate,
            "%04d-%02d-%02d  %02d:%02d:%04d  ",
            dt.year,
            dt.month,
            dt.day,
            dt.hours,
            dt.minutes,
            dt.seconds );


   ReturnCode = DosWrite (FileHandle,szTimeDate,
                          strlen(szTimeDate),&ulBytesWritten);

   ReturnCode = DosWrite (FileHandle,buffer,
                          strlen(buffer),&ulBytesWritten);

   ReturnCode = DosWrite (FileHandle, chEOL,
                          strlen(chEOL),&ulBytesWritten);

   return (ReturnCode);

}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    SetFont
*
* ARGUMENTS:   HWND hwnd, ULONG ulWhat, PSZ pszFontName
*
* RETURN:      None
*
* DESCRIPTION: This function sets font.
*
* OS/2 API's USED:
*
*
* C FUNCTION CALLS:
*
**************************************************************************/

BOOL SetFont( HWND hwnd, ULONG ulWhat, PSZ pszFontName)
{
   BOOL brc = FALSE;

   if ((pszFontName == (PSZ)NULL) || (ulWhat == 0L))
   return FALSE;

   brc =  WinSetPresParam( hwnd,
                         ulWhat,
                         (ULONG)((ULONG) strlen(pszFontName)+(ULONG)1),
                         (PSZ) pszFontName);

}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:  CreateStaticText
*
* ARGUMENTS: HWND hwnd
*
* RETURN:    None
*
* DESCRIPTION: This function Creates the static Text controls for display
*
* OS/2 API's USED: WinCreateWindow()
*                  WinQueryWindowRect()
*
* INTERNAL FUNCTION REFERENCES: CreateText Macro
*                               SetFont Macro.
*                               SetMLEFormat Macro
*
**************************************************************************/
VOID CreateStaticText (HWND hwnd)
{
    LONG    lPresParam;          /* Presenation parameters        */
    RECTL   rectl, rctlSize;     /* Client rectangular dimensions */
    HPS     hps;                 /* Handle to Presentation Space  */
    ULONG   flCreate1;           /* Window Creation Flags         */
    USHORT  usxloc;              /* Position x                    */
    USHORT  usyloc;              /* Position y                    */
    USHORT  usWidth;             /* Width                         */
    USHORT  usHeight;            /* Height                        */
    USHORT  usListWidth;         /* List box width                */


    /*
    ** Obtain the client dimensions.
    */
    WinQueryWindowRect (hwndClActWin, &rectl);

    /*
    ** Set up the dimensions and positions.
    */

    usxloc = rectl.xLeft;
    usyloc = (rectl.yTop - (cyChar*2 ));
    usWidth = ((cxChar * strlen (DISPCAT1))-(3*cxChar));
    usHeight = cyChar*2;

    /*
    ** This is a macro that expands to WinCreateWindow API.
    */

    /* Create the WC_STATIC, "Handler Name"  */
    CreateTxt (hwndClActWin,DISPCAT1,STX_);

    SetFont ( WinWindowFromID(hwnd,STX_), PP_FONTNAMESIZE, szDefaultFont );

    usxloc+=usWidth+cxChar/2;
    usWidth = ((cxChar * strlen(DISPCAT2))-(3*cxChar));

    /* Create the WC_STATIC, "Handler Class"  */

    CreateTxt (hwndClActWin,DISPCAT2,STX_1);

    SetFont ( WinWindowFromID(hwnd,STX_1), PP_FONTNAMESIZE, szDefaultFont );

    usxloc+=usWidth;
    usWidth = ((cxChar * strlen (DISPCAT3))-(4*cxChar));

    /* Create the WC_STATIC, "Source Handler ID"  */

    CreateTxt (hwndClActWin,DISPCAT3,STX_2);

    SetFont ( WinWindowFromID(hwnd,STX_2), PP_FONTNAMESIZE, szDefaultFont );

    usxloc+=usWidth+cxChar/3;
    usWidth = ((cxChar * strlen (DISPCAT4))-(5*cxChar));
    usListWidth =usxloc +usWidth+cyChar/2;

    /* Create the WC_STATIC, "Target Handler ID"  */

    CreateTxt (hwndClActWin,DISPCAT4,STX_3);

    SetFont ( WinWindowFromID(hwnd,STX_3), PP_FONTNAMESIZE, szDefaultFont );

    usxloc+=usWidth+5/2*cxChar;
    usWidth = ((cxChar * strlen (DISPCAT5))-(4*cxChar));

    /* Create the WC_STATIC, "Stream Device"  */

    CreateTxt (hwndClActWin,DISPCAT5,STX_4);

    SetFont ( WinWindowFromID(hwnd,STX_4), PP_FONTNAMESIZE, szDefaultFont );

    usxloc+=usWidth+cxChar/2;
    usWidth = ((cxChar * strlen (DISPCAT6))-(4*cxChar));

    /* Create the WC_STATIC, "Stream Number"  */

    CreateTxt (hwndClActWin,DISPCAT6,STX_5);

    SetFont ( WinWindowFromID(hwnd,STX_5), PP_FONTNAMESIZE, szDefaultFont );

    usxloc+=usWidth+4*cxChar;
    usWidth = ((cxChar * strlen (DISPCAT7))-(4*cxChar));

    /* Create the WC_STATIC, "Stream State"  */

    CreateTxt (hwndClActWin,DISPCAT7,STX_6);

    SetFont ( WinWindowFromID(hwnd,STX_6), PP_FONTNAMESIZE, szDefaultFont );

    usxloc+=usWidth + 4*cxChar;
    usWidth = ((cxChar * strlen (DISPCAT8))-(4*cxChar));

    /* Create the WC_STATIC, "Stream Source"  */

    CreateTxt (hwndClActWin,DISPCAT8,STX_7);

    SetFont ( WinWindowFromID(hwnd,STX_7), PP_FONTNAMESIZE, szDefaultFont );

    usxloc+=usWidth+cxChar/2;
    usWidth = ((cxChar * strlen (DISPCAT9))-(4*cxChar));

    /* Create the WC_STATIC, "Stream target"  */

    CreateTxt (hwndClActWin,DISPCAT9,STX_8);

    SetFont (WinWindowFromID(hwnd,STX_8), PP_FONTNAMESIZE, szDefaultFont );

    usxloc+=usWidth+cxChar/2;
    usWidth = ((cxChar * strlen (DISPCAT10))-(3*cxChar));

    /* Create the WC_STATIC, "Stream Data"  */

    CreateTxt (hwndClActWin,DISPCAT10,STX_9);

    SetFont (WinWindowFromID(hwnd,STX_9), PP_FONTNAMESIZE, szDefaultFont);

    /*
    ** Create the MLE and set it up as a client of
    ** event monitor window
    */
    hwndActLst = WinCreateWindow( hwndClActWin, WC_LISTBOX, "",
                                 LS_HORZSCROLL | LS_NOADJUSTPOS,
                                 0, 0,
                                 (SHORT)usListWidth ,
                                 (SHORT)rectl.yTop -cyChar*2,
                                 hwndClActWin, HWND_BOTTOM, ID_ACTLIST, 0L, NULL );

    hwndActMLE = WinCreateWindow( hwndClActWin, WC_MLE, "",
                                 MLS_BORDER |
                                 MLS_VSCROLL | MLS_HSCROLL | MLS_READONLY,
                                 usListWidth , 0,
                                 (SHORT)(rectl.xRight-usListWidth) ,
                                 (SHORT)rectl.yTop -cyChar*2,
                                 hwndClActWin, HWND_BOTTOM, FID_CLIENT, 0L, NULL );

    /*
    ** Set up MLE import export transaction format
    */
    SetMLEFormat(hwndActMLE);

    /*
    ** Obtain the client dimensions.
    */
    WinQueryWindowRect (hwndClEveWin, &rectl);

    /*
    ** Create the MLE and set it up as a
    ** client of event monitor window
    */
    hwndEveMLE = WinCreateWindow( hwndClEveWin, WC_MLE, "",
                                MLS_BORDER | MLS_VSCROLL |
                                MLS_HSCROLL| MLS_READONLY,
                                0, 0,
                                (SHORT)rectl.xRight,
                                (SHORT)rectl.yTop-cyChar*2,
                                hwndClEveWin, HWND_BOTTOM, FID_CLIENT, 0L, NULL );

    /*
    ** Set up MLE import export transaction format
    */
    SetMLEFormat(hwndEveMLE);

    usxloc = rectl.xLeft;
    usyloc = (rectl.yTop - (cyChar*2 ));
    usWidth = cxChar * strlen (DISPCAT11) ;

    CreateTxt (hwndClEveWin,DISPCAT11,STX_10);

    usxloc+=usWidth+cyChar;
    usWidth = cxChar * strlen (DISPCAT12) ;

    CreateTxt (hwndClEveWin,DISPCAT12,STX_11);

    /*
    ** Obtain the client dimensions.
    */
    WinQueryWindowRect (hwndClComWin, &rectl);

    /*
    ** Create the MLE and set it up as a
    ** client of event monitor window
    */

    hwndComMLE = WinCreateWindow( hwndClComWin, WC_MLE, "",
                                MLS_BORDER | MLS_VSCROLL |
                                MLS_HSCROLL | MLS_READONLY,
                                0, 0,
                                (SHORT)rectl.xRight,
                                (SHORT)rectl.yTop - cyChar*2,
                                hwndClComWin, HWND_BOTTOM, FID_CLIENT, 0L, NULL );

    /*
    ** Set up MLE import export transaction format
    */
    SetMLEFormat(hwndComMLE);

    usxloc = rectl.xLeft;
    usyloc = (rectl.yTop - (cyChar*2 ));
    usWidth = cxChar * strlen (DISPCAT13) ;

    CreateTxt (hwndClComWin,DISPCAT13,STX_12);

    usxloc+=usWidth+3*cyChar;
    usWidth = cxChar * strlen (DISPCAT11) ;

    CreateTxt (hwndClComWin,DISPCAT11,STX_17);

    usxloc+=usWidth+cxChar;
    usWidth = cxChar * strlen (DISPCAT14) ;

    CreateTxt (hwndClComWin,DISPCAT14,STX_13);

    usxloc+=usWidth;
    usWidth = cxChar * strlen (DISPCAT15) ;

    CreateTxt (hwndClComWin,DISPCAT15,STX_14);

} /* return to caller */

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    OpenShell
*
* ARGUMENTS:   None
*
* RETURN:      None
*
* DESCRIPTION: This function is to open an OS/2 Window from the application
*
* OS/2 API's USED:  DosStartSession, DosBeep
*
*
**************************************************************************/
VOID OpenShell ()
{
    CHAR       szBuf[150];                /* Temporary Storage */
    APIRET     rc;                        /* API return Code   */
    PID        pidProcess;                /* Process ID        */
    CHAR       szPgmName[] = "CMD.EXE";   /* Program name      */
    ULONG      ulSessionID,cbWritten;     /* Identifiers       */
    STARTDATA  sd;                        /* Control structure */


    sd.Length = sizeof(sd);
    sd.Related = SSF_RELATED_CHILD;
    sd.FgBg = SSF_FGBG_FORE;
    sd.TraceOpt = SSF_TRACEOPT_NONE;
    sd.PgmTitle = (PSZ) NULL;
    sd.PgmName = szPgmName;
    sd.PgmInputs = (PBYTE) NULL;
    sd.TermQ = (PBYTE) NULL;
    sd.Environment = 0;
    sd.InheritOpt = SSF_INHERTOPT_PARENT;
    sd.SessionType = SSF_TYPE_DEFAULT;
    sd.IconFile = (PSZ) NULL;
    sd.PgmHandle = 0;
    sd.PgmControl = SSF_CONTROL_VISIBLE | SSF_CONTROL_MAXIMIZE;
    sd.InitXPos =30;
    sd.InitYPos =40;
    sd.InitXSize =200;
    sd.InitYSize =140;
    sd.Reserved =0;
    sd.ObjectBuffer = szBuf;
    sd.ObjectBuffLen = sizeof(szBuf);

    rc = DosStartSession( &sd, &ulSessionID, &pidProcess);
    if (rc)  DosBeep (750,250);

}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    CreateFontFixed
*
* ARGUMENTS:   HWND hwnd
*
* RETURN:      None
*
* DESCRIPTION: This function sets the font to SystemMonospaced.
*
* OS/2 API's USED:  None
*
* C FUNCTION CALLS: Strcpy.
*
**************************************************************************/
VOID CreateFontFixed (HWND hwnd)
{
    HPS hps; /* Presentation space */

    /* Get hps */
    hps = WinGetPS (WinWindowFromID(hwnd, FID_CLIENT));

    /* Obtain font metrics */
    GpiQueryFontMetrics (hps,(LONG)sizeof(fm),&fm);

    fm.fsType = FM_TYPE_FIXED;
    fm.fsDefn = FM_DEFN_GENERIC;

    /* Default to system monospaced font */

    strcpy (fm.szFacename, "System Monospaced");

    cxChar = (SHORT)fm.lAveCharWidth;
    cyChar = (SHORT)fm.lMaxBaselineExt;

    /* Release hps */
    WinReleasePS(hps);

}
/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    QueryInstalledDevices
*
* ARGUMENTS:
*
* RETURN:
*
* DESCRIPTION: To Query MMPM/2 and obtain list of all installed
*              audio devices and to initialize device specific info.
*
* OS/2 API's USED:  mciSendCommand() MMPM/2 Command Interface API
*
*
* C FUNCTION CALLS: memset()  malloc()
*                   strxxx()  free ()
*
**************************************************************************/
BOOL QueryInstalledDevices ()
{
    USHORT                 usNumDevices ;        /* Total Number of devices */
    SZ                     pszAllDevNames[1024]; /* All MMPM/2 Device Names */
    SZ                     pszAllDevNames2[1024];/* Above as an Char array  */
    SZ                     buffer[MAX_SYSINFO_RETURN];/* Return buffer      */
    CHAR                   tokensep[] = " ";     /* Token Seperators        */
    PCHAR                  pchWord;              /* Word pointer            */
    PSZ                    pszTemp ;             /* Temporary Storage       */
    USHORT                 i,j = 0;              /* Loop Indicies           */
    APIRET                 ReturnCode ;          /* API Return Code         */
    ULONG                  ulResID ;             /* Local Resource ID       */
    MCI_SYSINFO_PARMS      SysInfo ;             /* Generic Sys Info Struct */
    MCI_SYSINFO_DEVPARAMS  SysInfoAmpParms ;     /* Device Parm Sys Info Str*/
    MCI_SYSINFO_QUERY_NAME SysInfoQName ;        /* Sys Info Qry Name Str   */
    MCI_SYSINFO_LOGDEVICE  SysInfoLogDev;        /* Sys Info Log Dev  Str   */



     /*
     ** Fix the device specific information. At Application initialization
     ** time QueryInstalledDevices () function is called to obtain
     ** Multimedia system configuration information. A Resource ID
     ** Exists for all installed audio devices, that tells us what
     ** the capabilities of the device are (PMADDEVS.rc). We call
     ** the function FixDeviceData () to set up the resource id for
     ** the current device in selection. Note that some application
     ** device specific defaults are set here.
     */

    /* Initialize the SysInfo structures */

    memset ((PCH)&SysInfo,0,sizeof(MCI_SYSINFO_PARMS));
    memset ((PCH)&SysInfoQName,0,sizeof(MCI_SYSINFO_QUERY_NAME));


    SysInfo.lpstrReturn = (LPSTR)buffer;
    SysInfo.dwRetSize = MAX_SYSINFO_RETURN;

    /* First obtain the number of devices installed */

    ReturnCode = mciSendCommand ((WORD)MCI_ALL_DEVICE_ID,
                                 (WORD)MCI_SYSINFO,
                                 (DWORD)MCI_SYSINFO_QUANTITY | MCI_WAIT,
                                 (DWORD)&SysInfo,
                                 (WORD)NULL );

    /* Allocate memory for return information */

    if (!ReturnCode)
    {
        usNumDevices = (USHORT)atoi ((PCHAR)SysInfo.lpstrReturn);
    }
    /* Obtain the names of devices installed */

    memset ((PCH)&SysInfo,0,sizeof(MCI_SYSINFO_PARMS));
    SysInfo.lpstrReturn = (LPSTR)pszAllDevNames;
    SysInfo.dwRetSize = usNumDevices * DEVICECHARS +usNumDevices;

    ReturnCode = mciSendCommand ((WORD)MCI_ALL_DEVICE_ID,
                                 (WORD)MCI_SYSINFO,
                                 (DWORD)MCI_SYSINFO_NAME | MCI_WAIT,
                                 (DWORD)&SysInfo,
                                 (WORD)NULL );


    strcpy (pszAllDevNames2, pszAllDevNames) ;

    pchWord = strtok (pszAllDevNames,tokensep);

    for (i = 0; i < usNumDevices ; i++)
    {
         if (pchWord == NULL)
             break;
         pszTemp = strstr (pchWord,"Wave");

         if (pszTemp == NULL)
             pszTemp = strstr (pchWord,"wave");

         if (pszTemp != NULL)
         {
             memset ((PCH)&SysInfo,0,sizeof(MCI_SYSINFO_PARMS));
             memset ((PCH)&SysInfoQName,0,sizeof(MCI_SYSINFO_QUERY_NAME));

             strcpy (SysInfoQName.szLogicalName,pchWord);

             /* Get the logical name */

             SysInfo.dwItem = MCI_SYSINFO_QUERY_NAMES ;
             SysInfo.pSysInfoParm = &SysInfoQName ;
             ReturnCode = mciSendCommand ((WORD)NULL,
                                          (WORD)MCI_SYSINFO,
                                          (DWORD)MCI_SYSINFO_ITEM | MCI_WAIT,
                                          (DWORD)&SysInfo,
                                          (WORD)NULL );

             strcpy(PhysicalDevData[j].szLogDevName,
                    SysInfoQName.szLogicalName);

             j++;

         } /* if "audio" */

         pchWord = strtok (NULL,tokensep); /* Get the Next Token */

    } /* For Loop */

    /*
    ** Tokenize the second copy of all the device names we
    ** obtanied from Sys Info call
    */
    pchWord = strtok (pszAllDevNames2,tokensep);
    j = 0 ;

    for (i = 0; i < usNumDevices ; i++)
    {
         if (pchWord == NULL)
             break;

         if ((strstr (pchWord,"Ampmix")) != NULL )
         {
              memset ((PCH)&SysInfo,0,sizeof(MCI_SYSINFO_PARMS));
              memset ((PCH)&SysInfoQName,0,sizeof(MCI_SYSINFO_QUERY_NAME));
              memset ((PCH)&SysInfoAmpParms,0,sizeof(MCI_SYSINFO_DEVPARAMS));
              memset ((PCH)&SysInfoLogDev,0,sizeof(MCI_SYSINFO_LOGDEVICE));

              strcpy (SysInfoQName.szLogicalName,pchWord);

              /* Get the logical name */

              SysInfo.dwItem = MCI_SYSINFO_QUERY_NAMES ;
              SysInfo.pSysInfoParm = &SysInfoQName ;
              ReturnCode = mciSendCommand ((WORD)NULL,
                                           (WORD)MCI_SYSINFO,
                                           (DWORD)MCI_SYSINFO_ITEM | MCI_WAIT,
                                           (DWORD)&SysInfo,
                                           (WORD)NULL );


            /* Now go ahead and do the extended sysinfo call */

              SysInfo.dwItem = MCI_SYSINFO_QUERY_PARAMS ;
              SysInfo.dwRetSize = MAX_DEV_PARAMS ;
              SysInfo.pSysInfoParm = &SysInfoAmpParms ;

              strcpy (SysInfoAmpParms.szInstallName,
                      SysInfoQName.szInstallName);


              ReturnCode = mciSendCommand ((WORD)NULL,
                                           (WORD)MCI_SYSINFO,
                                           (DWORD)MCI_SYSINFO_ITEM | MCI_WAIT,
                                           (DWORD)&SysInfo,
                                           (WORD)NULL );

              if (!ReturnCode)
              {
                  pchWord = strstr (SysInfoAmpParms.szDevParams,"RCID");

                  if (pchWord != NULL)
                  {
                      pchWord+=5;   /* Skip the 'RCID=' */

                      /* Got The Resource ID */
                      ulResID = atol (pchWord);
                      PhysicalDevData[j].ulResourceId = ulResID;

                  } /* not null pchword */
                 /*
                 ** Now Set Up Product Information and PDD Names
                 ** of associated devices.
                 */

                  SysInfo.dwItem = MCI_SYSINFO_QUERY_DRIVER ;
                  SysInfo.pSysInfoParm = &SysInfoLogDev ;

                  strcpy (SysInfoLogDev.szInstallName,
                          SysInfoAmpParms.szInstallName);

                  ReturnCode =mciSendCommand((WORD)NULL,
                                             (WORD)MCI_SYSINFO,
                                             (DWORD)MCI_SYSINFO_ITEM | MCI_WAIT,
                                             (DWORD)&SysInfo,
                                             (WORD)NULL );


                  strcpy(PhysicalDevData[j].szPhysicalDevName,
                         SysInfoLogDev.szPDDName);

                  strcpy(PhysicalDevData[j].szProductInfo,
                         SysInfoLogDev.szProductInfo);

                  j++;

              } /* Successfull Extended Ampmixer Sys Info */
         }  /* Amp Mixer */

         pchWord = strtok (NULL,tokensep);
    } /* For Loop */
    /* Free the memory allocated */

}

/****************************START OF SPECIFICATIONS *************************
*
* FUNCTION:    FixDeviceData
*
* ARGUMENTS:   None
*
* RETURN:
*
* DESCRIPTION:
*
* OS/2 API's USED: DosGetResource
*
*
* C FUNCTION CALLS:
*
*
**************************************************************************/
APIRET FixDeviceData ()
{
    APIRET  ReturnCode;        /* API Return Code    */
    PULONG  pulResPtr;         /* Resource pointer   */
    PSZ     pszTemp;           /* Temporary store    */
    ULONG   ulRIFFMapModes;    /* RIFF Mapping       */


    /*
    ** All the Device specific information for the selected
    ** Audio device is contained in the resource file. The
    ** resource id maps resource to device. The Device data
    ** table containing Known data types is retrieved.
    */

    ReturnCode = DosGetResource (NULLHANDLE,
                                 RT_RCDATA,
                                 aStreamData[StreamCount].ulResID,
                                 &(aStreamData[StreamCount].pAudDevData));

    if (ReturnCode)
    {
        mprintf ("Unable to Load Device Specific Info %d\n",ReturnCode);
        return (ReturnCode);
    }

    /* Get Product information */

    pszTemp = (PSZ)aStreamData[StreamCount].pAudDevData;

    pulResPtr = (PULONG)(pszTemp+strlen(pszTemp)+1);
    aStreamData[StreamCount].ulDeviceID = *(pulResPtr)++;
    ulRIFFMapModes = *(pulResPtr)++;
    aStreamData[StreamCount].ulNumDataTypes = *(pulResPtr)++;

    /* Set up Instance Device Data Table pointers */

    aStreamData[StreamCount].pDevDataTable = pulResPtr;
    aStreamData[ulIndex].pDevDTblStrt = pulResPtr;

}/* Return to caller */
