/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#define INCL_DEV              //Include all DEV definitions
#define INCL_DEVP             //Include all DEV PRIVATE definitions
#define INCL_WIN              //Include all WIN definitions
#define INCL_GPI              //Include all GPI definitions
#define INCL_ERRORS           //Include all GPI error definitions
#define INCL_DOS


#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <os2.h>
#include <stdio.h>
#include <bsedev.h>
#include "PTTDLLDF.h"        //PTT defines.
#include "common.h"
#include "devcalls.h"

/*****************************************************************************
 *\\ddd
 * Subtask Name: DevCalls.dll
 *
 * Module name: DevCalls.c
 *
 * Purpose: DEV* test functions
 *
 * Revision Log:
 *               Aug. 30, 1991:
 *                    Created initial file by collecting previous DEV*
 *                    testcase entrypoints from existing DLLs, modifying
 *                    and debugging where necessary - Rickey C. Brown
 *                    (DevQueryCapsExh)
 *
 *              - Sept. 17, 1991:
 *                    Created other entry points to test other functions.
 *                    Rickey C. Brown
 *
 *              - Oct. 1, 1991:
 *                    Corrected and improved logging capabilities of all tests.
 *                    Rickey Brown
 *
 *
 * Contents:
 *
 * Design Overview:
 *   This TestCase is designed to test Dev- functions.
 *   Functions tested in this testcase are listed below :
 *
 *   1.  DevEscape()
 *   2.  DevQueryCaps()
 *   3.  DevQueryDeviceNames()
 *   4.  DevQueryHardcopyCaps()
 *   5.  DevPostDeviceModes()
 *   6.  DevOpenDC()
 *   7.  DevCloseDC()
 *
 *
 * Limitations:
 *   As per 9/17/91, all of the return code tests (with the exception of
 *   provide
 *   questionable results.  This must be discussed with the device driver team.
 *
 *
 *\\end
****************************************************************************/
/****************************************************************************
 *\\ddd
 * Routine Name: InitTest()
 *
 * Purpose:  Initialize function name and address array for ptt.
 *
 * System Requirements:
 *
 * Revision Log:
 *               Aug 30, 1991:
 *               - initial file (basically copied from GPISTD.C)
 *               - expanded as needed
 *
 * Inputs:
 *   sel          // passed through to the InitTestSegVars function
 *   pInitInfo    // name and address array used by ptt interface
 *
 * Outputs:
 *   pInitInfo    // returned initialized with names and addresses
 *
 * Subroutines Required:
 *   InitTestSegVars()    // used by the PTT interface
 *
 * Limitations:
 *    None
 *\\end
 ***************************************************************************/

/****************************************************************************
 *\\algorithm
 * {
 *   for all test functions in this test case {
 *       set function name: pInitInfo->function name = function name[i]
 *       set function address: pInitInfo->function address = function addr[i]
 *   }
 *   call init set function
 * }
 *\\end
 ***************************************************************************/

VOID APIENTRY InitTest(PVOID sel, PTESTINITINFO pInitInfo)
{
/* initialize total number of tests in test case  */
  pInitInfo->ntests = NUMTESTS;

/* load test names and entry point addresses  */
  strcpy(pInitInfo->group_name, "DEV Calls");
  strcpy(pInitInfo->test_names[0], "DevEscapeExh");
  pInitInfo->test_addrs[0] = DevEscapeExh;
  strcpy(pInitInfo->test_names[1], "DevEscSupportListApp");
  pInitInfo->test_addrs[1] = DevEscSupportListApp;
  strcpy(pInitInfo->test_names[2], "DevQueryCapsExh");
  pInitInfo->test_addrs[2] = DevQueryCapsExh;
  strcpy(pInitInfo->test_names[3], "DevQueryDeviceNamesExh");
  pInitInfo->test_addrs[3] = DevQueryDeviceNamesExh;
  strcpy(pInitInfo->test_names[4], "DevQueryHardcopyCapsExh");
  pInitInfo->test_addrs[4] = DevQueryHardcopyCapsExh;
  strcpy(pInitInfo->test_names[5], "DevPostDeviceModesExh");
  pInitInfo->test_addrs[5] = DevPostDeviceModesExh;

  InitTestSegVars(sel);
  return;
}

/*****************************************************************************
 *\ddd
 * Routine Name: DevEscapeExh
 *
 * Purpose: This is an entry point for the PTT that exhaustively tests
 *          the DevEscape function.
 *
 * System Requirements:
 *
 *
 * Revision Log:      Sept. 7. 1991: Created; Rickey C. Brown
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *
 *  ENTER_MAIN
 *  CLEANUP
 *  MapPt
 *  WorldWriteString
 *  DeviceIsDisplay
 *  DeviceIsPrinter
 *  UnMapY
 *  TestEscCode
 *  EXIT_MAIN
 *
 * Limitations:
 *
 *\\end
 *****************************************************************************/
/*****************************************************************************
 *\\algorithm
 *
 *  1)  Get device context handle
 *  2)  map points for output
 *  3)  for i = 1 to (number of escape codes)
 *      a)  test if escape code is supported  or not by the device driver
 *      b)  output info in a)
 *      c)  if supported, test the escape code
 *\\end
 *****************************************************************************/
VOID APIENTRY DevEscapeExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static PCHAR EntryName = "DevEscapeExh";
  static HDC hDC;
  LONG   lresult;
  SHORT  index;
  LONG   Xmain;
  CHAR esccode_message[128];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_PTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

//  Get the Device context handle
    if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR)
    {
       CLEANUP("GpiQueryDevice()", EntryName);
       EXIT_MAIN(EntryName);
       return;
    }

//  Initialize coordinates to 3/4" from top of screen & 1/2" from left
    ptl.x = (LONG)(delta_x/5L);
    Xmain = ptl.x;
    ptl.y = (LONG)((pg_size_y * delta_y) - (delta_y *3L/4L));

//  Map the coordinates
    MapPt(ptl);

//  NOW FIND OUT WHICH ESCAPE CODES ARE SUPPORTED AND TEST THEM
    NBand = FALSE;
    for (index = 0; index < NUMESC; index++)
    {
       strcpy(esccode_message, esccodes[index].szID);
       if ((lresult = DevEscape(hDC, DEVESC_QUERYESCSUPPORT,
                             (LONG)sizeof(esccodes[index].lIndex),
                              (PBYTE)&esccodes[index].lIndex,
                               0, NULL)) == DEV_OK )
       {
          strcat(esccode_message, " SUPPORTED BY CURRENT DRIVER");
          WorldWriteString(ptl.x, ptl.y, esccode_message);
          ptl.x = delta_x*1L;
          ptl.y -= syPosition;
          if ((UnMapY(ptl.y)) <= 75L && DeviceIsDisplay())
             break;
          TestEscCode(hDC, hPS, hWndClient, esccodes[index].lIndex);
       }
       else if (lresult == DEVESC_NOTIMPLEMENTED)
          strcat(esccode_message, " NOT IMPLEMENTED BY CURRENT DRIVER");
       else if (lresult == DEVESC_ERROR)
       {
          strcat(esccode_message, ": CAUSED DEVESC_ERROR IN CURRENT DRIVER");
          CLEANUP("DevEscape", EntryName);
       }
       else
          strcat(esccode_message, ": invalid return value");
       if (lresult != DEV_OK)
          WorldWriteString(ptl.x, ptl.y, esccode_message);
       if (!NBand)
          ptl.x = Xmain;
       ptl.y -= syPosition;
       if ((UnMapY(ptl.y)) <= 75L && DeviceIsDisplay())
          break;
    }

    EXIT_MAIN(EntryName);
  }
  return;
}  /* end of function DevEscapeExh()  */

/*****************************************************************************
 *\ddd
 * Routine Name: DevEscSupportListApp
 *
 * Purpose: This is an entry point for the PTT that displays a list stating
 *          which escape codes are supported by the driver and which are not.
 *          This is basically the same as DevEscapeExh, only this test case
 *          does not test the supported functions.
 *
 *
 * System Requirements:
 *
 *
 * Revision Log:     Sept. 7, 1991:  Created by Rickey C. Brown
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *
 *  ENTER_MAIN
 *  CLEANUP
 *  WorldWriteString
 *  EXIT_MAIN
 *  MapPt
 *
 * Limitations:
 *
 *\\end
 *****************************************************************************/
/*****************************************************************************
 *\\algorithm
 *
 *\\end
 *****************************************************************************/
VOID APIENTRY DevEscSupportListApp(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static PCHAR EntryName = "DevEscSupportListApp";
  static HDC hDC;
  LONG   lresult;
  SHORT  index;
  CHAR esccode_message[128];

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_PTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {
    ENTER_MAIN(EntryName);

//  Get the Device context handle
    if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR)
    {
       CLEANUP("GpiQueryDevice()", EntryName);
       EXIT_MAIN(EntryName);
       return;
    }


//  Initialize coordinates to 3/4" from top of screen
    ptl.x = (LONG)(delta_x/5L);
    ptl.y = (LONG)((pg_size_y * delta_y) - (delta_y *3L/4L));

//  Map the coordinates
    MapPt(ptl);

    for (index = 0; index < NUMESC; index++)
    {
       strcpy(esccode_message, esccodes[index].szID);
       if ((lresult = DevEscape(hDC, DEVESC_QUERYESCSUPPORT,
                             (LONG)sizeof(esccodes[index].lIndex),
                              (PBYTE)&esccodes[index].lIndex,
                               0, NULL)) == DEV_OK )
          strcat(esccode_message, " SUPPORTED BY CURRENT DRIVER");
       else if (lresult == DEVESC_NOTIMPLEMENTED)
          strcat(esccode_message, " NOT IMPLEMENTED BY CURRENT DRIVER");
       else if (lresult == DEVESC_ERROR)
       {
          strcat(esccode_message, ": CAUSED DEVESC_ERROR WITH CURRENT DRIVER");
          CLEANUP("DevEscape()", EntryName);
       }
       else
          strcat(esccode_message, ": invalid return value");
       WorldWriteString(ptl.x, ptl.y, esccode_message);
       ptl.y -= syPosition;
    }

    EXIT_MAIN(EntryName);
  }
  return;
}  /* end of function DevEscSupportListApp()  */

/***************************************************************************/
/*****************************************************************************
 *\ddd
 * Routine Name: DevQueryCapsExh
 *
 * Purpose: This is an entry point for the PTT that exhaustively tests
 *          the DevQueryCaps function.
 *
 *
 * System Requirements:
 *
 *
 * Revision Log: 01/28/90 ,PDVT, Darin C. Hafer
 *                 -Test Case Design and Implementation
 *               08/30/91: Copied from GpiFont.c
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *
 *  ENTER_MAIN
 *  CLEANUP
 *  WorldWriteString
 *  DeviceIsDisplay
 *  DeviceIsPrinter
 *  EXIT_MAIN
 *
 * Limitations:
 * 9/18/91: This testcase was tested on a PostScript driver and caused
 *          a trap.  I did not attempt to locate the cause of the trap.
 *          - Rickey Brown
 *
 *\\end
 *****************************************************************************/
/*****************************************************************************
 *\\algorithm
 *
 *   For 0 to # of fields returned by DevQueryCaps
 *     1) Call DevQueryCaps to get one field at a time
 *     2) Print name of the field.
 *     3) Print value of the field.
 *   Endfor
 *
 *\\end
 *****************************************************************************/
VOID APIENTRY DevQueryCapsExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static PCHAR EntryName="DevQueryCapsExh";
  POINTL aptlTextBox[TXTBOX_COUNT];
  static HDC    hDC;
  LONG   lValue, lDev, xVal;
  SHORT  sLine;
  BOOL   bNewColumn = TRUE;


  if (SelectionCall)
    WinSendMsg (hWndClient, WM_PTHREAD_DOES_DEVCALLS, NULL, NULL);
  else {
    ENTER_MAIN(EntryName);

    // Initialize coordinates to 3/4 inch from the top of the screen.
    ptl.x = (LONG)(delta_x/5L);
    ptl.y = (LONG) ((pg_size_y * delta_y) - (delta_y*3L/4L));

    // Map the coordinates.
    MapPt(ptl);

    // Get the Device Context handle
    hDC = GpiQueryDevice(hPS);

    if (!DevQueryCaps(hDC, CAPS_DEVICE_FONTS, 1L, &lDev))
      CLEANUP("DevQueryCaps",EntryName);

    for (sLine = 0; sLine < NUMLINES; sLine++) {

       if ((sLine >=  (SHORT) (NUMLINES + 1L) / 2L) && (bNewColumn == TRUE)) {
          ptl.x = (LONG) ((pg_size_x * delta_x) / 2L);
          ptl.y = (LONG) ((pg_size_y * delta_y) - (delta_y*3L/4L));
          MapPt(ptl);
          bNewColumn = FALSE;
       }

        if (!DevQueryCaps(hDC, devcaps[sLine].lIndex, 1L, &lValue))
           CLEANUP("DevQueryCaps", EntryName);

        WorldWriteString(ptl.x, ptl.y, devcaps[sLine].szIdentifier);
        xVal = ptl.x;

        if DeviceIsDisplay() {
          GpiQueryTextBox(hPS, (LONG)strlen("CAPS_FOREGROUND_MIX_SUPPORTWW"),
               "CAPS_FOREGROUND_MIX_SUPPORTWW", TXTBOX_COUNT, aptlTextBox);
          ptl.x += aptlTextBox[TXTBOX_BOTTOMRIGHT].x;

          GpiQueryTextBox(hPS, (LONG) strlen(ltoa(lValue, achLogLine, 10)),
                          achLogLine, TXTBOX_COUNT, aptlTextBox);
          ptl.x -= aptlTextBox[TXTBOX_CONCAT].x;
          WorldWriteString(ptl.x, ptl.y, achLogLine);
        }
        else if DeviceIsPrinter() {
          ptl.x += MapX(300);
          sprintf(achLogLine, "%5ld", lValue);
          WorldWriteString(ptl.x, ptl.y, achLogLine);
        }
        else if (DeviceIsPrinter() && (lDev == 0)) {
          ptl.x += MapX(((pg_size_x * delta_x)/2L) - (delta_x/4L));
          sprintf(achLogLine, "%5ld", lValue);
          WorldWriteString(ptl.x, ptl.y, achLogLine);
        }

        ptl.x = xVal;
        ptl.y -= syPosition;

    } /* End For */


    /********************************************************************
    /*        Return the proper log messages for GPI's.
    /********************************************************************/

    EXIT_MAIN(EntryName);
  } /* End main else */
  return;
} /* End DevQueryCapsExh */

/****************************************************************************
 *\\ddd
 * Routine Name: DevQueryDeviceNamesExh()
 *
 * Purpose:  This function exhaustively tests the function
 *           DevqueryDeviceNames().
 *     The syntax used to call This device function is:
 *
 *     (BOOL) DevQueryDeviceNames( HAB       hAB,
 *                                 PSZ       pszDriverName,
 *                                 PLONG     pldn,
 *                                 PSTR32    aDeviceName,
 *                                 PSTR64    aDeviceDesc,
 *                                 PLONG     pldt,
 *                                 PSTR16    aDataType
 *                               )
 *
 *     This Device function causes a driver to return the names and
 *     descriptions of the devices it supports, and their datatypes.
 *
 *
 *    return value is:
 *       TRUE                    Successful completion
 *       FALSE                   Error occurred
 *
 *
 *    Possible returns from WinGetLastError:
 *       PMERR_INV_LENGTH_OR_COUNT
 *
 * System Requirements:
 *
 * Revision Log:
 *              09/10/91: Initial Creation; Rickey C. Brown
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 *
 * Subroutines Required:
 *     ENTER_MAIN()
 *     MapPt()
 *     CLEANUP()
 *     WorldWriteString()
 *     MapX()
 *     UnMapY()
 *     DeviceIsDisplay()
 *     DeviceIsPrinter()
 *     TEST_SIGNATURE()
 *     EXIT_MAIN()
 *
 * Limitations:
 *        None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *    get the name of the current driver
 *
 *   Make a call with DevQueryDeviceNames() with pldn and pldt set to
 *   zero in order to determine how much storage is necessary.
 *
 *   Make a second call, using info obtained from first call to get and
 *   store the device names, device descriptions, and the datatypes.
 *
 *   Output the information obtained.
 *\\end
 ***************************************************************************/
VOID APIENTRY DevQueryDeviceNamesExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static  PCHAR  EntryName="DevQueryDeviceNamesExh()";
   HAB hAB;
   HDC hDC;
   DEVOPENSTRUC dop;
   DRIVDATA driv;
   LONG lDCType;
   PSTR32 aDeviceName;
   PSTR64 aDeviceDesc;
   PSTR16 aDataType;
   LONG dn, dt;
   LONG i;
   CHAR outstring[128];
   CHAR FullDriverName[64];
   LONG xVal;
   LONG lReturnCode;
   USHORT Version;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_PTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);

//   GET CURRENT DEVICE PARAMETERS
    dop.pdriv = &driv;
    WinSendMsg(hWndClient, WM_GET_DEVOPENSTRUC, MPFROMP(&dop), MPFROMP(&lDCType));

//  Check if any printers are attached ... otherwise we will TRAP
   if (!dop.pszDriverName)
       {
       CWRITELOG(L_HDR, 0, "WARNING: CANNOT TEST DevQueryDeviceNamesExh IF ");
       CWRITELOG(L_HDR, 0, "THERE ARE NOT ANY SELECTABLE PRINTERS ATTACHED!");
       CLEANUP("WinSendMsg()", EntryName);
       EXIT_MAIN(EntryName);
       return;
       }


//  GET ANCHOR BLOCK
   if ((hAB = WinQueryAnchorBlock(hWndClient)) == NULL) {
      CLEANUP("WinQueryAnchorBlock()", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }

//  INITIALIZE COORDINATES TO 3/4" FROM TOP OF SCREEN
   ptl.x = (LONG)(delta_x/5L);
   ptl.y = (LONG)((pg_size_y * delta_y) - (delta_y *3L/4L));

//  MAP THE COORDINATES
   MapPt(ptl);

   sprintf(outstring, "CURRENT DRIVER:  %s", dop.pszDriverName);
   WorldWriteString(ptl.x, ptl.y, outstring);
   ptl.y -= 2*syPosition;

//  GET OS/2 VERSION IN ORDER TO SET CORRECT PATHNAME OF DRIVER
//32   if (DosGetVersion(&Version) != NO_ERROR)
//32   {
//32      CLEANUP("DosGetVersion()", EntryName);
//32      EXIT_MAIN(EntryName);
//32      return;
//32   }

//   SET DRIVER PATHNAME:
// IF THE OS/2 IS VERSION 1.2, PATH IS: C:\OS2\DLL\DriverName.DRV
// OTHERWISE THE PATH IS C:\OS2\DLL\Drivername\DriverName.DRV
// LASERJET PATH IS C:\OS2\DLL\HP\PCL\DriverName...
    strcpy(FullDriverName, "C:\\OS2\\DLL\\");
    if (!strcmpi("LASERJET", dop.pszDriverName))
       strcat(FullDriverName, "HP\\PCL\\");
    strcat(FullDriverName, dop.pszDriverName);

    if ( !(((SHORT)HIBYTE(Version) == 10) && ((SHORT)LOBYTE(Version) == 20)) )
    {
       strcat(FullDriverName, "\\");
       strcat(FullDriverName, dop.pszDriverName);
    }
    strcat(FullDriverName, ".DRV");

//  GET QUANTITY INFO
   dn = 0;
   dt = 0;
   aDeviceName = NULL;
   aDeviceDesc = NULL;
   aDataType = NULL;
   if (DevQueryDeviceNames(hAB, FullDriverName, &dn, aDeviceName,
                            aDeviceDesc, &dt, aDataType) == FALSE ) {
      CLEANUP("DevQueryDeviceNames()1", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }

//  ALLOCATE SPACE FOR THE ARRAYS
   aDeviceName = (PSTR32)malloc((size_t)(dn*sizeof(STR32)));
   if (aDeviceName == NULL)
   {
      CWRITELOG(L_HDR, 0, "1:malloc returned NULL in DevQueryDeviceNamesExh");
      EXIT_MAIN(EntryName);
      return;
   }
   aDeviceDesc = (PSTR64)malloc((size_t)(dn*sizeof(STR64)));
   if (aDeviceDesc == NULL)
   {
      CWRITELOG(L_HDR, 0, "2:malloc returned NULL in DevQueryDeviceNamesExh");
      EXIT_MAIN(EntryName);
      return;
   }
   aDataType = (PSTR16)malloc((size_t)(dt*sizeof(STR16)));
   if (aDataType == NULL)
   {
      CWRITELOG(L_HDR, 0, "3:malloc returned NULL in DevQueryDeviceNamesExh");
      EXIT_MAIN(EntryName);
      return;
   }

//  GET INFO VIA DEVQUERYDEVICENAMES() CALL
   if (!DevQueryDeviceNames(hAB, FullDriverName, &dn, aDeviceName,
                            aDeviceDesc, &dt, aDataType) ) {
      CLEANUP("DevQueryDeviceNames()2", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }

//  OUTPUT THE INFORMATION
   WorldWriteString(ptl.x, ptl.y, "DATA TYPES SUPPORTED:");
   ptl.y -= syPosition;
   for (i =0; i < dt; i++) {
      WorldWriteString(ptl.x, ptl.y, (CHAR *)aDataType);
      ptl.y -= syPosition;
      if (i != (dt-1))
         aDataType++;
   }
   ptl.y -= syPosition;
   WorldWriteString(ptl.x, ptl.y, "DEVICE NAME");
   xVal = ptl.x;
   ptl.x = MapX(400);
   WorldWriteString(ptl.x, ptl.y, "DEVICE DESCRIPTION");
   ptl.y -= 2*syPosition;

   for (i =0; i < dn; i++) {
      ptl.x = xVal;
      sprintf(outstring, "%s", aDeviceName);
      WorldWriteString(ptl.x, ptl.y, outstring);
      ptl.x = MapX(400);
      sprintf(outstring, "%s", aDeviceDesc);
      WorldWriteString(ptl.x, ptl.y, outstring);
      if (i != (dn-1))
      {
         aDeviceName++;
         aDeviceDesc++;
      }
      ptl.y -= syPosition;
//  IF OUTPUT IS TO SCREEN, OUTPUT ONLY UNTIL 3/4" FROM BOTTOM OF SCREEN
      if ((UnMapY(ptl.y) <= 75L) && DeviceIsDisplay())
         return;
//  IF OUTPUT IS TO PRINTER AND BOTTOM OF PAGE IS REACHED, GO TO NEW PAGE
      if ((UnMapY(ptl.y) <= 50L) && DeviceIsPrinter()) {
         if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR) {
            CLEANUP("GpiQueryDevice", EntryName);
            EXIT_MAIN(EntryName);
            return;
         }

         if ((lReturnCode = DevEscape(hDC, DEVESC_NEWFRAME,
                                   0L, NULL, 0L, NULL)) == DEVESC_ERROR)
           CLEANUP("DevEscape", EntryName);

         TEST_SIGNATURE(0,0);
         ptl.y = MapY((LONG) (pg_size_y) * delta_y);
         ptl.y -= 2*syPosition;
         ptl.x = xVal;
         WorldWriteString(ptl.x, ptl.y, "DEVICE NAME");
         xVal = ptl.x;
         ptl.x = MapX(400);
         WorldWriteString(ptl.x, ptl.y, "DEVICE DESCRIPTION");
         ptl.y -= 2*syPosition;
      }
   }  /* END OF FOR-LOOP  */

   if (aDeviceName != NULL)
      free(aDeviceName);
   if (aDeviceDesc != NULL)
      free(aDeviceDesc);
   if (aDataType != NULL)
      free(aDataType);

    EXIT_MAIN(EntryName);

  }
  return;
}  /* end of function DevQueryDeviceNamesExh()  */

/*****************************************************************************
 *\ddd
 * Routine Name: DevQueryHardcopyCapsExh
 *
 * Purpose: This is an entry point for the PTT that exhaustively tests
 *          the DevQueryHardcopyCaps function.
 *
 *
 * System Requirements:
 *
 *
 * Revision Log:    9/9/91: Initial creation - Rickey C. Brown
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    SelectionCall  // selection parameter
 *
 * Outputs:
 *  NONE
 *
 * Subroutines Required:
 *
 *  ENTER_MAIN
 *  CLEANUP
 *  MapPt
 *  WorldWriteString
 *  UnMapY
 *  DeviceIsDisplay
 *  DeviceIsPrinter
 *  TEST_SIGNATURE
 *  EXIT_MAIN
 *
 * Limitations:
 *
 *\\end
 *****************************************************************************/
/*****************************************************************************
 *\\algorithm
 *
 *   1)  Issue a call with Forms = 0 to get number of forms codes defined
 *   2)  Issue another call to get and store the hardcopy capabilities info
 *   3)  Output the obtained info
 *
 *\\end
 *****************************************************************************/
VOID APIENTRY DevQueryHardcopyCapsExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
  static PCHAR EntryName="DevQueryHardcopyCapsExh";
  HDC    hDC;
  LONG   lForms;
  LONG   lReturnCode;
  PHCINFO phcInfo;
  CHAR    outstring[128];
  CHAR    str1[128];
  LONG    i;


  if (SelectionCall)
    WinSendMsg (hWndClient, WM_PTHREAD_DOES_DEVCALLS, NULL, NULL);
  else {
    ENTER_MAIN(EntryName);

    // Initialize coordinates to 3/4 inch from the top of the screen.
    ptl.x = (LONG)(delta_x/5L);
    ptl.y = (LONG) ((pg_size_y * delta_y) - (delta_y*3L/4L));

    // Map the coordinates.
    MapPt(ptl);

    // Get the Device Context handle
    if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR)
    {
       CLEANUP("GpiQueryDevice()", EntryName);
       EXIT_MAIN(EntryName);
       return;
    }

//  GET NUMBER OF FORMS CODES DEFINED
   if ((lReturnCode = DevQueryHardcopyCaps(hDC, 0L, 0L, (PHCINFO)NULL)) == DQHC_ERROR)
   {
      CLEANUP("DevQueryHardcopyCaps()1", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }

//  NOW ALLOCATE SPACE FOR THE BUFFER
   phcInfo = (PHCINFO)malloc((size_t)(lReturnCode*sizeof(HCINFO)));
   if (phcInfo == NULL)
   {
      CWRITELOG(L_HDR, 0, "malloc returned NULL in DevQueryHardcopyCapsExh");
      EXIT_MAIN(EntryName);
      return;
   }

//  CALL AGAIN TO GET INFO
   lForms = lReturnCode;
   if ((lReturnCode = DevQueryHardcopyCaps(hDC, 0L, lForms, phcInfo)) == DQHC_ERROR)
   {
      CLEANUP("DevQueryHardcopyCaps()2", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }

//  NOW OUTPUT THE INFORMATION
   sprintf(outstring, "Number of forms codes defined: %d", lForms);
   WorldWriteString(ptl.x, ptl.y, outstring);
   ptl.y -= 2*syPosition;

  for (i = 0; i < lForms; i++) {
     sprintf(outstring, "Form name #%d:  ", i+1);
     strcat(outstring, phcInfo[i].szFormname);
     WorldWriteString(ptl.x, ptl.y, outstring);
     ptl.y -= syPosition;
     sprintf(outstring, "Width (mm): %d   ", phcInfo[i].cx);
     sprintf(str1, "Height (mm): %d", phcInfo[i].cy);
     strcat(outstring, str1);
     WorldWriteString(ptl.x, ptl.y, outstring);
     ptl.y -= syPosition;
     sprintf(outstring, "Left clip limit (mm): %d   ", phcInfo[i].xLeftClip);
     sprintf(str1, "Right clip limit (mm): %d", phcInfo[i].xRightClip);
     strcat(outstring, str1);
     WorldWriteString(ptl.x, ptl.y, outstring);
     ptl.y -= syPosition;
     sprintf(outstring, "Bottom clip limit (mm): %d   ", phcInfo[i].yBottomClip);
     sprintf(str1, "Top clip limit (mm): %d", phcInfo[i].yTopClip);
     strcat(outstring, str1);
     WorldWriteString(ptl.x, ptl.y, outstring);
     ptl.y -= syPosition;
     sprintf(outstring, "Pels horizontal: %d   ", phcInfo[i].xPels);
     sprintf(str1, "Pels vertical: %d", phcInfo[i].yPels);
     strcat(outstring, str1);
     WorldWriteString(ptl.x, ptl.y, outstring);
     ptl.y -= syPosition;
     sprintf(outstring, "Attributes of the form identifier: %d", phcInfo[i].flAttributes);
     WorldWriteString(ptl.x, ptl.y, outstring);
     ptl.y -= 2*syPosition;

        // If the output is to the printer and the current position
        // is 1 inch from the bottom of the page, start a new page,
        // print the test signature and reset the y coordinate.
     if ((UnMapY(ptl.y) <= 120L) && DeviceIsPrinter()) {

       if ((lReturnCode = DevEscape(hDC, DEVESC_NEWFRAME,
                                 0L, NULL, 0L, NULL)) == DEVESC_ERROR)
         CLEANUP("DevEscape", EntryName);

       TEST_SIGNATURE(0,0);

       // Initialize coordinates to 3/4 inch from the top of the screen.
       ptl.x = (LONG)(delta_x/5L);
       ptl.y = (LONG) ((pg_size_y * delta_y) - (delta_y*3L/4L));

       // Map the coordinates.
       MapPt(ptl);
     }

  } /* endfor */

    /********************************************************************
    /*        Clean up and return the proper log messages for GPI's.
    /********************************************************************/
    if (phcInfo != NULL)
       free(phcInfo);
    EXIT_MAIN(EntryName);
  } /* End main else */
  return;
} /* End DevQueryHardcopyCapsExh */

/****************************************************************************
 *\\ddd
 * Routine Name: DevPostDeviceModesExh()
 *
 * Purpose:  This function exhaustively tests the function
 *           DevPostDeviceModes().
 *     The syntax used to call this device function is:
 *
 *     (LONG) DevPostDeviceModes ( HAB       hAB,
 *                                 PDRIVDATA pdrivDriverData,
 *                                 PSZ       pszDriverName,
 *                                 PSZ       pszDeviceName,
 *                                 PSZ       pszName,
 *                                 ULONG     ulOptions
 *                               )
 *
 *     This device function returns, and optinally sets, printer and
 *     job properties.
 *
 *
 *    return value is:
 *       DPDM_ERROR              Error
 *       DPDM_NONE               No settable options (if pdrivDriverData is
 *                               NULL) or no device modes (otherwise)
 *       >0                      Size in bytes required for DriverData (if
 *                               pdrivDriverData input is NULL)
 *       DEV_OK                  OK (if pdrivDriverData is not NULL)
 *
 *
 *    Possible returns from WinGetLastError:
 *       PMERR_INV_DRIVER_DATA
 *       PMERR_DRIVER_NOT_FOUND
 *       PMERR_INV_DEVICE_NAME
 *       PMERR_INV_LOGICAL_ADDRESS
 *
 * System Requirements:
 *
 * Revision Log:
 *              09/16/91: Initial Creation; Rickey C. Brown
 * Inputs:
 *        hPS             -presentation space.
 *        hWndClient    -client window handle
 *        SelectionCall  -selection indicator.
 *
 * Outputs:
 *        None
 *
 * Subroutines Required:
 *     ENTER_MAIN()
 *     DeletePrinterOutput()
 *     CLEANUP()
 *     EXIT_MAIN()
 *
 * Limitations:
 *        None at this time.
 *\\end
 ***************************************************************************/
 /***************************************************************************
 *\\algorithm
 *
 *   A) Get all of the necessary parameters from OS2.INI using
 *      PrfQueryProfileSize() and PrfQueryProfileString().
 *   B) Call DevPostDeviceModes() with NULL data pointer to find out how much
 *      storage is needed.
 *   C) Allocate necessary storage.
 *   D) Call DevPostDeviceModes() again to get information; call with different
 *      options parameters to see that all possibilities work correctly.
 *
 *   NOTE: THERE IS NO PRINTED OUTPUT - THIS TEST DISPLAYS THE JOB PROPERTIES
 *         DIALOG TWICE AND THE PRINTER PROPERTIES DIALOG ONCE.
 *\\end
 ***************************************************************************/
VOID APIENTRY DevPostDeviceModesExh(HPS hPS,HWND hWndClient,BOOL SelectionCall)
{
   static  PCHAR  EntryName="DevPostDeviceModesExh()";
   HAB hAB;
   PSZ    pszDriverName;
   PSZ    pszDeviceName;
   PSZ    pszName;
   PDRIVDATA pdrivDriverData;
   PULONG puDataLength;
   ULONG ulSize;
   PCH pchScan;
   LONG DriverCount;
   PVOID pvProfileString;

  if (SelectionCall) {
    WinSendMsg (hWndClient, WM_PTHREAD_DOES_DEVCALLS, NULL, NULL);
    *pBasisX = 10000;
    *pBasisY = 10000;
  }
  else {

    ENTER_MAIN(EntryName);
    DeletePrinterOutput(EntryName);

//  GET ANCHOR BLOCK
   if ((hAB = WinQueryAnchorBlock(hWndClient)) == NULL) {
      CLEANUP("WinQueryAnchorBlock()", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }


// GET DEFAULT DRIVER NAME, DEVICE NAME, AND LOGICAL DEVICE NAME
// FROM THE OS2.INI FILE.
   puDataLength = (PULONG)malloc((size_t)sizeof(ULONG));
   if (puDataLength == NULL)
   {
      CWRITELOG(L_HDR, 0, "1:malloc returned NULL in DevPostDeviceModesExh");
      EXIT_MAIN(EntryName);
      return;
   }
   if (PrfQueryProfileSize(HINI_PROFILE, (PSZ)"PM_SPOOLER",(PSZ)"PRINTER",
                           puDataLength) == FALSE)
   {
      CLEANUP("PrfQueryProfileSize()", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }
   pvProfileString = (PVOID)malloc((size_t)*puDataLength);
   if (pvProfileString == NULL)
   {
      CWRITELOG(L_HDR, 0, "2:malloc returned NULL in DevPostDeviceModesExh");
      EXIT_MAIN(EntryName);
      return;
   }
   if ( ulSize = PrfQueryProfileString((HINI)HINI_PROFILE, (PSZ)"PM_SPOOLER",
                             (PSZ)"PRINTER", (PSZ)NULL,
                             pvProfileString, (ULONG)*puDataLength))
   {
      *((PSZ)pvProfileString + ulSize-2) = 0;
      pszName = (PSZ)pvProfileString;
      if (PrfQueryProfileSize(HINI_PROFILE, (PSZ)"PM_SPOOLER_PRINTER",
                             pszName, puDataLength) == FALSE)
      {
         CLEANUP("PrfQueryProfileSize()", EntryName);
         EXIT_MAIN(EntryName);
         return;
      }
      if (pvProfileString != NULL)
         free(pvProfileString);
      pvProfileString = (PVOID)malloc((size_t)*puDataLength);
      if (pvProfileString == NULL)
      {
         CWRITELOG(L_HDR, 0, "3:malloc returned NULL in DevPostDeviceModesExh");
         EXIT_MAIN(EntryName);
         return;
      }
      if (ulSize = PrfQueryProfileString((HINI)HINI_PROFILE, (PSZ)"PM_SPOOLER_PRINTER",
                                         pszName, (PSZ)NULL, pvProfileString,
                                         (ULONG)*puDataLength))
      {
         if (puDataLength != NULL)
            free(puDataLength);
         pchScan = strchr(pvProfileString, '.');
         pchScan++;
         *(pchScan + strcspn(pchScan, ".,;")) = 0;
         pszDeviceName = pchScan;
         pchScan = strchr(pvProfileString, ';');
         pchScan++;
         *(pchScan + strcspn(pchScan, ".,;")) = 0;
         pszDriverName = pchScan;

      }
   }
   if (pvProfileString != NULL)
      free(pvProfileString);

//  GET SIZE OF DATA AREA
   if ((DriverCount = DevPostDeviceModes(hAB, (PDRIVDATA)NULL, pszDriverName,
                                        pszDeviceName, pszName,
                                        DPDM_POSTJOBPROP)) == DPDM_ERROR)
   {
      CLEANUP("DevPostDeviceModes1()", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }

//  ALLOCATE SPACE FOR DATA AREA
   pdrivDriverData = (PDRIVDATA)malloc((size_t)DriverCount);
   if (pdrivDriverData == NULL)
   {
      CWRITELOG(L_HDR, 0, "4:malloc returned NULL in DevPostDeviceModesExh");
      EXIT_MAIN(EntryName);
      return;
   }

//  NOW DISPLAY DIALOG FOR JOB PROPERTIES
   if ((DriverCount = DevPostDeviceModes(hAB, pdrivDriverData, pszDriverName,
                                        pszDeviceName, pszName,
                                        DPDM_POSTJOBPROP)) == DPDM_ERROR)
   {
      CLEANUP("DevPostDeviceModes2()", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }
//  DISPLAY DIALOG FOR JOB PROPERTIES WITH pszName = NULL
   if ((DriverCount = DevPostDeviceModes(hAB, pdrivDriverData, pszDriverName,
                                        pszDeviceName, (PSZ)NULL,
                                        DPDM_POSTJOBPROP)) == DPDM_ERROR)
   {
      CLEANUP("DevPostDeviceModes3()", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }

//  NOW DISPLAY DIALOG FOR PRINTER PROPERTIES
   if ((DriverCount = DevPostDeviceModes(hAB, pdrivDriverData, pszDriverName,
                                        pszDeviceName, pszName,
                                        DPDM_CHANGEPROP)) == DPDM_ERROR)
   {
      CLEANUP("DevPostDeviceModes4()", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }

//  NOW TEST THE DPDM_QUERYJOBPROP (NOTHING WILL BE DISPLAYED)
   if ((DriverCount = DevPostDeviceModes(hAB, pdrivDriverData, pszDriverName,
                                        pszDeviceName, pszName,
                                        DPDM_QUERYJOBPROP)) == DPDM_ERROR)
   {
      CLEANUP("DevPostDeviceModes5()", EntryName);
      EXIT_MAIN(EntryName);
      return;
   }
   if (pdrivDriverData != NULL)
      free(pdrivDriverData);


    EXIT_MAIN(EntryName);
  }
  return;
}  /* end of function DevPostDeviceModesExh()   */


/**************************************************************************/
/* The function TestEscCode() is a support function that is called by
 * the entry point DevEscapeExh().
 *
 * Purpose: to test the escape code passed in the parameter "code"
 *
 * NOTE: This function is only called if the device driver supports the escape
 *       code "code".
 *
 *
 *  Revision Log:      Initial creation: Sept. 7, 1991 (Rickey C. Brown)
 *
 * Subroutines required:
 * WorldWriteString()
 * CLEANUP()
 * TEST_SIGNATURE()
 * MapPt()
 * DeviceIsPrinter()
 *
 * Inputs:      HDC   hDC          Device context handle
 *              HPS   hPS          Presentation space handle
 *              HWND  hWndClient   Client window handle
 *              LONG  code         Escape code to be tested
 *
 * Algorithm:
 *   Pseudo-switch statement constructed with "if - else if - else if...
 *   structure to take into account each possible escape code.  Therefore
 *   each call to this function tests only one escape code.  To test the
 *   escape code, DevEscape() is called with that escape code and the user
 *   is informed of the result.
 *
 *  LIMITATIONS:
 *  As of this writing (9/17/91), some of the code has not been completely
 *  tested due to the lack of a device driver that supports the specific
 *  escape codes; in particular:
 *  - DEVESC_GETSCALINGFACTOR
 *  - DEVESC_NEXTBAND
 *  - DEVESC_DRAFTMODE
 *  - DEVESC_BREAK_EXTRA
 *  - DEVESC_CHAR_EXTRA
 *  The last 2 codes, DEVESC_BREAK_EXTRA and DEVESC_CHAR_EXTRA, are supported
 *  only by PostScript drivers.  I attempted to run the test cases in this
 *  dll (DevCalls.dll) on a PostScript driver.  Unfortunately, as of 9/18/91,
 *  this particular test case was one of two that failed on the PostScript
 *  driver; i.e., caused a system trap. (The error log does indicate if these
 *  particular codes were causing trouble.)
 *
 *
 ****************************************************************************/

VOID TestEscCode(HDC hDC, HPS hPS, HWND hWndClient, LONG code)
{
   CHAR *ModuleName = " in TestEscCode";
   CHAR ErrorString[100];
   CHAR message[128];
   CHAR message2[128];
   SFACTORS ScalingFactor;
   LONG OutCount;
   PBYTE OutData;
   double base2 = 2.0;
   SHORT xScale, yScale;
   VIOSIZECOUNT *pVioSizeCount;
   INT i;
   VIOFONTCELLSIZE *pVIOfcs;
   CHAR *RawData = "This is raw data sent directly to driver via DEVESC_RAWDATA,";
   CHAR *RawData1 = "\r\nand this is the end of the testcase DevEscapeExh.";
   CHAR *RawData2 = "\r\n\n\nDon't expect a test signature at the bottom of the page.";
   BANDRECT NextBand;
   USHORT jobID;
   CHAR *DocName = "TestSTARTDOC";
   CHAR *NewDocName = "TestDEVESCs";
   SHORT DMindata;   /* buffer for DEVESC_DRAFTMODE  */
   ESCMODE EscSetMode;
   CODEPAGEINFO CodePageInfo;
   FIXED ExtraSpacing;
   PFONTMETRICS pfmMetrics;
   SHORT sBrChar;
   BOOL bSuccess;
   ULONG  CodePage;

/*********  DEVESC_QUERYESCSUPPORT  *************/
   if (code == DEVESC_QUERYESCSUPPORT)
   {
      strcpy(message, "--> otherwise this test would be useless.");
      WorldWriteString(ptl.x, ptl.y, message);
   }
/*********  DEVESC_GETSCALINGFACTOR  *************/
   else if (code == DEVESC_GETSCALINGFACTOR)
   {
      OutCount = (LONG)sizeof(ScalingFactor);
      if ((DevEscape(hDC, DEVESC_GETSCALINGFACTOR, 0L, (PBYTE)NULL, &OutCount,
                             (PBYTE)&ScalingFactor)) == DEV_OK)
      {
         xScale = (SHORT) (1 << ScalingFactor.x);
         yScale = (SHORT) (1 << ScalingFactor.y);

         // xScale = (SHORT)pow(base2, (double)(ScalingFactor.x));
         // yScale = (SHORT)pow(base2, (double)(ScalingFactor.y));
         strcpy(message,  "x-scaling factor: ");
         sprintf(message+strlen(message), "%d", xScale);
         strcat(message, "    y-scaling factor: ");
         sprintf(message+strlen(message), "%d", yScale);
         WorldWriteString(ptl.x, ptl.y, message);
      }
      else
      {
         strcpy(ErrorString, "DEVESC_GETSCALINGFACTOR");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
   }
/*********  DEVESC_STARTDOC  *************/
   else if (code == DEVESC_STARTDOC)
   {
      strcpy(message, "To test DEVESC_STARTDOC, we'll close the current document and open a new one.");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "(Thus we will also test DEVESC_ENDDOC - 2 for the price of one!)");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "This will be the end of this document - thus the last line on this page.");
      WorldWriteString(ptl.x, ptl.y, message);
      //  FIRST CLOSE THIS DOC
      OutCount = 2;
      if ((DevEscape(hDC, DEVESC_ENDDOC, 0L,(PBYTE)NULL,&OutCount,(PBYTE)&jobID)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_ENDDOC");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
         return;
      }
      // NOW START A NEW DOCUMENT
      if ((DevEscape(hDC, DEVESC_STARTDOC, 13L,(PBYTE)DocName,(PLONG)NULL,(PBYTE)NULL)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_STARTDOC");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
         return;
      }
      strcpy(message, "This should be the 1st line of a new document after testing DEVESC_STARTDOC.");
      WorldWriteString(ptl.x, ptl.y, message);

   }
/*********  DEVESC_ENDDOC  *************/
   else if (code == DEVESC_ENDDOC)
   {
      strcpy(message, "DEVESC_ENDDOC was already tested in conjunction with DEVESC_STARTDOC.");
      WorldWriteString(ptl.x, ptl.y, message);
   }
/*********  DEVESC_ABORTDOC  *************/
   else if (code == DEVESC_ABORTDOC)
   {
      strcpy(message, "To test DEVESC_ABORTDOC, we'll close the current document, open a new one,");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "and then we will send an ABORTDOC escape code.  Of course, after that we will");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "start a new document in order to continue the DEVESC testing.");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "This will be the end of this document - thus the last line on this page.");
      WorldWriteString(ptl.x, ptl.y, message);
      TEST_SIGNATURE(0,0);
      //  FIRST CLOSE THIS DOC
      OutCount = 2;
      if ((DevEscape(hDC, DEVESC_ENDDOC, 0L,(PBYTE)NULL,&OutCount,(PBYTE)&jobID)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_ENDDOC2");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
         return;
      }
      // NOW START A NEW DOCUMENT, IN ORDER TO ABORT IT
      if ((DevEscape(hDC, DEVESC_STARTDOC, 12L,(PBYTE)NewDocName,(PLONG)NULL,(PBYTE)NULL)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_STARTDOC2");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
         return;
      }
      if ((DevEscape(hDC, DEVESC_ABORTDOC, 0L,(PBYTE)NULL,(PLONG)NULL,(PBYTE)NULL)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_ABORTDOC");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      else
      {
         // NOW WE NEED TO START A NEW DOCUMENT
         if ((DevEscape(hDC, DEVESC_STARTDOC, 12L,(PBYTE)NewDocName,(PLONG)NULL,(PBYTE)NULL)) == DEVESC_ERROR)
         {
            strcpy(ErrorString, "DEVESC_STARTDOC3");
            strcat(ErrorString, ModuleName);
            CLEANUP("DevEscape", ErrorString);
         }
         strcpy(message, "This should be the 1st line of a new document after testing DEVESC_ABORTDOC.");
         WorldWriteString(ptl.x, ptl.y, message);
      }
   }
/*********  DEVESC_NEXTBAND  *************/
   else if (code == DEVESC_NEXTBAND)
   {
      OutCount = (LONG)sizeof(NextBand);
      if ((DevEscape(hDC, DEVESC_NEXTBAND, 0L, (PBYTE)NULL,&OutCount,(PBYTE)&NextBand)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_NEXTBAND");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      else
      {
     //  Initialize coordinates to 1/5" from left & 3/4" from top of next band
         ptl.x = NextBand.xleft + (LONG)(delta_x/5L);
         ptl.y = NextBand.ytop - (delta_y *3L/4L);

     //  Map the coordinates
         MapPt(ptl);

         strcpy(message, "This line should be at the beginning of the next band.");
         WorldWriteString(ptl.x, ptl.y, message);
         ptl.y -= syPosition;
         strcpy(message, "Here are the coordinates of this band:");
         WorldWriteString(ptl.x, ptl.y, message);
         ptl.y -= syPosition;
         sprintf(message, "xleft/ybottom coordinates: (%d, ", NextBand.xleft);
         sprintf(message2, "%d)", NextBand.ybottom);
         strcat(message, message2);
         WorldWriteString(ptl.x, ptl.y, message);
         ptl.y -= syPosition;
         sprintf(message, "xright/ytop coordinates: (%d, ", NextBand.xright);
         sprintf(message2, "%d)", NextBand.ytop);
         strcat(message, message2);
         WorldWriteString(ptl.x, ptl.y, message);
         NBand = TRUE;
      }
   }

/*********  DEVESC_NEWFRAME  *************/
   else if (code == DEVESC_NEWFRAME)
   {
      if DeviceIsPrinter()
      {
        strcpy(message, "This should be the LAST LINE printed on this page.");
        WorldWriteString(ptl.x, ptl.y, message);
        TEST_SIGNATURE(0,0);
        if ((DevEscape(hDC, DEVESC_NEWFRAME, 0L, (PBYTE)NULL,(PLONG)NULL,(PBYTE)NULL)) == DEVESC_ERROR)
        {
           strcpy(ErrorString, "DEVESC_NEWFRAME");
           strcat(ErrorString, ModuleName);
           CLEANUP("DevEscape", ErrorString);
        }
        ptl.y -= syPosition;
        strcpy(message, "This should be the FIRST LINE printed on this page after testing DEVESC_NEWFRAME.");
        WorldWriteString(ptl.x, ptl.y, message);
      }
   }

/*********  DEVESC_DRAFTMODE  *************/
   else if (code == DEVESC_DRAFTMODE)
   {
      // TURN DRAFTMODE ON
      strcpy(message, "Turning the DRAFTMODE on...");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      DMindata = 1;
      if ((DevEscape(hDC, DEVESC_DRAFTMODE, (LONG)sizeof(DMindata), (PBYTE)&DMindata,(PLONG)NULL,(PBYTE)NULL)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_DRAFTMODE ON");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
         return;
      }
      // TURN DRAFTMODE OFF
      strcpy(message, "Now turning the DRAFTMODE off...");
      WorldWriteString(ptl.x, ptl.y, message);
      DMindata = 0;
      if ((DevEscape(hDC, DEVESC_DRAFTMODE, (LONG)sizeof(DMindata), (PBYTE)&DMindata,(PLONG)NULL,(PBYTE)NULL)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_DRAFTMODE OFF");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
   }
/*********  DEVESC_FLUSHOUTPUT  *************/
   else if (code == DEVESC_FLUSHOUTPUT)
   {
      strcpy(message, "We will test DEVESC_FLUSHOUTPUT by issuing a NEWFRAME escape code;");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "Then we will send something to the buffer, and then flush it out.");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "This should be the last line of data on this page.  Now flushing...");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      TEST_SIGNATURE(0,0);
      if ((DevEscape(hDC, DEVESC_NEWFRAME, 0L, (PBYTE)NULL,(PLONG)NULL,(PBYTE)NULL)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_NEWFRAME");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
         return;
      }
      strcpy(message, "This better not be the first line on the new page.");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "This is line 1 of 5 in FLUSHOUTPUT test");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "This is line 2 of 5 in FLUSHOUTPUT test");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "This is line 3 of 5 in FLUSHOUTPUT test");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "This is line 4 of 5 in FLUSHOUTPUT test");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "This is line 5 of 5 in FLUSHOUTPUT test");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      strcpy(message, "If you see these FLUSHOUTPUT testlines, then it ain't flushing!!");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;

      if ((DevEscape(hDC, DEVESC_FLUSHOUTPUT, 0L, (PBYTE)NULL, (PLONG)NULL,(PBYTE)NULL)) == DEV_OK)
         strcpy(message, "FIRST LINE ON NEW PAGE: Device buffer has been flushed after a NEWFRAME.");
      else
      {
         strcpy(message, "ERROR: Device buffer not flushed as expected!!!");
         strcpy(ErrorString, "DEVESC_FLUSHOUTPUT");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      WorldWriteString(ptl.x, ptl.y, message);
   }
/*********  DEVESC_RAWDATA  *************/
   else if (code == DEVESC_RAWDATA)
   {
      strcpy(message, "Raw data will be printed starting on the next page...");
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      TEST_SIGNATURE(0,0);
      if ((DevEscape(hDC, DEVESC_NEWFRAME, 0L, (PBYTE)NULL,(PLONG)NULL,(PBYTE)NULL)) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_NEWFRAME");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
         return;
      }
      if ((DevEscape(hDC, DEVESC_RAWDATA, 61L, RawData, (PLONG)NULL, (PBYTE)NULL)) == DEVESC_ERROR)
      {
         WorldWriteString(ptl.x, ptl.y, "ERROR IN DEVESC_RAWDATA");
         strcpy(ErrorString, "DEVESC_RAWDATA");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      if ((DevEscape(hDC, DEVESC_RAWDATA, 52L, RawData1, (PLONG)NULL, (PBYTE)NULL)) == DEVESC_ERROR)
      {
         WorldWriteString(ptl.x, ptl.y, "ERROR IN DEVESC_RAWDATA");
         strcpy(ErrorString, "DEVESC_RAWDATA");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      if ((DevEscape(hDC, DEVESC_RAWDATA, 61L, RawData2, (PLONG)NULL, (PBYTE)NULL)) == DEVESC_ERROR)
      {
         WorldWriteString(ptl.x, ptl.y, "ERROR IN DEVESC_RAWDATA");
         strcpy(ErrorString, "DEVESC_RAWDATA");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
   }
/*********  DEVESC_QUERYVIOCELLSIZES  *************/
   else if (code == DEVESC_QUERYVIOCELLSIZES)
   {

      //  INITIALIZE OUTPUT PARAMETERS
      OutData = (PBYTE)malloc(100*sizeof(LONG));
      if (OutData == NULL)
      {
         CWRITELOG(L_HDR, 0, "malloc returned NULL in TestEscCode()");
         ErrorCount++;
         return;
      }
      OutCount = 100*sizeof(LONG);

      if ((DevEscape(hDC, DEVESC_QUERYVIOCELLSIZES, 0L, (PBYTE)NULL,
                            &OutCount, OutData ) ) == DEV_OK)
      {
        if (OutCount)
        {
           pVioSizeCount = (VIOSIZECOUNT *)OutData;
           strcpy(message, "Max number of VIO cell sizes supported: ");
           sprintf(message+strlen(message), "%d", pVioSizeCount->maxcount);
           WorldWriteString(ptl.x, ptl.y, message);
           ptl.y -= syPosition;
           strcpy(message, "Number of VIO cell sizes returned from DevEscape: ");
           sprintf(message+strlen(message), "%d", pVioSizeCount->count);
           WorldWriteString(ptl.x, ptl.y, message);
           ptl.y -= syPosition;
           // GET POINTER TO FIRST VIOFONTCELLSIZE STRUCTURE
           pVIOfcs = (VIOFONTCELLSIZE *)(pVioSizeCount + 1);

           for (i = 1; i <= (INT)(pVioSizeCount->count); i++) {
              sprintf(message, "%d:  ", i);
              strcat(message, "cell width: ");
              sprintf(message+strlen(message), "%d", pVIOfcs->cx);
              strcat(message, "   cell height: ");
              sprintf(message+strlen(message), "%d", pVIOfcs->cy);
              WorldWriteString(ptl.x, ptl.y, message);
              pVIOfcs++;
              ptl.y -= syPosition;
           }
        }
        else
        {
           strcpy(message, "Nothing returned by DEVESC_QUERYVIOCELLSIZES");
           WorldWriteString(ptl.x, ptl.y, message);
        }
      }
      else
      {
         strcpy(message, "ERROR returned by call with DEVESC_QUERYVIOCELLSIZES");
         WorldWriteString(ptl.x, ptl.y, message);
         strcpy(ErrorString, "DEVESC_QUERYVIOCELLSIZES");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      if (OutData != NULL)
         free(OutData);

   }
/*********  DEVESC_BREAK_EXTRA  *************/
   else if (code == DEVESC_BREAK_EXTRA)
   {
      //ONLY SUPPORTED ON DRIVERS THAT SUPPORT POSTSCRIPT

      //GET BREAK CHARACTER OF CURRENTLY SELECTED LOGICAL FONT
      if ((bSuccess = GpiQueryFontMetrics(hPS, (LONG)sizeof(pfmMetrics),
                                           pfmMetrics)) == FALSE)
      {
         strcpy(ErrorString, "GpiQueryFontMetrics");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
         return;
      }
      sBrChar = pfmMetrics->sBreakChar;
      ExtraSpacing = 10*FIXEDTOLONG;
      if ((DevEscape(hDC, DEVESC_BREAK_EXTRA, (LONG)sizeof(ExtraSpacing),
                          (PBYTE)&ExtraSpacing, (PLONG)NULL, (PBYTE)NULL)) == DEVESC_ERROR)
      {
         WorldWriteString(ptl.x, ptl.y, "ERROR IN DEVESC_BREAK_EXTRA");
         strcpy(ErrorString, "DEVESC_BREAK_EXTRA1");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      else
      {
         //OUTPUT MESSAGE CONTAINING BREAK CHARACTERS
         sprintf(message, "Testing %c DEVESC_BREAK_EXTRA with", sBrChar);
         sprintf(message2, "Break %c character extra spacing = +10", sBrChar);
         strcat(message, message2);
         WorldWriteString(ptl.x, ptl.y, message);
         ptl.y -= syPosition;
      }
      if ((DevEscape(hDC, DEVESC_BREAK_EXTRA, 0L,
                          (PBYTE)&ExtraSpacing, (PLONG)NULL, (PBYTE)NULL)) == DEVESC_ERROR)
      {
         WorldWriteString(ptl.x, ptl.y, "ERROR IN DEVESC_BREAK_EXTRA");
         strcpy(ErrorString, "DEVESC_BREAK_EXTRA2");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      else
      {
         sprintf(message, "Testing %c DEVESC_BREAK_EXTRA with", sBrChar);
         sprintf(message2, "Break %c character extra spacing = +10", sBrChar);
         strcat(message, message2);
         WorldWriteString(ptl.x, ptl.y, message);
      }
   }
/*********  DEVESC_CHAR_EXTRA  *************/
   else if (code == DEVESC_CHAR_EXTRA)
   {
      //ONLY SUPPORTED ON DRIVERS THAT SUPPORT POSTSCRIPT

      ExtraSpacing = 10*FIXEDTOLONG;
      if ((DevEscape(hDC, DEVESC_CHAR_EXTRA, (LONG)sizeof(ExtraSpacing),
                          (PBYTE)&ExtraSpacing, (PLONG)NULL, (PBYTE)NULL)) == DEVESC_ERROR)
      {
         WorldWriteString(ptl.x, ptl.y, "ERROR IN DEVESC_CHAR_EXTRA");
         strcpy(ErrorString, "DEVESC_CHAR_EXTRA1");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      else
      {
         strcpy(message, "Testing DEVESC_CHAR_EXTRA: spacing = spacing + 10");
         WorldWriteString(ptl.x, ptl.y, message);
         ptl.y -= syPosition;
      }
      if ((DevEscape(hDC, DEVESC_CHAR_EXTRA, 0L,
                          (PBYTE)&ExtraSpacing, (PLONG)NULL, (PBYTE)NULL)) == DEVESC_ERROR)
      {
         WorldWriteString(ptl.x, ptl.y, "ERROR IN DEVESC_CHAR_EXTRA");
         strcpy(ErrorString, "DEVESC_CHAR_EXTRA2");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      else
      {
         strcpy(message, "Testing DEVESC_CHAR_EXTRA: spacing back to normal");
         WorldWriteString(ptl.x, ptl.y, message);
      }
   }
/*********  DEVESC_SETMODE  *************/
   else if (code == DEVESC_SETMODE)
   {
      // SAVE ORIGINAL CODEPAGE SETTING
      CodePage = GpiQueryCp(hPS);
      sprintf(message, "Current code page is %d", CodePage);
      WorldWriteString(ptl.x, ptl.y, message);
      ptl.y -= syPosition;
      // NOW SET THE CODEPAGE USING DEVESC_SETMODE
      EscSetMode.mode = 0L;
      if (CodePage == 437)
         CodePageInfo.idCodePage = 850;
      else
         CodePageInfo.idCodePage = 437;
      if ((DevEscape(hDC, DEVESC_SETMODE, (LONG)sizeof(EscSetMode),
                     (PBYTE)&EscSetMode, (PLONG)NULL, (PBYTE)NULL) ) == DEVESC_ERROR)
      {
         strcpy(ErrorString, "DEVESC_SETMODE");
         strcat(ErrorString, ModuleName);
         CLEANUP("DevEscape", ErrorString);
      }
      else
      {
         sprintf(message, "Code page changed to %d using DEVESC_SETMODE.", CodePageInfo.idCodePage);
         WorldWriteString(ptl.x, ptl.y, message);
         ptl.y -= syPosition;

         // NOW RESET THE CODEPAGE TO THE ORIGINAL SETTING
         CodePageInfo.idCodePage = (USHORT) CodePage;
         if ((DevEscape(hDC, DEVESC_SETMODE, (LONG)sizeof(EscSetMode),
                        (PBYTE)&EscSetMode, (PLONG)NULL, (PBYTE)NULL) ) == DEVESC_ERROR)
         {
            strcpy(ErrorString, "DEVESC_SETMODE");
            strcat(ErrorString, ModuleName);
            CLEANUP("DevEscape", ErrorString);
         }
         else
         {
         sprintf(message, "Code page changed back to %d using DEVESC_SETMODE.", CodePageInfo.idCodePage);
         WorldWriteString(ptl.x, ptl.y, message);
         }
      }
   }
}  /* end of function TestEscCode()  */

