/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#define INCL_DEV      // include all DEV definitions
#define INCL_GPI      // include all Gpi definitions
#define INCL_WIN
#define INCL_DOSERRORS
#include <string.h>
#include <search.h>
#include <stdlib.h>
#include <os2.h>
#include <stdio.h>
#include <time.h>
#include <ctype.h>
#include "PTTDLLDF.h"           // test case defines
#include "common.h"             // format specifiers for logging

/****************************************************************************
 *\\ddd
 * Subtask Name: Graphics Engine Test;
 *
 * Module name: COMMON.C
 *
 * Purpose: This source file contains the function needed to log errors
 *          and functions that are needed to select a font that is
 *          used to draw text on current Device Context.
 *
 * Revision Log:
 *               01/22/91 ,PDVT ,Mike Abuzant and David MCcombs
 *               -Initial file.
 *               02/19/91 ,PDVT ,Mike Abuzant
 *               - Wrote functions to select and create logical
 *                 fonts.
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments throughout code.
 *
 * SAD References:  None.
 *
 * SRS References: Printer Driver Verification Tool
 *
 * Limitations:
 *  None at This time.
 *
 *\\end
****************************************************************************/

/* Global Variables */

static  LONG lCurrentPSId=0L,       //Logical Font ID.
             lHorzFontRes=0L,
             lVertFontRes=0L,
             lPelsPerInch=0L,
             lMyFontMatchNumber=0x0FFFF,
             AveFontCharWidth=0L;

static  BOOL FontIsFound=FALSE;     //Indicates if a font is found.

static  CHAR MyFontFaceName[34],    //Matched font name
         MyFontFamilyname  [34],    //Matched font family name.
         ieStorString      [100];
static CHAR ieStorage[150];

static  USHORT    usMyFontPointSize=0x0FFF,
                  aPointSize[]={
                                 8,
                                 10,
                                 12,
                                 14,
                                 18,
                                 24
                               };


LONG   pg_width, pg_height;
LONG   dev_height, dev_width, hori_pels_per_m, vert_pels_per_m;
LONG   pg_size_x, pg_size_y;
LONG   x_units_per_inch, y_units_per_inch;
LONG   delta_x,delta_y;
double CommonXMult;
double CommonYMult;


VOID AbortEntry(HWND hWndClient, CHAR * EntryName, CHAR * Setting)
{
 CHAR  achLogLine[200];

    sprintf(achLogLine, "Test:       %s\n"
                        "Invalidity: %s\n"
                        "Test was skipped due to invalid environment.",
                         EntryName, Setting);
    LOGINFO(L_HDR, "@", achLogLine);

 return;
}


/***************************************************************************
 *\\ddd
 * Routine Name: CleanUp()
 *
 * Purpose:  This function logs a message indicating that a Gpi or
 *            Gre function has failed. It additionally logs an exit
 *                       entry point notification if fExit is True.
 *
 *
 * System Requirements: None
 *
 * Revision Log: 01/23/91, PDVT, Mike Abuzant
 *               -initial creation
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 * Inputs:
 *   hPS            // presentation space
 *   hWndClient     //client window handle
 *   Function       // Function that failed.
 *   ExitPoint      // test case Entry point
 *   ErrorCount
 *   Filename       // file name where error occurred.
 *   LineNo         // Line number where error occurred.
 * Outputs:
 *   ErrorCount
 *
 * Subroutines Required:
 *       ReportSetupError()          //Defined in PTT.
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   Increment the error count.
 *   call ReportSetupError() to log the error information.
 *              if fExit is TRUE {
 *   Print a message to LOG FILE to indicate that this entry point will be
 *   terminated.
 *   }
 * }
 *\\end
 ***************************************************************************/
VOID CleanUp(CHAR *Function,HWND hWndClient,CHAR *ExitPoint,PLONG ErrorCount,
             PSZ szFileName,USHORT LineNo,BOOL fExit)
{
  CHAR  achLogLine[200];

  ++*ErrorCount;
  (VOID)ReportSetupError(hWndClient,Function ,szFileName,LineNo);
  if (fExit){
    CWRITELOG( L_LOTRACE, 0, CLEANUP_SPEC );
    sprintf( achLogLine, EXIT_SPEC, ExitPoint );
    CWRITELOG( L_LOTRACE, 0, achLogLine );
  }
  return;
}

/***************************************************************************
 *\\ddd
 * Routine Name: ExitMain()
 *
 * Purpose:  This function logs the pass/fail status
 *                               and the number of failures within the entry point.
 *                               It additionally logs the end of the entry point.
 *
 *
 * System Requirements: None
 *
 * Revision Log: 01/23/91, PDVT, Mike Abuzant
 *               -initial creation
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 * Inputs:
 *   hPS            // presentation space
 *   hWndClient     // client window handle
 *   ExitPoint      // test case Entry point
 *   ErrorCount     // Error count.
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *   None.
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   if (ErrorCount==0)
 *      log a PASS message
 *   else {
 *                issue a LOG_SIGNATURE command,
 *      log the Error Count and
 *      log a FAIL message.
 *        }
 *        log the exit entry point message.
 *
 *\\end
 ***************************************************************************/

VOID ExitMain( CHAR * ExitPoint, HWND hWndClient, LONG ErrorCount )
{
  CHAR  achLogLine[200];

  if (!ErrorCount) {
     sprintf( achLogLine, PASS_SPEC, ExitPoint );
  }
  else {
     LOG_SIGNATURE;
          sprintf( achLogLine, ERRORC_SPEC , ErrorCount );
     CWRITELOG(L_HDR, 0,achLogLine);
     sprintf( achLogLine, FAIL_SPEC, ExitPoint );
  }
  CWRITELOG( L_HDR, 0, achLogLine );
  sprintf( achLogLine, EXIT_SPEC , ExitPoint );
  CWRITELOG( L_LOTRACE, 0, achLogLine );
  return;

}

/***************************************************************************
 *\\ddd
 * Routine Name: EnterMain()
 *
 * Purpose:  This function will call functions to initialize variables
 *                               that are used within this entry point.
 *           This function will also select the font to use for drawing
 *           text on current Device Context.
 *
 * System Requirements: None
 *
 * Revision Log: 01/23/91, PDVT, Mike Abuzant
 *               -initial creation
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 * Inputs:
 *   hPS            // presentation space
 *   hWndClient     // client window handle
 *   EntryPoint     // test case Entry point
 *   ErrorCount     // Error count.
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *   set_delta()
 *   InitDefaultPrintFont()
 *   CreateCourFont()
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *         set_delta();
 *    ErrorCount=0;
 *    Select a font by calling InitDefaultPrintFont().
 *    Create a logical font .
 *    TEST_SIGNATURE(0,0);
 *              log an test entry point message.
 *
 * }
 *\\end
 ***************************************************************************/
#define RESOLUTION(Value)  (.0254*(Value)+.50)

VOID EnterMain(HPS hPS, HWND hWndClient, CHAR * EntryPoint, PLONG ErrorCount)
 {
  CHAR  achLogLine[200];
/* CHAR  DateBuffer[20],
   TimeBuffer[20];  */
  FONTMETRICS     fm;
  HDC             hDC;
  double          xPixels,
                  yPixels;

   //Get Device Context associated with current PS.
   if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
      REPORTSETUPERROR("GpiQueryDevice()");
      return ;      //Can't continue without this DC.
   }

   // Had to Do this step since PTT does not retrieve all
   // the device CAPS.
   if(DevQueryCaps (hDC, 0L,40L, alDevCaps)==FALSE)
       REPORTSETUPERROR ("DevQueryCaps()");

   (VOID)set_delta(hPS);
   FontIsFound=FALSE;                     //A good font not found yet.?!
   *ErrorCount=0L;
   lCurrentPSId=0;

   // Calculate the pixels per inch in X-direction
   xPixels=(double)RESOLUTION(hori_pels_per_m);

   // Calculate the pixels per inch in Y-direction
   yPixels=(double)RESOLUTION(vert_pels_per_m);

   if DeviceIsDisplay(){
                                          //Current Device is Screen
       FontIsFound=FALSE;                //Fonts are O.K for Screen,
                                         //Lets' not change it.
   }
   else {                                //Current Device is a printer.
      if ((lCurrentPSId=GpiSavePS(hPS))==GPI_ERROR){
           REPORTSETUPERROR ("GpiSavePS()");
           return;
      }
      FontIsFound=InitDefaultPrintFont(hPS,hWndClient);
      if (FontIsFound && CreateCourFont(hPS,DESIRED_FONT_ID)){
        if (GpiSetCharSet(hPS,DESIRED_FONT_ID)==FALSE){
             REPORTSETUPERROR ("GpiSetCharSet()");
             FontIsFound=FALSE;
        }
      }
      else{
         FontIsFound=FALSE;
      }
   }

   TEST_SIGNATURE(0,0);
        ErrorStatus(hWndClient, *ErrorCount, FALSE, NULL, NULL);

#if 1==2
        sprintf(achLogLine,",Resl %03d X %03d ,Date %s,Time %s",
           (SHORT)xPixels,(SHORT)yPixels,_strdate(DateBuffer),
           _strtime(TimeBuffer));

   GpiCharString(hPS,(LONG)strlen(achLogLine),achLogLine);

#endif

   sprintf( achLogLine, ENTRY_SPEC, EntryPoint );
   CWRITELOG( L_LOTRACE, 0, achLogLine );

   //Determine the char height of the selected font. This will be
   //used for placing text on the display.
   if (GpiQueryFontMetrics(hPS,(LONG)sizeof(fm),&fm )==FALSE){
      REPORTSETUPERROR("GpiQueryFontMetrics()");
   }
   syPosition=(SHORT)fm.lMaxBaselineExt;

   if (lCurrentPSId){
      if(GpiRestorePS(hPS,lCurrentPSId)==GPI_ERROR){
         REPORTSETUPERROR ("GpiRestorePS()");
      }
   }

   if (FontIsFound && (CreateCourFont(hPS,DESIRED_FONT_ID)==GPI_ERROR)){
       REPORTSETUPERROR ("GpiCreateLogFont()");
       FontIsFound=FALSE;
   }

   return;
}
/***************************************************************************
 *\\ddd
 * Routine Name: WriteString()
 *
 * Purpose:  Output a text string using GpiCharStringAt in a standard font.
 *
 * System Requirements: None
 *
 * Revision Log: 02/20/91, PDVT, Mike Abuzant
 *               -initial creation
 *               -This function has been taken from std_util.c and
 *                modified.
 * Inputs:
 *
 *   hPS        // presentation space handle
 *   x          // x coordinate
 *   y          // y coordinate
 *   String    // output string
 *
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *     CreateCourFont().
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *  ptl.x = x
 *  ptl.y = y
 *
 *  If we were successful in selecting a font for Current Device.
 *    Save current Presentation Space attributes.
 *    Create a logical font, call CreateCourFont().
 *    Print the text using GpiCharStringAt()
 *    Restore Presentation Space attributes.
 *
 *  else
 *    Print the text using GpiCharStringAt().
 *
 * }
 *\\end
 ***************************************************************************/
VOID WriteString(HWND hWndClient,HPS hPS, LONG x, LONG y, CHAR *String)
{
  POINTL ptl;
  CHARBUNDLE  Bundle;
  ptl.x = x;
  ptl.y = y;

  if (GpiQueryAttrs(hPS,PRIM_CHAR,CBB_BACK_MIX_MODE | CBB_MIX_MODE |
                       CBB_COLOR | CBB_BACK_COLOR,&Bundle) == FALSE){
    REPORTSETUPERROR ("GpiQueryAttrs()");
  }

  if ((lCurrentPSId=GpiSavePS(hPS))==GPI_ERROR){
    REPORTSETUPERROR ("GpiSavePS()");
  }

  if (GpiResetPS(hPS,GRES_SEGMENTS)==FALSE){
    REPORTSETUPERROR ("GpiResetPS()");
  }

  if (GpiSetAttrs(hPS,PRIM_CHAR,CBB_BACK_MIX_MODE | CBB_MIX_MODE |
                       CBB_COLOR | CBB_BACK_COLOR,0L,&Bundle) == FALSE){
    REPORTSETUPERROR ("GpiSetAttrs()");
  }

  if (FontIsFound && (GpiSetCharSet(hPS,DESIRED_FONT_ID)==FALSE)){                     // Font is Selected successfully.
    REPORTSETUPERROR ("GpiSetCharSet()");
  }

  GpiCharStringAt (hPS, &ptl, (LONG) strlen(String), String);

  if(GpiRestorePS(hPS,lCurrentPSId)==GPI_ERROR){
    REPORTSETUPERROR ("GpiRestorePS()");
  }

 return;
}

/***************************************************************************
 *\\ddd
 * Routine Name: CheckRc()
 *
 * Purpose: Output a PASS or FAIL message to indicate the status of
 *          testing a return code.  Log the error message status
 *                              to the ptt log.
 *
 * System Requirements: None
 *
 * Revision Log: 01/24/91, PDVT, Mike Abuzant
 *               -initial creation
 * Input:
 *
 *   hPS            //Presentation space
 *   hWndClient     //Client window handle
 *   EntryPoint     //Test case Entry point
 *   ReturnCode     //Gpi or Gre return code.
 *   ErrorToCheck   //Tested return code.
 *   message1       //a message
 *   message2       //a message
 *   szFileName     // .c file name
 *   LineNo         // location of the gpi /gre in .c file
 *
 * Outputs:
 *    0  If successfull in producing error code.
 *    1  If unsuccessfull in producing error code.
 *
 * Subroutines Required:
 *    VerifyReturnedErrorCode()
 *
 * Limitations:
 *   None at this time.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *              Concatenate messages.
 *    Call VerifyReturnedErrorCode() to check return code.
 * }
 *\\end
 ***************************************************************************/

LONG CheckRc(HWND hWndClient,LONG ReturnCode,USHORT ErrorToCheck,
             CHAR *Message1,CHAR *Message2,PSZ szFileName,USHORT LineNo)
{
  CHAR  achLogLine[200];

  strcpy(achLogLine,Message1);
  if (Message2){
     strcat(achLogLine," ");
     strcat(achLogLine,Message2);
  }
  return (LONG) VerifyReturnedErrorCode(hWndClient,ReturnCode,
                achLogLine,SEVERITY_ERROR,ErrorToCheck,szFileName,LineNo);

}
/***************************************************************************
 *\\ddd
 * Routine Name: CheckRc1()
 *
 * Purpose: Out a PASS or FAIL message to indicate the status of
 *          testing a return code.
 *
 * System Requirements: None
 *
 * Revision Log: 10/15/91, Rickey C. Brown
 *               -initial creation
 * NOTE: This function is a virtual copy of CheckRc() above, with the exception
 *       that it calls VerifyReturnCode() instead of VerifyReturnedErrorCode().
 *       This was needed specifically for testing the function GpiQueryPel(),
 *       in which a NULL return code does not constitute an error.  The
 *       corresponding #define statement is CHECKRC1. For more information,
 *       see DEBUGGING NOTE in GpiBtMpr.c, entrypoint GpiQueryPelRc.
 * Input:
 *
 *   hPS            //Presentation space
 *   hWndClient     //Client window handle
 *   EntryPoint     //Test case Entry point
 *   ReturnCode     //Gpi or Gre return code.
 *   ErrorToCheck   //Tested return code.
 *   message1       //a message
 *   message2       //a message
 *   szFileName     // .c file name
 *   LineNo         // location of the gpi /gre in .c file
 *
 * Outputs:
 *    0  If successful in producing error code.
 *    1  If unsuccessful in producing error code.
 *
 * Subroutines Required:
 *    VerifyReturnedErrorCode()
 *
 * Limitations:
 *   None at this time.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *
 *    Call VerifyReturnedErrorCode() check return code.
 * }
 *\\end

 ***************************************************************************/

LONG CheckRc1(HWND hWndClient,LONG ReturnCode,USHORT ErrorToCheck,
             CHAR *Message1,CHAR *Message2,PSZ szFileName,USHORT LineNo)
{
  CHAR  achLogLine[200];

  strcpy(achLogLine,Message1);
  if (Message2){
     strcat(achLogLine," ");
     strcat(achLogLine,Message2);
  }
  return (LONG) VerifyReturnCode(hWndClient,ReturnCode,
                achLogLine,SEVERITY_ERROR,ErrorToCheck,szFileName,LineNo);

}


/***************************************************************************
 *\\ddd
 * Routine Name: SubEnter()
 *
 * Purpose:  This function logs an Item Entry point message.
 *
 *
 *
 * System Requirements: None
 *
 * Revision Log: 01/23/91, PDVT, Mike Abuzant
 *               -initial creation
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 * Inputs:
 *   hWndClient     //client window handle
 *   ErrorToCheck   // Indicate which error message we testing for.
 *   addedmessga   // Additional message.
 *
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *  None.
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   Print a Message to the log file signaling that the current item
 *        sub-entry     point is about to start.
 * }
 *\\end
 ***************************************************************************/
VOID SubEnter(HWND hWndClient, CHAR * errortocheck, CHAR * addedmessage)
{
 CHAR  achLogLine[200];

   strcpy(ieStorString,errortocheck);
        if (addedmessage){
     strcat(ieStorString," ");
     strcat(ieStorString,addedmessage);
   }
        sprintf(achLogLine,IENTRY_SPEC,ieStorString);
        CWRITELOG(L_LOTRACE,0,achLogLine);
        return;

}
/***************************************************************************
 *\\ddd
 * Routine Name: SubExit()
 *
 * Purpose:  This function will log test item information before
 *           exiting the current item sub-entry point.
 *
 * System Requirements: None
 *
 * Revision Log: 01/23/91, PDVT, Mike Abuzant
 *               -initial creation
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 * Inputs:
 *   hWndClient     //client window handle
 *
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *   None.
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   Print a Message to the log file signaling that this current item
          sub-entry     point is terminating.
 * }
 *\\end
 ***************************************************************************/
VOID SubExit(HWND hWndClient)
{
 CHAR  achLogLine[200];

        sprintf( achLogLine, IEXIT_SPEC, ieStorString );
   CWRITELOG(L_LOTRACE,0,achLogLine);
   return;

}

 /***************************************************************************
 *\\ddd
 * Routine name:  CreateCourFont()
 *
 * Purpose:  This is a function to create logical font .
 *
 * System Requirements:  None
 *
 * Revision Log:  01/16/91, PDVT, Kim Cairns
 *                - modified EzfCreateLogFont to create standard font
 *                02/20/91 ,PDVT ,Mike Abuzant
 *                -modified version from Phase I
 *
 * Inputs:     hPS            - presentation space handle
 *             lcid           - font id
 *
 * Outputs:    The lcid of the selected font.
 *
 * Subroutines Required: None
 *
 * Limitations: None
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *   set FATTRS value from global variables set by InitDefaultPrintFont().
 *   GpiCreateLogFont for desired criteria and return lcid
 *
 *\\end
 ***************************************************************************/
static LONG CreateCourFont(HPS hPS, LONG lcid)
{
  static FATTRS FontAttributes;

  //
  // Use values set in InitDefaultPrintFont()
  //

  memset((VOID *)&FontAttributes,0x00,sizeof(FontAttributes));
  FontAttributes.lMatch = lMyFontMatchNumber;
  FontAttributes.usRecordLength = sizeof(FontAttributes);
  strcpy(FontAttributes.szFacename,MyFontFaceName);

  return GpiCreateLogFont(hPS, NULL, lcid, &FontAttributes);
}


 /***************************************************************************
 *\\ddd
 * Routine name:  InitDefaultPrintFont()
 *
 * Purpose:  This is used to select a font that is used to draw text
 *           on current device surface.
 *
 * System Requirements:  None
 *
 * Revision Log:  02/20/92, PDVT, Mike Abuzant
 *                - initial function design
 *
 *
 * Inputs:
 *             hPS            - presentation space handle
 *
 * Outputs:
 *             TRUE if useable font(s) available
 *             FALSE if Error Occured  within this function or
 *             if it could not find any fonts.
 *
 * Subroutines Required:
 *             None
 *
 * Limitations:
 *     "Helv" font must be installed using the CONTROL PANEL.
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *      if the Current Device has built-in fonts ,
 *
 *        Select        a font that is compatible with the current device resolution.
 *
 *    if a font is found
 *       try to find another font with the same attributes but
 *       smaller point size. This is needed since the text on printers
 *       always seems to be longer,in the X-Direction, than the screen.
 *      endif
 *
 *      return success or failure indicator.
 *
 *      endif
 *
 *
 *      if the Current Device has NO built-in fonts ,
 *
 *        Select        a system font that is compatible with the current device resolution.
 *
 *    if a font is found
 *       try to find another font with the same attributes but
 *       larger point size since the system fonts are designed
 *       for a device that has low resolution.
 *      endif
 *
 *      return success or failure indicator.
 *
 *      endif
 *
 *
 *
 *\\end
 ***************************************************************************/
static BOOL InitDefaultPrintFont(HPS hPS,HWND hWndClient)
{
  HDC              hDC;

  PFONTMETRICS     pfm=(PFONTMETRICS)NULL,FontPtr;
  LONG             Requested=0L,
                   Number;

  INT              Index,
                   Index1;
  USHORT           PointSize;
  UINT             uiNumber;
  BOOL             fDeviceHasFonts=TRUE,
                   FontIsFound=FALSE;
  PCHAR            FontNameToLookFor=(PCHAR)NULL;
  double           Increment;

    //Get device context associated of  Presentation Space.
    if ((hDC=GpiQueryDevice(hPS))==HDC_ERROR){
       REPORTSETUPERROR ("GpiQueryDevice()");
       return FALSE;
    }

    // Obtain the width and the height of the current device
    lHorzFontRes=alDevCaps[CAPS_HORIZONTAL_FONT_RES];
    lVertFontRes=alDevCaps[CAPS_VERTICAL_FONT_RES];

    lPelsPerInch=MapX(delta_x);  // # of pels per hori. inch
    AveFontCharWidth = 0x0fffL;                    // initialize to high value
    PointSize=8;
    Increment=((double)lHorzFontRes/67);           //67 is the scrren resolution.
    PointSize=(USHORT)(Increment*2.0+8.0);

    for (Index=0;Index <5; Index++){
        if (PointSize >= aPointSize[Index] &&
            PointSize <= aPointSize[Index+1]){
                Index++;
                break;
        }
    }

    PointSize=aPointSize[Index]*(USHORT)10;

    if (GpiResetPS(hPS,GRES_ALL)==FALSE){
         REPORTSETUPERROR("GpiResetPS()");
         return FALSE;
    }

    //alDevCaps[] is initialized By PTT.EXE.

    fDeviceHasFonts=(BOOL)((Number=alDevCaps[CAPS_DEVICE_FONTS])>0L);


    FontNameToLookFor=fDeviceHasFonts ? (PCHAR)NULL : (PCHAR)"Helv";

    if (!fDeviceHasFonts &&((Number=GpiQueryFonts(hPS, QF_PUBLIC,
                            FontNameToLookFor, &Requested, (LONG) NULL,
                            NULL))==GPI_ALTERROR)){
          REPORTSETUPERROR("GpiQueryFonts()");
          return FALSE;
    }


    if ((Number==0L) || ((pfm=(PFONTMETRICS)malloc((SHORT)Number*
                                  sizeof(FONTMETRICS)))==NULL)){
         return FALSE;
    }

    if ((GpiQueryFonts(hPS,QF_PUBLIC,FontNameToLookFor,
                    &Number,(LONG)sizeof(FONTMETRICS),pfm))==GPI_ALTERROR){
         REPORTSETUPERROR("GreDeviceQueryFonts()");
         free(pfm);
         return FALSE;
    }

    uiNumber=(USHORT)Number;

    for (Index1 = 0 ; Index1 < 2 ; Index1++)
        {

        if (fDeviceHasFonts)
            {
            FontPtr=(PFONTMETRICS)lfind((CHAR *)&usMyFontPointSize,
                                (CHAR *)pfm,&uiNumber,
                                sizeof(FONTMETRICS),
                                FontResolutionFunction);
            FontPtr=(PFONTMETRICS)NULL;
            while (!FontPtr && usMyFontPointSize)
                {
                usMyFontPointSize--;
                FontPtr=(PFONTMETRICS)lfind((CHAR *)&usMyFontPointSize,
                                    (CHAR *)pfm,&uiNumber,
                                    sizeof(FONTMETRICS),
                                    DevicePointSizeFunction);
                }
            break;
            }
        else
            FontPtr=(PFONTMETRICS)lfind((CHAR *)&PointSize,
                                 (CHAR *)pfm,&uiNumber,
                                 (USHORT)sizeof(FONTMETRICS),
                                 PointSizeFunction);

        if (FontPtr)
            {
            if (FontPtr->sXDeviceRes >(SHORT)lHorzFontRes)
                {
                Index = (Index == (0 ? Index : (Index-1)));
                }
            else
                {
                if (Increment < 2.0)
                    Index = (Index == (5 ? Index : (Index+1)));
                else if (Increment > 2.0 && Increment < 4.0)
                    Index += 2;
                else
                    Index += 3;

                }

            Index = (SHORT) (Index > 5 ? 5 : (Index));
            PointSize=aPointSize[Index]*(USHORT)10;
            }
        }

    if (FontPtr){
        lMyFontMatchNumber = FontPtr->lMatch;
        AveFontCharWidth = FontPtr->lAveCharWidth;
        strcpy (MyFontFaceName, FontPtr->szFacename);
        free(pfm);
      return TRUE;
    }

    free(pfm);
    return fDeviceHasFonts;
}

/**********************************************************************
   The next three function are used as arguments to the C run-time
   function lfind(). They are the comparison functions that help
   find the desired font.

   1. PointSizeFunction() : Used to find the System font that
                            matches the specified point size.

   2. DevicePointSizeFunction() : Used to find the Device font that
                            matches the specified point size.

   3. FontResolutionFunction() : Used to find the Device font that
                            matches the device current resolution.

***********************************************************************/
static INT _Optlink PointSizeFunction(const USHORT *PointSize ,const  PFONTMETRICS pfm)
{
             //Normal Font.
      return pfm->fsSelection ? TRUE :
                                    *PointSize-pfm->sNominalPointSize;
}

static INT _Optlink DevicePointSizeFunction(const USHORT *PointSize ,const PFONTMETRICS pfm)
{
  if (pfm->sXDeviceRes == (SHORT) lHorzFontRes &&
      pfm->sYDeviceRes == (SHORT) lVertFontRes &&
      *PointSize == (USHORT) pfm->sNominalPointSize &&
      stricmp(pfm->szFamilyname,MyFontFamilyname)==0){
        lMyFontMatchNumber = pfm->lMatch;
        AveFontCharWidth = pfm->lAveCharWidth;
        strcpy (MyFontFaceName, pfm->szFacename);
        return *PointSize-pfm->sNominalPointSize;
  }
  else {
    return TRUE;
  }
}

static INT _Optlink FontResolutionFunction(const USHORT *Dummy ,const  PFONTMETRICS pfm)
{
  *Dummy;
  if (pfm->sXDeviceRes <= (SHORT) lHorzFontRes &&
      pfm->sYDeviceRes <= (SHORT) lVertFontRes &&
                 pfm->fsSelection == 0x00        ){   //Normal Font.
      //
      //  AveCharWidth closer to 1/10in than previous value
      //
      if (abs((SHORT) (pfm->lAveCharWidth - lPelsPerInch/10))
          < abs((SHORT) (AveFontCharWidth - lPelsPerInch/10))){
          //
          // Set values to create logical font with
          //

         lMyFontMatchNumber = pfm->lMatch;
         AveFontCharWidth = pfm->lAveCharWidth;
         usMyFontPointSize=pfm->sNominalPointSize;
         strcpy (MyFontFaceName, pfm->szFacename);
         strcpy (MyFontFamilyname, pfm->szFamilyname);
      } /* endif */
  } /* endif */

  return TRUE;

}


 /***************************************************************************
 *\\ddd
 * Routine name:  set_delta()
 *
 * Purpose:
 *            This is the initialization routine that sets the following
 *            global variables for use in formatting output to the proper
 *            screen or printer location.  Sets:
 *                 dev_width  - device width
 *                 dev_height - device height
 *                 hori_pels_per_m- horizontal resolution
 *                 vert_pels_per_m- vertical resolution
 *                 pg_size_x      - horizontal page size in smallest whole inch
 *                 pg_size_y      - vertical page size in smallest whole inch
 *
 *
 * System Requirements:  None
 *
 * Revision Log:  11/28/90, PDVT, Kim Cairns
 *                    - initial module completion
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 *
 * Inputs:
 *            hPS                 - presentation space handle
 *
 * Outputs:
 *            hDC                 - device context handle
 *
 * Subroutines Required:
 *            None
 *
 * Limitations:
 *            None
 *
 *\\end
 ***************************************************************************/
/***************************************************************************
 *\\algorithm
 *
 *  use DevQueryCaps() to get:
 *    dev_width
 *    dev_height
 *    hori_pels_per_m
 *    vert_pels_per_m
 *  and calculate:
 *    pg_size_x
 *    pg_size_y
 *  use GpiQueryPS() to get:
 *    pg_width
 *    pg_height
 *    CommonXMult=0.000254*pg_width *hori_pels_per_m;
 *    CommonYMult=0.000254*pg_height *vert_pels_per_m;
 *
 *
 *\\end
 ***************************************************************************/

HDC set_delta(HPS hPS)
{
    ULONG  flOptions;
    SIZEL  sizlPS;
    HDC    hDC;

         hDC = GpiQueryDevice(hPS);

         dev_width=alDevCaps[CAPS_WIDTH];

         dev_height=alDevCaps[CAPS_HEIGHT];

         hori_pels_per_m=alDevCaps[CAPS_HORIZONTAL_RESOLUTION];

         vert_pels_per_m=alDevCaps[CAPS_VERTICAL_RESOLUTION];

    // page size in whole inches, first get resolution of device
    x_units_per_inch=alDevCaps[CAPS_HORIZONTAL_FONT_RES];
    y_units_per_inch=alDevCaps[CAPS_VERTICAL_FONT_RES];

    // if device is a display don't count on font resolution being accurate
    if DeviceIsPrinter(){
      pg_size_x = dev_width / x_units_per_inch;
      pg_size_y = dev_height / y_units_per_inch;
    }
    else {
      pg_size_x = (LONG)( ( (double)dev_width * 10000.0 ) /
                         ( (double)hori_pels_per_m * 254.0 ) );
      pg_size_y = (LONG)( ( (double)dev_height * 10000.0 ) /
                         ( (double)vert_pels_per_m * 254.0 ) );
    }

    // get presentation space size
    flOptions = GpiQueryPS(hPS, &sizlPS);
    pg_width = sizlPS.cx;
    pg_height = sizlPS.cy;

    CommonXMult=0.000254*pg_width *hori_pels_per_m;
    CommonYMult=0.000254*pg_height *vert_pels_per_m;

    delta_x=100;
    delta_y=100;

         return(hDC);                                                                                    // device context handle
}


VOID gpi_grid (HPS hPS, LONG grid_type)
{
  POINTL ptPoint;           // variable to move around in page space with
  LONG i,                   // looping variable
       num_lines,           // number of lines to draw between edges
       last_x,              // value in page coord. of last vertical line
       last_y;              // value in page coord. of last horizontal line
  double factor;            // mutiplication factor to apply to index

  /***********************************************************************
   *  This draws the grid used by most of the GPI Precision tests.
   ***********************************************************************/

  // This function sets the line type
     GpiSetLineType(hPS, LINETYPE_SOLID);

  // calculate last horizontal position to draw a line. This is necessary
  // for devices that are exact multiples of an inch wide as zero is the
  // first pixel/page-space-coord, so moving exactly so many inches would
  // place the last line one coordinate past the visible range.
  last_x=MapX(pg_size_x*delta_x);
  if ( last_x >= pg_width )
  {
    // may want to add some sort of logging here !!!!!!!!!!!!!
    last_x = pg_width - 1;    // set to last visible index in page
  }

  // do the same for the last vertical position
  last_y=MapY(pg_size_y*delta_y);
  if ( last_y >= pg_height )
  {
    // may want to add some sort of logging here !!!!!!!!!!!!!
    last_y = pg_height - 1;   // set to last visible index in page
  }

  // Draw first vertical line on left of page
  ptPoint.x = 0L;
  ptPoint.y = 0L;
  GpiMove (hPS, &ptPoint);
  ptPoint.y = last_y;
  GpiLine (hPS, &ptPoint);

  // Draw last vertical line on right of page
  ptPoint.x = last_x;
  ptPoint.y = 0L;
  GpiMove (hPS, &ptPoint);
  ptPoint.y = last_y;
  GpiLine (hPS, &ptPoint);

  // grid_type = HALF_INCH (1L) then factor = 0.5
  // grid_type = ONE_INCH (2L) then factor = 1.0
  // grid_type = TWO_INCH (4L) then factor = 2.0
  // use this to calculate actual position for line number (i)
  factor = (double)(grid_type) / 2.0;
  // calculate number of vertical lines to draw, not counting left edge.
  num_lines = pg_size_x * 2L / grid_type;

  // Draw the rest of the vertical lines, don't draw last as already done.
  for (i = 1; i < num_lines; i ++)
  {
    ptPoint.y = 0L;
    ptPoint.x=MapX(factor*i*delta_x);
    GpiMove(hPS, &ptPoint);
    ptPoint.y = last_y;
    GpiLine(hPS, &ptPoint);
  } /* End the FOR */

  // Draw first horizontal line at bottom of page
  ptPoint.x = 0L;
  ptPoint.y = 0L;
  GpiMove (hPS, &ptPoint);
  ptPoint.x = last_x;
  GpiLine (hPS, &ptPoint);

  // Draw last horizontal line at top of page
  ptPoint.x = 0L;
  ptPoint.y = last_y;
  GpiMove (hPS, &ptPoint);
  ptPoint.x = last_x;
  GpiLine (hPS, &ptPoint);

  // calculate number of vertical line to draw not counting bottom line
  num_lines = pg_size_y * 2L / grid_type;

  // Draw the rest of the horizontal lines, don't draw last as already done
  for (i = 1; i < num_lines; i++)
  {
    ptPoint.x = 0L;
    ptPoint.y=MapY(i* factor*delta_y);
    GpiMove(hPS, &ptPoint);
    ptPoint.x = last_x;
    GpiLine(hPS, &ptPoint);
  } /* End the FOR */

}  /* End gpi_grid */


/***************************************************************************
 *\\ddd
 * Routine Name: SetEnter()
 *
 * Purpose:  This function indicates the beginning of setup to test.
 *
 * System Requirements: None
 *
 * Revision Log: 03/04/91, PDVT, D.K. McCombs
 *               -initial creation
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 * Inputs:
 *   hWndClient     //client window handle
 *   ErrorToCheck   // Indicate which error message we testing for.
 *   addedmessga   // Additional message.
 *
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *  None.
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   Print a Message to the log file indicating Set Up functions
 *   Are to follow.
 * }
 *\\end
 ***************************************************************************/
VOID SetEnter(HWND hWndClient, CHAR * text1, CHAR * text2)
{
 CHAR  achLogLine[200];

   strcpy(ieStorage, text1);
        if (text2){
     strcat(ieStorage," ");
     strcat(ieStorage, text2);
   }
        sprintf(achLogLine,SET_SPEC,ieStorage);
        CWRITELOG(L_LOTRACE,0,achLogLine);
        return;

}
/***************************************************************************
 *\\ddd
 * Routine Name: SetExit()
 *
 * Purpose:  This function will print needed information before
 *           exiting the current setup entry point.
 *
 * System Requirements: None
 *
 * Revision Log: 03/04/91, PDVT, D.K. McCombs
 *               -initial creation
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 * Inputs:
 *   hWndClient     //client window handle
 *
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *   None.
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   Print a Message indicating that Set Up functions have completed.
 * }
 *\\end
 ***************************************************************************/

VOID SetExit(HWND hWndClient)
{
 CHAR  achLogLine[200];

        sprintf( achLogLine, SET_EXIT_SPEC, ieStorage );
   CWRITELOG(L_LOTRACE,0,achLogLine);
   return;

}




/***************************************************************************
 *\\ddd
 * Routine Name: ErrStatus()
 *
 * Purpose:  This function will indicate error status of API under test.
 *
 * System Requirements: None
 *
 * Revision Log: 03/04/91, PDVT, D.K. McCombs
 *                                        - initial creation
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 * Inputs:
 *   hWndClient     // client window handle.
 *   bFailOption    // option to check for pass / fail.
 *   text1, text2         // two text strings passed to log.
 *
 * Outputs:
 *   Success indication.
 *
 * Subroutines Required:
 *  None.
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *       Initialize a static variable "lComparison" for comparison to errorcount
 *       Build the text string from two strings and a space.
 *       If no text then syncronize lComparison to ErrorCount and return true.
 *  or else
 *      If the failoption is TRUE:
 *                      if Errorcount is not equal to lComparison:
 *                      set bPassFail to FALSE and log failure.
 *                      or else
 *                              set bPassFail to TRUE and log pass.
 *      If the failoption is FALSE:
 *              log an informational message with a * in front of it.
 *  Set lComparison to ErrorCount.
 *       Return API bPassfail variable.
 *
 *  LOGGING LEVEL IS LEVEL 7, L_TRACE.
 *
 * }
 *\\end
 ***************************************************************************/
BOOL ErrorStatus(HWND hWndClient, LONG  ErrorCount, BOOL bFailOption, CHAR * text1, CHAR * text2)
{

        CHAR  LogLine[100];
        CHAR  LogLine2[100];
        static LONG lComparison = 0;
        BOOL bPassFail;
        LONG ErrNum;

   if ( text1 )
                {

                strcpy( LogLine2, text1);
                if (text2)
                        {
                strcat( LogLine2," ");
                strcat( LogLine2,text2);
                }               // end if text2

                if ( bFailOption )
                        {                                                                                                       //check for failure option,

                        if (lComparison != ErrorCount)
                                {                                                                                               // Failed: log as failed
                                bPassFail = TRUE;                                                       // Set return to TRUE.
                                ErrNum = WinGetLastError( *phabPThread );
                                sprintf( LogLine,ERR_FAIL_SPEC, LogLine2,ErrNum);
                                }  // end if

                        else
                                {                                                                                                 // Pass: log as pass
                                bPassFail = FALSE;                                                // Set return FALSE
                                sprintf( LogLine,ERR_PASS_SPEC, LogLine2,ErrNum);
                                }       // end else

                        CWRITELOG(L_TRACE,0, LogLine);

                        }       //      end if (text1or2)

                else
                        {                                                                                               // information logging.
                        bPassFail = TRUE;
                        sprintf( LogLine,STAT_INFO_SPEC, LogLine2);
                        CWRITELOG(L_TRACE, 0,  LogLine);
                        }       // end else

                } // end if bFailOption

        else
                {
                bPassFail = FALSE;
                }                               // end else no text

   lComparison = ErrorCount;
        return (bPassFail) ;

}

/***************************************************************************
 *\\ddd
 * Routine Name: LogInfo()
 *
 * Purpose:  This function will print information to ptt_log
 *           In formatted paragraph form.
 *                       LOGGING LEVEL IS VARIABLE.
 *
 *
 *
 * System Requirements: None
 *
 * Revision Log: 03/04/91, PDVT, D.K. McCombs
 *               -initial creation
 *                                        07/18/91, GrEngineTest, David McCombs
 *                                        - Updated comments.
 * Inputs:
 *   hWndClient     //client window handle
 *
 * Outputs:
 *   None.
 *
 * Subroutines Required:
 *   CWRITELOG().
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   Print information to ptt_log in formatted paragraph form.
 *   LOGGING LEVEL IS VARIABLE.
 *
 *              while (TextToLog character is not \0) {
 *                      Put the characters in the ptest substring array;
 *                      On each white space, mark the position as the last
 *                              white space encountered;
 *                      if (newline character encountered,
 *                                or max line size is reached ) {
 *                              if ( no newline regardless of max line size) {
 *                                      if (a last white space exists) {
 *                                              Move MainVar to last WhitePos as
 *                                                      old text ending point and new text starting point;
 *                                              Add an EOT to substring to mark end for sprintf;
 *                                              sprintf the substring into LogEntry buffer;
 *                                              }
 *                                      else {
 *                                              Change any possible EOL to EOT;
 *                                              sprintf the substring into LogEntry buffer;
 *                                              }
 *                                      CWRITELOG(LogEntry buffer);
 *                                      }
 *                              MainVar++;  // Increment MainVar
 *                              SubVar++;   // Increment Subvar
 *
 *                              }
 *                      sprintf substring left when \0 is encountered into LogEntry buffer;
 *                      CWRITELOG(LogOption, 0, LogEntry);
 *                      }
 *
 * }
 *\\end
 ***************************************************************************/

VOID LogInfo(HWND hWndClient, SHORT LogOption, CHAR * TypeFlag, CHAR  * TextToLog )
{
        CHAR LogEntry[100];
        CHAR ptest[100];
        LONG MainVar, SubVar, WhitePos, Maxsize, LineNum;
        WhitePos = 0L;
        SubVar = 0L;
        MainVar = 0L;
        LineNum = 0L;
        Maxsize = 50L;

  /* Get characters sequentially till \0 is encountered. */

        while ( TextToLog[MainVar] )
                {

                /*
                        Put the characters in the ptest substring
                */

                ptest[SubVar] = TextToLog[MainVar];
                ptest[SubVar+1]='\0';                         // always attach EOT.

                /*
                        On each white space, mark position as the last
                        white space encountered.
                */

                if (isspace(ptest[SubVar])) WhitePos = SubVar;

                /*
                        Check for newline characters
                        and also check for a max line   size reached condition.
                */

                if ( ptest[SubVar] == '\n' || SubVar == Maxsize )
                        {
                /*
                        If no newline but there is a white space then
                                 print from beginning of substring to WhitePos.
                */
                        if (WhitePos && ptest[SubVar] != '\n')
                                {
                                /*
                                        Move MainVar to last WhitePos as
                                        old text ending point and new text starting point
                                */
                                MainVar = MainVar - SubVar + WhitePos;
                                SubVar = WhitePos;
                                ptest[SubVar] = '\0';  // add EOT to mark end for sprintf
                                SubVar = -1L;                     // Reset Subvar.(-1L goes ++ to 0 )
                                sprintf(LogEntry, LOG_SPEC,TypeFlag, ptest);
                                }

                        else
                                {

                                /* Change any possible \n to a \0 */

                                if (ptest[SubVar] == '\n') ptest[SubVar] = '\0';

                                sprintf(LogEntry, LOG_SPEC,TypeFlag, ptest);

                                SubVar = -1L;                    // Reset Subvar.(-1L goes ++ to 0 )

                                }

                        CWRITELOG(LogOption, 0, LogEntry);
                        LineNum++;
                        }

                MainVar++;                                               // Increment MainVar
                SubVar++;                                                // Reset Subvar.(-1L goes ++ to 0 )
                }
        /* End of TextToLog when \0 is encountered is printed */

        sprintf(LogEntry, LOG_SPEC,TypeFlag, ptest);
        CWRITELOG(LogOption, 0, LogEntry);
}




/***************************************************************************
 *\\ddd
 * Routine Name: EzQueryFont
 *
 * Purpose:  The purpose of this function is to Query and store all the
 *           fonts supported By current device. If none is supported,
 *           The Graphic Engine fonts are retreived.
 *
 * System Requirements: NONE
 *
 * Revision Log: 12/18/90, PDVT, Mike Abuzant \
 *               - Function implementation.    \ For GreTxtst.dll
 *               12/24/90, PDVT, Mike Abuzant  /
 *               - Function Design            /
 *
 *               1/3/91, PDVT, Darin C. Hafer \
 *               - Function Re-implementation. \ For GpiFnt1.dll
 *               1/4/91, PDVT, Darin C. Hafer  /
 *               - Function Re-Design         /
 *
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    Count          //Number of font found
 *    FontStorage    // Array of fonts names and match numbers.
 *
 *
 * Outputs:
 *    1. All font facenames and match numbers are returned in the
 *       dynamically allocated array 'FontStorage'.
 *    2. The number of fonts found are returned in 'Count'.
 *
 * Subroutines Required:
 *  None
 *
 * Limitations:
 *
 *
 *\\end
 ***************************************************************************/

/***************************************************************************
 *\\algorithm
 *
 *    // Get the Device Context associated with Presentation Space.
 *      hDC=GpiQueryDevice(hPS) ....
 *
 *    // Get the current device Character horizontal resl.
 *       DevQueryCaps(hDC,CAPS_HORIZONTAL_FONT_RES,1L,&Hori_res)
 *
 *    // Get the current device Character vertical resl.
 *        DevQueryCaps(hDC,CAPS_VERTICAL_FONT_RES,1L,&Vert_res)
 *
 *
 *     Setup condition to query number of fonts supported by current device
 *     by passing a value of zero in the parameter 'Requested'.
 *
 *       Number=GpiQueryFonts(,,,&(Requested=0),,)
 *
 *     Allocate enough storage hold the all the metrics of all fonts.
 *     and return with an error if could not allocate storage.
 *
 *         pfm=malloc(Number*sizeof(fm));
 *
 *     Query all font metrics supported by current device
 *
 *         GpiQueryFonts(hPS,QF_PUBLIC,NULL,&Number,
 *                       (LONG)sizeof(FONTMETRICS), pfm)
 *
 *
 *     // For all the fonts found, return the Facename and Match
 *     // numbers via FontStorage.
 *
 *     FOR (All the fonts queried ) Do
 *
 *          //Allocate storage to new font
 *
 *       increment the font counter.
 *       store the font facename and match number in the array 'FontStorage'
 *
 *     ENDDO
 *
 *\\end
 ***************************************************************************/
BOOL EzQueryFonts(HPS hPS, HWND hWndClient, LONG Enum, PLONG Count ,
                  PTEMPTYPE *FontStorage)
{
  HDC    hDC;
  PFONTMETRICS  pfm=(PFONTMETRICS)NULL;
  LONG          Hori_res,
                Vert_res,
                Returned=0L,
                Requested=0L,
                Number;
  SHORT         Index;

//Get device context associated of Presentation Space.

   if ((hDC = GpiQueryDevice(hPS)) == HDC_ERROR) {
       REPORTSETUPERROR("GpiQueryDevice()");
       return FALSE;
   }

   //Get the device horizontal character resolution.

   if (DevQueryCaps(hDC,CAPS_HORIZONTAL_FONT_RES,1L,&Hori_res) == FALSE) {
        REPORTSETUPERROR("DevQueryCaps()");
        return FALSE;
   }

   //Get the device vertical character resolution.

   if (DevQueryCaps(hDC,CAPS_VERTICAL_FONT_RES,1L,&Vert_res) == FALSE) {
        REPORTSETUPERROR("DevQueryCaps()");
        return FALSE;
   }

   //Get total number of fonts supported by the current device

   if ((Number = GpiQueryFonts(hPS,Enum,NULL,
                               &Requested, (LONG)NULL, NULL)) == GPI_ALTERROR) {
       REPORTSETUPERROR("GpiQueryFonts() # 1");
       return(FALSE);
   }

   // Allocate storage for "Number" fonts

   if ((pfm=(PFONTMETRICS)malloc((SHORT)Number*sizeof(FONTMETRICS)))==NULL){
       REPORTSETUPERROR("malloc()");
       return FALSE;
   }

   // Call GpiQueryFonts again to request "Number" fonts and their
   // metrics to be returned in the space pointed to by "pfm"

   if ((Returned = GpiQueryFonts(hPS,Enum,NULL,
                               &Number,
                               (LONG)sizeof(FONTMETRICS),
                               pfm)) == GPI_ALTERROR){
     REPORTSETUPERROR("GpiQueryFonts() # 2");
     free(pfm);
     return(FALSE);
   }
        *Count = 0;

  // For each font, allocate space for the Facename and Match
  // number and FontStorage will point to it.

        if ((*FontStorage=(PTEMPTYPE)malloc((size_t)Number*sizeof(TEMPTYPE)))
                                                                                                                                                                                        ==NULL){
                REPORTSETUPERROR("malloc()");
                free(pfm);
                return FALSE;
        }

        for(Index = 0; Index < (SHORT)Number ; Index++){

                strcpy((*FontStorage+*Count)->Fontname,pfm[Index].szFacename);
                (*FontStorage+*Count)->lMatch=pfm[Index].lMatch ;
                (*FontStorage+*Count)->fsSelection=pfm[Index].fsSelection;
                (*FontStorage+*Count)->usCodePage=pfm[Index].usCodePage;
                (*FontStorage+*Count)->fsFontUse=(USHORT) NULL;
                if (pfm[Index].fsDefn == FM_DEFN_OUTLINE)
                        (*FontStorage+*Count)->fsFontUse|=FATTR_FONTUSE_OUTLINE;
                if (pfm[Index].fsCapabilities == FM_CAP_NOMIX)
                        (*FontStorage+*Count)->fsFontUse|=FATTR_FONTUSE_NOMIX;
                ++*Count;
        }

  free(pfm);
  return TRUE;
}




/***************************************************************************
 *\\ddd
 * Routine Name: EzCreateLogicalFont()
 *
 * Purpose:  The purpose of this function is to realize a logical font
 *          of a known face name and font matching number.
 *
 * System Requirements: NONE
 *
 * Revision Log: 12/18/90, PDVT, Mike Abuzant \
 *               - Function implementation.    \ For GreTxtst.dll
 *               12/24/90, PDVT, Mike Abuzant  /
 *               - Function Design            /
 *
 *               1/3/91, PDVT, Darin C. Hafer \
 *               - Function Re-implementation. \ For GpiFnt1.dll
 *               1/4/91, PDVT, Darin C. Hafer  /
 *               - Function Re-Design         /
 *
 * Inputs:
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    Lcid           // Local indentifier
 *    fsSelection    // Kind of font needed (Italic ,Bold ...)
 *    FontStorage    // Font name and match number.
 *
 *
 * Outputs:
 *    A new logical font associated with a local identifier (lcid).
 *
 * Subroutines Required:
 *  None
 *
 * Limitations:
 *  None
 *
 *\\end
 ***************************************************************************/

/***************************************************************************
 *\\algorithm
 *
 *    Store the logical font face name, match number,and selection type
 *    into the structure 'fat' that will be passed as a parameter to the
 *    function GpiCreateLogFont().
 *
 *    //Create the desired logical font
 *
 *      GpiCreateLogFont(hDC,Name,lcid,&fat);
 *
 *     return TRUE if the logical font has been created or
 *            FALSE otherwise.
 *\\end
 ***************************************************************************/

LONG EzCreateLogicalFonts(HPS hPS, HWND hWndClient, LONG lcid,
                           TEMPTYPE FontStorage)
{

  static FATTRS   Fattr;
  STR8     Name;
  LONG     ReturnCode;


  // Set up the FATTRS structure with appropriate values
  // that you need to pass to GpiCreateLogFont

    Fattr.usRecordLength = sizeof(Fattr);
    Fattr.fsSelection    = FontStorage.fsSelection;
    Fattr.lMatch         = FontStorage.lMatch;
    Fattr.fsType         = FATTR_TYPE_KERNING;
    Fattr.fsFontUse      = FontStorage.fsFontUse;
    Fattr.usCodePage     = FontStorage.usCodePage;
    strcpy(Fattr.szFacename,FontStorage.Fontname);
    strcpy(Name, "Testing");

    if ((ReturnCode=GpiCreateLogFont(hPS,(PSTR8)Name,lcid,&Fattr))==GPI_ERROR)
         REPORTSETUPERROR("GpiCreateLogFont");

  return ReturnCode;

}

/***************************************************************************
 *\\ddd
 * Routine Name: InitNoMemoryTesting()
 *
 * Purpose:  This function will call functions to setup/clean the enviroment
 *           for testing for PMERR_INSUFFICIENT_MEMORY
 *
 * System Requirements: None
 *
 * Revision Log: 10/29/91, PDVT, Mike Abuzant
 *               -initial creation
 *
 * Inputs:
 *
 *   usOperationToPerform     Indicates setting/Cleaning Testing Env.
 *
 * Outputs:
 *
 *  TRUE | FALSE              Success/no success
 *
 * Subroutines Required:
 *
 *     CreatTestingEnv();
 *     CleanTestingEnv();
 *
 * Limitations:
 *   None at this time.
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *    IF (Creating test Env )  THEN
 *      Allocate memory to hold all Segment Selectors.
 *      call CreatTestingEnv() to allocate all available memory. All
 *      handles to the allocated memory will be saved in the an
 *      storage area.
 *
 *    ELSE {clean up test Env.}
 *     IF ( We have succeeded in creating test Env) THEN
 *       Call CleanTestingEnv() to free all allocated memory.
 *       Free the storage area where we stored all handles
 *       to allocated memory.
 *
 *
 *    ENDIF
 *
 *   Return results.
 * }
 *\\end
 ***************************************************************************/
BOOL InitNoMemoryTesting(USHORT usOperationToPerform)
{
 static PSEL    pahSEL=(PSEL)NULL;

#ifdef INCL_32
 USHORT junk=usOperationToPerform;
#else
   if (usOperationToPerform==CREATE_TESTING_ENV){
    if (DosAllocSeg(MAX_STORAGE,&SELECTOROF(pahSEL),
                                 SEG_NONSHARED)==NO_ERROR){
      memset((PVOID)pahSEL,0x00,MAX_STORAGE);
      return CreatTestingEnv(pahSEL);
    }
   }
   else {
     if ( pahSEL != (PSEL)NULL){
       (VOID)CleanTestingEnv(pahSEL);
       DosFreeSeg(SELECTOROF(pahSEL));
       pahSEL=(PSEL)NULL;
     }
   }
#endif
   return TRUE;
}

/***************************************************************************
 *\\ddd
 * Routine Name: CreatTestingEnv()
 *
 * Purpose:  This function will create testing Env to test for
 *           PMERR_INSUFFICIENT_MEMORY error code.
 *
 * System Requirements: None
 *
 * Revision Log: 10/29/91, PDVT, Mike Abuzant
 *               -initial creation
 *
 * Inputs:
 *
 *   pahSEL   //An array of selecters to store allocated memory handles
 *
 * Outputs:
 *
 *  TRUE | FALSE              Success/no success
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   1. Create a file which is as a large as the available disk space. This
 *     will prevent OS/2 from swapping to the disk.
 *
 *   2. Allocate all memory available until no more can be allocated
 *
 *   Return results.
 * }
 *\\end
 ***************************************************************************/
BOOL CreatTestingEnv(PSEL ahSEL)
{
  HFILE            FileHandle=(HFILE)NULL;
  USHORT           Counter=0;
  ULONG            FileSize=0;
#ifdef INCL_32
  ULONG            OpenAction;
  PSEL             junk = ahSEL;
#else
  USHORT           OpenAction;
#endif
  FSALLOCATE       DiskInfo;
  USHORT           SegSize=65000u,
                   StorageSize=MAX_STORAGE/sizeof(SEL);

  //Create an empty file. Its size is 0 initially.
  if (DosOpen(BIG_FILENAME,&FileHandle,&OpenAction,
                          (ULONG)FileSize,FILE_NORMAL,
                          FILE_CREATE|FILE_TRUNCATE,
                          OPEN_SHARE_DENYREADWRITE | OPEN_ACCESS_READWRITE,
                         (ULONG)NULL)==NO_ERROR){

      //Assume that OS/2 Swap file in at c: drive.
      if (DosQFSInfo(3,1,(PBYTE)&DiskInfo,sizeof(DiskInfo))==NO_ERROR){
         //Calculate free disk space.
         FileSize=DiskInfo.cSectorUnit *DiskInfo.cUnitAvail*DiskInfo.cbSector;
         //Leave only one MEGByte for OS/2 Swapping
         if (FileSize > ONE_MEGBYTE)
            FileSize-= ONE_MEGBYTE;

         //Expand the file size to fill most of the disk's available space.
         if (DosNewSize(FileHandle,FileSize)==NO_ERROR){
            DosClose(FileHandle);
#ifndef INCL_32
            for(  ; (SegSize-= 2048U) > 2048U && Counter < StorageSize; )
              do{
                DosAllocSeg(SegSize, &ahSEL[Counter],SEG_NONSHARED);

              }while ((ahSEL[Counter]!=(SEL)NULL) &&
                                (++Counter < StorageSize));
#endif
         }
         else
           return FALSE;
      }
      else
        return FALSE;
  }
  else
    return FALSE;

 return TRUE;

}
/***************************************************************************
 *\\ddd
 * Routine Name: CleanTestingEnv()
 *
 * Purpose:  This function will clean up after testing for
 *           PMERR_INSUFFICIENT_MEMORY error code.
 *
 * System Requirements: None
 *
 * Revision Log: 10/29/91, PDVT, Mike Abuzant
 *               -initial creation
 *
 * Inputs:
 *
 *   pahSEL   //An array of selecters to store allocated memory handles
 *
 * Outputs:                None.
 *
 * Subroutines Required:   None.
 *
 * Limitations:            None.
 *
 *\\end
 ***************************************************************************/
/****************************************************************************
 *\\algorithm
 * {
 *   1. Free all allocted memory.
 *
 *   2. Delete the temparay file we have created.
 *
 * }
 *\\end
 ***************************************************************************/
VOID CleanTestingEnv(PSEL ahSEL)
{
  USHORT           Counter=0,
                   StorageSize=MAX_STORAGE/sizeof(SEL);

#ifdef INCL_32
  PSEL junk = ahSEL;
#else
  // Release all allocated memory;
  while (Counter < StorageSize && ahSEL[Counter])
    DosFreeSeg(ahSEL[Counter++]);
#endif

#ifdef INCL_32
  DosDelete(BIG_FILENAME);
#else
  DosDelete(BIG_FILENAME,(ULONG)NULL);
#endif

  return;
}
